@@ -2,4 +2,4 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "checkdeps/testdata/disallowed/teststuff/good.h"
+#include "buildtools/checkdeps/testdata/disallowed/teststuff/good.h"
