@@ -2,10 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "checkdeps/testdata/allowed/good.h"
-#include "checkdeps/testdata/disallowed/bad.h"
-#include "checkdeps/testdata/disallowed/allowed/good.h"
-#include "checkdeps/testdata/disallowed/temporarily_allowed.h"
+#include "buildtools/checkdeps/testdata/allowed/good.h"
+#include "buildtools/checkdeps/testdata/disallowed/bad.h"
+#include "buildtools/checkdeps/testdata/disallowed/allowed/good.h"
+#include "buildtools/checkdeps/testdata/disallowed/temporarily_allowed.h"
 #include "third_party/explicitly_disallowed/bad.h"
 #include "third_party/allowed_may_use/good.h"
 #include "third_party/no_rule/bad.h"
