@@ -14,10 +14,8 @@
 #include <string>
 #include <vector>
 
-#include "base/logging.h"
-#include "base/macros.h"
+#include "base/check.h"
 #include "base/memory/ptr_util.h"
-#include "base/values.h"
 #include "cc/base/region.h"
 #include "cc/base/synced_property.h"
 #include "cc/cc_export.h"
@@ -26,41 +24,33 @@
 #include "cc/layers/draw_mode.h"
 #include "cc/layers/draw_properties.h"
 #include "cc/layers/layer_collections.h"
-#include "cc/layers/layer_impl_test_properties.h"
-#include "cc/layers/layer_position_constraint.h"
 #include "cc/layers/performance_properties.h"
 #include "cc/layers/render_surface_impl.h"
 #include "cc/layers/touch_action_region.h"
+#include "cc/paint/element_id.h"
 #include "cc/tiles/tile_priority.h"
-#include "cc/trees/element_id.h"
 #include "cc/trees/target_property.h"
 #include "components/viz/common/quads/shared_quad_state.h"
 #include "third_party/skia/include/core/SkColor.h"
+#include "ui/gfx/display_color_spaces.h"
 #include "ui/gfx/geometry/point3_f.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/rect_f.h"
 #include "ui/gfx/geometry/scroll_offset.h"
 #include "ui/gfx/transform.h"
 
-namespace base {
-namespace trace_event {
-class TracedValue;
-}
-class DictionaryValue;
-}
-
 namespace viz {
 class ClientResourceProvider;
-class RenderPass;
+class CompositorRenderPass;
 }
 
 namespace cc {
 
 class AppendQuadsData;
+struct LayerDebugInfo;
 class LayerTreeImpl;
 class MicroBenchmarkImpl;
 class PrioritizedTile;
-class ScrollbarLayerImplBase;
 class SimpleEnclosedRegion;
 class Tile;
 
@@ -79,8 +69,11 @@ class CC_EXPORT LayerImpl {
     return base::WrapUnique(new LayerImpl(tree_impl, id));
   }
 
+  LayerImpl(const LayerImpl&) = delete;
   virtual ~LayerImpl();
 
+  LayerImpl& operator=(const LayerImpl&) = delete;
+
   int id() const { return layer_id_; }
 
   // Whether this layer is on the active tree, return false if it's on the
@@ -88,7 +81,7 @@ class CC_EXPORT LayerImpl {
   bool IsActive() const;
 
   void SetHasTransformNode(bool val) { has_transform_node_ = val; }
-  bool has_transform_node() { return has_transform_node_; }
+  bool has_transform_node() const { return has_transform_node_; }
 
   void set_property_tree_sequence_number(int sequence_number) {}
 
@@ -112,31 +105,33 @@ class CC_EXPORT LayerImpl {
     return offset_to_transform_parent_;
   }
 
-  void SetShouldFlattenScreenSpaceTransformFromPropertyTree(
-      bool should_flatten) {
-    should_flatten_screen_space_transform_from_property_tree_ = should_flatten;
-  }
-  bool should_flatten_screen_space_transform_from_property_tree() const {
-    return should_flatten_screen_space_transform_from_property_tree_;
-  }
-
   bool is_clipped() const { return draw_properties_.is_clipped; }
 
   LayerTreeImpl* layer_tree_impl() const { return layer_tree_impl_; }
 
   void PopulateSharedQuadState(viz::SharedQuadState* state,
                                bool contents_opaque) const;
+
+  // If using these two, you need to override GetEnclosingRectInTargetSpace() to
+  // use GetScaledEnclosingRectInTargetSpace(). To do otherwise may result in
+  // inconsistent values, and drawing/clipping problems.
   void PopulateScaledSharedQuadState(viz::SharedQuadState* state,
-                                     float layer_to_content_scale_x,
-                                     float layer_to_content_scale_y,
+                                     float layer_to_content_scale,
                                      bool contents_opaque) const;
+  void PopulateScaledSharedQuadStateWithContentRects(
+      viz::SharedQuadState* state,
+      float layer_to_content_scale,
+      const gfx::Rect& content_rect,
+      const gfx::Rect& content_visible_rect,
+      bool contents_opaque) const;
+
   // WillDraw must be called before AppendQuads. If WillDraw returns false,
   // AppendQuads and DidDraw will not be called. If WillDraw returns true,
   // DidDraw is guaranteed to be called before another WillDraw or before
   // the layer is destroyed.
   virtual bool WillDraw(DrawMode draw_mode,
                         viz::ClientResourceProvider* resource_provider);
-  virtual void AppendQuads(viz::RenderPass* render_pass,
+  virtual void AppendQuads(viz::CompositorRenderPass* render_pass,
                            AppendQuadsData* append_quads_data) {}
   virtual void DidDraw(viz::ClientResourceProvider* resource_provider);
 
@@ -153,42 +148,34 @@ class CC_EXPORT LayerImpl {
 
   virtual void NotifyTileStateChanged(const Tile* tile) {}
 
-  virtual ScrollbarLayerImplBase* ToScrollbarLayer();
+  virtual bool IsScrollbarLayer() const;
+
+  bool IsScrollerOrScrollbar() const;
 
   // Returns true if this layer has content to draw.
   void SetDrawsContent(bool draws_content);
   bool DrawsContent() const { return draws_content_; }
 
-  // Make the layer hit test (see: |should_hit_test|) even if !draws_content_.
-  void SetHitTestableWithoutDrawsContent(bool should_hit_test);
-  bool hit_testable_without_draws_content() const {
-    return hit_testable_without_draws_content_;
-  }
-
-  // True if either the layer draws content or has been marked as hit testable
-  // without draws_content.
-  bool should_hit_test() const {
-    return draws_content_ || hit_testable_without_draws_content_;
-  }
-
-  LayerImplTestProperties* test_properties() {
-    if (!test_properties_)
-      test_properties_.reset(new LayerImplTestProperties(this));
-    return test_properties_.get();
-  }
+  // Make the layer hit testable.
+  void SetHitTestable(bool should_hit_test);
+  bool HitTestable() const;
 
   void SetBackgroundColor(SkColor background_color);
   SkColor background_color() const { return background_color_; }
   void SetSafeOpaqueBackgroundColor(SkColor background_color);
-  // If contents_opaque(), return an opaque color else return a
-  // non-opaque color.  Tries to return background_color(), if possible.
-  SkColor SafeOpaqueBackgroundColor() const;
-
-  void SetMasksToBounds(bool masks_to_bounds);
-  bool masks_to_bounds() const { return masks_to_bounds_; }
+  SkColor safe_opaque_background_color() const {
+    // Layer::SafeOpaqueBackgroundColor() should ensure this.
+    DCHECK_EQ(contents_opaque(),
+              SkColorGetA(safe_opaque_background_color_) == SK_AlphaOPAQUE);
+    return safe_opaque_background_color_;
+  }
 
+  // See Layer::SetContentsOpaque() and SetContentsOpaqueForText() for the
+  // relationship between the two flags.
   void SetContentsOpaque(bool opaque);
   bool contents_opaque() const { return contents_opaque_; }
+  void SetContentsOpaqueForText(bool opaque);
+  bool contents_opaque_for_text() const { return contents_opaque_for_text_; }
 
   float Opacity() const;
 
@@ -196,23 +183,10 @@ class CC_EXPORT LayerImpl {
   void SetElementId(ElementId element_id);
   ElementId element_id() const { return element_id_; }
 
-  void SetPosition(const gfx::PointF& position);
-  gfx::PointF position() const { return position_; }
-
   bool IsAffectedByPageScale() const;
 
   bool Is3dSorted() const { return GetSortingContextId() != 0; }
 
-  void SetUseParentBackfaceVisibility(bool use) {
-    use_parent_backface_visibility_ = use;
-  }
-  bool use_parent_backface_visibility() const {
-    return use_parent_backface_visibility_;
-  }
-
-  bool IsResizedByBrowserControls() const;
-  void SetIsResizedByBrowserControls(bool resized);
-
   void SetShouldCheckBackfaceVisibility(bool should_check_backface_visibility) {
     should_check_backface_visibility_ = should_check_backface_visibility;
   }
@@ -236,8 +210,6 @@ class CC_EXPORT LayerImpl {
     return performance_properties_;
   }
 
-  bool CanUseLCDText() const;
-
   // Setter for draw_properties_.
   void set_visible_layer_rect(const gfx::Rect& visible_rect) {
     draw_properties_.visible_layer_rect = visible_rect;
@@ -253,8 +225,8 @@ class CC_EXPORT LayerImpl {
     return draw_properties_.screen_space_transform_is_animating;
   }
   gfx::Rect clip_rect() const { return draw_properties_.clip_rect; }
-  gfx::Rect drawable_content_rect() const {
-    return draw_properties_.drawable_content_rect;
+  gfx::Rect visible_drawable_content_rect() const {
+    return draw_properties_.visible_drawable_content_rect;
   }
   gfx::Rect visible_layer_rect() const {
     return draw_properties_.visible_layer_rect;
@@ -266,56 +238,21 @@ class CC_EXPORT LayerImpl {
 
   void SetBounds(const gfx::Size& bounds);
   gfx::Size bounds() const;
-  // Like bounds() but doesn't snap to int. Lossy on giant pages (e.g. millions
-  // of pixels) due to use of single precision float.
-  gfx::SizeF BoundsForScrolling() const;
 
-  // Viewport bounds delta are only used for viewport layers and account for
-  // changes in the viewport layers from browser controls and page scale
-  // factors. These deltas are only set on the active tree.
-  // TODO(bokan): These methods should be unneeded now that LTHI sets these
-  // directly on the property trees.
-  void SetViewportBoundsDelta(const gfx::Vector2dF& bounds_delta);
-  gfx::Vector2dF ViewportBoundsDelta() const;
-
-  void SetViewportLayerType(ViewportLayerType type) {
-    // Once set as a viewport layer type, the viewport type should not change.
-    DCHECK(viewport_layer_type() == NOT_VIEWPORT_LAYER ||
-           viewport_layer_type() == type);
-    viewport_layer_type_ = type;
-  }
-  ViewportLayerType viewport_layer_type() const {
-    return static_cast<ViewportLayerType>(viewport_layer_type_);
-  }
-  bool is_viewport_layer_type() const {
-    return viewport_layer_type() != NOT_VIEWPORT_LAYER;
+  void set_is_inner_viewport_scroll_layer() {
+    is_inner_viewport_scroll_layer_ = true;
   }
 
   void SetCurrentScrollOffset(const gfx::ScrollOffset& scroll_offset);
-  gfx::ScrollOffset CurrentScrollOffset() const;
-
-  gfx::ScrollOffset MaxScrollOffset() const;
-  gfx::ScrollOffset ClampScrollOffsetToLimits(gfx::ScrollOffset offset) const;
-  gfx::Vector2dF ClampScrollToMaxScrollOffset();
 
   // Returns the delta of the scroll that was outside of the bounds of the
   // initial scroll
   gfx::Vector2dF ScrollBy(const gfx::Vector2dF& scroll);
 
-  // Marks this layer as being scrollable and needing an associated scroll node.
-  // The scroll node's bounds and container_bounds will be kept in sync with
-  // this layer.
-  void SetScrollable(const gfx::Size& bounds);
-  gfx::Size scroll_container_bounds() const { return scroll_container_bounds_; }
-  bool scrollable() const { return scrollable_; }
-
-  void set_main_thread_scrolling_reasons(
-      uint32_t main_thread_scrolling_reasons) {
-    main_thread_scrolling_reasons_ = main_thread_scrolling_reasons;
-  }
-  uint32_t main_thread_scrolling_reasons() const {
-    return main_thread_scrolling_reasons_;
-  }
+  // Called during a commit or activation, after the property trees are pushed.
+  // It detects changes of scrollable status and scroll container size in the
+  // scroll property, and invalidate scrollbar geometries etc. for the changes.
+  void UpdateScrollable();
 
   void SetNonFastScrollableRegion(const Region& region) {
     non_fast_scrollable_region_ = region;
@@ -324,12 +261,14 @@ class CC_EXPORT LayerImpl {
     return non_fast_scrollable_region_;
   }
 
-  void SetTouchActionRegion(TouchActionRegion touch_action_region) {
-    touch_action_region_ = std::move(touch_action_region);
-  }
+  void SetTouchActionRegion(TouchActionRegion);
   const TouchActionRegion& touch_action_region() const {
     return touch_action_region_;
   }
+  const Region& GetAllTouchActionRegions() const;
+  bool has_touch_action_regions() const {
+    return !touch_action_region_.IsEmpty();
+  }
 
   // Set or get the region that contains wheel event handler.
   // The |wheel_event_handler_region| specify the area where wheel event handler
@@ -341,17 +280,17 @@ class CC_EXPORT LayerImpl {
     return wheel_event_handler_region_;
   }
 
+  // The main thread may commit multiple times before the impl thread actually
+  // draws, so we need to accumulate (i.e. union) any update changes that have
+  // occurred on the main thread until we draw.
   // Note this rect is in layer space (not content space).
-  void SetUpdateRect(const gfx::Rect& update_rect);
+  void UnionUpdateRect(const gfx::Rect& update_rect);
   const gfx::Rect& update_rect() const { return update_rect_; }
 
-  void AddDamageRect(const gfx::Rect& damage_rect);
-  const gfx::Rect& damage_rect() const { return damage_rect_; }
-
-  virtual std::unique_ptr<base::DictionaryValue> LayerAsJson();
-  // TODO(pdr): This should be removed because there is no longer a tree
-  // of layers, only a list.
-  std::unique_ptr<base::DictionaryValue> LayerTreeAsJson();
+  // Denotes an area that is damaged and needs redraw. This is in the layer's
+  // space. By default returns empty rect, but can be overridden by subclasses
+  // as appropriate.
+  virtual gfx::Rect GetDamageRect() const;
 
   // This includes |layer_property_changed_not_from_property_trees_| and
   // property_trees changes.
@@ -362,7 +301,7 @@ class CC_EXPORT LayerImpl {
   // from property_trees changes in animaiton.
   bool LayerPropertyChangedNotFromPropertyTrees() const;
 
-  void ResetChangeTracking();
+  virtual void ResetChangeTracking();
 
   virtual SimpleEnclosedRegion VisibleOpaqueRegion() const;
 
@@ -398,14 +337,21 @@ class CC_EXPORT LayerImpl {
   virtual void GetAllPrioritizedTilesForTracing(
       std::vector<PrioritizedTile>* prioritized_tiles) const;
   virtual void AsValueInto(base::trace_event::TracedValue* dict) const;
+  std::string ToString() const;
 
   virtual size_t GPUMemoryUsageInBytes() const;
 
+  // Mark a layer on pending tree that needs to push its properties to the
+  // active tree. These properties should not be changed during pending tree
+  // lifetime, and only changed by being pushed from the main thread. There are
+  // two cases where this function needs to be called: when main thread layer
+  // has properties that need to be pushed, or when a new LayerImpl is created
+  // on pending tree when syncing layers from main thread.
   void SetNeedsPushProperties();
 
   virtual void RunMicroBenchmark(MicroBenchmarkImpl* benchmark);
 
-  void SetDebugInfo(std::unique_ptr<base::trace_event::TracedValue> debug_info);
+  void UpdateDebugInfo(LayerDebugInfo* debug_info);
 
   void set_contributes_to_drawn_render_surface(bool is_member) {
     contributes_to_drawn_render_surface_ = is_member;
@@ -415,10 +361,6 @@ class CC_EXPORT LayerImpl {
     return contributes_to_drawn_render_surface_;
   }
 
-  bool IsDrawnScrollbar() {
-    return ToScrollbarLayer() && contributes_to_drawn_render_surface_;
-  }
-
   void set_may_contain_video(bool yes) { may_contain_video_ = yes; }
   bool may_contain_video() const { return may_contain_video_; }
 
@@ -431,20 +373,23 @@ class CC_EXPORT LayerImpl {
   // for layers that provide it.
   virtual Region GetInvalidationRegionForDebugging();
 
+  // If you override this, and are making use of
+  // PopulateScaledSharedQuadState(), make sure you call
+  // GetScaledEnclosingRectInTargetSpace(). See comment for
+  // PopulateScaledSharedQuadState().
   virtual gfx::Rect GetEnclosingRectInTargetSpace() const;
 
-  void UpdatePropertyTreeForAnimationIfNeeded(ElementId element_id);
+  // Returns the bounds of this layer in target space when scaled by |scale|.
+  // This function scales in the same way as
+  // PopulateScaledSharedQuadStateQuadState(). See
+  // PopulateScaledSharedQuadStateQuadState() for more details.
+  gfx::Rect GetScaledEnclosingRectInTargetSpace(float scale) const;
 
   float GetIdealContentsScale() const;
 
   void NoteLayerPropertyChanged();
   void NoteLayerPropertyChangedFromPropertyTrees();
 
-  void SetHasWillChangeTransformHint(bool has_will_change);
-  bool has_will_change_transform_hint() const {
-    return has_will_change_transform_hint_;
-  }
-
   ElementListType GetElementTypeForAnimation() const;
 
   void set_needs_show_scrollbars(bool yes) { needs_show_scrollbars_ = yes; }
@@ -460,54 +405,63 @@ class CC_EXPORT LayerImpl {
   // TODO(sunxd): Remove this function and replace it with visitor pattern.
   virtual bool is_surface_layer() const;
 
+  int CalculateJitter();
+
+  std::string DebugName() const;
+
+  virtual gfx::ContentColorUsage GetContentColorUsage() const;
+
  protected:
+  // When |will_always_push_properties| is true, the layer will not itself set
+  // its SetNeedsPushProperties() state, as it expects to be always pushed to
+  // the active tree regardless.
   LayerImpl(LayerTreeImpl* layer_impl,
             int id,
-            scoped_refptr<SyncedScrollOffset> scroll_offset);
-  LayerImpl(LayerTreeImpl* layer_impl, int id);
+            bool will_always_push_properties = false);
 
   // Get the color and size of the layer's debug border.
   virtual void GetDebugBorderProperties(SkColor* color, float* width) const;
 
-  void AppendDebugBorderQuad(viz::RenderPass* render_pass,
+  void AppendDebugBorderQuad(viz::CompositorRenderPass* render_pass,
                              const gfx::Rect& quad_rect,
                              const viz::SharedQuadState* shared_quad_state,
                              AppendQuadsData* append_quads_data) const;
-  void AppendDebugBorderQuad(viz::RenderPass* render_pass,
+  void AppendDebugBorderQuad(viz::CompositorRenderPass* render_pass,
                              const gfx::Rect& quad_rect,
                              const viz::SharedQuadState* shared_quad_state,
                              AppendQuadsData* append_quads_data,
                              SkColor color,
                              float width) const;
 
-  gfx::Rect GetScaledEnclosingRectInTargetSpace(float scale) const;
+  static float GetPreferredRasterScale(
+      gfx::Vector2dF raster_space_scale_factor);
 
  private:
   void ValidateQuadResourcesInternal(viz::DrawQuad* quad) const;
 
   virtual const char* LayerTypeAsString() const;
 
-  int layer_id_;
-  LayerTreeImpl* layer_tree_impl_;
-
-  std::unique_ptr<LayerImplTestProperties> test_properties_;
+  const int layer_id_;
+  LayerTreeImpl* const layer_tree_impl_;
+  const bool will_always_push_properties_ : 1;
 
   // Properties synchronized from the associated Layer.
   gfx::Size bounds_;
 
   gfx::Vector2dF offset_to_transform_parent_;
-  uint32_t main_thread_scrolling_reasons_;
 
-  // Size of the scroll container that this layer scrolls in.
+  // These fields are copies of |container_bounds|, |bounds| and |scrollable|
+  // fields in the associated ScrollNode, and are updated in UpdateScrollable().
+  // The copy is for change detection only.
+  // TODO(wangxianzhu): Actually we only need scroll_container_bounds_ in
+  // pre-CompositeAfterPaint where the scroll node is associated with the
+  // scrolling contents layer, and only need scroll_contents_bounds_ in
+  // CompositeAfterPaint where the scroll node is associated with the scroll
+  // container layer. Remove scroll_container_bounds_ when we launch CAP.
   gfx::Size scroll_container_bounds_;
-
-  // Indicates that this layer will have a scroll property node and that this
-  // layer's bounds correspond to the scroll node's bounds (both |bounds| and
-  // |scroll_container_bounds|).
+  gfx::Size scroll_contents_bounds_;
   bool scrollable_ : 1;
 
-  bool should_flatten_screen_space_transform_from_property_tree_ : 1;
-
   // Tracks if drawing-related properties have changed since last redraw.
   // TODO(wutao): We want to distinquish the sources of change so that we can
   // reuse the cache of render pass. For example, we can reuse the cache when
@@ -517,26 +471,18 @@ class CC_EXPORT LayerImpl {
   // damage from animations. http://crbug.com/755828.
   bool layer_property_changed_not_from_property_trees_ : 1;
   bool layer_property_changed_from_property_trees_ : 1;
-  bool may_contain_video_ : 1;
 
-  bool masks_to_bounds_ : 1;
+  bool may_contain_video_ : 1;
   bool contents_opaque_ : 1;
-  bool use_parent_backface_visibility_ : 1;
+  bool contents_opaque_for_text_ : 1;
   bool should_check_backface_visibility_ : 1;
   bool draws_content_ : 1;
   bool contributes_to_drawn_render_surface_ : 1;
 
-  // Hit testing depends on draws_content (see: |LayerImpl::should_hit_test|)
-  // and this bit can be set to cause the layer to be hit testable without
-  // draws_content.
-  bool hit_testable_without_draws_content_ : 1;
-  bool is_resized_by_browser_controls_ : 1;
+  // Tracks if this layer should participate in hit testing.
+  bool hit_testable_ : 1;
 
-  // TODO(bokan): This can likely be removed after blink-gen-property-trees
-  // is shipped. https://crbug.com/836884.
-  static_assert(LAST_VIEWPORT_LAYER_TYPE < (1u << 3),
-                "enough bits for ViewportLayerType (viewport_layer_type_)");
-  uint8_t viewport_layer_type_ : 3;  // ViewportLayerType
+  bool is_inner_viewport_scroll_layer_ : 1;
 
   Region non_fast_scrollable_region_;
   TouchActionRegion touch_action_region_;
@@ -544,8 +490,6 @@ class CC_EXPORT LayerImpl {
   SkColor background_color_;
   SkColor safe_opaque_background_color_;
 
-  gfx::PointF position_;
-
   int transform_tree_index_;
   int effect_tree_index_;
   int clip_tree_index_;
@@ -555,33 +499,30 @@ class CC_EXPORT LayerImpl {
   friend class TreeSynchronizer;
 
   DrawMode current_draw_mode_;
-
- private:
+  EffectTree& GetEffectTree() const;
   PropertyTrees* GetPropertyTrees() const;
   ClipTree& GetClipTree() const;
-  EffectTree& GetEffectTree() const;
   ScrollTree& GetScrollTree() const;
   TransformTree& GetTransformTree() const;
 
+ private:
   ElementId element_id_;
   // Rect indicating what was repainted/updated during update.
   // Note that plugin layers bypass this and leave it empty.
   // This is in the layer's space.
   gfx::Rect update_rect_;
 
-  // Denotes an area that is damaged and needs redraw. This is in the layer's
-  // space.
-  gfx::Rect damage_rect_;
-
   // Group of properties that need to be computed based on the layer tree
   // hierarchy before layers can be drawn.
   DrawProperties draw_properties_;
   PerformanceProperties<LayerImpl> performance_properties_;
 
-  std::unique_ptr<base::trace_event::TracedValue> owned_debug_info_;
-  base::trace_event::TracedValue* debug_info_;
+  std::unique_ptr<LayerDebugInfo> debug_info_;
+
+  // Cache of all regions represented by any touch action from
+  // |touch_action_region_|.
+  mutable std::unique_ptr<Region> all_touch_action_regions_;
 
-  bool has_will_change_transform_hint_ : 1;
   bool needs_push_properties_ : 1;
   bool scrollbars_hidden_ : 1;
 
@@ -597,8 +538,6 @@ class CC_EXPORT LayerImpl {
   bool raster_even_if_not_drawn_ : 1;
 
   bool has_transform_node_ : 1;
-
-  DISALLOW_COPY_AND_ASSIGN(LayerImpl);
 };
 
 }  // namespace cc
