@@ -5,7 +5,9 @@
 #ifndef CC_LAYERS_PICTURE_LAYER_H_
 #define CC_LAYERS_PICTURE_LAYER_H_
 
-#include "base/macros.h"
+#include <memory>
+#include <vector>
+
 #include "cc/base/devtools_instrumentation.h"
 #include "cc/base/invalidation_region.h"
 #include "cc/benchmarks/micro_benchmark_controller.h"
@@ -21,6 +23,9 @@ class CC_EXPORT PictureLayer : public Layer {
  public:
   static scoped_refptr<PictureLayer> Create(ContentLayerClient* client);
 
+  PictureLayer(const PictureLayer&) = delete;
+  PictureLayer& operator=(const PictureLayer&) = delete;
+
   void ClearClient();
 
   void SetNearestNeighbor(bool nearest_neighbor);
@@ -28,9 +33,9 @@ class CC_EXPORT PictureLayer : public Layer {
     return picture_layer_inputs_.nearest_neighbor;
   }
 
-  void SetTransformedRasterizationAllowed(bool allowed);
-  bool transformed_rasterization_allowed() const {
-    return picture_layer_inputs_.transformed_rasterization_allowed;
+  void SetIsBackdropFilterMask(bool is_backdrop_filter_mask);
+  bool is_backdrop_filter_mask() const {
+    return picture_layer_inputs_.is_backdrop_filter_mask;
   }
 
   // Layer interface.
@@ -38,14 +43,11 @@ class CC_EXPORT PictureLayer : public Layer {
   void SetLayerTreeHost(LayerTreeHost* host) override;
   void PushPropertiesTo(LayerImpl* layer) override;
   void SetNeedsDisplayRect(const gfx::Rect& layer_rect) override;
-  bool Update() override;
-  void SetLayerMaskType(LayerMaskType mask_type) override;
   sk_sp<SkPicture> GetPicture() const override;
-
-  bool HasSlowPaths() const override;
-  bool HasNonAAPaint() const override;
-
+  bool Update() override;
   void RunMicroBenchmark(MicroBenchmark* benchmark) override;
+  void CaptureContent(const gfx::Rect& rect,
+                      std::vector<NodeInfo>* content) override;
 
   ContentLayerClient* client() { return picture_layer_inputs_.client; }
 
@@ -55,8 +57,6 @@ class CC_EXPORT PictureLayer : public Layer {
 
   const DisplayItemList* GetDisplayItemList();
 
-  LayerMaskType mask_type() { return mask_type_; }
-
  protected:
   // Encapsulates all data, callbacks or interfaces received from the embedder.
   struct PictureLayerInputs {
@@ -65,10 +65,9 @@ class CC_EXPORT PictureLayer : public Layer {
 
     ContentLayerClient* client = nullptr;
     bool nearest_neighbor = false;
-    bool transformed_rasterization_allowed = false;
-    gfx::Rect recorded_viewport;
+    bool is_backdrop_filter_mask = false;
     scoped_refptr<DisplayItemList> display_list;
-    size_t painter_reported_memory_usage = 0;
+    base::Optional<gfx::Size> directly_composited_image_size = base::nullopt;
   };
 
   explicit PictureLayer(ContentLayerClient* client);
@@ -86,8 +85,6 @@ class CC_EXPORT PictureLayer : public Layer {
 
   void DropRecordingSourceContentIfInvalid();
 
-  bool ShouldUseTransformedRasterization() const;
-
   std::unique_ptr<RecordingSource> recording_source_;
   devtools_instrumentation::
       ScopedLayerObjectTracker instrumentation_object_tracker_;
@@ -95,9 +92,6 @@ class CC_EXPORT PictureLayer : public Layer {
   Region last_updated_invalidation_;
 
   int update_source_frame_number_;
-  LayerMaskType mask_type_;
-
-  DISALLOW_COPY_AND_ASSIGN(PictureLayer);
 };
 
 }  // namespace cc
