@@ -8,8 +8,8 @@
 #include <memory>
 
 #include "base/bind.h"
-#include "base/macros.h"
 #include "base/memory/ptr_util.h"
+#include "base/synchronization/waitable_event.h"
 #include "cc/cc_export.h"
 #include "cc/layers/layer_impl.h"
 #include "components/viz/common/quads/surface_draw_quad.h"
@@ -19,27 +19,25 @@
 
 namespace cc {
 
-// This must match SurfaceLayer::UpdateSubmissionStateCB.
-using UpdateSubmissionStateCB = base::RepeatingCallback<void(bool is_visible)>;
+// This must match surface_layer.h's UpdateSubmissionStateCB.
+using UpdateSubmissionStateCB =
+    base::RepeatingCallback<void(bool is_visible, base::WaitableEvent*)>;
 
 class CC_EXPORT SurfaceLayerImpl : public LayerImpl {
  public:
   static std::unique_ptr<SurfaceLayerImpl> Create(
       LayerTreeImpl* tree_impl,
       int id,
-      UpdateSubmissionStateCB update_submission_state_callback) {
-    return base::WrapUnique(new SurfaceLayerImpl(
-        tree_impl, id, std::move(update_submission_state_callback)));
-  }
+      UpdateSubmissionStateCB update_submission_state_callback);
 
   static std::unique_ptr<SurfaceLayerImpl> Create(LayerTreeImpl* tree_impl,
-                                                  int id) {
-    return base::WrapUnique(
-        new SurfaceLayerImpl(tree_impl, id, base::BindRepeating([](bool) {})));
-  }
+                                                  int id);
 
+  SurfaceLayerImpl(const SurfaceLayerImpl&) = delete;
   ~SurfaceLayerImpl() override;
 
+  SurfaceLayerImpl& operator=(const SurfaceLayerImpl&) = delete;
+
   void SetRange(const viz::SurfaceRange& surface_range,
                 base::Optional<uint32_t> deadline_in_frames);
   const viz::SurfaceRange& range() const { return surface_range_; }
@@ -56,25 +54,28 @@ class CC_EXPORT SurfaceLayerImpl : public LayerImpl {
   void SetSurfaceHitTestable(bool surface_hit_testable);
   bool surface_hit_testable() const { return surface_hit_testable_; }
 
+  void SetHasPointerEventsNone(bool has_pointer_events_none);
+  bool has_pointer_events_none() const { return has_pointer_events_none_; }
+
+  void SetIsReflection(bool is_reflection);
+  bool is_reflection() const { return is_reflection_; }
+
   // LayerImpl overrides.
   std::unique_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl) override;
   void PushPropertiesTo(LayerImpl* layer) override;
   bool WillDraw(DrawMode draw_mode,
                 viz::ClientResourceProvider* resource_provider) override;
-  void AppendQuads(viz::RenderPass* render_pass,
+  void AppendQuads(viz::CompositorRenderPass* render_pass,
                    AppendQuadsData* append_quads_data) override;
   bool is_surface_layer() const override;
+  gfx::Rect GetEnclosingRectInTargetSpace() const override;
 
  protected:
   SurfaceLayerImpl(LayerTreeImpl* tree_impl, int id, UpdateSubmissionStateCB);
 
  private:
-  viz::SurfaceDrawQuad* CreateSurfaceDrawQuad(
-      viz::RenderPass* render_pass,
-      const viz::SurfaceRange& surface_range);
-
   void GetDebugBorderProperties(SkColor* color, float* width) const override;
-  void AppendRainbowDebugBorder(viz::RenderPass* render_pass);
+  void AppendRainbowDebugBorder(viz::CompositorRenderPass* render_pass);
   void AsValueInto(base::trace_event::TracedValue* dict) const override;
   const char* LayerTypeAsString() const override;
 
@@ -84,9 +85,9 @@ class CC_EXPORT SurfaceLayerImpl : public LayerImpl {
 
   bool stretch_content_to_fill_bounds_ = false;
   bool surface_hit_testable_ = false;
+  bool has_pointer_events_none_ = false;
+  bool is_reflection_ = false;
   bool will_draw_ = false;
-
-  DISALLOW_COPY_AND_ASSIGN(SurfaceLayerImpl);
 };
 
 }  // namespace cc
