@@ -5,15 +5,22 @@
 #include "cc/paint/paint_op_reader.h"
 
 #include <stddef.h>
-#include <algorithm>
 
+#include <algorithm>
+#include <memory>
+#include <utility>
+
+#include "base/bits.h"
+#include "base/compiler_specific.h"
+#include "base/debug/dump_without_crashing.h"
+#include "base/rand_util.h"
 #include "base/stl_util.h"
 #include "cc/paint/image_transfer_cache_entry.h"
+#include "cc/paint/paint_cache.h"
 #include "cc/paint/paint_flags.h"
 #include "cc/paint/paint_image_builder.h"
 #include "cc/paint/paint_op_buffer.h"
 #include "cc/paint/paint_shader.h"
-#include "cc/paint/path_transfer_cache_entry.h"
 #include "cc/paint/shader_transfer_cache_entry.h"
 #include "cc/paint/transfer_cache_deserialize_helper.h"
 #include "third_party/skia/include/core/SkPath.h"
@@ -22,25 +29,18 @@
 #include "third_party/skia/include/core/SkTextBlob.h"
 #include "third_party/skia/src/core/SkRemoteGlyphCache.h"
 
+#if !defined(OS_ANDROID)
+#include "cc/paint/skottie_transfer_cache_entry.h"
+#endif
+
 namespace cc {
 namespace {
 
-// If we have more than this many colors, abort deserialization.
-const size_t kMaxShaderColorsSupported = 10000;
-const size_t kMaxMergeFilterCount = 10000;
-const size_t kMaxKernelSize = 1000;
-const size_t kMaxRegionByteSize = 10 * 1024;
-
 bool IsValidPaintShaderType(PaintShader::Type type) {
   return static_cast<uint8_t>(type) <
          static_cast<uint8_t>(PaintShader::Type::kShaderCount);
 }
 
-bool IsValidSkShaderTileMode(SkShader::TileMode mode) {
-  // When Skia adds Decal, update this (skbug.com/7638)
-  return mode <= SkShader::kMirror_TileMode;
-}
-
 bool IsValidPaintShaderScalingBehavior(PaintShader::ScalingBehavior behavior) {
   return behavior == PaintShader::ScalingBehavior::kRasterAtScale ||
          behavior == PaintShader::ScalingBehavior::kFixedScale;
@@ -100,7 +100,12 @@ template <typename T>
 void PaintOpReader::ReadSimple(T* val) {
   static_assert(base::is_trivially_copyable<T>::value,
                 "Not trivially copyable");
-  if (remaining_bytes_ < sizeof(T))
+
+  // Align everything to 4 bytes, as the writer does.
+  static constexpr size_t kAlign = 4;
+  size_t size = base::bits::Align(sizeof(T), kAlign);
+
+  if (remaining_bytes_ < size)
     SetInvalid();
   if (!valid_)
     return;
@@ -111,8 +116,18 @@ void PaintOpReader::ReadSimple(T* val) {
   // use assignment.
   *val = *reinterpret_cast<const T*>(const_cast<const char*>(memory_));
 
-  memory_ += sizeof(T);
-  remaining_bytes_ -= sizeof(T);
+  memory_ += size;
+  remaining_bytes_ -= size;
+}
+
+uint8_t* PaintOpReader::CopyScratchSpace(size_t bytes) {
+  DCHECK(SkIsAlign4(reinterpret_cast<uintptr_t>(memory_)));
+
+  if (options_.scratch_buffer->size() < bytes)
+    options_.scratch_buffer->resize(bytes);
+  memcpy(options_.scratch_buffer->data(), const_cast<const char*>(memory_),
+         bytes);
+  return options_.scratch_buffer->data();
 }
 
 template <typename T>
@@ -126,13 +141,9 @@ void PaintOpReader::ReadFlattenable(sk_sp<T>* val) {
   if (bytes == 0)
     return;
 
-  // This is assumed safe from TOCTOU violations as the flattenable
-  // deserializing function uses an SkReadBuffer which reads each piece of
-  // memory once much like PaintOpReader does.
-  DCHECK(SkIsAlign4(reinterpret_cast<uintptr_t>(memory_)));
+  auto* scratch = CopyScratchSpace(bytes);
   val->reset(static_cast<T*>(
-      SkFlattenable::Deserialize(T::GetFlattenableType(),
-                                 const_cast<const char*>(memory_), bytes)
+      SkFlattenable::Deserialize(T::GetFlattenableType(), scratch, bytes)
           .release()));
   if (!val)
     SetInvalid();
@@ -194,26 +205,57 @@ void PaintOpReader::Read(SkRRect* rect) {
 }
 
 void PaintOpReader::Read(SkPath* path) {
-  uint32_t transfer_cache_entry_id;
-  ReadSimple(&transfer_cache_entry_id);
+  uint32_t path_id;
+  ReadSimple(&path_id);
   if (!valid_)
     return;
-  auto* entry =
-      options_.transfer_cache->GetEntryAs<ServicePathTransferCacheEntry>(
-          transfer_cache_entry_id);
-  if (entry) {
-    *path = entry->path();
-  } else {
+
+  uint32_t entry_state_int = 0u;
+  ReadSimple(&entry_state_int);
+  if (entry_state_int > static_cast<uint32_t>(PaintCacheEntryState::kLast)) {
     valid_ = false;
+    return;
+  }
+
+  auto entry_state = static_cast<PaintCacheEntryState>(entry_state_int);
+  switch (entry_state) {
+    case PaintCacheEntryState::kEmpty:
+      return;
+    case PaintCacheEntryState::kCached:
+      if (!options_.paint_cache->GetPath(path_id, path))
+        SetInvalid();
+      return;
+    case PaintCacheEntryState::kInlined: {
+      size_t path_bytes = 0u;
+      ReadSize(&path_bytes);
+      if (path_bytes > remaining_bytes_)
+        SetInvalid();
+      if (path_bytes == 0u)
+        SetInvalid();
+      if (!valid_)
+        return;
+
+      auto* scratch = CopyScratchSpace(path_bytes);
+      size_t bytes_read = path->readFromMemory(scratch, path_bytes);
+      if (bytes_read == 0u) {
+        SetInvalid();
+        return;
+      }
+      options_.paint_cache->PutPath(path_id, *path);
+      memory_ += path_bytes;
+      remaining_bytes_ -= path_bytes;
+      return;
+    }
   }
 }
 
 void PaintOpReader::Read(PaintFlags* flags) {
-  Read(&flags->text_size_);
   ReadSimple(&flags->color_);
   Read(&flags->width_);
   Read(&flags->miter_limit_);
-  ReadSimple(&flags->blend_mode_);
+
+  Read(&flags->blend_mode_);
+
   ReadSimple(&flags->bitfields_uint_);
 
   ReadFlattenable(&flags->path_effect_);
@@ -277,12 +319,13 @@ void PaintOpReader::Read(PaintImage* image) {
 
         *image = PaintImageBuilder::WithDefault()
                      .set_id(PaintImage::GetNextId())
-                     .set_image(SkImage::MakeRasterCopy(pixmap),
-                                PaintImage::kNonLazyStableId)
+                     .set_texture_image(SkImage::MakeRasterCopy(pixmap),
+                                        PaintImage::kNonLazyStableId)
                      .TakePaintImage();
       }
         return;
       case PaintOp::SerializedImageType::kTransferCacheEntry:
+      case PaintOp::SerializedImageType::kMailbox:
         SetInvalid();
         return;
     }
@@ -291,6 +334,34 @@ void PaintOpReader::Read(PaintImage* image) {
     return;
   }
 
+  if (serialized_type == PaintOp::SerializedImageType::kMailbox) {
+    if (!options_.shared_image_provider) {
+      SetInvalid();
+      return;
+    }
+
+    gpu::Mailbox mailbox;
+    Read(&mailbox);
+    if (mailbox.IsZero()) {
+      SetInvalid();
+      return;
+    }
+
+    sk_sp<SkImage> sk_image =
+        options_.shared_image_provider->OpenSharedImageForRead(mailbox);
+    if (!sk_image) {
+      SetInvalid();
+      return;
+    }
+
+    *image = PaintImageBuilder::WithDefault()
+                 .set_id(PaintImage::GetNextId())
+                 .set_texture_image(std::move(sk_image),
+                                    PaintImage::kNonLazyStableId)
+                 .TakePaintImage();
+    return;
+  }
+
   if (serialized_type != PaintOp::SerializedImageType::kTransferCacheEntry) {
     SetInvalid();
     return;
@@ -311,19 +382,17 @@ void PaintOpReader::Read(PaintImage* image) {
   if (transfer_cache_entry_id == kInvalidImageTransferCacheEntryId)
     return;
 
+  // The transfer cache entry for an image may not exist if the upload fails.
   if (auto* entry =
           options_.transfer_cache->GetEntryAs<ServiceImageTransferCacheEntry>(
               transfer_cache_entry_id)) {
     if (needs_mips)
       entry->EnsureMips();
-    *image = PaintImageBuilder::WithDefault()
-                 .set_id(PaintImage::GetNextId())
-                 .set_image(entry->image(), PaintImage::kNonLazyStableId)
-                 .TakePaintImage();
-  } else {
-    // If a transfer cache id exists, we must have a valid entry for it in the
-    // cache.
-    SetInvalid();
+    *image =
+        PaintImageBuilder::WithDefault()
+            .set_id(PaintImage::GetNextId())
+            .set_texture_image(entry->image(), PaintImage::kNonLazyStableId)
+            .TakePaintImage();
   }
 }
 
@@ -359,13 +428,8 @@ void PaintOpReader::Read(sk_sp<SkColorSpace>* color_space) {
   if (!valid_ || size == 0)
     return;
 
-  // To avoid TOCTOU issues, make a copy of this prior to turning it
-  // into an SkColorSpace.  SkColorSpace::Deserialize reads header
-  // fields multiple times, so is not safe to pass memory_ to directly.
-  std::unique_ptr<char[]> data(new char[size]);
-  memcpy(data.get(), const_cast<const char*>(memory_), size);
-
-  *color_space = SkColorSpace::Deserialize(data.get(), size);
+  auto* scratch = CopyScratchSpace(size);
+  *color_space = SkColorSpace::Deserialize(scratch, size);
   // If this had non-zero bytes, it should be a valid color space.
   if (!color_space)
     SetInvalid();
@@ -374,28 +438,51 @@ void PaintOpReader::Read(sk_sp<SkColorSpace>* color_space) {
   remaining_bytes_ -= size;
 }
 
-void PaintOpReader::Read(scoped_refptr<PaintTextBlob>* paint_blob) {
+void PaintOpReader::Read(sk_sp<SkTextBlob>* blob) {
+  AlignMemory(4);
+  uint32_t blob_id = 0u;
+  Read(&blob_id);
+  if (!valid_)
+    return;
+
   size_t data_bytes = 0u;
   ReadSize(&data_bytes);
-  if (remaining_bytes_ < data_bytes || data_bytes == 0u)
+  if (remaining_bytes_ < data_bytes)
     SetInvalid();
   if (!valid_)
     return;
 
+  if (data_bytes == 0u) {
+    auto cached_blob = options_.paint_cache->GetTextBlob(blob_id);
+    if (!cached_blob) {
+      // TODO(khushalsagar): Temporary for debugging crbug.com/1019634.
+      SetInvalid(true /* skip_crash_dump*/);
+      return;
+    }
+
+    *blob = std::move(cached_blob);
+    return;
+  }
+
   DCHECK(options_.strike_client);
   SkDeserialProcs procs;
   TypefaceCtx typeface_ctx(options_.strike_client);
   procs.fTypefaceProc = &DeserializeTypeface;
   procs.fTypefaceCtx = &typeface_ctx;
-  sk_sp<SkTextBlob> blob = SkTextBlob::Deserialize(
-      const_cast<const char*>(memory_), data_bytes, procs);
-  if (!blob || typeface_ctx.invalid_typeface) {
+  auto* scratch = CopyScratchSpace(data_bytes);
+  sk_sp<SkTextBlob> deserialized_blob =
+      SkTextBlob::Deserialize(scratch, data_bytes, procs);
+  if (!deserialized_blob) {
     SetInvalid();
     return;
   }
+  if (typeface_ctx.invalid_typeface) {
+    SetInvalid();
+    return;
+  }
+  options_.paint_cache->PutTextBlob(blob_id, deserialized_blob);
 
-  *paint_blob = base::MakeRefCounted<PaintTextBlob>(
-      std::move(blob), std::vector<sk_sp<SkTypeface>>());
+  *blob = std::move(deserialized_blob);
   memory_ += data_bytes;
   remaining_bytes_ -= data_bytes;
 }
@@ -420,10 +507,8 @@ void PaintOpReader::Read(sk_sp<PaintShader>* shader) {
   ReadSimple(&ref.flags_);
   ReadSimple(&ref.end_radius_);
   ReadSimple(&ref.start_radius_);
-  ReadSimple(&ref.tx_);
-  ReadSimple(&ref.ty_);
-  if (!IsValidSkShaderTileMode(ref.tx_) || !IsValidSkShaderTileMode(ref.ty_))
-    SetInvalid();
+  Read(&ref.tx_);
+  Read(&ref.ty_);
   ReadSimple(&ref.fallback_color_);
   ReadSimple(&ref.scaling_behavior_);
   if (!IsValidPaintShaderScalingBehavior(ref.scaling_behavior_))
@@ -469,7 +554,7 @@ void PaintOpReader::Read(sk_sp<PaintShader>* shader) {
   ReadSize(&colors_size);
 
   // If there are too many colors, abort.
-  if (colors_size > kMaxShaderColorsSupported) {
+  if (colors_size > remaining_bytes_) {
     SetInvalid();
     return;
   }
@@ -518,19 +603,17 @@ void PaintOpReader::Read(sk_sp<PaintShader>* shader) {
   auto* entry =
       options_.transfer_cache->GetEntryAs<ServiceShaderTransferCacheEntry>(
           shader_id);
-  // Only consider entries that use the same scale and color space.
-  // This limits the service side transfer cache to only having one entry
-  // per shader but this will hit the common case of enabling Skia reuse.
-  if (entry && entry->shader()->tile_ == ref.tile_ &&
-      entry->raster_color_space_id() == options_.raster_color_space_id) {
+  // Only consider entries that use the same scale.  This limits the service
+  // side transfer cache to only having one entry per shader but this will hit
+  // the common case of enabling Skia reuse.
+  if (entry && entry->shader()->tile_ == ref.tile_) {
     DCHECK(!ref.cached_shader_);
     ref.cached_shader_ = entry->shader()->GetSkShader();
   } else {
     ref.CreateSkShader();
-    std::unique_ptr<ServiceShaderTransferCacheEntry> entry(
-        new ServiceShaderTransferCacheEntry(
-            *shader, options_.raster_color_space_id, shader_size));
-    options_.transfer_cache->CreateLocalEntry(shader_id, std::move(entry));
+    options_.transfer_cache->CreateLocalEntry(
+        shader_id, std::make_unique<ServiceShaderTransferCacheEntry>(
+                       *shader, shader_size));
   }
 }
 
@@ -539,29 +622,107 @@ void PaintOpReader::Read(SkMatrix* matrix) {
   FixupMatrixPostSerialization(matrix);
 }
 
-void PaintOpReader::Read(SkColorType* color_type) {
-  uint32_t raw_color_type = kUnknown_SkColorType;
-  ReadSimple(&raw_color_type);
+void PaintOpReader::Read(SkM44* matrix) {
+  ReadSimple(matrix);
+}
 
-  if (raw_color_type > kLastEnum_SkColorType) {
+void PaintOpReader::Read(SkSamplingOptions* sampling) {
+  bool useCubic;
+  Read(&useCubic);
+  if (useCubic) {
+    SkCubicResampler cubic;
+    Read(&cubic.B);
+    Read(&cubic.C);
+    *sampling = SkSamplingOptions(cubic);
+  } else {
+    SkFilterMode filter;
+    SkMipmapMode mipmap;
+    Read(&filter);
+    Read(&mipmap);
+    *sampling = SkSamplingOptions(filter, mipmap);
+  }
+}
+
+void PaintOpReader::Read(SkYUVColorSpace* yuv_color_space) {
+  uint32_t raw_yuv_color_space = kIdentity_SkYUVColorSpace;
+  ReadSimple(&raw_yuv_color_space);
+
+  if (raw_yuv_color_space > kLastEnum_SkYUVColorSpace) {
     SetInvalid();
     return;
   }
 
-  *color_type = static_cast<SkColorType>(raw_color_type);
+  *yuv_color_space = static_cast<SkYUVColorSpace>(raw_yuv_color_space);
 }
 
-void PaintOpReader::AlignMemory(size_t alignment) {
-  // Due to the math below, alignment must be a power of two.
-  DCHECK_GT(alignment, 0u);
-  DCHECK_EQ(alignment & (alignment - 1), 0u);
+void PaintOpReader::Read(SkYUVAInfo::PlaneConfig* plane_config) {
+  uint32_t raw_plane_config =
+      static_cast<uint32_t>(SkYUVAInfo::PlaneConfig::kUnknown);
+  ReadSimple(&raw_plane_config);
 
-  uintptr_t memory = reinterpret_cast<uintptr_t>(memory_);
-  // The following is equivalent to:
-  //   padding = (alignment - memory % alignment) % alignment;
-  // because alignment is a power of two. This doesn't use modulo operator
-  // however, since it can be slow.
-  size_t padding = ((memory + alignment - 1) & ~(alignment - 1)) - memory;
+  if (raw_plane_config >
+      static_cast<uint32_t>(SkYUVAInfo::PlaneConfig::kLast)) {
+    SetInvalid();
+    return;
+  }
+
+  *plane_config = static_cast<SkYUVAInfo::PlaneConfig>(raw_plane_config);
+}
+
+void PaintOpReader::Read(SkYUVAInfo::Subsampling* subsampling) {
+  uint32_t raw_subsampling =
+      static_cast<uint32_t>(SkYUVAInfo::Subsampling::kUnknown);
+  ReadSimple(&raw_subsampling);
+
+  if (raw_subsampling > static_cast<uint32_t>(SkYUVAInfo::Subsampling::kLast)) {
+    SetInvalid();
+    return;
+  }
+
+  *subsampling = static_cast<SkYUVAInfo::Subsampling>(raw_subsampling);
+}
+
+void PaintOpReader::Read(gpu::Mailbox* mailbox) {
+  ReadData(sizeof(gpu::Mailbox::Name), (*mailbox).name);
+}
+
+// Android does not use skottie. Remove below section to keep binary size to a
+// minimum.
+#if !defined(OS_ANDROID)
+void PaintOpReader::Read(scoped_refptr<SkottieWrapper>* skottie) {
+  if (!options_.is_privileged) {
+    valid_ = false;
+    return;
+  }
+
+  uint32_t transfer_cache_entry_id;
+  ReadSimple(&transfer_cache_entry_id);
+  if (!valid_)
+    return;
+  auto* entry =
+      options_.transfer_cache->GetEntryAs<ServiceSkottieTransferCacheEntry>(
+          transfer_cache_entry_id);
+  if (entry) {
+    *skottie = entry->skottie();
+  } else {
+    valid_ = false;
+  }
+
+  size_t bytes_to_skip = 0u;
+  ReadSize(&bytes_to_skip);
+  if (!valid_)
+    return;
+  if (bytes_to_skip > remaining_bytes_) {
+    valid_ = false;
+    return;
+  }
+  memory_ += bytes_to_skip;
+  remaining_bytes_ -= bytes_to_skip;
+}
+#endif  // !defined(OS_ANDROID)
+
+void PaintOpReader::AlignMemory(size_t alignment) {
+  size_t padding = base::bits::Align(memory_, alignment) - memory_;
   if (padding > remaining_bytes_)
     SetInvalid();
 
@@ -569,7 +730,12 @@ void PaintOpReader::AlignMemory(size_t alignment) {
   remaining_bytes_ -= padding;
 }
 
-inline void PaintOpReader::SetInvalid() {
+// Don't inline this function so that crash reports can show the caller.
+NOINLINE void PaintOpReader::SetInvalid(bool skip_crash_dump) {
+  if (!skip_crash_dump && valid_ && options_.crash_dump_on_failure &&
+      base::RandInt(1, 10) == 1) {
+    base::debug::DumpWithoutCrashing();
+  }
   valid_ = false;
 }
 
@@ -588,14 +754,11 @@ const volatile void* PaintOpReader::ExtractReadableMemory(size_t bytes) {
 }
 
 void PaintOpReader::Read(sk_sp<PaintFilter>* filter) {
-  uint32_t type_int = 0;
-  ReadSimple(&type_int);
-  if (type_int > static_cast<uint32_t>(PaintFilter::Type::kMaxFilterType))
-    SetInvalid();
+  PaintFilter::Type type;
+  ReadEnum(&type);
   if (!valid_)
     return;
 
-  auto type = static_cast<PaintFilter::Type>(type_int);
   if (type == PaintFilter::Type::kNullFilter) {
     *filter = nullptr;
     return;
@@ -605,11 +768,9 @@ void PaintOpReader::Read(sk_sp<PaintFilter>* filter) {
   base::Optional<PaintFilter::CropRect> crop_rect;
   ReadSimple(&has_crop_rect);
   if (has_crop_rect) {
-    uint32_t flags = 0;
     SkRect rect = SkRect::MakeEmpty();
-    ReadSimple(&flags);
     ReadSimple(&rect);
-    crop_rect.emplace(rect, flags);
+    crop_rect.emplace(rect);
   }
 
   AlignMemory(4);
@@ -708,12 +869,12 @@ void PaintOpReader::ReadBlurPaintFilter(
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
   SkScalar sigma_x = 0.f;
   SkScalar sigma_y = 0.f;
-  BlurPaintFilter::TileMode tile_mode = SkBlurImageFilter::kClamp_TileMode;
+  SkTileMode tile_mode;
   sk_sp<PaintFilter> input;
 
   Read(&sigma_x);
   Read(&sigma_y);
-  ReadSimple(&tile_mode);
+  Read(&tile_mode);
   Read(&input);
   if (!valid_)
     return;
@@ -730,8 +891,7 @@ void PaintOpReader::ReadDropShadowPaintFilter(
   SkScalar sigma_x = 0.f;
   SkScalar sigma_y = 0.f;
   SkColor color = SK_ColorBLACK;
-  DropShadowPaintFilter::ShadowMode shadow_mode =
-      SkDropShadowImageFilter::kDrawShadowAndForeground_ShadowMode;
+  DropShadowPaintFilter::ShadowMode shadow_mode;
   sk_sp<PaintFilter> input;
 
   Read(&dx);
@@ -739,11 +899,9 @@ void PaintOpReader::ReadDropShadowPaintFilter(
   Read(&sigma_x);
   Read(&sigma_y);
   Read(&color);
-  ReadSimple(&shadow_mode);
+  ReadEnum(&shadow_mode);
   Read(&input);
 
-  if (shadow_mode > SkDropShadowImageFilter::kLast_ShadowMode)
-    SetInvalid();
   if (!valid_)
     return;
   filter->reset(new DropShadowPaintFilter(dx, dy, sigma_x, sigma_y, color,
@@ -802,19 +960,15 @@ void PaintOpReader::ReadAlphaThresholdPaintFilter(
 void PaintOpReader::ReadXfermodePaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  uint32_t blend_mode_int = 0;
+  SkBlendMode blend_mode;
   sk_sp<PaintFilter> background;
   sk_sp<PaintFilter> foreground;
 
-  Read(&blend_mode_int);
+  Read(&blend_mode);
   Read(&background);
   Read(&foreground);
-  SkBlendMode blend_mode = SkBlendMode::kClear;
-  if (blend_mode_int > static_cast<uint32_t>(SkBlendMode::kLastMode))
-    SetInvalid();
   if (!valid_)
     return;
-  blend_mode = static_cast<SkBlendMode>(blend_mode_int);
 
   filter->reset(new XfermodePaintFilter(blend_mode, std::move(background),
                                         std::move(foreground),
@@ -852,7 +1006,7 @@ void PaintOpReader::ReadMatrixConvolutionPaintFilter(
   SkScalar gain = 0.f;
   SkScalar bias = 0.f;
   SkIPoint kernel_offset = SkIPoint::Make(0, 0);
-  uint32_t tile_mode_int = 0;
+  SkTileMode tile_mode;
   bool convolve_alpha = false;
   sk_sp<PaintFilter> input;
 
@@ -861,7 +1015,7 @@ void PaintOpReader::ReadMatrixConvolutionPaintFilter(
     return;
   auto size =
       static_cast<size_t>(sk_64_mul(kernel_size.width(), kernel_size.height()));
-  if (size > kMaxKernelSize) {
+  if (size > remaining_bytes_) {
     SetInvalid();
     return;
   }
@@ -871,15 +1025,11 @@ void PaintOpReader::ReadMatrixConvolutionPaintFilter(
   Read(&gain);
   Read(&bias);
   ReadSimple(&kernel_offset);
-  Read(&tile_mode_int);
+  Read(&tile_mode);
   Read(&convolve_alpha);
   Read(&input);
-  if (tile_mode_int > SkMatrixConvolutionImageFilter::kMax_TileMode)
-    SetInvalid();
   if (!valid_)
     return;
-  MatrixConvolutionPaintFilter::TileMode tile_mode =
-      static_cast<MatrixConvolutionPaintFilter::TileMode>(tile_mode_int);
   filter->reset(new MatrixConvolutionPaintFilter(
       kernel_size, kernel.data(), gain, bias, kernel_offset, tile_mode,
       convolve_alpha, std::move(input), base::OptionalOrNullptr(crop_rect)));
@@ -888,33 +1038,20 @@ void PaintOpReader::ReadMatrixConvolutionPaintFilter(
 void PaintOpReader::ReadDisplacementMapEffectPaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  // Unknown, R, G, B, A: max type is 4.
-  static const int kMaxChannelSelectorType = 4;
-
-  uint32_t channel_x_int = 0;
-  uint32_t channel_y_int = 0;
+  SkColorChannel channel_x;
+  SkColorChannel channel_y;
   SkScalar scale = 0.f;
   sk_sp<PaintFilter> displacement;
   sk_sp<PaintFilter> color;
 
-  Read(&channel_x_int);
-  Read(&channel_y_int);
+  ReadEnum<SkColorChannel, SkColorChannel::kA>(&channel_x);
+  ReadEnum<SkColorChannel, SkColorChannel::kA>(&channel_y);
   Read(&scale);
   Read(&displacement);
   Read(&color);
 
-  if (channel_x_int > kMaxChannelSelectorType ||
-      channel_y_int > kMaxChannelSelectorType) {
-    SetInvalid();
-  }
   if (!valid_)
     return;
-  DisplacementMapEffectPaintFilter::ChannelSelectorType channel_x =
-      static_cast<DisplacementMapEffectPaintFilter::ChannelSelectorType>(
-          channel_x_int);
-  DisplacementMapEffectPaintFilter::ChannelSelectorType channel_y =
-      static_cast<DisplacementMapEffectPaintFilter::ChannelSelectorType>(
-          channel_y_int);
   filter->reset(new DisplacementMapEffectPaintFilter(
       channel_x, channel_y, scale, std::move(displacement), std::move(color),
       base::OptionalOrNullptr(crop_rect)));
@@ -959,8 +1096,13 @@ void PaintOpReader::ReadMergePaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
   size_t input_count = 0;
-  ReadSimple(&input_count);
-  if (input_count > kMaxMergeFilterCount)
+  ReadSize(&input_count);
+
+  // The minimum size for a serialized filter is 4 bytes (a zero uint32_t to
+  // indicate a null filter). Make sure the |input_count| doesn't exceed the
+  // maximum number of filters possible for the remaining data.
+  const size_t max_filters = remaining_bytes_ / 4u;
+  if (input_count > max_filters)
     SetInvalid();
   if (!valid_)
     return;
@@ -977,22 +1119,16 @@ void PaintOpReader::ReadMergePaintFilter(
 void PaintOpReader::ReadMorphologyPaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  uint32_t morph_type_int = 0;
-  int radius_x = 0;
-  int radius_y = 0;
+  MorphologyPaintFilter::MorphType morph_type;
+  float radius_x = 0;
+  float radius_y = 0;
   sk_sp<PaintFilter> input;
-  Read(&morph_type_int);
+  ReadEnum(&morph_type);
   Read(&radius_x);
   Read(&radius_y);
   Read(&input);
-  if (morph_type_int >
-      static_cast<uint32_t>(MorphologyPaintFilter::MorphType::kMaxMorphType)) {
-    SetInvalid();
-  }
   if (!valid_)
     return;
-  MorphologyPaintFilter::MorphType morph_type =
-      static_cast<MorphologyPaintFilter::MorphType>(morph_type_int);
   filter->reset(new MorphologyPaintFilter(morph_type, radius_x, radius_y,
                                           std::move(input),
                                           base::OptionalOrNullptr(crop_rect)));
@@ -1032,28 +1168,21 @@ void PaintOpReader::ReadTilePaintFilter(
 void PaintOpReader::ReadTurbulencePaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  uint32_t turbulence_type_int = 0;
+  TurbulencePaintFilter::TurbulenceType turbulence_type;
   SkScalar base_frequency_x = 0.f;
   SkScalar base_frequency_y = 0.f;
   int num_octaves = 0;
   SkScalar seed = 0.f;
   SkISize tile_size = SkISize::MakeEmpty();
 
-  Read(&turbulence_type_int);
+  ReadEnum(&turbulence_type);
   Read(&base_frequency_x);
   Read(&base_frequency_y);
   Read(&num_octaves);
   Read(&seed);
   ReadSimple(&tile_size);
-  if (turbulence_type_int >
-      static_cast<uint32_t>(
-          TurbulencePaintFilter::TurbulenceType::kMaxTurbulenceType)) {
-    SetInvalid();
-  }
   if (!valid_)
     return;
-  TurbulencePaintFilter::TurbulenceType turbulence_type =
-      static_cast<TurbulencePaintFilter::TurbulenceType>(turbulence_type_int);
   filter->reset(new TurbulencePaintFilter(
       turbulence_type, base_frequency_x, base_frequency_y, num_octaves, seed,
       &tile_size, base::OptionalOrNullptr(crop_rect)));
@@ -1062,7 +1191,6 @@ void PaintOpReader::ReadTurbulencePaintFilter(
 void PaintOpReader::ReadPaintFlagsPaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  AlignMemory(4);
   PaintFlags flags;
   Read(&flags);
   if (!valid_)
@@ -1079,10 +1207,8 @@ void PaintOpReader::ReadMatrixPaintFilter(
   sk_sp<PaintFilter> input;
 
   Read(&matrix);
-  ReadSimple(&filter_quality);
+  Read(&filter_quality);
   Read(&input);
-  if (filter_quality > kLast_SkFilterQuality)
-    SetInvalid();
   if (!valid_)
     return;
   filter->reset(
@@ -1092,7 +1218,7 @@ void PaintOpReader::ReadMatrixPaintFilter(
 void PaintOpReader::ReadLightingDistantPaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  uint32_t lighting_type_int = 0;
+  PaintFilter::LightingType lighting_type;
   SkPoint3 direction = SkPoint3::Make(0.f, 0.f, 0.f);
   SkColor light_color = SK_ColorBLACK;
   SkScalar surface_scale = 0.f;
@@ -1100,21 +1226,15 @@ void PaintOpReader::ReadLightingDistantPaintFilter(
   SkScalar shininess = 0.f;
   sk_sp<PaintFilter> input;
 
-  Read(&lighting_type_int);
+  ReadEnum(&lighting_type);
   ReadSimple(&direction);
   Read(&light_color);
   Read(&surface_scale);
   Read(&kconstant);
   Read(&shininess);
   Read(&input);
-  if (lighting_type_int >
-      static_cast<uint32_t>(PaintFilter::LightingType::kMaxLightingType)) {
-    SetInvalid();
-  }
   if (!valid_)
     return;
-  PaintFilter::LightingType lighting_type =
-      static_cast<PaintFilter::LightingType>(lighting_type_int);
   filter->reset(new LightingDistantPaintFilter(
       lighting_type, direction, light_color, surface_scale, kconstant,
       shininess, std::move(input), base::OptionalOrNullptr(crop_rect)));
@@ -1123,7 +1243,7 @@ void PaintOpReader::ReadLightingDistantPaintFilter(
 void PaintOpReader::ReadLightingPointPaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  uint32_t lighting_type_int = 0;
+  PaintFilter::LightingType lighting_type;
   SkPoint3 location = SkPoint3::Make(0.f, 0.f, 0.f);
   SkColor light_color = SK_ColorBLACK;
   SkScalar surface_scale = 0.f;
@@ -1131,21 +1251,15 @@ void PaintOpReader::ReadLightingPointPaintFilter(
   SkScalar shininess = 0.f;
   sk_sp<PaintFilter> input;
 
-  Read(&lighting_type_int);
+  ReadEnum(&lighting_type);
   ReadSimple(&location);
   Read(&light_color);
   Read(&surface_scale);
   Read(&kconstant);
   Read(&shininess);
   Read(&input);
-  if (lighting_type_int >
-      static_cast<uint32_t>(PaintFilter::LightingType::kMaxLightingType)) {
-    SetInvalid();
-  }
   if (!valid_)
     return;
-  PaintFilter::LightingType lighting_type =
-      static_cast<PaintFilter::LightingType>(lighting_type_int);
   filter->reset(new LightingPointPaintFilter(
       lighting_type, location, light_color, surface_scale, kconstant, shininess,
       std::move(input), base::OptionalOrNullptr(crop_rect)));
@@ -1154,7 +1268,7 @@ void PaintOpReader::ReadLightingPointPaintFilter(
 void PaintOpReader::ReadLightingSpotPaintFilter(
     sk_sp<PaintFilter>* filter,
     const base::Optional<PaintFilter::CropRect>& crop_rect) {
-  uint32_t lighting_type_int = 0;
+  PaintFilter::LightingType lighting_type;
   SkPoint3 location = SkPoint3::Make(0.f, 0.f, 0.f);
   SkPoint3 target = SkPoint3::Make(0.f, 0.f, 0.f);
   SkScalar specular_exponent = 0.f;
@@ -1165,7 +1279,7 @@ void PaintOpReader::ReadLightingSpotPaintFilter(
   SkScalar shininess = 0.f;
   sk_sp<PaintFilter> input;
 
-  Read(&lighting_type_int);
+  ReadEnum(&lighting_type);
   ReadSimple(&location);
   ReadSimple(&target);
   Read(&specular_exponent);
@@ -1176,14 +1290,8 @@ void PaintOpReader::ReadLightingSpotPaintFilter(
   Read(&shininess);
   Read(&input);
 
-  if (lighting_type_int >
-      static_cast<uint32_t>(PaintFilter::LightingType::kMaxLightingType)) {
-    SetInvalid();
-  }
   if (!valid_)
     return;
-  PaintFilter::LightingType lighting_type =
-      static_cast<PaintFilter::LightingType>(lighting_type_int);
   filter->reset(new LightingSpotPaintFilter(
       lighting_type, location, target, specular_exponent, cutoff_angle,
       light_color, surface_scale, kconstant, shininess, std::move(input),
@@ -1223,7 +1331,7 @@ size_t PaintOpReader::Read(sk_sp<PaintRecord>* record) {
 void PaintOpReader::Read(SkRegion* region) {
   size_t region_bytes = 0;
   ReadSize(&region_bytes);
-  if (region_bytes == 0 || region_bytes > kMaxRegionByteSize)
+  if (region_bytes == 0 || region_bytes > remaining_bytes_)
     SetInvalid();
   if (!valid_)
     return;
