@@ -4,27 +4,31 @@
 
 #include "cc/paint/paint_op_writer.h"
 
+#include <memory>
+
+#include "base/bits.h"
 #include "cc/paint/draw_image.h"
 #include "cc/paint/image_provider.h"
 #include "cc/paint/image_transfer_cache_entry.h"
+#include "cc/paint/paint_cache.h"
 #include "cc/paint/paint_flags.h"
 #include "cc/paint/paint_op_buffer_serializer.h"
 #include "cc/paint/paint_shader.h"
-#include "cc/paint/path_transfer_cache_entry.h"
 #include "cc/paint/transfer_cache_serialize_helper.h"
+#include "gpu/command_buffer/common/mailbox.h"
 #include "third_party/skia/include/core/SkSerialProcs.h"
 #include "third_party/skia/include/core/SkTextBlob.h"
 #include "third_party/skia/src/core/SkRemoteGlyphCache.h"
 #include "ui/gfx/geometry/rect_conversions.h"
 #include "ui/gfx/skia_util.h"
 
+#if !defined(OS_ANDROID)
+#include "cc/paint/skottie_transfer_cache_entry.h"
+#endif
+
 namespace cc {
 namespace {
-const size_t kSkiaAlignment = 4u;
-
-size_t RoundDownToAlignment(size_t bytes, size_t alignment) {
-  return bytes - (bytes & (alignment - 1));
-}
+constexpr size_t kSkiaAlignment = 4u;
 
 SkIRect MakeSrcRect(const PaintImage& image) {
   if (!image)
@@ -38,7 +42,7 @@ SkIRect MakeSrcRect(const PaintImage& image) {
 size_t PaintOpWriter::GetFlattenableSize(const SkFlattenable* flattenable) {
   // The first bit is always written to indicate the serialized size of the
   // flattenable, or zero if it doesn't exist.
-  size_t total_size = sizeof(uint64_t) + alignof(uint64_t);
+  size_t total_size = sizeof(uint64_t) + sizeof(uint64_t) /* alignment */;
   if (!flattenable)
     return total_size;
 
@@ -59,7 +63,7 @@ size_t PaintOpWriter::GetImageSize(const PaintImage& image) {
     image_size += sizeof(info.colorType());
     image_size += sizeof(info.width());
     image_size += sizeof(info.height());
-    image_size += sizeof(uint64_t) + alignof(uint64_t);
+    image_size += sizeof(uint64_t) + sizeof(uint64_t) /* alignment */;
     image_size += info.computeMinByteSize();
   }
   return image_size;
@@ -92,45 +96,48 @@ PaintOpWriter::~PaintOpWriter() = default;
 template <typename T>
 void PaintOpWriter::WriteSimple(const T& val) {
   static_assert(base::is_trivially_copyable<T>::value, "");
-  EnsureBytes(sizeof(T));
+
+  // Round up each write to 4 bytes.  This is not technically perfect alignment,
+  // but it is about 30% faster to post-align each write to 4 bytes than it is
+  // to pre-align memory to the correct alignment.
+  // TODO(enne): maybe we should do this correctly and DCHECK alignment.
+  static constexpr size_t kAlign = 4;
+  size_t size = base::bits::Align(sizeof(T), kAlign);
+  EnsureBytes(size);
   if (!valid_)
     return;
 
   reinterpret_cast<T*>(memory_)[0] = val;
 
-  memory_ += sizeof(T);
-  remaining_bytes_ -= sizeof(T);
+  memory_ += size;
+  remaining_bytes_ -= size;
 }
-
 void PaintOpWriter::WriteFlattenable(const SkFlattenable* val) {
-  AlignMemory(8);
   if (!val) {
     WriteSize(static_cast<size_t>(0u));
     return;
   }
 
-  size_t size_offset = sizeof(uint64_t);
-  EnsureBytes(size_offset);
+  uint64_t* size_memory = WriteSize(0u);
   if (!valid_)
     return;
-  char* size_memory = memory_;
-  memory_ += size_offset;
-  remaining_bytes_ -= size_offset;
 
   size_t bytes_written = val->serialize(
-      memory_, RoundDownToAlignment(remaining_bytes_, kSkiaAlignment));
+      memory_, base::bits::AlignDown(remaining_bytes_, kSkiaAlignment));
   if (bytes_written == 0u) {
     valid_ = false;
     return;
   }
-  reinterpret_cast<uint64_t*>(size_memory)[0] = bytes_written;
+  *size_memory = bytes_written;
   memory_ += bytes_written;
   remaining_bytes_ -= bytes_written;
 }
 
-void PaintOpWriter::WriteSize(size_t size) {
+uint64_t* PaintOpWriter::WriteSize(size_t size) {
   AlignMemory(8);
+  uint64_t* memory = reinterpret_cast<uint64_t*>(memory_);
   WriteSimple<uint64_t>(size);
+  return memory;
 }
 
 void PaintOpWriter::Write(SkScalar data) {
@@ -167,21 +174,43 @@ void PaintOpWriter::Write(const SkRRect& rect) {
 
 void PaintOpWriter::Write(const SkPath& path) {
   auto id = path.getGenerationID();
-  auto locked =
-      options_.transfer_cache->LockEntry(TransferCacheEntryType::kPath, id);
-  if (!locked) {
-    options_.transfer_cache->CreateEntry(ClientPathTransferCacheEntry(path));
-    options_.transfer_cache->AssertLocked(TransferCacheEntryType::kPath, id);
+  if (!options_.for_identifiability_study)
+    Write(id);
+
+  if (options_.paint_cache->Get(PaintCacheDataType::kPath, id)) {
+    Write(static_cast<uint32_t>(PaintCacheEntryState::kCached));
+    return;
   }
-  Write(id);
+
+  // The SkPath may fail to serialize if the bytes required would overflow.
+  uint64_t bytes_required = path.writeToMemory(nullptr);
+  if (bytes_required == 0u) {
+    Write(static_cast<uint32_t>(PaintCacheEntryState::kEmpty));
+    return;
+  }
+
+  Write(static_cast<uint32_t>(PaintCacheEntryState::kInlined));
+  uint64_t* bytes_to_skip = WriteSize(0u);
+  if (!valid_)
+    return;
+
+  if (bytes_required > remaining_bytes_) {
+    valid_ = false;
+    return;
+  }
+  size_t bytes_written = path.writeToMemory(memory_);
+  DCHECK_EQ(bytes_written, bytes_required);
+  options_.paint_cache->Put(PaintCacheDataType::kPath, id, bytes_written);
+  *bytes_to_skip = bytes_written;
+  memory_ += bytes_written;
+  remaining_bytes_ -= bytes_written;
 }
 
 void PaintOpWriter::Write(const PaintFlags& flags) {
-  Write(flags.text_size_);
   WriteSimple(flags.color_);
   Write(flags.width_);
   Write(flags.miter_limit_);
-  WriteSimple(flags.blend_mode_);
+  Write(flags.blend_mode_);
   WriteSimple(flags.bitfields_uint_);
 
   WriteFlattenable(flags.path_effect_.get());
@@ -214,7 +243,7 @@ void PaintOpWriter::Write(const DrawImage& draw_image,
   // Security constrained serialization inlines the image bitmap.
   if (enable_security_constraints_) {
     SkBitmap bm;
-    if (!draw_image.paint_image().GetSkImage()->asLegacyBitmap(&bm)) {
+    if (!draw_image.paint_image().GetSwSkImage()->asLegacyBitmap(&bm)) {
       Write(static_cast<uint8_t>(PaintOp::SerializedImageType::kNoImage));
       return;
     }
@@ -231,15 +260,54 @@ void PaintOpWriter::Write(const DrawImage& draw_image,
   }
 
   // Default mode uses the transfer cache.
-  auto decoded_image = options_.image_provider->GetDecodedDrawImage(draw_image);
+  auto decoded_image = options_.image_provider->GetRasterContent(draw_image);
   DCHECK(!decoded_image.decoded_image().image())
       << "Use transfer cache for image serialization";
   const DecodedDrawImage& decoded_draw_image = decoded_image.decoded_image();
   DCHECK(decoded_draw_image.src_rect_offset().isEmpty())
       << "We shouldn't ask for image subsets";
 
-  base::Optional<uint32_t> id = decoded_draw_image.transfer_cache_entry_id();
   *scale_adjustment = decoded_draw_image.scale_adjustment();
+
+  WriteImage(decoded_draw_image);
+}
+
+// Android does not use skottie. Remove below section to keep binary size to a
+// minimum.
+#if !defined(OS_ANDROID)
+void PaintOpWriter::Write(scoped_refptr<SkottieWrapper> skottie) {
+  uint32_t id = skottie->id();
+  Write(id);
+
+  uint64_t* bytes_to_skip = WriteSize(0u);
+  if (!valid_)
+    return;
+
+  bool locked =
+      options_.transfer_cache->LockEntry(TransferCacheEntryType::kSkottie, id);
+
+  // Add a cache entry for the skottie animation.
+  uint64_t bytes_written = 0u;
+  if (!locked) {
+    bytes_written = options_.transfer_cache->CreateEntry(
+        ClientSkottieTransferCacheEntry(skottie), memory_);
+    options_.transfer_cache->AssertLocked(TransferCacheEntryType::kSkottie, id);
+  }
+
+  DCHECK_LE(bytes_written, remaining_bytes_);
+  *bytes_to_skip = bytes_written;
+  memory_ += bytes_written;
+  remaining_bytes_ -= bytes_written;
+}
+#endif  // !defined(OS_ANDROID)
+
+void PaintOpWriter::WriteImage(const DecodedDrawImage& decoded_draw_image) {
+  if (!decoded_draw_image.mailbox().IsZero()) {
+    WriteImage(decoded_draw_image.mailbox());
+    return;
+  }
+
+  base::Optional<uint32_t> id = decoded_draw_image.transfer_cache_entry_id();
   // In the case of a decode failure, id may not be set. Send an invalid ID.
   WriteImage(id.value_or(kInvalidImageTransferCacheEntryId),
              decoded_draw_image.transfer_cache_entry_needs_mips());
@@ -258,6 +326,20 @@ void PaintOpWriter::WriteImage(uint32_t transfer_cache_entry_id,
   Write(needs_mips);
 }
 
+void PaintOpWriter::WriteImage(const gpu::Mailbox& mailbox) {
+  DCHECK(!mailbox.IsZero());
+
+  Write(static_cast<uint8_t>(PaintOp::SerializedImageType::kMailbox));
+
+  EnsureBytes(sizeof(mailbox.name));
+  if (!valid_)
+    return;
+
+  memcpy(memory_, mailbox.name, sizeof(mailbox.name));
+  memory_ += sizeof(mailbox.name);
+  remaining_bytes_ -= sizeof(mailbox.name);
+}
+
 void PaintOpWriter::Write(const sk_sp<SkData>& data) {
   if (data.get() && data->size()) {
     WriteSize(data->size());
@@ -270,6 +352,17 @@ void PaintOpWriter::Write(const sk_sp<SkData>& data) {
   }
 }
 
+void PaintOpWriter::Write(const SkSamplingOptions& sampling) {
+  Write(sampling.useCubic);
+  if (sampling.useCubic) {
+    Write(sampling.cubic.B);
+    Write(sampling.cubic.C);
+  } else {
+    Write(sampling.filter);
+    Write(sampling.mipmap);
+  }
+}
+
 void PaintOpWriter::Write(const SkColorSpace* color_space) {
   if (!color_space) {
     WriteSize(static_cast<size_t>(0));
@@ -289,22 +382,20 @@ void PaintOpWriter::Write(const SkColorSpace* color_space) {
   remaining_bytes_ -= written;
 }
 
-void PaintOpWriter::Write(const scoped_refptr<PaintTextBlob>& paint_blob) {
-  DCHECK(paint_blob);
+void PaintOpWriter::Write(const sk_sp<SkTextBlob>& blob) {
+  DCHECK(blob);
   if (!valid_)
     return;
 
-  AlignMemory(8);
-
-  const auto& blob = paint_blob->ToSkTextBlob();
-  size_t size_offset = sizeof(uint64_t);
-  EnsureBytes(size_offset);
+  AlignMemory(4);
+  uint32_t blob_id = blob->uniqueID();
+  Write(blob_id);
+  uint64_t* size_memory = WriteSize(0u);
   if (!valid_)
     return;
 
-  char* size_memory = memory_;
-  memory_ += size_offset;
-  remaining_bytes_ -= size_offset;
+  if (options_.paint_cache->Get(PaintCacheDataType::kTextBlob, blob_id))
+    return;
 
   auto encodeTypeface = [](SkTypeface* tf, void* ctx) -> sk_sp<SkData> {
     return static_cast<SkStrikeServer*>(ctx)->serializeTypeface(tf);
@@ -315,12 +406,15 @@ void PaintOpWriter::Write(const scoped_refptr<PaintTextBlob>& paint_blob) {
   procs.fTypefaceCtx = options_.strike_server;
 
   size_t bytes_written = blob->serialize(
-      procs, memory_, RoundDownToAlignment(remaining_bytes_, kSkiaAlignment));
+      procs, memory_, base::bits::AlignDown(remaining_bytes_, kSkiaAlignment));
   if (bytes_written == 0u) {
     valid_ = false;
     return;
   }
-  reinterpret_cast<uint64_t*>(size_memory)[0] = bytes_written;
+
+  options_.paint_cache->Put(PaintCacheDataType::kTextBlob, blob_id,
+                            bytes_written);
+  *size_memory = bytes_written;
   memory_ += bytes_written;
   remaining_bytes_ -= bytes_written;
 }
@@ -330,20 +424,31 @@ sk_sp<PaintShader> PaintOpWriter::TransformShaderIfNecessary(
     SkFilterQuality quality,
     uint32_t* paint_image_transfer_cache_entry_id,
     gfx::SizeF* paint_record_post_scale,
-    bool* paint_image_needs_mips) {
+    bool* paint_image_needs_mips,
+    gpu::Mailbox* mailbox_out) {
   DCHECK(!enable_security_constraints_);
 
   const auto type = original->shader_type();
   const auto& ctm = options_.canvas->getTotalMatrix();
 
   if (type == PaintShader::Type::kImage) {
-    return original->CreateDecodedImage(ctm, quality, options_.image_provider,
-                                        paint_image_transfer_cache_entry_id,
-                                        &quality, paint_image_needs_mips);
+    if (!original->paint_image().IsPaintWorklet()) {
+      return original->CreateDecodedImage(ctm, quality, options_.image_provider,
+                                          paint_image_transfer_cache_entry_id,
+                                          &quality, paint_image_needs_mips,
+                                          mailbox_out);
+    }
+    sk_sp<PaintShader> record_shader =
+        original->CreatePaintWorkletRecord(options_.image_provider);
+    if (!record_shader)
+      return nullptr;
+    return record_shader->CreateScaledPaintRecord(
+        ctm, options_.max_texture_size, paint_record_post_scale);
   }
 
   if (type == PaintShader::Type::kPaintRecord) {
-    return original->CreateScaledPaintRecord(ctm, paint_record_post_scale);
+    return original->CreateScaledPaintRecord(ctm, options_.max_texture_size,
+                                             paint_record_post_scale);
   }
 
   return sk_ref_sp<PaintShader>(original);
@@ -355,16 +460,21 @@ void PaintOpWriter::Write(SkMatrix matrix) {
   WriteSimple(matrix);
 }
 
+void PaintOpWriter::Write(const SkM44& matrix) {
+  WriteSimple(matrix);
+}
+
 void PaintOpWriter::Write(const PaintShader* shader, SkFilterQuality quality) {
   sk_sp<PaintShader> transformed_shader;
   uint32_t paint_image_transfer_cache_id = kInvalidImageTransferCacheEntryId;
   gfx::SizeF paint_record_post_scale(1.f, 1.f);
   bool paint_image_needs_mips = false;
+  gpu::Mailbox mailbox;
 
   if (!enable_security_constraints_ && shader) {
     transformed_shader = TransformShaderIfNecessary(
         shader, quality, &paint_image_transfer_cache_id,
-        &paint_record_post_scale, &paint_image_needs_mips);
+        &paint_record_post_scale, &paint_image_needs_mips, &mailbox);
     shader = transformed_shader.get();
   }
 
@@ -381,8 +491,8 @@ void PaintOpWriter::Write(const PaintShader* shader, SkFilterQuality quality) {
   WriteSimple(shader->flags_);
   WriteSimple(shader->end_radius_);
   WriteSimple(shader->start_radius_);
-  WriteSimple(shader->tx_);
-  WriteSimple(shader->ty_);
+  Write(shader->tx_);
+  Write(shader->ty_);
   WriteSimple(shader->fallback_color_);
   WriteSimple(shader->scaling_behavior_);
   if (shader->local_matrix_) {
@@ -399,20 +509,24 @@ void PaintOpWriter::Write(const PaintShader* shader, SkFilterQuality quality) {
   WriteSimple(shader->end_degrees_);
 
   if (enable_security_constraints_) {
-    DrawImage draw_image(shader->image_, MakeSrcRect(shader->image_), quality,
-                         SkMatrix::I());
+    DrawImage draw_image(shader->image_, false, MakeSrcRect(shader->image_),
+                         quality, SkMatrix::I());
     SkSize scale_adjustment = SkSize::Make(1.f, 1.f);
     Write(draw_image, &scale_adjustment);
     DCHECK_EQ(scale_adjustment.width(), 1.f);
     DCHECK_EQ(scale_adjustment.height(), 1.f);
   } else {
-    WriteImage(paint_image_transfer_cache_id, paint_image_needs_mips);
+    if (!mailbox.IsZero())
+      WriteImage(mailbox);
+    else
+      WriteImage(paint_image_transfer_cache_id, paint_image_needs_mips);
   }
 
   if (shader->record_) {
     Write(true);
     DCHECK_NE(shader->id_, PaintShader::kInvalidRecordShaderId);
-    Write(shader->id_);
+    if (!options_.for_identifiability_study)
+      Write(shader->id_);
     const gfx::Rect playback_rect(
         gfx::ToEnclosingRect(gfx::SkRectToRectF(shader->tile())));
 
@@ -433,8 +547,16 @@ void PaintOpWriter::Write(const PaintShader* shader, SkFilterQuality quality) {
   // using other fields.
 }
 
-void PaintOpWriter::Write(SkColorType color_type) {
-  WriteSimple(static_cast<uint32_t>(color_type));
+void PaintOpWriter::Write(SkYUVColorSpace yuv_color_space) {
+  WriteSimple(static_cast<uint32_t>(yuv_color_space));
+}
+
+void PaintOpWriter::Write(SkYUVAInfo::PlaneConfig plane_config) {
+  WriteSimple(static_cast<uint32_t>(plane_config));
+}
+
+void PaintOpWriter::Write(SkYUVAInfo::Subsampling subsampling) {
+  WriteSimple(static_cast<uint32_t>(subsampling));
 }
 
 void PaintOpWriter::WriteData(size_t bytes, const void* input) {
@@ -470,21 +592,20 @@ void PaintOpWriter::AlignMemory(size_t alignment) {
 
 void PaintOpWriter::Write(const PaintFilter* filter) {
   if (!filter) {
-    WriteSimple(static_cast<uint32_t>(PaintFilter::Type::kNullFilter));
+    WriteEnum(PaintFilter::Type::kNullFilter);
     return;
   }
-  WriteSimple(static_cast<uint32_t>(filter->type()));
+  WriteEnum(filter->type());
   auto* crop_rect = filter->crop_rect();
   WriteSimple(static_cast<uint32_t>(!!crop_rect));
   if (crop_rect) {
-    WriteSimple(crop_rect->flags());
-    WriteSimple(crop_rect->rect());
+    WriteSimple(*crop_rect);
   }
 
   if (!valid_)
     return;
 
-  AlignMemory(4);
+  AlignMemory(kSkiaAlignment);
   switch (filter->type()) {
     case PaintFilter::Type::kNullFilter:
       NOTREACHED();
@@ -566,7 +687,7 @@ void PaintOpWriter::Write(const ColorFilterPaintFilter& filter) {
 void PaintOpWriter::Write(const BlurPaintFilter& filter) {
   WriteSimple(filter.sigma_x());
   WriteSimple(filter.sigma_y());
-  WriteSimple(filter.tile_mode());
+  Write(filter.tile_mode());
   Write(filter.input().get());
 }
 
@@ -576,7 +697,7 @@ void PaintOpWriter::Write(const DropShadowPaintFilter& filter) {
   WriteSimple(filter.sigma_x());
   WriteSimple(filter.sigma_y());
   WriteSimple(filter.color());
-  WriteSimple(filter.shadow_mode());
+  WriteEnum(filter.shadow_mode());
   Write(filter.input().get());
 }
 
@@ -599,7 +720,7 @@ void PaintOpWriter::Write(const AlphaThresholdPaintFilter& filter) {
 }
 
 void PaintOpWriter::Write(const XfermodePaintFilter& filter) {
-  WriteSimple(static_cast<uint32_t>(filter.blend_mode()));
+  Write(filter.blend_mode());
   Write(filter.background().get());
   Write(filter.foreground().get());
 }
@@ -623,14 +744,14 @@ void PaintOpWriter::Write(const MatrixConvolutionPaintFilter& filter) {
   WriteSimple(filter.gain());
   WriteSimple(filter.bias());
   WriteSimple(filter.kernel_offset());
-  WriteSimple(static_cast<uint32_t>(filter.tile_mode()));
+  Write(filter.tile_mode());
   WriteSimple(filter.convolve_alpha());
   Write(filter.input().get());
 }
 
 void PaintOpWriter::Write(const DisplacementMapEffectPaintFilter& filter) {
-  WriteSimple(static_cast<uint32_t>(filter.channel_x()));
-  WriteSimple(static_cast<uint32_t>(filter.channel_y()));
+  WriteEnum(filter.channel_x());
+  WriteEnum(filter.channel_y());
   WriteSimple(filter.scale());
   Write(filter.displacement().get());
   Write(filter.color().get());
@@ -638,7 +759,7 @@ void PaintOpWriter::Write(const DisplacementMapEffectPaintFilter& filter) {
 
 void PaintOpWriter::Write(const ImagePaintFilter& filter) {
   DrawImage draw_image(
-      filter.image(),
+      filter.image(), false,
       SkIRect::MakeWH(filter.image().width(), filter.image().height()),
       filter.filter_quality(), SkMatrix::I());
   SkSize scale_adjustment = SkSize::Make(1.f, 1.f);
@@ -653,18 +774,32 @@ void PaintOpWriter::Write(const ImagePaintFilter& filter) {
 
 void PaintOpWriter::Write(const RecordPaintFilter& filter) {
   WriteSimple(filter.record_bounds());
-  Write(filter.record().get(), gfx::Rect(), gfx::SizeF(1.f, 1.f),
-        options_.canvas ? options_.canvas->getTotalMatrix() : SkMatrix::I());
+  if (!options_.canvas) {
+    Write(filter.record().get(), gfx::Rect(), gfx::SizeF(1.f, 1.f),
+          SkMatrix::I());
+    return;
+  }
+
+  // The logic here to only use the scale component of the matrix during
+  // analysis is for consistency with the rasterization of the filter later in
+  // pipeline in skia. For every draw with a filter, SkCanvas creates a layer
+  // for the draw and modifies the scale for these filters.
+  // See SkCanvas::internalSaveLayer.
+  SkMatrix mat = options_.canvas->getTotalMatrix();
+  SkSize scale;
+  if (!mat.isScaleTranslate() && mat.decomposeScale(&scale))
+    mat = SkMatrix::Scale(scale.width(), scale.height());
+  Write(filter.record().get(), gfx::Rect(), gfx::SizeF(1.f, 1.f), mat);
 }
 
 void PaintOpWriter::Write(const MergePaintFilter& filter) {
-  WriteSimple(filter.input_count());
+  WriteSize(filter.input_count());
   for (size_t i = 0; i < filter.input_count(); ++i)
     Write(filter.input_at(i));
 }
 
 void PaintOpWriter::Write(const MorphologyPaintFilter& filter) {
-  WriteSimple(filter.morph_type());
+  WriteEnum(filter.morph_type());
   WriteSimple(filter.radius_x());
   WriteSimple(filter.radius_y());
   Write(filter.input().get());
@@ -683,7 +818,7 @@ void PaintOpWriter::Write(const TilePaintFilter& filter) {
 }
 
 void PaintOpWriter::Write(const TurbulencePaintFilter& filter) {
-  WriteSimple(filter.turbulence_type());
+  WriteEnum(filter.turbulence_type());
   WriteSimple(filter.base_frequency_x());
   WriteSimple(filter.base_frequency_y());
   WriteSimple(filter.num_octaves());
@@ -697,12 +832,12 @@ void PaintOpWriter::Write(const PaintFlagsPaintFilter& filter) {
 
 void PaintOpWriter::Write(const MatrixPaintFilter& filter) {
   Write(filter.matrix());
-  WriteSimple(filter.filter_quality());
+  Write(filter.filter_quality());
   Write(filter.input().get());
 }
 
 void PaintOpWriter::Write(const LightingDistantPaintFilter& filter) {
-  WriteSimple(filter.lighting_type());
+  WriteEnum(filter.lighting_type());
   WriteSimple(filter.direction());
   WriteSimple(filter.light_color());
   WriteSimple(filter.surface_scale());
@@ -712,7 +847,7 @@ void PaintOpWriter::Write(const LightingDistantPaintFilter& filter) {
 }
 
 void PaintOpWriter::Write(const LightingPointPaintFilter& filter) {
-  WriteSimple(filter.lighting_type());
+  WriteEnum(filter.lighting_type());
   WriteSimple(filter.location());
   WriteSimple(filter.light_color());
   WriteSimple(filter.surface_scale());
@@ -722,7 +857,7 @@ void PaintOpWriter::Write(const LightingPointPaintFilter& filter) {
 }
 
 void PaintOpWriter::Write(const LightingSpotPaintFilter& filter) {
-  WriteSimple(filter.lighting_type());
+  WriteEnum(filter.lighting_type());
   WriteSimple(filter.location());
   WriteSimple(filter.target());
   WriteSimple(filter.specular_exponent());
@@ -748,16 +883,12 @@ void PaintOpWriter::Write(const PaintRecord* record,
   if (!valid_)
     return;
 
-  char* size_memory = memory_;
-
-  memory_ += size_offset;
-  remaining_bytes_ -= size_offset;
+  uint64_t* size_memory = WriteSize(0u);
   if (!valid_)
     return;
 
   if (enable_security_constraints_) {
     // We don't serialize PaintRecords when security constraints are enabled.
-    reinterpret_cast<size_t*>(size_memory)[0] = 0u;
     return;
   }
 
@@ -767,9 +898,9 @@ void PaintOpWriter::Write(const PaintRecord* record,
   const bool can_use_lcd_text = false;
   SimpleBufferSerializer serializer(
       memory_, remaining_bytes_, options_.image_provider,
-      options_.transfer_cache, options_.strike_server, options_.color_space,
-      can_use_lcd_text, options_.context_supports_distance_field_text,
-      options_.max_texture_size, options_.max_texture_bytes);
+      options_.transfer_cache, options_.paint_cache, options_.strike_server,
+      options_.color_space, can_use_lcd_text,
+      options_.context_supports_distance_field_text, options_.max_texture_size);
   serializer.Serialize(record, playback_rect, post_scale,
                        post_matrix_for_analysis);
 
@@ -784,7 +915,7 @@ void PaintOpWriter::Write(const PaintRecord* record,
 
   // Write the size to the size memory, which preceeds the memory for the
   // record.
-  reinterpret_cast<uint64_t*>(size_memory)[0] = serializer.written();
+  *size_memory = serializer.written();
 
   // The serializer should have failed if it ran out of space. DCHECK to verify
   // that it wrote at most as many bytes as we had left.
