@@ -7,15 +7,22 @@
 
 #include <unordered_set>
 
+#include "build/build_config.h"
 #include "cc/paint/paint_canvas.h"
 #include "cc/paint/paint_export.h"
 #include "cc/paint/paint_filter.h"
 #include "cc/paint/paint_op_buffer_serializer.h"
+#include "third_party/skia/include/core/SkImageInfo.h"
+#include "third_party/skia/include/core/SkYUVAInfo.h"
 
 struct SkRect;
 struct SkIRect;
 class SkRRect;
 
+namespace gpu {
+struct Mailbox;
+}
+
 namespace cc {
 
 class DrawImage;
@@ -42,10 +49,11 @@ class CC_PAINT_EXPORT PaintOpWriter {
 
   size_t size() const { return valid_ ? size_ - remaining_bytes_ : 0u; }
 
-  void WriteSize(size_t size);
+  uint64_t* WriteSize(size_t size);
 
   void Write(SkScalar data);
   void Write(SkMatrix data);
+  void Write(const SkM44& data);
   void Write(uint8_t data);
   void Write(uint32_t data);
   void Write(uint64_t data);
@@ -58,21 +66,25 @@ class CC_PAINT_EXPORT PaintOpWriter {
   void Write(const PaintFlags& flags);
   void Write(const sk_sp<SkData>& data);
   void Write(const SkColorSpace* data);
+  void Write(const SkSamplingOptions&);
   void Write(const PaintShader* shader, SkFilterQuality quality);
   void Write(const PaintFilter* filter);
-  void Write(const scoped_refptr<PaintTextBlob>& blob);
-  void Write(SkColorType color_type);
+  void Write(const sk_sp<SkTextBlob>& blob);
+  void Write(SkYUVColorSpace yuv_color_space);
+  void Write(SkYUVAInfo::PlaneConfig plane_config);
+  void Write(SkYUVAInfo::Subsampling subsampling);
+  void Write(const gpu::Mailbox& mailbox);
+
+  void Write(SkClipOp op) { WriteEnum(op); }
+  void Write(PaintCanvas::AnnotationType type) { WriteEnum(type); }
+  void Write(SkCanvas::SrcRectConstraint constraint) { WriteEnum(constraint); }
+  void Write(SkColorType color_type) { WriteEnum(color_type); }
+  void Write(SkFilterQuality filter_quality) { WriteEnum(filter_quality); }
+  void Write(SkBlendMode blend_mode) { WriteEnum(blend_mode); }
+  void Write(SkTileMode tile_mode) { WriteEnum(tile_mode); }
+  void Write(SkFilterMode filter_mode) { WriteEnum(filter_mode); }
+  void Write(SkMipmapMode mipmap_mode) { WriteEnum(mipmap_mode); }
 
-  void Write(SkClipOp op) { Write(static_cast<uint8_t>(op)); }
-  void Write(PaintCanvas::AnnotationType type) {
-    Write(static_cast<uint8_t>(type));
-  }
-  void Write(PaintCanvas::SrcRectConstraint constraint) {
-    Write(static_cast<uint8_t>(constraint));
-  }
-  void Write(SkFilterQuality filter_quality) {
-    Write(static_cast<uint8_t>(filter_quality));
-  }
   void Write(bool data) { Write(static_cast<uint8_t>(data)); }
 
   // Aligns the memory to the given alignment.
@@ -101,12 +113,22 @@ class CC_PAINT_EXPORT PaintOpWriter {
   // image.
   void Write(const DrawImage& draw_image, SkSize* scale_adjustment);
 
+#if !defined(OS_ANDROID)
+  // Serializes the given |skottie| vector graphic.
+  void Write(scoped_refptr<SkottieWrapper> skottie);
+#endif
+
  private:
   template <typename T>
   void WriteSimple(const T& val);
 
   void WriteFlattenable(const SkFlattenable* val);
 
+  template <typename Enum>
+  void WriteEnum(Enum value) {
+    Write(base::checked_cast<uint8_t>(value));
+  }
+
   // The main entry point is Write(const PaintFilter* filter) which casts the
   // filter and calls one of the following functions.
   void Write(const ColorFilterPaintFilter& filter);
@@ -137,7 +159,9 @@ class CC_PAINT_EXPORT PaintOpWriter {
              const gfx::SizeF& post_scale,
              const SkMatrix& post_matrix_for_analysis);
   void Write(const SkRegion& region);
+  void WriteImage(const DecodedDrawImage& decoded_draw_image);
   void WriteImage(uint32_t transfer_cache_entry_id, bool needs_mips);
+  void WriteImage(const gpu::Mailbox& mailbox);
 
   void EnsureBytes(size_t required_bytes);
   sk_sp<PaintShader> TransformShaderIfNecessary(
@@ -145,7 +169,8 @@ class CC_PAINT_EXPORT PaintOpWriter {
       SkFilterQuality quality,
       uint32_t* paint_image_transfer_cache_entry_id,
       gfx::SizeF* paint_record_post_scale,
-      bool* paint_image_needs_mips);
+      bool* paint_image_needs_mips,
+      gpu::Mailbox* mailbox_out);
 
   char* memory_ = nullptr;
   size_t size_ = 0u;
