@@ -20,7 +20,9 @@ sk_sp<SkPicture> ToSkPicture(sk_sp<PaintRecord> record,
                              const gfx::SizeF* raster_scale,
                              ImageProvider* image_provider) {
   SkPictureRecorder recorder;
-  SkCanvas* canvas = recorder.beginRecording(bounds);
+  SkCanvas* canvas =
+      recorder.beginRecording(SkRect::MakeWH(bounds.width(), bounds.height()));
+  canvas->translate(-bounds.fLeft, -bounds.fTop);
   if (raster_scale)
     canvas->scale(raster_scale->width(), raster_scale->height());
   record->Playback(canvas, PlaybackParams(image_provider));
@@ -36,18 +38,41 @@ bool CompareMatrices(const SkMatrix& a,
   SkSize scale;
   SkMatrix a_without_scale;
   SkMatrix b_without_scale;
-  if (a.decomposeScale(&scale, &a_without_scale) !=
-      b.decomposeScale(&scale, &b_without_scale)) {
-    return false;
-  }
 
+  const bool decomposes = a.decomposeScale(&scale, &a_without_scale);
+  if (decomposes != b.decomposeScale(&scale, &b_without_scale))
+    return false;
+
+  if (!decomposes)
+    return true;
   return PaintOp::AreSkMatricesEqual(a_without_scale, b_without_scale);
 }
 
+SkRect AdjustForMaxTextureSize(SkRect tile, int max_texture_size) {
+  if (max_texture_size == 0)
+    return tile;
+
+  if (tile.width() < max_texture_size && tile.height() < max_texture_size)
+    return tile;
+
+  float down_scale = max_texture_size / std::max(tile.width(), tile.height());
+  tile = SkRect::MakeXYWH(tile.x(), tile.y(),
+                          SkScalarFloorToScalar(tile.width() * down_scale),
+                          SkScalarFloorToScalar(tile.height() * down_scale));
+  return tile;
+}
+
 }  // namespace
 
 const PaintShader::RecordShaderId PaintShader::kInvalidRecordShaderId = -1;
 
+sk_sp<PaintShader> PaintShader::MakeEmpty() {
+  sk_sp<PaintShader> shader(new PaintShader(Type::kEmpty));
+
+  shader->CreateSkShader();
+  return shader;
+}
+
 sk_sp<PaintShader> PaintShader::MakeColor(SkColor color) {
   sk_sp<PaintShader> shader(new PaintShader(Type::kColor));
 
@@ -62,7 +87,7 @@ sk_sp<PaintShader> PaintShader::MakeLinearGradient(const SkPoint points[],
                                                    const SkColor colors[],
                                                    const SkScalar pos[],
                                                    int count,
-                                                   SkShader::TileMode mode,
+                                                   SkTileMode mode,
                                                    uint32_t flags,
                                                    const SkMatrix* local_matrix,
                                                    SkColor fallback_color) {
@@ -84,7 +109,7 @@ sk_sp<PaintShader> PaintShader::MakeRadialGradient(const SkPoint& center,
                                                    const SkColor colors[],
                                                    const SkScalar pos[],
                                                    int count,
-                                                   SkShader::TileMode mode,
+                                                   SkTileMode mode,
                                                    uint32_t flags,
                                                    const SkMatrix* local_matrix,
                                                    SkColor fallback_color) {
@@ -108,7 +133,7 @@ sk_sp<PaintShader> PaintShader::MakeTwoPointConicalGradient(
     const SkColor colors[],
     const SkScalar pos[],
     int count,
-    SkShader::TileMode mode,
+    SkTileMode mode,
     uint32_t flags,
     const SkMatrix* local_matrix,
     SkColor fallback_color) {
@@ -131,7 +156,7 @@ sk_sp<PaintShader> PaintShader::MakeSweepGradient(SkScalar cx,
                                                   const SkColor colors[],
                                                   const SkScalar pos[],
                                                   int color_count,
-                                                  SkShader::TileMode mode,
+                                                  SkTileMode mode,
                                                   SkScalar start_degrees,
                                                   SkScalar end_degrees,
                                                   uint32_t flags,
@@ -151,13 +176,18 @@ sk_sp<PaintShader> PaintShader::MakeSweepGradient(SkScalar cx,
 }
 
 sk_sp<PaintShader> PaintShader::MakeImage(const PaintImage& image,
-                                          SkShader::TileMode tx,
-                                          SkShader::TileMode ty,
-                                          const SkMatrix* local_matrix) {
+                                          SkTileMode tx,
+                                          SkTileMode ty,
+                                          const SkMatrix* local_matrix,
+                                          const SkRect* tile_rect) {
   sk_sp<PaintShader> shader(new PaintShader(Type::kImage));
 
   shader->image_ = image;
   shader->SetMatrixAndTiling(local_matrix, tx, ty);
+  if (tile_rect) {
+    DCHECK(image.IsPaintWorklet());
+    shader->tile_ = *tile_rect;
+  }
 
   shader->CreateSkShader();
   return shader;
@@ -166,8 +196,8 @@ sk_sp<PaintShader> PaintShader::MakeImage(const PaintImage& image,
 sk_sp<PaintShader> PaintShader::MakePaintRecord(
     sk_sp<PaintRecord> record,
     const SkRect& tile,
-    SkShader::TileMode tx,
-    SkShader::TileMode ty,
+    SkTileMode tx,
+    SkTileMode ty,
     const SkMatrix* local_matrix,
     ScalingBehavior scaling_behavior) {
   sk_sp<PaintShader> shader(new PaintShader(Type::kPaintRecord));
@@ -210,7 +240,7 @@ PaintShader::PaintShader(Type type) : shader_type_(type) {}
 PaintShader::~PaintShader() = default;
 
 bool PaintShader::has_discardable_images() const {
-  return (image_ && image_.IsLazyGenerated()) ||
+  return (image_ && !image_.IsTextureBacked()) ||
          (record_ && record_->HasDiscardableImages());
 }
 
@@ -237,33 +267,41 @@ bool PaintShader::GetRasterizationTileRect(const SkMatrix& ctm,
               SkScalarSqrt(matrix.getScaleY() * matrix.getScaleY() +
                            matrix.getSkewY() * matrix.getSkewY()));
   }
-  SkSize scaled_size =
-      SkSize::Make(SkScalarAbs(scale.width() * tile_.width()),
-                   SkScalarAbs(scale.height() * tile_.height()));
+
+  SkScalar tile_area =
+      tile_.width() * tile_.height() * scale.width() * scale.height();
 
   // Clamp the tile size to about 4M pixels.
   // TODO(khushalsagar): We need to consider the max texture size as well.
   static const SkScalar kMaxTileArea = 2048 * 2048;
-  SkScalar tile_area = scaled_size.width() * scaled_size.height();
   if (tile_area > kMaxTileArea) {
     SkScalar clamp_scale = SkScalarSqrt(kMaxTileArea / tile_area);
-    scaled_size.set(scaled_size.width() * clamp_scale,
-                    scaled_size.height() * clamp_scale);
+    scale.set(clamp_scale, clamp_scale);
   }
 
-  scaled_size = scaled_size.toCeil();
-  if (scaled_size.isEmpty())
+  *tile_rect = SkRect::MakeXYWH(
+      tile_.fLeft * scale.width(), tile_.fTop * scale.height(),
+      SkScalarCeilToInt(SkScalarAbs(scale.width() * tile_.width())),
+      SkScalarCeilToInt(SkScalarAbs(scale.height() * tile_.height())));
+
+  if (tile_rect->isEmpty())
     return false;
 
-  *tile_rect = SkRect::MakeWH(scaled_size.width(), scaled_size.height());
   return true;
 }
 
 sk_sp<PaintShader> PaintShader::CreateScaledPaintRecord(
     const SkMatrix& ctm,
+    int max_texture_size,
     gfx::SizeF* raster_scale) const {
   DCHECK_EQ(shader_type_, Type::kPaintRecord);
 
+  // If this is already fixed scale, then this is already good to go.
+  if (scaling_behavior_ == ScalingBehavior::kFixedScale) {
+    *raster_scale = gfx::SizeF(1.f, 1.f);
+    return sk_ref_sp<PaintShader>(this);
+  }
+
   // For creating a decoded PaintRecord shader, we need to do the following:
   // 1) Figure out the scale at which the record should be rasterization given
   //    the ctm and local_matrix on the shader.
@@ -279,6 +317,7 @@ sk_sp<PaintShader> PaintShader::CreateScaledPaintRecord(
   SkRect tile_rect;
   if (!GetRasterizationTileRect(ctm, &tile_rect))
     return nullptr;
+  tile_rect = AdjustForMaxTextureSize(tile_rect, max_texture_size);
 
   sk_sp<PaintShader> shader(new PaintShader(Type::kPaintRecord));
   shader->record_ = record_;
@@ -300,12 +339,29 @@ sk_sp<PaintShader> PaintShader::CreateScaledPaintRecord(
   return shader;
 }
 
+sk_sp<PaintShader> PaintShader::CreatePaintWorkletRecord(
+    ImageProvider* image_provider) const {
+  DCHECK_EQ(shader_type_, Type::kImage);
+  DCHECK(image_ && image_.IsPaintWorklet());
+
+  ImageProvider::ScopedResult result =
+      image_provider->GetRasterContent(DrawImage(image_));
+  if (!result || !result.paint_record())
+    return nullptr;
+  SkMatrix local_matrix = GetLocalMatrix();
+  return PaintShader::MakePaintRecord(
+      sk_ref_sp<PaintRecord>(result.paint_record()), tile_, tx_, ty_,
+      &local_matrix);
+}
+
 sk_sp<PaintShader> PaintShader::CreateDecodedImage(
     const SkMatrix& ctm,
     SkFilterQuality quality,
     ImageProvider* image_provider,
     uint32_t* transfer_cache_entry_id,
-    SkFilterQuality* raster_quality) const {
+    SkFilterQuality* raster_quality,
+    bool* needs_mips,
+    gpu::Mailbox* mailbox) const {
   DCHECK_EQ(shader_type_, Type::kImage);
   if (!image_)
     return nullptr;
@@ -315,8 +371,9 @@ sk_sp<PaintShader> PaintShader::CreateDecodedImage(
   SkRect src_rect = SkRect::MakeIWH(image_.width(), image_.height());
   SkIRect int_src_rect;
   src_rect.roundOut(&int_src_rect);
-  DrawImage draw_image(image_, int_src_rect, quality, total_image_matrix);
-  auto decoded_draw_image = image_provider->GetDecodedDrawImage(draw_image);
+  DrawImage draw_image(image_, false, int_src_rect, quality,
+                       total_image_matrix);
+  auto decoded_draw_image = image_provider->GetRasterContent(draw_image);
   if (!decoded_draw_image)
     return nullptr;
 
@@ -332,6 +389,9 @@ sk_sp<PaintShader> PaintShader::CreateDecodedImage(
   if (decoded_image.transfer_cache_entry_id()) {
     decoded_paint_image = image_;
     *transfer_cache_entry_id = *decoded_image.transfer_cache_entry_id();
+  } else if (!decoded_image.mailbox().IsZero()) {
+    decoded_paint_image = image_;
+    *mailbox = decoded_image.mailbox();
   } else {
     DCHECK(decoded_image.image());
 
@@ -340,7 +400,8 @@ sk_sp<PaintShader> PaintShader::CreateDecodedImage(
     decoded_paint_image =
         PaintImageBuilder::WithDefault()
             .set_id(image_.stable_id())
-            .set_image(std::move(sk_image), image_.content_id())
+            .set_texture_image(std::move(sk_image),
+                               image_.GetContentIdForFrame(0u))
             .TakePaintImage();
   }
 
@@ -348,6 +409,7 @@ sk_sp<PaintShader> PaintShader::CreateDecodedImage(
   // want to do is cap the filter quality used, but Gpu and Sw cache have
   // different behaviour. D:
   *raster_quality = decoded_image.filter_quality();
+  *needs_mips = decoded_image.transfer_cache_entry_needs_mips();
   return PaintShader::MakeImage(decoded_paint_image, tx_, ty_, &final_matrix);
 }
 
@@ -360,6 +422,9 @@ void PaintShader::CreateSkShader(const gfx::SizeF* raster_scale,
   DCHECK(!cached_shader_);
 
   switch (shader_type_) {
+    case Type::kEmpty:
+      cached_shader_ = SkShaders::Empty();
+      break;
     case Type::kColor:
       // This will be handled by the fallback check below.
       break;
@@ -394,7 +459,7 @@ void PaintShader::CreateSkShader(const gfx::SizeF* raster_scale,
           flags_, base::OptionalOrNullptr(local_matrix_));
       break;
     case Type::kImage:
-      if (image_) {
+      if (image_ && !image_.IsPaintWorklet()) {
         cached_shader_ = image_.GetSkImage()->makeShader(
             tx_, ty_, base::OptionalOrNullptr(local_matrix_));
       }
@@ -407,9 +472,8 @@ void PaintShader::CreateSkShader(const gfx::SizeF* raster_scale,
       switch (scaling_behavior_) {
         // For raster scale, we create a picture shader directly.
         case ScalingBehavior::kRasterAtScale:
-          cached_shader_ = SkShader::MakePictureShader(
-              std::move(picture), tx_, ty_,
-              base::OptionalOrNullptr(local_matrix_), nullptr);
+          cached_shader_ = picture->makeShader(
+              tx_, ty_, base::OptionalOrNullptr(local_matrix_), nullptr);
           break;
         // For fixed scale, we create an image shader with an image backed by
         // the picture.
@@ -433,7 +497,7 @@ void PaintShader::CreateSkShader(const gfx::SizeF* raster_scale,
   // If we didn't create a shader for whatever reason, create a fallback color
   // one.
   if (!cached_shader_)
-    cached_shader_ = SkShader::MakeColorShader(fallback_color_);
+    cached_shader_ = SkShaders::Color(fallback_color_);
 }
 
 void PaintShader::SetColorsAndPositions(const SkColor* colors,
@@ -450,8 +514,8 @@ void PaintShader::SetColorsAndPositions(const SkColor* colors,
 }
 
 void PaintShader::SetMatrixAndTiling(const SkMatrix* matrix,
-                                     SkShader::TileMode tx,
-                                     SkShader::TileMode ty) {
+                                     SkTileMode tx,
+                                     SkTileMode ty) {
   if (matrix)
     local_matrix_ = *matrix;
   tx_ = tx;
@@ -474,6 +538,7 @@ bool PaintShader::IsValid() const {
     return true;
 
   switch (shader_type_) {
+    case Type::kEmpty:
     case Type::kColor:
       return true;
     case Type::kSweepGradient:
@@ -532,6 +597,7 @@ bool PaintShader::operator==(const PaintShader& other) const {
 
   // Variables that only some shaders use.
   switch (shader_type_) {
+    case Type::kEmpty:
     case Type::kColor:
       break;
     case Type::kSweepGradient:
