@@ -4,7 +4,8 @@
 
 #include "cc/test/fake_content_layer_client.h"
 
-#include <stddef.h>
+#include <algorithm>
+#include <cstddef>
 
 #include "cc/paint/paint_op_buffer.h"
 #include "ui/gfx/geometry/rect_conversions.h"
@@ -14,36 +15,34 @@ namespace cc {
 
 FakeContentLayerClient::ImageData::ImageData(PaintImage img,
                                              const gfx::Point& point,
+                                             const SkSamplingOptions& sampling,
                                              const PaintFlags& flags)
-    : image(std::move(img)), point(point), flags(flags) {}
+    : image(std::move(img)), point(point), sampling(sampling), flags(flags) {}
 
 FakeContentLayerClient::ImageData::ImageData(PaintImage img,
                                              const gfx::Transform& transform,
+                                             const SkSamplingOptions& sampling,
                                              const PaintFlags& flags)
-    : image(std::move(img)), transform(transform), flags(flags) {}
+    : image(std::move(img)),
+      transform(transform),
+      sampling(sampling),
+      flags(flags) {}
 
 FakeContentLayerClient::ImageData::ImageData(const ImageData& other) = default;
 
 FakeContentLayerClient::ImageData::~ImageData() = default;
 
-FakeContentLayerClient::FakeContentLayerClient()
-    : fill_with_nonsolid_color_(false),
-      last_canvas_(nullptr),
-      last_painting_control_(PAINTING_BEHAVIOR_NORMAL),
-      reported_memory_usage_(0),
-      bounds_set_(false),
-      contains_slow_paths_(false) {}
+FakeContentLayerClient::FakeContentLayerClient() = default;
 
 FakeContentLayerClient::~FakeContentLayerClient() = default;
 
-gfx::Rect FakeContentLayerClient::PaintableRegion() {
+gfx::Rect FakeContentLayerClient::PaintableRegion() const {
   CHECK(bounds_set_);
   return gfx::Rect(bounds_);
 }
 
 scoped_refptr<DisplayItemList>
-FakeContentLayerClient::PaintContentsToDisplayList(
-    PaintingControlSetting painting_control) {
+FakeContentLayerClient::PaintContentsToDisplayList() {
   auto display_list = base::MakeRefCounted<DisplayItemList>();
 
   for (RectPaintVector::const_iterator it = draw_rects_.begin();
@@ -61,8 +60,7 @@ FakeContentLayerClient::PaintContentsToDisplayList(
     if (!it->transform.IsIdentity()) {
       display_list->StartPaint();
       display_list->push<SaveOp>();
-      display_list->push<ConcatOp>(
-          static_cast<SkMatrix>(it->transform.matrix()));
+      display_list->push<ConcatOp>(it->transform.GetMatrixAsSkM44());
       display_list->EndPaintOfPairedBegin();
     }
 
@@ -72,7 +70,10 @@ FakeContentLayerClient::PaintContentsToDisplayList(
                                    SkClipOp::kIntersect, false);
     display_list->push<DrawImageOp>(
         it->image, static_cast<float>(it->point.x()),
-        static_cast<float>(it->point.y()), &it->flags);
+        static_cast<float>(it->point.y()),
+        SkSamplingOptions(it->flags.getFilterQuality(),
+                          SkSamplingOptions::kMedium_asMipmapLinear),
+        &it->flags);
     display_list->push<RestoreOp>();
     display_list->EndPaintOfUnpaired(PaintableRegion());
 
@@ -108,14 +109,25 @@ FakeContentLayerClient::PaintContentsToDisplayList(
     display_list->EndPaintOfUnpaired(PaintableRegion());
   }
 
+  if (has_non_aa_paint_) {
+    PaintFlags flags;
+    flags.setAntiAlias(false);
+    display_list->StartPaint();
+    display_list->push<DrawRectOp>(SkRect::MakeWH(10, 10), flags);
+    display_list->EndPaintOfUnpaired(PaintableRegion());
+  }
+
+  if (has_draw_text_op_) {
+    display_list->StartPaint();
+    display_list->push<DrawTextBlobOp>(
+        SkTextBlob::MakeFromString("any", SkFont()), 0, 0, PaintFlags());
+    display_list->EndPaintOfUnpaired(PaintableRegion());
+  }
+
   display_list->Finalize();
   return display_list;
 }
 
 bool FakeContentLayerClient::FillsBoundsCompletely() const { return false; }
 
-size_t FakeContentLayerClient::GetApproximateUnsharedMemoryUsage() const {
-  return reported_memory_usage_;
-}
-
 }  // namespace cc
