@@ -9,12 +9,14 @@
 #include <vector>
 
 #include "base/callback.h"
-#include "base/logging.h"
+#include "base/check.h"
 #include "base/memory/ptr_util.h"
+#include "base/memory/read_only_shared_memory_region.h"
 #include "base/memory/weak_ptr.h"
 #include "base/time/time.h"
 #include "cc/trees/layer_tree_frame_sink.h"
 #include "components/viz/common/frame_sinks/begin_frame_args.h"
+#include "components/viz/common/frame_timing_details.h"
 #include "components/viz/common/quads/compositor_frame.h"
 #include "components/viz/service/display/software_output_device.h"
 #include "components/viz/test/test_context_provider.h"
@@ -86,9 +88,24 @@ class FakeLayerTreeFrameSink : public LayerTreeFrameSink {
   }
 
   static std::unique_ptr<FakeLayerTreeFrameSink> Create3dForGpuRasterization(
-      int max_msaa_samples = 0) {
+      int max_msaa_samples = -1,
+      bool msaa_is_slow = false) {
     return Builder()
         .AllContexts(&viz::TestGLES2Interface::set_gpu_rasterization, true)
+        .AllContexts(&viz::TestGLES2Interface::set_msaa_is_slow, msaa_is_slow)
+        .AllContexts(&viz::TestGLES2Interface::SetMaxSamples, max_msaa_samples)
+        .Build();
+  }
+
+  static std::unique_ptr<FakeLayerTreeFrameSink> Create3dForOopRasterization(
+      int max_msaa_samples = -1,
+      bool msaa_is_slow = false) {
+    // TODO(enne): this should really use a TestRasterInterface.
+    // It's very fake to use "supports oop raster" on a gles2 interface.
+    return Builder()
+        .AllContexts(&viz::TestGLES2Interface::set_gpu_rasterization, true)
+        .AllContexts(&viz::TestGLES2Interface::set_supports_oop_raster, true)
+        .AllContexts(&viz::TestGLES2Interface::set_msaa_is_slow, msaa_is_slow)
         .AllContexts(&viz::TestGLES2Interface::SetMaxSamples, max_msaa_samples)
         .Build();
   }
@@ -100,9 +117,11 @@ class FakeLayerTreeFrameSink : public LayerTreeFrameSink {
   // LayerTreeFrameSink implementation.
   bool BindToClient(LayerTreeFrameSinkClient* client) override;
   void DetachFromClient() override;
-  void SubmitCompositorFrame(viz::CompositorFrame frame) override;
+  void SubmitCompositorFrame(viz::CompositorFrame frame,
+                             bool hit_test_data_changed,
+                             bool show_hit_test_borders) override;
   void DidNotProduceFrame(const viz::BeginFrameAck& ack) override;
-  void DidAllocateSharedBitmap(mojo::ScopedSharedBufferHandle buffer,
+  void DidAllocateSharedBitmap(base::ReadOnlySharedMemoryRegion region,
                                const viz::SharedBitmapId& id) override;
   void DidDeleteSharedBitmap(const viz::SharedBitmapId& id) override;
 
@@ -123,6 +142,12 @@ class FakeLayerTreeFrameSink : public LayerTreeFrameSink {
 
   void ReturnResourcesHeldByParent();
 
+  // A BeginFrame request usually comes with the frames that have been
+  // presented. This allows a test to inform the compositor when a frame should
+  // be considered as presented to the user.
+  void NotifyDidPresentCompositorFrame(uint32_t frame_token,
+                                       const viz::FrameTimingDetails& details);
+
  protected:
   FakeLayerTreeFrameSink(
       scoped_refptr<viz::ContextProvider> context_provider,
@@ -140,7 +165,7 @@ class FakeLayerTreeFrameSink : public LayerTreeFrameSink {
   void DidReceiveCompositorFrameAck();
 
   std::unique_ptr<viz::BeginFrameSource> begin_frame_source_;
-  base::WeakPtrFactory<FakeLayerTreeFrameSink> weak_ptr_factory_;
+  base::WeakPtrFactory<FakeLayerTreeFrameSink> weak_ptr_factory_{this};
 };
 
 }  // namespace cc
