@@ -145,6 +145,14 @@ class PaintOpHelper {
             << ", flags=" << PaintOpHelper::FlagsToString(op->flags) << ")";
         break;
       }
+      case PaintOpType::DrawSkottie: {
+        const auto* op = static_cast<const DrawSkottieOp*>(base_op);
+        str << "DrawSkottieOp("
+            << "skottie=" << PaintOpHelper::SkottieToString(op->skottie)
+            << ", dst=" << PaintOpHelper::SkiaTypeToString(op->dst)
+            << ", t=" << op->t << ")";
+        break;
+      }
       case PaintOpType::DrawTextBlob: {
         const auto* op = static_cast<const DrawTextBlobOp*>(base_op);
         str << "DrawTextBlobOp(blob="
@@ -184,7 +192,6 @@ class PaintOpHelper {
         str << "SaveLayerAlphaOp(bounds="
             << PaintOpHelper::SkiaTypeToString(op->bounds)
             << ", alpha=" << static_cast<uint32_t>(op->alpha)
-            << ", preserve_lcd_text_requests=" << op->preserve_lcd_text_requests
             << ")";
         break;
       }
@@ -206,6 +213,11 @@ class PaintOpHelper {
             << ", dy=" << PaintOpHelper::SkiaTypeToString(op->dy) << ")";
         break;
       }
+      case PaintOpType::SetNodeId: {
+        const auto* op = static_cast<const SetNodeIdOp*>(base_op);
+        str << "SetNodeIdOp(id=" << op->node_id << ")";
+        break;
+      }
     }
     return str.str();
   }
@@ -239,15 +251,14 @@ class PaintOpHelper {
                               rect.rect().width(), rect.rect().height());
   }
 
-  static std::string SkiaTypeToString(const ThreadsafeMatrix& matrix) {
-    return SkiaTypeToString(static_cast<const SkMatrix&>(matrix));
-  }
-
-  static std::string SkiaTypeToString(const SkMatrix& matrix) {
+  static std::string SkiaTypeToString(const SkM44& matrix) {
     return base::StringPrintf(
-        "[%8.4f %8.4f %8.4f][%8.4f %8.4f %8.4f][%8.4f %8.4f %8.4f]", matrix[0],
-        matrix[1], matrix[2], matrix[3], matrix[4], matrix[5], matrix[6],
-        matrix[7], matrix[8]);
+        "[%8.4f %8.4f %8.4f %8.4f][%8.4f %8.4f %8.4f %8.4f][%8.4f %8.4f %8.4f "
+        "%8.4f][%8.4f %8.4f %8.4f %8.4f]]",
+        matrix.rc(0, 0), matrix.rc(0, 1), matrix.rc(0, 2), matrix.rc(0, 3),
+        matrix.rc(1, 0), matrix.rc(1, 1), matrix.rc(1, 2), matrix.rc(1, 3),
+        matrix.rc(2, 0), matrix.rc(2, 1), matrix.rc(2, 2), matrix.rc(2, 3),
+        matrix.rc(3, 0), matrix.rc(3, 1), matrix.rc(3, 2), matrix.rc(3, 3));
   }
 
   static std::string SkiaTypeToString(const SkColor& color) {
@@ -347,34 +358,6 @@ class PaintOpHelper {
     return "<SkPath>";
   }
 
-  static std::string SkiaTypeToString(PaintFlags::Hinting hinting) {
-    switch (hinting) {
-      case PaintFlags::kNo_Hinting:
-        return "kNo_Hinting";
-      case PaintFlags::kSlight_Hinting:
-        return "kSlight_Hinting";
-      case PaintFlags::kNormal_Hinting:
-        return "kNormal_Hinting";
-      case PaintFlags::kFull_Hinting:
-        return "kFull_Hinting";
-    }
-    return "<unknown PaintFlags::Hinting>";
-  }
-
-  static std::string SkiaTypeToString(PaintFlags::TextEncoding encoding) {
-    switch (encoding) {
-      case PaintFlags::kUTF8_TextEncoding:
-        return "kUTF8_TextEncoding";
-      case PaintFlags::kUTF16_TextEncoding:
-        return "kUTF16_TextEncoding";
-      case PaintFlags::kUTF32_TextEncoding:
-        return "kUTF32_TextEncoding";
-      case PaintFlags::kGlyphID_TextEncoding:
-        return "kGlyphID_TextEncoding";
-    }
-    return "<unknown PaintFlags::TextEncoding>";
-  }
-
   static std::string SkiaTypeToString(SkFilterQuality quality) {
     switch (quality) {
       case kNone_SkFilterQuality:
@@ -413,10 +396,6 @@ class PaintOpHelper {
     return "<unknown PaintFlags::Join>";
   }
 
-  static std::string SkiaTypeToString(const sk_sp<SkTypeface>& typeface) {
-    return typeface ? "<SkTypeface>" : "(nil)";
-  }
-
   static std::string SkiaTypeToString(const sk_sp<SkColorFilter>& filter) {
     if (!filter)
       return "(nil)";
@@ -461,13 +440,13 @@ class PaintOpHelper {
   }
 
   static std::string EnumToString(
-      const PaintCanvas::SrcRectConstraint& constraint) {
+      const SkCanvas::SrcRectConstraint& constraint) {
     switch (constraint) {
       default:
         break;
-      case PaintCanvas::kStrict_SrcRectConstraint:
+      case SkCanvas::kStrict_SrcRectConstraint:
         return "kStrict_SrcRectConstraint";
-      case PaintCanvas::kFast_SrcRectConstraint:
+      case SkCanvas::kFast_SrcRectConstraint:
         return "kFast_SrcRectConstraint";
     }
     return "<unknown SrcRectConstraint>";
@@ -485,6 +464,8 @@ class PaintOpHelper {
 
   static std::string EnumToString(PaintShader::Type type) {
     switch (type) {
+      case PaintShader::Type::kEmpty:
+        return "kEmpty";
       case PaintShader::Type::kColor:
         return "kColor";
       case PaintShader::Type::kLinearGradient:
@@ -509,13 +490,24 @@ class PaintOpHelper {
     return "<paint image>";
   }
 
+  static std::string SkottieToString(scoped_refptr<SkottieWrapper> skottie) {
+    std::ostringstream str;
+    str << "<skottie [";
+    str << "duration=" << skottie->duration() << " seconds";
+    str << ", width="
+        << PaintOpHelper::SkiaTypeToString(skottie->size().width());
+    str << ", height="
+        << PaintOpHelper::SkiaTypeToString(skottie->size().height());
+    str << "]";
+    return str.str();
+  }
+
   static std::string RecordToString(const sk_sp<const PaintRecord>& record) {
     return record ? "<paint record>" : "(nil)";
   }
 
-  static std::string TextBlobToString(
-      const scoped_refptr<PaintTextBlob>& blob) {
-    return blob ? "<paint text blob>" : "(nil)";
+  static std::string TextBlobToString(const sk_sp<SkTextBlob>& blob) {
+    return blob ? "<sk text blob>" : "(nil)";
   }
 
   static std::string PaintShaderToString(const PaintShader* shader) {
@@ -526,8 +518,8 @@ class PaintOpHelper {
     str << ", flags=" << shader->flags_;
     str << ", end_radius=" << shader->end_radius_;
     str << ", start_radius=" << shader->start_radius_;
-    str << ", tx=" << shader->tx_;
-    str << ", ty=" << shader->ty_;
+    str << ", tx=" << static_cast<unsigned>(shader->tx_);
+    str << ", ty=" << static_cast<unsigned>(shader->ty_);
     str << ", fallback_color=" << shader->fallback_color_;
     str << ", scaling_behavior=" << EnumToString(shader->scaling_behavior_);
     if (shader->local_matrix_.has_value()) {
@@ -587,16 +579,7 @@ class PaintOpHelper {
     str << ", blendMode="
         << PaintOpHelper::SkiaTypeToString(flags.getBlendMode());
     str << ", isAntiAlias=" << flags.isAntiAlias();
-    str << ", isVerticalText=" << flags.isVerticalText();
-    str << ", isSubpixelText=" << flags.isSubpixelText();
-    str << ", isLCDRenderText=" << flags.isLCDRenderText();
-    str << ", hinting=" << PaintOpHelper::SkiaTypeToString(flags.getHinting());
-    str << ", isAutohinted=" << flags.isAutohinted();
     str << ", isDither=" << flags.isDither();
-    str << ", textEncoding="
-        << PaintOpHelper::SkiaTypeToString(flags.getTextEncoding());
-    str << ", textSize="
-        << PaintOpHelper::SkiaTypeToString(flags.getTextSize());
     str << ", filterQuality="
         << PaintOpHelper::SkiaTypeToString(flags.getFilterQuality());
     str << ", strokeWidth="
@@ -607,8 +590,6 @@ class PaintOpHelper {
         << PaintOpHelper::SkiaTypeToString(flags.getStrokeCap());
     str << ", strokeJoin="
         << PaintOpHelper::SkiaTypeToString(flags.getStrokeJoin());
-    str << ", typeface="
-        << PaintOpHelper::SkiaTypeToString(flags.getTypeface());
     str << ", colorFilter="
         << PaintOpHelper::SkiaTypeToString(flags.getColorFilter());
     str << ", maskFilter="
