@@ -4,23 +4,25 @@
 
 #include "cc/test/resource_provider_test_utils.h"
 
-#include "base/bind.h"
+#include <unordered_map>
+#include <vector>
+
+#include "base/callback_helpers.h"
 
 namespace cc {
 
-const std::unordered_map<viz::ResourceId, viz::ResourceId>&
-SendResourceAndGetChildToParentMap(
-    const std::vector<viz::ResourceId>& resource_ids,
-    viz::DisplayResourceProvider* resource_provider,
-    viz::ClientResourceProvider* child_resource_provider,
-    viz::ContextProvider* child_context_provider) {
+const std::
+    unordered_map<viz::ResourceId, viz::ResourceId, viz::ResourceIdHasher>&
+    SendResourceAndGetChildToParentMap(
+        const std::vector<viz::ResourceId>& resource_ids,
+        viz::DisplayResourceProvider* resource_provider,
+        viz::ClientResourceProvider* child_resource_provider,
+        viz::ContextProvider* child_context_provider) {
   DCHECK(resource_provider);
   DCHECK(child_resource_provider);
   // Transfer resources to the parent.
   std::vector<viz::TransferableResource> send_to_parent;
-  int child_id = resource_provider->CreateChild(
-      base::BindRepeating([](const std::vector<viz::ReturnedResource>&) {}),
-      true);
+  int child_id = resource_provider->CreateChild(base::DoNothing());
   child_resource_provider->PrepareSendToParent(resource_ids, &send_to_parent,
                                                child_context_provider);
   resource_provider->ReceiveFromChild(child_id, send_to_parent);
