@@ -4,10 +4,16 @@
 
 #include "cc/tiles/software_image_decode_cache_utils.h"
 
+#include <algorithm>
+#include <sstream>
+#include <utility>
+
 #include "base/atomic_sequence_num.h"
-#include "base/hash.h"
+#include "base/callback_helpers.h"
+#include "base/hash/hash.h"
 #include "base/memory/discardable_memory_allocator.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/process/memory.h"
 #include "base/trace_event/trace_event.h"
 #include "cc/tiles/mipmap_util.h"
 #include "ui/gfx/skia_util.h"
@@ -44,11 +50,15 @@ SkImageInfo CreateImageInfo(const SkISize& size, SkColorType color_type) {
                            kPremul_SkAlphaType);
 }
 
+// Does *not* return nullptr.
 std::unique_ptr<base::DiscardableMemory> AllocateDiscardable(
-    const SkImageInfo& info) {
+    const SkImageInfo& info,
+    base::OnceClosure on_no_memory) {
   TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("cc.debug"), "AllocateDiscardable");
-  return base::DiscardableMemoryAllocator::GetInstance()
-      ->AllocateLockedDiscardableMemory(info.minRowBytes() * info.height());
+  size_t size = info.minRowBytes() * info.height();
+  auto* allocator = base::DiscardableMemoryAllocator::GetInstance();
+  return allocator->AllocateLockedDiscardableMemoryWithRetryOrDie(
+      size, std::move(on_no_memory));
 }
 
 }  // namespace
@@ -59,15 +69,16 @@ SoftwareImageDecodeCacheUtils::DoDecodeImage(
     const CacheKey& key,
     const PaintImage& paint_image,
     SkColorType color_type,
-    PaintImage::GeneratorClientId client_id) {
+    PaintImage::GeneratorClientId client_id,
+    base::OnceClosure on_no_memory) {
   SkISize target_size =
       SkISize::Make(key.target_size().width(), key.target_size().height());
   DCHECK(target_size == paint_image.GetSupportedDecodeSize(target_size));
 
   SkImageInfo target_info = CreateImageInfo(target_size, color_type);
   std::unique_ptr<base::DiscardableMemory> target_pixels =
-      AllocateDiscardable(target_info);
-  if (!target_pixels || !target_pixels->data())
+      AllocateDiscardable(target_info, std::move(on_no_memory));
+  if (!target_pixels->data())
     return nullptr;
 
   TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
@@ -94,9 +105,10 @@ SoftwareImageDecodeCacheUtils::GenerateCacheEntryFromCandidate(
   SkISize target_size =
       SkISize::Make(key.target_size().width(), key.target_size().height());
   SkImageInfo target_info = CreateImageInfo(target_size, color_type);
+  // TODO(crbug.com/983348): If this turns into a crasher, pass an actual
+  // "free memory" closure.
   std::unique_ptr<base::DiscardableMemory> target_pixels =
-      AllocateDiscardable(target_info);
-  DCHECK(target_pixels);
+      AllocateDiscardable(target_info, base::DoNothing());
 
   if (key.type() == CacheKey::kSubrectOriginal) {
     DCHECK(needs_extract_subset);
@@ -142,7 +154,10 @@ SoftwareImageDecodeCacheUtils::GenerateCacheEntryFromCandidate(
     result = ImageDecodeCacheUtils::ScaleToHalfFloatPixmapUsingN32Intermediate(
         decoded_pixmap, &target_pixmap, filter_quality);
   } else {
-    result = decoded_pixmap.scalePixels(target_pixmap, filter_quality);
+    result = decoded_pixmap.scalePixels(
+        target_pixmap,
+        SkSamplingOptions(filter_quality,
+                          SkSamplingOptions::kMedium_asMipmapLinear));
   }
   DCHECK(result) << key.ToString();
 
@@ -157,7 +172,7 @@ SoftwareImageDecodeCacheUtils::GenerateCacheEntryFromCandidate(
 SoftwareImageDecodeCacheUtils::CacheKey
 SoftwareImageDecodeCacheUtils::CacheKey::FromDrawImage(const DrawImage& image,
                                                        SkColorType color_type) {
-  DCHECK(!image.paint_image().GetSkImage()->isTextureBacked());
+  DCHECK(!image.paint_image().IsTextureBacked());
 
   const PaintImage::FrameKey frame_key = image.frame_key();
   const PaintImage::Id stable_id = image.paint_image().stable_id();
@@ -202,14 +217,7 @@ SoftwareImageDecodeCacheUtils::CacheKey::FromDrawImage(const DrawImage& image,
   } else {
     type = kSubrectAndScale;
     // Update the target size to be a mip level size.
-    // TODO(vmpstr): MipMapUtil and JPEG decoders disagree on what to do with
-    // odd sizes. If width = 2k + 1, and the mip level is 1, then this will
-    // return width = k; JPEG decoder, however, will support decoding to width =
-    // k + 1. We need to figure out what to do in this case.
-    SkSize mip_scale_adjustment =
-        MipMapUtil::GetScaleAdjustmentForLevel(src_rect.size(), mip_level);
-    target_size.set_width(src_rect.width() * mip_scale_adjustment.width());
-    target_size.set_height(src_rect.height() * mip_scale_adjustment.height());
+    target_size = MipMapUtil::GetSizeForLevel(src_rect.size(), mip_level);
   }
 
   // If the original image is large, we might want to do a subrect instead if
