@@ -7,6 +7,7 @@
 #include <stddef.h>
 
 #include <algorithm>
+#include <utility>
 
 #include "base/memory/ptr_util.h"
 #include "cc/base/math_util.h"
@@ -15,7 +16,6 @@
 #include "cc/layers/render_surface_impl.h"
 #include "cc/paint/filter_operations.h"
 #include "cc/trees/effect_node.h"
-#include "cc/trees/layer_tree_host_common.h"
 #include "cc/trees/layer_tree_impl.h"
 #include "ui/gfx/geometry/rect_conversions.h"
 
@@ -25,13 +25,10 @@ std::unique_ptr<DamageTracker> DamageTracker::Create() {
   return base::WrapUnique(new DamageTracker());
 }
 
-DamageTracker::DamageTracker() : mailboxId_(0) {}
-
+DamageTracker::DamageTracker() = default;
 DamageTracker::~DamageTracker() = default;
 
-void DamageTracker::UpdateDamageTracking(
-    LayerTreeImpl* layer_tree_impl,
-    const RenderSurfaceList& render_surface_list) {
+void DamageTracker::UpdateDamageTracking(LayerTreeImpl* layer_tree_impl) {
   //
   // This function computes the "damage rect" of each target surface, and
   // updates the state that is used to correctly track damage across frames. The
@@ -64,7 +61,7 @@ void DamageTracker::UpdateDamageTracking(
   //
   // - This algorithm requires that descendant surfaces compute their damage
   //   before ancestor surfaces. Further, since contributing surfaces with
-  //   background filters can expand the damage caused by contributors
+  //   backdrop filters can expand the damage caused by contributors
   //   underneath them (that is, before them in draw order), the exact damage
   //   caused by these contributors must be computed before computing the damage
   //   caused by the contributing surface. This is implemented by visiting
@@ -105,7 +102,7 @@ void DamageTracker::UpdateDamageTracking(
   //         erased from map.
   //
 
-  for (RenderSurfaceImpl* render_surface : render_surface_list) {
+  for (auto* render_surface : layer_tree_impl->GetRenderSurfaceList()) {
     render_surface->damage_tracker()->PrepareForUpdate();
   }
 
@@ -171,13 +168,40 @@ void DamageTracker::ComputeSurfaceDamage(RenderSurfaceImpl* render_surface) {
   // These functions cannot be bypassed with early-exits, even if we know what
   // the damage will be for this frame, because we need to update the damage
   // tracker state to correctly track the next frame.
-  DamageAccumulator damage_from_surface_mask =
-      TrackDamageFromSurfaceMask(render_surface->MaskLayer());
   DamageAccumulator damage_from_leftover_rects = TrackDamageFromLeftoverRects();
   // True if any layer is removed.
   has_damage_from_contributing_content_ |=
       !damage_from_leftover_rects.IsEmpty();
 
+  gfx::Rect expanded_damage_rect;
+  bool valid = damage_from_leftover_rects.GetAsRect(&expanded_damage_rect);
+  bool expanded = false;
+  // Iterate through the surfaces rendering to the current target back to
+  // front, intersect their surface rects with the damage from leftover rects.
+  // Update surfaces' |intersects_damage_under| flags accordingly and expand the
+  // damage by surface rects for surfaces with pixel-moving backdrop filters
+  // when appropriate.
+  for (auto& contributing_surface : contributing_surfaces_) {
+    RenderSurfaceImpl* surface = contributing_surface.render_surface;
+    bool has_pixel_moving_backdrop_filters =
+        surface->BackdropFilters().HasFilterThatMovesPixels();
+    if (!surface->intersects_damage_under() ||
+        has_pixel_moving_backdrop_filters) {
+      if (!valid || contributing_surface.rect_in_target_space.Intersects(
+                        expanded_damage_rect)) {
+        surface->set_intersects_damage_under(true);
+        if (has_pixel_moving_backdrop_filters) {
+          expanded_damage_rect.Union(contributing_surface.rect_in_target_space);
+          expanded = true;
+        }
+      }
+    }
+  }
+  if (expanded)
+    damage_for_this_update_.Union(expanded_damage_rect);
+
+  contributing_surfaces_.clear();
+
   if (render_surface->SurfacePropertyChangedOnlyFromDescendant()) {
     damage_for_this_update_ = DamageAccumulator();
     damage_for_this_update_.Union(render_surface->content_rect());
@@ -186,14 +210,13 @@ void DamageTracker::ComputeSurfaceDamage(RenderSurfaceImpl* render_surface) {
   } else {
     // TODO(shawnsingh): can we clamp this damage to the surface's content rect?
     // (affects performance, but not correctness)
-    damage_for_this_update_.Union(damage_from_surface_mask);
     damage_for_this_update_.Union(damage_from_leftover_rects);
 
     gfx::Rect damage_rect;
     bool is_rect_valid = damage_for_this_update_.GetAsRect(&damage_rect);
     if (is_rect_valid && !damage_rect.IsEmpty()) {
       damage_rect = render_surface->Filters().MapRect(
-          damage_rect, render_surface->SurfaceScale().matrix());
+          damage_rect, SkMatrix(render_surface->SurfaceScale().matrix()));
       damage_for_this_update_ = DamageAccumulator();
       damage_for_this_update_.Union(damage_rect);
     }
@@ -240,28 +263,11 @@ DamageTracker::SurfaceRectMapData& DamageTracker::RectDataForSurface(
   return *it;
 }
 
-DamageTracker::DamageAccumulator DamageTracker::TrackDamageFromSurfaceMask(
-    LayerImpl* target_surface_mask_layer) {
-  DamageAccumulator damage;
-
-  if (!target_surface_mask_layer)
-    return damage;
-
-  // Currently, if there is any change to the mask, we choose to damage the
-  // entire surface. This could potentially be optimized later, but it is not
-  // expected to be a common case.
-  if (target_surface_mask_layer->LayerPropertyChanged() ||
-      !target_surface_mask_layer->update_rect().IsEmpty()) {
-    damage.Union(gfx::Rect(target_surface_mask_layer->bounds()));
-  }
-
-  return damage;
-}
-
 void DamageTracker::PrepareForUpdate() {
   mailboxId_++;
   damage_for_this_update_ = DamageAccumulator();
   has_damage_from_contributing_content_ = false;
+  contributing_surfaces_.clear();
 }
 
 DamageTracker::DamageAccumulator DamageTracker::TrackDamageFromLeftoverRects() {
@@ -328,26 +334,6 @@ DamageTracker::DamageAccumulator DamageTracker::TrackDamageFromLeftoverRects() {
   return damage;
 }
 
-void DamageTracker::ExpandDamageInsideRectWithFilters(
-    const gfx::Rect& pre_filter_rect,
-    const FilterOperations& filters) {
-  gfx::Rect damage_rect;
-  bool is_valid_rect = damage_for_this_update_.GetAsRect(&damage_rect);
-  // If the damage accumulated so far isn't a valid rect or empty, then there is
-  // no point in trying to make it bigger.
-  if (!is_valid_rect || damage_rect.IsEmpty())
-    return;
-
-  // Compute the pixels in the background of the surface that could be affected
-  // by the damage in the content below.
-  gfx::Rect expanded_damage_rect = filters.MapRect(damage_rect, SkMatrix::I());
-
-  // Restrict it to the rectangle in which the background filter is shown.
-  expanded_damage_rect.Intersect(pre_filter_rect);
-
-  damage_for_this_update_.Union(expanded_damage_rect);
-}
-
 void DamageTracker::AccumulateDamageFromLayer(LayerImpl* layer) {
   // There are two ways that a layer can damage a region of the target surface:
   //   1. Property change (e.g. opacity, position, transforms):
@@ -365,18 +351,20 @@ void DamageTracker::AccumulateDamageFromLayer(LayerImpl* layer) {
   // This method is called when we want to consider how a layer contributes to
   // its target RenderSurface, even if that layer owns the target RenderSurface
   // itself. To consider how a layer's target surface contributes to the
-  // ancestor surface, ExtendDamageForRenderSurface() must be called instead.
+  // ancestor surface, AccumulateDamageFromRenderSurface() must be called
+  // instead.
   bool layer_is_new = false;
   LayerRectMapData& data = RectDataForLayer(layer->id(), &layer_is_new);
   gfx::Rect old_rect_in_target_space = data.rect_;
 
-  gfx::Rect rect_in_target_space = layer->GetEnclosingRectInTargetSpace();
-  data.Update(rect_in_target_space, mailboxId_);
+  gfx::Rect visible_rect_in_target_space =
+      layer->visible_drawable_content_rect();
+  data.Update(visible_rect_in_target_space, mailboxId_);
 
   if (layer_is_new || layer->LayerPropertyChanged()) {
     // If a layer is new or has changed, then its entire layer rect affects the
     // target surface.
-    damage_for_this_update_.Union(rect_in_target_space);
+    damage_for_this_update_.Union(visible_rect_in_target_space);
 
     // The layer's old region is now exposed on the target surface, too.
     // Note old_rect_in_target_space is already in target space.
@@ -385,7 +373,7 @@ void DamageTracker::AccumulateDamageFromLayer(LayerImpl* layer) {
     // If the layer properties haven't changed, then the the target surface is
     // only affected by the layer's damaged area, which could be empty.
     gfx::Rect damage_rect =
-        gfx::UnionRects(layer->update_rect(), layer->damage_rect());
+        gfx::UnionRects(layer->update_rect(), layer->GetDamageRect());
     damage_rect.Intersect(gfx::Rect(layer->bounds()));
 
     if (!damage_rect.IsEmpty()) {
@@ -395,10 +383,27 @@ void DamageTracker::AccumulateDamageFromLayer(LayerImpl* layer) {
     }
   }
 
-  // Property changes from animaiton will not be considered as damage from
-  // contributing content.
-  if (layer_is_new || layer->LayerPropertyChangedNotFromPropertyTrees() ||
-      !layer->update_rect().IsEmpty() || !layer->damage_rect().IsEmpty()) {
+  // Property changes on effect or transform nodes that are shared by the
+  // render target are not considered damage to that target itself.  This
+  // is the case where the render target itself changes opacity or moves.
+  // The damage goes to the target's target instead.  This is not perfect,
+  // as the target and layer could share an effect but not a transform,
+  // but there's no tracking on the layer to differentiate that the
+  // LayerPropertyChangedFromPropertyTrees is for the effect not the transform.
+  bool property_change_on_non_target_node = false;
+  if (layer->LayerPropertyChangedFromPropertyTrees()) {
+    auto effect_id = layer->render_target()->EffectTreeIndex();
+    auto* effect_node =
+        layer->layer_tree_impl()->property_trees()->effect_tree.Node(effect_id);
+    auto transform_id = effect_node->transform_id;
+    property_change_on_non_target_node =
+        layer->effect_tree_index() != effect_id ||
+        layer->transform_tree_index() != transform_id;
+  }
+
+  if (layer_is_new || !layer->update_rect().IsEmpty() ||
+      layer->LayerPropertyChangedNotFromPropertyTrees() ||
+      !layer->GetDamageRect().IsEmpty() || property_change_on_non_target_node) {
     has_damage_from_contributing_content_ |= !damage_for_this_update_.IsEmpty();
   }
 }
@@ -428,6 +433,20 @@ void DamageTracker::AccumulateDamageFromRenderSurface(
   gfx::Rect surface_rect_in_target_space =
       gfx::ToEnclosingRect(render_surface->DrawableContentRect());
   data.Update(surface_rect_in_target_space, mailboxId_);
+  contributing_surfaces_.emplace_back(render_surface,
+                                      surface_rect_in_target_space);
+
+  // If the render surface has pixel-moving backdrop filters and the surface
+  // rect intersects current accumulated damage, expand the damage by surface
+  // rect.
+  gfx::Rect damage_on_target;
+  bool valid = damage_for_this_update_.GetAsRect(&damage_on_target);
+  bool intersects_damage_under =
+      !valid || damage_on_target.Intersects(surface_rect_in_target_space);
+  if (render_surface->BackdropFilters().HasFilterThatMovesPixels() &&
+      intersects_damage_under) {
+    damage_for_this_update_.Union(surface_rect_in_target_space);
+  }
 
   if (surface_is_new || render_surface->SurfacePropertyChanged()) {
     // The entire surface contributes damage.
@@ -435,6 +454,7 @@ void DamageTracker::AccumulateDamageFromRenderSurface(
 
     // The surface's old region is now exposed on the target surface, too.
     damage_for_this_update_.Union(old_surface_rect);
+    intersects_damage_under = true;
   } else {
     // Only the surface's damage_rect will damage the target surface.
     gfx::Rect damage_rect_in_local_space;
@@ -446,25 +466,15 @@ void DamageTracker::AccumulateDamageFromRenderSurface(
       const gfx::Transform& draw_transform = render_surface->draw_transform();
       gfx::Rect damage_rect_in_target_space = MathUtil::MapEnclosingClippedRect(
           draw_transform, damage_rect_in_local_space);
+      damage_rect_in_target_space.Intersect(
+          gfx::ToEnclosingRect(render_surface->DrawableContentRect()));
       damage_for_this_update_.Union(damage_rect_in_target_space);
     } else if (!is_valid_rect) {
       damage_for_this_update_.Union(surface_rect_in_target_space);
     }
   }
 
-  // If the layer has a background filter, this may cause pixels in our surface
-  // to be expanded, so we will need to expand any damage at or below this
-  // layer. We expand the damage from this layer too, as we need to readback
-  // those pixels from the surface with only the contents of layers below this
-  // one in them. This means we need to redraw any pixels in the surface being
-  // used for the blur in this layer this frame.
-  const FilterOperations& background_filters =
-      render_surface->BackgroundFilters();
-  if (background_filters.HasFilterThatMovesPixels()) {
-    ExpandDamageInsideRectWithFilters(surface_rect_in_target_space,
-                                      background_filters);
-  }
-
+  render_surface->set_intersects_damage_under(intersects_damage_under);
   // True if any changes from contributing render surface.
   has_damage_from_contributing_content_ |= !damage_for_this_update_.IsEmpty();
 }
