@@ -8,9 +8,11 @@
 
 #include <set>
 
+#include "base/check_op.h"
+#include "base/containers/contains.h"
 #include "base/containers/flat_set.h"
-#include "base/logging.h"
-#include "base/stl_util.h"
+#include "base/debug/crash_logging.h"
+#include "base/debug/dump_without_crashing.h"
 #include "base/trace_event/trace_event.h"
 #include "cc/layers/layer.h"
 #include "cc/layers/layer_collections.h"
@@ -39,14 +41,14 @@ static bool LayerHasValidPropertyTreeIndices(LayerImpl* layer) {
 }
 
 static bool LayerWillPushProperties(LayerTreeHost* host, Layer* layer) {
-  return base::ContainsKey(host->LayersThatShouldPushProperties(), layer);
+  return base::Contains(host->LayersThatShouldPushProperties(), layer);
 }
 
 static bool LayerWillPushProperties(LayerTreeImpl* tree, LayerImpl* layer) {
-  return base::ContainsKey(tree->LayersThatShouldPushProperties(), layer) ||
+  return base::Contains(tree->LayersThatShouldPushProperties(), layer) ||
          // TODO(crbug.com/303943): Stop always pushing PictureLayerImpl
          // properties.
-         base::ContainsValue(tree->picture_layers(), layer);
+         base::Contains(tree->picture_layers(), layer);
 }
 #endif
 
@@ -66,7 +68,7 @@ template <typename LayerTreeType>
 void PushLayerList(OwnedLayerImplMap* old_layers,
                    LayerTreeType* host,
                    LayerTreeImpl* tree_impl) {
-  tree_impl->ClearLayerList();
+  DCHECK(tree_impl->LayerListIsEmpty());
   for (auto* layer : *host) {
     std::unique_ptr<LayerImpl> layer_impl(
         ReuseOrCreateLayerImpl(old_layers, layer, tree_impl));
@@ -80,7 +82,6 @@ void PushLayerList(OwnedLayerImplMap* old_layers,
            LayerWillPushProperties(host, layer));
 #endif
 
-    tree_impl->AddToLayerList(layer_impl.get());
     tree_impl->AddLayer(std::move(layer_impl));
   }
   tree_impl->OnCanDrawStateChangedForTree();
@@ -88,26 +89,19 @@ void PushLayerList(OwnedLayerImplMap* old_layers,
 
 template <typename LayerTreeType>
 void SynchronizeTreesInternal(LayerTreeType* source_tree,
-                              LayerTreeImpl* tree_impl,
-                              PropertyTrees* property_trees) {
+                              LayerTreeImpl* tree_impl) {
   DCHECK(tree_impl);
 
   TRACE_EVENT0("cc", "TreeSynchronizer::SynchronizeTrees");
-  std::unique_ptr<OwnedLayerImplList> old_layers(tree_impl->DetachLayers());
+  OwnedLayerImplList old_layers = tree_impl->DetachLayers();
 
   OwnedLayerImplMap old_layer_map;
-  for (auto& it : *old_layers) {
+  for (auto& it : old_layers) {
     DCHECK(it);
     old_layer_map[it->id()] = std::move(it);
   }
 
   PushLayerList(&old_layer_map, source_tree, tree_impl);
-
-  for (int id : property_trees->effect_tree.mask_layer_ids()) {
-    std::unique_ptr<LayerImpl> layer_impl(ReuseOrCreateLayerImpl(
-        &old_layer_map, source_tree->LayerById(id), tree_impl));
-    tree_impl->AddLayer(std::move(layer_impl));
-  }
 }
 
 }  // namespace
@@ -117,8 +111,7 @@ void TreeSynchronizer::SynchronizeTrees(Layer* layer_root,
   if (!layer_root) {
     tree_impl->DetachLayers();
   } else {
-    SynchronizeTreesInternal(layer_root->layer_tree_host(), tree_impl,
-                             layer_root->layer_tree_host()->property_trees());
+    SynchronizeTreesInternal(layer_root->layer_tree_host(), tree_impl);
   }
 }
 
@@ -127,8 +120,53 @@ void TreeSynchronizer::SynchronizeTrees(LayerTreeImpl* pending_tree,
   if (pending_tree->LayerListIsEmpty()) {
     active_tree->DetachLayers();
   } else {
-    SynchronizeTreesInternal(pending_tree, active_tree,
-                             pending_tree->property_trees());
+    SynchronizeTreesInternal(pending_tree, active_tree);
+  }
+}
+
+template <typename Iterator>
+static void PushLayerPropertiesInternal(Iterator source_layers_begin,
+                                        Iterator source_layers_end,
+                                        LayerTreeHost* host_tree,
+                                        LayerTreeImpl* target_impl_tree) {
+  for (Iterator it = source_layers_begin; it != source_layers_end; ++it) {
+    auto* source_layer = *it;
+    LayerImpl* target_layer = target_impl_tree->LayerById(source_layer->id());
+    DCHECK(target_layer);
+    // TODO(enne): http://crbug.com/918126 debugging
+    CHECK(source_layer);
+    if (!target_layer) {
+      bool host_set_on_source = source_layer->layer_tree_host() == host_tree;
+
+      bool source_found_by_iterator = false;
+      for (auto host_tree_it = host_tree->begin();
+           host_tree_it != host_tree->end(); ++it) {
+        if (*host_tree_it == source_layer) {
+          source_found_by_iterator = true;
+          break;
+        }
+      }
+
+      bool root_layer_valid = !!host_tree->root_layer();
+      bool found_root = false;
+      Layer* layer = source_layer;
+      while (layer) {
+        if (layer == host_tree->root_layer()) {
+          found_root = true;
+          break;
+        }
+        layer = layer->parent();
+      }
+
+      auto str = base::StringPrintf(
+          "hs: %d, sf: %d, rlv: %d, fr: %d", host_set_on_source,
+          source_found_by_iterator, root_layer_valid, found_root);
+      static auto* crash_key = base::debug::AllocateCrashKeyString(
+          "cc_null_layer_sync", base::debug::CrashKeySize::Size32);
+      base::debug::SetCrashKeyString(crash_key, str);
+      base::debug::DumpWithoutCrashing();
+    }
+    source_layer->PushPropertiesTo(target_layer);
   }
 }
 
@@ -162,7 +200,8 @@ void TreeSynchronizer::PushLayerProperties(LayerTreeHost* host_tree,
   auto layers = host_tree->LayersThatShouldPushProperties();
   TRACE_EVENT1("cc", "TreeSynchronizer::PushLayerPropertiesTo.Main",
                "layer_count", layers.size());
-  PushLayerPropertiesInternal(layers.begin(), layers.end(), impl_tree);
+  PushLayerPropertiesInternal(layers.begin(), layers.end(), host_tree,
+                              impl_tree);
   host_tree->ClearLayersThatShouldPushProperties();
 }
 
