@@ -4,8 +4,9 @@
 
 #include "chromeos/attestation/mock_attestation_flow.h"
 
-#include "base/memory/scoped_ptr.h"
-#include "testing/gmock/include/gmock/gmock.h"
+#include <memory>
+
+#include "components/account_id/account_id.h"
 
 using testing::_;
 using testing::DefaultValue;
@@ -16,23 +17,30 @@ namespace attestation {
 
 FakeServerProxy::FakeServerProxy() : result_(true) {}
 
-FakeServerProxy::~FakeServerProxy() {}
+FakeServerProxy::~FakeServerProxy() = default;
 
 void FakeServerProxy::SendEnrollRequest(const std::string& request,
-                                        const DataCallback& callback) {
-  callback.Run(result_, request + "_response");
+                                        DataCallback callback) {
+  std::move(callback).Run(result_, enroll_response_.empty()
+                                       ? request + "_response"
+                                       : enroll_response_);
 }
 
 void FakeServerProxy::SendCertificateRequest(const std::string& request,
-                                             const DataCallback& callback) {
-  callback.Run(result_, request + "_response");
+                                             DataCallback callback) {
+  std::move(callback).Run(
+      result_, cert_response_.empty() ? request + "_response" : cert_response_);
+}
+
+void FakeServerProxy::CheckIfAnyProxyPresent(ProxyPresenceCallback callback) {
+  std::move(callback).Run(true);
 }
 
 MockServerProxy::MockServerProxy() {
   DefaultValue<PrivacyCAType>::Set(DEFAULT_PCA);
 }
 
-MockServerProxy::~MockServerProxy() {}
+MockServerProxy::~MockServerProxy() = default;
 
 void MockServerProxy::DeferToFake(bool success) {
   fake_.set_result(success);
@@ -42,14 +50,14 @@ void MockServerProxy::DeferToFake(bool success) {
       .WillByDefault(Invoke(&fake_, &FakeServerProxy::SendCertificateRequest));
 }
 
-MockObserver::MockObserver() {}
+MockObserver::MockObserver() = default;
 
-MockObserver::~MockObserver() {}
+MockObserver::~MockObserver() = default;
 
 MockAttestationFlow::MockAttestationFlow()
-    : AttestationFlow(NULL, NULL, scoped_ptr<ServerProxy>()) {}
+    : AttestationFlow(std::unique_ptr<ServerProxy>()) {}
 
-MockAttestationFlow::~MockAttestationFlow() {}
+MockAttestationFlow::~MockAttestationFlow() = default;
 
 }  // namespace attestation
 }  // namespace chromeos
