@@ -10,7 +10,7 @@
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/values.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 #include "chromeos/components/proximity_auth/proximity_auth_pref_names.h"
 #include "chromeos/services/multidevice_setup/public/cpp/prefs.h"
 #include "components/prefs/pref_registry_simple.h"
@@ -30,6 +30,13 @@ void ProximityAuthLocalStatePrefManager::RegisterPrefs(
     PrefRegistrySimple* registry) {
   // Prefs for all users are stored in a dictionary under this pref name.
   registry->RegisterDictionaryPref(prefs::kEasyUnlockLocalStateUserPrefs);
+
+  // Most Smart Lock prefs are stored in regular user prefs, and then copied out
+  // to local state for reference. This particular pref, in contrast, needs its
+  // source of truth to be in the local state, because it needs to be written
+  // to from the login screen.
+  registry->RegisterDictionaryPref(
+      prefs::kProximityAuthHasShownLoginDisabledMessage);
 }
 
 void ProximityAuthLocalStatePrefManager::SetIsEasyUnlockEnabled(
@@ -66,11 +73,6 @@ int ProximityAuthLocalStatePrefManager::GetPromotionShownCount() const {
   return 0;
 }
 
-void ProximityAuthLocalStatePrefManager::SetProximityThreshold(
-    ProximityThreshold value) {
-  NOTREACHED();
-}
-
 bool ProximityAuthLocalStatePrefManager::IsEasyUnlockAllowed() const {
   bool pref_value;
   const base::DictionaryValue* user_prefs = GetActiveUserPrefsDictionary();
@@ -100,18 +102,6 @@ bool ProximityAuthLocalStatePrefManager::IsEasyUnlockEnabledStateSet() const {
   return false;
 }
 
-ProximityAuthLocalStatePrefManager::ProximityThreshold
-ProximityAuthLocalStatePrefManager::GetProximityThreshold() const {
-  int pref_value;
-  const base::DictionaryValue* user_prefs = GetActiveUserPrefsDictionary();
-  if (!user_prefs || !user_prefs->GetIntegerWithoutPathExpansion(
-                         prefs::kEasyUnlockProximityThreshold, &pref_value)) {
-    PA_LOG(ERROR) << "Failed to get proximity_threshold.";
-    return ProximityThreshold::kClose;
-  }
-  return static_cast<ProximityThreshold>(pref_value);
-}
-
 bool ProximityAuthLocalStatePrefManager::IsChromeOSLoginAllowed() const {
   bool pref_value;
   const base::DictionaryValue* user_prefs = GetActiveUserPrefsDictionary();
@@ -119,7 +109,7 @@ bool ProximityAuthLocalStatePrefManager::IsChromeOSLoginAllowed() const {
       !user_prefs->GetBooleanWithoutPathExpansion(
           chromeos::multidevice_setup::kSmartLockSigninAllowedPrefName,
           &pref_value)) {
-    PA_LOG(INFO) << "Failed to get is_chrome_login_allowed, not disallowing";
+    PA_LOG(VERBOSE) << "Failed to get is_chrome_login_allowed, not disallowing";
     return true;
   }
   return pref_value;
@@ -142,6 +132,43 @@ bool ProximityAuthLocalStatePrefManager::IsChromeOSLoginEnabled() const {
   return pref_value;
 }
 
+void ProximityAuthLocalStatePrefManager::SetHasShownLoginDisabledMessage(
+    bool has_shown) {
+  DictionaryPrefUpdate update(local_state_,
+                              prefs::kEasyUnlockLocalStateUserPrefs);
+
+  base::DictionaryValue* current_user_prefs;
+  update.Get()->GetDictionaryWithoutPathExpansion(active_user_.GetUserEmail(),
+                                                  &current_user_prefs);
+  if (current_user_prefs) {
+    current_user_prefs->SetKey(
+        prefs::kProximityAuthHasShownLoginDisabledMessage,
+        base::Value(has_shown));
+    return;
+  }
+
+  // Create an otherwise empty dictionary in order to ensure |has_shown| is
+  // persisted for |active_user_|.
+  std::unique_ptr<base::DictionaryValue> new_current_user_prefs =
+      std::make_unique<base::DictionaryValue>();
+  new_current_user_prefs->SetKey(
+      prefs::kProximityAuthHasShownLoginDisabledMessage,
+      base::Value(has_shown));
+  update->SetWithoutPathExpansion(active_user_.GetUserEmail(),
+                                  std::move(new_current_user_prefs));
+}
+
+bool ProximityAuthLocalStatePrefManager::HasShownLoginDisabledMessage() const {
+  const base::DictionaryValue* user_prefs = GetActiveUserPrefsDictionary();
+  if (!user_prefs)
+    return false;
+
+  bool pref_value = false;
+  user_prefs->GetBooleanWithoutPathExpansion(
+      prefs::kProximityAuthHasShownLoginDisabledMessage, &pref_value);
+  return pref_value;
+}
+
 const base::DictionaryValue*
 ProximityAuthLocalStatePrefManager::GetActiveUserPrefsDictionary() const {
   if (!active_user_.is_valid()) {
