@@ -45,8 +45,8 @@ class ProximityAuthLocalStatePrefManager : public ProximityAuthPrefManager {
   bool IsEasyUnlockAllowed() const override;
   bool IsEasyUnlockEnabled() const override;
   bool IsEasyUnlockEnabledStateSet() const override;
-  ProximityThreshold GetProximityThreshold() const override;
-  bool IsChromeOSLoginEnabled() override;
+  bool IsChromeOSLoginAllowed() const override;
+  bool IsChromeOSLoginEnabled() const override;
 
  private:
   // ProximityAuthPrefManager:
@@ -56,12 +56,13 @@ class ProximityAuthLocalStatePrefManager : public ProximityAuthPrefManager {
   int64_t GetLastPromotionCheckTimestampMs() const override;
   void SetPromotionShownCount(int count) override;
   int GetPromotionShownCount() const override;
-  void SetProximityThreshold(ProximityThreshold value) override;
   void SetIsChromeOSLoginEnabled(bool is_enabled) override;
+  void SetHasShownLoginDisabledMessage(bool has_shown) override;
+  bool HasShownLoginDisabledMessage() const override;
 
   const base::DictionaryValue* GetActiveUserPrefsDictionary() const;
 
-  // Contains local state perferences that outlive the lifetime of this object
+  // Contains local state preferences that outlive the lifetime of this object
   // and across process restarts. Not owned and must outlive this instance.
   PrefService* local_state_;
 
