@@ -9,12 +9,9 @@
 #include <vector>
 
 #include "base/macros.h"
-#include "base/memory/weak_ptr.h"
-#include "base/time/clock.h"
-#include "chromeos/components/proximity_auth/remote_device_life_cycle.h"
+#include "chromeos/components/multidevice/remote_device_ref.h"
 #include "chromeos/components/proximity_auth/screenlock_bridge.h"
 #include "components/account_id/account_id.h"
-#include "components/cryptauth/remote_device_ref.h"
 
 namespace chromeos {
 namespace secure_channel {
@@ -25,7 +22,6 @@ class SecureChannelClient;
 namespace proximity_auth {
 
 class ProximityAuthClient;
-class ProximityAuthPrefManager;
 class RemoteDeviceLifeCycle;
 class UnlockManager;
 
@@ -34,8 +30,7 @@ class UnlockManager;
 // phone) for each registered user, the system will handle the connection,
 // authentication, and messenging protocol when the screen is locked and the
 // registered user is focused.
-class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
-                            public ScreenlockBridge::Observer {
+class ProximityAuthSystem : public ScreenlockBridge::Observer {
  public:
   enum ScreenlockType { SESSION_LOCK, SIGN_IN };
 
@@ -58,17 +53,17 @@ class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
   // previously registered for the user, then they will be replaced.
   void SetRemoteDevicesForUser(
       const AccountId& account_id,
-      const cryptauth::RemoteDeviceRefList& remote_devices,
-      base::Optional<cryptauth::RemoteDeviceRef> local_device);
+      const chromeos::multidevice::RemoteDeviceRefList& remote_devices,
+      base::Optional<chromeos::multidevice::RemoteDeviceRef> local_device);
 
   // Returns the RemoteDevices registered for |account_id|. Returns an empty
   // list
   // if no devices are registered for |account_id|.
-  cryptauth::RemoteDeviceRefList GetRemoteDevicesForUser(
+  chromeos::multidevice::RemoteDeviceRefList GetRemoteDevicesForUser(
       const AccountId& account_id) const;
 
   // Called when the user clicks the user pod and attempts to unlock/sign-in.
-  void OnAuthAttempted(const AccountId& account_id);
+  void OnAuthAttempted();
 
   // Called when the system suspends.
   void OnSuspend();
@@ -76,15 +71,15 @@ class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
   // Called when the system wakes up from a suspended state.
   void OnSuspendDone();
 
+  // Called in order to disable attempts to get RemoteStatus from host devices.
+  void CancelConnectionAttempt();
+
  protected:
   // Constructor which allows passing in a custom |unlock_manager_|.
   // Exposed for testing.
   ProximityAuthSystem(
-      ScreenlockType screenlock_type,
-      ProximityAuthClient* proximity_auth_client,
       chromeos::secure_channel::SecureChannelClient* secure_channel_client,
-      std::unique_ptr<UnlockManager> unlock_manager,
-      ProximityAuthPrefManager* pref_manager);
+      std::unique_ptr<UnlockManager> unlock_manager);
 
   // Creates the RemoteDeviceLifeCycle for |remote_device| and |local_device|.
   // |remote_device| is the host intended to be connected to, and |local_device|
@@ -92,12 +87,8 @@ class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
   // user profile context.
   // Exposed for testing.
   virtual std::unique_ptr<RemoteDeviceLifeCycle> CreateRemoteDeviceLifeCycle(
-      cryptauth::RemoteDeviceRef remote_device,
-      base::Optional<cryptauth::RemoteDeviceRef> local_device);
-
-  // RemoteDeviceLifeCycle::Observer:
-  void OnLifeCycleStateChanged(RemoteDeviceLifeCycle::State old_state,
-                               RemoteDeviceLifeCycle::State new_state) override;
+      chromeos::multidevice::RemoteDeviceRef remote_device,
+      base::Optional<chromeos::multidevice::RemoteDeviceRef> local_device);
 
   // ScreenlockBridge::Observer:
   void OnScreenDidLock(
@@ -107,23 +98,14 @@ class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
   void OnFocusedUserChanged(const AccountId& account_id) override;
 
  private:
-  // Resumes |remote_device_life_cycle_| after device wakes up and waits a
-  // timeout.
-  void ResumeAfterWakeUpTimeout();
-
-  // The type of the screenlock (i.e. login or unlock).
-  ScreenlockType screenlock_type_;
-
   // Lists of remote devices, keyed by user account id.
-  std::map<AccountId, cryptauth::RemoteDeviceRefList> remote_devices_map_;
+  std::map<AccountId, chromeos::multidevice::RemoteDeviceRefList>
+      remote_devices_map_;
 
   // A mapping from each profile's account ID to the profile-specific
   // representation of this device (i.e. this Chrome OS device) for that
   // particular user profile.
-  std::map<AccountId, cryptauth::RemoteDeviceRef> local_device_map_;
-
-  // Delegate for Chrome dependent functionality.
-  ProximityAuthClient* proximity_auth_client_;
+  std::map<AccountId, chromeos::multidevice::RemoteDeviceRef> local_device_map_;
 
   // Entry point to the SecureChannel API.
   chromeos::secure_channel::SecureChannelClient* secure_channel_client_;
@@ -132,9 +114,6 @@ class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
   // the RemoteDevice of the currently focused user.
   std::unique_ptr<RemoteDeviceLifeCycle> remote_device_life_cycle_;
 
-  // Fetches EasyUnlock preferences. Must outlive this instance.
-  ProximityAuthPrefManager* pref_manager_;
-
   // Handles the interaction with the lock screen UI.
   std::unique_ptr<UnlockManager> unlock_manager_;
 
@@ -144,8 +123,6 @@ class ProximityAuthSystem : public RemoteDeviceLifeCycle::Observer,
   // True if the system is started_.
   bool started_;
 
-  base::WeakPtrFactory<ProximityAuthSystem> weak_ptr_factory_;
-
   DISALLOW_COPY_AND_ASSIGN(ProximityAuthSystem);
 };
 
