@@ -13,7 +13,7 @@
 #include "base/observer_list.h"
 #include "base/strings/string16.h"
 #include "base/values.h"
-#include "chromeos/components/proximity_auth/public/interfaces/auth_type.mojom.h"
+#include "chromeos/components/proximity_auth/public/mojom/auth_type.mojom.h"
 #include "components/account_id/account_id.h"
 
 namespace proximity_auth {
@@ -68,10 +68,6 @@ class ScreenlockBridge {
     // go to state where password is required for unlock.
     void SetHardlockOnClick();
 
-    // If the current lock screen is a trial run to introduce users to Easy
-    // Unlock, the icon will record metrics upon click.
-    void SetTrialRun();
-
     std::string GetIDString() const;
 
     UserPodCustomIcon icon() const { return icon_; }
@@ -84,8 +80,6 @@ class ScreenlockBridge {
 
     bool hardlock_on_click() const { return hardlock_on_click_; }
 
-    bool is_trial_run() const { return is_trial_run_; }
-
    private:
     UserPodCustomIcon icon_;
 
@@ -96,8 +90,6 @@ class ScreenlockBridge {
 
     bool hardlock_on_click_;
 
-    bool is_trial_run_;
-
     DISALLOW_COPY_AND_ASSIGN(UserPodCustomIconOptions);
   };
 
@@ -184,7 +176,7 @@ class ScreenlockBridge {
   ScreenlockBridge();
   ~ScreenlockBridge();
 
-  LockHandler* lock_handler_;  // Not owned
+  LockHandler* lock_handler_ = nullptr;  // Not owned
 
   // The last focused user's id.
   AccountId focused_account_id_;
