@@ -9,7 +9,7 @@
 #include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/time/default_clock.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 #include "chromeos/components/tether/message_wrapper.h"
 #include "chromeos/components/tether/proto/tether.pb.h"
 
@@ -23,47 +23,36 @@ DisconnectTetheringOperation::Factory*
 
 // static
 std::unique_ptr<DisconnectTetheringOperation>
-DisconnectTetheringOperation::Factory::NewInstance(
-    cryptauth::RemoteDeviceRef device_to_connect,
+DisconnectTetheringOperation::Factory::Create(
+    multidevice::RemoteDeviceRef device_to_connect,
     device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client,
-    BleConnectionManager* connection_manager) {
-  if (!factory_instance_) {
-    factory_instance_ = new Factory();
+    secure_channel::SecureChannelClient* secure_channel_client) {
+  if (factory_instance_) {
+    return factory_instance_->CreateInstance(
+        device_to_connect, device_sync_client, secure_channel_client);
   }
-  return factory_instance_->BuildInstance(device_to_connect, device_sync_client,
-                                          secure_channel_client,
-                                          connection_manager);
+
+  return base::WrapUnique(new DisconnectTetheringOperation(
+      device_to_connect, device_sync_client, secure_channel_client));
 }
 
 // static
-void DisconnectTetheringOperation::Factory::SetInstanceForTesting(
+void DisconnectTetheringOperation::Factory::SetFactoryForTesting(
     Factory* factory) {
   factory_instance_ = factory;
 }
 
-std::unique_ptr<DisconnectTetheringOperation>
-DisconnectTetheringOperation::Factory::BuildInstance(
-    cryptauth::RemoteDeviceRef device_to_connect,
-    device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client,
-    BleConnectionManager* connection_manager) {
-  return base::WrapUnique(new DisconnectTetheringOperation(
-      device_to_connect, device_sync_client, secure_channel_client,
-      connection_manager));
-}
+DisconnectTetheringOperation::Factory::~Factory() = default;
 
 DisconnectTetheringOperation::DisconnectTetheringOperation(
-    cryptauth::RemoteDeviceRef device_to_connect,
+    multidevice::RemoteDeviceRef device_to_connect,
     device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client,
-    BleConnectionManager* connection_manager)
+    secure_channel::SecureChannelClient* secure_channel_client)
     : MessageTransferOperation(
-          cryptauth::RemoteDeviceRefList{device_to_connect},
+          multidevice::RemoteDeviceRefList{device_to_connect},
           secure_channel::ConnectionPriority::kHigh,
           device_sync_client,
-          secure_channel_client,
-          connection_manager),
+          secure_channel_client),
       remote_device_(device_to_connect),
       has_sent_message_(false),
       clock_(base::DefaultClock::GetInstance()) {}
@@ -86,7 +75,7 @@ void DisconnectTetheringOperation::NotifyObserversOperationFinished(
 }
 
 void DisconnectTetheringOperation::OnDeviceAuthenticated(
-    cryptauth::RemoteDeviceRef remote_device) {
+    multidevice::RemoteDeviceRef remote_device) {
   DCHECK(remote_devices().size() == 1u && remote_devices()[0] == remote_device);
 
   disconnect_message_sequence_number_ = SendMessageToDevice(
