@@ -23,7 +23,6 @@ class SecureChannelClient;
 
 namespace tether {
 
-class BleConnectionManager;
 class TetherHostFetcher;
 
 class DisconnectTetheringRequestSenderImpl
@@ -32,20 +31,19 @@ class DisconnectTetheringRequestSenderImpl
  public:
   class Factory {
    public:
-    static std::unique_ptr<DisconnectTetheringRequestSender> NewInstance(
+    static std::unique_ptr<DisconnectTetheringRequestSender> Create(
         device_sync::DeviceSyncClient* device_sync_client,
         secure_channel::SecureChannelClient* secure_channel_client,
-        BleConnectionManager* ble_connection_manager,
         TetherHostFetcher* tether_host_fetcher);
 
-    static void SetInstanceForTesting(Factory* factory);
+    static void SetFactoryForTesting(Factory* factory);
 
    protected:
-    virtual std::unique_ptr<DisconnectTetheringRequestSender> BuildInstance(
+    virtual ~Factory();
+    virtual std::unique_ptr<DisconnectTetheringRequestSender> CreateInstance(
         device_sync::DeviceSyncClient* device_sync_client,
         secure_channel::SecureChannelClient* secure_channel_client,
-        BleConnectionManager* ble_connection_manager,
-        TetherHostFetcher* tether_host_fetcher);
+        TetherHostFetcher* tether_host_fetcher) = 0;
 
    private:
     static Factory* factory_instance_;
@@ -64,24 +62,23 @@ class DisconnectTetheringRequestSenderImpl
   DisconnectTetheringRequestSenderImpl(
       device_sync::DeviceSyncClient* device_sync_client,
       secure_channel::SecureChannelClient* secure_channel_client,
-      BleConnectionManager* ble_connection_manager,
       TetherHostFetcher* tether_host_fetcher);
 
  private:
   void OnTetherHostFetched(
       const std::string& device_id,
-      base::Optional<cryptauth::RemoteDeviceRef> tether_host);
+      base::Optional<multidevice::RemoteDeviceRef> tether_host);
 
   device_sync::DeviceSyncClient* device_sync_client_;
   secure_channel::SecureChannelClient* secure_channel_client_;
-  BleConnectionManager* ble_connection_manager_;
   TetherHostFetcher* tether_host_fetcher_;
 
   int num_pending_host_fetches_ = 0;
   std::map<std::string, std::unique_ptr<DisconnectTetheringOperation>>
       device_id_to_operation_map_;
 
-  base::WeakPtrFactory<DisconnectTetheringRequestSenderImpl> weak_ptr_factory_;
+  base::WeakPtrFactory<DisconnectTetheringRequestSenderImpl> weak_ptr_factory_{
+      this};
 
   DISALLOW_COPY_AND_ASSIGN(DisconnectTetheringRequestSenderImpl);
 };
