@@ -21,19 +21,19 @@ FakeWifiHotspotConnector::~FakeWifiHotspotConnector() = default;
 
 void FakeWifiHotspotConnector::CallMostRecentCallback(
     const std::string& wifi_guid) {
-  EXPECT_FALSE(most_recent_callback_.is_null());
-  most_recent_callback_.Run(wifi_guid);
+  EXPECT_FALSE(!most_recent_callback_);
+  std::move(most_recent_callback_).Run(wifi_guid);
 }
 
 void FakeWifiHotspotConnector::ConnectToWifiHotspot(
     const std::string& ssid,
     const std::string& password,
     const std::string& tether_network_guid,
-    const WifiHotspotConnector::WifiConnectionCallback& callback) {
+    WifiHotspotConnector::WifiConnectionCallback callback) {
   most_recent_ssid_ = ssid;
   most_recent_password_ = password;
   most_recent_tether_network_guid_ = tether_network_guid;
-  most_recent_callback_ = callback;
+  most_recent_callback_ = std::move(callback);
 }
 
 }  // namespace tether
