@@ -14,14 +14,14 @@ FakeWifiHotspotDisconnector::~FakeWifiHotspotDisconnector() = default;
 
 void FakeWifiHotspotDisconnector::DisconnectFromWifiHotspot(
     const std::string& wifi_network_guid,
-    const base::Closure& success_callback,
-    const network_handler::StringResultCallback& error_callback) {
+    base::OnceClosure success_callback,
+    StringErrorCallback error_callback) {
   last_disconnected_wifi_network_guid_ = wifi_network_guid;
 
   if (disconnection_error_name_.empty())
-    success_callback.Run();
+    std::move(success_callback).Run();
   else
-    error_callback.Run(disconnection_error_name_);
+    std::move(error_callback).Run(disconnection_error_name_);
 }
 
 }  // namespace tether
