@@ -29,10 +29,9 @@ class FakeWifiHotspotDisconnector : public WifiHotspotDisconnector {
   }
 
   // WifiHotspotDisconnector:
-  void DisconnectFromWifiHotspot(
-      const std::string& wifi_network_guid,
-      const base::Closure& success_callback,
-      const network_handler::StringResultCallback& error_callback) override;
+  void DisconnectFromWifiHotspot(const std::string& wifi_network_guid,
+                                 base::OnceClosure success_callback,
+                                 StringErrorCallback error_callback) override;
 
  private:
   std::string last_disconnected_wifi_network_guid_;
