@@ -7,7 +7,7 @@
 #include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/time/default_clock.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 #include "chromeos/components/tether/message_wrapper.h"
 #include "chromeos/components/tether/proto/tether.pb.h"
 
@@ -20,45 +20,35 @@ KeepAliveOperation::Factory* KeepAliveOperation::Factory::factory_instance_ =
     nullptr;
 
 // static
-std::unique_ptr<KeepAliveOperation> KeepAliveOperation::Factory::NewInstance(
-    cryptauth::RemoteDeviceRef device_to_connect,
+std::unique_ptr<KeepAliveOperation> KeepAliveOperation::Factory::Create(
+    multidevice::RemoteDeviceRef device_to_connect,
     device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client,
-    BleConnectionManager* connection_manager) {
-  if (!factory_instance_) {
-    factory_instance_ = new Factory();
+    secure_channel::SecureChannelClient* secure_channel_client) {
+  if (factory_instance_) {
+    return factory_instance_->CreateInstance(
+        device_to_connect, device_sync_client, secure_channel_client);
   }
-  return factory_instance_->BuildInstance(device_to_connect, device_sync_client,
-                                          secure_channel_client,
-                                          connection_manager);
+
+  return base::WrapUnique(new KeepAliveOperation(
+      device_to_connect, device_sync_client, secure_channel_client));
 }
 
 // static
-void KeepAliveOperation::Factory::SetInstanceForTesting(Factory* factory) {
+void KeepAliveOperation::Factory::SetFactoryForTesting(Factory* factory) {
   factory_instance_ = factory;
 }
 
-std::unique_ptr<KeepAliveOperation> KeepAliveOperation::Factory::BuildInstance(
-    cryptauth::RemoteDeviceRef device_to_connect,
-    device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client,
-    BleConnectionManager* connection_manager) {
-  return base::WrapUnique(
-      new KeepAliveOperation(device_to_connect, device_sync_client,
-                             secure_channel_client, connection_manager));
-}
+KeepAliveOperation::Factory::~Factory() = default;
 
 KeepAliveOperation::KeepAliveOperation(
-    cryptauth::RemoteDeviceRef device_to_connect,
+    multidevice::RemoteDeviceRef device_to_connect,
     device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client,
-    BleConnectionManager* connection_manager)
+    secure_channel::SecureChannelClient* secure_channel_client)
     : MessageTransferOperation(
-          cryptauth::RemoteDeviceRefList{device_to_connect},
+          multidevice::RemoteDeviceRefList{device_to_connect},
           secure_channel::ConnectionPriority::kMedium,
           device_sync_client,
-          secure_channel_client,
-          connection_manager),
+          secure_channel_client),
       remote_device_(device_to_connect),
       clock_(base::DefaultClock::GetInstance()) {}
 
@@ -73,7 +63,7 @@ void KeepAliveOperation::RemoveObserver(Observer* observer) {
 }
 
 void KeepAliveOperation::OnDeviceAuthenticated(
-    cryptauth::RemoteDeviceRef remote_device) {
+    multidevice::RemoteDeviceRef remote_device) {
   DCHECK(remote_devices().size() == 1u && remote_devices()[0] == remote_device);
   keep_alive_tickle_request_start_time_ = clock_->Now();
   SendMessageToDevice(remote_device,
@@ -82,7 +72,7 @@ void KeepAliveOperation::OnDeviceAuthenticated(
 
 void KeepAliveOperation::OnMessageReceived(
     std::unique_ptr<MessageWrapper> message_wrapper,
-    cryptauth::RemoteDeviceRef remote_device) {
+    multidevice::RemoteDeviceRef remote_device) {
   if (message_wrapper->GetMessageType() !=
       MessageType::KEEP_ALIVE_TICKLE_RESPONSE) {
     // If another type of message has been received, ignore it.
