@@ -5,6 +5,7 @@
 #include "chromeos/components/tether/keep_alive_scheduler.h"
 
 #include "base/bind.h"
+#include "base/metrics/histogram_functions.h"
 #include "chromeos/components/tether/device_id_tether_network_guid_map.h"
 #include "chromeos/components/tether/host_scan_cache.h"
 #include "chromeos/services/secure_channel/public/cpp/client/secure_channel_client.h"
@@ -14,19 +15,17 @@ namespace chromeos {
 namespace tether {
 
 // static
-const uint32_t KeepAliveScheduler::kKeepAliveIntervalMinutes = 4;
+const uint32_t KeepAliveScheduler::kKeepAliveIntervalMinutes = 3;
 
 KeepAliveScheduler::KeepAliveScheduler(
     device_sync::DeviceSyncClient* device_sync_client,
     secure_channel::SecureChannelClient* secure_channel_client,
     ActiveHost* active_host,
-    BleConnectionManager* connection_manager,
     HostScanCache* host_scan_cache,
     DeviceIdTetherNetworkGuidMap* device_id_tether_network_guid_map)
     : KeepAliveScheduler(device_sync_client,
                          secure_channel_client,
                          active_host,
-                         connection_manager,
                          host_scan_cache,
                          device_id_tether_network_guid_map,
                          std::make_unique<base::RepeatingTimer>()) {}
@@ -35,18 +34,15 @@ KeepAliveScheduler::KeepAliveScheduler(
     device_sync::DeviceSyncClient* device_sync_client,
     secure_channel::SecureChannelClient* secure_channel_client,
     ActiveHost* active_host,
-    BleConnectionManager* connection_manager,
     HostScanCache* host_scan_cache,
     DeviceIdTetherNetworkGuidMap* device_id_tether_network_guid_map,
     std::unique_ptr<base::RepeatingTimer> timer)
     : device_sync_client_(device_sync_client),
       secure_channel_client_(secure_channel_client),
       active_host_(active_host),
-      connection_manager_(connection_manager),
       host_scan_cache_(host_scan_cache),
       device_id_tether_network_guid_map_(device_id_tether_network_guid_map),
-      timer_(std::move(timer)),
-      weak_ptr_factory_(this) {
+      timer_(std::move(timer)) {
   active_host_->AddObserver(this);
 }
 
@@ -71,27 +67,33 @@ void KeepAliveScheduler::OnActiveHostChanged(
     active_host_device_ = change_info.new_active_host;
     timer_->Start(FROM_HERE,
                   base::TimeDelta::FromMinutes(kKeepAliveIntervalMinutes),
-                  base::Bind(&KeepAliveScheduler::SendKeepAliveTickle,
-                             weak_ptr_factory_.GetWeakPtr()));
+                  base::BindRepeating(&KeepAliveScheduler::SendKeepAliveTickle,
+                                      weak_ptr_factory_.GetWeakPtr()));
     SendKeepAliveTickle();
   }
 }
 
 void KeepAliveScheduler::OnOperationFinished(
-    cryptauth::RemoteDeviceRef remote_device,
+    multidevice::RemoteDeviceRef remote_device,
     std::unique_ptr<DeviceStatus> device_status) {
   // Make a copy before destroying the operation below.
-  const cryptauth::RemoteDeviceRef device_copy = remote_device;
+  const multidevice::RemoteDeviceRef device_copy = remote_device;
 
   keep_alive_operation_->RemoveObserver(this);
   keep_alive_operation_.reset();
 
+  base::UmaHistogramBoolean("InstantTethering.KeepAliveTickle.Result",
+                            device_status.get());
+
   if (!device_status) {
     // If the operation did not complete successfully, there is no new
     // information with which to update the cache.
+    PA_LOG(WARNING) << "Failed to send KeepAliveTickle message.";
     return;
   }
 
+  PA_LOG(INFO) << "Successfully sent KeepAliveTickle message.";
+
   std::string carrier;
   int32_t battery_percentage;
   int32_t signal_strength;
@@ -116,9 +118,8 @@ void KeepAliveScheduler::OnOperationFinished(
 void KeepAliveScheduler::SendKeepAliveTickle() {
   DCHECK(active_host_device_);
 
-  keep_alive_operation_ = KeepAliveOperation::Factory::NewInstance(
-      *active_host_device_, device_sync_client_, secure_channel_client_,
-      connection_manager_);
+  keep_alive_operation_ = KeepAliveOperation::Factory::Create(
+      *active_host_device_, device_sync_client_, secure_channel_client_);
   keep_alive_operation_->AddObserver(this);
   keep_alive_operation_->Initialize();
 }
