@@ -5,16 +5,14 @@
 #include "chromeos/components/tether/mock_host_connection_metrics_logger.h"
 
 #include "chromeos/components/tether/active_host.h"
-#include "chromeos/components/tether/ble_connection_manager.h"
 
 namespace chromeos {
 
 namespace tether {
 
 MockHostConnectionMetricsLogger::MockHostConnectionMetricsLogger(
-    BleConnectionManager* connection_manager,
     ActiveHost* active_host)
-    : HostConnectionMetricsLogger(connection_manager, active_host) {}
+    : HostConnectionMetricsLogger(active_host) {}
 
 MockHostConnectionMetricsLogger::~MockHostConnectionMetricsLogger() = default;
 
