@@ -19,7 +19,6 @@
 #include "chromeos/components/tether/host_scanner_impl.h"
 #include "chromeos/components/tether/hotspot_usage_duration_tracker.h"
 #include "chromeos/components/tether/keep_alive_scheduler.h"
-#include "chromeos/components/tether/master_host_scan_cache.h"
 #include "chromeos/components/tether/network_connection_handler_tether_delegate.h"
 #include "chromeos/components/tether/network_host_scan_cache.h"
 #include "chromeos/components/tether/network_list_sorter.h"
@@ -31,6 +30,7 @@
 #include "chromeos/components/tether/tether_network_disconnection_handler.h"
 #include "chromeos/components/tether/tether_session_completion_logger.h"
 #include "chromeos/components/tether/timer_factory.h"
+#include "chromeos/components/tether/top_level_host_scan_cache.h"
 #include "chromeos/components/tether/wifi_hotspot_connector.h"
 
 namespace chromeos {
@@ -44,7 +44,7 @@ SynchronousShutdownObjectContainerImpl::Factory*
 
 // static
 std::unique_ptr<SynchronousShutdownObjectContainer>
-SynchronousShutdownObjectContainerImpl::Factory::NewInstance(
+SynchronousShutdownObjectContainerImpl::Factory::Create(
     AsynchronousShutdownObjectContainer* asychronous_container,
     NotificationPresenter* notification_presenter,
     GmsCoreNotificationsStateTrackerImpl* gms_core_notifications_state_tracker,
@@ -55,36 +55,14 @@ SynchronousShutdownObjectContainerImpl::Factory::NewInstance(
     session_manager::SessionManager* session_manager,
     device_sync::DeviceSyncClient* device_sync_client,
     secure_channel::SecureChannelClient* secure_channel_client) {
-  if (!factory_instance_)
-    factory_instance_ = new Factory();
+  if (factory_instance_) {
+    return factory_instance_->CreateInstance(
+        asychronous_container, notification_presenter,
+        gms_core_notifications_state_tracker, pref_service,
+        network_state_handler, network_connect, network_connection_handler,
+        session_manager, device_sync_client, secure_channel_client);
+  }
 
-  return factory_instance_->BuildInstance(
-      asychronous_container, notification_presenter,
-      gms_core_notifications_state_tracker, pref_service, network_state_handler,
-      network_connect, network_connection_handler, session_manager,
-      device_sync_client, secure_channel_client);
-}
-
-// static
-void SynchronousShutdownObjectContainerImpl::Factory::SetInstanceForTesting(
-    Factory* factory) {
-  factory_instance_ = factory;
-}
-
-SynchronousShutdownObjectContainerImpl::Factory::~Factory() = default;
-
-std::unique_ptr<SynchronousShutdownObjectContainer>
-SynchronousShutdownObjectContainerImpl::Factory::BuildInstance(
-    AsynchronousShutdownObjectContainer* asychronous_container,
-    NotificationPresenter* notification_presenter,
-    GmsCoreNotificationsStateTrackerImpl* gms_core_notifications_state_tracker,
-    PrefService* pref_service,
-    NetworkStateHandler* network_state_handler,
-    NetworkConnect* network_connect,
-    NetworkConnectionHandler* network_connection_handler,
-    session_manager::SessionManager* session_manager,
-    device_sync::DeviceSyncClient* device_sync_client,
-    secure_channel::SecureChannelClient* secure_channel_client) {
   return base::WrapUnique(new SynchronousShutdownObjectContainerImpl(
       asychronous_container, notification_presenter,
       gms_core_notifications_state_tracker, pref_service, network_state_handler,
@@ -92,6 +70,14 @@ SynchronousShutdownObjectContainerImpl::Factory::BuildInstance(
       device_sync_client, secure_channel_client));
 }
 
+// static
+void SynchronousShutdownObjectContainerImpl::Factory::SetFactoryForTesting(
+    Factory* factory) {
+  factory_instance_ = factory;
+}
+
+SynchronousShutdownObjectContainerImpl::Factory::~Factory() = default;
+
 SynchronousShutdownObjectContainerImpl::SynchronousShutdownObjectContainerImpl(
     AsynchronousShutdownObjectContainer* asychronous_container,
     NotificationPresenter* notification_presenter,
@@ -130,22 +116,21 @@ SynchronousShutdownObjectContainerImpl::SynchronousShutdownObjectContainerImpl(
           network_state_handler_,
           tether_host_response_recorder_.get(),
           device_id_tether_network_guid_map_.get())),
-      master_host_scan_cache_(std::make_unique<MasterHostScanCache>(
+      top_level_host_scan_cache_(std::make_unique<TopLevelHostScanCache>(
           std::make_unique<TimerFactory>(),
           active_host_.get(),
           network_host_scan_cache_.get(),
           persistent_host_scan_cache_.get())),
-      notification_remover_(
-          std::make_unique<NotificationRemover>(network_state_handler_,
-                                                notification_presenter,
-                                                master_host_scan_cache_.get(),
-                                                active_host_.get())),
+      notification_remover_(std::make_unique<NotificationRemover>(
+          network_state_handler_,
+          notification_presenter,
+          top_level_host_scan_cache_.get(),
+          active_host_.get())),
       keep_alive_scheduler_(std::make_unique<KeepAliveScheduler>(
           device_sync_client,
           secure_channel_client,
           active_host_.get(),
-          asychronous_container->ble_connection_manager(),
-          master_host_scan_cache_.get(),
+          top_level_host_scan_cache_.get(),
           device_id_tether_network_guid_map_.get())),
       hotspot_usage_duration_tracker_(
           std::make_unique<HotspotUsageDurationTracker>(
@@ -154,7 +139,6 @@ SynchronousShutdownObjectContainerImpl::SynchronousShutdownObjectContainerImpl(
       connection_preserver_(std::make_unique<ConnectionPreserverImpl>(
           device_sync_client,
           secure_channel_client,
-          asychronous_container->ble_connection_manager(),
           network_state_handler_,
           active_host_.get(),
           tether_host_response_recorder_.get())),
@@ -164,13 +148,12 @@ SynchronousShutdownObjectContainerImpl::SynchronousShutdownObjectContainerImpl(
           network_state_handler_,
           session_manager,
           asychronous_container->tether_host_fetcher(),
-          asychronous_container->ble_connection_manager(),
           host_scan_device_prioritizer_.get(),
           tether_host_response_recorder_.get(),
           gms_core_notifications_state_tracker,
           notification_presenter,
           device_id_tether_network_guid_map_.get(),
-          master_host_scan_cache_.get(),
+          top_level_host_scan_cache_.get(),
           connection_preserver_.get(),
           base::DefaultClock::GetInstance())),
       host_scan_scheduler_(
@@ -178,9 +161,7 @@ SynchronousShutdownObjectContainerImpl::SynchronousShutdownObjectContainerImpl(
                                                   host_scanner_.get(),
                                                   session_manager)),
       host_connection_metrics_logger_(
-          std::make_unique<HostConnectionMetricsLogger>(
-              asychronous_container->ble_connection_manager(),
-              active_host_.get())),
+          std::make_unique<HostConnectionMetricsLogger>(active_host_.get())),
       tether_connector_(std::make_unique<TetherConnectorImpl>(
           device_sync_client,
           secure_channel_client,
@@ -188,10 +169,9 @@ SynchronousShutdownObjectContainerImpl::SynchronousShutdownObjectContainerImpl(
           wifi_hotspot_connector_.get(),
           active_host_.get(),
           asychronous_container->tether_host_fetcher(),
-          asychronous_container->ble_connection_manager(),
           tether_host_response_recorder_.get(),
           device_id_tether_network_guid_map_.get(),
-          master_host_scan_cache_.get(),
+          top_level_host_scan_cache_.get(),
           notification_presenter,
           host_connection_metrics_logger_.get(),
           asychronous_container->disconnect_tethering_request_sender(),
@@ -229,7 +209,7 @@ ActiveHost* SynchronousShutdownObjectContainerImpl::active_host() {
 }
 
 HostScanCache* SynchronousShutdownObjectContainerImpl::host_scan_cache() {
-  return master_host_scan_cache_.get();
+  return top_level_host_scan_cache_.get();
 }
 
 HostScanScheduler*
