@@ -16,10 +16,6 @@
 
 class PrefService;
 
-namespace cryptauth {
-class CryptAuthService;
-}  // namespace cryptauth
-
 namespace session_manager {
 class SessionManager;
 }  // namespace session_manager
@@ -59,8 +55,7 @@ class TetherComponentImpl : public TetherComponent {
 
   class Factory {
    public:
-    static std::unique_ptr<TetherComponent> NewInstance(
-        cryptauth::CryptAuthService* cryptauth_service,
+    static std::unique_ptr<TetherComponent> Create(
         device_sync::DeviceSyncClient* device_sync_client,
         secure_channel::SecureChannelClient* secure_channel_client,
         TetherHostFetcher* tether_host_fetcher,
@@ -76,11 +71,10 @@ class TetherComponentImpl : public TetherComponent {
         scoped_refptr<device::BluetoothAdapter> adapter,
         session_manager::SessionManager* session_manager);
 
-    static void SetInstanceForTesting(Factory* factory);
+    static void SetFactoryForTesting(Factory* factory);
 
    protected:
-    virtual std::unique_ptr<TetherComponent> BuildInstance(
-        cryptauth::CryptAuthService* cryptauth_service,
+    virtual std::unique_ptr<TetherComponent> CreateInstance(
         device_sync::DeviceSyncClient* device_sync_client,
         secure_channel::SecureChannelClient* secure_channel_client,
         TetherHostFetcher* tether_host_fetcher,
@@ -94,7 +88,7 @@ class TetherComponentImpl : public TetherComponent {
         NetworkConnect* network_connect,
         NetworkConnectionHandler* network_connection_handler,
         scoped_refptr<device::BluetoothAdapter> adapter,
-        session_manager::SessionManager* session_manager);
+        session_manager::SessionManager* session_manager) = 0;
 
    private:
     static Factory* factory_instance_;
@@ -107,7 +101,6 @@ class TetherComponentImpl : public TetherComponent {
 
  protected:
   TetherComponentImpl(
-      cryptauth::CryptAuthService* cryptauth_service,
       device_sync::DeviceSyncClient* device_sync_client,
       secure_channel::SecureChannelClient* secure_channel_client,
       TetherHostFetcher* tether_host_fetcher,
@@ -136,7 +129,7 @@ class TetherComponentImpl : public TetherComponent {
   bool has_shutdown_been_requested_ = false;
   ShutdownReason shutdown_reason_;
 
-  base::WeakPtrFactory<TetherComponentImpl> weak_ptr_factory_;
+  base::WeakPtrFactory<TetherComponentImpl> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(TetherComponentImpl);
 };
