@@ -5,10 +5,10 @@
 #include "chromeos/components/tether/tether_connector_impl.h"
 
 #include "base/bind.h"
+#include "base/callback_helpers.h"
 #include "base/metrics/histogram_macros.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 #include "chromeos/components/tether/active_host.h"
-#include "chromeos/components/tether/ble_connection_manager.h"
 #include "chromeos/components/tether/device_id_tether_network_guid_map.h"
 #include "chromeos/components/tether/disconnect_tethering_request_sender.h"
 #include "chromeos/components/tether/host_connection_metrics_logger.h"
@@ -30,7 +30,7 @@ namespace {
 void OnDisconnectFromWifiFailure(const std::string& device_id,
                                  const std::string& error_name) {
   PA_LOG(WARNING) << "Failed to disconnect from tether hotspot for device ID "
-                  << cryptauth::RemoteDeviceRef::TruncateDeviceIdForLogs(
+                  << multidevice::RemoteDeviceRef::TruncateDeviceIdForLogs(
                          device_id)
                   << ". Error: " << error_name;
 }
@@ -44,7 +44,6 @@ TetherConnectorImpl::TetherConnectorImpl(
     WifiHotspotConnector* wifi_hotspot_connector,
     ActiveHost* active_host,
     TetherHostFetcher* tether_host_fetcher,
-    BleConnectionManager* connection_manager,
     TetherHostResponseRecorder* tether_host_response_recorder,
     DeviceIdTetherNetworkGuidMap* device_id_tether_network_guid_map,
     HostScanCache* host_scan_cache,
@@ -58,15 +57,13 @@ TetherConnectorImpl::TetherConnectorImpl(
       wifi_hotspot_connector_(wifi_hotspot_connector),
       active_host_(active_host),
       tether_host_fetcher_(tether_host_fetcher),
-      connection_manager_(connection_manager),
       tether_host_response_recorder_(tether_host_response_recorder),
       device_id_tether_network_guid_map_(device_id_tether_network_guid_map),
       host_scan_cache_(host_scan_cache),
       notification_presenter_(notification_presenter),
       host_connection_metrics_logger_(host_connection_metrics_logger),
       disconnect_tethering_request_sender_(disconnect_tethering_request_sender),
-      wifi_hotspot_disconnector_(wifi_hotspot_disconnector),
-      weak_ptr_factory_(this) {}
+      wifi_hotspot_disconnector_(wifi_hotspot_disconnector) {}
 
 TetherConnectorImpl::~TetherConnectorImpl() {
   if (connect_tethering_operation_)
@@ -75,14 +72,14 @@ TetherConnectorImpl::~TetherConnectorImpl() {
 
 void TetherConnectorImpl::ConnectToNetwork(
     const std::string& tether_network_guid,
-    const base::Closure& success_callback,
-    const network_handler::StringResultCallback& error_callback) {
+    base::OnceClosure success_callback,
+    StringErrorCallback error_callback) {
   DCHECK(!tether_network_guid.empty());
   DCHECK(!success_callback.is_null());
   DCHECK(!error_callback.is_null());
 
-  PA_LOG(INFO) << "Attempting to connect to network with GUID "
-               << tether_network_guid << ".";
+  PA_LOG(VERBOSE) << "Attempting to connect to network with GUID "
+                  << tether_network_guid << ".";
   notification_presenter_->RemoveConnectionToHostFailedNotification();
 
   const std::string device_id =
@@ -97,24 +94,24 @@ void TetherConnectorImpl::ConnectToNetwork(
   DCHECK(device_id_pending_connection_ != device_id);
 
   if (!device_id_pending_connection_.empty()) {
-    PA_LOG(INFO) << "A connection attempt was already in progress to device "
-                 << "with ID " << device_id_pending_connection_ << ". "
-                 << "Canceling that connection attempt before continuing.";
+    PA_LOG(VERBOSE) << "A connection attempt was already in progress to device "
+                    << "with ID " << device_id_pending_connection_ << ". "
+                    << "Canceling that connection attempt before continuing.";
     CancelConnectionAttempt(
         device_id_tether_network_guid_map_->GetTetherNetworkGuidForDeviceId(
             device_id_pending_connection_));
   }
 
   device_id_pending_connection_ = device_id;
-  success_callback_ = success_callback;
-  error_callback_ = error_callback;
+  success_callback_ = std::move(success_callback);
+  error_callback_ = std::move(error_callback);
   active_host_->SetActiveHostConnecting(device_id, tether_network_guid);
 
   tether_host_fetcher_->FetchTetherHost(
       device_id_pending_connection_,
-      base::Bind(&TetherConnectorImpl::OnTetherHostToConnectFetched,
-                 weak_ptr_factory_.GetWeakPtr(),
-                 device_id_pending_connection_));
+      base::BindOnce(&TetherConnectorImpl::OnTetherHostToConnectFetched,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     device_id_pending_connection_));
 }
 
 bool TetherConnectorImpl::CancelConnectionAttempt(
@@ -130,8 +127,8 @@ bool TetherConnectorImpl::CancelConnectionAttempt(
     return false;
   }
 
-  PA_LOG(INFO) << "Canceling connection attempt to Tether network with GUID "
-               << tether_network_guid;
+  PA_LOG(VERBOSE) << "Canceling connection attempt to Tether network with GUID "
+                  << tether_network_guid;
 
   if (connect_tethering_operation_) {
     // If a ConnectTetheringOperation is in progress, stop it.
@@ -152,7 +149,9 @@ bool TetherConnectorImpl::CancelConnectionAttempt(
 }
 
 void TetherConnectorImpl::OnConnectTetheringRequestSent(
-    cryptauth::RemoteDeviceRef remote_device) {
+    multidevice::RemoteDeviceRef remote_device) {
+  did_send_successful_request_ = true;
+
   // If setup is required for the phone, display a notification so that the
   // user knows to follow instructions on the phone. Note that the notification
   // is displayed only after a request has been sent successfully. If the
@@ -172,23 +171,24 @@ void TetherConnectorImpl::OnConnectTetheringRequestSent(
 }
 
 void TetherConnectorImpl::OnSuccessfulConnectTetheringResponse(
-    cryptauth::RemoteDeviceRef remote_device,
+    multidevice::RemoteDeviceRef remote_device,
     const std::string& ssid,
     const std::string& password) {
   if (device_id_pending_connection_ != remote_device.GetDeviceId()) {
     // If the success was part of a previous attempt for a different device,
     // ignore it.
-    PA_LOG(INFO) << "Received successful ConnectTetheringResponse from "
-                 << "device with ID "
-                 << remote_device.GetTruncatedDeviceIdForLogs() << ", but the "
-                 << "connection attempt to that device has been canceled.";
+    PA_LOG(VERBOSE) << "Received successful ConnectTetheringResponse from "
+                    << "device with ID "
+                    << remote_device.GetTruncatedDeviceIdForLogs()
+                    << ", but the "
+                    << "connection attempt to that device has been canceled.";
 
     return;
   }
 
-  PA_LOG(INFO) << "Received successful ConnectTetheringResponse from device "
-               << "with ID " << remote_device.GetTruncatedDeviceIdForLogs()
-               << ". SSID: \"" << ssid << "\".";
+  PA_LOG(VERBOSE) << "Received successful ConnectTetheringResponse from device "
+                  << "with ID " << remote_device.GetTruncatedDeviceIdForLogs()
+                  << ". SSID: \"" << ssid << "\".";
 
   // Make a copy of the device ID, SSID, and password to pass below before
   // destroying |connect_tethering_operation_|.
@@ -201,20 +201,21 @@ void TetherConnectorImpl::OnSuccessfulConnectTetheringResponse(
 
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       ssid_copy, password_copy, active_host_->GetTetherNetworkGuid(),
-      base::Bind(&TetherConnectorImpl::OnWifiConnection,
-                 weak_ptr_factory_.GetWeakPtr(), remote_device_id));
+      base::BindOnce(&TetherConnectorImpl::OnWifiConnection,
+                     weak_ptr_factory_.GetWeakPtr(), remote_device_id));
 }
 
 void TetherConnectorImpl::OnConnectTetheringFailure(
-    cryptauth::RemoteDeviceRef remote_device,
+    multidevice::RemoteDeviceRef remote_device,
     ConnectTetheringOperation::HostResponseErrorCode error_code) {
   std::string device_id_copy = remote_device.GetDeviceId();
   if (device_id_pending_connection_ != device_id_copy) {
     // If the failure was part of a previous attempt for a different device,
     // ignore it.
-    PA_LOG(INFO) << "Received failed ConnectTetheringResponse from device with "
-                 << "ID " << remote_device.GetTruncatedDeviceIdForLogs()
-                 << ", but a connection to another device has already started.";
+    PA_LOG(VERBOSE)
+        << "Received failed ConnectTetheringResponse from device with "
+        << "ID " << remote_device.GetTruncatedDeviceIdForLogs()
+        << ", but a connection to another device has already started.";
     return;
   }
 
@@ -231,18 +232,18 @@ void TetherConnectorImpl::OnConnectTetheringFailure(
 
 void TetherConnectorImpl::OnTetherHostToConnectFetched(
     const std::string& device_id,
-    base::Optional<cryptauth::RemoteDeviceRef> tether_host_to_connect) {
+    base::Optional<multidevice::RemoteDeviceRef> tether_host_to_connect) {
   if (device_id_pending_connection_ != device_id) {
-    PA_LOG(INFO) << "Device to connect to has changed while device with ID "
-                 << cryptauth::RemoteDeviceRef::TruncateDeviceIdForLogs(
-                        device_id)
-                 << " was being fetched.";
+    PA_LOG(VERBOSE) << "Device to connect to has changed while device with ID "
+                    << multidevice::RemoteDeviceRef::TruncateDeviceIdForLogs(
+                           device_id)
+                    << " was being fetched.";
     return;
   }
 
   if (!tether_host_to_connect) {
     PA_LOG(ERROR) << "Could not fetch tether host with device ID "
-                  << cryptauth::RemoteDeviceRef::TruncateDeviceIdForLogs(
+                  << multidevice::RemoteDeviceRef::TruncateDeviceIdForLogs(
                          device_id)
                   << ". Cannot connect.";
     SetConnectionFailed(
@@ -257,11 +258,10 @@ void TetherConnectorImpl::OnTetherHostToConnectFetched(
   const std::string tether_network_guid =
       device_id_tether_network_guid_map_->GetTetherNetworkGuidForDeviceId(
           device_id);
-  connect_tethering_operation_ =
-      ConnectTetheringOperation::Factory::NewInstance(
-          *tether_host_to_connect, device_sync_client_, secure_channel_client_,
-          connection_manager_, tether_host_response_recorder_,
-          host_scan_cache_->DoesHostRequireSetup(tether_network_guid));
+  connect_tethering_operation_ = ConnectTetheringOperation::Factory::Create(
+      *tether_host_to_connect, device_sync_client_, secure_channel_client_,
+      tether_host_response_recorder_,
+      host_scan_cache_->DoesHostRequireSetup(tether_network_guid));
   connect_tethering_operation_->AddObserver(this);
   connect_tethering_operation_->Initialize();
 }
@@ -276,7 +276,7 @@ void TetherConnectorImpl::SetConnectionFailed(
   notification_presenter_->RemoveSetupRequiredNotification();
 
   // Save a copy of the callback before resetting it below.
-  network_handler::StringResultCallback error_callback = error_callback_;
+  StringErrorCallback error_callback = std::move(error_callback_);
 
   std::string failed_connection_device_id = device_id_pending_connection_;
   device_id_pending_connection_.clear();
@@ -284,7 +284,7 @@ void TetherConnectorImpl::SetConnectionFailed(
   success_callback_.Reset();
   error_callback_.Reset();
 
-  error_callback.Run(error_name);
+  std::move(error_callback).Run(error_name);
   active_host_->SetActiveHostDisconnected();
 
   host_connection_metrics_logger_->RecordConnectionToHostResult(
@@ -312,14 +312,14 @@ void TetherConnectorImpl::SetConnectionSucceeded(
 
   notification_presenter_->RemoveSetupRequiredNotification();
 
-  // Save a copy of the callback before resetting it below.
-  base::Closure success_callback = success_callback_;
+  // Save the callback before resetting it below.
+  base::OnceClosure success_callback = std::move(success_callback_);
 
   device_id_pending_connection_.clear();
   success_callback_.Reset();
   error_callback_.Reset();
 
-  success_callback.Run();
+  std::move(success_callback).Run();
   active_host_->SetActiveHostConnected(
       device_id,
       device_id_tether_network_guid_map_->GetTetherNetworkGuidForDeviceId(
@@ -334,24 +334,24 @@ void TetherConnectorImpl::OnWifiConnection(
     if (wifi_network_guid.empty()) {
       PA_LOG(WARNING)
           << "Failed to connect to Wi-Fi hotspot for device with ID "
-          << cryptauth::RemoteDeviceRef::TruncateDeviceIdForLogs(device_id)
+          << multidevice::RemoteDeviceRef::TruncateDeviceIdForLogs(device_id)
           << ", "
           << "but the connection to that device was canceled.";
       return;
     }
 
-    PA_LOG(INFO) << "Connected to Wi-Fi hotspot for device with ID "
-                 << cryptauth::RemoteDeviceRef::TruncateDeviceIdForLogs(
-                        device_id)
-                 << ", but the connection to that device was canceled. "
-                 << "Disconnecting.";
+    PA_LOG(VERBOSE) << "Connected to Wi-Fi hotspot for device with ID "
+                    << multidevice::RemoteDeviceRef::TruncateDeviceIdForLogs(
+                           device_id)
+                    << ", but the connection to that device was canceled. "
+                    << "Disconnecting.";
 
     // Disconnect from the Wi-Fi hotspot; otherwise, it is possible to be
     // connected to the Wi-Fi hotspot despite there being no active host. See
     // crbug.com/761171.
     wifi_hotspot_disconnector_->DisconnectFromWifiHotspot(
         wifi_network_guid, base::DoNothing(),
-        base::Bind(&OnDisconnectFromWifiFailure, device_id));
+        base::BindOnce(&OnDisconnectFromWifiFailure, device_id));
     return;
   }
 
@@ -359,7 +359,7 @@ void TetherConnectorImpl::OnWifiConnection(
     // If the Wi-Fi network ID is empty, then the connection did not succeed.
     PA_LOG(ERROR) << "Failed to connect to the hotspot belonging to the device "
                   << "with ID "
-                  << cryptauth::RemoteDeviceRef::TruncateDeviceIdForLogs(
+                  << multidevice::RemoteDeviceRef::TruncateDeviceIdForLogs(
                          device_id)
                   << ".";
 
@@ -433,8 +433,19 @@ TetherConnectorImpl::GetConnectionToHostResultFromErrorCode(
         CONNECTION_RESULT_FAILURE_INVALID_HOTSPOT_CREDENTIALS;
   }
 
+  if (error_code ==
+      ConnectTetheringOperation::HostResponseErrorCode::NO_RESPONSE) {
+    if (did_send_successful_request_) {
+      return HostConnectionMetricsLogger::ConnectionToHostResult::
+          CONNECTION_RESULT_FAILURE_SUCCESSFUL_REQUEST_BUT_NO_RESPONSE;
+    } else {
+      return HostConnectionMetricsLogger::ConnectionToHostResult::
+          CONNECTION_RESULT_FAILURE_NO_RESPONSE;
+    }
+  }
+
   return HostConnectionMetricsLogger::ConnectionToHostResult::
-      CONNECTION_RESULT_FAILURE_NO_RESPONSE;
+      CONNECTION_RESULT_FAILURE_UNRECOGNIZED_RESPONSE_ERROR;
 }
 
 }  // namespace tether
