@@ -7,21 +7,19 @@
 #include <memory>
 #include <sstream>
 
+#include "base/bind.h"
 #include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
 #include "base/test/metrics/histogram_tester.h"
-#include "base/test/scoped_task_environment.h"
 #include "base/test/simple_test_clock.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/timer/mock_timer.h"
 #include "base/values.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_device_client.h"
-#include "chromeos/dbus/shill_service_client.h"
 #include "chromeos/network/network_connect.h"
 #include "chromeos/network/network_state.h"
 #include "chromeos/network/network_state_handler.h"
-#include "chromeos/network/network_state_test.h"
+#include "chromeos/network/network_state_test_helper.h"
 #include "chromeos/network/shill_property_util.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -56,12 +54,12 @@ std::string CreateConfigurationJsonString(const std::string& guid) {
 
 }  // namespace
 
-class WifiHotspotConnectorTest : public NetworkStateTest {
+class WifiHotspotConnectorTest : public testing::Test {
  public:
   class TestNetworkConnect : public NetworkConnect {
    public:
-    explicit TestNetworkConnect(NetworkStateTest* network_state_test)
-        : network_state_test_(network_state_test) {}
+    explicit TestNetworkConnect(NetworkStateTestHelper* helper)
+        : helper_(helper) {}
     ~TestNetworkConnect() override = default;
 
     void set_is_running_in_test_task_runner(
@@ -91,8 +89,8 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
       EXPECT_TRUE(
           last_configuration_->GetString(shill::kGuidProperty, &wifi_guid));
 
-      last_service_path_created_ = network_state_test_->ConfigureService(
-          CreateConfigurationJsonString(wifi_guid));
+      last_service_path_created_ =
+          helper_->ConfigureService(CreateConfigurationJsonString(wifi_guid));
     }
 
     // NetworkConnect:
@@ -112,7 +110,7 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
       last_configuration_ = shill_properties->CreateDeepCopy();
 
       // Prevent nested RunLoops when ConfigureServiceWithLastNetworkConfig()
-      // calls NetworkStateTest::ConfigureService(); that causes threading
+      // calls NetworkStateTestHelper::ConfigureService(); that causes threading
       // issues. If |test_task_runner_| is causing this function to be run, the
       // client which triggered this call can manually call
       // ConfigureServiceWithLastNetworkConfig() once done.
@@ -131,7 +129,7 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
     }
 
    private:
-    NetworkStateTest* network_state_test_;
+    NetworkStateTestHelper* helper_;
     std::unique_ptr<base::DictionaryValue> last_configuration_;
     std::string last_service_path_created_;
     std::string network_id_to_connect_;
@@ -148,31 +146,29 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
     other_wifi_service_path_.clear();
     connection_callback_responses_.clear();
 
-    DBusThreadManager::Initialize();
-    NetworkStateTest::SetUp();
-    network_state_handler()->SetTetherTechnologyState(
+    helper_.network_state_handler()->SetTetherTechnologyState(
         NetworkStateHandler::TechnologyState::TECHNOLOGY_ENABLED);
-    network_state_handler()->SetTechnologyEnabled(
+    helper_.network_state_handler()->SetTechnologyEnabled(
         NetworkTypePattern::WiFi(), true /* enabled */,
         chromeos::network_handler::ErrorCallback());
     base::RunLoop().RunUntilIdle();
 
     SetUpShillState();
 
-    test_network_connect_ = base::WrapUnique(new TestNetworkConnect(this));
+    test_network_connect_ = base::WrapUnique(new TestNetworkConnect(&helper_));
 
-    network_state_handler()->AddTetherNetworkState(
+    helper_.network_state_handler()->AddTetherNetworkState(
         kTetherNetworkGuid, "tetherNetworkName" /* name */,
         "tetherNetworkCarrier" /* carrier */, 100 /* full battery */,
         100 /* full signal strength */, false /* has_connected_to_host */);
 
-    network_state_handler()->AddTetherNetworkState(
+    helper_.network_state_handler()->AddTetherNetworkState(
         kTetherNetworkGuid2, "tetherNetworkName2" /* name */,
         "tetherNetworkCarrier2" /* carrier */, 100 /* full battery */,
         100 /* full signal strength */, false /* has_connected_to_host */);
 
     wifi_hotspot_connector_ = base::WrapUnique(new WifiHotspotConnector(
-        network_state_handler(), test_network_connect_.get()));
+        helper_.network_state_handler(), test_network_connect_.get()));
 
     mock_timer_ = new base::MockOneShotTimer();
     test_clock_.SetNow(base::Time::UnixEpoch());
@@ -184,15 +180,12 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
   void SetUpShillState() {
     // Add a Wi-Fi service unrelated to the one under test. In some tests, a
     // connection from another device is tested.
-    other_wifi_service_path_ = ConfigureService(
+    other_wifi_service_path_ = helper_.ConfigureService(
         CreateConfigurationJsonString(std::string(kOtherWifiServiceGuid)));
   }
 
   void TearDown() override {
     wifi_hotspot_connector_.reset();
-    ShutdownNetworkState();
-    NetworkStateTest::TearDown();
-    DBusThreadManager::Shutdown();
   }
 
   void VerifyTimerSet() {
@@ -216,14 +209,15 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
   }
 
   void NotifyConnectable(const std::string& service_path) {
-    SetServiceProperty(service_path, std::string(shill::kConnectableProperty),
-                       base::Value(true));
+    helper_.SetServiceProperty(service_path,
+                               std::string(shill::kConnectableProperty),
+                               base::Value(true));
     RunTestTaskRunner();
   }
 
   void NotifyConnected(const std::string& service_path) {
-    SetServiceProperty(service_path, std::string(shill::kStateProperty),
-                       base::Value(shill::kStateReady));
+    helper_.SetServiceProperty(service_path, std::string(shill::kStateProperty),
+                               base::Value(shill::kStateReady));
     RunTestTaskRunner();
   }
 
@@ -278,12 +272,12 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
       const std::string& tether_guid,
       uint32_t expected_num_connection_attempts) {
     const NetworkState* wifi_network_state =
-        network_state_handler()->GetNetworkStateFromGuid(wifi_guid);
+        helper_.network_state_handler()->GetNetworkStateFromGuid(wifi_guid);
     ASSERT_TRUE(wifi_network_state);
     EXPECT_EQ(tether_guid, wifi_network_state->tether_guid());
 
     const NetworkState* tether_network_state =
-        network_state_handler()->GetNetworkStateFromGuid(tether_guid);
+        helper_.network_state_handler()->GetNetworkStateFromGuid(tether_guid);
     ASSERT_TRUE(tether_network_state);
     EXPECT_EQ(wifi_guid, tether_network_state->tether_guid());
 
@@ -293,7 +287,7 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
 
   void VerifyNetworkNotAssociated(const std::string& guid) {
     const NetworkState* network_state =
-        network_state_handler()->GetNetworkStateFromGuid(guid);
+        helper_.network_state_handler()->GetNetworkStateFromGuid(guid);
     ASSERT_TRUE(network_state);
     EXPECT_TRUE(network_state->tether_guid().empty());
   }
@@ -302,7 +296,12 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
     connection_callback_responses_.push_back(wifi_guid);
   }
 
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  NetworkStateHandler* network_state_handler() {
+    return helper_.network_state_handler();
+  }
+
+  base::test::SingleThreadTaskEnvironment task_environment_;
+  NetworkStateTestHelper helper_{true /* use_default_devices_and_services */};
 
   std::string other_wifi_service_path_;
   std::vector<std::string> connection_callback_responses_;
@@ -323,8 +322,8 @@ class WifiHotspotConnectorTest : public NetworkStateTest {
 TEST_F(WifiHotspotConnectorTest, TestConnect_NetworkDoesNotBecomeConnectable) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid =
       VerifyLastConfiguration(std::string(kSsid), std::string(kPassword));
@@ -346,8 +345,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_NetworkDoesNotBecomeConnectable) {
 TEST_F(WifiHotspotConnectorTest, TestConnect_AnotherNetworkBecomesConnectable) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid =
       VerifyLastConfiguration(std::string(kSsid), std::string(kPassword));
@@ -376,8 +375,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_AnotherNetworkBecomesConnectable) {
 TEST_F(WifiHotspotConnectorTest, TestConnect_CannotConnectToNetwork) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid =
       VerifyLastConfiguration(std::string(kSsid), std::string(kPassword));
@@ -405,8 +404,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_CannotConnectToNetwork) {
 TEST_F(WifiHotspotConnectorTest, TestConnect_DeletedWhileConnectionPending) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid =
       VerifyLastConfiguration(std::string(kSsid), std::string(kPassword));
@@ -431,8 +430,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_DeletedWhileConnectionPending) {
 TEST_F(WifiHotspotConnectorTest, TestConnect_Success) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid =
       VerifyLastConfiguration(std::string(kSsid), std::string(kPassword));
@@ -460,8 +459,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_Success) {
 TEST_F(WifiHotspotConnectorTest, TestConnect_Success_EmptyPassword) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string() /* password */, kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid = VerifyLastConfiguration(kSsid, std::string());
   EXPECT_FALSE(wifi_guid.empty());
@@ -489,8 +488,8 @@ TEST_F(WifiHotspotConnectorTest,
        TestConnect_SecondConnectionWhileWaitingForFirstToBecomeConnectable) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       "ssid1", "password1", "tetherNetworkGuid1",
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid1 = VerifyLastConfiguration("ssid1", "password1");
   EXPECT_FALSE(wifi_guid1.empty());
@@ -507,8 +506,8 @@ TEST_F(WifiHotspotConnectorTest,
   EXPECT_EQ(0u, connection_callback_responses_.size());
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       "ssid2", "password2", kTetherNetworkGuid2,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   std::string wifi_guid2 = VerifyLastConfiguration("ssid2", "password2");
   EXPECT_FALSE(wifi_guid2.empty());
@@ -554,8 +553,8 @@ TEST_F(WifiHotspotConnectorTest,
        TestConnect_SecondConnectionWhileWaitingForFirstToConnect) {
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       "ssid1", "password1", kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // Pass some arbitrary time -- this should not affect the
   // recorded duration because the start time should be reset
@@ -577,8 +576,8 @@ TEST_F(WifiHotspotConnectorTest,
   // After network becomes connectable, request a connection to second network.
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       "ssid2", "password2", kTetherNetworkGuid2,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // The first Tether and Wi-Fi networks should no longer be associated.
   VerifyNetworkNotAssociated(kTetherNetworkGuid);
@@ -629,8 +628,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_WifiDisabled_Success) {
 
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // Allow the asyncronous call to NetworkStateHandler::SetTechnologyEnabled()
   // within WifiHotspotConnector::ConnectToWifiHotspot() to synchronously
@@ -679,8 +678,8 @@ TEST_F(WifiHotspotConnectorTest,
 
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // Ensure that WifiHotspotConnector only begins configuring the Wi-Fi network
   // once Wi-Fi is enabled.
@@ -734,8 +733,8 @@ TEST_F(WifiHotspotConnectorTest, TestConnect_WifiDisabled_AttemptTimesOut) {
 
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       std::string(kSsid), std::string(kPassword), kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // Timeout timer fires.
   InvokeTimerTask();
@@ -769,8 +768,8 @@ TEST_F(WifiHotspotConnectorTest,
 
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       "ssid1", "password1", kTetherNetworkGuid,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // Pass some arbitrary time -- this should not affect the
   // recorded duration because the start time should be reset
@@ -781,8 +780,8 @@ TEST_F(WifiHotspotConnectorTest,
 
   wifi_hotspot_connector_->ConnectToWifiHotspot(
       "ssid2", "password2", kTetherNetworkGuid2,
-      base::Bind(&WifiHotspotConnectorTest::WifiConnectionCallback,
-                 base::Unretained(this)));
+      base::BindOnce(&WifiHotspotConnectorTest::WifiConnectionCallback,
+                     base::Unretained(this)));
 
   // Allow the asyncronous call to NetworkStateHandler::SetTechnologyEnabled()
   // within WifiHotspotConnector::ConnectToWifiHotspot() to synchronously
