@@ -13,10 +13,8 @@
 #include "chromeos/cryptohome/cryptohome_util.h"
 #include "chromeos/dbus/cryptohome/key.pb.h"
 #include "chromeos/dbus/cryptohome/rpc.pb.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
 #include "components/device_event_log/device_event_log.h"
 
-using chromeos::DBusThreadManager;
 using google::protobuf::RepeatedPtrField;
 
 namespace cryptohome {
@@ -28,65 +26,65 @@ HomedirMethods* g_homedir_methods = NULL;
 // The implementation of HomedirMethods
 class HomedirMethodsImpl : public HomedirMethods {
  public:
-  HomedirMethodsImpl() : weak_ptr_factory_(this) {}
+  HomedirMethodsImpl() {}
 
   ~HomedirMethodsImpl() override = default;
 
   void CheckKeyEx(const Identification& id,
                   const cryptohome::AuthorizationRequest& auth,
                   const cryptohome::CheckKeyRequest& request,
-                  const Callback& callback) override {
-    DBusThreadManager::Get()->GetCryptohomeClient()->CheckKeyEx(
+                  Callback callback) override {
+    chromeos::CryptohomeClient::Get()->CheckKeyEx(
         CreateAccountIdentifierFromIdentification(id), auth, request,
         base::BindOnce(&HomedirMethodsImpl::OnBaseReplyCallback,
-                       weak_ptr_factory_.GetWeakPtr(), callback));
+                       weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
   }
 
   void AddKeyEx(const Identification& id,
                 const AuthorizationRequest& auth,
                 const AddKeyRequest& request,
-                const Callback& callback) override {
-    DBusThreadManager::Get()->GetCryptohomeClient()->AddKeyEx(
+                Callback callback) override {
+    chromeos::CryptohomeClient::Get()->AddKeyEx(
         CreateAccountIdentifierFromIdentification(id), auth, request,
         base::BindOnce(&HomedirMethodsImpl::OnBaseReplyCallback,
-                       weak_ptr_factory_.GetWeakPtr(), callback));
+                       weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
   }
 
   void RemoveKeyEx(const Identification& id,
                    const AuthorizationRequest& auth,
                    const RemoveKeyRequest& request,
-                   const Callback& callback) override {
-    DBusThreadManager::Get()->GetCryptohomeClient()->RemoveKeyEx(
+                   Callback callback) override {
+    chromeos::CryptohomeClient::Get()->RemoveKeyEx(
         CreateAccountIdentifierFromIdentification(id), auth, request,
         base::BindOnce(&HomedirMethodsImpl::OnBaseReplyCallback,
-                       weak_ptr_factory_.GetWeakPtr(), callback));
+                       weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
   }
 
-  void UpdateKeyEx(const Identification& id,
-                   const AuthorizationRequest& auth,
-                   const UpdateKeyRequest& request,
-                   const Callback& callback) override {
-    DBusThreadManager::Get()->GetCryptohomeClient()->UpdateKeyEx(
+  void MassRemoveKeys(const Identification& id,
+                      const AuthorizationRequest& auth,
+                      const MassRemoveKeysRequest& request,
+                      Callback callback) override {
+    chromeos::CryptohomeClient::Get()->MassRemoveKeys(
         CreateAccountIdentifierFromIdentification(id), auth, request,
         base::BindOnce(&HomedirMethodsImpl::OnBaseReplyCallback,
-                       weak_ptr_factory_.GetWeakPtr(), callback));
+                       weak_ptr_factory_.GetWeakPtr(), std::move(callback)));
   }
 
  private:
-  void OnBaseReplyCallback(const Callback& callback,
-                           base::Optional<BaseReply> reply) {
+  void OnBaseReplyCallback(Callback callback, base::Optional<BaseReply> reply) {
     if (!reply.has_value()) {
-      callback.Run(false, MOUNT_ERROR_FATAL);
+      std::move(callback).Run(false, MOUNT_ERROR_FATAL);
       return;
     }
     if (reply->has_error() && reply->error() != CRYPTOHOME_ERROR_NOT_SET) {
-      callback.Run(false, CryptohomeErrorToMountError(reply->error()));
+      std::move(callback).Run(false,
+                              CryptohomeErrorToMountError(reply->error()));
       return;
     }
-    callback.Run(true, MOUNT_ERROR_NONE);
+    std::move(callback).Run(true, MOUNT_ERROR_NONE);
   }
 
-  base::WeakPtrFactory<HomedirMethodsImpl> weak_ptr_factory_;
+  base::WeakPtrFactory<HomedirMethodsImpl> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(HomedirMethodsImpl);
 };
