@@ -11,19 +11,18 @@
 #include <vector>
 
 #include "base/callback_forward.h"
+#include "base/component_export.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/optional.h"
-#include "chromeos/chromeos_export.h"
 
 namespace chromeos {
 
 // This class is used to get the system salt from cryptohome and cache it.
-class CHROMEOS_EXPORT SystemSaltGetter {
+class COMPONENT_EXPORT(CHROMEOS_CRYPTOHOME) SystemSaltGetter {
  public:
-  typedef base::Callback<void(const std::string& system_salt)>
-      GetSystemSaltCallback;
-
+  using GetSystemSaltCallback =
+      base::OnceCallback<void(const std::string& system_salt)>;
   using RawSalt = std::vector<uint8_t>;
 
   // Manage an explicitly initialized global instance.
@@ -37,14 +36,16 @@ class CHROMEOS_EXPORT SystemSaltGetter {
 
   // Returns system hash in hex encoded ascii format. Note: this may return
   // an empty string (e.g. errors in D-Bus layer)
-  void GetSystemSalt(const GetSystemSaltCallback& callback);
+  void GetSystemSalt(GetSystemSaltCallback callback);
 
   // Adds another callback to be called when system salt is received.
   // (If system salt is available, closure will be called immediately).
-  void AddOnSystemSaltReady(const base::Closure& closure);
+  void AddOnSystemSaltReady(base::OnceClosure closure);
 
   // Returns pointer to binary system salt if it is already known.
   // Returns nullptr if system salt is not known.
+  // WARNING: This pointer is null early in startup. Do not assume it is valid.
+  // Prefer GetSystemSalt() above. https://crbug.com/1122674
   const RawSalt* GetRawSalt() const;
 
   // This is for browser tests API.
@@ -56,18 +57,18 @@ class CHROMEOS_EXPORT SystemSaltGetter {
 
  private:
   // Used to implement GetSystemSalt().
-  void DidWaitForServiceToBeAvailable(const GetSystemSaltCallback& callback,
+  void DidWaitForServiceToBeAvailable(GetSystemSaltCallback callback,
                                       bool service_is_available);
-  void DidGetSystemSalt(const GetSystemSaltCallback& callback,
+  void DidGetSystemSalt(GetSystemSaltCallback callback,
                         base::Optional<std::vector<uint8_t>> system_salt);
 
   RawSalt raw_salt_;
   std::string system_salt_;
 
   // List of callbacks waiting for system salt ready event.
-  std::vector<base::Closure> on_system_salt_ready_;
+  std::vector<base::OnceClosure> on_system_salt_ready_;
 
-  base::WeakPtrFactory<SystemSaltGetter> weak_ptr_factory_;
+  base::WeakPtrFactory<SystemSaltGetter> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(SystemSaltGetter);
 };
