@@ -10,9 +10,9 @@
 #include <utility>
 
 #include "base/bind.h"
-#include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
+#include "base/test/task_environment.h"
 #include "chromeos/dbus/biod/messages.pb.h"
 #include "chromeos/dbus/biod/test_utils.h"
 #include "dbus/mock_bus.h"
@@ -82,16 +82,16 @@ class BiodClientTest : public testing::Test {
     EXPECT_CALL(*proxy_.get(), DoConnectToSignal(kInterface, _, _, _))
         .WillRepeatedly(Invoke(this, &BiodClientTest::ConnectToSignal));
 
-    client_.reset(BiodClient::Create(REAL_DBUS_CLIENT_IMPLEMENTATION));
-    client_->Init(bus_.get());
+    BiodClient::Initialize(bus_.get());
+    client_ = BiodClient::Get();
 
     // Execute callbacks posted by Init().
     base::RunLoop().RunUntilIdle();
   }
 
-  void GetBiometricType(uint32_t type) {
-    biometric_type_ = static_cast<biod::BiometricType>(type);
-  }
+  void TearDown() override { BiodClient::Shutdown(); }
+
+  void GetBiometricType(biod::BiometricType type) { biometric_type_ = type; }
 
  protected:
   // Add an expectation for method with |method_name| to be called. When the
@@ -158,13 +158,14 @@ class BiodClientTest : public testing::Test {
 
   std::map<std::string, std::unique_ptr<dbus::Response>> pending_method_calls_;
 
-  base::MessageLoop message_loop_;
+  base::test::SingleThreadTaskEnvironment task_environment_;
 
   // Mock bus and proxy for simulating calls.
   scoped_refptr<dbus::MockBus> bus_;
   scoped_refptr<dbus::MockObjectProxy> proxy_;
 
-  std::unique_ptr<BiodClient> client_;
+  // Convenience pointer to the global instance.
+  BiodClient* client_;
 
   // Maps from biod signal name to the corresponding callback provided by
   // |client_|.
@@ -181,7 +182,7 @@ class BiodClientTest : public testing::Test {
       dbus::ObjectProxy::OnConnectedCallback* on_connected_callback) {
     EXPECT_EQ(interface_name, kInterface);
     signal_callbacks_[signal_name] = signal_callback;
-    message_loop_.task_runner()->PostTask(
+    task_environment_.GetMainThreadTaskRunner()->PostTask(
         FROM_HERE,
         base::BindOnce(std::move(*on_connected_callback), interface_name,
                        signal_name, true /* success */));
@@ -196,7 +197,7 @@ class BiodClientTest : public testing::Test {
     auto pending_response = std::move(it->second);
     pending_method_calls_.erase(it);
 
-    message_loop_.task_runner()->PostTask(
+    task_environment_.GetMainThreadTaskRunner()->PostTask(
         FROM_HERE, base::BindOnce(&RunResponseCallback, std::move(*callback),
                                   std::move(pending_response)));
   }
@@ -217,7 +218,7 @@ TEST_F(BiodClientTest, TestStartEnrollSession) {
   dbus::ObjectPath returned_path(kInvalidTestPath);
   client_->StartEnrollSession(
       kFakeId, kFakeLabel,
-      base::Bind(&test_utils::CopyObjectPath, &returned_path));
+      base::BindOnce(&test_utils::CopyObjectPath, &returned_path));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(dbus::ObjectPath(), returned_path);
 
@@ -229,7 +230,7 @@ TEST_F(BiodClientTest, TestStartEnrollSession) {
   returned_path = dbus::ObjectPath(kInvalidTestPath);
   client_->StartEnrollSession(
       kFakeId, kFakeLabel,
-      base::Bind(&test_utils::CopyObjectPath, &returned_path));
+      base::BindOnce(&test_utils::CopyObjectPath, &returned_path));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(dbus::ObjectPath(), returned_path);
 
@@ -244,7 +245,7 @@ TEST_F(BiodClientTest, TestStartEnrollSession) {
   returned_path = dbus::ObjectPath(kInvalidTestPath);
   client_->StartEnrollSession(
       kFakeId, kFakeLabel,
-      base::Bind(&test_utils::CopyObjectPath, &returned_path));
+      base::BindOnce(&test_utils::CopyObjectPath, &returned_path));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(kFakeObjectPath, returned_path);
 }
@@ -268,7 +269,7 @@ TEST_F(BiodClientTest, TestGetRecordsForUser) {
       dbus::ObjectPath(kInvalidTestPath)};
   client_->GetRecordsForUser(
       kFakeId,
-      base::Bind(&test_utils::CopyObjectPathArray, &returned_object_paths));
+      base::BindOnce(&test_utils::CopyObjectPathArray, &returned_object_paths));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(kFakeObjectPaths, returned_object_paths);
 
@@ -279,7 +280,7 @@ TEST_F(BiodClientTest, TestGetRecordsForUser) {
   returned_object_paths = {dbus::ObjectPath(kInvalidTestPath)};
   client_->GetRecordsForUser(
       kFakeId,
-      base::Bind(&test_utils::CopyObjectPathArray, &returned_object_paths));
+      base::BindOnce(&test_utils::CopyObjectPathArray, &returned_object_paths));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(std::vector<dbus::ObjectPath>(), returned_object_paths);
 }
@@ -315,7 +316,7 @@ TEST_F(BiodClientTest, TestStartAuthentication) {
   AddMethodExpectation(biod::kBiometricsManagerStartAuthSessionMethod, nullptr);
   dbus::ObjectPath returned_path(kInvalidTestPath);
   client_->StartAuthSession(
-      base::Bind(&test_utils::CopyObjectPath, &returned_path));
+      base::BindOnce(&test_utils::CopyObjectPath, &returned_path));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(dbus::ObjectPath(), returned_path);
 
@@ -326,7 +327,7 @@ TEST_F(BiodClientTest, TestStartAuthentication) {
                        std::move(bad_response));
   returned_path = dbus::ObjectPath(kInvalidTestPath);
   client_->StartAuthSession(
-      base::Bind(&test_utils::CopyObjectPath, &returned_path));
+      base::BindOnce(&test_utils::CopyObjectPath, &returned_path));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(dbus::ObjectPath(), returned_path);
 
@@ -340,14 +341,13 @@ TEST_F(BiodClientTest, TestStartAuthentication) {
                        std::move(response));
   returned_path = dbus::ObjectPath(kInvalidTestPath);
   client_->StartAuthSession(
-      base::Bind(&test_utils::CopyObjectPath, &returned_path));
+      base::BindOnce(&test_utils::CopyObjectPath, &returned_path));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(kFakeObjectPath, returned_path);
 }
 
 TEST_F(BiodClientTest, TestRequestBiometricType) {
-  const biod::BiometricType kFakeBiometricType =
-      biod::BIOMETRIC_TYPE_FINGERPRINT;
+  const auto kFakeBiometricType = biod::BIOMETRIC_TYPE_FINGERPRINT;
 
   std::unique_ptr<dbus::Response> response(dbus::Response::CreateEmpty());
   dbus::MessageWriter writer(response.get());
@@ -357,8 +357,8 @@ TEST_F(BiodClientTest, TestRequestBiometricType) {
   // return this exact biometric type.
   biometric_type_ = biod::BIOMETRIC_TYPE_MAX;
   AddMethodExpectation(dbus::kDBusPropertiesGet, std::move(response));
-  client_->RequestType(
-      base::Bind(&BiodClientTest::GetBiometricType, base::Unretained(this)));
+  client_->RequestType(base::BindOnce(&BiodClientTest::GetBiometricType,
+                                      base::Unretained(this)));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(kFakeBiometricType, biometric_type_);
 
@@ -366,10 +366,10 @@ TEST_F(BiodClientTest, TestRequestBiometricType) {
   // type.
   biometric_type_ = biod::BIOMETRIC_TYPE_MAX;
   AddMethodExpectation(dbus::kDBusPropertiesGet, nullptr);
-  client_->RequestType(
-      base::Bind(&BiodClientTest::GetBiometricType, base::Unretained(this)));
+  client_->RequestType(base::BindOnce(&BiodClientTest::GetBiometricType,
+                                      base::Unretained(this)));
   base::RunLoop().RunUntilIdle();
-  EXPECT_EQ(biod::BiometricType::BIOMETRIC_TYPE_UNKNOWN, biometric_type_);
+  EXPECT_EQ(biod::BIOMETRIC_TYPE_UNKNOWN, biometric_type_);
 }
 
 TEST_F(BiodClientTest, TestRequestRecordLabel) {
@@ -385,7 +385,8 @@ TEST_F(BiodClientTest, TestRequestRecordLabel) {
   std::string returned_label = kInvalidString;
   AddMethodExpectation(dbus::kDBusPropertiesGet, std::move(response));
   client_->RequestRecordLabel(
-      kFakeRecordPath, base::Bind(&test_utils::CopyString, &returned_label));
+      kFakeRecordPath,
+      base::BindOnce(&test_utils::CopyString, &returned_label));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ(kFakeLabel, returned_label);
 
@@ -393,7 +394,8 @@ TEST_F(BiodClientTest, TestRequestRecordLabel) {
   returned_label = kInvalidString;
   AddMethodExpectation(dbus::kDBusPropertiesGet, nullptr);
   client_->RequestRecordLabel(
-      kFakeRecordPath, base::Bind(&test_utils::CopyString, &returned_label));
+      kFakeRecordPath,
+      base::BindOnce(&test_utils::CopyString, &returned_label));
   base::RunLoop().RunUntilIdle();
   EXPECT_EQ("", returned_label);
 }
