@@ -57,12 +57,12 @@ TEST(CrosDisksClientTest, DiskInfo) {
   const std::string kDriveModel = "DriveModel";
   const std::string kIdLabel = "UNTITLED";
   const std::string kIdUuid = "XXXX-YYYY";
-  const std::string kNativePath = "/sys/devices/.../sdb/sdb1";
   const std::string kProductId = "1234";
   const std::string kProductName = "Product Name";
   const std::string kVendorId = "0000";
   const std::string kVendorName = "Vendor Name";
   const std::string kFileSystemType = "exfat";
+  const bool kIsAutoMountable = true;
 
   // Construct a fake response of GetDeviceProperties().
   std::unique_ptr<dbus::Response> response(dbus::Response::CreateEmpty());
@@ -84,6 +84,8 @@ TEST(CrosDisksClientTest, DiskInfo) {
                         kDeviceIsOnRemovableDevice);
     AppendBoolDictEntry(&array_writer, cros_disks::kDeviceIsReadOnly,
                         kDeviceIsReadOnly);
+    AppendBoolDictEntry(&array_writer, cros_disks::kIsAutoMountable,
+                        kIsAutoMountable);
     {
       dbus::MessageWriter entry_writer(NULL);
       array_writer.OpenDictEntry(&entry_writer);
@@ -116,7 +118,6 @@ TEST(CrosDisksClientTest, DiskInfo) {
     AppendStringDictEntry(&array_writer, cros_disks::kDriveModel, kDriveModel);
     AppendStringDictEntry(&array_writer, cros_disks::kIdLabel, kIdLabel);
     AppendStringDictEntry(&array_writer, cros_disks::kIdUuid, kIdUuid);
-    AppendStringDictEntry(&array_writer, cros_disks::kNativePath, kNativePath);
     AppendStringDictEntry(&array_writer, cros_disks::kProductId, kProductId);
     AppendStringDictEntry(&array_writer, cros_disks::kProductName,
                           kProductName);
@@ -144,7 +145,6 @@ TEST(CrosDisksClientTest, DiskInfo) {
   EXPECT_EQ(kDeviceIsMediaAvailable, result.has_media());
   EXPECT_EQ(kDeviceIsOnBootDevice, result.on_boot_device());
   EXPECT_EQ(kDeviceIsOnRemovableDevice, result.on_removable_device());
-  EXPECT_EQ(kNativePath, result.system_path());
   EXPECT_EQ(kDeviceFile, result.file_path());
   EXPECT_EQ(kVendorId, result.vendor_id());
   EXPECT_EQ(kVendorName, result.vendor_name());
@@ -157,6 +157,7 @@ TEST(CrosDisksClientTest, DiskInfo) {
   EXPECT_EQ(DEVICE_TYPE_SD, result.device_type());
   EXPECT_EQ(kMountPath, result.mount_path());
   EXPECT_EQ(kFileSystemType, result.file_system_type());
+  EXPECT_EQ(kIsAutoMountable, result.is_auto_mountable());
 }
 
 TEST(CrosDisksClientTest, ComposeMountOptions) {
