@@ -11,7 +11,8 @@
 #include "base/location.h"
 #include "base/logging.h"
 #include "base/single_thread_task_runner.h"
-#include "base/task_scheduler/post_task.h"
+#include "base/task/post_task.h"
+#include "base/task/thread_pool.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "url/gurl.h"
 
@@ -54,14 +55,7 @@ MountError PerformFakeMount(const std::string& source_path,
 
 }  // namespace
 
-FakeCrosDisksClient::FakeCrosDisksClient()
-    : unmount_call_count_(0),
-      unmount_success_(true),
-      format_call_count_(0),
-      format_success_(true),
-      rename_call_count_(0),
-      rename_success_(true),
-      weak_ptr_factory_(this) {}
+FakeCrosDisksClient::FakeCrosDisksClient() = default;
 
 FakeCrosDisksClient::~FakeCrosDisksClient() = default;
 
@@ -83,6 +77,10 @@ void FakeCrosDisksClient::Mount(const std::string& source_path,
                                 MountAccessMode access_mode,
                                 RemountOption remount,
                                 VoidDBusMethodCallback callback) {
+  if (block_mount_) {
+    return;
+  }
+
   // This fake implementation assumes mounted path is device when source_format
   // is empty, or an archive otherwise.
   MountType type =
@@ -117,7 +115,7 @@ void FakeCrosDisksClient::Mount(const std::string& source_path,
   }
   mounted_paths_.insert(mounted_path);
 
-  base::PostTaskWithTraitsAndReplyWithResult(
+  base::ThreadPool::PostTaskAndReplyWithResult(
       FROM_HERE,
       {base::MayBlock(), base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
       base::BindOnce(&PerformFakeMount, source_path, mounted_path, type),
@@ -143,57 +141,60 @@ void FakeCrosDisksClient::DidMount(const std::string& source_path,
 }
 
 void FakeCrosDisksClient::Unmount(const std::string& device_path,
-                                  UnmountOptions options,
-                                  VoidDBusMethodCallback callback) {
+                                  UnmountCallback callback) {
   DCHECK(!callback.is_null());
 
   unmount_call_count_++;
   last_unmount_device_path_ = device_path;
-  last_unmount_options_ = options;
 
   // Remove the dummy mounted directory if it exists.
   if (mounted_paths_.erase(base::FilePath::FromUTF8Unsafe(device_path))) {
-    base::PostTaskWithTraitsAndReply(
+    base::ThreadPool::PostTaskAndReply(
         FROM_HERE,
-        {base::MayBlock(), base::TaskPriority::BACKGROUND,
+        {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
          base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
-        base::BindOnce(base::IgnoreResult(&base::DeleteFile),
-                       base::FilePath::FromUTF8Unsafe(device_path),
-                       true /* recursive */),
-        base::BindOnce(std::move(callback), unmount_success_));
+        base::BindOnce(base::GetDeletePathRecursivelyCallback(),
+                       base::FilePath::FromUTF8Unsafe(device_path)),
+        base::BindOnce(std::move(callback), unmount_error_));
   } else {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::BindOnce(std::move(callback), unmount_success_));
+        FROM_HERE, base::BindOnce(std::move(callback), unmount_error_));
   }
   if (!unmount_listener_.is_null())
     unmount_listener_.Run();
 }
 
-void FakeCrosDisksClient::EnumerateAutoMountableDevices(
-    const EnumerateDevicesCallback& callback,
-    const base::Closure& error_callback) {}
-
-void FakeCrosDisksClient::EnumerateDevices(
-    const EnumerateDevicesCallback& callback,
-    const base::Closure& error_callback) {}
+void FakeCrosDisksClient::EnumerateDevices(EnumerateDevicesCallback callback,
+                                           base::OnceClosure error_callback) {}
 
 void FakeCrosDisksClient::EnumerateMountEntries(
-    const EnumerateMountEntriesCallback& callback,
-    const base::Closure& error_callback) {
-}
+    EnumerateMountEntriesCallback callback,
+    base::OnceClosure error_callback) {}
 
 void FakeCrosDisksClient::Format(const std::string& device_path,
                                  const std::string& filesystem,
+                                 const std::string& label,
                                  VoidDBusMethodCallback callback) {
   DCHECK(!callback.is_null());
 
   format_call_count_++;
   last_format_device_path_ = device_path;
   last_format_filesystem_ = filesystem;
+  last_format_label_ = label;
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE, base::BindOnce(std::move(callback), format_success_));
 }
 
+void FakeCrosDisksClient::SinglePartitionFormat(const std::string& device_path,
+                                                PartitionCallback callback) {
+  DCHECK(!callback.is_null());
+
+  partition_call_count_++;
+  last_partition_device_path_ = device_path;
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::BindOnce(std::move(callback), partition_error_));
+}
+
 void FakeCrosDisksClient::Rename(const std::string& device_path,
                                  const std::string& volume_name,
                                  VoidDBusMethodCallback callback) {
@@ -208,8 +209,21 @@ void FakeCrosDisksClient::Rename(const std::string& device_path,
 
 void FakeCrosDisksClient::GetDeviceProperties(
     const std::string& device_path,
-    const GetDevicePropertiesCallback& callback,
-    const base::Closure& error_callback) {
+    GetDevicePropertiesCallback callback,
+    base::OnceClosure error_callback) {
+  DCHECK(!callback.is_null());
+  if (!next_get_device_properties_disk_info_ ||
+      next_get_device_properties_disk_info_->device_path() != device_path) {
+    base::ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
+                                                  std::move(error_callback));
+    return;
+  }
+
+  get_device_properties_success_count_++;
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE,
+      base::BindOnce(std::move(callback),
+                     std::cref(*next_get_device_properties_disk_info_)));
 }
 
 void FakeCrosDisksClient::NotifyMountCompleted(MountError error_code,
@@ -236,6 +250,13 @@ void FakeCrosDisksClient::NotifyRenameCompleted(
     observer.OnRenameCompleted(error_code, device_path);
 }
 
+void FakeCrosDisksClient::NotifyMountEvent(MountEventType mount_event,
+                                           const std::string& device_path) {
+  for (auto& observer : observer_list_) {
+    observer.OnMountEvent(mount_event, device_path);
+  }
+}
+
 void FakeCrosDisksClient::AddCustomMountPointCallback(
     FakeCrosDisksClient::CustomMountPointCallback custom_mount_point_callback) {
   custom_mount_point_callbacks_.emplace_back(custom_mount_point_callback);
