@@ -7,7 +7,6 @@
 #include <utility>
 
 #include "base/bind.h"
-#include "base/callback_helpers.h"
 #include "base/posix/eintr_wrapper.h"
 #include "base/task_runner.h"
 #include "net/base/file_stream.h"
@@ -17,9 +16,8 @@
 namespace chromeos {
 
 PipeReader::PipeReader(const scoped_refptr<base::TaskRunner>& task_runner)
-    : io_buffer_(new net::IOBufferWithSize(4096)),
-      task_runner_(task_runner),
-      weak_ptr_factory_(this) {}
+    : io_buffer_(base::MakeRefCounted<net::IOBufferWithSize>(4096)),
+      task_runner_(task_runner) {}
 
 PipeReader::~PipeReader() = default;
 
@@ -76,7 +74,7 @@ void PipeReader::OnRead(int byte_count) {
     // Clear members before calling the |callback|.
     data_.clear();
     data_stream_.reset();
-    base::ResetAndReturn(&callback_).Run(std::move(result));
+    std::move(callback_).Run(std::move(result));
     return;
   }
 
