@@ -10,9 +10,9 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/logging.h"
 #include "base/stl_util.h"
-#include "base/sys_info.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
+#include "base/system/sys_info.h"
 #include "dbus/bus.h"
 #include "dbus/exported_object.h"
 #include "dbus/object_path.h"
@@ -60,9 +60,10 @@ class CrosDBusServiceImpl : public CrosDBusService {
     // situation if we don't allow the new browser instance to replace the old
     // as the owner of |service_name_| as seen in http://crbug.com/234382.
     // Hence, REQUIRE_PRIMARY_ALLOW_REPLACEMENT.
-    bus_->RequestOwnership(
-        service_name_, dbus::Bus::REQUIRE_PRIMARY_ALLOW_REPLACEMENT,
-        base::Bind(&CrosDBusServiceImpl::OnOwnership, base::Unretained(this)));
+    bus_->RequestOwnership(service_name_,
+                           dbus::Bus::REQUIRE_PRIMARY_ALLOW_REPLACEMENT,
+                           base::BindOnce(&CrosDBusServiceImpl::OnOwnership,
+                                          base::Unretained(this)));
 
     service_started_ = true;
   }
@@ -105,14 +106,15 @@ class CrosDBusServiceStubImpl : public CrosDBusService {
 
 // static
 std::unique_ptr<CrosDBusService> CrosDBusService::Create(
+    dbus::Bus* system_bus,
     const std::string& service_name,
     const dbus::ObjectPath& object_path,
     ServiceProviderList service_providers) {
-  if (DBusThreadManager::Get()->IsUsingFakes())
+  if (!system_bus)
     return std::make_unique<CrosDBusServiceStubImpl>();
 
-  return CreateRealImpl(DBusThreadManager::Get()->GetSystemBus(), service_name,
-                        object_path, std::move(service_providers));
+  return CreateRealImpl(system_bus, service_name, object_path,
+                        std::move(service_providers));
 }
 
 // static
