@@ -4,9 +4,12 @@
 
 #include "chromeos/dbus/smb_provider_client.h"
 
+#include <memory>
+
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/files/file_util.h"
+#include "base/logging.h"
 #include "base/memory/weak_ptr.h"
 #include "dbus/bus.h"
 #include "dbus/message.h"
@@ -56,6 +59,14 @@ bool ParseDeleteList(const base::ScopedFD& fd,
          delete_list->ParseFromArray(buffer.data(), buffer.size());
 }
 
+std::unique_ptr<smbprovider::MountConfigProto> CreateMountConfigProto(
+    bool enable_ntlm) {
+  auto mount_config = std::make_unique<smbprovider::MountConfigProto>();
+  mount_config->set_enable_ntlm(enable_ntlm);
+
+  return mount_config;
+}
+
 class SmbProviderClientImpl : public SmbProviderClient {
  public:
   SmbProviderClientImpl() = default;
@@ -63,48 +74,38 @@ class SmbProviderClientImpl : public SmbProviderClient {
   ~SmbProviderClientImpl() override {}
 
   void Mount(const base::FilePath& share_path,
-             const std::string& workgroup,
-             const std::string& username,
+             const MountOptions& options,
              base::ScopedFD password_fd,
              MountCallback callback) override {
-    smbprovider::MountOptionsProto options;
-    options.set_path(share_path.value());
-    options.set_workgroup(workgroup);
-    options.set_username(username);
+    smbprovider::MountOptionsProto options_proto;
+    options_proto.set_path(share_path.value());
+    options_proto.set_original_path(options.original_path);
+    options_proto.set_workgroup(options.workgroup);
+    options_proto.set_username(options.username);
+    options_proto.set_skip_connect(options.skip_connect);
+    options_proto.set_account_hash(options.account_hash);
+    options_proto.set_save_password(options.save_password);
+    options_proto.set_restore_password(options.restore_password);
+
+    std::unique_ptr<smbprovider::MountConfigProto> config =
+        CreateMountConfigProto(options.ntlm_enabled);
+    options_proto.set_allocated_mount_config(config.release());
 
     dbus::MethodCall method_call(smbprovider::kSmbProviderInterface,
                                  smbprovider::kMountMethod);
     dbus::MessageWriter writer(&method_call);
-    writer.AppendProtoAsArrayOfBytes(options);
+    writer.AppendProtoAsArrayOfBytes(options_proto);
     writer.AppendFileDescriptor(password_fd.get());
     CallMethod(&method_call, &SmbProviderClientImpl::HandleMountCallback,
                &callback);
   }
 
-  void Remount(const base::FilePath& share_path,
-               int32_t mount_id,
-               const std::string& workgroup,
-               const std::string& username,
-               base::ScopedFD password_fd,
+  void Unmount(int32_t mount_id,
+               bool remove_password,
                StatusCallback callback) override {
-    smbprovider::RemountOptionsProto options;
-    options.set_path(share_path.value());
-    options.set_mount_id(mount_id);
-    options.set_workgroup(workgroup);
-    options.set_username(username);
-
-    dbus::MethodCall method_call(smbprovider::kSmbProviderInterface,
-                                 smbprovider::kRemountMethod);
-    dbus::MessageWriter writer(&method_call);
-    writer.AppendProtoAsArrayOfBytes(options);
-    writer.AppendFileDescriptor(password_fd.get());
-
-    CallDefaultMethod(&method_call, &callback);
-  }
-
-  void Unmount(int32_t mount_id, StatusCallback callback) override {
     smbprovider::UnmountOptionsProto options;
     options.set_mount_id(mount_id);
+    options.set_remove_password(remove_password);
     CallDefaultMethod(smbprovider::kUnmountMethod, options, &callback);
   }
 
@@ -318,6 +319,64 @@ class SmbProviderClientImpl : public SmbProviderClient {
     CallDefaultMethod(&method_call, &callback);
   }
 
+  void StartReadDirectory(int32_t mount_id,
+                          const base::FilePath& directory_path,
+                          StartReadDirectoryCallback callback) override {
+    smbprovider::ReadDirectoryOptionsProto options;
+    options.set_mount_id(mount_id);
+    options.set_directory_path(directory_path.value());
+    CallMethod(smbprovider::kStartReadDirectoryMethod, options,
+               &SmbProviderClientImpl::HandleStartReadDirectoryCallback,
+               &callback);
+  }
+
+  void ContinueReadDirectory(int32_t mount_id,
+                             int32_t read_dir_token,
+                             ReadDirectoryCallback callback) override {
+    dbus::MethodCall method_call(smbprovider::kSmbProviderInterface,
+                                 smbprovider::kContinueReadDirectoryMethod);
+    dbus::MessageWriter writer(&method_call);
+    writer.AppendInt32(mount_id);
+    writer.AppendInt32(read_dir_token);
+    CallMethod(&method_call,
+               &SmbProviderClientImpl::HandleContinueReadDirectoryCallback,
+               &callback);
+  }
+
+  void UpdateMountCredentials(int32_t mount_id,
+                              std::string workgroup,
+                              std::string username,
+                              base::ScopedFD password_fd,
+                              StatusCallback callback) override {
+    smbprovider::UpdateMountCredentialsOptionsProto options;
+    options.set_mount_id(mount_id);
+    options.set_workgroup(workgroup);
+    options.set_username(username);
+
+    dbus::MethodCall method_call(smbprovider::kSmbProviderInterface,
+                                 smbprovider::kUpdateMountCredentialsMethod);
+    dbus::MessageWriter writer(&method_call);
+    writer.AppendProtoAsArrayOfBytes(options);
+    writer.AppendFileDescriptor(password_fd.get());
+
+    CallDefaultMethod(&method_call, &callback);
+  }
+
+  void UpdateSharePath(int32_t mount_id,
+                       const std::string& share_path,
+                       StatusCallback callback) override {
+    smbprovider::UpdateSharePathOptionsProto options;
+    options.set_mount_id(mount_id);
+    options.set_path(share_path);
+
+    dbus::MethodCall method_call(smbprovider::kSmbProviderInterface,
+                                 smbprovider::kUpdateSharePathMethod);
+    dbus::MessageWriter writer(&method_call);
+    writer.AppendProtoAsArrayOfBytes(options);
+
+    CallDefaultMethod(&method_call, &callback);
+  }
+
  protected:
   // DBusClient override.
   void Init(dbus::Bus* bus) override {
@@ -553,6 +612,7 @@ class SmbProviderClientImpl : public SmbProviderClient {
       LOG(ERROR) << "StartCopy: parse failure.";
       std::move(callback).Run(smbprovider::ERROR_DBUS_PARSE_FAILED,
                               -1 /* copy_token*/);
+      return;
     }
 
     if (error != smbprovider::ERROR_COPY_PENDING) {
@@ -563,6 +623,58 @@ class SmbProviderClientImpl : public SmbProviderClient {
     std::move(callback).Run(smbprovider::ERROR_COPY_PENDING, copy_token);
   }
 
+  void HandleStartReadDirectoryCallback(StartReadDirectoryCallback callback,
+                                        dbus::Response* response) {
+    if (!response) {
+      LOG(ERROR) << "StartReadDirectory: failed to call smbprovider";
+      std::move(callback).Run(smbprovider::ERROR_DBUS_PARSE_FAILED,
+                              -1 /* read_dir_token */,
+                              smbprovider::DirectoryEntryListProto());
+      return;
+    }
+
+    dbus::MessageReader reader(response);
+
+    smbprovider::ErrorType error = GetErrorFromReader(&reader);
+
+    smbprovider::DirectoryEntryListProto entries;
+    int32_t read_dir_token;
+    if (!reader.PopArrayOfBytesAsProto(&entries) ||
+        !reader.PopInt32(&read_dir_token)) {
+      LOG(ERROR) << "StartReadDirectory: Failed to parse protobuf.";
+      std::move(callback).Run(smbprovider::ERROR_DBUS_PARSE_FAILED,
+                              -1 /* read_dir_token */,
+                              smbprovider::DirectoryEntryListProto());
+      return;
+    }
+
+    std::move(callback).Run(error, read_dir_token, entries);
+  }
+
+  void HandleContinueReadDirectoryCallback(ReadDirectoryCallback callback,
+                                           dbus::Response* response) {
+    if (!response) {
+      LOG(ERROR) << "ContinueReadDirectory: failed to call smbprovider";
+      std::move(callback).Run(smbprovider::ERROR_DBUS_PARSE_FAILED,
+                              smbprovider::DirectoryEntryListProto());
+      return;
+    }
+
+    dbus::MessageReader reader(response);
+
+    smbprovider::ErrorType error = GetErrorFromReader(&reader);
+
+    smbprovider::DirectoryEntryListProto entries;
+    if (!reader.PopArrayOfBytesAsProto(&entries)) {
+      LOG(ERROR) << "ContinueReadDirectory: Failed to parse protobuf.";
+      std::move(callback).Run(smbprovider::ERROR_DBUS_PARSE_FAILED,
+                              smbprovider::DirectoryEntryListProto());
+      return;
+    }
+
+    std::move(callback).Run(error, entries);
+  }
+
   // Default callback handler for D-Bus calls.
   void HandleDefaultCallback(const std::string& method_name,
                              StatusCallback callback,
@@ -598,13 +710,17 @@ class SmbProviderClientImpl : public SmbProviderClient {
 
 }  // namespace
 
+SmbProviderClient::MountOptions::MountOptions() = default;
+
+SmbProviderClient::MountOptions::~MountOptions() = default;
+
 SmbProviderClient::SmbProviderClient() = default;
 
 SmbProviderClient::~SmbProviderClient() = default;
 
 // static
-SmbProviderClient* SmbProviderClient::Create() {
-  return new SmbProviderClientImpl();
+std::unique_ptr<SmbProviderClient> SmbProviderClient::Create() {
+  return std::make_unique<SmbProviderClientImpl>();
 }
 
 }  // namespace chromeos
