@@ -5,11 +5,14 @@
 #ifndef CHROMEOS_DBUS_SMB_PROVIDER_CLIENT_H_
 #define CHROMEOS_DBUS_SMB_PROVIDER_CLIENT_H_
 
+#include <memory>
+#include <string>
+
 #include "base/callback.h"
+#include "base/component_export.h"
 #include "base/files/file_path.h"
 #include "base/files/scoped_file.h"
 #include "base/memory/weak_ptr.h"
-#include "chromeos/chromeos_export.h"
 #include "chromeos/dbus/dbus_client.h"
 #include "chromeos/dbus/smbprovider/directory_entry.pb.h"
 #include "dbus/object_proxy.h"
@@ -20,7 +23,7 @@ namespace chromeos {
 // SmbProviderClient is used to communicate with the org.chromium.SmbProvider
 // service. All methods should be called from the origin thread (UI thread)
 // which initializes the DBusThreadManager instance.
-class CHROMEOS_EXPORT SmbProviderClient
+class COMPONENT_EXPORT(CHROMEOS_DBUS) SmbProviderClient
     : public DBusClient,
       public base::SupportsWeakPtr<SmbProviderClient> {
  public:
@@ -46,35 +49,54 @@ class CHROMEOS_EXPORT SmbProviderClient
   using StartCopyCallback =
       base::OnceCallback<void(smbprovider::ErrorType error,
                               int32_t copy_token)>;
+  using StartReadDirectoryCallback = base::OnceCallback<void(
+      smbprovider::ErrorType error,
+      int32_t read_dir_token,
+      const smbprovider::DirectoryEntryListProto& entries)>;
+
+  // Optional arguments to pass to Mount().
+  struct MountOptions {
+    MountOptions();
+    ~MountOptions();
+
+    std::string original_path;
+    std::string username;
+    std::string workgroup;
+    std::string account_hash;
+
+    // Enable NTLM Authentication.
+    bool ntlm_enabled = false;
+
+    // Do not attempt to connect to and authenticate the mounted share.
+    bool skip_connect = false;
+
+    // Save the password for this share if it is successfully mounted.
+    bool save_password = false;
+
+    // Use a saved password for authenticating the share.
+    bool restore_password = false;
+  };
 
   ~SmbProviderClient() override;
 
   // Factory function, creates a new instance and returns ownership.
   // For normal usage, access the singleton via DBusThreadManager::Get().
-  static SmbProviderClient* Create();
+  static std::unique_ptr<SmbProviderClient> Create();
 
   // Calls Mount. It runs OpenDirectory() on |share_path| to check that it is a
-  // valid share. |workgroup|, |username|, and |password_fd| will be used as
-  // credentials to access the mount. |callback| is called after getting (or
-  // failing to get) D-BUS response.
+  // valid share. |options.workgroup|, |options.username|, and |password_fd|
+  // will be used as credentials to access the mount. |callback| is called after
+  // getting (or failing to get) D-BUS response.
   virtual void Mount(const base::FilePath& share_path,
-                     const std::string& workgroup,
-                     const std::string& username,
+                     const MountOptions& options,
                      base::ScopedFD password_fd,
                      MountCallback callback) = 0;
 
-  // Calls Remount. This attempts to remount the share at |share_path| with its
-  // original |mount_id|.
-  virtual void Remount(const base::FilePath& share_path,
-                       int32_t mount_id,
-                       const std::string& workgroup,
-                       const std::string& username,
-                       base::ScopedFD password_fd,
-                       StatusCallback callback) = 0;
-
   // Calls Unmount. This removes the corresponding mount of |mount_id| from
   // the list of valid mounts. Subsequent operations on |mount_id| will fail.
-  virtual void Unmount(int32_t mount_id, StatusCallback callback) = 0;
+  virtual void Unmount(int32_t mount_id,
+                       bool remove_password,
+                       StatusCallback callback) = 0;
 
   // Calls ReadDirectory. Using the corresponding mount of |mount_id|, this
   // reads the directory on a given |directory_path| and passes the
@@ -183,8 +205,9 @@ class CHROMEOS_EXPORT SmbProviderClient
                          ReadDirectoryCallback callback) = 0;
 
   // Calls SetupKerberos. This sets up Kerberos for the user |account_id|,
-  // fetching the user's Kerberos files from AuthPolicy. The user must be
-  // ChromAD enrolled.
+  // fetching the user's Kerberos files from AuthPolicy (if user is enrolled to
+  // ChromAD) or Kerberos. If user is enrolled to ChromAD, |account_id| is
+  // expected to be an object guid, otherwise it must be a principal name.
   virtual void SetupKerberos(const std::string& account_id,
                              SetupKerberosCallback callback) = 0;
 
@@ -212,6 +235,37 @@ class CHROMEOS_EXPORT SmbProviderClient
                             int32_t copy_token,
                             StatusCallback callback) = 0;
 
+  // Calls StartReadDirectory. This starts a read directory of |directory_path|.
+  // Returns smbprovider::ERROR_OPERATION_PENDING if there is more work to do.
+  virtual void StartReadDirectory(int32_t mount_id,
+                                  const base::FilePath& directory_path,
+                                  StartReadDirectoryCallback callback) = 0;
+
+  // Calls ContinueReadDirectory. This continues the copy corresponding to
+  // |read_dir_token|. Returns smbprovider::ERROR_OPERATION_PENDING if there is
+  // more work to do.
+  virtual void ContinueReadDirectory(int32_t mount_id,
+                                     int32_t read_dir_token,
+                                     ReadDirectoryCallback callback) = 0;
+
+  // Calls UpdateMountCredentials. This will update a mount's credentials with
+  // |workgroup|, |username|, and |password_fd|. Returns smbprovider::ERROR_OK
+  // if the mount's credentials successfully updated. Returns
+  // smbprovider::ERROR_NOT_FOUND if the mount's credentials were not updated.
+  virtual void UpdateMountCredentials(int32_t mount_id,
+                                      std::string workgroup,
+                                      std::string username,
+                                      base::ScopedFD password_fd,
+                                      StatusCallback callback) = 0;
+
+  // Calls UpdateSharePath. This will update a mount's share path with
+  // |share_path|. Returns smbprovider::ERROR_OK if the mount's share path was
+  // successfully updated. Returns smbprovider::ERROR_NOT_FOUND if the mount's
+  // share path were not updated.
+  virtual void UpdateSharePath(int32_t mount_id,
+                               const std::string& share_path,
+                               StatusCallback callback) = 0;
+
  protected:
   // Create() should be used instead.
   SmbProviderClient();
