@@ -13,12 +13,15 @@
 #include <utility>
 #include <vector>
 
+#include "ash/constants/ash_features.h"
 #include "base/barrier_closure.h"
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
+#include "base/logging.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/observer_list.h"
 #include "base/strings/string_util.h"
 #include "chromeos/dbus/cros_disks_client.h"
@@ -31,8 +34,6 @@ namespace disks {
 
 namespace {
 
-constexpr char kDefaultFormattedDeviceName[] = "UNTITLED";
-constexpr char kDefaultFormatVFAT[] = "vfat";
 constexpr char kDeviceNotFound[] = "Device could not be found";
 DiskMountManager* g_disk_mount_manager = NULL;
 
@@ -50,19 +51,30 @@ void OnAllUnmountDeviceRecursively(
   std::move(cb_data->callback).Run(cb_data->error_code);
 }
 
+std::string FormatFileSystemTypeToString(FormatFileSystemType filesystem) {
+  switch (filesystem) {
+    case FormatFileSystemType::kUnknown:
+      return "";
+    case FormatFileSystemType::kVfat:
+      return "vfat";
+    case FormatFileSystemType::kExfat:
+      return "exfat";
+    case FormatFileSystemType::kNtfs:
+      return "ntfs";
+  }
+  NOTREACHED() << "Unknown filesystem type " << static_cast<int>(filesystem);
+  return "";
+}
+
 // The DiskMountManager implementation.
 class DiskMountManagerImpl : public DiskMountManager,
                              public CrosDisksClient::Observer {
  public:
-  DiskMountManagerImpl() :
-    already_refreshed_(false),
-    weak_ptr_factory_(this) {
+  DiskMountManagerImpl() : already_refreshed_(false) {
     DBusThreadManager* dbus_thread_manager = DBusThreadManager::Get();
     cros_disks_client_ = dbus_thread_manager->GetCrosDisksClient();
-    PowerManagerClient* power_manager_client =
-        dbus_thread_manager->GetPowerManagerClient();
-    suspend_unmount_manager_.reset(
-        new SuspendUnmountManager(this, power_manager_client));
+    suspend_unmount_manager_.reset(new SuspendUnmountManager(this));
+
     cros_disks_client_->AddObserver(this);
   }
 
@@ -94,8 +106,11 @@ class DiskMountManagerImpl : public DiskMountManager,
         return;
       }
     }
+    std::vector<std::string> options = mount_options;
+    if (base::FeatureList::IsEnabled(chromeos::features::kFsNosymfollow))
+      options.push_back("nosymfollow");
     cros_disks_client_->Mount(
-        source_path, source_format, mount_label, mount_options, access_mode,
+        source_path, source_format, mount_label, options, access_mode,
         REMOUNT_OPTION_MOUNT_NEW_DEVICE,
         base::BindOnce(&DiskMountManagerImpl::OnMount,
                        weak_ptr_factory_.GetWeakPtr(), source_path, type));
@@ -108,14 +123,12 @@ class DiskMountManagerImpl : public DiskMountManager,
 
   // DiskMountManager override.
   void UnmountPath(const std::string& mount_path,
-                   UnmountOptions options,
                    UnmountPathCallback callback) override {
     UnmountChildMounts(mount_path);
     cros_disks_client_->Unmount(
-        mount_path, options,
-        base::BindOnce(&DiskMountManagerImpl::OnUnmountPath,
-                       weak_ptr_factory_.GetWeakPtr(), std::move(callback),
-                       mount_path));
+        mount_path, base::BindOnce(&DiskMountManagerImpl::OnUnmountPath,
+                                   weak_ptr_factory_.GetWeakPtr(),
+                                   std::move(callback), mount_path));
   }
 
   void RemountAllRemovableDrives(MountAccessMode mode) override {
@@ -134,15 +147,23 @@ class DiskMountManagerImpl : public DiskMountManager,
   }
 
   // DiskMountManager override.
-  void FormatMountedDevice(const std::string& mount_path) override {
+  void FormatMountedDevice(const std::string& mount_path,
+                           FormatFileSystemType filesystem,
+                           const std::string& label) override {
     MountPointMap::const_iterator mount_point = mount_points_.find(mount_path);
     if (mount_point == mount_points_.end()) {
       LOG(ERROR) << "Mount point with path \"" << mount_path << "\" not found.";
-      OnFormatCompleted(FORMAT_ERROR_UNKNOWN, mount_path);
+      // We can't call OnFormatCompleted until |pending_format_changes_| has
+      // been populated.
+      NotifyFormatStatusUpdate(FORMAT_COMPLETED, FORMAT_ERROR_UNKNOWN,
+                               mount_path, label);
       return;
     }
 
     std::string device_path = mount_point->second.source_path;
+    const std::string filesystem_str = FormatFileSystemTypeToString(filesystem);
+    pending_format_changes_[device_path] = {filesystem_str, label};
+
     DiskMap::const_iterator disk = disks_.find(device_path);
     if (disk == disks_.end()) {
       LOG(ERROR) << "Device with path \"" << device_path << "\" not found.";
@@ -150,15 +171,40 @@ class DiskMountManagerImpl : public DiskMountManager,
       return;
     }
     if (disk->second->is_read_only()) {
-      LOG(ERROR) << "Mount point with path \"" << mount_path
-                 << "\" is read-only.";
-      OnFormatCompleted(FORMAT_ERROR_DEVICE_NOT_ALLOWED, mount_path);
+      LOG(ERROR) << "Device with path \"" << device_path << "\" is read-only.";
+      OnFormatCompleted(FORMAT_ERROR_DEVICE_NOT_ALLOWED, device_path);
       return;
     }
 
-    UnmountPath(disk->second->mount_path(), UNMOUNT_OPTIONS_NONE,
+    if (filesystem == FormatFileSystemType::kUnknown) {
+      LOG(ERROR) << "Unknown filesystem passed to FormatMountedDevice";
+      OnFormatCompleted(FORMAT_ERROR_UNSUPPORTED_FILESYSTEM, device_path);
+      return;
+    }
+
+    UnmountPath(disk->second->mount_path(),
                 base::BindOnce(&DiskMountManagerImpl::OnUnmountPathForFormat,
-                               weak_ptr_factory_.GetWeakPtr(), device_path));
+                               weak_ptr_factory_.GetWeakPtr(), device_path,
+                               filesystem, label));
+  }
+
+  // DiskMountManager override.
+  void SinglePartitionFormatDevice(const std::string& device_path,
+                                   FormatFileSystemType filesystem,
+                                   const std::string& label) override {
+    DiskMap::const_iterator disk_iter = disks_.find(device_path);
+    if (disk_iter == disks_.end()) {
+      LOG(ERROR) << "Device with path \"" << device_path << "\" not found.";
+      OnPartitionCompleted(device_path, filesystem, label,
+                           PARTITION_ERROR_INVALID_DEVICE_PATH);
+      return;
+    }
+
+    UnmountDeviceRecursively(
+        device_path,
+        base::BindOnce(
+            &DiskMountManagerImpl::OnUnmountDeviceForSinglePartitionFormat,
+            weak_ptr_factory_.GetWeakPtr(), device_path, filesystem, label));
   }
 
   void RenameMountedDevice(const std::string& mount_path,
@@ -166,11 +212,16 @@ class DiskMountManagerImpl : public DiskMountManager,
     MountPointMap::const_iterator mount_point = mount_points_.find(mount_path);
     if (mount_point == mount_points_.end()) {
       LOG(ERROR) << "Mount point with path '" << mount_path << "' not found.";
-      OnRenameCompleted(RENAME_ERROR_UNKNOWN, mount_path);
+      // We can't call OnRenameCompleted until |pending_rename_changes_| has
+      // been populated.
+      NotifyRenameStatusUpdate(RENAME_COMPLETED, RENAME_ERROR_UNKNOWN,
+                               mount_path, volume_name);
       return;
     }
 
     std::string device_path = mount_point->second.source_path;
+    pending_rename_changes_[device_path] = volume_name;
+
     DiskMap::const_iterator iter = disks_.find(device_path);
     if (iter == disks_.end()) {
       LOG(ERROR) << "Device with path '" << device_path << "' not found.";
@@ -178,13 +229,12 @@ class DiskMountManagerImpl : public DiskMountManager,
       return;
     }
     if (iter->second->is_read_only()) {
-      LOG(ERROR) << "Mount point with path '" << mount_path
-                 << "' is read-only.";
-      OnRenameCompleted(RENAME_ERROR_DEVICE_NOT_ALLOWED, mount_path);
+      LOG(ERROR) << "Device with path '" << device_path << "' is read-only.";
+      OnRenameCompleted(RENAME_ERROR_DEVICE_NOT_ALLOWED, device_path);
       return;
     }
 
-    UnmountPath(iter->second->mount_path(), UNMOUNT_OPTIONS_NONE,
+    UnmountPath(iter->second->mount_path(),
                 base::BindOnce(&DiskMountManagerImpl::OnUnmountPathForRename,
                                weak_ptr_factory_.GetWeakPtr(), device_path,
                                volume_name));
@@ -230,7 +280,7 @@ class DiskMountManagerImpl : public DiskMountManager,
 
     for (size_t i = 0; i < devices_to_unmount.size(); ++i) {
       cros_disks_client_->Unmount(
-          devices_to_unmount[i], UNMOUNT_OPTIONS_NONE,
+          devices_to_unmount[i],
           base::BindOnce(&DiskMountManagerImpl::OnUnmountDeviceRecursively,
                          weak_ptr_factory_.GetWeakPtr(), raw_cb_data,
                          devices_to_unmount[i], done_callback));
@@ -311,6 +361,10 @@ class DiskMountManagerImpl : public DiskMountManager,
   // |disks_|. The key is a device_path and the value is a FormatChange.
   std::map<std::string, FormatChange> pending_format_changes_;
 
+  // Stores device path are being partitioning.
+  // It allows preventing auto-mount of the disks in this set.
+  std::set<std::string> pending_partitioning_disks_;
+
   // Stores new volume name for a device on which renaming is invoked on, so
   // that OnRenameCompleted can set it back to |disks_|. The key is a
   // device_path and the value is new volume_name.
@@ -370,9 +424,7 @@ class DiskMountManagerImpl : public DiskMountManager,
       if (base::StartsWith(it->second.source_path, mount_path,
                            base::CompareCase::SENSITIVE)) {
         // TODO(tbarzic): Handle the case where this fails.
-        UnmountPath(it->second.mount_path,
-                    UNMOUNT_OPTIONS_NONE,
-                    UnmountPathCallback());
+        UnmountPath(it->second.mount_path, UnmountPathCallback());
       }
     }
   }
@@ -382,6 +434,12 @@ class DiskMountManagerImpl : public DiskMountManager,
                                   const std::string& mount_path,
                                   base::OnceClosure done_callback,
                                   MountError error_code) {
+    if (error_code == MOUNT_ERROR_PATH_NOT_MOUNTED ||
+        error_code == MOUNT_ERROR_INVALID_PATH) {
+      // The path was already unmounted by something else.
+      error_code = MOUNT_ERROR_NONE;
+    }
+
     if (error_code == MOUNT_ERROR_NONE) {
       // Do standard processing for Unmount event.
       OnUnmountPath(UnmountPathCallback(), mount_path, MOUNT_ERROR_NONE);
@@ -398,6 +456,12 @@ class DiskMountManagerImpl : public DiskMountManager,
 
   // CrosDisksClient::Observer override.
   void OnMountCompleted(const MountEntry& entry) override {
+    auto iter = deferred_mount_events_.find(entry.source_path());
+    if (iter != deferred_mount_events_.end()) {
+      iter->second.push_back(entry);
+      return;
+    }
+
     MountCondition mount_condition = MOUNT_CONDITION_NONE;
     if (entry.mount_type() == MOUNT_TYPE_DEVICE) {
       if (entry.error_code() == MOUNT_ERROR_UNKNOWN_FILESYSTEM) {
@@ -421,6 +485,7 @@ class DiskMountManagerImpl : public DiskMountManager,
                                                      mount_info));
     }
 
+    Disk* disk = nullptr;
     if ((entry.error_code() == MOUNT_ERROR_NONE ||
          mount_info.mount_condition) &&
         mount_info.mount_type == MOUNT_TYPE_DEVICE &&
@@ -428,7 +493,7 @@ class DiskMountManagerImpl : public DiskMountManager,
         !mount_info.mount_path.empty()) {
       DiskMap::iterator iter = disks_.find(mount_info.source_path);
       if (iter != disks_.end()) {  // disk might have been removed by now?
-        Disk* disk = iter->second.get();
+        disk = iter->second.get();
         DCHECK(disk);
         // Currently the MountCompleted signal doesn't tell whether the device
         // is mounted in read-only mode or not. Instead use the mount option
@@ -456,6 +521,9 @@ class DiskMountManagerImpl : public DiskMountManager,
     // Observers may read the values of disks_. So notify them after tweaking
     // values of disks_.
     NotifyMountStatusUpdate(MOUNTING, entry.error_code(), mount_info);
+    if (disk) {
+      disk->set_is_first_mount(false);
+    }
   }
 
   // Callback for UnmountPath.
@@ -471,6 +539,12 @@ class DiskMountManagerImpl : public DiskMountManager,
       return;
     }
 
+    if (error_code == MOUNT_ERROR_PATH_NOT_MOUNTED ||
+        error_code == MOUNT_ERROR_INVALID_PATH) {
+      // The path was already unmounted by something else.
+      error_code = MOUNT_ERROR_NONE;
+    }
+
     NotifyMountStatusUpdate(
         UNMOUNTING, error_code,
         MountPointInfo(mount_points_it->second.source_path,
@@ -479,6 +553,7 @@ class DiskMountManagerImpl : public DiskMountManager,
                        mount_points_it->second.mount_condition));
 
     std::string path(mount_points_it->second.source_path);
+
     if (error_code == MOUNT_ERROR_NONE)
       mount_points_.erase(mount_points_it);
 
@@ -496,42 +571,69 @@ class DiskMountManagerImpl : public DiskMountManager,
   }
 
   void OnUnmountPathForFormat(const std::string& device_path,
+                              FormatFileSystemType filesystem,
+                              const std::string& label,
                               MountError error_code) {
     if (error_code == MOUNT_ERROR_NONE &&
         disks_.find(device_path) != disks_.end()) {
-      FormatUnmountedDevice(device_path);
+      FormatUnmountedDevice(device_path, filesystem, label);
     } else {
       OnFormatCompleted(FORMAT_ERROR_UNKNOWN, device_path);
     }
   }
 
+  void OnUnmountDeviceForSinglePartitionFormat(const std::string& device_path,
+                                               FormatFileSystemType filesystem,
+                                               const std::string& label,
+                                               MountError error_code) {
+    if (error_code != MOUNT_ERROR_NONE ||
+        disks_.find(device_path) == disks_.end()) {
+      OnPartitionCompleted(device_path, filesystem, label,
+                           PARTITION_ERROR_UNKNOWN);
+      return;
+    }
+
+    SinglePartitionFormatUnmountedDevice(device_path, filesystem, label);
+  }
+
   // Starts device formatting.
-  void FormatUnmountedDevice(const std::string& device_path) {
+  void FormatUnmountedDevice(const std::string& device_path,
+                             FormatFileSystemType filesystem,
+                             const std::string& label) {
     DiskMap::const_iterator disk = disks_.find(device_path);
     DCHECK(disk != disks_.end() && disk->second->mount_path().empty());
 
-    pending_format_changes_[device_path] = {kDefaultFormatVFAT,
-                                            kDefaultFormattedDeviceName};
+    base::UmaHistogramEnumeration("FileBrowser.FormatFileSystemType",
+                                  filesystem);
 
     cros_disks_client_->Format(
-        device_path, kDefaultFormatVFAT,
+        device_path, FormatFileSystemTypeToString(filesystem), label,
         base::BindOnce(&DiskMountManagerImpl::OnFormatStarted,
-                       weak_ptr_factory_.GetWeakPtr(), device_path));
+                       weak_ptr_factory_.GetWeakPtr(), device_path, label));
   }
 
   // Callback for Format.
-  void OnFormatStarted(const std::string& device_path, bool success) {
+  void OnFormatStarted(const std::string& device_path,
+                       const std::string& device_label,
+                       bool success) {
     if (!success) {
       OnFormatCompleted(FORMAT_ERROR_UNKNOWN, device_path);
       return;
     }
 
-    NotifyFormatStatusUpdate(FORMAT_STARTED, FORMAT_ERROR_NONE, device_path);
+    NotifyFormatStatusUpdate(FORMAT_STARTED, FORMAT_ERROR_NONE, device_path,
+                             device_label);
   }
 
   // CrosDisksClient::Observer override.
   void OnFormatCompleted(FormatError error_code,
                          const std::string& device_path) override {
+    std::string device_label;
+    auto pending_change = pending_format_changes_.find(device_path);
+    if (pending_change != pending_format_changes_.end()) {
+      device_label = pending_change->second.volume_name;
+    }
+
     auto iter = disks_.find(device_path);
 
     // disk might have been removed by now?
@@ -539,7 +641,6 @@ class DiskMountManagerImpl : public DiskMountManager,
       Disk* disk = iter->second.get();
       DCHECK(disk);
 
-      auto pending_change = pending_format_changes_.find(device_path);
       if (pending_change != pending_format_changes_.end() &&
           error_code == FORMAT_ERROR_NONE) {
         disk->set_device_label(pending_change->second.volume_name);
@@ -549,7 +650,108 @@ class DiskMountManagerImpl : public DiskMountManager,
 
     pending_format_changes_.erase(device_path);
 
-    NotifyFormatStatusUpdate(FORMAT_COMPLETED, error_code, device_path);
+    EnsureMountInfoRefreshed(base::DoNothing(), true /* force */);
+
+    NotifyFormatStatusUpdate(FORMAT_COMPLETED, error_code, device_path,
+                             device_label);
+  }
+
+  void SinglePartitionFormatUnmountedDevice(const std::string& device_path,
+                                            FormatFileSystemType filesystem,
+                                            const std::string& label) {
+    DiskMap::const_iterator disk = disks_.find(device_path);
+    DCHECK(disk != disks_.end() && disk->second->mount_path().empty());
+
+    pending_partitioning_disks_.insert(disk->second->device_path());
+
+    NotifyPartitionStatusUpdate(PARTITION_STARTED, PARTITION_ERROR_NONE,
+                                device_path, label);
+
+    cros_disks_client_->SinglePartitionFormat(
+        disk->second->file_path(),
+        base::BindOnce(&DiskMountManagerImpl::OnPartitionCompleted,
+                       weak_ptr_factory_.GetWeakPtr(), device_path, filesystem,
+                       label));
+  }
+
+  void OnPartitionCompleted(const std::string& device_path,
+                            FormatFileSystemType filesystem,
+                            const std::string& label,
+                            PartitionError error_code) {
+    auto iter = disks_.find(device_path);
+
+    // disk might have been removed by now?
+    if (iter != disks_.end()) {
+      Disk* disk = iter->second.get();
+      DCHECK(disk);
+
+      if (error_code == PARTITION_ERROR_NONE) {
+        EnsureMountInfoRefreshed(
+            base::BindOnce(&DiskMountManagerImpl::OnRefreshAfterPartition,
+                           weak_ptr_factory_.GetWeakPtr(), device_path,
+                           filesystem, label),
+            true /* force */);
+      }
+
+    } else {
+      // Remove disk from pending partitioning list if disk removed.
+      pending_partitioning_disks_.erase(iter->second->device_path());
+    }
+
+    NotifyPartitionStatusUpdate(PARTITION_COMPLETED, error_code, device_path,
+                                label);
+  }
+
+  void OnRefreshAfterPartition(const std::string& device_path,
+                               FormatFileSystemType filesystem,
+                               const std::string& label,
+                               bool success) {
+    DiskMap::const_iterator device_disk = disks_.find(device_path);
+    if (device_disk == disks_.end()) {
+      LOG(ERROR) << "Device not found, maybe ejected";
+      pending_partitioning_disks_.erase(device_path);
+      NotifyPartitionStatusUpdate(PARTITION_COMPLETED,
+                                  PARTITION_ERROR_INVALID_DEVICE_PATH,
+                                  device_path, label);
+      return;
+    }
+
+    std::string new_partition_device_path;
+    // Find new partition using common storage path with parent device.
+    for (DiskMountManager::DiskMap::const_iterator it = disks_.begin();
+         it != disks_.end(); ++it) {
+      if (it->second->storage_device_path() ==
+              device_disk->second->storage_device_path() &&
+          !it->second->is_parent()) {
+        new_partition_device_path = it->second->device_path();
+        break;
+      }
+    }
+
+    if (new_partition_device_path.empty()) {
+      LOG(ERROR) << "New partition couldn't be found";
+      pending_partitioning_disks_.erase(device_path);
+      NotifyPartitionStatusUpdate(PARTITION_COMPLETED,
+                                  PARTITION_ERROR_INVALID_DEVICE_PATH,
+                                  device_path, label);
+      return;
+    }
+
+    const std::string filesystem_str = FormatFileSystemTypeToString(filesystem);
+    pending_format_changes_[new_partition_device_path] = {filesystem_str,
+                                                          label};
+
+    // It's expected the disks (parent device and new partition) are not
+    // mounted, but try unmounting before starting format if it got
+    // mounted through another flow.
+    UnmountDeviceRecursively(
+        device_path,
+        base::BindOnce(&DiskMountManagerImpl::OnUnmountPathForFormat,
+                       weak_ptr_factory_.GetWeakPtr(),
+                       new_partition_device_path, filesystem, label));
+
+    // It's ok to remove it from pending partitioning as format flow started.
+    pending_partitioning_disks_.erase(device_path);
   }
 
   void OnUnmountPathForRename(const std::string& device_path,
@@ -570,26 +772,35 @@ class DiskMountManagerImpl : public DiskMountManager,
     DiskMap::const_iterator disk = disks_.find(device_path);
     DCHECK(disk != disks_.end() && disk->second->mount_path().empty());
 
-    pending_rename_changes_[device_path] = volume_name;
     cros_disks_client_->Rename(
         device_path, volume_name,
         base::BindOnce(&DiskMountManagerImpl::OnRenameStarted,
-                       weak_ptr_factory_.GetWeakPtr(), device_path));
+                       weak_ptr_factory_.GetWeakPtr(), device_path,
+                       volume_name));
   }
 
   // Callback for Rename.
-  void OnRenameStarted(const std::string& device_path, bool success) {
+  void OnRenameStarted(const std::string& device_path,
+                       const std::string& volume_name,
+                       bool success) {
     if (!success) {
       OnRenameCompleted(RENAME_ERROR_UNKNOWN, device_path);
       return;
     }
 
-    NotifyRenameStatusUpdate(RENAME_STARTED, RENAME_ERROR_NONE, device_path);
+    NotifyRenameStatusUpdate(RENAME_STARTED, RENAME_ERROR_NONE, device_path,
+                             volume_name);
   }
 
   // CrosDisksClient::Observer override.
   void OnRenameCompleted(RenameError error_code,
                          const std::string& device_path) override {
+    std::string device_label;
+    auto pending_change = pending_rename_changes_.find(device_path);
+    if (pending_change != pending_rename_changes_.end()) {
+      device_label = pending_change->second;
+    }
+
     auto iter = disks_.find(device_path);
 
     // disk might have been removed by now?
@@ -597,7 +808,6 @@ class DiskMountManagerImpl : public DiskMountManager,
       Disk* disk = iter->second.get();
       DCHECK(disk);
 
-      auto pending_change = pending_rename_changes_.find(device_path);
       if (pending_change != pending_rename_changes_.end() &&
           error_code == RENAME_ERROR_NONE)
         disk->set_device_label(pending_change->second);
@@ -605,20 +815,37 @@ class DiskMountManagerImpl : public DiskMountManager,
 
     pending_rename_changes_.erase(device_path);
 
-    NotifyRenameStatusUpdate(RENAME_COMPLETED, error_code, device_path);
+    NotifyRenameStatusUpdate(RENAME_COMPLETED, error_code, device_path,
+                             device_label);
+  }
+
+  // Fire observer mount events that were deferred due to an in-progress
+  // GetDeviceProperties() call.
+  void RunDeferredMountEvents(const std::string& device_path) {
+    auto mount_events_iter = deferred_mount_events_.find(device_path);
+    if (mount_events_iter == deferred_mount_events_.end())
+      return;
+    std::vector<MountEntry> entries = std::move(mount_events_iter->second);
+    deferred_mount_events_.erase(mount_events_iter);
+    for (const MountEntry& entry : entries)
+      OnMountCompleted(entry);
   }
 
   // Callback for GetDeviceProperties.
   void OnGetDeviceProperties(const DiskInfo& disk_info) {
-    if (disk_info.is_virtual())
+    if (disk_info.is_virtual()) {
+      RunDeferredMountEvents(disk_info.device_path());
       return;
+    }
 
     DVLOG(1) << "Found disk " << disk_info.device_path();
     // Delete previous disk info for this path:
     bool is_new = true;
+    bool is_first_mount = false;
     std::string base_mount_path = std::string();
     DiskMap::iterator iter = disks_.find(disk_info.device_path());
     if (iter != disks_.end()) {
+      is_first_mount = iter->second->is_first_mount();
       base_mount_path = iter->second->base_mount_path();
       disks_.erase(iter);
       is_new = false;
@@ -632,11 +859,14 @@ class DiskMountManagerImpl : public DiskMountManager,
     bool write_disabled_by_policy = access_mode != access_modes_.end()
         && access_mode->second == chromeos::MOUNT_ACCESS_MODE_READ_ONLY;
     Disk* disk = new Disk(disk_info, write_disabled_by_policy,
-                          FindSystemPathPrefix(disk_info.system_path()),
                           base_mount_path);
+    if (!is_new) {
+      disk->set_is_first_mount(is_first_mount);
+    }
     disks_.insert(
         std::make_pair(disk_info.device_path(), base::WrapUnique(disk)));
     NotifyDiskStatusUpdate(is_new ? DISK_ADDED : DISK_CHANGED, *disk);
+    RunDeferredMountEvents(disk_info.device_path());
   }
 
   // Part of EnsureMountInfoRefreshed(). Called after the list of devices are
@@ -706,6 +936,9 @@ class DiskMountManagerImpl : public DiskMountManager,
     std::string device_path = device_path_arg;
     switch (event) {
       case CROS_DISKS_DISK_ADDED: {
+        // Ensure we have an entry indicating we're waiting for
+        // GetDeviceProperties() to complete.
+        deferred_mount_events_[device_path];
         cros_disks_client_->GetDeviceProperties(
             device_path,
             base::BindOnce(&DiskMountManagerImpl::OnGetDeviceProperties,
@@ -724,12 +957,10 @@ class DiskMountManagerImpl : public DiskMountManager,
         break;
       }
       case CROS_DISKS_DEVICE_ADDED: {
-        system_path_prefixes_.insert(device_path);
         NotifyDeviceStatusUpdate(DEVICE_ADDED, device_path);
         break;
       }
       case CROS_DISKS_DEVICE_REMOVED: {
-        system_path_prefixes_.erase(device_path);
         NotifyDeviceStatusUpdate(DEVICE_REMOVED, device_path);
         break;
       }
@@ -746,6 +977,10 @@ class DiskMountManagerImpl : public DiskMountManager,
   // Notifies all observers about disk status update.
   void NotifyDiskStatusUpdate(DiskEvent event, const Disk& disk) {
     for (auto& observer : observers_) {
+      // Skip mounting of new partitioned disks while waiting for the format.
+      if (IsPendingPartitioningDisk(disk.device_path())) {
+        continue;
+      }
       disk.is_auto_mountable() ? observer.OnAutoMountableDiskEvent(event, disk)
                                : observer.OnBootDeviceDiskEvent(event, disk);
     }
@@ -768,34 +1003,46 @@ class DiskMountManagerImpl : public DiskMountManager,
 
   void NotifyFormatStatusUpdate(FormatEvent event,
                                 FormatError error_code,
-                                const std::string& device_path) {
+                                const std::string& device_path,
+                                const std::string& device_label) {
     for (auto& observer : observers_)
-      observer.OnFormatEvent(event, error_code, device_path);
+      observer.OnFormatEvent(event, error_code, device_path, device_label);
+  }
+
+  void NotifyPartitionStatusUpdate(PartitionEvent event,
+                                   PartitionError error_code,
+                                   const std::string& device_path,
+                                   const std::string& device_label) {
+    for (auto& observer : observers_)
+      observer.OnPartitionEvent(event, error_code, device_path, device_label);
   }
 
   void NotifyRenameStatusUpdate(RenameEvent event,
                                 RenameError error_code,
-                                const std::string& device_path) {
+                                const std::string& device_path,
+                                const std::string& device_label) {
     for (auto& observer : observers_)
-      observer.OnRenameEvent(event, error_code, device_path);
+      observer.OnRenameEvent(event, error_code, device_path, device_label);
   }
 
-  // Finds system path prefix from |system_path|.
-  const std::string& FindSystemPathPrefix(const std::string& system_path) {
-    if (system_path.empty())
-      return base::EmptyString();
-    for (SystemPathPrefixSet::const_iterator it = system_path_prefixes_.begin();
-         it != system_path_prefixes_.end();
-         ++it) {
-      const std::string& prefix = *it;
-      if (base::StartsWith(system_path, prefix, base::CompareCase::SENSITIVE))
-        return prefix;
+  bool IsPendingPartitioningDisk(const std::string& device_path) {
+    if (pending_partitioning_disks_.find(device_path) !=
+        pending_partitioning_disks_.end()) {
+      return true;
     }
-    return base::EmptyString();
+
+    // If device path doesn't match check whether if it's a child path.
+    for (auto it = pending_partitioning_disks_.begin();
+         it != pending_partitioning_disks_.end(); ++it) {
+      if (base::StartsWith(device_path, *it, base::CompareCase::SENSITIVE)) {
+        return true;
+      }
+    }
+    return false;
   }
 
   // Mount event change observers.
-  base::ObserverList<DiskMountManager::Observer>::Unchecked observers_;
+  base::ObserverList<DiskMountManager::Observer> observers_;
 
   CrosDisksClient* cros_disks_client_;
 
@@ -804,8 +1051,11 @@ class DiskMountManagerImpl : public DiskMountManager,
 
   DiskMountManager::MountPointMap mount_points_;
 
-  typedef std::set<std::string> SystemPathPrefixSet;
-  SystemPathPrefixSet system_path_prefixes_;
+  // A map entry with a key of the device path will be created upon calling
+  // GetDeviceProperties(), for deferring mount events, and removed once it has
+  // completed. This prevents a race resulting in mount events being fired with
+  // the corresponding Disk entry unexpectedly missing.
+  std::map<std::string, std::vector<MountEntry>> deferred_mount_events_;
 
   bool already_refreshed_;
   std::vector<EnsureMountInfoRefreshedCallback> refresh_callbacks_;
@@ -817,13 +1067,17 @@ class DiskMountManagerImpl : public DiskMountManager,
   typedef std::map<std::string, chromeos::MountAccessMode> AccessModeMap;
   AccessModeMap access_modes_;
 
-  base::WeakPtrFactory<DiskMountManagerImpl> weak_ptr_factory_;
+  base::WeakPtrFactory<DiskMountManagerImpl> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(DiskMountManagerImpl);
 };
 
 }  // namespace
 
+DiskMountManager::Observer::~Observer() {
+  DCHECK(!IsInObserverList());
+}
+
 bool DiskMountManager::AddDiskForTest(std::unique_ptr<Disk> disk) {
   return false;
 }
