@@ -11,6 +11,7 @@
 #include <string>
 #include <utility>
 
+#include "base/bind.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
 #include "base/metrics/histogram_functions.h"
@@ -25,7 +26,6 @@
 #include "net/base/escape.h"
 #include "net/base/load_flags.h"
 #include "net/http/http_status_code.h"
-#include "net/url_request/url_request_status.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "services/network/public/cpp/simple_url_loader.h"
 
@@ -71,7 +71,7 @@ constexpr char kAccuracyString[] = "accuracy";
 
 // Error object and its contents.
 constexpr char kErrorString[] = "error";
-// "errors" array in "erorr" object is ignored.
+// "errors" array in "error" object is ignored.
 constexpr char kCodeString[] = "code";
 constexpr char kMessageString[] = "message";
 
@@ -192,24 +192,24 @@ bool ParseServerResponse(const GURL& server_url,
              "Parsing response '" << response_body << "'";
 
   // Parse the response, ignoring comments.
-  std::string error_msg;
-  std::unique_ptr<base::Value> response_value =
-      base::JSONReader::ReadAndReturnError(response_body, base::JSON_PARSE_RFC,
-                                           NULL, &error_msg);
-  if (response_value == NULL) {
-    PrintGeolocationError(
-        server_url, "JSONReader failed: " + error_msg, position);
+  auto response_result =
+      base::JSONReader::ReadAndReturnValueWithError(response_body);
+  if (!response_result.value) {
+    PrintGeolocationError(server_url,
+                          "JSONReader failed: " + response_result.error_message,
+                          position);
     RecordUmaEvent(SIMPLE_GEOLOCATION_REQUEST_EVENT_RESPONSE_MALFORMED);
     return false;
   }
+  base::Value response_value = std::move(*response_result.value);
 
   base::DictionaryValue* response_object = NULL;
-  if (!response_value->GetAsDictionary(&response_object)) {
+  if (!response_value.GetAsDictionary(&response_object)) {
     PrintGeolocationError(
         server_url,
         "Unexpected response type : " +
             base::StringPrintf(
-                "%u", static_cast<unsigned int>(response_value->type())),
+                "%u", static_cast<unsigned int>(response_value.type())),
         position);
     RecordUmaEvent(SIMPLE_GEOLOCATION_REQUEST_EVENT_RESPONSE_MALFORMED);
     return false;
@@ -288,7 +288,7 @@ bool GetGeolocationFromResponse(bool http_success,
   }
   if (status_code != net::HTTP_OK) {
     std::string message = "Returned error code ";
-    message += base::IntToString(status_code);
+    message += base::NumberToString(status_code);
     PrintGeolocationError(server_url, message, position);
     RecordUmaEvent(SIMPLE_GEOLOCATION_REQUEST_EVENT_RESPONSE_NOT_OK);
     return false;
@@ -316,7 +316,7 @@ std::unique_ptr<base::DictionaryValue> CreateAccessPointDictionary(
   if (!access_point.timestamp.is_null()) {
     access_point_dictionary->SetKey(
         kAge,
-        base::Value(base::Int64ToString(
+        base::Value(base::NumberToString(
             (base::Time::Now() - access_point.timestamp).InMilliseconds())));
   }
 
@@ -340,7 +340,7 @@ std::unique_ptr<base::DictionaryValue> CreateCellTowerDictionary(
   if (!cell_tower.timestamp.is_null()) {
     cell_tower_dictionary->SetKey(
         kAge,
-        base::Value(base::Int64ToString(
+        base::Value(base::NumberToString(
             (base::Time::Now() - cell_tower.timestamp).InMilliseconds())));
   }
   return cell_tower_dictionary;
@@ -369,7 +369,7 @@ SimpleGeolocationRequest::~SimpleGeolocationRequest() {
   DCHECK(thread_checker_.CalledOnValidThread());
 
   // If callback is not empty, request is cancelled.
-  if (!callback_.is_null()) {
+  if (callback_) {
     RecordUmaResponseTime(base::Time::Now() - request_started_at_, false);
     RecordUmaResult(SIMPLE_GEOLOCATION_REQUEST_RESULT_CANCELLED, retries_);
   }
@@ -441,7 +441,7 @@ void SimpleGeolocationRequest::StartRequest() {
   request->url = request_url_;
   request->method = "POST";
   request->load_flags = net::LOAD_BYPASS_CACHE | net::LOAD_DISABLE_CACHE;
-  request->allow_credentials = false;
+  request->credentials_mode = network::mojom::CredentialsMode::kOmit;
 
   simple_url_loader_ = network::SimpleURLLoader::Create(
       std::move(request), NO_TRAFFIC_ANNOTATION_YET);
@@ -457,8 +457,8 @@ void SimpleGeolocationRequest::StartRequest() {
                      base::Unretained(this)));
 }
 
-void SimpleGeolocationRequest::MakeRequest(const ResponseCallback& callback) {
-  callback_ = callback;
+void SimpleGeolocationRequest::MakeRequest(ResponseCallback callback) {
+  callback_ = std::move(callback);
   request_url_ = GeolocationRequestURL(service_url_);
   timeout_timer_.Start(
       FROM_HERE, timeout_, this, &SimpleGeolocationRequest::OnTimeout);
@@ -529,14 +529,14 @@ void SimpleGeolocationRequest::ReplyAndDestroySelf(
   timeout_timer_.Stop();
   request_scheduled_.Stop();
 
-  ResponseCallback callback = callback_;
+  ResponseCallback callback = std::move(callback_);
 
   // Empty callback is used to identify "completed or not yet started request".
   callback_.Reset();
 
   // callback.Run() usually destroys SimpleGeolocationRequest, because this is
   // the way callback is implemented in GeolocationProvider.
-  callback.Run(position_, server_error, elapsed);
+  std::move(callback).Run(position_, server_error, elapsed);
   // "this" is already destroyed here.
 }
 
