@@ -4,20 +4,20 @@
 
 #include <stddef.h>
 
+#include "base/bind.h"
 #include "base/macros.h"
-#include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_manager_client.h"
+#include "base/test/task_environment.h"
+#include "chromeos/dbus/shill/shill_clients.h"
+#include "chromeos/dbus/shill/shill_manager_client.h"
 #include "chromeos/geolocation/simple_geolocation_provider.h"
 #include "chromeos/geolocation/simple_geolocation_request_test_monitor.h"
 #include "chromeos/network/geolocation_handler.h"
 #include "chromeos/network/network_handler.h"
 #include "net/http/http_response_headers.h"
 #include "net/http/http_status_code.h"
-#include "net/url_request/url_request_status.h"
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
 #include "services/network/test/test_url_loader_factory.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -114,12 +114,12 @@ class TestGeolocationAPILoaderFactory : public network::TestURLLoaderFactory {
 
  private:
   void AddResponseWithCode(int error_code) {
-    network::ResourceResponseHead response_head;
-    response_head.headers = base::MakeRefCounted<net::HttpResponseHeaders>("");
-    response_head.headers->AddHeader("Content-Type: application/json");
+    auto response_head = network::mojom::URLResponseHead::New();
+    response_head->headers = base::MakeRefCounted<net::HttpResponseHeaders>("");
+    response_head->headers->SetHeader("Content-Type", "application/json");
     // If AddResponse() is called multiple times for the same URL, the last
     // one is the one used so there is no need for ClearResponses().
-    AddResponse(url_, response_head, response_,
+    AddResponse(url_, std::move(response_head), response_,
                 network::URLLoaderCompletionStatus(error_code));
   }
 
@@ -181,7 +181,7 @@ class WirelessTestMonitor : public SimpleGeolocationRequestTestMonitor {
 
 class SimpleGeolocationTest : public testing::Test {
  private:
-  base::MessageLoop message_loop_;
+  base::test::SingleThreadTaskEnvironment task_environment_;
 };
 
 TEST_F(SimpleGeolocationTest, ResponseOK) {
@@ -195,9 +195,10 @@ TEST_F(SimpleGeolocationTest, ResponseOK) {
   url_factory.SetSimpleGeolocationProvider(&provider);
 
   GeolocationReceiver receiver;
-  provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), false, false,
-                              base::Bind(&GeolocationReceiver::OnRequestDone,
-                                         base::Unretained(&receiver)));
+  provider.RequestGeolocation(
+      base::TimeDelta::FromSeconds(1), false, false,
+      base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                     base::Unretained(&receiver)));
   receiver.WaitUntilRequestDone();
 
   EXPECT_EQ(kExpectedPosition, receiver.position().ToString());
@@ -217,9 +218,10 @@ TEST_F(SimpleGeolocationTest, ResponseOKWithRetries) {
   url_factory.SetSimpleGeolocationProvider(&provider);
 
   GeolocationReceiver receiver;
-  provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), false, false,
-                              base::Bind(&GeolocationReceiver::OnRequestDone,
-                                         base::Unretained(&receiver)));
+  provider.RequestGeolocation(
+      base::TimeDelta::FromSeconds(1), false, false,
+      base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                     base::Unretained(&receiver)));
   receiver.WaitUntilRequestDone();
   EXPECT_EQ(kExpectedPosition, receiver.position().ToString());
   EXPECT_FALSE(receiver.server_error());
@@ -243,18 +245,20 @@ TEST_F(SimpleGeolocationTest, InvalidResponse) {
       timeout_seconds * 1000 / kRequestRetryIntervalMilliSeconds);
   ASSERT_GE(expected_retries, 2U);
 
-  provider.RequestGeolocation(base::TimeDelta::FromSeconds(timeout_seconds),
-                              false, false,
-                              base::Bind(&GeolocationReceiver::OnRequestDone,
-                                         base::Unretained(&receiver)));
+  provider.RequestGeolocation(
+      base::TimeDelta::FromSeconds(timeout_seconds), false, false,
+      base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                     base::Unretained(&receiver)));
   receiver.WaitUntilRequestDone();
 
-  EXPECT_EQ(
-      "latitude=200.000000, longitude=200.000000, accuracy=-1.000000, "
-      "error_code=0, error_message='SimpleGeolocation provider at "
-      "'https://localhost/' : JSONReader failed: Line: 1, column: 1, "
-      "Unexpected token..', status=4 (TIMEOUT)",
-      receiver.position().ToString());
+  std::string receiver_position = receiver.position().ToString();
+  EXPECT_NE(
+      std::string::npos,
+      receiver_position.find(
+          "latitude=200.000000, longitude=200.000000, accuracy=-1.000000, "
+          "error_code=0, error_message='SimpleGeolocation provider at "
+          "'https://localhost/' : JSONReader failed:"));
+  EXPECT_NE(std::string::npos, receiver_position.find("status=4 (TIMEOUT)"));
   EXPECT_TRUE(receiver.server_error());
   EXPECT_GE(url_factory.attempts(), 2U);
   if (url_factory.attempts() > expected_retries + 1) {
@@ -272,8 +276,7 @@ TEST_F(SimpleGeolocationTest, InvalidResponse) {
 }
 
 TEST_F(SimpleGeolocationTest, NoWiFi) {
-  // This initializes DBusThreadManager and markes it "for tests only".
-  DBusThreadManager::GetSetterForTesting();
+  shill_clients::InitializeFakes();
   NetworkHandler::Initialize();
 
   WirelessTestMonitor requests_monitor;
@@ -289,9 +292,10 @@ TEST_F(SimpleGeolocationTest, NoWiFi) {
   url_factory.SetSimpleGeolocationProvider(&provider);
 
   GeolocationReceiver receiver;
-  provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), true, false,
-                              base::Bind(&GeolocationReceiver::OnRequestDone,
-                                         base::Unretained(&receiver)));
+  provider.RequestGeolocation(
+      base::TimeDelta::FromSeconds(1), true, false,
+      base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                     base::Unretained(&receiver)));
   receiver.WaitUntilRequestDone();
   EXPECT_EQ(kIPOnlyRequestBody, requests_monitor.last_request_body());
 
@@ -300,7 +304,7 @@ TEST_F(SimpleGeolocationTest, NoWiFi) {
   EXPECT_EQ(1U, url_factory.attempts());
 
   NetworkHandler::Shutdown();
-  DBusThreadManager::Shutdown();
+  shill_clients::Shutdown();
 }
 
 // Test sending of WiFi Access points and Cell Towers.
@@ -312,11 +316,9 @@ class SimpleGeolocationWirelessTest : public ::testing::TestWithParam<bool> {
   ~SimpleGeolocationWirelessTest() override = default;
 
   void SetUp() override {
-    // This initializes DBusThreadManager and markes it "for tests only".
-    DBusThreadManager::GetSetterForTesting();
+    shill_clients::InitializeFakes();
     // Get the test interface for manager / device.
-    manager_test_ =
-        DBusThreadManager::Get()->GetShillManagerClient()->GetTestInterface();
+    manager_test_ = ShillManagerClient::Get()->GetTestInterface();
     ASSERT_TRUE(manager_test_);
     geolocation_handler_.reset(new GeolocationHandler());
     geolocation_handler_->Init();
@@ -325,7 +327,7 @@ class SimpleGeolocationWirelessTest : public ::testing::TestWithParam<bool> {
 
   void TearDown() override {
     geolocation_handler_.reset();
-    DBusThreadManager::Shutdown();
+    shill_clients::Shutdown();
   }
 
   bool GetWifiAccessPoints() {
@@ -342,8 +344,8 @@ class SimpleGeolocationWirelessTest : public ::testing::TestWithParam<bool> {
     base::DictionaryValue properties;
     std::string mac_address =
         base::StringPrintf("%02X:%02X:%02X:%02X:%02X:%02X", idx, 0, 0, 0, 0, 0);
-    std::string channel = base::IntToString(idx);
-    std::string strength = base::IntToString(idx * 10);
+    std::string channel = base::NumberToString(idx);
+    std::string strength = base::NumberToString(idx * 10);
     properties.SetKey(shill::kGeoMacAddressProperty, base::Value(mac_address));
     properties.SetKey(shill::kGeoChannelProperty, base::Value(channel));
     properties.SetKey(shill::kGeoSignalStrengthProperty, base::Value(strength));
@@ -355,10 +357,10 @@ class SimpleGeolocationWirelessTest : public ::testing::TestWithParam<bool> {
   // This should remain in sync with the format of shill (chromeos) dict entries
   void AddCellTower(int idx) {
     base::DictionaryValue properties;
-    std::string ci = base::IntToString(idx);
-    std::string lac = base::IntToString(idx * 3);
-    std::string mcc = base::IntToString(idx * 100);
-    std::string mnc = base::IntToString(idx * 100 + 1);
+    std::string ci = base::NumberToString(idx);
+    std::string lac = base::NumberToString(idx * 3);
+    std::string mcc = base::NumberToString(idx * 100);
+    std::string mnc = base::NumberToString(idx * 100 + 1);
 
     properties.SetKey(shill::kGeoCellIdProperty, base::Value(ci));
     properties.SetKey(shill::kGeoLocationAreaCodeProperty, base::Value(lac));
@@ -370,7 +372,8 @@ class SimpleGeolocationWirelessTest : public ::testing::TestWithParam<bool> {
   }
 
  protected:
-  base::MessageLoopForUI message_loop_;
+  base::test::SingleThreadTaskEnvironment task_environment_{
+      base::test::SingleThreadTaskEnvironment::MainThreadType::UI};
   std::unique_ptr<GeolocationHandler> geolocation_handler_;
   ShillManagerClient::TestInterface* manager_test_;
   WifiAccessPointVector wifi_access_points_;
@@ -395,12 +398,13 @@ TEST_P(SimpleGeolocationWirelessTest, WiFiExists) {
           &url_factory),
       GURL(kTestGeolocationProviderUrl));
   url_factory.SetSimpleGeolocationProvider(&provider);
+  provider.set_geolocation_handler(geolocation_handler_.get());
   {
     GeolocationReceiver receiver;
-    provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), GetParam(),
-                                false,
-                                base::Bind(&GeolocationReceiver::OnRequestDone,
-                                           base::Unretained(&receiver)));
+    provider.RequestGeolocation(
+        base::TimeDelta::FromSeconds(1), GetParam(), false,
+        base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                       base::Unretained(&receiver)));
     receiver.WaitUntilRequestDone();
     EXPECT_EQ(kIPOnlyRequestBody, requests_monitor.last_request_body());
 
@@ -424,10 +428,10 @@ TEST_P(SimpleGeolocationWirelessTest, WiFiExists) {
 
   {
     GeolocationReceiver receiver;
-    provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), GetParam(),
-                                false,
-                                base::Bind(&GeolocationReceiver::OnRequestDone,
-                                           base::Unretained(&receiver)));
+    provider.RequestGeolocation(
+        base::TimeDelta::FromSeconds(1), GetParam(), false,
+        base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                       base::Unretained(&receiver)));
     receiver.WaitUntilRequestDone();
     if (GetParam()) {
       // Sending WiFi data is enabled.
@@ -446,9 +450,9 @@ TEST_P(SimpleGeolocationWirelessTest, WiFiExists) {
 }
 
 // This test verifies that WiFi data is sent only if sending was requested.
-INSTANTIATE_TEST_CASE_P(EnableDisableSendingWifiData,
-                        SimpleGeolocationWirelessTest,
-                        testing::Bool());
+INSTANTIATE_TEST_SUITE_P(EnableDisableSendingWifiData,
+                         SimpleGeolocationWirelessTest,
+                         testing::Bool());
 
 TEST_P(SimpleGeolocationWirelessTest, CellularExists) {
   NetworkHandler::Initialize();
@@ -464,12 +468,13 @@ TEST_P(SimpleGeolocationWirelessTest, CellularExists) {
           &url_factory),
       GURL(kTestGeolocationProviderUrl));
   url_factory.SetSimpleGeolocationProvider(&provider);
+  provider.set_geolocation_handler(geolocation_handler_.get());
   {
     GeolocationReceiver receiver;
-    provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), false,
-                                GetParam(),
-                                base::Bind(&GeolocationReceiver::OnRequestDone,
-                                           base::Unretained(&receiver)));
+    provider.RequestGeolocation(
+        base::TimeDelta::FromSeconds(1), false, GetParam(),
+        base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                       base::Unretained(&receiver)));
     receiver.WaitUntilRequestDone();
     EXPECT_EQ(kIPOnlyRequestBody, requests_monitor.last_request_body());
 
@@ -487,14 +492,14 @@ TEST_P(SimpleGeolocationWirelessTest, CellularExists) {
   EXPECT_TRUE(GetCellTowers());
   ASSERT_EQ(1u, cell_towers_.size());
   EXPECT_EQ(kCellTower1MNC, cell_towers_[0].mnc);
-  EXPECT_EQ(base::IntToString(1), cell_towers_[0].ci);
+  EXPECT_EQ(base::NumberToString(1), cell_towers_[0].ci);
 
   {
     GeolocationReceiver receiver;
-    provider.RequestGeolocation(base::TimeDelta::FromSeconds(1), false,
-                                GetParam(),
-                                base::Bind(&GeolocationReceiver::OnRequestDone,
-                                           base::Unretained(&receiver)));
+    provider.RequestGeolocation(
+        base::TimeDelta::FromSeconds(1), false, GetParam(),
+        base::BindOnce(&GeolocationReceiver::OnRequestDone,
+                       base::Unretained(&receiver)));
     receiver.WaitUntilRequestDone();
     if (GetParam()) {
       // Sending Cellular data is enabled.
