@@ -6,13 +6,16 @@
 //
 // Library support to remap process executable elf segment with hugepages.
 //
-// ReloadElfTextInHugePages() will search for an ELF executable segment,
+// InitHugepagesAndMlockSelf() will search for an ELF executable segment,
 // and remap it using hugepage.
 
 #ifndef CHROMEOS_HUGEPAGE_TEXT_HUGEPAGE_TEXT_H_
 #define CHROMEOS_HUGEPAGE_TEXT_HUGEPAGE_TEXT_H_
 
 #include <string>
+
+#include "base/feature_list.h"
+#include "base/metrics/field_trial_params.h"
 #include "chromeos/chromeos_export.h"
 
 #if defined(__clang__) || defined(__GNUC__)
@@ -23,13 +26,20 @@
 
 namespace chromeos {
 
-// This function will scan ELF
-// segments and attempt to remap text segment from small page to hugepage.
+// A feature which controls remapping the zygotes hot text section as hugepages
+// and locking.
+extern const base::Feature kCrOSHugepageRemapAndLockZygote;
+
+// This function will scan ELF segments and attempt to do two things:
+// - Reload some of .text into hugepages
+// - Lock some of .text into memory, so it can't be swapped out.
+//
 // When this function returns, text segments that are naturally aligned on
-// hugepage size will be backed by hugepages.  In the event of errors, the
-// remapping operation will be aborted and rolled back, e.g. they are all
-// soft fail.
-CHROMEOS_EXPORT extern void ReloadElfTextInHugePages(void);
+// hugepage size will be backed by hugepages.
+//
+// Any and all errors encountered by this function are soft errors; Chrome
+// should still be able to run.
+CHROMEOS_EXPORT extern void InitHugepagesAndMlockSelf(void);
 }  // namespace chromeos
 
 #endif  // CHROMEOS_HUGEPAGE_TEXT_HUGEPAGE_TEXT_H_
