@@ -3,15 +3,18 @@
 // found in the LICENSE file.
 
 #include "chromeos/login/auth/auth_status_consumer.h"
+#include "base/notreached.h"
 
 namespace chromeos {
 
-void AuthStatusConsumer::OnRetailModeAuthSuccess(
+void AuthStatusConsumer::OnPasswordChangeDetected(
     const UserContext& user_context) {
-  OnAuthSuccess(user_context);
+  NOTREACHED();
 }
 
-void AuthStatusConsumer::OnPasswordChangeDetected() {
+void AuthStatusConsumer::OnOldEncryptionDetected(
+    const UserContext& user_context,
+    bool has_incomplete_migration) {
   NOTREACHED();
 }
 
