@@ -7,8 +7,10 @@
 
 #include <string>
 
+#include "base/component_export.h"
 #include "base/logging.h"
-#include "chromeos/chromeos_export.h"
+#include "base/notreached.h"
+#include "base/observer_list_types.h"
 #include "google_apis/gaia/gaia_auth_consumer.h"
 #include "google_apis/gaia/google_service_auth_error.h"
 #include "net/base/net_errors.h"
@@ -17,10 +19,10 @@ namespace chromeos {
 
 class UserContext;
 
-class CHROMEOS_EXPORT AuthFailure {
+class COMPONENT_EXPORT(CHROMEOS_LOGIN_AUTH) AuthFailure {
  public:
   // Enum used for UMA. Do NOT reorder or remove entry. Don't forget to
-  // update histograms.xml when adding new entries.
+  // update LoginFailureReason enum in enums.xml when adding new entries.
   enum FailureReason {
     NONE = 0,
     COULD_NOT_MOUNT_CRYPTOHOME = 1,
@@ -31,7 +33,7 @@ class CHROMEOS_EXPORT AuthFailure {
     UNLOCK_FAILED = 6,
     NETWORK_AUTH_FAILED = 7,     // Could not authenticate against Google
     OWNER_REQUIRED = 8,          // Only the device owner can log-in.
-    WHITELIST_CHECK_FAILED = 9,  // Login attempt blocked by whitelist. This
+    ALLOWLIST_CHECK_FAILED = 9,  // Login attempt blocked by allowlist. This
     // value is synthesized by the ExistingUserController and passed to the
     // login_status_consumer_ in tests only. It is never generated or seen by
     // any of the other authenticator classes.
@@ -39,6 +41,9 @@ class CHROMEOS_EXPORT AuthFailure {
     USERNAME_HASH_FAILED = 11,        // Could not get username hash.
     FAILED_TO_INITIALIZE_TOKEN = 12,  // Could not get OAuth2 Token,
     MISSING_CRYPTOHOME = 13,          // cryptohome missing from disk.
+    AUTH_DISABLED = 14,               // Authentication disabled for user.
+    TPM_UPDATE_REQUIRED = 15,         // TPM firmware update is required.
+    UNRECOVERABLE_CRYPTOHOME = 16,    // cryptohome is corrupted.
     NUM_FAILURE_REASONS,              // This has to be the last item.
   };
 
@@ -85,12 +90,20 @@ class CHROMEOS_EXPORT AuthFailure {
         return "Google authentication failed.";
       case OWNER_REQUIRED:
         return "Login is restricted to the owner's account only.";
-      case WHITELIST_CHECK_FAILED:
-        return "Login attempt blocked by whitelist.";
+      case ALLOWLIST_CHECK_FAILED:
+        return "Login attempt blocked by allowlist.";
       case FAILED_TO_INITIALIZE_TOKEN:
         return "OAuth2 token fetch failed.";
       case MISSING_CRYPTOHOME:
         return "Cryptohome missing from disk.";
+      case AUTH_DISABLED:
+        return "Auth disabled for user.";
+      case TPM_ERROR:
+        return "Critical TPM error encountered.";
+      case TPM_UPDATE_REQUIRED:
+        return "TPM firmware update required.";
+      case UNRECOVERABLE_CRYPTOHOME:
+        return "Cryptohome is corrupted.";
       default:
         NOTREACHED();
         return std::string();
@@ -119,9 +132,10 @@ enum SuccessReason {
 // An interface that defines the callbacks for objects that the
 // Authenticator class will call to report the success/failure of
 // authentication for Chromium OS.
-class CHROMEOS_EXPORT AuthStatusConsumer {
+class COMPONENT_EXPORT(CHROMEOS_LOGIN_AUTH) AuthStatusConsumer
+    : public base::CheckedObserver {
  public:
-  virtual ~AuthStatusConsumer() {}
+  ~AuthStatusConsumer() override = default;
   // The current login attempt has ended in failure, with error |error|.
   virtual void OnAuthFailure(const AuthFailure& error) = 0;
 
@@ -130,7 +144,7 @@ class CHROMEOS_EXPORT AuthStatusConsumer {
   // The current guest login attempt has succeeded.
   virtual void OnOffTheRecordAuthSuccess() {}
   // The same password didn't work both online and offline.
-  virtual void OnPasswordChangeDetected();
+  virtual void OnPasswordChangeDetected(const UserContext& user_context);
   // The cryptohome is encrypted in old format and needs migration.
   virtual void OnOldEncryptionDetected(const UserContext& user_context,
                                        bool has_incomplete_migration);
@@ -138,4 +152,10 @@ class CHROMEOS_EXPORT AuthStatusConsumer {
 
 }  // namespace chromeos
 
+// TODO(https://crbug.com/1164001): remove after //chrome/browser/chromeos
+// source migration is finished.
+namespace ash {
+using ::chromeos::AuthFailure;
+}
+
 #endif  // CHROMEOS_LOGIN_AUTH_AUTH_STATUS_CONSUMER_H_
