@@ -8,16 +8,10 @@
 
 namespace chromeos {
 
-// static
-scoped_refptr<ExtendedAuthenticator> ExtendedAuthenticator::Create(
-    NewAuthStatusConsumer* consumer) {
-  return base::MakeRefCounted<ExtendedAuthenticatorImpl>(consumer);
-}
-
 // static
 scoped_refptr<ExtendedAuthenticator> ExtendedAuthenticator::Create(
       AuthStatusConsumer* consumer) {
-  return base::MakeRefCounted<ExtendedAuthenticatorImpl>(consumer);
+  return ExtendedAuthenticatorImpl::Create(consumer);
 }
 
 ExtendedAuthenticator::ExtendedAuthenticator() = default;
