@@ -9,17 +9,16 @@
 #include "base/bind.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
-#include "chromeos/cryptohome/async_method_caller.h"
 #include "chromeos/cryptohome/cryptohome_parameters.h"
 #include "chromeos/cryptohome/cryptohome_util.h"
 #include "chromeos/cryptohome/homedir_methods.h"
 #include "chromeos/cryptohome/system_salt_getter.h"
-#include "chromeos/dbus/cryptohome_client.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
+#include "chromeos/dbus/cryptohome/cryptohome_client.h"
 #include "chromeos/login/auth/auth_status_consumer.h"
+#include "chromeos/login/auth/cryptohome_parameter_utils.h"
 #include "chromeos/login/auth/key.h"
+#include "chromeos/login/auth/login_event_recorder.h"
 #include "chromeos/login/auth/user_context.h"
-#include "chromeos/login_event_recorder.h"
 #include "components/account_id/account_id.h"
 #include "crypto/sha2.h"
 #include "google_apis/gaia/gaia_auth_util.h"
@@ -44,98 +43,121 @@ void RecordEndMarker(const std::string& marker) {
 
 }  // namespace
 
-ExtendedAuthenticatorImpl::ExtendedAuthenticatorImpl(
-    NewAuthStatusConsumer* consumer)
-    : salt_obtained_(false), consumer_(consumer), old_consumer_(NULL) {
-  SystemSaltGetter::Get()->GetSystemSalt(
-      base::Bind(&ExtendedAuthenticatorImpl::OnSaltObtained, this));
+// static
+scoped_refptr<ExtendedAuthenticatorImpl> ExtendedAuthenticatorImpl::Create(
+    AuthStatusConsumer* consumer) {
+  auto extended_authenticator =
+      base::WrapRefCounted(new ExtendedAuthenticatorImpl(consumer));
+  SystemSaltGetter::Get()->GetSystemSalt(base::BindOnce(
+      &ExtendedAuthenticatorImpl::OnSaltObtained, extended_authenticator));
+  return extended_authenticator;
 }
 
 ExtendedAuthenticatorImpl::ExtendedAuthenticatorImpl(
     AuthStatusConsumer* consumer)
-    : salt_obtained_(false), consumer_(NULL), old_consumer_(consumer) {
-  SystemSaltGetter::Get()->GetSystemSalt(
-      base::Bind(&ExtendedAuthenticatorImpl::OnSaltObtained, this));
-}
+    : salt_obtained_(false), consumer_(consumer) {}
 
 void ExtendedAuthenticatorImpl::SetConsumer(AuthStatusConsumer* consumer) {
-  old_consumer_ = consumer;
-}
-
-void ExtendedAuthenticatorImpl::AuthenticateToMount(
-    const UserContext& context,
-    const ResultCallback& success_callback) {
-  TransformKeyIfNeeded(
-      context,
-      base::Bind(&ExtendedAuthenticatorImpl::DoAuthenticateToMount,
-                 this,
-                 success_callback));
+  consumer_ = consumer;
 }
 
 void ExtendedAuthenticatorImpl::AuthenticateToCheck(
     const UserContext& context,
-    const base::Closure& success_callback) {
+    base::OnceClosure success_callback) {
   TransformKeyIfNeeded(
-      context,
-      base::Bind(&ExtendedAuthenticatorImpl::DoAuthenticateToCheck,
-                 this,
-                 success_callback));
+      context, base::BindOnce(&ExtendedAuthenticatorImpl::DoAuthenticateToCheck,
+                              this, std::move(success_callback)));
+}
+
+void ExtendedAuthenticatorImpl::StartFingerprintAuthSession(
+    const AccountId& account_id,
+    base::OnceCallback<void(bool)> callback) {
+  CryptohomeClient::Get()->StartFingerprintAuthSession(
+      cryptohome::CreateAccountIdentifierFromAccountId(account_id),
+      cryptohome::StartFingerprintAuthSessionRequest(),
+      base::BindOnce(
+          &ExtendedAuthenticatorImpl::OnStartFingerprintAuthSessionComplete,
+          this, std::move(callback)));
+}
+
+void ExtendedAuthenticatorImpl::OnStartFingerprintAuthSessionComplete(
+    base::OnceCallback<void(bool)> callback,
+    base::Optional<cryptohome::BaseReply> reply) {
+  std::move(callback).Run(reply && !reply->has_error());
+}
+
+void ExtendedAuthenticatorImpl::EndFingerprintAuthSession() {
+  CryptohomeClient::Get()->EndFingerprintAuthSession(
+      cryptohome::EndFingerprintAuthSessionRequest(),
+      base::BindOnce([](base::Optional<cryptohome::BaseReply> reply) {
+        // Only check for existence of the reply, because if there is a reply,
+        // it's always a BaseReply without errors.
+        if (!reply)
+          LOG(ERROR) << "EndFingerprintAuthSession call had no reply.";
+      }));
+}
+
+void ExtendedAuthenticatorImpl::AuthenticateWithFingerprint(
+    const UserContext& context,
+    base::OnceCallback<void(cryptohome::CryptohomeErrorCode)> callback) {
+  cryptohome::KeyDefinition key_def;
+  key_def.type = cryptohome::KeyDefinition::TYPE_FINGERPRINT;
+  CryptohomeClient::Get()->CheckKeyEx(
+      cryptohome::CreateAccountIdentifierFromAccountId(context.GetAccountId()),
+      cryptohome::CreateAuthorizationRequestFromKeyDef(key_def),
+      cryptohome::CheckKeyRequest(),
+      base::BindOnce(&ExtendedAuthenticatorImpl::OnFingerprintScanComplete,
+                     this, std::move(callback)));
+}
+
+void ExtendedAuthenticatorImpl::OnFingerprintScanComplete(
+    base::OnceCallback<void(cryptohome::CryptohomeErrorCode)> callback,
+    base::Optional<cryptohome::BaseReply> reply) {
+  if (!reply) {
+    std::move(callback).Run(cryptohome::CryptohomeErrorCode::
+                                CRYPTOHOME_ERROR_FINGERPRINT_ERROR_INTERNAL);
+    return;
+  }
+
+  std::move(callback).Run(reply->error());
 }
 
 void ExtendedAuthenticatorImpl::AddKey(const UserContext& context,
                                        const cryptohome::KeyDefinition& key,
                                        bool clobber_if_exists,
-                                       const base::Closure& success_callback) {
+                                       base::OnceClosure success_callback) {
   TransformKeyIfNeeded(
-      context, base::Bind(&ExtendedAuthenticatorImpl::DoAddKey, this, key,
-                          clobber_if_exists, success_callback));
-}
-
-void ExtendedAuthenticatorImpl::UpdateKeyAuthorized(
-    const UserContext& context,
-    const cryptohome::KeyDefinition& key,
-    const std::string& signature,
-    const base::Closure& success_callback) {
-  TransformKeyIfNeeded(
-      context,
-      base::Bind(&ExtendedAuthenticatorImpl::DoUpdateKeyAuthorized,
-                 this,
-                 key,
-                 signature,
-                 success_callback));
+      context, base::BindOnce(&ExtendedAuthenticatorImpl::DoAddKey, this, key,
+                              clobber_if_exists, std::move(success_callback)));
 }
 
 void ExtendedAuthenticatorImpl::RemoveKey(const UserContext& context,
-                                      const std::string& key_to_remove,
-                                      const base::Closure& success_callback) {
-  TransformKeyIfNeeded(context,
-                       base::Bind(&ExtendedAuthenticatorImpl::DoRemoveKey,
-                                  this,
-                                  key_to_remove,
-                                  success_callback));
+                                          const std::string& key_to_remove,
+                                          base::OnceClosure success_callback) {
+  TransformKeyIfNeeded(
+      context, base::BindOnce(&ExtendedAuthenticatorImpl::DoRemoveKey, this,
+                              key_to_remove, std::move(success_callback)));
 }
 
 void ExtendedAuthenticatorImpl::TransformKeyIfNeeded(
     const UserContext& user_context,
-    const ContextCallback& callback) {
+    ContextCallback callback) {
   if (user_context.GetKey()->GetKeyType() != Key::KEY_TYPE_PASSWORD_PLAIN) {
-    callback.Run(user_context);
+    std::move(callback).Run(user_context);
     return;
   }
 
   if (!salt_obtained_) {
     system_salt_callbacks_.push_back(
-        base::Bind(&ExtendedAuthenticatorImpl::TransformKeyIfNeeded,
-                   this,
-                   user_context,
-                   callback));
+        base::BindOnce(&ExtendedAuthenticatorImpl::TransformKeyIfNeeded, this,
+                       user_context, std::move(callback)));
     return;
   }
 
   UserContext transformed_context = user_context;
   transformed_context.GetKey()->Transform(Key::KEY_TYPE_SALTED_SHA256_TOP_HALF,
                                           system_salt_);
-  callback.Run(transformed_context);
+  std::move(callback).Run(transformed_context);
 }
 
 ExtendedAuthenticatorImpl::~ExtendedAuthenticatorImpl() = default;
@@ -143,44 +165,28 @@ ExtendedAuthenticatorImpl::~ExtendedAuthenticatorImpl() = default;
 void ExtendedAuthenticatorImpl::OnSaltObtained(const std::string& system_salt) {
   salt_obtained_ = true;
   system_salt_ = system_salt;
-  for (std::vector<base::Closure>::const_iterator it =
-           system_salt_callbacks_.begin();
-       it != system_salt_callbacks_.end();
-       ++it) {
-    it->Run();
-  }
+  for (auto& callback : system_salt_callbacks_)
+    std::move(callback).Run();
   system_salt_callbacks_.clear();
 }
 
-void ExtendedAuthenticatorImpl::DoAuthenticateToMount(
-    const ResultCallback& success_callback,
-    const UserContext& user_context) {
-  RecordStartMarker("MountEx");
-  const Key* const key = user_context.GetKey();
-  DBusThreadManager::Get()->GetCryptohomeClient()->MountEx(
-      cryptohome::Identification(user_context.GetAccountId()),
-      cryptohome::CreateAuthorizationRequest(key->GetLabel(), key->GetSecret()),
-      cryptohome::MountRequest(),
-      base::BindOnce(&ExtendedAuthenticatorImpl::OnMountComplete, this,
-                     "MountEx", user_context, success_callback));
-}
-
 void ExtendedAuthenticatorImpl::DoAuthenticateToCheck(
-    const base::Closure& success_callback,
+    base::OnceClosure success_callback,
     const UserContext& user_context) {
   RecordStartMarker("CheckKeyEx");
-  const Key* const key = user_context.GetKey();
   cryptohome::HomedirMethods::GetInstance()->CheckKeyEx(
       cryptohome::Identification(user_context.GetAccountId()),
-      cryptohome::CreateAuthorizationRequest(key->GetLabel(), key->GetSecret()),
+      cryptohome::CreateAuthorizationRequestFromKeyDef(
+          cryptohome_parameter_utils::CreateAuthorizationKeyDefFromUserContext(
+              user_context)),
       cryptohome::CheckKeyRequest(),
-      base::Bind(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
-                 "CheckKeyEx", user_context, success_callback));
+      base::BindOnce(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
+                     "CheckKeyEx", user_context, std::move(success_callback)));
 }
 
 void ExtendedAuthenticatorImpl::DoAddKey(const cryptohome::KeyDefinition& key,
                                          bool clobber_if_exists,
-                                         const base::Closure& success_callback,
+                                         base::OnceClosure success_callback,
                                          const UserContext& user_context) {
   RecordStartMarker("AddKeyEx");
 
@@ -193,33 +199,13 @@ void ExtendedAuthenticatorImpl::DoAddKey(const cryptohome::KeyDefinition& key,
       cryptohome::CreateAuthorizationRequest(auth_key->GetLabel(),
                                              auth_key->GetSecret()),
       request,
-      base::Bind(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
-                 "AddKeyEx", user_context, success_callback));
-}
-
-void ExtendedAuthenticatorImpl::DoUpdateKeyAuthorized(
-    const cryptohome::KeyDefinition& key,
-    const std::string& signature,
-    const base::Closure& success_callback,
-    const UserContext& user_context) {
-  RecordStartMarker("UpdateKeyAuthorized");
-
-  const Key* const auth_key = user_context.GetKey();
-  cryptohome::UpdateKeyRequest request;
-  cryptohome::KeyDefinitionToKey(key, request.mutable_changes());
-  request.set_authorization_signature(signature);
-  cryptohome::HomedirMethods::GetInstance()->UpdateKeyEx(
-      cryptohome::Identification(user_context.GetAccountId()),
-      cryptohome::CreateAuthorizationRequest(auth_key->GetLabel(),
-                                             auth_key->GetSecret()),
-      request,
-      base::Bind(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
-                 "UpdateKeyAuthorized", user_context, success_callback));
+      base::BindOnce(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
+                     "AddKeyEx", user_context, std::move(success_callback)));
 }
 
 void ExtendedAuthenticatorImpl::DoRemoveKey(const std::string& key_to_remove,
-                                        const base::Closure& success_callback,
-                                        const UserContext& user_context) {
+                                            base::OnceClosure success_callback,
+                                            const UserContext& user_context) {
   RecordStartMarker("RemoveKeyEx");
 
   cryptohome::RemoveKeyRequest request;
@@ -230,65 +216,27 @@ void ExtendedAuthenticatorImpl::DoRemoveKey(const std::string& key_to_remove,
       cryptohome::CreateAuthorizationRequest(auth_key->GetLabel(),
                                              auth_key->GetSecret()),
       request,
-      base::Bind(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
-                 "RemoveKeyEx", user_context, success_callback));
-}
-
-void ExtendedAuthenticatorImpl::OnMountComplete(
-    const std::string& time_marker,
-    const UserContext& user_context,
-    const ResultCallback& success_callback,
-    base::Optional<cryptohome::BaseReply> reply) {
-  cryptohome::MountError return_code =
-      cryptohome::MountExReplyToMountError(reply);
-  RecordEndMarker(time_marker);
-  if (return_code == cryptohome::MOUNT_ERROR_NONE) {
-    const std::string& mount_hash =
-        cryptohome::MountExReplyToMountHash(reply.value());
-    if (!success_callback.is_null())
-      success_callback.Run(mount_hash);
-    if (old_consumer_) {
-      UserContext copy = user_context;
-      copy.SetUserIDHash(mount_hash);
-      old_consumer_->OnAuthSuccess(copy);
-    }
-    return;
-  }
-  LOG(ERROR) << "MountEx failed. Error: " << return_code;
-  AuthState state = FAILED_MOUNT;
-  if (return_code == cryptohome::MOUNT_ERROR_TPM_COMM_ERROR ||
-      return_code == cryptohome::MOUNT_ERROR_TPM_DEFEND_LOCK ||
-      return_code == cryptohome::MOUNT_ERROR_TPM_NEEDS_REBOOT) {
-    state = FAILED_TPM;
-  }
-  if (return_code == cryptohome::MOUNT_ERROR_USER_DOES_NOT_EXIST)
-    state = NO_MOUNT;
-
-  if (consumer_)
-    consumer_->OnAuthenticationFailure(state);
-
-  if (old_consumer_) {
-    AuthFailure failure(AuthFailure::COULD_NOT_MOUNT_CRYPTOHOME);
-    old_consumer_->OnAuthFailure(failure);
-  }
+      base::BindOnce(&ExtendedAuthenticatorImpl::OnOperationComplete, this,
+                     "RemoveKeyEx", user_context, std::move(success_callback)));
 }
 
 void ExtendedAuthenticatorImpl::OnOperationComplete(
     const std::string& time_marker,
     const UserContext& user_context,
-    const base::Closure& success_callback,
+    base::OnceClosure success_callback,
     bool success,
     cryptohome::MountError return_code) {
   RecordEndMarker(time_marker);
   if (return_code == cryptohome::MOUNT_ERROR_NONE) {
-    if (!success_callback.is_null())
-      success_callback.Run();
-    if (old_consumer_)
-      old_consumer_->OnAuthSuccess(user_context);
+    if (success_callback)
+      std::move(success_callback).Run();
+    if (consumer_)
+      consumer_->OnAuthSuccess(user_context);
     return;
   }
 
-  LOG(ERROR) << "Supervised user cryptohome error, code: " << return_code;
+  LOG(ERROR) << "Extended authenticator cryptohome error, code: "
+             << return_code;
 
   AuthState state = FAILED_MOUNT;
 
@@ -301,12 +249,9 @@ void ExtendedAuthenticatorImpl::OnOperationComplete(
   if (return_code == cryptohome::MOUNT_ERROR_USER_DOES_NOT_EXIST)
     state = NO_MOUNT;
 
-  if (consumer_)
-    consumer_->OnAuthenticationFailure(state);
-
-  if (old_consumer_) {
+  if (consumer_) {
     AuthFailure failure(AuthFailure::UNLOCK_FAILED);
-    old_consumer_->OnAuthFailure(failure);
+    consumer_->OnAuthFailure(failure);
   }
 }
 
