@@ -10,23 +10,26 @@
 
 #include <algorithm>
 #include <memory>
+#include <utility>
 
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/location.h"
 #include "base/logging.h"
 #include "base/optional.h"
 #include "base/stl_util.h"
 #include "base/task/post_task.h"
+#include "base/task/thread_pool.h"
 #include "base/time/clock.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_service_client.h"
+#include "chromeos/dbus/shill/shill_service_client.h"
 #include "chromeos/network/certificate_helper.h"
 #include "chromeos/network/managed_network_configuration_handler.h"
 #include "chromeos/network/network_event_log.h"
 #include "chromeos/network/network_state.h"
-#include "chromeos/tools/variable_expander.h"
+#include "chromeos/network/onc/onc_certificate_pattern.h"
+#include "chromeos/network/onc/variable_expander.h"
 #include "components/onc/onc_constants.h"
+#include "crypto/scoped_nss_types.h"
 #include "dbus/object_path.h"
 #include "net/cert/scoped_nss_types.h"
 #include "net/cert/x509_certificate.h"
@@ -67,10 +70,10 @@ struct MatchingCert {
 };
 
 // Describes a network that is configured with |client_cert_config|, which
-// includes the certificate pattern.
-struct NetworkAndCertPattern {
-  NetworkAndCertPattern(const std::string& network_path,
-                        const client_cert::ClientCertConfig& client_cert_config)
+// includes the certificate config.
+struct NetworkAndCertConfig {
+  NetworkAndCertConfig(const std::string& network_path,
+                       const client_cert::ClientCertConfig& client_cert_config)
       : service_path(network_path), cert_config(client_cert_config) {}
 
   std::string service_path;
@@ -95,7 +98,7 @@ std::map<std::string, std::string> GetSubstitutionsForCert(
     std::string firstSANEmail;
     if (!names.empty())
       firstSANEmail = names[0];
-    substitutions[onc::substitutes::kCertSANEmail] = firstSANEmail;
+    substitutions[::onc::substitutes::kCertSANEmail] = firstSANEmail;
   }
 
   {
@@ -105,15 +108,23 @@ std::map<std::string, std::string> GetSubstitutionsForCert(
     std::string firstSANUPN;
     if (!names.empty())
       firstSANUPN = names[0];
-    substitutions[onc::substitutes::kCertSANUPN] = firstSANUPN;
+    substitutions[::onc::substitutes::kCertSANUPN] = firstSANUPN;
   }
 
-  substitutions[onc::substitutes::kCertSubjectCommonName] =
+  substitutions[::onc::substitutes::kCertSubjectCommonName] =
       certificate::GetCertAsciiSubjectCommonName(cert);
 
   return substitutions;
 }
 
+// Returns true if |client_cert_config| specifies a pattern or reference, i.e.
+// if client certificate resolution should be attempted.
+bool ShouldResolveCert(
+    const client_cert::ClientCertConfig& client_cert_config) {
+  return client_cert_config.client_cert_type == ::onc::client_cert::kPattern ||
+         client_cert_config.client_cert_type == ::onc::client_cert::kRef;
+}
+
 }  // namespace
 
 namespace internal {
@@ -134,10 +145,10 @@ struct MatchingCertAndResolveStatus {
 
 // Describes a network |network_path| and the client cert resolution result.
 struct NetworkAndMatchingCert {
-  NetworkAndMatchingCert(const NetworkAndCertPattern& network_and_pattern,
+  NetworkAndMatchingCert(const NetworkAndCertConfig& network_and_cert_config,
                          base::Optional<MatchingCert> matching_cert)
-      : service_path(network_and_pattern.service_path),
-        cert_config_type(network_and_pattern.cert_config.location),
+      : service_path(network_and_cert_config.service_path),
+        cert_config_type(network_and_cert_config.cert_config.location),
         matching_cert(matching_cert) {}
 
   std::string service_path;
@@ -155,27 +166,45 @@ using internal::NetworkAndMatchingCert;
 
 namespace {
 
-// Returns true if a private key for certificate |cert| is installed.
-// Note that HasPrivateKey is not a cheap operation: it iterates all tokens and
-// attempts to look up the private key.
-bool HasPrivateKey(CERTCertificate* cert) {
-  PK11SlotInfo* slot = PK11_KeyForCertExists(cert, nullptr, nullptr);
-  if (!slot)
-    return false;
+// Returns the nickname of the private key for certificate |cert|, if such a
+// private key is installed. Note that this is not a cheap operation: it
+// iterates all tokens and attempts to look up the private key.
+// A return value of |base::nullopt| means that no private key could be found
+// for |cert|.
+// If a private key could be found for |cert| but it did not have a nickname,
+// will return the empty string.
+base::Optional<std::string> GetPrivateKeyNickname(CERTCertificate* cert) {
+  crypto::ScopedSECKEYPrivateKey key(
+      PK11_FindKeyByAnyCert(cert, nullptr /* wincx */));
+  if (!key)
+    return base::nullopt;
 
-  PK11_FreeSlot(slot);
-  return true;
+  std::string key_nickname;
+  char* nss_key_nickname = PK11_GetPrivateKeyNickname(key.get());
+  if (nss_key_nickname) {
+    key_nickname = nss_key_nickname;
+    PORT_Free(nss_key_nickname);
+  }
+  return key_nickname;
 }
 
 // Describes a certificate which is issued by |issuer| (encoded as PEM).
 // |issuer| can be empty if no issuer certificate is found in the database.
 struct CertAndIssuer {
   CertAndIssuer(net::ScopedCERTCertificate certificate,
-                const std::string& issuer)
-      : cert(std::move(certificate)), pem_encoded_issuer(issuer) {}
+                const std::string& issuer,
+                const std::string& key_nickname)
+      : cert(std::move(certificate)),
+        pem_encoded_issuer(issuer),
+        private_key_nickname(key_nickname) {}
 
   net::ScopedCERTCertificate cert;
   std::string pem_encoded_issuer;
+  // The nickname of the private key associated with |cert|. This is used
+  // for resolving ClientCertRef ONC references, using the fact that
+  // |CertificateImporterImpl| sets the private key nickname to the
+  // ONC-specified GUID.
+  std::string private_key_nickname;
 };
 
 bool CompareCertExpiration(const CertAndIssuer& a, const CertAndIssuer& b) {
@@ -187,13 +216,14 @@ bool CompareCertExpiration(const CertAndIssuer& a, const CertAndIssuer& b) {
 }
 
 // A unary predicate that returns true if the given CertAndIssuer matches the
-// given certificate pattern.
-struct MatchCertWithPattern {
-  explicit MatchCertWithPattern(const CertificatePattern& cert_pattern)
-      : pattern(cert_pattern) {}
+// given certificate config.
+struct MatchCertWithCertConfig {
+  explicit MatchCertWithCertConfig(
+      const client_cert::ClientCertConfig& client_cert_config)
+      : cert_config(client_cert_config) {}
 
   bool operator()(const CertAndIssuer& cert_and_issuer) {
-    if (!pattern.issuer().Empty() || !pattern.subject().Empty()) {
+    if (cert_config.client_cert_type == ::onc::client_cert::kPattern) {
       // Allow UTF-8 inside PrintableStrings in client certificates. See
       // crbug.com/770323 and crbug.com/788655.
       net::X509Certificate::UnsafeCreateOptions options;
@@ -203,28 +233,21 @@ struct MatchCertWithPattern {
               cert_and_issuer.cert.get(), {}, options);
       if (!x509_cert)
         return false;
-      if (!pattern.issuer().Empty() &&
-          !client_cert::CertPrincipalMatches(pattern.issuer(),
-                                             x509_cert->issuer())) {
-        return false;
-      }
-      if (!pattern.subject().Empty() &&
-          !client_cert::CertPrincipalMatches(pattern.subject(),
-                                             x509_cert->subject())) {
-        return false;
-      }
+      return cert_config.pattern.Matches(*x509_cert,
+                                         cert_and_issuer.pem_encoded_issuer);
     }
 
-    const std::vector<std::string>& issuer_ca_pems = pattern.issuer_ca_pems();
-    if (!issuer_ca_pems.empty() &&
-        !base::ContainsValue(issuer_ca_pems,
-                             cert_and_issuer.pem_encoded_issuer)) {
-      return false;
+    if (cert_config.client_cert_type == ::onc::client_cert::kRef) {
+      // This relies on the fact that |CertificateImporterImpl| sets the
+      // nickname for the imported private key to the GUID.
+      return cert_config.guid == cert_and_issuer.private_key_nickname;
     }
-    return true;
+
+    NOTREACHED();
+    return false;
   }
 
-  const CertificatePattern pattern;
+  const client_cert::ClientCertConfig cert_config;
 };
 
 // Lookup the issuer certificate of |cert|. If it is available, return the PEM
@@ -253,70 +276,88 @@ std::string GetPEMEncodedIssuer(CERTCertificate* cert) {
   return pem_encoded_issuer;
 }
 
-std::vector<CertAndIssuer> CreateSortedCertAndIssuerList(
-    net::ScopedCERTCertificateList certs,
-    base::Time now) {
+void CreateSortedCertAndIssuerList(
+    const NetworkCertLoader::NetworkCertList& network_certs,
+    base::Time now,
+    std::vector<CertAndIssuer>* all_cert_and_issuers,
+    std::vector<CertAndIssuer>* device_wide_cert_and_issuers) {
   // Filter all client certs and determines each certificate's issuer, which is
   // required for the pattern matching.
-  // TODO(crbug.com/781693): Consider moving the filtering of client certs into
-  // NetworkCertLoader. It should not be in ClientCertResolver's responsibility
-  // to decide if a certificate is a valid client certificate or not. Other
-  // consumers of NetworkCertLoader could also use a pre-filtered list (e.g.
-  // NetworkCertMigrator).
-  std::vector<CertAndIssuer> client_certs;
-  for (net::ScopedCERTCertificate& scoped_cert : certs) {
-    CERTCertificate* cert = scoped_cert.get();
+  for (const NetworkCertLoader::NetworkCert& network_cert : network_certs) {
+    // If the caller is interested in device-wide certificates only, skip
+    // user-specific certificates.
+    if (!all_cert_and_issuers && !network_cert.is_device_wide())
+      continue;
+
+    CERTCertificate* cert = network_cert.cert();
     base::Time not_after;
-    // HasPrivateKey should be invoked after IsCertificateHardwareBacked for
-    // performance reasons.
     if (!net::x509_util::GetValidityTimes(cert, nullptr, &not_after) ||
         now > not_after ||
-        !NetworkCertLoader::IsCertificateHardwareBacked(cert) ||
-        !HasPrivateKey(cert)) {
+        !NetworkCertLoader::IsCertificateHardwareBacked(cert)) {
+      continue;
+    }
+    // GetPrivateKeyNickname should be invoked after the checks above for
+    // performance reasons.
+    base::Optional<std::string> private_key_nickname =
+        GetPrivateKeyNickname(cert);
+    if (!private_key_nickname.has_value()) {
+      // No private key has been found for this certificate.
       continue;
     }
     std::string pem_encoded_issuer = GetPEMEncodedIssuer(cert);
-    client_certs.push_back(
-        CertAndIssuer(std::move(scoped_cert), pem_encoded_issuer));
+    if (all_cert_and_issuers) {
+      all_cert_and_issuers->push_back(
+          CertAndIssuer(net::x509_util::DupCERTCertificate(cert),
+                        pem_encoded_issuer, private_key_nickname.value()));
+    }
+    if (device_wide_cert_and_issuers && network_cert.is_device_wide()) {
+      device_wide_cert_and_issuers->push_back(
+          CertAndIssuer(net::x509_util::DupCERTCertificate(cert),
+                        pem_encoded_issuer, private_key_nickname.value()));
+    }
   }
 
-  std::sort(client_certs.begin(), client_certs.end(), &CompareCertExpiration);
-  return client_certs;
+  if (all_cert_and_issuers) {
+    std::sort(all_cert_and_issuers->begin(), all_cert_and_issuers->end(),
+              &CompareCertExpiration);
+  }
+  if (device_wide_cert_and_issuers) {
+    std::sort(device_wide_cert_and_issuers->begin(),
+              device_wide_cert_and_issuers->end(), &CompareCertExpiration);
+  }
 }
 
-// Searches for matches between |networks| and |all_certs| (for networks
-// configured in user policy) / |system_token_client_certs| (for networks
-// configured in device policy). Returns the matches that were found. Because
-// this calls NSS functions and is potentially slow, it must be run on a worker
-// thread.
+// Searches for matches between |networks| and |network_certs|. Returns the
+// matches that were found. Because this calls NSS functions and is potentially
+// slow, it must be run on a worker thread.
 std::vector<NetworkAndMatchingCert> FindCertificateMatches(
-    net::ScopedCERTCertificateList all_certs,
-    net::ScopedCERTCertificateList system_token_client_certs,
-    const std::vector<NetworkAndCertPattern>& networks,
+    NetworkCertLoader::NetworkCertList network_certs,
+    const std::vector<NetworkAndCertConfig>& networks,
     base::Time now) {
   std::vector<NetworkAndMatchingCert> matches;
 
-  std::vector<CertAndIssuer> all_client_certs(
-      CreateSortedCertAndIssuerList(std::move(all_certs), now));
-  std::vector<CertAndIssuer> system_client_certs(
-      CreateSortedCertAndIssuerList(std::move(system_token_client_certs), now));
+  std::vector<CertAndIssuer> all_client_cert_and_issuers;
+  std::vector<CertAndIssuer> device_wide_client_cert_and_issuers;
+  CreateSortedCertAndIssuerList(network_certs, now,
+                                &all_client_cert_and_issuers,
+                                &device_wide_client_cert_and_issuers);
 
-  for (const NetworkAndCertPattern& network_and_pattern : networks) {
+  for (const NetworkAndCertConfig& network_and_cert_config : networks) {
     // Use only certs from the system token if the source of the client cert
     // pattern is device policy.
     std::vector<CertAndIssuer>* client_certs =
-        network_and_pattern.cert_config.onc_source ==
+        network_and_cert_config.cert_config.onc_source ==
                 ::onc::ONC_SOURCE_DEVICE_POLICY
-            ? &system_client_certs
-            : &all_client_certs;
+            ? &device_wide_client_cert_and_issuers
+            : &all_client_cert_and_issuers;
     auto cert_it = std::find_if(
         client_certs->begin(), client_certs->end(),
-        MatchCertWithPattern(network_and_pattern.cert_config.pattern));
+        MatchCertWithCertConfig(network_and_cert_config.cert_config));
     if (cert_it == client_certs->end()) {
       VLOG(1) << "Couldn't find a matching client cert for network "
-              << network_and_pattern.service_path;
+              << network_and_cert_config.service_path;
       matches.push_back(
-          NetworkAndMatchingCert(network_and_pattern, base::nullopt));
+          NetworkAndMatchingCert(network_and_cert_config, base::nullopt));
       continue;
     }
 
@@ -336,13 +377,13 @@ std::vector<NetworkAndMatchingCert> FindCertificateMatches(
     // client certificate.
     VariableExpander variable_expander(
         GetSubstitutionsForCert(cert_it->cert.get()));
-    std::string identity = network_and_pattern.cert_config.policy_identity;
+    std::string identity = network_and_cert_config.cert_config.policy_identity;
     const bool success = variable_expander.ExpandString(&identity);
     LOG_IF(ERROR, !success)
         << "Error during variable expansion in ONC-configured identity";
 
     matches.push_back(NetworkAndMatchingCert(
-        network_and_pattern, MatchingCert(pkcs11_id, slot_id, identity)));
+        network_and_cert_config, MatchingCert(pkcs11_id, slot_id, identity)));
   }
   return matches;
 }
@@ -373,8 +414,7 @@ ClientCertResolver::ClientCertResolver()
       network_properties_changed_(false),
       network_state_handler_(nullptr),
       managed_network_config_handler_(nullptr),
-      testing_clock_(nullptr),
-      weak_ptr_factory_(this) {
+      testing_clock_(nullptr) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 }
 
@@ -419,31 +459,34 @@ bool ClientCertResolver::IsAnyResolveTaskRunning() const {
 }
 
 // static
-bool ClientCertResolver::ResolveCertificatePatternSync(
+bool ClientCertResolver::ResolveClientCertificateSync(
     const client_cert::ConfigType client_cert_type,
     const client_cert::ClientCertConfig& client_cert_config,
     base::DictionaryValue* shill_properties) {
+  if (!ShouldResolveCert(client_cert_config))
+    return false;
+
   // Prepare and sort the list of known client certs. Use only certs from the
-  // system token if the source of the client cert pattern is device policy.
-  std::vector<CertAndIssuer> client_certs;
+  // system token if the source of the client cert config is device policy.
+  std::vector<CertAndIssuer> client_cert_and_issuers;
   if (client_cert_config.onc_source == ::onc::ONC_SOURCE_DEVICE_POLICY) {
-    client_certs = CreateSortedCertAndIssuerList(
-        net::x509_util::DupCERTCertificateList(
-            NetworkCertLoader::Get()->system_token_client_certs()),
-        base::Time::Now());
+    CreateSortedCertAndIssuerList(
+        NetworkCertLoader::Get()->client_certs(), base::Time::Now(),
+        nullptr /* all_cert_and_issuers */,
+        &client_cert_and_issuers /* device_wide_cert_and_issuers */);
   } else {
-    client_certs = CreateSortedCertAndIssuerList(
-        net::x509_util::DupCERTCertificateList(
-            NetworkCertLoader::Get()->all_certs()),
-        base::Time::Now());
+    CreateSortedCertAndIssuerList(
+        NetworkCertLoader::Get()->client_certs(), base::Time::Now(),
+        &client_cert_and_issuers /* all_cert_and_issuers */,
+        nullptr /* device_wide_cert_and_issuers */);
   }
 
-  // Search for a certificate matching the pattern.
-  std::vector<CertAndIssuer>::iterator cert_it =
-      std::find_if(client_certs.begin(), client_certs.end(),
-                   MatchCertWithPattern(client_cert_config.pattern));
+  // Search for a certificate matching the pattern or reference.
+  std::vector<CertAndIssuer>::iterator cert_it = std::find_if(
+      client_cert_and_issuers.begin(), client_cert_and_issuers.end(),
+      MatchCertWithCertConfig(client_cert_config));
 
-  if (cert_it == client_certs.end()) {
+  if (cert_it == client_cert_and_issuers.end()) {
     VLOG(1) << "Couldn't find a matching client cert";
     client_cert::SetEmptyShillProperties(client_cert_type, shill_properties);
     return false;
@@ -511,13 +554,12 @@ void ClientCertResolver::NetworkConnectionStateChanged(
     return;
   if (!network->IsConnectingOrConnected()) {
     NET_LOG(EVENT) << "ClientCertResolver: ConnectionStateChanged: "
-                   << network->name();
+                   << NetworkId(network);
     ResolveNetworks(NetworkStateHandler::NetworkStateList(1, network));
   }
 }
 
-void ClientCertResolver::OnCertificatesLoaded(
-    const net::ScopedCERTCertificateList& cert_list) {
+void ClientCertResolver::OnCertificatesLoaded() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   VLOG(2) << "OnCertificatesLoaded.";
   if (!ClientCertificatesLoaded())
@@ -549,7 +591,7 @@ void ClientCertResolver::PolicyAppliedToNetwork(
     return;
   }
   NET_LOG(EVENT) << "ClientCertResolver: PolicyAppliedToNetwork: "
-                 << network->name();
+                 << NetworkId(network);
   NetworkStateHandler::NetworkStateList networks;
   networks.push_back(network);
   ResolveNetworks(networks);
@@ -558,10 +600,10 @@ void ClientCertResolver::PolicyAppliedToNetwork(
 void ClientCertResolver::ResolveNetworks(
     const NetworkStateHandler::NetworkStateList& networks) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  std::vector<NetworkAndCertPattern> networks_to_resolve;
+  std::vector<NetworkAndCertConfig> networks_to_resolve;
 
-  // Filter networks with ClientCertPattern. As ClientCertPatterns can only be
-  // set by policy, we check there.
+  // Filter networks with ClientCertPattern or ClientCertRef. As these can only
+  // be set by policy, we check there.
   for (const NetworkState* network : networks) {
     // If the network was not known before, mark it as known but with resolving
     // pending.
@@ -569,11 +611,12 @@ void ClientCertResolver::ResolveNetworks(
       networks_status_.insert_or_assign(network->path(),
                                         MatchingCertAndResolveStatus());
 
-    // If this network is not configured, it cannot have a ClientCertPattern.
+    // If this network is not configured, it cannot have a
+    // ClientCertPattern/ClientCertRef.
     if (network->profile_path().empty())
       continue;
 
-    onc::ONCSource onc_source = onc::ONC_SOURCE_NONE;
+    ::onc::ONCSource onc_source = ::onc::ONC_SOURCE_NONE;
     const base::DictionaryValue* policy =
         managed_network_config_handler_->FindPolicyByGuidAndProfile(
             network->guid(), network->profile_path(), &onc_source);
@@ -590,12 +633,12 @@ void ClientCertResolver::ResolveNetworks(
     client_cert::ClientCertConfig cert_config;
     OncToClientCertConfig(onc_source, *policy, &cert_config);
 
-    // Skip networks that don't have a ClientCertPattern.
-    if (cert_config.client_cert_type != ::onc::client_cert::kPattern)
+    // Skip networks that don't have a ClientCertPattern or ClientCertRef.
+    if (!ShouldResolveCert(cert_config))
       continue;
 
     networks_to_resolve.push_back(
-        NetworkAndCertPattern(network->path(), cert_config));
+        NetworkAndCertConfig(network->path(), cert_config));
   }
 
   if (networks_to_resolve.empty()) {
@@ -609,23 +652,19 @@ void ClientCertResolver::ResolveNetworks(
 
   if (resolve_task_running_) {
     VLOG(1) << "A resolve task is already running. Queue this request.";
-    for (const NetworkAndCertPattern& network_and_pattern :
-         networks_to_resolve) {
-      queued_networks_to_resolve_.insert(network_and_pattern.service_path);
-    }
+    for (const NetworkAndCertConfig& network_to_resolve : networks_to_resolve)
+      queued_networks_to_resolve_.insert(network_to_resolve.service_path);
     return;
   }
 
   VLOG(2) << "Start task for resolving client cert patterns.";
   resolve_task_running_ = true;
-  base::PostTaskWithTraitsAndReplyWithResult(
+  base::ThreadPool::PostTaskAndReplyWithResult(
       FROM_HERE,
       {base::MayBlock(), base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
       base::BindOnce(&FindCertificateMatches,
-                     net::x509_util::DupCERTCertificateList(
-                         NetworkCertLoader::Get()->all_certs()),
-                     net::x509_util::DupCERTCertificateList(
-                         NetworkCertLoader::Get()->system_token_client_certs()),
+                     NetworkCertLoader::CloneNetworkCertList(
+                         NetworkCertLoader::Get()->client_certs()),
                      networks_to_resolve, Now()),
       base::BindOnce(&ClientCertResolver::ConfigureCertificates,
                      weak_ptr_factory_.GetWeakPtr()));
@@ -667,7 +706,7 @@ void ClientCertResolver::ConfigureCertificates(
     network_properties_changed_ = true;
 
     NET_LOG(EVENT) << "Configuring certificate for network: "
-                   << match.service_path;
+                   << NetworkPathId(match.service_path);
 
     base::DictionaryValue shill_properties;
     if (match.matching_cert.has_value()) {
@@ -683,9 +722,9 @@ void ClientCertResolver::ConfigureCertificates(
       client_cert::SetEmptyShillProperties(match.cert_config_type,
                                            &shill_properties);
     }
-    DBusThreadManager::Get()->GetShillServiceClient()->SetProperties(
+    ShillServiceClient::Get()->SetProperties(
         dbus::ObjectPath(match.service_path), shill_properties,
-        base::DoNothing(), base::BindRepeating(&LogError, match.service_path));
+        base::DoNothing(), base::BindOnce(&LogError, match.service_path));
     network_state_handler_->RequestUpdateForNetwork(match.service_path);
   }
   resolve_task_running_ = false;
