@@ -8,31 +8,35 @@
 
 #include <memory>
 
+#include "base/base64.h"
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/json/json_reader.h"
 #include "base/macros.h"
+#include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
-#include "base/test/scoped_task_environment.h"
 #include "base/test/simple_test_clock.h"
+#include "base/test/task_environment.h"
 #include "base/values.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_manager_client.h"
-#include "chromeos/dbus/shill_profile_client.h"
-#include "chromeos/dbus/shill_service_client.h"
+#include "chromeos/dbus/shill/shill_clients.h"
+#include "chromeos/dbus/shill/shill_manager_client.h"
+#include "chromeos/dbus/shill/shill_profile_client.h"
+#include "chromeos/dbus/shill/shill_service_client.h"
 #include "chromeos/network/managed_network_configuration_handler_impl.h"
 #include "chromeos/network/network_cert_loader.h"
 #include "chromeos/network/network_configuration_handler.h"
 #include "chromeos/network/network_profile_handler.h"
 #include "chromeos/network/network_state_handler.h"
+#include "chromeos/network/onc/onc_certificate_importer_impl.h"
+#include "chromeos/network/onc/onc_test_utils.h"
 #include "components/onc/onc_constants.h"
 #include "crypto/scoped_nss_types.h"
 #include "crypto/scoped_test_nss_db.h"
 #include "net/base/net_errors.h"
 #include "net/cert/nss_cert_database_chromeos.h"
-#include "net/cert/pem_tokenizer.h"
+#include "net/cert/pem.h"
 #include "net/cert/x509_certificate.h"
 #include "net/cert/x509_util_nss.h"
 #include "net/test/cert_test_util.h"
@@ -49,6 +53,40 @@ constexpr char kWifiSSID[] = "wifi_ssid";
 constexpr char kUserProfilePath[] = "user_profile";
 constexpr char kUserHash[] = "user_hash";
 
+void OnImportCompleted(base::OnceClosure loop_quit_closure, bool success) {
+  EXPECT_TRUE(success);
+  std::move(loop_quit_closure).Run();
+}
+
+void OnListCertsDone(base::OnceClosure loop_quit_closure,
+                     net::ScopedCERTCertificateList* out_cert_list,
+                     net::ScopedCERTCertificateList cert_list) {
+  out_cert_list->swap(cert_list);
+  std::move(loop_quit_closure).Run();
+}
+
+// Returns a |OncParsedCertificates| which contains exactly one client
+// certificate with the contents of |client_cert_pkcs12_file| and the GUID
+// |guid|. Returns nullptr if the file could not be read.
+std::unique_ptr<onc::OncParsedCertificates> OncParsedCertificatesForPkcs12File(
+    const base::FilePath& client_cert_pkcs12_file,
+    base::StringPiece guid) {
+  std::string pkcs12_raw;
+  if (!base::ReadFileToString(client_cert_pkcs12_file, &pkcs12_raw))
+    return nullptr;
+
+  std::string pkcs12_base64_encoded;
+  base::Base64Encode(pkcs12_raw, &pkcs12_base64_encoded);
+
+  base::Value onc_certificate(base::Value::Type::DICTIONARY);
+  onc_certificate.SetKey("GUID", base::Value(guid));
+  onc_certificate.SetKey("Type", base::Value("Client"));
+  onc_certificate.SetKey("PKCS12", base::Value(pkcs12_base64_encoded));
+  base::Value onc_certificates(base::Value::Type::LIST);
+  onc_certificates.Append(std::move(onc_certificate));
+  return std::make_unique<onc::OncParsedCertificates>(onc_certificates);
+}
+
 }  // namespace
 
 class ClientCertResolverTest : public testing::Test,
@@ -60,21 +98,27 @@ class ClientCertResolverTest : public testing::Test,
   void SetUp() override {
     ASSERT_TRUE(test_nssdb_.is_open());
     ASSERT_TRUE(test_system_nssdb_.is_open());
-
-    // Use the same DB for public and private slot.
-    test_nsscertdb_.reset(new net::NSSCertDatabaseChromeOS(
+    // Use the same slot as public and private slot for the user's
+    // NSSCertDatabse for testing.
+    test_nsscertdb_ = std::make_unique<net::NSSCertDatabaseChromeOS>(
         crypto::ScopedPK11Slot(PK11_ReferenceSlot(test_nssdb_.slot())),
-        crypto::ScopedPK11Slot(PK11_ReferenceSlot(test_nssdb_.slot()))));
+        crypto::ScopedPK11Slot(PK11_ReferenceSlot(test_nssdb_.slot())));
+    // Create a NSSCertDatabase for the system slot. While NetworkCertLoader
+    // does not care about the public slot in this database, NSSCertDatabase
+    // requires a public slot. Pass the system slot there for testing.
+    test_system_nsscertdb_ = std::make_unique<net::NSSCertDatabaseChromeOS>(
+        crypto::ScopedPK11Slot(PK11_ReferenceSlot(test_system_nssdb_.slot())),
+        crypto::ScopedPK11Slot() /* private_slot */);
+    test_system_nsscertdb_->SetSystemSlot(
+        crypto::ScopedPK11Slot(PK11_ReferenceSlot(test_system_nssdb_.slot())));
 
-    DBusThreadManager::Initialize();
-    service_test_ =
-        DBusThreadManager::Get()->GetShillServiceClient()->GetTestInterface();
-    profile_test_ =
-        DBusThreadManager::Get()->GetShillProfileClient()->GetTestInterface();
+    shill_clients::InitializeFakes();
+    service_test_ = ShillServiceClient::Get()->GetTestInterface();
+    profile_test_ = ShillProfileClient::Get()->GetTestInterface();
     profile_test_->AddProfile(kUserProfilePath, kUserHash);
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
     service_test_->ClearServices();
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
 
     NetworkCertLoader::Initialize();
     network_cert_loader_ = NetworkCertLoader::Get();
@@ -93,12 +137,13 @@ class ClientCertResolverTest : public testing::Test,
     network_profile_handler_.reset();
     network_state_handler_.reset();
     NetworkCertLoader::Shutdown();
-    DBusThreadManager::Shutdown();
+    shill_clients::Shutdown();
   }
 
  protected:
   void StartNetworkCertLoader() {
     network_cert_loader_->SetUserNSSDB(test_nsscertdb_.get());
+    network_cert_loader_->SetSystemNSSDB(test_system_nsscertdb_.get());
     if (test_client_cert_.get()) {
       int slot_id = 0;
       const std::string pkcs11_id =
@@ -167,9 +212,6 @@ class ClientCertResolverTest : public testing::Test,
   }
 
   void SetupTestCertInSystemToken(const std::string& prefix) {
-    test_nsscertdb_->SetSystemSlot(
-        crypto::ScopedPK11Slot(PK11_ReferenceSlot(test_system_nssdb_.slot())));
-
     net::ImportClientCertAndKeyFromFile(
         net::GetTestCertsDirectory(), prefix + ".pem", prefix + ".pk8",
         test_system_nssdb_.slot(), &test_client_cert_);
@@ -195,7 +237,7 @@ class ClientCertResolverTest : public testing::Test,
         network_config_handler_.get(), nullptr /* network_device_handler */,
         nullptr /* prohibited_technologies_handler */);
     // Run all notifications before starting the cert loader to reduce run time.
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
 
     client_cert_resolver_->Init(network_state_handler_.get(),
                                 managed_config_handler_.get());
@@ -215,150 +257,122 @@ class ClientCertResolverTest : public testing::Test,
                                       base::Value("invalid id"));
     profile_test_->AddService(kUserProfilePath, kWifiStub);
 
-    DBusThreadManager::Get()
-        ->GetShillManagerClient()
-        ->GetTestInterface()
-        ->AddManagerService(kWifiStub, true);
+    ShillManagerClient::Get()->GetTestInterface()->AddManagerService(kWifiStub,
+                                                                     true);
   }
 
   // Sets up a policy with a certificate pattern that matches any client cert
   // with a certain Issuer CN. It will match the test client cert imported by
   // SetupTestCerts.
-  void SetupPolicyMatchingIssuerCN(onc::ONCSource onc_source) {
-    const char* test_policy =
-        "[ { \"GUID\": \"wifi_stub\","
-        "    \"Name\": \"wifi_stub\","
-        "    \"Type\": \"WiFi\","
-        "    \"WiFi\": {"
-        "      \"Security\": \"WPA-EAP\","
-        "      \"SSID\": \"wifi_ssid\","
-        "      \"EAP\": {"
-        "        \"Outer\": \"EAP-TLS\","
-        "        \"ClientCertType\": \"Pattern\","
-        "        \"ClientCertPattern\": {"
-        "          \"Issuer\": {"
-        "            \"CommonName\": \"B CA\""
-        "          }"
-        "        }"
-        "      }"
-        "    }"
-        "} ]";
-
-    std::string error;
-    std::unique_ptr<base::Value> policy_value =
-        base::JSONReader::ReadAndReturnError(
-            test_policy, base::JSON_ALLOW_TRAILING_COMMAS, nullptr, &error);
-    ASSERT_TRUE(policy_value) << error;
-
-    base::ListValue* policy = nullptr;
-    ASSERT_TRUE(policy_value->GetAsList(&policy));
-
-    std::string user_hash =
-        onc_source == onc::ONC_SOURCE_USER_POLICY ? kUserHash : "";
-    managed_config_handler_->SetPolicy(
-        onc_source, user_hash, *policy,
-        base::DictionaryValue() /* no global network config */);
+  void SetupPolicyMatchingIssuerCN(::onc::ONCSource onc_source) {
+    const char* test_policy = R"(
+        [ { "GUID": "wifi_stub",
+            "Name": "wifi_stub",
+            "Type": "WiFi",
+            "WiFi": {
+              "Security": "WPA-EAP",
+              "SSID": "wifi_ssid",
+              "EAP": {
+                "Outer": "EAP-TLS",
+                "ClientCertType": "Pattern",
+                "ClientCertPattern": {
+                  "Issuer": {
+                    "CommonName": "B CA"
+                  }
+                }
+              }
+            }
+        } ])";
+    ASSERT_NO_FATAL_FAILURE(SetManagedNetworkPolicy(onc_source, test_policy));
   }
 
   // Sets up a policy with a certificate pattern that matches any client cert
   // with a Subject Organization set to "Blar". It will match the test client
   // cert imported by SetupTestCertWithBadPrintableString.
   void SetupPolicyMatchingSubjectOrgForBadPrintableStringCert(
-      onc::ONCSource onc_source) {
-    const char* test_policy =
-        "[ { \"GUID\": \"wifi_stub\","
-        "    \"Name\": \"wifi_stub\","
-        "    \"Type\": \"WiFi\","
-        "    \"WiFi\": {"
-        "      \"Security\": \"WPA-EAP\","
-        "      \"SSID\": \"wifi_ssid\","
-        "      \"EAP\": {"
-        "        \"Outer\": \"EAP-TLS\","
-        "        \"ClientCertType\": \"Pattern\","
-        "        \"ClientCertPattern\": {"
-        "          \"Subject\": {"
-        "            \"Organization\": \"Blar\""
-        "          }"
-        "        }"
-        "      }"
-        "    }"
-        "} ]";
-
-    std::string error;
-    std::unique_ptr<base::Value> policy_value =
-        base::JSONReader::ReadAndReturnError(
-            test_policy, base::JSON_ALLOW_TRAILING_COMMAS, nullptr, &error);
-    ASSERT_TRUE(policy_value) << error;
-
-    base::ListValue* policy = nullptr;
-    ASSERT_TRUE(policy_value->GetAsList(&policy));
-
-    std::string user_hash =
-        onc_source == onc::ONC_SOURCE_USER_POLICY ? kUserHash : "";
-    managed_config_handler_->SetPolicy(
-        onc_source, user_hash, *policy,
-        base::DictionaryValue() /* no global network config */);
+      ::onc::ONCSource onc_source) {
+    const char* test_policy = R"(
+        [ { "GUID": "wifi_stub",
+            "Name": "wifi_stub",
+            "Type": "WiFi",
+            "WiFi": {
+              "Security": "WPA-EAP",
+              "SSID": "wifi_ssid",
+              "EAP": {
+                "Outer": "EAP-TLS",
+                "ClientCertType": "Pattern",
+                "ClientCertPattern": {
+                  "Subject": {
+                    "Organization": "Blar"
+                  }
+                }
+              }
+            }
+        } ])";
+    ASSERT_NO_FATAL_FAILURE(SetManagedNetworkPolicy(onc_source, test_policy));
   }
 
   void SetupCertificateConfigMatchingIssuerCN(
-      onc::ONCSource onc_source,
+      ::onc::ONCSource onc_source,
       client_cert::ClientCertConfig* client_cert_config) {
-    const char* test_onc_pattern =
-        "{"
-        "  \"Issuer\": {"
-        "    \"CommonName\": \"B CA\""
-        "  }"
-        "}";
-    std::string error;
-    std::unique_ptr<base::Value> onc_pattern_value =
-        base::JSONReader::ReadAndReturnError(test_onc_pattern,
-                                             base::JSON_ALLOW_TRAILING_COMMAS,
-                                             nullptr, &error);
-    ASSERT_TRUE(onc_pattern_value) << error;
+    const char* test_onc_pattern = R"(
+        {
+          "Issuer": {
+            "CommonName": "B CA"
+          }
+        })";
+    base::JSONReader::ValueWithError parsed_json =
+        base::JSONReader::ReadAndReturnValueWithError(
+            test_onc_pattern, base::JSON_ALLOW_TRAILING_COMMAS);
+    ASSERT_TRUE(parsed_json.value) << parsed_json.error_message;
 
     base::DictionaryValue* onc_pattern_dict;
-    onc_pattern_value->GetAsDictionary(&onc_pattern_dict);
+    parsed_json.value->GetAsDictionary(&onc_pattern_dict);
 
     client_cert_config->onc_source = onc_source;
+    client_cert_config->client_cert_type = ::onc::client_cert::kPattern;
     client_cert_config->pattern.ReadFromONCDictionary(*onc_pattern_dict);
   }
 
   // Sets up a policy with a certificate pattern that matches any client cert
   // that is signed by the test CA cert (stored in |test_ca_cert_pem_|). In
   // particular it will match the test client cert.
-  void SetupPolicyMatchingIssuerPEM(onc::ONCSource onc_source,
+  void SetupPolicyMatchingIssuerPEM(::onc::ONCSource onc_source,
                                     const std::string& identity) {
-    const char* test_policy_template =
-        "[ { \"GUID\": \"wifi_stub\","
-        "    \"Name\": \"wifi_stub\","
-        "    \"Type\": \"WiFi\","
-        "    \"WiFi\": {"
-        "      \"Security\": \"WPA-EAP\","
-        "      \"SSID\": \"wifi_ssid\","
-        "      \"EAP\": {"
-        "        \"Identity\": \"%s\","
-        "        \"Outer\": \"EAP-TLS\","
-        "        \"ClientCertType\": \"Pattern\","
-        "        \"ClientCertPattern\": {"
-        "          \"IssuerCAPEMs\": [ \"%s\" ]"
-        "        }"
-        "      }"
-        "    }"
-        "} ]";
+    const char* test_policy_template = R"(
+        [ { "GUID": "wifi_stub",
+            "Name": "wifi_stub",
+            "Type": "WiFi",
+            "WiFi": {
+              "Security": "WPA-EAP",
+              "SSID": "wifi_ssid",
+              "EAP": {
+                "Identity": "%s",
+                "Outer": "EAP-TLS",
+                "ClientCertType": "Pattern",
+                "ClientCertPattern": {
+                  "IssuerCAPEMs": [ "%s" ]
+                }
+              }
+            }
+        } ])";
     std::string policy_json = base::StringPrintf(
         test_policy_template, identity.c_str(), test_ca_cert_pem_.c_str());
+    ASSERT_NO_FATAL_FAILURE(SetManagedNetworkPolicy(onc_source, policy_json));
+  }
 
-    std::string error;
-    std::unique_ptr<base::Value> policy_value =
-        base::JSONReader::ReadAndReturnError(
-            policy_json, base::JSON_ALLOW_TRAILING_COMMAS, nullptr, &error);
-    ASSERT_TRUE(policy_value) << error;
+  void SetManagedNetworkPolicy(::onc::ONCSource onc_source,
+                               base::StringPiece policy_json) {
+    base::JSONReader::ValueWithError parsed_json =
+        base::JSONReader::ReadAndReturnValueWithError(
+            policy_json, base::JSON_ALLOW_TRAILING_COMMAS);
+    ASSERT_TRUE(parsed_json.value) << parsed_json.error_message;
 
     base::ListValue* policy = nullptr;
-    ASSERT_TRUE(policy_value->GetAsList(&policy));
+    ASSERT_TRUE(parsed_json.value->GetAsList(&policy));
 
     std::string user_hash =
-        onc_source == onc::ONC_SOURCE_USER_POLICY ? kUserHash : "";
+        onc_source == ::onc::ONC_SOURCE_USER_POLICY ? kUserHash : "";
     managed_config_handler_->SetPolicy(
         onc_source, user_hash, *policy,
         base::DictionaryValue() /* no global network config */);
@@ -372,19 +386,35 @@ class ClientCertResolverTest : public testing::Test,
   void GetServiceProperty(const std::string& prop_name,
                           std::string* prop_value) {
     prop_value->clear();
-    const base::DictionaryValue* properties =
+    const base::Value* properties =
         service_test_->GetServiceProperties(kWifiStub);
     if (!properties)
       return;
-    properties->GetStringWithoutPathExpansion(prop_name, prop_value);
+    const std::string* value = properties->FindStringKey(prop_name);
+    if (value)
+      *prop_value = *value;
   }
 
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  // Returns a list of all certificates that are stored on |test_nsscertdb_|'s
+  // private slot.
+  net::ScopedCERTCertificateList ListCertsOnPrivateSlot() {
+    net::ScopedCERTCertificateList certs;
+    base::RunLoop run_loop;
+    test_nsscertdb_->ListCertsInSlot(
+        base::BindOnce(&OnListCertsDone, run_loop.QuitClosure(), &certs),
+        test_nsscertdb_->GetPrivateSlot().get());
+    run_loop.Run();
+    return certs;
+  }
+
+  base::test::TaskEnvironment task_environment_;
   int network_properties_changed_count_ = 0;
   std::string test_cert_id_;
   std::unique_ptr<base::SimpleTestClock> test_clock_;
   std::unique_ptr<ClientCertResolver> client_cert_resolver_;
   NetworkCertLoader* network_cert_loader_ = nullptr;
+  std::unique_ptr<net::NSSCertDatabaseChromeOS> test_nsscertdb_;
+  std::unique_ptr<net::NSSCertDatabaseChromeOS> test_system_nsscertdb_;
 
  private:
   // ClientCertResolver::Observer:
@@ -404,7 +434,6 @@ class ClientCertResolverTest : public testing::Test,
   std::string test_ca_cert_pem_;
   crypto::ScopedTestNSSDB test_nssdb_;
   crypto::ScopedTestNSSDB test_system_nssdb_;
-  std::unique_ptr<net::NSSCertDatabaseChromeOS> test_nsscertdb_;
 
   DISALLOW_COPY_AND_ASSIGN(ClientCertResolverTest);
 };
@@ -413,11 +442,12 @@ TEST_F(ClientCertResolverTest, NoMatchingCertificates) {
   SetupTestCerts("client_1", false /* do not import the issuer */);
   StartNetworkCertLoader();
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   network_properties_changed_count_ = 0;
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY, "");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerPEM(::onc::ONC_SOURCE_USER_POLICY, ""));
+  task_environment_.RunUntilIdle();
 
   // Verify that no client certificate was configured.
   std::string pkcs11_id;
@@ -430,15 +460,16 @@ TEST_F(ClientCertResolverTest, NoMatchingCertificates) {
 TEST_F(ClientCertResolverTest, MatchIssuerCNWithoutIssuerInstalled) {
   SetupTestCerts("client_1", false /* do not import the issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerCN(onc::ONC_SOURCE_USER_POLICY);
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerCN(::onc::ONC_SOURCE_USER_POLICY));
+  task_environment_.RunUntilIdle();
 
   network_properties_changed_count_ = 0;
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -454,17 +485,17 @@ TEST_F(ClientCertResolverTest, MatchSubjectOrgOnBadPrintableStringCert) {
   ASSERT_NO_FATAL_FAILURE(SetupTestCertWithBadPrintableString());
 
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
   ASSERT_NO_FATAL_FAILURE(
       SetupPolicyMatchingSubjectOrgForBadPrintableStringCert(
-          onc::ONC_SOURCE_USER_POLICY));
-  scoped_task_environment_.RunUntilIdle();
+          ::onc::ONC_SOURCE_USER_POLICY));
+  task_environment_.RunUntilIdle();
 
   network_properties_changed_count_ = 0;
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -477,15 +508,16 @@ TEST_F(ClientCertResolverTest, MatchSubjectOrgOnBadPrintableStringCert) {
 TEST_F(ClientCertResolverTest, ResolveOnCertificatesLoaded) {
   SetupTestCerts("client_1", true /* import issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY, "");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerPEM(::onc::ONC_SOURCE_USER_POLICY, ""));
+  task_environment_.RunUntilIdle();
 
   network_properties_changed_count_ = 0;
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -498,15 +530,16 @@ TEST_F(ClientCertResolverTest, ResolveOnCertificatesLoaded) {
 TEST_F(ClientCertResolverTest, ResolveAfterPolicyApplication) {
   SetupTestCerts("client_1", true /* import issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   StartNetworkCertLoader();
   SetupNetworkHandlers();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Policy application will trigger the ClientCertResolver.
   network_properties_changed_count_ = 0;
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY, "");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerPEM(::onc::ONC_SOURCE_USER_POLICY, ""));
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -519,17 +552,18 @@ TEST_F(ClientCertResolverTest, ResolveAfterPolicyApplication) {
 TEST_F(ClientCertResolverTest, ExpiringCertificate) {
   SetupTestCerts("client_1", true /* import issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY, "");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerPEM(::onc::ONC_SOURCE_USER_POLICY, ""));
+  task_environment_.RunUntilIdle();
 
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetWifiState(shill::kStateOnline);
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -543,7 +577,7 @@ TEST_F(ClientCertResolverTest, ExpiringCertificate) {
   network_properties_changed_count_ = 0;
   test_clock_->SetNow(base::Time::Max());
   SetWifiState(shill::kStateOffline);
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   GetServiceProperty(shill::kEapCertIdProperty, &pkcs11_id);
   EXPECT_EQ(std::string(), pkcs11_id);
   EXPECT_EQ(1, network_properties_changed_count_);
@@ -556,17 +590,18 @@ TEST_F(ClientCertResolverTest, ExpiringCertificate) {
 TEST_F(ClientCertResolverTest, SameCertAfterNetworkConnectionStateChanged) {
   SetupTestCerts("client_1", true /* import issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY, "");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerPEM(::onc::ONC_SOURCE_USER_POLICY, ""));
+  task_environment_.RunUntilIdle();
 
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetWifiState(shill::kStateOnline);
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -579,7 +614,7 @@ TEST_F(ClientCertResolverTest, SameCertAfterNetworkConnectionStateChanged) {
   // observers with |network_properties_changed| = true.
   network_properties_changed_count_ = 0;
   SetWifiState(shill::kStateOffline);
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   GetServiceProperty(shill::kEapCertIdProperty, &pkcs11_id);
   EXPECT_EQ(test_cert_id_, pkcs11_id);
   EXPECT_EQ(0, network_properties_changed_count_);
@@ -588,15 +623,17 @@ TEST_F(ClientCertResolverTest, SameCertAfterNetworkConnectionStateChanged) {
 TEST_F(ClientCertResolverTest, UserPolicyUsesSystemToken) {
   SetupTestCertInSystemToken("client_1");
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerCN(onc::ONC_SOURCE_USER_POLICY);
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerCN(::onc::ONC_SOURCE_USER_POLICY));
+  task_environment_.RunUntilIdle();
 
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
-  EXPECT_EQ(1U, network_cert_loader_->system_token_client_certs().size());
+  task_environment_.RunUntilIdle();
+  ASSERT_EQ(1U, network_cert_loader_->client_certs().size());
+  EXPECT_TRUE(network_cert_loader_->client_certs()[0].is_device_wide());
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -608,14 +645,14 @@ TEST_F(ClientCertResolverTest, UserPolicyUsesSystemToken) {
 TEST_F(ClientCertResolverTest, UserPolicyUsesSystemTokenSync) {
   SetupTestCertInSystemToken("client_1");
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   client_cert::ClientCertConfig client_cert_config;
-  SetupCertificateConfigMatchingIssuerCN(onc::ONC_SOURCE_USER_POLICY,
+  SetupCertificateConfigMatchingIssuerCN(::onc::ONC_SOURCE_USER_POLICY,
                                          &client_cert_config);
 
   base::DictionaryValue shill_properties;
-  ClientCertResolver::ResolveCertificatePatternSync(
+  ClientCertResolver::ResolveClientCertificateSync(
       client_cert::CONFIG_TYPE_EAP, client_cert_config, &shill_properties);
   std::string pkcs11_id;
   shill_properties.GetStringWithoutPathExpansion(shill::kEapCertIdProperty,
@@ -626,15 +663,17 @@ TEST_F(ClientCertResolverTest, UserPolicyUsesSystemTokenSync) {
 TEST_F(ClientCertResolverTest, DevicePolicyUsesSystemToken) {
   SetupTestCertInSystemToken("client_1");
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerCN(onc::ONC_SOURCE_USER_POLICY);
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerCN(::onc::ONC_SOURCE_USER_POLICY));
+  task_environment_.RunUntilIdle();
 
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
-  EXPECT_EQ(1U, network_cert_loader_->system_token_client_certs().size());
+  task_environment_.RunUntilIdle();
+  ASSERT_EQ(1U, network_cert_loader_->client_certs().size());
+  EXPECT_TRUE(network_cert_loader_->client_certs()[0].is_device_wide());
 
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -646,14 +685,14 @@ TEST_F(ClientCertResolverTest, DevicePolicyUsesSystemToken) {
 TEST_F(ClientCertResolverTest, DevicePolicyUsesSystemTokenSync) {
   SetupTestCertInSystemToken("client_1");
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   client_cert::ClientCertConfig client_cert_config;
-  SetupCertificateConfigMatchingIssuerCN(onc::ONC_SOURCE_DEVICE_POLICY,
+  SetupCertificateConfigMatchingIssuerCN(::onc::ONC_SOURCE_DEVICE_POLICY,
                                          &client_cert_config);
 
   base::DictionaryValue shill_properties;
-  ClientCertResolver::ResolveCertificatePatternSync(
+  ClientCertResolver::ResolveClientCertificateSync(
       client_cert::CONFIG_TYPE_EAP, client_cert_config, &shill_properties);
   std::string pkcs11_id;
   shill_properties.GetStringWithoutPathExpansion(shill::kEapCertIdProperty,
@@ -664,16 +703,18 @@ TEST_F(ClientCertResolverTest, DevicePolicyUsesSystemTokenSync) {
 TEST_F(ClientCertResolverTest, DevicePolicyDoesNotUseUserToken) {
   SetupTestCerts("client_1", false /* do not import the issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerCN(onc::ONC_SOURCE_DEVICE_POLICY);
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerCN(::onc::ONC_SOURCE_DEVICE_POLICY));
+  task_environment_.RunUntilIdle();
 
   network_properties_changed_count_ = 0;
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
-  EXPECT_EQ(0U, network_cert_loader_->system_token_client_certs().size());
+  task_environment_.RunUntilIdle();
+  ASSERT_EQ(1U, network_cert_loader_->client_certs().size());
+  EXPECT_FALSE(network_cert_loader_->client_certs()[0].is_device_wide());
 
   // Verify that no client certificate was configured.
   std::string pkcs11_id;
@@ -686,14 +727,14 @@ TEST_F(ClientCertResolverTest, DevicePolicyDoesNotUseUserToken) {
 TEST_F(ClientCertResolverTest, DevicePolicyDoesNotUseUserTokenSync) {
   SetupTestCerts("client_1", false /* do not import the issuer */);
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   client_cert::ClientCertConfig client_cert_config;
-  SetupCertificateConfigMatchingIssuerCN(onc::ONC_SOURCE_DEVICE_POLICY,
+  SetupCertificateConfigMatchingIssuerCN(::onc::ONC_SOURCE_DEVICE_POLICY,
                                          &client_cert_config);
 
   base::DictionaryValue shill_properties;
-  ClientCertResolver::ResolveCertificatePatternSync(
+  ClientCertResolver::ResolveClientCertificateSync(
       client_cert::CONFIG_TYPE_EAP, client_cert_config, &shill_properties);
   std::string pkcs11_id;
   shill_properties.GetStringWithoutPathExpansion(shill::kEapCertIdProperty,
@@ -704,16 +745,16 @@ TEST_F(ClientCertResolverTest, DevicePolicyDoesNotUseUserTokenSync) {
 TEST_F(ClientCertResolverTest, PopulateIdentityFromCert) {
   SetupTestCerts("client_3", true /* import issuer */);
   SetupWifi();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY,
-                               "${CERT_SAN_EMAIL}");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(SetupPolicyMatchingIssuerPEM(
+      ::onc::ONC_SOURCE_USER_POLICY, "${CERT_SAN_EMAIL}"));
+  task_environment_.RunUntilIdle();
 
   network_properties_changed_count_ = 0;
   StartNetworkCertLoader();
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the resolver read the subjectAltName email field from the
   // cert, and wrote it into the shill service entry.
@@ -725,9 +766,9 @@ TEST_F(ClientCertResolverTest, PopulateIdentityFromCert) {
   // Verify that after changing the ONC policy to request a variant of the
   // Microsoft Universal Principal Name field instead, the correct value is
   // substituted into the shill service entry.
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY,
-                               "upn-${CERT_SAN_UPN}-suffix");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(SetupPolicyMatchingIssuerPEM(
+      ::onc::ONC_SOURCE_USER_POLICY, "upn-${CERT_SAN_UPN}-suffix"));
+  task_environment_.RunUntilIdle();
 
   GetServiceProperty(shill::kEapIdentityProperty, &identity);
   EXPECT_EQ("upn-santest@ad.corp.example.com-suffix", identity);
@@ -735,9 +776,10 @@ TEST_F(ClientCertResolverTest, PopulateIdentityFromCert) {
 
   // Verify that after changing the ONC policy to request the subject CommonName
   // field, the correct value is substituted into the shill service entry.
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY,
-                               "subject-cn-${CERT_SUBJECT_COMMON_NAME}-suffix");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(SetupPolicyMatchingIssuerPEM(
+      ::onc::ONC_SOURCE_USER_POLICY,
+      "subject-cn-${CERT_SUBJECT_COMMON_NAME}-suffix"));
+  task_environment_.RunUntilIdle();
 
   GetServiceProperty(shill::kEapIdentityProperty, &identity);
   EXPECT_EQ("subject-cn-Client Cert F-suffix", identity);
@@ -752,8 +794,9 @@ TEST_F(ClientCertResolverTest, TestResolveTaskQueued) {
   StartNetworkCertLoader();
   SetupWifi();
   SetupNetworkHandlers();
-  SetupPolicyMatchingIssuerPEM(onc::ONC_SOURCE_USER_POLICY, "");
-  scoped_task_environment_.RunUntilIdle();
+  ASSERT_NO_FATAL_FAILURE(
+      SetupPolicyMatchingIssuerPEM(::onc::ONC_SOURCE_USER_POLICY, ""));
+  task_environment_.RunUntilIdle();
 
   // Pretend that policy was applied, this shall queue a resolving task.
   static_cast<NetworkPolicyObserver*>(client_cert_resolver_.get())
@@ -767,10 +810,10 @@ TEST_F(ClientCertResolverTest, TestResolveTaskQueued) {
   // Pretend that certificates have changed. One resolving task should still be
   // queued.
   static_cast<NetworkCertLoader::Observer*>(client_cert_resolver_.get())
-      ->OnCertificatesLoaded(NetworkCertLoader::Get()->all_certs());
+      ->OnCertificatesLoaded();
   EXPECT_TRUE(client_cert_resolver_->IsAnyResolveTaskRunning());
 
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   EXPECT_FALSE(client_cert_resolver_->IsAnyResolveTaskRunning());
   // Verify that the resolver positively matched the pattern in the policy with
   // the test client cert and configured the network.
@@ -779,4 +822,81 @@ TEST_F(ClientCertResolverTest, TestResolveTaskQueued) {
   EXPECT_EQ(test_cert_id_, pkcs11_id);
 }
 
+// Tests that a ClientCertRef reference is resolved by |ClientCertResolver|.
+// Uses the |CertificateImporterImpl| to import the client certificate from ONC
+// policy, ensuring that setting the cert's key's nickname (in the import step)
+// and evaluating it (in the matching step) work well together.
+TEST_F(ClientCertResolverTest, ResolveClientCertRef) {
+  const char* test_policy_network =
+      R"([ { "GUID": "wifi_stub",
+               "Name": "wifi_stub",
+               "Type": "WiFi",
+               "WiFi": {
+                 "Security": "WPA-EAP",
+                 "SSID": "wifi_ssid",
+                 "EAP": {
+                   "Identity": "TestIdentity",
+                   "Outer": "EAP-TLS",
+                   "ClientCertType": "Ref",
+                   "ClientCertRef": "{some-unique-guid}"
+                 }
+               }
+           } ])";
+
+  SetupWifi();
+  task_environment_.RunUntilIdle();
+  StartNetworkCertLoader();
+  task_environment_.RunUntilIdle();
+  SetupNetworkHandlers();
+  task_environment_.RunUntilIdle();
+
+  // Make sure that expiring client certs don't cause issues.
+  test_clock_->SetNow(base::Time::Min());
+
+  // Apply the network policy.
+  network_properties_changed_count_ = 0;
+  ASSERT_NO_FATAL_FAILURE(SetManagedNetworkPolicy(::onc::ONC_SOURCE_USER_POLICY,
+                                                  test_policy_network));
+  task_environment_.RunUntilIdle();
+
+  // The referenced client cert does not exist yet, so expect that it has not
+  // been resolved.
+  std::string pkcs11_id;
+  GetServiceProperty(shill::kEapCertIdProperty, &pkcs11_id);
+  EXPECT_TRUE(pkcs11_id.empty());
+  EXPECT_EQ(1, network_properties_changed_count_);
+
+  // Now import a client certificate which has the GUID required using the
+  // |CertificateImporterImpl|.
+  auto onc_parsed_certificates = OncParsedCertificatesForPkcs12File(
+      net::GetTestCertsDirectory().AppendASCII("client-empty-password.p12"),
+      "{some-unique-guid}");
+  ASSERT_TRUE(onc_parsed_certificates);
+
+  onc::CertificateImporterImpl importer(
+      task_environment_.GetMainThreadTaskRunner(), test_nsscertdb_.get());
+  base::RunLoop import_loop;
+  importer.ImportClientCertificates(
+      onc_parsed_certificates->client_certificates(),
+      base::BindOnce(&OnImportCompleted, import_loop.QuitClosure()));
+  import_loop.Run();
+  task_environment_.RunUntilIdle();
+
+  // Find the imported cert and get its id.
+  net::ScopedCERTCertificateList private_slot_certs = ListCertsOnPrivateSlot();
+  ASSERT_EQ(1u, private_slot_certs.size());
+  int slot_id = 0;
+  const std::string imported_cert_pkcs11_id =
+      NetworkCertLoader::GetPkcs11IdAndSlotForCert(private_slot_certs[0].get(),
+                                                   &slot_id);
+  std::string imported_cert_formatted_pkcs11_id =
+      base::StringPrintf("%i:%s", slot_id, imported_cert_pkcs11_id.c_str());
+
+  // Verify that the resolver positively matched the pattern in the policy with
+  // the test client cert and configured the network.
+  GetServiceProperty(shill::kEapCertIdProperty, &pkcs11_id);
+  EXPECT_EQ(imported_cert_formatted_pkcs11_id, pkcs11_id);
+  EXPECT_EQ(2, network_properties_changed_count_);
+}
+
 }  // namespace chromeos
