@@ -8,10 +8,10 @@
 #include <stdint.h>
 
 #include "base/bind.h"
+#include "base/logging.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/values.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_manager_client.h"
+#include "chromeos/dbus/shill/shill_manager_client.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 
 namespace chromeos {
@@ -25,25 +25,27 @@ std::string HexToDecimal(std::string hex_str) {
   return std::to_string(std::stoi(hex_str, nullptr, 16));
 }
 
+std::string FindStringOrEmpty(const base::Value& dict,
+                              const base::StringPiece key) {
+  const std::string* val = dict.FindStringKey(key);
+  return val ? *val : std::string();
+}
+
 }  // namespace
 
 GeolocationHandler::GeolocationHandler()
-    : cellular_enabled_(false), wifi_enabled_(false), weak_ptr_factory_(this) {}
+    : cellular_enabled_(false), wifi_enabled_(false) {}
 
 GeolocationHandler::~GeolocationHandler() {
-  ShillManagerClient* manager_client =
-      DBusThreadManager::Get()->GetShillManagerClient();
-  if (manager_client)
-    manager_client->RemovePropertyChangedObserver(this);
+  if (ShillManagerClient::Get())
+    ShillManagerClient::Get()->RemovePropertyChangedObserver(this);
 }
 
 void GeolocationHandler::Init() {
-  ShillManagerClient* manager_client =
-      DBusThreadManager::Get()->GetShillManagerClient();
-  manager_client->GetProperties(
-      base::Bind(&GeolocationHandler::ManagerPropertiesCallback,
-                 weak_ptr_factory_.GetWeakPtr()));
-  manager_client->AddPropertyChangedObserver(this);
+  ShillManagerClient::Get()->GetProperties(
+      base::BindOnce(&GeolocationHandler::ManagerPropertiesCallback,
+                     weak_ptr_factory_.GetWeakPtr()));
+  ShillManagerClient::Get()->AddPropertyChangedObserver(this);
 }
 
 bool GeolocationHandler::GetWifiAccessPoints(
@@ -95,10 +97,13 @@ void GeolocationHandler::OnPropertyChanged(const std::string& key,
 // Private methods
 
 void GeolocationHandler::ManagerPropertiesCallback(
-    DBusMethodCallStatus call_status,
-    const base::DictionaryValue& properties) {
-  const base::Value* value = nullptr;
-  if (properties.Get(shill::kEnabledTechnologiesProperty, &value) && value)
+    base::Optional<base::Value> properties) {
+  if (!properties)
+    return;
+
+  const base::Value* value =
+      properties->FindKey(shill::kEnabledTechnologiesProperty);
+  if (value)
     HandlePropertyChanged(shill::kEnabledTechnologiesProperty, *value);
 }
 
@@ -134,21 +139,20 @@ void GeolocationHandler::HandlePropertyChanged(const std::string& key,
 }
 
 void GeolocationHandler::RequestGeolocationObjects() {
-  DBusThreadManager::Get()->GetShillManagerClient()->GetNetworksForGeolocation(
-      base::Bind(&GeolocationHandler::GeolocationCallback,
-                 weak_ptr_factory_.GetWeakPtr()));
+  ShillManagerClient::Get()->GetNetworksForGeolocation(
+      base::BindOnce(&GeolocationHandler::GeolocationCallback,
+                     weak_ptr_factory_.GetWeakPtr()));
 }
 
 void GeolocationHandler::GeolocationCallback(
-    DBusMethodCallStatus call_status,
-    const base::DictionaryValue& properties) {
-  if (call_status != DBUS_METHOD_CALL_SUCCESS) {
-    LOG(ERROR) << "Failed to get Geolocation data: " << call_status;
+    base::Optional<base::Value> properties) {
+  if (!properties) {
+    LOG(ERROR) << "Failed to get Geolocation data";
     return;
   }
   wifi_access_points_.clear();
   cell_towers_.clear();
-  if (properties.empty())
+  if (properties->DictEmpty())
     return;  // No enabled devices, don't update received time.
 
   // Dictionary<device_type, entry_list>
@@ -160,22 +164,21 @@ void GeolocationHandler::GeolocationCallback(
   //   kGeoCellTowersProperty: [ {kGeoCellIdProperty: cell_id_value, ...}, ... ]
   // }
   for (auto* device_type : kDevicePropertyNames) {
-    if (!properties.HasKey(device_type)) {
+    const base::Value* entry_list = properties->FindKey(device_type);
+    if (!entry_list) {
       continue;
     }
 
-    const base::ListValue* entry_list = nullptr;
-    if (!properties.GetList(device_type, &entry_list)) {
+    if (!entry_list->is_list()) {
       LOG(WARNING) << "Geolocation dictionary value not a List: "
                    << device_type;
       continue;
     }
 
     // List[Dictionary<key, value_str>]
-    for (size_t i = 0; i < entry_list->GetSize(); ++i) {
-      const base::DictionaryValue* entry = nullptr;
-      if (!entry_list->GetDictionary(i, &entry) || !entry) {
-        LOG(WARNING) << "Geolocation list value not a Dictionary: " << i;
+    for (const auto& entry : entry_list->GetList()) {
+      if (!entry.is_dict()) {
+        LOG(WARNING) << "Geolocation list value not a Dictionary";
         continue;
       }
       if (device_type == shill::kGeoWifiAccessPointsProperty) {
@@ -188,68 +191,74 @@ void GeolocationHandler::GeolocationCallback(
   geolocation_received_time_ = base::Time::Now();
 }
 
-void GeolocationHandler::AddAccessPointFromDict(
-    const base::DictionaryValue* entry) {
+void GeolocationHandler::AddAccessPointFromDict(const base::Value& entry) {
   // Docs: developers.google.com/maps/documentation/business/geolocation
   WifiAccessPoint wap;
 
-  std::string age_str;
-  if (entry->GetString(shill::kGeoAgeProperty, &age_str)) {
+  const std::string* age_str = entry.FindStringKey(shill::kGeoAgeProperty);
+  if (age_str) {
     int64_t age_ms;
-    if (base::StringToInt64(age_str, &age_ms)) {
+    if (base::StringToInt64(*age_str, &age_ms)) {
       wap.timestamp =
           base::Time::Now() - base::TimeDelta::FromMilliseconds(age_ms);
     }
   }
-  entry->GetString(shill::kGeoMacAddressProperty, &wap.mac_address);
 
-  std::string strength_str;
-  if (entry->GetString(shill::kGeoSignalStrengthProperty, &strength_str))
-    base::StringToInt(strength_str, &wap.signal_strength);
+  wap.mac_address = FindStringOrEmpty(entry, shill::kGeoMacAddressProperty);
 
-  std::string signal_str;
-  if (entry->GetString(shill::kGeoSignalToNoiseRatioProperty, &signal_str)) {
-    base::StringToInt(signal_str, &wap.signal_to_noise);
+  const std::string* strength_str =
+      entry.FindStringKey(shill::kGeoSignalStrengthProperty);
+  if (strength_str) {
+    base::StringToInt(*strength_str, &wap.signal_strength);
   }
 
-  std::string channel_str;
-  if (entry->GetString(shill::kGeoChannelProperty, &channel_str))
-    base::StringToInt(channel_str, &wap.channel);
+  const std::string* signal_str =
+      entry.FindStringKey(shill::kGeoSignalToNoiseRatioProperty);
+  if (signal_str) {
+    base::StringToInt(*signal_str, &wap.signal_to_noise);
+  }
+
+  const std::string* channel_str =
+      entry.FindStringKey(shill::kGeoChannelProperty);
+  if (channel_str) {
+    base::StringToInt(*channel_str, &wap.channel);
+  }
 
   wifi_access_points_.push_back(wap);
 }
 
-void GeolocationHandler::AddCellTowerFromDict(
-    const base::DictionaryValue* entry) {
+void GeolocationHandler::AddCellTowerFromDict(const base::Value& entry) {
   // Docs: developers.google.com/maps/documentation/business/geolocation
 
   // Create object.
   CellTower ct;
 
   // Read time fields into object.
-  std::string age_str;
-  if (entry->GetString(shill::kGeoAgeProperty, &age_str)) {
+  const std::string* age_str = entry.FindStringKey(shill::kGeoAgeProperty);
+  if (age_str) {
     int64_t age_ms;
-    if (base::StringToInt64(age_str, &age_ms)) {
+    if (base::StringToInt64(*age_str, &age_ms)) {
       ct.timestamp =
           base::Time::Now() - base::TimeDelta::FromMilliseconds(age_ms);
     }
   }
 
   // Read hex fields into object.
-  std::string hex_cell_id;
-  if (entry->GetString(shill::kGeoCellIdProperty, &hex_cell_id)) {
-    ct.ci = HexToDecimal(hex_cell_id);
+  const std::string* hex_cell_id =
+      entry.FindStringKey(shill::kGeoCellIdProperty);
+  if (hex_cell_id) {
+    ct.ci = HexToDecimal(*hex_cell_id);
   }
 
-  std::string hex_lac;
-  if (entry->GetString(shill::kGeoLocationAreaCodeProperty, &hex_lac)) {
-    ct.lac = HexToDecimal(hex_lac);
+  const std::string* hex_lac =
+      entry.FindStringKey(shill::kGeoLocationAreaCodeProperty);
+  if (hex_lac) {
+    ct.lac = HexToDecimal(*hex_lac);
   }
 
   // Read decimal fields into object.
-  entry->GetString(shill::kGeoMobileCountryCodeProperty, &ct.mcc);
-  entry->GetString(shill::kGeoMobileNetworkCodeProperty, &ct.mnc);
+  ct.mcc = FindStringOrEmpty(entry, shill::kGeoMobileCountryCodeProperty);
+  ct.mnc = FindStringOrEmpty(entry, shill::kGeoMobileNetworkCodeProperty);
 
   // Add new object to vector.
   cell_towers_.push_back(ct);
