@@ -6,13 +6,14 @@
 #define CHROMEOS_NETWORK_MANAGED_NETWORK_CONFIGURATION_HANDLER_H_
 
 #include <map>
+#include <memory>
 #include <string>
 
 #include "base/callback.h"
 #include "base/compiler_specific.h"
+#include "base/component_export.h"
 #include "base/macros.h"
 #include "base/observer_list.h"
-#include "chromeos/chromeos_export.h"
 #include "chromeos/network/network_handler.h"
 #include "chromeos/network/network_handler_callbacks.h"
 #include "components/onc/onc_constants.h"
@@ -20,12 +21,15 @@
 namespace base {
 class DictionaryValue;
 class ListValue;
-class Value;
-}
+}  // namespace base
 
 namespace chromeos {
 
+class NetworkConfigurationHandler;
+class NetworkDeviceHandler;
 class NetworkPolicyObserver;
+class NetworkProfileHandler;
+class NetworkStateHandler;
 
 // The ManagedNetworkConfigurationHandler class is used to create and configure
 // networks in ChromeOS using ONC and takes care of network policies.
@@ -51,7 +55,7 @@ class NetworkPolicyObserver;
 // error, including a symbolic name for the error and often some error message
 // that is suitable for logging. None of the error message text is meant for
 // user consumption.
-class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) ManagedNetworkConfigurationHandler {
  public:
   using GuidToPolicyMap =
       std::map<std::string, std::unique_ptr<base::DictionaryValue>>;
@@ -64,11 +68,9 @@ class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
   // Provides the properties of the network with |service_path| to |callback|.
   // |userhash| is used to set the "Source" property. If not provided then
   // user policies will be ignored.
-  virtual void GetProperties(
-      const std::string& userhash,
-      const std::string& service_path,
-      const network_handler::DictionaryResultCallback& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  virtual void GetProperties(const std::string& userhash,
+                             const std::string& service_path,
+                             network_handler::PropertiesCallback callback) = 0;
 
   // Provides the managed properties of the network with |service_path| to
   // |callback|. |userhash| is used to ensure that the user's policy is
@@ -77,25 +79,17 @@ class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
   virtual void GetManagedProperties(
       const std::string& userhash,
       const std::string& service_path,
-      const network_handler::DictionaryResultCallback& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+      network_handler::PropertiesCallback callback) = 0;
 
   // Sets the user's settings of an already configured network with
   // |service_path|. A network can be initially configured by calling
   // CreateConfiguration or if it is managed by a policy. The given properties
   // will be merged with the existing settings, and it won't clear any existing
   // properties.
-  virtual void SetProperties(
-      const std::string& service_path,
-      const base::DictionaryValue& user_settings,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
-
-  virtual void SetManagerProperty(
-      const std::string& property_name,
-      const base::Value& value,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  virtual void SetProperties(const std::string& service_path,
+                             const base::DictionaryValue& user_settings,
+                             base::OnceClosure callback,
+                             network_handler::ErrorCallback error_callback) = 0;
 
   // Initially configures an unconfigured network with the given user settings
   // and returns the new identifier to |callback| if successful. Fails if the
@@ -105,16 +99,16 @@ class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
   virtual void CreateConfiguration(
       const std::string& userhash,
       const base::DictionaryValue& properties,
-      const network_handler::ServiceResultCallback& callback,
-      const network_handler::ErrorCallback& error_callback) const = 0;
+      network_handler::ServiceResultCallback callback,
+      network_handler::ErrorCallback error_callback) const = 0;
 
   // Removes the user's configuration from the network with |service_path|. The
   // network may still show up in the visible networks after this, but no user
   // configuration will remain. If it was managed, it will still be configured.
   virtual void RemoveConfiguration(
       const std::string& service_path,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) const = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) const = 0;
 
   // Removes the user's configuration from the network with |service_path| in
   // the network's active network profile.
@@ -122,8 +116,8 @@ class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
   // configuration is only removed from a single network profile.
   virtual void RemoveConfigurationFromCurrentProfile(
       const std::string& service_path,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) const = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) const = 0;
 
   // Only to be called by NetworkConfigurationUpdater or from tests. Sets
   // |network_configs_onc| and |global_network_config| as the current policy of
@@ -165,6 +159,18 @@ class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
       const std::string& profile_path,
       ::onc::ONCSource* onc_source) const = 0;
 
+  // Returns true if the network with |guid| is configured by device or user
+  // policy for profile |profile_path|.
+  virtual bool IsNetworkConfiguredByPolicy(
+      const std::string& guid,
+      const std::string& profile_path) const = 0;
+
+  // Returns true if the configuration of the network with |guid| is not
+  // managed by policy for profile with |profile_path| and thus can be removed.
+  virtual bool CanRemoveNetworkConfig(
+      const std::string& guid,
+      const std::string& profile_path) const = 0;
+
   // Return true if the AllowOnlyPolicyNetworksToConnect policy is enabled.
   virtual bool AllowOnlyPolicyNetworksToConnect() const = 0;
 
@@ -175,8 +181,16 @@ class CHROMEOS_EXPORT ManagedNetworkConfigurationHandler {
   // Return true if the AllowOnlyPolicyNetworksToAutoconnect policy is enabled.
   virtual bool AllowOnlyPolicyNetworksToAutoconnect() const = 0;
 
-  // Return the list of blacklisted WiFi networks (identified by HexSSIDs).
-  virtual std::vector<std::string> GetBlacklistedHexSSIDs() const = 0;
+  // Return the list of blocked WiFi networks (identified by HexSSIDs).
+  virtual std::vector<std::string> GetBlockedHexSSIDs() const = 0;
+
+  static std::unique_ptr<ManagedNetworkConfigurationHandler>
+  InitializeForTesting(
+      NetworkStateHandler* network_state_handler,
+      NetworkProfileHandler* network_profile_handler,
+      NetworkDeviceHandler* network_device_handler,
+      NetworkConfigurationHandler* network_configuration_handler,
+      UIProxyConfigService* ui_proxy_config_service);
 
  private:
   DISALLOW_ASSIGN(ManagedNetworkConfigurationHandler);
