@@ -11,14 +11,8 @@
 #include <string>
 
 #include "base/compiler_specific.h"
+#include "base/component_export.h"
 #include "base/macros.h"
-#include "chromeos/chromeos_export.h"
-
-namespace ash {
-namespace network_icon {
-class NetworkIconTest;
-}  // namespace network_icon
-}  // namesapce ash
 
 namespace base {
 class Value;
@@ -36,7 +30,7 @@ class NetworkListSorterTest;
 
 // Base class for states managed by NetworkStateManger which are associated
 // with a Shill path (e.g. service path or device path).
-class CHROMEOS_EXPORT ManagedState {
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) ManagedState {
  public:
   enum ManagedType {
     MANAGED_TYPE_NETWORK,
@@ -50,10 +44,12 @@ class CHROMEOS_EXPORT ManagedState {
   static std::unique_ptr<ManagedState> Create(ManagedType type,
                                               const std::string& path);
 
-  // Returns the specific class pointer if this is the correct type, or
-  // NULL if it is not.
+  // Returns the specific class pointer if this is the correct type, or null if
+  // it is not.
   NetworkState* AsNetworkState();
+  const NetworkState* AsNetworkState() const;
   DeviceState* AsDeviceState();
+  const DeviceState* AsDeviceState() const;
 
   // Called by NetworkStateHandler when a property was received. The return
   // value indicates if the state changed and is used to reduce the number of
@@ -78,6 +74,10 @@ class CHROMEOS_EXPORT ManagedState {
   // properties are included.
   virtual void GetStateProperties(base::Value* dictionary) const;
 
+  // Returns true if a state is "Active". For networks that means connected,
+  // connecting, or activating. Devices are always "active".
+  virtual bool IsActive() const = 0;
+
   ManagedType managed_type() const { return managed_type_; }
   const std::string& path() const { return path_; }
   const std::string& name() const { return name_; }
@@ -89,6 +89,9 @@ class CHROMEOS_EXPORT ManagedState {
     update_requested_ = update_requested;
   }
 
+  void set_path_for_testing(const std::string& path) { path_ = path; }
+  void set_type_for_testing(const std::string& type) { type_ = type; }
+
   // Returns true if |type_| matches |pattern|.
   bool Matches(const NetworkTypePattern& pattern) const;
 
@@ -120,9 +123,8 @@ class CHROMEOS_EXPORT ManagedState {
   void set_type(const std::string& type) { type_ = type; }
 
  private:
-  friend class NetworkChangeNotifierChromeosUpdateTest;
   friend class NetworkStateHandler;
-  friend class ash::network_icon::NetworkIconTest;
+  friend class NetworkStateTestHelper;
   friend class chromeos::tether::NetworkListSorterTest;
 
   ManagedType managed_type_;
@@ -135,10 +137,10 @@ class CHROMEOS_EXPORT ManagedState {
   std::string type_;  // shill::kTypeProperty
 
   // Set to true when the an update has been received.
-  bool update_received_;
+  bool update_received_ = false;
 
   // Tracks when an update has been requested.
-  bool update_requested_;
+  bool update_requested_ = false;
 
   DISALLOW_COPY_AND_ASSIGN(ManagedState);
 };
