@@ -6,14 +6,16 @@
 
 #include <cert.h>
 #include <string>
+#include <utility>
 
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/location.h"
+#include "base/logging.h"
 #include "base/metrics/histogram_macros.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_service_client.h"
+#include "chromeos/dbus/shill/shill_service_client.h"
 #include "chromeos/network/client_cert_util.h"
+#include "chromeos/network/network_event_log.h"
 #include "chromeos/network/network_handler_callbacks.h"
 #include "chromeos/network/network_state.h"
 #include "chromeos/network/network_state_handler.h"
@@ -37,10 +39,9 @@ namespace chromeos {
 class NetworkCertMigrator::MigrationTask
     : public base::RefCounted<MigrationTask> {
  public:
-  MigrationTask(const net::ScopedCERTCertificateList& certs,
+  MigrationTask(net::ScopedCERTCertificateList certs,
                 const base::WeakPtr<NetworkCertMigrator>& cert_migrator)
-      : certs_(net::x509_util::DupCERTCertificateList(certs)),
-        cert_migrator_(cert_migrator) {}
+      : certs_(std::move(certs)), cert_migrator_(cert_migrator) {}
 
   void Run(const NetworkStateHandler::NetworkStateList& networks) {
     // Request properties for each network that could be configured with a
@@ -62,24 +63,28 @@ class NetworkCertMigrator::MigrationTask
         continue;
       }
 
-      DBusThreadManager::Get()->GetShillServiceClient()->GetProperties(
+      ShillServiceClient::Get()->GetProperties(
           dbus::ObjectPath(service_path),
-          base::Bind(&network_handler::GetPropertiesCallback,
-                     base::Bind(&MigrationTask::MigrateNetwork, this),
-                     network_handler::ErrorCallback(),
-                     service_path));
+          base::BindOnce(&MigrationTask::MigrateNetwork, this, service_path));
     }
   }
 
   void MigrateNetwork(const std::string& service_path,
-                      const base::DictionaryValue& properties) {
+                      base::Optional<base::Value> properties) {
     if (!cert_migrator_) {
       VLOG(2) << "NetworkCertMigrator already destroyed. Aborting migration.";
       return;
     }
 
+    if (!properties) {
+      NET_LOG(ERROR) << "GetProperties failed: " << NetworkPathId(service_path);
+      return;
+    }
+
     base::DictionaryValue new_properties;
-    MigrateClientCertProperties(service_path, properties, &new_properties);
+    MigrateClientCertProperties(service_path,
+                                base::Value::AsDictionaryValue(*properties),
+                                &new_properties);
 
     if (new_properties.empty())
       return;
@@ -145,9 +150,9 @@ class NetworkCertMigrator::MigrationTask
 
   void SendPropertiesToShill(const std::string& service_path,
                              const base::DictionaryValue& properties) {
-    DBusThreadManager::Get()->GetShillServiceClient()->SetProperties(
+    ShillServiceClient::Get()->SetProperties(
         dbus::ObjectPath(service_path), properties, base::DoNothing(),
-        base::Bind(&LogError, service_path));
+        base::BindOnce(&LogError, service_path));
   }
 
   static void LogError(const std::string& service_path,
@@ -175,10 +180,7 @@ class NetworkCertMigrator::MigrationTask
   base::WeakPtr<NetworkCertMigrator> cert_migrator_;
 };
 
-NetworkCertMigrator::NetworkCertMigrator()
-    : network_state_handler_(nullptr),
-      weak_ptr_factory_(this) {
-}
+NetworkCertMigrator::NetworkCertMigrator() : network_state_handler_(nullptr) {}
 
 NetworkCertMigrator::~NetworkCertMigrator() {
   network_state_handler_->RemoveObserver(this, FROM_HERE);
@@ -203,8 +205,10 @@ void NetworkCertMigrator::NetworkListChanged() {
   // Run the migration process to fix missing or incorrect slot ids of client
   // certificates.
   VLOG(2) << "Start certificate migration of network configurations.";
-  scoped_refptr<MigrationTask> helper(new MigrationTask(
-      NetworkCertLoader::Get()->all_certs(), weak_ptr_factory_.GetWeakPtr()));
+  scoped_refptr<MigrationTask> helper(base::MakeRefCounted<MigrationTask>(
+      NetworkCertLoader::GetAllCertsFromNetworkCertList(
+          NetworkCertLoader::Get()->client_certs()),
+      weak_ptr_factory_.GetWeakPtr()));
   NetworkStateHandler::NetworkStateList networks;
   network_state_handler_->GetNetworkListByType(
       NetworkTypePattern::Default(),
@@ -215,8 +219,7 @@ void NetworkCertMigrator::NetworkListChanged() {
   helper->Run(networks);
 }
 
-void NetworkCertMigrator::OnCertificatesLoaded(
-    const net::ScopedCERTCertificateList& cert_list) {
+void NetworkCertMigrator::OnCertificatesLoaded() {
   NetworkListChanged();
 }
 
