@@ -6,10 +6,11 @@
 #define CHROMEOS_NETWORK_NETWORK_DEVICE_HANDLER_H_
 
 #include <string>
+#include <vector>
 
 #include "base/callback.h"
+#include "base/component_export.h"
 #include "base/macros.h"
-#include "chromeos/chromeos_export.h"
 #include "chromeos/network/network_handler_callbacks.h"
 
 namespace base {
@@ -22,6 +23,8 @@ class IPEndPoint;
 
 namespace chromeos {
 
+class NetworkStateHandler;
+
 // The NetworkDeviceHandler class allows making device specific requests on a
 // ChromeOS network device. All calls are asynchronous and interact with the
 // Shill device API. No calls will block on DBus calls.
@@ -35,7 +38,7 @@ namespace chromeos {
 // |error_callback| will be called with information about the error, including a
 // symbolic name for the error and often some error message that is suitable for
 // logging. None of the error message text is meant for user consumption.
-class CHROMEOS_EXPORT NetworkDeviceHandler {
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) NetworkDeviceHandler {
  public:
   // Constants for |error_name| from |error_callback|.
   static const char kErrorDeviceMissing[];
@@ -51,12 +54,11 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   NetworkDeviceHandler();
   virtual ~NetworkDeviceHandler();
 
-  // Gets the properties of the device with id |device_path|. See note on
-  // |callback| and |error_callback|, in class description above.
+  // Invokes |callback| with the properties for the device matching
+  // |device_path| on success, or nullopt on failure.
   virtual void GetDeviceProperties(
       const std::string& device_path,
-      const network_handler::DictionaryResultCallback& callback,
-      const network_handler::ErrorCallback& error_callback) const = 0;
+      network_handler::ResultCallback callback) const = 0;
 
   // Sets the value of property |name| on device with id |device_path| to
   // |value|. This function provides a generic setter to be used by the UI or
@@ -66,16 +68,8 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
       const std::string& device_path,
       const std::string& property_name,
       const base::Value& value,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
-
-  // Requests a refresh of the IP configuration for the device specified by
-  // |device_path| if it exists. This will apply any newly configured
-  // properties and renew the DHCP lease.
-  virtual void RequestRefreshIPConfigs(
-      const std::string& device_path,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
 
   // Tells the device specified by |device_path| to register to the cellular
   // network with id |network_id|. If |network_id| is empty then registration
@@ -86,23 +80,8 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   virtual void RegisterCellularNetwork(
       const std::string& device_path,
       const std::string& network_id,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
-
-  // Tells the device to set the modem carrier firmware, as specified by
-  // |carrier|.
-  //
-  // See note on |callback| and |error_callback| in the class description
-  // above. The operation will fail if:
-  //    - Device |device_path| could not be found.
-  //    - |carrier| doesn't match one of the supported carriers, as reported by
-  //    - Shill.
-  //    - Operation is not supported by the device.
-  virtual void SetCarrier(
-      const std::string& device_path,
-      const std::string& carrier,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
 
   // SIM PIN/PUK methods
 
@@ -121,12 +100,11 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   //
   // This method applies to Cellular devices only. The call will fail with a
   // "not-supported" error if called on a non-cellular device.
-  virtual void RequirePin(
-      const std::string& device_path,
-      bool require_pin,
-      const std::string& pin,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  virtual void RequirePin(const std::string& device_path,
+                          bool require_pin,
+                          const std::string& pin,
+                          base::OnceClosure callback,
+                          network_handler::ErrorCallback error_callback) = 0;
 
   // Sends the PIN code |pin| to the device |device_path|.
   //
@@ -138,11 +116,10 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   //
   // This method applies to Cellular devices only. The call will fail with a
   // "not-supported" error if called on a non-cellular device.
-  virtual void EnterPin(
-      const std::string& device_path,
-      const std::string& pin,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  virtual void EnterPin(const std::string& device_path,
+                        const std::string& pin,
+                        base::OnceClosure callback,
+                        network_handler::ErrorCallback error_callback) = 0;
 
   // Sends the PUK code |puk| to the SIM to unblock a blocked SIM. On success,
   // the SIM will be unblocked and its PIN code will be set to |pin|.
@@ -154,12 +131,11 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   //
   // This method applies to Cellular devices only. The call will fail with a
   // "not-supported" error if called on a non-cellular device.
-  virtual void UnblockPin(
-      const std::string& device_path,
-      const std::string& puk,
-      const std::string& new_pin,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  virtual void UnblockPin(const std::string& device_path,
+                          const std::string& puk,
+                          const std::string& new_pin,
+                          base::OnceClosure callback,
+                          network_handler::ErrorCallback error_callback) = 0;
 
   // Tells the device to change the PIN code used to unlock a locked SIM card.
   //
@@ -172,12 +148,11 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   //
   // This method applies to Cellular devices only. The call will fail with a
   // "not-supported" error if called on a non-cellular device.
-  virtual void ChangePin(
-      const std::string& device_path,
-      const std::string& old_pin,
-      const std::string& new_pin,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  virtual void ChangePin(const std::string& device_path,
+                         const std::string& old_pin,
+                         const std::string& new_pin,
+                         base::OnceClosure callback,
+                         network_handler::ErrorCallback error_callback) = 0;
 
   // Enables/disables roaming of all cellular devices. This happens
   // asychronously in the background and applies also to devices which become
@@ -188,40 +163,46 @@ class CHROMEOS_EXPORT NetworkDeviceHandler {
   // which become available in the future.
   virtual void SetMACAddressRandomizationEnabled(bool enabled) = 0;
 
-  // Attempts to enable or disable TDLS for the specified IP or MAC address for
-  // the active wifi device.
-  virtual void SetWifiTDLSEnabled(
-      const std::string& ip_or_mac_address,
-      bool enabled,
-      const network_handler::StringResultCallback& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
-
-  // Returns the TDLS status for the specified IP or MAC address for
-  // the active wifi device.
-  virtual void GetWifiTDLSStatus(
-      const std::string& ip_or_mac_address,
-      const network_handler::StringResultCallback& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+  // Sets up USB Ethernet MAC address source. This applies to primary enabled
+  // USB Ethernet device.
+  virtual void SetUsbEthernetMacAddressSource(const std::string& source) = 0;
 
   // Adds |ip_endpoint| to the list of tcp connections that the wifi device
   // should monitor to wake the system from suspend.
   virtual void AddWifiWakeOnPacketConnection(
       const net::IPEndPoint& ip_endpoint,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
+
+  // Adds |types| to the list of packet types that the device should monitor to
+  // wake the system from suspend.
+  virtual void AddWifiWakeOnPacketOfTypes(
+      const std::vector<std::string>& types,
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
 
   // Removes |ip_endpoint| from the list of tcp connections that the wifi device
   // should monitor to wake the system from suspend.
   virtual void RemoveWifiWakeOnPacketConnection(
       const net::IPEndPoint& ip_endpoint,
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
+
+  // Removes |types| from the list of packet types that the device should
+  // monitor to wake the system from suspend.
+  virtual void RemoveWifiWakeOnPacketOfTypes(
+      const std::vector<std::string>& types,
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
 
   // Clears the list of tcp connections that the wifi device should monitor to
   // wake the system from suspend.
   virtual void RemoveAllWifiWakeOnPacketConnections(
-      const base::Closure& callback,
-      const network_handler::ErrorCallback& error_callback) = 0;
+      base::OnceClosure callback,
+      network_handler::ErrorCallback error_callback) = 0;
+
+  static std::unique_ptr<NetworkDeviceHandler> InitializeForTesting(
+      NetworkStateHandler* network_state_handler);
 
  private:
   DISALLOW_COPY_AND_ASSIGN(NetworkDeviceHandler);
