@@ -9,7 +9,7 @@
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/values.h"
-#include "components/device_event_log/device_event_log.h"
+#include "chromeos/network/network_event_log.h"
 
 namespace {
 
@@ -35,29 +35,24 @@ const char kDbusErrorName[] = "dbusErrorName";
 const char kDbusErrorMessage[] = "dbusErrorMessage";
 const char kPath[] = "path";
 
-base::DictionaryValue* CreateErrorData(const std::string& path,
-                                       const std::string& error_name,
-                                       const std::string& error_detail) {
-  return CreateDBusErrorData(path, error_name, error_detail, "", "");
-}
-
-void RunErrorCallback(const ErrorCallback& error_callback,
+void RunErrorCallback(ErrorCallback error_callback,
                       const std::string& path,
                       const std::string& error_name,
                       const std::string& error_detail) {
   if (error_callback.is_null())
     return;
-  error_callback.Run(error_name, base::WrapUnique(CreateErrorData(
-                                     path, error_name, error_detail)));
+  std::move(error_callback)
+      .Run(error_name,
+           CreateDBusErrorData(path, error_name, error_detail, "", ""));
 }
 
-base::DictionaryValue* CreateDBusErrorData(
+std::unique_ptr<base::DictionaryValue> CreateDBusErrorData(
     const std::string& path,
     const std::string& error_name,
     const std::string& error_detail,
     const std::string& dbus_error_name,
     const std::string& dbus_error_message) {
-  base::DictionaryValue* error_data(new base::DictionaryValue);
+  auto error_data = std::make_unique<base::DictionaryValue>();
   error_data->SetString(kErrorName, error_name);
   error_data->SetString(kErrorDetail, error_detail);
   error_data->SetString(kDbusErrorName, dbus_error_name);
@@ -69,7 +64,7 @@ base::DictionaryValue* CreateDBusErrorData(
 
 void ShillErrorCallbackFunction(const std::string& error_name,
                                 const std::string& path,
-                                const ErrorCallback& error_callback,
+                                ErrorCallback error_callback,
                                 const std::string& dbus_error_name,
                                 const std::string& dbus_error_message) {
   std::string detail = error_name + ": ";
@@ -85,24 +80,10 @@ void ShillErrorCallbackFunction(const std::string& error_name,
 
   if (error_callback.is_null())
     return;
-  std::unique_ptr<base::DictionaryValue> error_data(CreateDBusErrorData(
-      path, error_name, detail, dbus_error_name, dbus_error_message));
-  error_callback.Run(error_name, std::move(error_data));
-}
-
-void GetPropertiesCallback(const DictionaryResultCallback& callback,
-                           const ErrorCallback& error_callback,
-                           const std::string& path,
-                           DBusMethodCallStatus call_status,
-                           const base::DictionaryValue& value) {
-  if (call_status != DBUS_METHOD_CALL_SUCCESS) {
-    NET_LOG(ERROR) << "GetProperties failed: " << path
-                   << " Status: " << call_status;
-    RunErrorCallback(
-        error_callback, path, kDBusFailedError, kDBusFailedErrorMessage);
-  } else if (!callback.is_null()) {
-    callback.Run(path, value);
-  }
+  std::move(error_callback)
+      .Run(error_name,
+           CreateDBusErrorData(path, error_name, detail, dbus_error_name,
+                               dbus_error_message));
 }
 
 }  // namespace network_handler
