@@ -8,9 +8,8 @@
 #include <string>
 #include <vector>
 
-#include "base/basictypes.h"
 #include "base/callback.h"
-#include "chromeos/chromeos_export.h"
+#include "base/component_export.h"
 
 namespace chromeos {
 
@@ -27,7 +26,7 @@ enum IPConfigType {
 };
 
 // IP Configuration.
-struct CHROMEOS_EXPORT NetworkIPConfig {
+struct COMPONENT_EXPORT(CHROMEOS_NETWORK) NetworkIPConfig {
   NetworkIPConfig(const std::string& device_path, IPConfigType type,
                   const std::string& address, const std::string& netmask,
                   const std::string& gateway, const std::string& name_servers);
@@ -45,14 +44,6 @@ struct CHROMEOS_EXPORT NetworkIPConfig {
 
 typedef std::vector<NetworkIPConfig> NetworkIPConfigVector;
 
-// Used to return the list of IP configs and hardware address from an
-// asynchronous call to Shill. The hardware address is usually a MAC address
-// like "0011AA22BB33". |hardware_address| will be an empty string, if no
-// hardware address is found.
-typedef base::Callback<void(const NetworkIPConfigVector& ip_configs,
-                            const std::string& hardware_address)>
-    NetworkGetIPConfigsCallback;
-
 }  // namespace chromeos
 
 #endif  // CHROMEOS_NETWORK_NETWORK_IP_CONFIG_H_
