@@ -4,14 +4,17 @@
 
 #include "chromeos/network/network_profile_handler.h"
 
+#include <stddef.h>
+
 #include <algorithm>
 
 #include "base/bind.h"
+#include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_manager_client.h"
-#include "chromeos/dbus/shill_profile_client.h"
+#include "chromeos/dbus/shill/shill_manager_client.h"
+#include "chromeos/dbus/shill/shill_profile_client.h"
 #include "chromeos/network/network_profile_observer.h"
 #include "dbus/object_path.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
@@ -68,15 +71,13 @@ void NetworkProfileHandler::RemoveObserver(NetworkProfileObserver* observer) {
 }
 
 void NetworkProfileHandler::GetManagerPropertiesCallback(
-    DBusMethodCallStatus call_status,
-    const base::DictionaryValue& properties) {
-  if (DBUS_METHOD_CALL_FAILURE) {
+    base::Optional<base::Value> properties) {
+  if (!properties) {
     LOG(ERROR) << "Error when requesting manager properties.";
     return;
   }
 
-  const base::Value* profiles = NULL;
-  properties.GetWithoutPathExpansion(shill::kProfilesProperty, &profiles);
+  const base::Value* profiles = properties->FindKey(shill::kProfilesProperty);
   if (!profiles) {
     LOG(ERROR) << "Manager properties returned from Shill don't contain "
                << "the field " << shill::kProfilesProperty;
@@ -111,42 +112,50 @@ void NetworkProfileHandler::OnPropertyChanged(const std::string& name,
     }
   }
 
-  for (std::vector<std::string>::const_iterator it =
-           removed_profile_paths.begin();
-       it != removed_profile_paths.end(); ++it) {
-    RemoveProfile(*it);
+  for (const std::string& profile_path : removed_profile_paths) {
+    RemoveProfile(profile_path);
+    // Also stop pending creations of this profile.
+    pending_profile_creations_.erase(profile_path);
   }
 
   for (std::vector<std::string>::const_iterator it = new_profile_paths.begin();
        it != new_profile_paths.end(); ++it) {
     // Skip known profiles. The associated userhash should never change.
-    if (GetProfileForPath(*it))
+    if (GetProfileForPath(*it) || pending_profile_creations_.count(*it) > 0)
       continue;
+    pending_profile_creations_.insert(*it);
 
     VLOG(2) << "Requesting properties of profile path " << *it << ".";
-    DBusThreadManager::Get()->GetShillProfileClient()->GetProperties(
+    ShillProfileClient::Get()->GetProperties(
         dbus::ObjectPath(*it),
-        base::Bind(&NetworkProfileHandler::GetProfilePropertiesCallback,
-                   weak_ptr_factory_.GetWeakPtr(),
-                   *it),
-        base::Bind(&LogProfileRequestError, *it));
+        base::BindOnce(&NetworkProfileHandler::GetProfilePropertiesCallback,
+                       weak_ptr_factory_.GetWeakPtr(), *it),
+        base::BindOnce(&LogProfileRequestError, *it));
   }
 }
 
 void NetworkProfileHandler::GetProfilePropertiesCallback(
     const std::string& profile_path,
-    const base::DictionaryValue& properties) {
-  std::string userhash;
-  properties.GetStringWithoutPathExpansion(shill::kUserHashProperty, &userhash);
+    base::Value properties) {
+  if (pending_profile_creations_.erase(profile_path) == 0) {
+    VLOG(1) << "Ignore received properties, profile was removed.";
+    return;
+  }
+  if (GetProfileForPath(profile_path)) {
+    VLOG(1) << "Ignore received properties, profile is already created.";
+    return;
+  }
+  const std::string* userhash =
+      properties.FindStringKey(shill::kUserHashProperty);
 
-  AddProfile(NetworkProfile(profile_path, userhash));
+  AddProfile(NetworkProfile(profile_path, userhash ? *userhash : ""));
 }
 
 void NetworkProfileHandler::AddProfile(const NetworkProfile& profile) {
   VLOG(2) << "Adding profile " << profile.ToDebugString() << ".";
   profiles_.push_back(profile);
-  FOR_EACH_OBSERVER(NetworkProfileObserver, observers_,
-                    OnProfileAdded(profiles_.back()));
+  for (auto& observer : observers_)
+    observer.OnProfileAdded(profiles_.back());
 }
 
 void NetworkProfileHandler::RemoveProfile(const std::string& profile_path) {
@@ -157,8 +166,8 @@ void NetworkProfileHandler::RemoveProfile(const std::string& profile_path) {
     return;
   NetworkProfile profile = *found;
   profiles_.erase(found);
-  FOR_EACH_OBSERVER(NetworkProfileObserver, observers_,
-                    OnProfileRemoved(profile));
+  for (auto& observer : observers_)
+    observer.OnProfileRemoved(profile);
 }
 
 const NetworkProfile* NetworkProfileHandler::GetProfileForPath(
@@ -193,23 +202,27 @@ const NetworkProfile* NetworkProfileHandler::GetDefaultUserProfile() const {
   return NULL;
 }
 
-NetworkProfileHandler::NetworkProfileHandler()
-    : weak_ptr_factory_(this) {
-}
+NetworkProfileHandler::NetworkProfileHandler() {}
 
 void NetworkProfileHandler::Init() {
-  DBusThreadManager::Get()->GetShillManagerClient()->
-      AddPropertyChangedObserver(this);
+  ShillManagerClient::Get()->AddPropertyChangedObserver(this);
 
   // Request the initial profile list.
-  DBusThreadManager::Get()->GetShillManagerClient()->GetProperties(
-      base::Bind(&NetworkProfileHandler::GetManagerPropertiesCallback,
-                 weak_ptr_factory_.GetWeakPtr()));
+  ShillManagerClient::Get()->GetProperties(
+      base::BindOnce(&NetworkProfileHandler::GetManagerPropertiesCallback,
+                     weak_ptr_factory_.GetWeakPtr()));
 }
 
 NetworkProfileHandler::~NetworkProfileHandler() {
-  DBusThreadManager::Get()->GetShillManagerClient()->
-      RemovePropertyChangedObserver(this);
+  ShillManagerClient::Get()->RemovePropertyChangedObserver(this);
+}
+
+// static
+std::unique_ptr<NetworkProfileHandler>
+NetworkProfileHandler::InitializeForTesting() {
+  auto* handler = new NetworkProfileHandler();
+  handler->Init();
+  return base::WrapUnique(handler);
 }
 
 }  // namespace chromeos
