@@ -10,24 +10,20 @@
 #include <vector>
 
 #include "base/compiler_specific.h"
+#include "base/component_export.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
-#include "chromeos/chromeos_export.h"
-#include "chromeos/dbus/dbus_method_call_status.h"
-#include "chromeos/dbus/shill_property_changed_observer.h"
+#include "base/optional.h"
+#include "chromeos/dbus/shill/shill_property_changed_observer.h"
 #include "chromeos/network/network_handler.h"
 #include "chromeos/network/network_profile.h"
 
-namespace base {
-class DictionaryValue;
-}
-
 namespace chromeos {
 
 class NetworkProfileObserver;
 
-class CHROMEOS_EXPORT NetworkProfileHandler
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) NetworkProfileHandler
     : public ShillPropertyChangedObserver {
  public:
   typedef std::vector<NetworkProfile> ProfileList;
@@ -37,15 +33,14 @@ class CHROMEOS_EXPORT NetworkProfileHandler
   void AddObserver(NetworkProfileObserver* observer);
   void RemoveObserver(NetworkProfileObserver* observer);
 
-  void GetManagerPropertiesCallback(DBusMethodCallStatus call_status,
-                                    const base::DictionaryValue& properties);
+  void GetManagerPropertiesCallback(base::Optional<base::Value> properties);
 
   // ShillPropertyChangedObserver overrides
   void OnPropertyChanged(const std::string& name,
                          const base::Value& value) override;
 
   void GetProfilePropertiesCallback(const std::string& profile_path,
-                                    const base::DictionaryValue& properties);
+                                    base::Value properties);
 
   const NetworkProfile* GetProfileForPath(
       const std::string& profile_path) const;
@@ -59,6 +54,8 @@ class CHROMEOS_EXPORT NetworkProfileHandler
 
   static std::string GetSharedProfilePath();
 
+  static std::unique_ptr<NetworkProfileHandler> InitializeForTesting();
+
  protected:
   friend class AutoConnectHandlerTest;
   friend class ClientCertResolverTest;
@@ -83,7 +80,7 @@ class CHROMEOS_EXPORT NetworkProfileHandler
   base::ObserverList<NetworkProfileObserver, true>::Unchecked observers_;
 
   // For Shill client callbacks
-  base::WeakPtrFactory<NetworkProfileHandler> weak_ptr_factory_;
+  base::WeakPtrFactory<NetworkProfileHandler> weak_ptr_factory_{this};
 
  private:
   DISALLOW_COPY_AND_ASSIGN(NetworkProfileHandler);
