@@ -10,10 +10,10 @@
 
 #include "base/command_line.h"
 #include "base/run_loop.h"
-#include "base/test/scoped_task_environment.h"
-#include "chromeos/chromeos_switches.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_device_client.h"
+#include "base/test/task_environment.h"
+#include "chromeos/dbus/constants/dbus_switches.h"
+#include "chromeos/dbus/shill/shill_clients.h"
+#include "chromeos/dbus/shill/shill_device_client.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 
@@ -26,12 +26,11 @@ class TestObserver : public NetworkSmsHandler::Observer {
   TestObserver() = default;
   ~TestObserver() override = default;
 
-  void MessageReceived(const base::DictionaryValue& message) override {
-    std::string text;
-    if (message.GetStringWithoutPathExpansion(
-            NetworkSmsHandler::kTextKey, &text)) {
-      messages_.insert(text);
-    }
+  void MessageReceived(const base::Value& message) override {
+    const std::string* text =
+        message.FindStringKey(NetworkSmsHandler::kTextKey);
+    if (text)
+      messages_.insert(*text);
   }
 
   void ClearMessages() {
@@ -52,8 +51,8 @@ class TestObserver : public NetworkSmsHandler::Observer {
 class NetworkSmsHandlerTest : public testing::Test {
  public:
   NetworkSmsHandlerTest()
-      : scoped_task_environment_(
-            base::test::ScopedTaskEnvironment::MainThreadType::UI) {}
+      : task_environment_(
+            base::test::SingleThreadTaskEnvironment::MainThreadType::UI) {}
   ~NetworkSmsHandlerTest() override = default;
 
   void SetUp() override {
@@ -62,17 +61,16 @@ class NetworkSmsHandlerTest : public testing::Test {
     base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
     command_line->AppendSwitch(chromeos::switches::kSmsTestMessages);
 
-    // Initialize DBusThreadManager with a stub implementation.
-    DBusThreadManager::Initialize();
+    shill_clients::InitializeFakes();
     ShillDeviceClient::TestInterface* device_test =
-        DBusThreadManager::Get()->GetShillDeviceClient()->GetTestInterface();
+        ShillDeviceClient::Get()->GetTestInterface();
     ASSERT_TRUE(device_test);
     device_test->AddDevice("/org/freedesktop/ModemManager1/stub/0",
                            shill::kTypeCellular,
                            "stub_cellular_device2");
 
     // This relies on the stub dbus implementations for ShillManagerClient,
-    // ShillDeviceClient, GsmSMSClient, ModemMessagingClient and SMSClient.
+    // ShillDeviceClient, ModemMessagingClient and SMSClient.
     // Initialize a sms handler. The stub dbus clients will not send the
     // first test message until RequestUpdate has been called.
     network_sms_handler_.reset(new NetworkSmsHandler());
@@ -86,11 +84,11 @@ class NetworkSmsHandlerTest : public testing::Test {
   void TearDown() override {
     network_sms_handler_->RemoveObserver(test_observer_.get());
     network_sms_handler_.reset();
-    DBusThreadManager::Shutdown();
+    shill_clients::Shutdown();
   }
 
  protected:
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::SingleThreadTaskEnvironment task_environment_;
   std::unique_ptr<NetworkSmsHandler> network_sms_handler_;
   std::unique_ptr<TestObserver> test_observer_;
 };
