@@ -7,6 +7,7 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -14,6 +15,7 @@
 #include "base/macros.h"
 #include "base/values.h"
 #include "chromeos/network/managed_state.h"
+#include "chromeos/services/network_config/public/mojom/cros_network_config.mojom-forward.h"
 #include "components/onc/onc_constants.h"
 #include "url/gurl.h"
 
@@ -21,6 +23,11 @@ namespace base {
 class Value;
 }
 
+// TODO(https://crbug.com/1164001): remove when moved to ash.
+namespace ash {
+class MobileActivatorTest;
+}
+
 namespace chromeos {
 
 // Simple class to provide network state information about a network service.
@@ -33,18 +40,11 @@ namespace chromeos {
 // Manager.ServiceCompleteList. The visible() method indicates whether the
 // network is visible, and the IsInProfile() method indicates whether the
 // network is saved in a profile.
-class CHROMEOS_EXPORT NetworkState : public ManagedState {
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) NetworkState : public ManagedState {
  public:
   explicit NetworkState(const std::string& path);
   ~NetworkState() override;
 
-  struct CaptivePortalProviderInfo {
-    // The id used by chrome to identify the provider (i.e. an extension id).
-    std::string id;
-    // The display name for the captive portal provider (i.e. extension name).
-    std::string name;
-  };
-
   struct VpnProviderInfo {
     // The id used by chrome to identify the provider (i.e. an extension id).
     std::string id;
@@ -52,12 +52,30 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
     std::string type;
   };
 
+  enum class PortalState {
+    // The network is not connected or the portal state is not available.
+    kUnknown,
+    // The network is connected and no portal is detected.
+    kOnline,
+    // A portal is suspected but no redirect was provided.
+    kPortalSuspected,
+    // The network is in a portal state with a redirect URL.
+    kPortal,
+    // A proxy requiring authentication is detected.
+    kProxyAuthRequired,
+    // The network is connected but no internet is available and no proxy was
+    // detected.
+    kNoInternet,
+    kMaxValue = kNoInternet  // For UMA_HISTOGRAM_ENUMERATION
+  };
+
   // ManagedState overrides
   // If you change this method, update GetProperties too.
   bool PropertyChanged(const std::string& key,
                        const base::Value& value) override;
   bool InitialPropertiesReceived(const base::Value& properties) override;
   void GetStateProperties(base::Value* dictionary) const override;
+  bool IsActive() const override;
 
   // Called when the IPConfig properties may have changed. |properties| is
   // expected to be of type DICTIONARY.
@@ -76,17 +94,29 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   const std::string& device_path() const { return device_path_; }
   const std::string& guid() const { return guid_; }
   const std::string& profile_path() const { return profile_path_; }
-  const std::string& error() const { return error_; }
-  const std::string& last_error() const { return last_error_; }
-  void clear_last_error() { last_error_.clear(); }
+  const GURL& probe_url() const { return probe_url_; }
   ::onc::ONCSource onc_source() const { return onc_source_; }
 
-  // Returns |connection_state_| if visible, kStateDisconnect otherwise.
-  std::string connection_state() const;
-  void set_connection_state(const std::string connection_state);
+  // Provides the error for the last attempt to connect/configure the network
+  // (an empty string signifies no error at all). Note that this value can be
+  // cleared - see ClearError() below.
+  const std::string& GetError() const;
 
-  const base::Value* proxy_config() const { return proxy_config_.get(); }
-  const base::Value* ipv4_config() const { return ipv4_config_.get(); }
+  // Clears the error associated with this network. Should be called whenever
+  // a connection to this network is initiated or the associated configuration
+  // is updated/removed.
+  void ClearError();
+
+  // Returns |connection_state_| if visible, kStateIdle otherwise.
+  std::string connection_state() const;
+
+  // Updates the connection state and saves the previous connection state.
+  void SetConnectionState(const std::string& connection_state);
+
+  int priority() const { return priority_; }
+
+  const base::Value& proxy_config() const { return proxy_config_; }
+  const base::Value& ipv4_config() const { return ipv4_config_; }
   std::string GetIpAddress() const;
   std::string GetGateway() const;
   GURL GetWebProxyAutoDiscoveryUrl() const;
@@ -94,32 +124,31 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   // Wireless property accessors
   bool connectable() const { return connectable_; }
   void set_connectable(bool connectable) { connectable_ = connectable; }
-  bool is_captive_portal() const { return is_captive_portal_; }
-  const CaptivePortalProviderInfo* captive_portal_provider() const {
-    return captive_portal_provider_.get();
-  }
-  void SetCaptivePortalProvider(const std::string& id, const std::string& name);
   int signal_strength() const { return signal_strength_; }
   void set_signal_strength(int signal_strength) {
     signal_strength_ = signal_strength;
   }
+  const std::string& bssid() const { return bssid_; }
+  int frequency() const { return frequency_; }
   bool blocked_by_policy() const { return blocked_by_policy_; }
   void set_blocked_by_policy(bool blocked_by_policy) {
     blocked_by_policy_ = blocked_by_policy;
   }
+  bool hidden_ssid() const { return hidden_ssid_; }
 
   // Wifi property accessors
   const std::string& eap_method() const { return eap_method_; }
   const std::vector<uint8_t>& raw_ssid() const { return raw_ssid_; }
 
   // Cellular property accessors
+  const std::string& eid() const { return eid_; }
+  const std::string& iccid() const { return iccid_; }
   const std::string& network_technology() const { return network_technology_; }
   const std::string& activation_type() const { return activation_type_; }
   const std::string& activation_state() const { return activation_state_; }
-  const std::string& roaming() const { return roaming_; }
   const std::string& payment_url() const { return payment_url_; }
+  const std::string& payment_post_data() const { return payment_post_data_; }
   bool cellular_out_of_credits() const { return cellular_out_of_credits_; }
-  const std::string& tethering_state() const { return tethering_state_; }
 
   // VPN property accessors
   const VpnProviderInfo* vpn_provider() const { return vpn_provider_.get(); }
@@ -130,8 +159,10 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   void set_battery_percentage(int battery_percentage) {
     battery_percentage_ = battery_percentage;
   }
-  const std::string& carrier() const { return carrier_; }
-  void set_carrier(const std::string& carrier) { carrier_ = carrier; }
+  const std::string& tether_carrier() const { return tether_carrier_; }
+  void set_tether_carrier(const std::string& tether_carrier) {
+    tether_carrier_ = tether_carrier;
+  }
   bool tether_has_connected_to_host() const {
     return tether_has_connected_to_host_;
   }
@@ -141,12 +172,21 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   const std::string& tether_guid() const { return tether_guid_; }
   void set_tether_guid(const std::string& guid) { tether_guid_ = guid; }
 
+  bool connect_requested() const { return connect_requested_; }
+
+  const std::string& shill_connect_error() const {
+    return shill_connect_error_;
+  }
+
+  PortalState portal_state() const { return portal_state_; }
+
   // Returns true if the network is managed by policy (determined by
   // |onc_source_|).
   bool IsManagedByPolicy() const;
 
-  // Returns true if current connection is using mobile data.
-  bool IsUsingMobileData() const;
+  // Returns true if the network is romaing and the provider does not require
+  // roaming.
+  bool IndicateRoaming() const;
 
   // Returns true if the network securty is WEP_8021x (Dynamic WEP)
   bool IsDynamicWep() const;
@@ -156,9 +196,8 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   bool IsConnectingState() const;
   bool IsConnectingOrConnected() const;
 
-  // Returns true if |last_connection_state_| is connected, and
-  // |connection_state_| is connecting.
-  bool IsReconnecting() const;
+  // Returns true if |connection_state_| is online.
+  bool IsOnline() const;
 
   // Returns true if this is a network stored in a profile.
   bool IsInProfile() const;
@@ -174,6 +213,15 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   // NetworkStateHandler::EnsureCellularNetwork()).
   bool IsDefaultCellular() const;
 
+  // Returns true if Shill has detected a captive portal state.
+  bool IsShillCaptivePortal() const;
+
+  // Returns true if Shill or Chrome have detected a captive portal state.
+  bool IsCaptivePortal() const;
+
+  // Returns true if the security type is non-empty and not 'none'.
+  bool IsSecure() const;
+
   // Returns the |raw_ssid| as a hex-encoded string
   std::string GetHexSsid() const;
 
@@ -191,28 +239,59 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   // Set the GUID. Called exclusively by NetworkStateHandler.
   void SetGuid(const std::string& guid);
 
-  // Returns |error_| if valid, otherwise returns |last_error_|.
-  std::string GetErrorState() const;
+  // Helpers for returning mojo types.
+  network_config::mojom::ActivationStateType GetMojoActivationState() const;
+  network_config::mojom::SecurityType GetMojoSecurity() const;
+
+  // Helper for UMA stats. Corresponds to NetworkTechnology in enums.xml
+  // which is also used by Shill metrics.
+  enum class NetworkTechnologyType {
+    kCellular = 0,
+    kEthernet = 1,
+    kEthernetEap = 2,
+    kWiFi = 3,
+    kTether = 4,
+    kVPN = 5,
+    kUnknown = 6,
+    kMaxValue = kUnknown,
+  };
+  NetworkTechnologyType GetNetworkTechnologyType() const;
+
+  // Setters for testing.
+  void set_connection_state_for_testing(const std::string& connection_state) {
+    connection_state_ = connection_state;
+  }
+  void set_connect_requested_for_testing(bool connect_requested) {
+    connect_requested_ = connect_requested;
+  }
+  void set_network_technology_for_testing(const std::string& technology) {
+    network_technology_ = technology;
+  }
 
   // Helpers (used e.g. when a state, error, or shill dictionary is cached)
   static bool StateIsConnected(const std::string& connection_state);
   static bool StateIsConnecting(const std::string& connection_state);
-  static bool NetworkStateIsCaptivePortal(const base::Value& shill_properties);
+  static bool StateIsPortalled(const std::string& connection_state);
   static bool ErrorIsValid(const std::string& error);
   static std::unique_ptr<NetworkState> CreateDefaultCellular(
       const std::string& device_path);
 
+  // Ignore changes to signal strength less than this value.
+  constexpr static const int kSignalStrengthChangeThreshold = 5;
+
  private:
-  friend class MobileActivatorTest;
+  // TODO(https://crbug.com/1164001): remove namespace declaration for ash
+  // when moved to ash.
+  friend class ::ash::MobileActivatorTest;
   friend class NetworkStateHandler;
-  friend class NetworkChangeNotifierChromeosUpdateTest;
-  FRIEND_TEST_ALL_PREFIXES(NetworkStateTest, TetherProperties);
 
   // Updates |name_| from the 'WiFi.HexSSID' entry in |properties|, which must
   // be of type DICTIONARY, if the key exists, and validates |name_|. Returns
   // true if |name_| changes.
   bool UpdateName(const base::Value& properties);
 
+  void UpdateCaptivePortalState(const base::Value& properties);
+
   void SetVpnProvider(const std::string& id, const std::string& type);
 
   // Set to true if the network is a member of Manager.Services.
@@ -230,39 +309,43 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   std::string connection_state_;
   std::string last_connection_state_;
   std::string profile_path_;
+  GURL probe_url_;
   std::vector<uint8_t> raw_ssid_;  // Unknown encoding. Not necessarily UTF-8.
-  int priority_ = 0;
+  int priority_ = 0;  // kPriority, used for organizing known networks.
   ::onc::ONCSource onc_source_ = ::onc::ONC_SOURCE_UNKNOWN;
 
-  // Reflects the current Shill Service.Error property. This might get cleared
-  // by Shill shortly after a failure.
-  std::string error_;
-
-  // Last non empty Service.Error property. Cleared by NetworkConnectionHandler
-  // when a connection attempt is initiated.
+  // Last non empty Service.Error property. Expected to be cleared via
+  // ClearError() when a connection attempt is initiated and when an associated
+  // configuration is updated/removed.
   std::string last_error_;
 
+  // The error message provided by the shill Service.Connect dbus method if the
+  // most recent connect attempt failed. Otherwise empty.
+  std::string shill_connect_error_;
+
   // Cached copy of the Shill Service IPConfig object. For ipv6 properties use
   // the ip_configs_ property in the corresponding DeviceState.
-  std::unique_ptr<base::Value> ipv4_config_;
+  base::Value ipv4_config_;
 
   // Wireless properties, used for icons and Connect logic.
   bool connectable_ = false;
-  bool is_captive_portal_ = false;
-  std::unique_ptr<CaptivePortalProviderInfo> captive_portal_provider_;
   int signal_strength_ = 0;
-  std::string bssid_;  // For ARC
-  int frequency_ = 0;  // For ARC
+  std::string bssid_;
+  int frequency_ = 0;
   bool blocked_by_policy_ = false;
+  bool hidden_ssid_ = false;
 
   // Cellular properties, used for icons, Connect, and Activation.
+  std::string eid_;
+  std::string iccid_;
   std::string network_technology_;
   std::string activation_type_;
   std::string activation_state_;
   std::string roaming_;
+  bool provider_requires_roaming_ = false;
   std::string payment_url_;
+  std::string payment_post_data_;
   bool cellular_out_of_credits_ = false;
-  std::string tethering_state_;
 
   // VPN properties, used to construct the display name and to show the correct
   // configuration dialog. The id is the Extension ID or Arc package name for
@@ -270,9 +353,13 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
   std::unique_ptr<VpnProviderInfo> vpn_provider_;
 
   // Tether properties.
-  std::string carrier_;
+  std::string tether_carrier_;
   int battery_percentage_ = 0;
 
+  // Portal state is derived from connection_state_ and Shill portal properties.
+  PortalState portal_state_ = PortalState::kUnknown;
+  int portal_status_code_ = 0;
+
   // Whether the current device has already connected to the tether host device
   // providing the hotspot corresponding to this NetworkState.
   // Note: this means that the current device has already connected to the
@@ -282,11 +369,24 @@ class CHROMEOS_EXPORT NetworkState : public ManagedState {
 
   // TODO(pneubeck): Remove this once (Managed)NetworkConfigurationHandler
   // provides proxy configuration. crbug.com/241775
-  std::unique_ptr<base::Value> proxy_config_;
+  base::Value proxy_config_;
+
+  // Set while a network connect request is queued. Cleared on connect or
+  // if the request is aborted.
+  bool connect_requested_ = false;
+
+  // Set by NetworkStateHandler if Chrome detects a captive portal state.
+  // See IsCaptivePortal() for details.
+  bool is_chrome_captive_portal_ = false;
 
   DISALLOW_COPY_AND_ASSIGN(NetworkState);
 };
 
 }  // namespace chromeos
 
+// TODO(https://crbug.com/1164001): remove when moved to ash.
+namespace ash {
+using ::chromeos::NetworkState;
+}
+
 #endif  // CHROMEOS_NETWORK_NETWORK_STATE_H_
