@@ -13,7 +13,7 @@
 #include "base/strings/string_tokenizer.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
-#include "chromeos/login/login_state.h"
+#include "chromeos/login/login_state/login_state.h"
 #include "chromeos/network/device_state.h"
 #include "chromeos/network/managed_network_configuration_handler.h"
 #include "chromeos/network/network_state.h"
@@ -22,6 +22,7 @@
 #include "chromeos/network/onc/onc_signature.h"
 #include "chromeos/network/onc/onc_translation_tables.h"
 #include "chromeos/network/onc/onc_translator.h"
+#include "components/device_event_log/device_event_log.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 
 namespace chromeos {
@@ -49,6 +50,13 @@ CellularScanResult::CellularScanResult(const CellularScanResult& other) =
 
 CellularScanResult::~CellularScanResult() = default;
 
+CellularSIMSlotInfo::CellularSIMSlotInfo() = default;
+
+CellularSIMSlotInfo::CellularSIMSlotInfo(const CellularSIMSlotInfo& other) =
+    default;
+
+CellularSIMSlotInfo::~CellularSIMSlotInfo() = default;
+
 namespace network_util {
 
 std::string PrefixLengthToNetmask(int32_t prefix_length) {
@@ -68,7 +76,7 @@ std::string PrefixLengthToNetmask(int32_t prefix_length) {
       netmask += ".";
     int value = remainder == 0 ? 0 :
         ((2L << (remainder - 1)) - 1) << (8 - remainder);
-    netmask += base::IntToString(value);
+    netmask += base::NumberToString(value);
   }
   return netmask;
 }
@@ -156,6 +164,37 @@ bool ParseCellularScanResults(const base::ListValue& list,
   return true;
 }
 
+bool ParseCellularSIMSlotInfo(
+    const base::Value::ConstListView list,
+    std::vector<CellularSIMSlotInfo>* sim_slot_infos) {
+  sim_slot_infos->clear();
+  sim_slot_infos->reserve(list.size());
+  for (size_t i = 0; i < list.size(); i++) {
+    const auto& value = list[i];
+    if (!value.is_dict())
+      return false;
+
+    CellularSIMSlotInfo sim_slot_info;
+    // The |slot_id| should start with 1.
+    sim_slot_info.slot_id = i + 1;
+
+    const std::string* eid = value.FindStringKey(shill::kSIMSlotInfoEID);
+    if (eid)
+      sim_slot_info.eid = *eid;
+
+    const std::string* iccid = value.FindStringKey(shill::kSIMSlotInfoICCID);
+    if (iccid)
+      sim_slot_info.iccid = *iccid;
+
+    base::Optional<bool> primary =
+        value.FindBoolKey(shill::kSIMSlotInfoPrimary);
+    sim_slot_info.primary = primary.has_value() ? *primary : false;
+
+    sim_slot_infos->push_back(sim_slot_info);
+  }
+  return true;
+}
+
 std::unique_ptr<base::DictionaryValue> TranslateNetworkStateToONC(
     const NetworkState* network) {
   // Get the properties from the NetworkState.
@@ -194,6 +233,15 @@ std::unique_ptr<base::DictionaryValue> TranslateNetworkStateToONC(
   std::unique_ptr<base::DictionaryValue> onc_dictionary =
       TranslateShillServiceToONCPart(*shill_dictionary, onc_source,
                                      &onc::kNetworkWithStateSignature, network);
+
+  // Remove IPAddressConfigType/NameServersConfigType as these were
+  // historically not provided by TranslateNetworkStateToONC.
+  // The source shill properties for those ONC properties are not provided by
+  // NetworkState::GetStateProperties, however since CL:2530330 these are
+  // assumed to have defaults that are always enforced during ONC translation.
+  onc_dictionary->RemoveKey(::onc::network_config::kIPAddressConfigType);
+  onc_dictionary->RemoveKey(::onc::network_config::kNameServersConfigType);
+
   return onc_dictionary;
 }
 
