@@ -8,7 +8,7 @@
 #include <string>
 
 #include "base/bind.h"
-#include "base/logging.h"
+#include "base/notreached.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
@@ -51,7 +51,7 @@ class ONCCertificateImporterImplTest : public testing::Test {
 
   void TearDown() override {
     thread_task_runner_handle_.reset();
-    task_runner_ = NULL;
+    task_runner_.reset();
   }
 
  protected:
@@ -74,16 +74,13 @@ class ONCCertificateImporterImplTest : public testing::Test {
                                bool expected_import_success) {
     std::unique_ptr<base::DictionaryValue> onc =
         test_utils::ReadTestDictionary(filename);
-    std::unique_ptr<base::Value> certificates_value;
-    base::ListValue* certificates = NULL;
-    onc->RemoveWithoutPathExpansion(::onc::toplevel_config::kCertificates,
-                                    &certificates_value);
-    certificates_value.release()->GetAsList(&certificates);
-    onc_certificates_.reset(certificates);
+    base::Optional<base::Value> certificates_value =
+        onc->ExtractKey(::onc::toplevel_config::kCertificates);
+    onc_certificates_ = std::move(*certificates_value);
 
     CertificateImporterImpl importer(task_runner_, test_nssdb_.get());
     auto onc_parsed_certificates =
-        std::make_unique<OncParsedCertificates>(*certificates);
+        std::make_unique<OncParsedCertificates>(onc_certificates_);
     EXPECT_EQ(expected_parse_success, !onc_parsed_certificates->has_error());
     switch (import_type) {
       case ImportType::kClientCertificatesOnly:
@@ -132,9 +129,10 @@ class ONCCertificateImporterImplTest : public testing::Test {
                 certificate::GetCertType(private_list_[0].get()));
     }
 
-    base::DictionaryValue* certificate = NULL;
-    onc_certificates_->GetDictionary(0, &certificate);
-    certificate->GetStringWithoutPathExpansion(::onc::certificate::kGUID, guid);
+    const base::Value& certificate = onc_certificates_.GetList()[0];
+    const std::string* guid_value =
+        certificate.FindStringKey(::onc::certificate::kGUID);
+    *guid = *guid_value;
   }
 
   // Certificates and the NSSCertDatabase depend on these test DBs. Destroy them
@@ -145,7 +143,7 @@ class ONCCertificateImporterImplTest : public testing::Test {
   scoped_refptr<base::TestSimpleTaskRunner> task_runner_;
   std::unique_ptr<base::ThreadTaskRunnerHandle> thread_task_runner_handle_;
   std::unique_ptr<net::NSSCertDatabaseChromeOS> test_nssdb_;
-  std::unique_ptr<base::ListValue> onc_certificates_;
+  base::Value onc_certificates_;
   // List of certs in the nssdb's public slot.
   net::ScopedCERTCertificateList public_list_;
   // List of certs in the nssdb's "private" slot.
@@ -163,6 +161,8 @@ class ONCCertificateImporterImplTest : public testing::Test {
   net::ScopedCERTCertificateList ListCertsInSlot(PK11SlotInfo* slot) {
     net::ScopedCERTCertificateList result;
     CERTCertList* cert_list = PK11_ListCertsInSlot(slot);
+    if (!cert_list)
+      return result;
     for (CERTCertListNode* node = CERT_LIST_HEAD(cert_list);
          !CERT_LIST_END(node, cert_list);
          node = CERT_LIST_NEXT(node)) {
@@ -184,7 +184,7 @@ TEST_F(ONCCertificateImporterImplTest, MultipleCertificates) {
   AddCertificatesFromFile("managed_toplevel2.onc", ImportType::kAllCertificates,
                           true /* expected_parse_success */,
                           true /* expected_import_success */);
-  EXPECT_EQ(onc_certificates_->GetSize(), public_list_.size());
+  EXPECT_EQ(onc_certificates_.GetList().size(), public_list_.size());
   EXPECT_TRUE(private_list_.empty());
   EXPECT_EQ(2ul, public_list_.size());
 }
@@ -204,7 +204,7 @@ TEST_F(ONCCertificateImporterImplTest, MultipleCertificatesWithFailures) {
   AddCertificatesFromFile(
       "toplevel_partially_invalid.onc", ImportType::kAllCertificates,
       false /* expected_parse_success */, true /* expected_import_success */);
-  EXPECT_EQ(3ul, onc_certificates_->GetSize());
+  EXPECT_EQ(3ul, onc_certificates_.GetList().size());
   EXPECT_EQ(1ul, private_list_.size());
   EXPECT_TRUE(public_list_.empty());
 }
@@ -331,26 +331,25 @@ TEST_P(ONCCertificateImporterImplTestWithParam, UpdateCertificate) {
 TEST_P(ONCCertificateImporterImplTestWithParam, ReimportCertificate) {
   // Verify that reimporting a client certificate works.
   for (int i = 0; i < 2; ++i) {
-    SCOPED_TRACE("Import certificate, iteration " + base::IntToString(i));
+    SCOPED_TRACE("Import certificate, iteration " + base::NumberToString(i));
     AddCertificateFromFile(GetParam().original_file,
                            ImportType::kAllCertificates, GetParam().cert_type,
                            NULL);
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     ONCCertificateImporterImplTestWithParam,
     ONCCertificateImporterImplTestWithParam,
-    ::testing::Values(
-        CertParam(net::USER_CERT,
-                  "certificate-client.onc",
-                  "certificate-client-update.onc"),
-        CertParam(net::SERVER_CERT,
-                  "certificate-server.onc",
-                  "certificate-server-update.onc"),
-        CertParam(net::CA_CERT,
-                  "certificate-web-authority.onc",
-                  "certificate-web-authority-update.onc")));
+    ::testing::Values(CertParam(net::USER_CERT,
+                                "certificate-client.onc",
+                                "certificate-client-update.onc"),
+                      CertParam(net::SERVER_CERT,
+                                "certificate-server.onc",
+                                "certificate-server-update.onc"),
+                      CertParam(net::CA_CERT,
+                                "certificate-web-authority.onc",
+                                "certificate-web-authority-update.onc")));
 
 }  // namespace onc
 }  // namespace chromeos
