@@ -5,15 +5,15 @@
 #ifndef CHROMEOS_NETWORK_ONC_ONC_TEST_UTILS_H_
 #define CHROMEOS_NETWORK_ONC_ONC_TEST_UTILS_H_
 
+#include <memory>
 #include <string>
 
-#include "base/memory/scoped_ptr.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace base {
 class DictionaryValue;
 class Value;
-}
+}  // namespace base
 
 namespace chromeos {
 namespace onc {
@@ -22,13 +22,16 @@ namespace test_utils {
 // Read the file at |filename| as a string. CHECKs if any error occurs.
 std::string ReadTestData(const std::string& filename);
 
-// Read a JSON dictionary from |filename| and return it as a
-// DictionaryValue. CHECKs if any error occurs.
-scoped_ptr<base::DictionaryValue> ReadTestDictionary(
+// Read a JSON dictionary from |filename| and return it as a base::Value.
+// CHECKs if any error occurs.
+base::Value ReadTestDictionaryValue(const std::string& filename);
+
+// Deprecated version of the above.
+std::unique_ptr<base::DictionaryValue> ReadTestDictionary(
     const std::string& filename);
 
 // Checks that the pointer |actual| is not NULL but points to a value that
-// equals |expected| (using Value::Equals). The intended use case is:
+// equals |expected|. The intended use case is:
 // EXPECT_TRUE(test_utils::Equals(expected, actual));
 ::testing::AssertionResult Equals(const base::Value* expected,
                                   const base::Value* actual);
