@@ -20,6 +20,10 @@ namespace onc {
 
 namespace {
 
+// Cellular Service EID property.
+// TODO(crbug.com/1093185): Use dbus-constants when property is added in shill.
+const char kCellularEidProperty[] = "Cellular.EID";
+
 const FieldTranslationEntry eap_fields[] = {
     {::onc::eap::kAnonymousIdentity, shill::kEapAnonymousIdentityProperty},
     // This field is converted during translation, see onc_translator_*.
@@ -33,11 +37,14 @@ const FieldTranslationEntry eap_fields[] = {
     {::onc::eap::kSaveCredentials, shill::kSaveCredentialsProperty},
     {::onc::eap::kServerCAPEMs, shill::kEapCaCertPemProperty},
     {::onc::eap::kSubjectMatch, shill::kEapSubjectMatchProperty},
+    // This field is converted during translation, see onc_translator_*.
+    // {::onc::eap::kSubjectAlternativeNameMatch,
+    //  shill::kEapSubjectAlternativeNameMatchProperty},
     {::onc::eap::kTLSVersionMax, shill::kEapTLSVersionMaxProperty},
     {::onc::eap::kUseSystemCAs, shill::kEapUseSystemCasProperty},
     {::onc::eap::kUseProactiveKeyCaching,
      shill::kEapUseProactiveKeyCachingProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry ipsec_fields[] = {
     // This field is converted during translation, see onc_translator_*.
@@ -52,12 +59,12 @@ const FieldTranslationEntry ipsec_fields[] = {
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::vpn::kSaveCredentials, shill::kSaveCredentialsProperty},
     {::onc::ipsec::kServerCAPEMs, shill::kL2tpIpsecCaCertPemProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry xauth_fields[] = {
     {::onc::vpn::kPassword, shill::kL2tpIpsecXauthPasswordProperty},
     {::onc::vpn::kUsername, shill::kL2tpIpsecXauthUserProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry l2tp_fields[] = {
     {::onc::l2tp::kPassword, shill::kL2tpIpsecPasswordProperty},
@@ -65,8 +72,10 @@ const FieldTranslationEntry l2tp_fields[] = {
     // doesn't support separate settings for ipsec and l2tp.
     // { ::onc::l2tp::kSaveCredentials, &kBoolSignature },
     {::onc::l2tp::kUsername, shill::kL2tpIpsecUserProperty},
-    {::onc::l2tp::kLcpEchoDisabled, shill::kL2tpIpsecLcpEchoDisabledProperty},
-    {NULL}};
+    // kLcpEchoDisabled is a bool in ONC and a string in Shill.
+    // {::onc::l2tp::kLcpEchoDisabled,
+    // shill::kL2tpIpsecLcpEchoDisabledProperty},
+    {nullptr}};
 
 const FieldTranslationEntry openvpn_fields[] = {
     {::onc::openvpn::kAuth, shill::kOpenVPNAuthProperty},
@@ -78,6 +87,8 @@ const FieldTranslationEntry openvpn_fields[] = {
     //  shill::kOpenVPNClientCertIdProperty},
     {::onc::openvpn::kCompLZO, shill::kOpenVPNCompLZOProperty},
     {::onc::openvpn::kCompNoAdapt, shill::kOpenVPNCompNoAdaptProperty},
+    // This field is converted during translation, see onc_translator_*
+    // {::onc::openvpn::kCompressionAlgorithm, shill::kOpenVPNCompressProperty},
     {::onc::openvpn::kExtraHosts, shill::kOpenVPNExtraHostsProperty},
     {::onc::openvpn::kIgnoreDefaultRoute,
      shill::kOpenVPNIgnoreDefaultRouteProperty},
@@ -108,30 +119,30 @@ const FieldTranslationEntry openvpn_fields[] = {
     {::onc::vpn::kUsername, shill::kOpenVPNUserProperty},
     {::onc::openvpn::kVerb, shill::kOpenVPNVerbProperty},
     {::onc::openvpn::kVerifyHash, shill::kOpenVPNVerifyHashProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry arc_vpn_fields[] = {
     {::onc::arc_vpn::kTunnelChrome, shill::kArcVpnTunnelChromeProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry verify_x509_fields[] = {
     {::onc::verify_x509::kName, shill::kOpenVPNVerifyX509NameProperty},
     {::onc::verify_x509::kType, shill::kOpenVPNVerifyX509TypeProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry vpn_fields[] = {
     {::onc::vpn::kAutoConnect, shill::kAutoConnectProperty},
     // These fields are converted during translation, see onc_translator_*.
     // { ::onc::vpn::kHost, shill::kProviderHostProperty},
     // { ::onc::vpn::kType, shill::kProviderTypeProperty },
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry tether_fields[] = {
     {::onc::tether::kBatteryPercentage, kTetherBatteryPercentage},
     {::onc::tether::kCarrier, kTetherCarrier},
     {::onc::tether::kHasConnectedToHost, kTetherHasConnectedToHost},
     {::onc::tether::kSignalStrength, kTetherSignalStrength},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry wifi_fields[] = {
     {::onc::wifi::kAutoConnect, shill::kAutoConnectProperty},
@@ -140,23 +151,13 @@ const FieldTranslationEntry wifi_fields[] = {
     // { ::onc::wifi::kEAP, shill::kEap*},
     {::onc::wifi::kFrequency, shill::kWifiFrequency},
     {::onc::wifi::kFrequencyList, shill::kWifiFrequencyListProperty},
-    {::onc::wifi::kFTEnabled, shill::kWifiFTEnabled},
     {::onc::wifi::kHexSSID, shill::kWifiHexSsid},
     {::onc::wifi::kHiddenSSID, shill::kWifiHiddenSsid},
     {::onc::wifi::kPassphrase, shill::kPassphraseProperty},
-    {::onc::wifi::kRoamThreshold, shill::kWifiRoamThresholdProperty},
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::wifi::kSecurity, shill::kSecurityClassProperty },
     {::onc::wifi::kSignalStrength, shill::kSignalStrengthProperty},
-    {::onc::wifi::kTetheringState, shill::kTetheringProperty},
-    {NULL}};
-
-const FieldTranslationEntry wimax_fields[] = {
-    {::onc::wimax::kAutoConnect, shill::kAutoConnectProperty},
-    // This dictionary is converted during translation, see onc_translator_*.
-    // { ::onc::wimax::kEAP, shill::kEap*},
-    {::onc::wimax::kSignalStrength, shill::kSignalStrengthProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry cellular_apn_fields[] = {
     {::onc::cellular_apn::kAccessPointName, shill::kApnProperty},
@@ -166,7 +167,7 @@ const FieldTranslationEntry cellular_apn_fields[] = {
     {::onc::cellular_apn::kAuthentication, shill::kApnAuthenticationProperty},
     {::onc::cellular_apn::kLocalizedName, shill::kApnLocalizedNameProperty},
     {::onc::cellular_apn::kLanguage, shill::kApnLanguageProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry cellular_found_network_fields[] = {
     {::onc::cellular_found_network::kNetworkId, shill::kNetworkIdProperty},
@@ -174,25 +175,25 @@ const FieldTranslationEntry cellular_found_network_fields[] = {
     {::onc::cellular_found_network::kTechnology, shill::kTechnologyProperty},
     {::onc::cellular_found_network::kShortName, shill::kShortNameProperty},
     {::onc::cellular_found_network::kLongName, shill::kLongNameProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry cellular_payment_portal_fields[] = {
     {::onc::cellular_payment_portal::kMethod, shill::kPaymentPortalMethod},
     {::onc::cellular_payment_portal::kPostData, shill::kPaymentPortalPostData},
     {::onc::cellular_payment_portal::kUrl, shill::kPaymentPortalURL},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry cellular_provider_fields[] = {
     {::onc::cellular_provider::kCode, shill::kOperatorCodeKey},
     {::onc::cellular_provider::kCountry, shill::kOperatorCountryKey},
     {::onc::cellular_provider::kName, shill::kOperatorNameKey},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry sim_lock_status_fields[] = {
     {::onc::sim_lock_status::kLockEnabled, shill::kSIMLockEnabledProperty},
     {::onc::sim_lock_status::kLockType, shill::kSIMLockTypeProperty},
     {::onc::sim_lock_status::kRetriesLeft, shill::kSIMLockRetriesLeftProperty},
-    {NULL}};
+    {nullptr}};
 
 // This must only contain Service properties and not Device properties.
 // For Device properties see kCellularDeviceTable.
@@ -201,6 +202,9 @@ const FieldTranslationEntry cellular_fields[] = {
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::cellular::kActivationState, shill::kActivationStateProperty},
     {::onc::cellular::kAutoConnect, shill::kAutoConnectProperty},
+    {::onc::cellular::kEID, kCellularEidProperty},
+    {::onc::cellular::kICCID, shill::kIccidProperty},
+    {::onc::cellular::kIMSI, shill::kImsiProperty},
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::cellular::kNetworkTechnology,
     //   shill::kNetworkTechnologyProperty},
@@ -209,11 +213,12 @@ const FieldTranslationEntry cellular_fields[] = {
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::cellular::kRoamingState, shill::kRoamingStateProperty},
     {::onc::cellular::kSignalStrength, shill::kSignalStrengthProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry network_fields[] = {
     {::onc::network_config::kGUID, shill::kGuidProperty},
     {::onc::network_config::kConnectable, shill::kConnectableProperty},
+    {::onc::network_config::kMetered, shill::kMeteredProperty},
     {::onc::network_config::kPriority, shill::kPriorityProperty},
 
     // Shill doesn't allow setting the name for non-VPN networks.
@@ -232,7 +237,7 @@ const FieldTranslationEntry network_fields[] = {
     // {::onc::network_config::kRestrictedConnectivity, shill::kStateProperty },
     // {::onc::network_config::kSource, shill::kProfileProperty },
     // {::onc::network_config::kMacAddress, shill::kAddressProperty },
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry ipconfig_fields[] = {
     {::onc::ipconfig::kIPAddress, shill::kAddressProperty},
@@ -244,7 +249,7 @@ const FieldTranslationEntry ipconfig_fields[] = {
     // { ::onc::ipconfig::kType, shill::kMethodProperty},
     {::onc::ipconfig::kWebProxyAutoDiscoveryUrl,
      shill::kWebProxyAutoDiscoveryUrlProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry static_or_saved_ipconfig_fields[] = {
     {::onc::ipconfig::kIPAddress, shill::kAddressProperty},
@@ -254,7 +259,7 @@ const FieldTranslationEntry static_or_saved_ipconfig_fields[] = {
     {::onc::ipconfig::kSearchDomains, shill::kSearchDomainsProperty},
     {::onc::ipconfig::kIncludedRoutes, shill::kIncludedRoutesProperty},
     {::onc::ipconfig::kExcludedRoutes, shill::kExcludedRoutesProperty},
-    {NULL}};
+    {nullptr}};
 
 struct OncValueTranslationEntry {
   const OncValueSignature* onc_signature;
@@ -274,8 +279,6 @@ const OncValueTranslationEntry onc_value_translation_table[] = {
     {&kTetherWithStateSignature, tether_fields},
     {&kWiFiSignature, wifi_fields},
     {&kWiFiWithStateSignature, wifi_fields},
-    {&kWiMAXSignature, wimax_fields},
-    {&kWiMAXWithStateSignature, wimax_fields},
     {&kCellularApnSignature, cellular_apn_fields},
     {&kCellularFoundNetworkSignature, cellular_found_network_fields},
     {&kCellularPaymentPortalSignature, cellular_payment_portal_fields},
@@ -288,43 +291,45 @@ const OncValueTranslationEntry onc_value_translation_table[] = {
     {&kIPConfigSignature, ipconfig_fields},
     {&kSavedIPConfigSignature, static_or_saved_ipconfig_fields},
     {&kStaticIPConfigSignature, static_or_saved_ipconfig_fields},
-    {NULL}};
+    {nullptr}};
 
 struct NestedShillDictionaryEntry {
   const OncValueSignature* onc_signature;
-  // NULL terminated list of Shill property keys.
+  // nullptr terminated list of Shill property keys.
   const char* const* shill_property_path;
 };
 
-const char* cellular_apn_path_entries[] = {shill::kCellularApnProperty, NULL};
+const char* cellular_apn_path_entries[] = {shill::kCellularApnProperty,
+                                           nullptr};
 
 const char* static_ip_config_path_entries[] = {shill::kStaticIPConfigProperty,
-                                               NULL};
+                                               nullptr};
 
 const NestedShillDictionaryEntry nested_shill_dictionaries[] = {
     {&kCellularApnSignature, cellular_apn_path_entries},
     {&kStaticIPConfigSignature, static_ip_config_path_entries},
-    {NULL}};
+    {nullptr}};
 
 }  // namespace
 
 const StringTranslationEntry kNetworkTypeTable[] = {
-    // This mapping is ensured in the translation code.
-    //  { network_type::kEthernet, shill::kTypeEthernet },
-    //  { network_type::kEthernet, shill::kTypeEthernetEap },
+    {::onc::network_type::kEthernet, shill::kTypeEthernet},
+    // kTypeEthernetEap is set in onc_translator_onc_to_shill.cc.
+    //  { ::onc::network_type::kEthernet, shill::kTypeEthernetEap },
     {::onc::network_type::kWiFi, shill::kTypeWifi},
-    {::onc::network_type::kWimax, shill::kTypeWimax},
+    // wimax entries are ignored in onc_translator_onc_to_shill.cc.
+    // {::onc::network_type::kWimax, shill::kTypeWimax},
     {::onc::network_type::kCellular, shill::kTypeCellular},
     {::onc::network_type::kVPN, shill::kTypeVPN},
     {::onc::network_type::kTether, kTypeTether},
-    {NULL}};
+    {nullptr}};
 
 const StringTranslationEntry kVPNTypeTable[] = {
     {::onc::vpn::kTypeL2TP_IPsec, shill::kProviderL2tpIpsec},
     {::onc::vpn::kOpenVPN, shill::kProviderOpenVpn},
     {::onc::vpn::kThirdPartyVpn, shill::kProviderThirdPartyVpn},
     {::onc::vpn::kArcVpn, shill::kProviderArcVpn},
-    {NULL}};
+    {nullptr}};
 
 const StringTranslationEntry kWiFiSecurityTable[] = {
     {::onc::wifi::kSecurityNone, shill::kSecurityNone},
@@ -332,21 +337,21 @@ const StringTranslationEntry kWiFiSecurityTable[] = {
     {::onc::wifi::kWPA_PSK, shill::kSecurityPsk},
     {::onc::wifi::kWPA_EAP, shill::kSecurity8021x},
     {::onc::wifi::kWEP_8021X, shill::kSecurityWep},
-    {NULL}};
+    {nullptr}};
 
 const StringTranslationEntry kEAPOuterTable[] = {
     {::onc::eap::kPEAP, shill::kEapMethodPEAP},
     {::onc::eap::kEAP_TLS, shill::kEapMethodTLS},
     {::onc::eap::kEAP_TTLS, shill::kEapMethodTTLS},
     {::onc::eap::kLEAP, shill::kEapMethodLEAP},
-    {NULL}};
+    {nullptr}};
 
 // Translation of the EAP.Inner field in case of EAP.Outer == PEAP
 const StringTranslationEntry kEAP_PEAP_InnerTable[] = {
     {::onc::eap::kGTC, shill::kEapPhase2AuthPEAPGTC},
     {::onc::eap::kMD5, shill::kEapPhase2AuthPEAPMD5},
     {::onc::eap::kMSCHAPv2, shill::kEapPhase2AuthPEAPMSCHAPV2},
-    {NULL}};
+    {nullptr}};
 
 // Translation of the EAP.Inner field in case of EAP.Outer == TTLS
 const StringTranslationEntry kEAP_TTLS_InnerTable[] = {
@@ -355,7 +360,7 @@ const StringTranslationEntry kEAP_TTLS_InnerTable[] = {
     {::onc::eap::kMSCHAP, shill::kEapPhase2AuthTTLSMSCHAP},
     {::onc::eap::kMSCHAPv2, shill::kEapPhase2AuthTTLSMSCHAPV2},
     {::onc::eap::kPAP, shill::kEapPhase2AuthTTLSPAP},
-    {NULL}};
+    {nullptr}};
 
 const StringTranslationEntry kActivationStateTable[] = {
     {::onc::cellular::kActivated, shill::kActivationStateActivated},
@@ -363,7 +368,7 @@ const StringTranslationEntry kActivationStateTable[] = {
     {::onc::cellular::kNotActivated, shill::kActivationStateNotActivated},
     {::onc::cellular::kPartiallyActivated,
      shill::kActivationStatePartiallyActivated},
-    {NULL}};
+    {nullptr}};
 
 const StringTranslationEntry kNetworkTechnologyTable[] = {
     {::onc::cellular::kTechnologyCdma1Xrtt, shill::kNetworkTechnology1Xrtt},
@@ -377,21 +382,21 @@ const StringTranslationEntry kNetworkTechnologyTable[] = {
     {::onc::cellular::kTechnologyLteAdvanced,
      shill::kNetworkTechnologyLteAdvanced},
     {::onc::cellular::kTechnologyUmts, shill::kNetworkTechnologyUmts},
-    {NULL}};
+    {nullptr}};
 
 const StringTranslationEntry kRoamingStateTable[] = {
     {::onc::cellular::kRoamingHome, shill::kRoamingStateHome},
     {::onc::cellular::kRoamingRoaming, shill::kRoamingStateRoaming},
-    {NULL}};
+    {nullptr}};
 
-const StringTranslationEntry kTetheringStateTable[] = {
-    {::onc::tethering_state::kTetheringConfirmedState,
-     shill::kTetheringConfirmedState},
-    {::onc::tethering_state::kTetheringNotDetectedState,
-     shill::kTetheringNotDetectedState},
-    {::onc::tethering_state::kTetheringSuspectedState,
-     shill::kTetheringSuspectedState},
-    {NULL}};
+const StringTranslationEntry kOpenVpnCompressionAlgorithmTable[] = {
+    {::onc::openvpn_compression_algorithm::kFramingOnly,
+     shill::kOpenVPNCompressFramingOnly},
+    {::onc::openvpn_compression_algorithm::kLz4, shill::kOpenVPNCompressLz4},
+    {::onc::openvpn_compression_algorithm::kLz4V2,
+     shill::kOpenVPNCompressLz4V2},
+    {::onc::openvpn_compression_algorithm::kLzo, shill::kOpenVPNCompressLzo},
+    {nullptr}};
 
 // This must contain only Shill Device properties and no Service properties.
 // For Service properties see cellular_fields.
@@ -399,7 +404,6 @@ const FieldTranslationEntry kCellularDeviceTable[] = {
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::cellular::kAPNList, shill::kCellularApnListProperty},
     {::onc::cellular::kAllowRoaming, shill::kCellularAllowRoamingProperty},
-    {::onc::cellular::kCarrier, shill::kCarrierProperty},
     {::onc::cellular::kESN, shill::kEsnProperty},
     {::onc::cellular::kFamily, shill::kTechnologyFamilyProperty},
     {::onc::cellular::kFirmwareRevision, shill::kFirmwareRevisionProperty},
@@ -408,40 +412,40 @@ const FieldTranslationEntry kCellularDeviceTable[] = {
     {::onc::cellular::kHardwareRevision, shill::kHardwareRevisionProperty},
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::cellular::kHomeProvider, shill::kHomeProviderProperty},
-    {::onc::cellular::kICCID, shill::kIccidProperty},
+    // ICCID is only copied from the Device if not provided by the Service.
+    // {::onc::cellular::kICCID, shill::kIccidProperty},
     {::onc::cellular::kIMEI, shill::kImeiProperty},
-    {::onc::cellular::kIMSI, shill::kImsiProperty},
+    // IMSI is only copied from the Device if not provided by the Service.
+    // {::onc::cellular::kIMSI, shill::kImsiProperty},
     {::onc::cellular::kManufacturer, shill::kManufacturerProperty},
     {::onc::cellular::kMDN, shill::kMdnProperty},
     {::onc::cellular::kMEID, shill::kMeidProperty},
     {::onc::cellular::kMIN, shill::kMinProperty},
     {::onc::cellular::kModelID, shill::kModelIdProperty},
-    {::onc::cellular::kPRLVersion, shill::kPRLVersionProperty},
     {::onc::cellular::kScanning, shill::kScanningProperty},
     // This field is converted during translation, see onc_translator_*.
     // { ::onc::cellular::kSIMLockStatus, shill::kSIMLockStatusProperty},
     {::onc::cellular::kSIMPresent, shill::kSIMPresentProperty},
-    {::onc::cellular::kSupportedCarriers, shill::kSupportedCarriersProperty},
     {::onc::cellular::kSupportNetworkScan, shill::kSupportNetworkScanProperty},
-    {NULL}};
+    {nullptr}};
 
 const FieldTranslationEntry* GetFieldTranslationTable(
     const OncValueSignature& onc_signature) {
   for (const OncValueTranslationEntry* it = onc_value_translation_table;
-       it->onc_signature != NULL; ++it) {
+       it->onc_signature != nullptr; ++it) {
     if (it->onc_signature == &onc_signature)
       return it->field_translation_table;
   }
-  return NULL;
+  return nullptr;
 }
 
 std::vector<std::string> GetPathToNestedShillDictionary(
     const OncValueSignature& onc_signature) {
   std::vector<std::string> shill_property_path;
   for (const NestedShillDictionaryEntry* it = nested_shill_dictionaries;
-       it->onc_signature != NULL; ++it) {
+       it->onc_signature != nullptr; ++it) {
     if (it->onc_signature == &onc_signature) {
-      for (const char* const* key = it->shill_property_path; *key != NULL;
+      for (const char* const* key = it->shill_property_path; *key != nullptr;
            ++key) {
         shill_property_path.push_back(std::string(*key));
       }
@@ -454,7 +458,7 @@ std::vector<std::string> GetPathToNestedShillDictionary(
 bool GetShillPropertyName(const std::string& onc_field_name,
                           const FieldTranslationEntry table[],
                           std::string* shill_property_name) {
-  for (const FieldTranslationEntry* it = table; it->onc_field_name != NULL;
+  for (const FieldTranslationEntry* it = table; it->onc_field_name != nullptr;
        ++it) {
     if (it->onc_field_name != onc_field_name)
       continue;
@@ -467,26 +471,30 @@ bool GetShillPropertyName(const std::string& onc_field_name,
 bool TranslateStringToShill(const StringTranslationEntry table[],
                             const std::string& onc_value,
                             std::string* shill_value) {
-  for (int i = 0; table[i].onc_value != NULL; ++i) {
+  for (int i = 0; table[i].onc_value != nullptr; ++i) {
     if (onc_value != table[i].onc_value)
       continue;
     *shill_value = table[i].shill_value;
     return true;
   }
-  LOG(ERROR) << "Value '" << onc_value << "' cannot be translated to Shill";
+  LOG(ERROR) << "Value '" << onc_value << "' cannot be translated to Shill"
+             << " table[0]: " << table[0].onc_value << " => "
+             << table[0].shill_value;
   return false;
 }
 
 bool TranslateStringToONC(const StringTranslationEntry table[],
                           const std::string& shill_value,
                           std::string* onc_value) {
-  for (int i = 0; table[i].shill_value != NULL; ++i) {
+  for (int i = 0; table[i].shill_value != nullptr; ++i) {
     if (shill_value != table[i].shill_value)
       continue;
     *onc_value = table[i].onc_value;
     return true;
   }
-  LOG(ERROR) << "Value '" << shill_value << "' cannot be translated to ONC";
+  LOG(ERROR) << "Value '" << shill_value << "' cannot be translated to ONC"
+             << " table[0]: " << table[0].shill_value << " => "
+             << table[0].onc_value;
   return false;
 }
 
