@@ -8,7 +8,7 @@
 #include <string>
 #include <vector>
 
-#include "chromeos/chromeos_export.h"
+#include "base/component_export.h"
 #include "chromeos/network/onc/onc_signature.h"
 
 namespace chromeos {
@@ -26,16 +26,26 @@ struct StringTranslationEntry {
 
 // These tables contain the mapping from ONC strings to Shill strings.
 // These are NULL-terminated arrays.
-CHROMEOS_EXPORT extern const StringTranslationEntry kNetworkTypeTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kVPNTypeTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kWiFiSecurityTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kEAPOuterTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kEAP_PEAP_InnerTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kEAP_TTLS_InnerTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kActivationStateTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kNetworkTechnologyTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kRoamingStateTable[];
-CHROMEOS_EXPORT extern const StringTranslationEntry kTetheringStateTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kNetworkTypeTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kVPNTypeTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kWiFiSecurityTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kEAPOuterTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kEAP_PEAP_InnerTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kEAP_TTLS_InnerTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kActivationStateTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kNetworkTechnologyTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kRoamingStateTable[];
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+extern const StringTranslationEntry kOpenVpnCompressionAlgorithmTable[];
 
 // A separate translation table for cellular properties that are stored in a
 // Shill Device instead of a Service. The |shill_property_name| entries
@@ -57,15 +67,16 @@ bool GetShillPropertyName(const std::string& onc_field_name,
                           std::string* shill_property_name);
 
 // Translate individual strings to Shill using the above tables.
-CHROMEOS_EXPORT bool TranslateStringToShill(
-    const StringTranslationEntry table[],
-    const std::string& onc_value,
-    std::string* shill_value);
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+bool TranslateStringToShill(const StringTranslationEntry table[],
+                            const std::string& onc_value,
+                            std::string* shill_value);
 
 // Translate individual strings to ONC using the above tables.
-CHROMEOS_EXPORT bool TranslateStringToONC(const StringTranslationEntry table[],
-                                          const std::string& shill_value,
-                                          std::string* onc_value);
+COMPONENT_EXPORT(CHROMEOS_NETWORK)
+bool TranslateStringToONC(const StringTranslationEntry table[],
+                          const std::string& shill_value,
+                          std::string* onc_value);
 
 }  // namespace onc
 }  // namespace chromeos
