@@ -13,10 +13,12 @@
 #include <utility>
 
 #include "base/json/json_reader.h"
+#include "base/json/json_string_value_serializer.h"
 #include "base/json/json_writer.h"
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
+#include "base/notreached.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
 #include "chromeos/network/client_cert_util.h"
@@ -27,33 +29,46 @@
 #include "chromeos/network/onc/onc_utils.h"
 #include "chromeos/network/shill_property_util.h"
 #include "components/onc/onc_constants.h"
+#include "net/base/ip_address.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
+#include "third_party/cros_system_api/dbus/shill/dbus-constants.h"
 
 namespace chromeos {
 namespace onc {
 
 namespace {
 
-std::unique_ptr<base::Value> ConvertValueToString(const base::Value& value) {
+// Converts values to JSON strings. This will turn booleans into "true" or
+// "false" which is what Shill expects for VPN values (including L2TP).
+base::Value ConvertVpnValueToString(const base::Value& value) {
+  if (value.is_string())
+    return value.Clone();
   std::string str;
-  if (!value.GetAsString(&str))
-    base::JSONWriter::Write(value, &str);
-  return std::make_unique<base::Value>(str);
+  CHECK(base::JSONWriter::Write(value, &str));
+  return base::Value(str);
+}
+
+// Returns the string value of |key| from |dict| if found, or the empty string
+// otherwise.
+std::string FindStringKeyOrEmpty(const base::Value* dict,
+                                 base::StringPiece key) {
+  const std::string* value = dict->FindStringKey(key);
+  return value ? *value : std::string();
 }
 
 // Sets any client cert properties when ClientCertType is PKCS11Id.
 void SetClientCertProperties(client_cert::ConfigType config_type,
-                             const base::DictionaryValue* onc_object,
-                             base::DictionaryValue* shill_dictionary) {
-  std::string cert_type;
-  onc_object->GetStringWithoutPathExpansion(::onc::client_cert::kClientCertType,
-                                            &cert_type);
-  if (cert_type != ::onc::client_cert::kPKCS11Id)
+                             const base::Value* onc_object,
+                             base::Value* shill_dictionary) {
+  const std::string cert_type =
+      FindStringKeyOrEmpty(onc_object, ::onc::client_cert::kClientCertType);
+  if (cert_type != ::onc::client_cert::kPKCS11Id) {
+    client_cert::SetEmptyShillProperties(config_type, shill_dictionary);
     return;
+  }
 
-  std::string pkcs11_id;
-  onc_object->GetStringWithoutPathExpansion(
-      ::onc::client_cert::kClientCertPKCS11Id, &pkcs11_id);
+  const std::string pkcs11_id =
+      FindStringKeyOrEmpty(onc_object, ::onc::client_cert::kClientCertPKCS11Id);
   if (pkcs11_id.empty()) {
     // If the cert type is PKCS11 but the pkcs11 id is empty, set empty
     // properties to indicate 'no certificate'.
@@ -76,8 +91,8 @@ void SetClientCertProperties(client_cert::ConfigType config_type,
 class LocalTranslator {
  public:
   LocalTranslator(const OncValueSignature& onc_signature,
-                  const base::DictionaryValue& onc_object,
-                  base::DictionaryValue* shill_dictionary)
+                  const base::Value& onc_object,
+                  base::Value* shill_dictionary)
       : onc_signature_(&onc_signature),
         onc_object_(&onc_object),
         shill_dictionary_(shill_dictionary) {
@@ -94,6 +109,7 @@ class LocalTranslator {
   void TranslateVPN();
   void TranslateWiFi();
   void TranslateEAP();
+  void TranslateStaticIPConfig();
   void TranslateNetworkConfiguration();
 
   // Copies all entries from |onc_object_| to |shill_dictionary_| for which a
@@ -108,9 +124,9 @@ class LocalTranslator {
 
   // Adds |value| to |shill_dictionary| at the field shill_property_name given
   // by the associated signature. Takes ownership of |value|. Does nothing if
-  // |value| is NULL or the property name cannot be read from the signature.
+  // |value| is none or the property name cannot be read from the signature.
   void AddValueAccordingToSignature(const std::string& onc_field_name,
-                                    std::unique_ptr<base::Value> value);
+                                    const base::Value& value);
 
   // Translates the value |onc_value| using |table|. It is an error if no
   // matching table entry is found. Writes the result as entry at
@@ -121,8 +137,8 @@ class LocalTranslator {
 
   const OncValueSignature* onc_signature_;
   const FieldTranslationEntry* field_translation_table_;
-  const base::DictionaryValue* onc_object_;
-  base::DictionaryValue* shill_dictionary_;
+  const base::Value* onc_object_;
+  base::Value* shill_dictionary_;
 
   DISALLOW_COPY_AND_ASSIGN(LocalTranslator);
 };
@@ -144,17 +160,18 @@ void LocalTranslator::TranslateFields() {
     TranslateWiFi();
   else if (onc_signature_ == &kEAPSignature)
     TranslateEAP();
+  else if (onc_signature_ == &kStaticIPConfigSignature)
+    TranslateStaticIPConfig();
   else
     CopyFieldsAccordingToSignature();
 }
 
 void LocalTranslator::TranslateEthernet() {
-  std::string authentication;
-  onc_object_->GetStringWithoutPathExpansion(::onc::ethernet::kAuthentication,
-                                             &authentication);
+  const std::string* authentication =
+      onc_object_->FindStringKey(::onc::ethernet::kAuthentication);
 
   const char* shill_type = shill::kTypeEthernet;
-  if (authentication == ::onc::ethernet::k8021X)
+  if (authentication && *authentication == ::onc::ethernet::k8021X)
     shill_type = shill::kTypeEthernetEap;
   shill_dictionary_->SetKey(shill::kTypeProperty, base::Value(shill_type));
 
@@ -167,9 +184,8 @@ void LocalTranslator::TranslateOpenVPN() {
   CopyFieldFromONCToShill(::onc::vpn::kSaveCredentials,
                           shill::kSaveCredentialsProperty);
 
-  std::string user_auth_type;
-  onc_object_->GetStringWithoutPathExpansion(
-      ::onc::openvpn::kUserAuthenticationType, &user_auth_type);
+  std::string user_auth_type = FindStringKeyOrEmpty(
+      onc_object_, ::onc::openvpn::kUserAuthenticationType);
   // The default behavior (if user_auth_type is empty) is to use both password
   // and OTP in a static challenge and only the password otherwise. As long as
   // Shill doe not know about the exact user authentication type, this is
@@ -190,11 +206,14 @@ void LocalTranslator::TranslateOpenVPN() {
   // Shill supports only one RemoteCertKU but ONC specifies a list, so copy only
   // the first entry if the lists exists. Otherwise copy an empty string to
   // reset any previous configuration.
-  const base::ListValue* cert_kus = nullptr;
+  const base::Value* cert_kus =
+      onc_object_->FindListKey(::onc::openvpn::kRemoteCertKU);
   std::string cert_ku;
-  if (onc_object_->GetListWithoutPathExpansion(::onc::openvpn::kRemoteCertKU,
-                                               &cert_kus)) {
-    cert_kus->GetString(0, &cert_ku);
+  if (cert_kus) {
+    const auto cert_kus_list = cert_kus->GetList();
+    if (!cert_kus_list.empty() && cert_kus_list[0].is_string()) {
+      cert_ku = cert_kus_list[0].GetString();
+    }
   }
   shill_dictionary_->SetKey(shill::kOpenVPNRemoteCertKUProperty,
                             base::Value(cert_ku));
@@ -202,21 +221,29 @@ void LocalTranslator::TranslateOpenVPN() {
   SetClientCertProperties(client_cert::CONFIG_TYPE_OPENVPN, onc_object_,
                           shill_dictionary_);
 
+  const std::string* compression_algorithm =
+      onc_object_->FindStringKey(::onc::openvpn::kCompressionAlgorithm);
+  if (compression_algorithm) {
+    TranslateWithTableAndSet(*compression_algorithm,
+                             kOpenVpnCompressionAlgorithmTable,
+                             shill::kOpenVPNCompressProperty);
+  }
+
   // Modified CopyFieldsAccordingToSignature to handle RemoteCertKU and
   // ServerCAPEMs and handle all other fields as strings.
-  for (base::DictionaryValue::Iterator it(*onc_object_); !it.IsAtEnd();
-       it.Advance()) {
-    std::string key = it.key();
-    std::unique_ptr<base::Value> translated;
+  for (const auto& it : onc_object_->DictItems()) {
+    std::string key = it.first;
+    base::Value translated;
     if (key == ::onc::openvpn::kRemoteCertKU ||
         key == ::onc::openvpn::kServerCAPEMs ||
         key == ::onc::openvpn::kExtraHosts) {
-      translated.reset(it.value().DeepCopy());
+      translated = it.second.Clone();
     } else {
       // Shill wants all Provider/VPN fields to be strings.
-      translated = ConvertValueToString(it.value());
+      translated = ConvertVpnValueToString(it.second);
     }
-    AddValueAccordingToSignature(key, std::move(translated));
+    if (!translated.is_none())
+      AddValueAccordingToSignature(key, translated);
   }
 }
 
@@ -238,28 +265,36 @@ void LocalTranslator::TranslateL2TP() {
   CopyFieldFromONCToShill(::onc::vpn::kSaveCredentials,
                           shill::kSaveCredentialsProperty);
 
+  const base::Value* lcp_echo_disabled =
+      onc_object_->FindKey(::onc::l2tp::kLcpEchoDisabled);
+  if (lcp_echo_disabled) {
+    base::Value lcp_echo_disabled_value =
+        ConvertVpnValueToString(*lcp_echo_disabled);
+    shill_dictionary_->SetKey(shill::kL2tpIpsecLcpEchoDisabledProperty,
+                              std::move(lcp_echo_disabled_value));
+  }
+
   CopyFieldsAccordingToSignature();
 }
 
 void LocalTranslator::TranslateVPN() {
-  std::string onc_type;
-  if (onc_object_->GetStringWithoutPathExpansion(::onc::vpn::kType,
-                                                 &onc_type)) {
-    TranslateWithTableAndSet(onc_type, kVPNTypeTable,
+  const std::string* onc_type = onc_object_->FindStringKey(::onc::vpn::kType);
+  if (onc_type) {
+    TranslateWithTableAndSet(*onc_type, kVPNTypeTable,
                              shill::kProviderTypeProperty);
   }
-  if (onc_type == ::onc::vpn::kThirdPartyVpn) {
+  if (onc_type && *onc_type == ::onc::vpn::kThirdPartyVpn) {
     // For third-party VPNs, |shill::kProviderHostProperty| is used to store the
     // provider's extension ID.
-    const base::DictionaryValue* onc_third_party_vpn = nullptr;
-    onc_object_->GetDictionaryWithoutPathExpansion(::onc::vpn::kThirdPartyVpn,
-                                                   &onc_third_party_vpn);
-    std::string onc_extension_id;
-    if (onc_third_party_vpn &&
-        onc_third_party_vpn->GetStringWithoutPathExpansion(
-            ::onc::third_party_vpn::kExtensionID, &onc_extension_id)) {
-      shill_dictionary_->SetKey(shill::kProviderHostProperty,
-                                base::Value(onc_extension_id));
+    const base::Value* onc_third_party_vpn =
+        onc_object_->FindDictKey(::onc::vpn::kThirdPartyVpn);
+    if (onc_third_party_vpn) {
+      const std::string* onc_extension_id = onc_third_party_vpn->FindStringKey(
+          ::onc::third_party_vpn::kExtensionID);
+      if (onc_extension_id) {
+        shill_dictionary_->SetKey(shill::kProviderHostProperty,
+                                  base::Value(*onc_extension_id));
+      }
     }
   } else {
     CopyFieldFromONCToShill(::onc::vpn::kHost, shill::kProviderHostProperty);
@@ -269,12 +304,12 @@ void LocalTranslator::TranslateVPN() {
 }
 
 void LocalTranslator::TranslateWiFi() {
-  std::string security;
-  if (onc_object_->GetStringWithoutPathExpansion(::onc::wifi::kSecurity,
-                                                 &security)) {
-    TranslateWithTableAndSet(security, kWiFiSecurityTable,
+  const std::string* security =
+      onc_object_->FindStringKey(::onc::wifi::kSecurity);
+  if (security) {
+    TranslateWithTableAndSet(*security, kWiFiSecurityTable,
                              shill::kSecurityClassProperty);
-    if (security == ::onc::wifi::kWEP_8021X) {
+    if (*security == ::onc::wifi::kWEP_8021X) {
       shill_dictionary_->SetKey(shill::kEapKeyMgmtProperty,
                                 base::Value(shill::kKeyManagementIEEE8021X));
     }
@@ -284,30 +319,30 @@ void LocalTranslator::TranslateWiFi() {
   shill_dictionary_->SetKey(shill::kModeProperty,
                             base::Value(shill::kModeManaged));
 
-  bool allow_gateway_arp_polling;
-  if (onc_object_->GetBooleanWithoutPathExpansion(
-          ::onc::wifi::kAllowGatewayARPPolling, &allow_gateway_arp_polling)) {
+  base::Optional<bool> allow_gateway_arp_polling =
+      onc_object_->FindBoolKey(::onc::wifi::kAllowGatewayARPPolling);
+  if (allow_gateway_arp_polling) {
     shill_dictionary_->SetKey(shill::kLinkMonitorDisableProperty,
-                              base::Value(!allow_gateway_arp_polling));
+                              base::Value(!*allow_gateway_arp_polling));
   }
 
   CopyFieldsAccordingToSignature();
 }
 
 void LocalTranslator::TranslateEAP() {
-  // Note: EAP.Outer may be empty for WiMAX configurations.
-  std::string outer;
-  onc_object_->GetStringWithoutPathExpansion(::onc::eap::kOuter, &outer);
+  const std::string outer =
+      FindStringKeyOrEmpty(onc_object_, ::onc::eap::kOuter);
   if (!outer.empty())
     TranslateWithTableAndSet(outer, kEAPOuterTable, shill::kEapMethodProperty);
 
   // Translate the inner protocol only for outer tunneling protocols.
   if (outer == ::onc::eap::kPEAP || outer == ::onc::eap::kEAP_TTLS) {
+    std::string inner = FindStringKeyOrEmpty(onc_object_, ::onc::eap::kInner);
     // In ONC the Inner protocol defaults to "Automatic".
-    std::string inner = ::onc::eap::kAutomatic;
+    if (inner.empty())
+      inner = ::onc::eap::kAutomatic;
     // ONC's Inner == "Automatic" translates to omitting the Phase2 property in
     // Shill.
-    onc_object_->GetStringWithoutPathExpansion(::onc::eap::kInner, &inner);
     if (inner != ::onc::eap::kAutomatic) {
       const StringTranslationEntry* table = outer == ::onc::eap::kPEAP
                                                 ? kEAP_PEAP_InnerTable
@@ -329,44 +364,82 @@ void LocalTranslator::TranslateEAP() {
                               base::Value(true));
   }
 
+  // Set shill::kEapSubjectAlternativeNameMatchProperty to the serialized form
+  // of the subject alternative name match list of dictionaries.
+  const base::Value* subject_alternative_name_match =
+      onc_object_->FindListKey(::onc::eap::kSubjectAlternativeNameMatch);
+  if (subject_alternative_name_match) {
+    base::Value serialized_dicts(base::Value::Type::LIST);
+    std::string serialized_dict;
+    JSONStringValueSerializer serializer(&serialized_dict);
+    for (const base::Value& v : subject_alternative_name_match->GetList()) {
+      if (serializer.Serialize(v)) {
+        serialized_dicts.Append(serialized_dict);
+      }
+    }
+    shill_dictionary_->SetKey(shill::kEapSubjectAlternativeNameMatchProperty,
+                              std::move(serialized_dicts));
+  }
+
   CopyFieldsAccordingToSignature();
 }
 
-void LocalTranslator::TranslateNetworkConfiguration() {
-  std::string type;
-  onc_object_->GetStringWithoutPathExpansion(::onc::network_config::kType,
-                                             &type);
+void LocalTranslator::TranslateStaticIPConfig() {
+  CopyFieldsAccordingToSignature();
+  // Shill expects 4 valid nameserver values. Ensure all values are valid and
+  // replace any invalid values with 0.0.0.0 (which has no effect). See
+  // https://crbug.com/922219 for details.
+  base::Value* name_servers =
+      shill_dictionary_->FindKey(shill::kNameServersProperty);
+  if (name_servers) {
+    static const char kDefaultIpAddr[] = "0.0.0.0";
+    net::IPAddress ip_addr;
+    for (base::Value& value_ref : name_servers->GetList()) {
+      // AssignFromIPLiteral returns true if a string is valid ipv4 or ipv6.
+      if (!ip_addr.AssignFromIPLiteral(value_ref.GetString()))
+        value_ref = base::Value(kDefaultIpAddr);
+    }
+    while (name_servers->GetList().size() < 4)
+      name_servers->Append(base::Value(kDefaultIpAddr));
+  }
+}
 
-  // Set the type except for Ethernet which is set in TranslateEthernet.
-  if (type != ::onc::network_type::kEthernet)
-    TranslateWithTableAndSet(type, kNetworkTypeTable, shill::kTypeProperty);
+void LocalTranslator::TranslateNetworkConfiguration() {
+  const std::string type =
+      FindStringKeyOrEmpty(onc_object_, ::onc::network_config::kType);
+  if (type == ::onc::network_type::kWimaxDeprecated) {
+    NET_LOG(ERROR) << "WiMAX ONC configuration is no longer supported.";
+    return;
+  }
+
+  // Note; The Ethernet type might be overridden to EthernetEap in
+  // TranslateEthernet if Ethernet specific properties are provided.
+  TranslateWithTableAndSet(type, kNetworkTypeTable, shill::kTypeProperty);
 
   // Shill doesn't allow setting the name for non-VPN networks.
   if (type == ::onc::network_type::kVPN)
     CopyFieldFromONCToShill(::onc::network_config::kName, shill::kNameProperty);
 
-  std::string ip_address_config_type, name_servers_config_type;
-  onc_object_->GetStringWithoutPathExpansion(
-      ::onc::network_config::kIPAddressConfigType, &ip_address_config_type);
-  onc_object_->GetStringWithoutPathExpansion(
-      ::onc::network_config::kNameServersConfigType, &name_servers_config_type);
+  const std::string ip_address_config_type = FindStringKeyOrEmpty(
+      onc_object_, ::onc::network_config::kIPAddressConfigType);
+  const std::string name_servers_config_type = FindStringKeyOrEmpty(
+      onc_object_, ::onc::network_config::kNameServersConfigType);
   if ((ip_address_config_type == ::onc::network_config::kIPConfigTypeDHCP) ||
       (name_servers_config_type == ::onc::network_config::kIPConfigTypeDHCP)) {
     // If either type is set to DHCP, provide an empty dictionary to ensure
     // that any unset properties are cleared. Note: if either type is specified,
     // the other type defaults to DHCP if not specified.
-    shill_dictionary_->SetWithoutPathExpansion(
-        shill::kStaticIPConfigProperty,
-        std::make_unique<base::DictionaryValue>());
+    shill_dictionary_->SetKey(shill::kStaticIPConfigProperty,
+                              base::Value(base::Value::Type::DICTIONARY));
   }
 
-  const base::DictionaryValue* proxy_settings = nullptr;
-  if (onc_object_->GetDictionaryWithoutPathExpansion(
-          ::onc::network_config::kProxySettings, &proxy_settings)) {
-    std::unique_ptr<base::DictionaryValue> proxy_config =
+  const base::Value* proxy_settings =
+      onc_object_->FindDictKey(::onc::network_config::kProxySettings);
+  if (proxy_settings) {
+    base::Value proxy_config =
         ConvertOncProxySettingsToProxyConfig(*proxy_settings);
     std::string proxy_config_str;
-    base::JSONWriter::Write(*proxy_config.get(), &proxy_config_str);
+    base::JSONWriter::Write(proxy_config, &proxy_config_str);
     shill_dictionary_->SetKey(shill::kProxyConfigProperty,
                               base::Value(proxy_config_str));
   }
@@ -375,18 +448,16 @@ void LocalTranslator::TranslateNetworkConfiguration() {
 }
 
 void LocalTranslator::CopyFieldsAccordingToSignature() {
-  for (base::DictionaryValue::Iterator it(*onc_object_); !it.IsAtEnd();
-       it.Advance()) {
-    AddValueAccordingToSignature(it.key(),
-                                 base::WrapUnique(it.value().DeepCopy()));
+  for (const auto& it : onc_object_->DictItems()) {
+    AddValueAccordingToSignature(it.first, it.second);
   }
 }
 
 void LocalTranslator::CopyFieldFromONCToShill(
     const std::string& onc_field_name,
     const std::string& shill_property_name) {
-  const base::Value* value = NULL;
-  if (!onc_object_->GetWithoutPathExpansion(onc_field_name, &value))
+  const base::Value* value = onc_object_->FindKey(onc_field_name);
+  if (!value)
     return;
 
   const OncFieldSignature* field_signature =
@@ -407,19 +478,16 @@ void LocalTranslator::CopyFieldFromONCToShill(
   shill_dictionary_->SetKey(shill_property_name, value->Clone());
 }
 
-void LocalTranslator::AddValueAccordingToSignature(
-    const std::string& onc_name,
-    std::unique_ptr<base::Value> value) {
-  if (!value || !field_translation_table_)
+void LocalTranslator::AddValueAccordingToSignature(const std::string& onc_name,
+                                                   const base::Value& value) {
+  if (value.is_none() || !field_translation_table_)
     return;
   std::string shill_property_name;
   if (!GetShillPropertyName(onc_name, field_translation_table_,
                             &shill_property_name)) {
     return;
   }
-
-  shill_dictionary_->SetWithoutPathExpansion(shill_property_name,
-                                             std::move(value));
+  shill_dictionary_->SetKey(shill_property_name, value.Clone());
 }
 
 void LocalTranslator::TranslateWithTableAndSet(
@@ -443,7 +511,7 @@ void LocalTranslator::TranslateWithTableAndSet(
 // applies the local translation using LocalTranslator::TranslateFields. The
 // results are written to |shill_dictionary|.
 void TranslateONCHierarchy(const OncValueSignature& signature,
-                           const base::DictionaryValue& onc_object,
+                           const base::Value& onc_object,
                            base::Value* shill_dictionary) {
   const std::vector<std::string> path =
       GetPathToNestedShillDictionary(signature);
@@ -457,22 +525,21 @@ void TranslateONCHierarchy(const OncValueSignature& signature,
 
   // Translates fields of |onc_object| and writes them to
   // |target_shill_dictionary_| nested in |shill_dictionary|.
-  LocalTranslator translator(
-      signature, onc_object,
-      static_cast<base::DictionaryValue*>(target_shill_dictionary));
+  LocalTranslator translator(signature, onc_object, target_shill_dictionary);
   translator.TranslateFields();
 
   // Recurse into nested objects.
-  for (base::DictionaryValue::Iterator it(onc_object); !it.IsAtEnd();
-       it.Advance()) {
-    const base::DictionaryValue* inner_object = NULL;
-    if (!it.value().GetAsDictionary(&inner_object))
+  for (const auto& it : onc_object.DictItems()) {
+    if (!it.second.is_dict())
       continue;
 
     const OncFieldSignature* field_signature =
-        GetFieldSignature(signature, it.key());
-
-    TranslateONCHierarchy(*field_signature->value_signature, *inner_object,
+        GetFieldSignature(signature, it.first);
+    if (!field_signature) {
+      NET_LOG(ERROR) << "Unexpected or deprecated ONC key: " << it.first;
+      continue;
+    }
+    TranslateONCHierarchy(*field_signature->value_signature, it.second,
                           shill_dictionary);
   }
 }
@@ -481,7 +548,7 @@ void TranslateONCHierarchy(const OncValueSignature& signature,
 
 std::unique_ptr<base::DictionaryValue> TranslateONCObjectToShill(
     const OncValueSignature* onc_signature,
-    const base::DictionaryValue& onc_object) {
+    const base::Value& onc_object) {
   CHECK(onc_signature != NULL);
   std::unique_ptr<base::DictionaryValue> shill_dictionary(
       new base::DictionaryValue);
