@@ -10,21 +10,44 @@
 #include <string>
 #include <vector>
 
+#include "base/component_export.h"
 #include "base/macros.h"
-#include "chromeos/chromeos_export.h"
 #include "chromeos/network/onc/onc_mapper.h"
 #include "components/onc/onc_constants.h"
 
 namespace base {
 class DictionaryValue;
 class Value;
-}
+}  // namespace base
 
 namespace chromeos {
 namespace onc {
 
 struct OncValueSignature;
 
+// *** ONC Validator Modes of Operation ***
+// The ONC validator supports different modes of operation depending on the
+// combination of flags passed to the constructor.
+//
+// ** |log_warnings| **
+// If this flag is set to true, warnings will be logged.
+//
+// ** |error_on_unknown_field| **
+// If this flag is set to true, and error will be logged in case of unknown
+// fields are encountered in the ONC to be validated and the validation will
+// fail. If it is set to false, a warning will be logged instead and the
+// validation will not fail for that cause.
+//
+// ** |error_on_wrong_recommended| **
+// If this flag is set to true, an error will be logged and the validation will
+// fail in case of encountering recommended fields that are not expected to be
+// recommended. If it is set to false, a warning will be logged instead and the
+// validation will not fail for that cause.
+//
+// ** |managed_onc| **
+// ONC set by policy are validated differently from ONC set through UI.
+// Set this flag to true if policy is the source of the ONC to be validated.
+//
 // The ONC Validator searches for the following invalid cases:
 // - a value is found that has the wrong type or is not expected according to
 //   the ONC spec (always an error)
@@ -56,19 +79,26 @@ struct OncValueSignature;
 //
 // If no error occurred, |result| is set to VALID and an exact DeepCopy is
 // returned.
-class CHROMEOS_EXPORT Validator : public Mapper {
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) Validator : public Mapper {
  public:
-  enum Result {
-    VALID,
-    VALID_WITH_WARNINGS,
-    INVALID
+  enum Result { VALID, VALID_WITH_WARNINGS, INVALID };
+
+  struct ValidationIssue {
+    // If true, the ONC value does not adhere to the specification and may be
+    // rejected.
+    bool is_error;
+
+    // Detailed description containing the error/warning type and specific
+    // location (path).
+    std::string message;
   };
 
   // See the class comment.
   Validator(bool error_on_unknown_field,
             bool error_on_wrong_recommended,
             bool error_on_missing_field,
-            bool managed_onc);
+            bool managed_onc,
+            bool log_warnings);
 
   ~Validator() override;
 
@@ -77,9 +107,7 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   // checks:
   // - only the network types Wifi and Ethernet are allowed
   // - client certificate patterns are disallowed
-  void SetOncSource(::onc::ONCSource source) {
-    onc_source_ = source;
-  }
+  void SetOncSource(::onc::ONCSource source) { onc_source_ = source; }
 
   // Validate the given |onc_object| dictionary according to |object_signature|.
   // The |object_signature| has to be a pointer to one of the signatures in
@@ -99,6 +127,12 @@ class CHROMEOS_EXPORT Validator : public Mapper {
       const base::Value& onc_object,
       Result* result);
 
+  // Returns the list of validation results that occured within validation
+  // initiated by the last call to |ValidateAndRepairObject|.
+  const std::vector<ValidationIssue>& validation_issues() const {
+    return validation_issues_;
+  }
+
  private:
   // Overridden from Mapper:
   // Compare |onc_value|s type with |onc_type| and validate/repair according to
@@ -113,7 +147,7 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   // repaired dictionary. Only on error returns NULL.
   std::unique_ptr<base::DictionaryValue> MapObject(
       const OncValueSignature& signature,
-      const base::DictionaryValue& onc_object,
+      const base::Value& onc_object,
       bool* error) override;
 
   // Pushes/pops the |field_name| to |path_|, otherwise like |Mapper::MapField|.
@@ -141,7 +175,7 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   // |onc_object| according to |object_signature|. |result| must point to a
   // dictionary into which the repaired fields are written.
   bool ValidateObjectDefault(const OncValueSignature& object_signature,
-                             const base::DictionaryValue& onc_object,
+                             const base::Value& onc_object,
                              base::DictionaryValue* result);
 
   // Validates/repairs the kRecommended array in |result| according to
@@ -158,7 +192,9 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   bool ValidateToplevelConfiguration(base::DictionaryValue* result);
   bool ValidateNetworkConfiguration(base::DictionaryValue* result);
   bool ValidateEthernet(base::DictionaryValue* result);
-  bool ValidateIPConfig(base::DictionaryValue* result);
+  bool ValidateIPConfig(base::DictionaryValue* result,
+                        bool require_fields = true);
+  bool ValidateNameServersConfig(base::DictionaryValue* result);
   bool ValidateWiFi(base::DictionaryValue* result);
   bool ValidateVPN(base::DictionaryValue* result);
   bool ValidateIPsec(base::DictionaryValue* result);
@@ -171,7 +207,9 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   bool ValidateProxySettings(base::DictionaryValue* result);
   bool ValidateProxyLocation(base::DictionaryValue* result);
   bool ValidateEAP(base::DictionaryValue* result);
+  bool ValidateSubjectAlternativeNameMatch(base::DictionaryValue* result);
   bool ValidateCertificate(base::DictionaryValue* result);
+  bool ValidateScope(base::DictionaryValue* result);
   bool ValidateTether(base::DictionaryValue* result);
 
   bool IsValidValue(const std::string& field_value,
@@ -186,13 +224,28 @@ class CHROMEOS_EXPORT Validator : public Mapper {
       const std::vector<const char*>& valid_values);
 
   bool FieldExistsAndIsNotInRange(const base::DictionaryValue& object,
-                                  const std::string &field_name,
+                                  const std::string& field_name,
                                   int lower_bound,
                                   int upper_bound);
 
   bool FieldExistsAndIsEmpty(const base::DictionaryValue& object,
                              const std::string& field_name);
 
+  // Validates 'StaticIPConfig' field of the given network configuration. This
+  // method takes 'NetworkConfiguration' dict instead of 'StaticIPConfig' dict
+  // because it needs other 'NetworkConfiguration' fields (e.g.
+  // 'IPAddressConfigType' and 'NameServersConfigType') to check correctness of
+  // the 'StaticIPConfig' field.
+  bool NetworkHasCorrectStaticIPConfig(base::DictionaryValue* network);
+
+  // Validates that the given field either exists or is recommended.
+  bool FieldShouldExistOrBeRecommended(const base::DictionaryValue& object,
+                                       const std::string& field_name);
+
+  bool OnlyOneFieldSet(const base::DictionaryValue& object,
+                       const std::string& field_name1,
+                       const std::string& field_name2);
+
   bool ListFieldContainsValidValues(
       const base::DictionaryValue& object,
       const std::string& field_name,
@@ -201,8 +254,7 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   bool ValidateSSIDAndHexSSID(base::DictionaryValue* object);
 
   // Returns true if |key| is a key of |dict|. Otherwise, returns false and,
-  // depending on |error_on_missing_field_|, logs a message and sets
-  // |error_or_warning_found_|.
+  // depending on |error_on_missing_field_| raises an error or a warning.
   bool RequireField(const base::DictionaryValue& dict, const std::string& key);
 
   // Returns true if the GUID is unique or if the GUID is not a string
@@ -210,18 +262,18 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   // order to identify duplicates.
   bool CheckGuidIsUniqueAndAddToSet(const base::DictionaryValue& dict,
                                     const std::string& kGUID,
-                                    std::set<std::string> *guids);
+                                    std::set<std::string>* guids);
 
   // Prohibit global network configuration in user ONC imports.
-  bool IsGlobalNetworkConfigInUserImport(
-      const base::DictionaryValue& onc_object);
+  bool IsGlobalNetworkConfigInUserImport(const base::Value& onc_object);
 
-  std::string MessageHeader();
+  void AddValidationIssue(bool is_error, const std::string& debug_info);
 
   const bool error_on_unknown_field_;
   const bool error_on_wrong_recommended_;
   const bool error_on_missing_field_;
   const bool managed_onc_;
+  const bool log_warnings_;
 
   ::onc::ONCSource onc_source_;
 
@@ -237,9 +289,9 @@ class CHROMEOS_EXPORT Validator : public Mapper {
   // duplicate GUIDs.
   std::set<std::string> certificate_guids_;
 
-  // Tracks if an error or warning occurred within validation initiated by
+  // List of all validation issues that occured within validation initiated by
   // function ValidateAndRepairObject.
-  bool error_or_warning_found_;
+  std::vector<ValidationIssue> validation_issues_;
 
   DISALLOW_COPY_AND_ASSIGN(Validator);
 };
