@@ -8,7 +8,7 @@
 #include <string>
 #include <utility>
 
-#include "base/logging.h"
+#include "base/check.h"
 #include "base/memory/ptr_util.h"
 #include "base/values.h"
 #include "chromeos/network/onc/onc_signature.h"
@@ -28,33 +28,40 @@ class ONCValidatorTest : public ::testing::Test {
   // validation is stored, so that expectations can be checked afterwards using
   // one of the Expect* functions below.
   void Validate(bool strict,
-                std::unique_ptr<base::DictionaryValue> onc_object,
+                base::Value onc_object,
                 const OncValueSignature* signature,
                 bool managed_onc,
                 ::onc::ONCSource onc_source) {
     std::unique_ptr<Validator> validator;
     if (strict) {
       // Create a strict validator that complains about every error.
-      validator.reset(new Validator(true, true, true, managed_onc));
+      validator =
+          std::make_unique<Validator>(true,  // error_on_unknown_field
+                                      true,  // error_on_wrong_recommended
+                                      true,  // error_on_missing_field,
+                                      managed_onc,  // managed_onc
+                                      true);        // log_warnings
     } else {
       // Create a liberal validator that ignores or repairs non-critical errors.
-      validator.reset(new Validator(false, false, false, managed_onc));
+      validator =
+          std::make_unique<Validator>(false,  // error_on_unknown_field
+                                      false,  // error_on_wrong_recommended
+                                      false,  // error_on_missing_field,
+                                      managed_onc,  // managed_onc
+                                      true);        // log_warnings
     }
     validator->SetOncSource(onc_source);
     original_object_ = std::move(onc_object);
-    repaired_object_ = validator->ValidateAndRepairObject(signature,
-                                                          *original_object_,
-                                                          &validation_result_);
+    repaired_object_ = validator->ValidateAndRepairObject(
+        signature, original_object_, &validation_result_);
   }
 
   void ExpectValid() {
     EXPECT_EQ(Validator::VALID, validation_result_);
-    EXPECT_TRUE(test_utils::Equals(original_object_.get(),
-                                   repaired_object_.get()));
+    EXPECT_TRUE(test_utils::Equals(&original_object_, repaired_object_.get()));
   }
 
-  void ExpectRepairWithWarnings(
-      const base::DictionaryValue& expected_repaired) {
+  void ExpectRepairWithWarnings(const base::Value& expected_repaired) {
     EXPECT_EQ(Validator::VALID_WITH_WARNINGS, validation_result_);
     EXPECT_TRUE(test_utils::Equals(&expected_repaired, repaired_object_.get()));
   }
@@ -66,7 +73,7 @@ class ONCValidatorTest : public ::testing::Test {
 
  private:
   Validator::Result validation_result_;
-  std::unique_ptr<const base::DictionaryValue> original_object_;
+  base::Value original_object_;
   std::unique_ptr<const base::DictionaryValue> repaired_object_;
 };
 
@@ -82,8 +89,7 @@ struct OncParams {
       : location(location_of_object),
         signature(onc_signature),
         is_managed(is_managed_onc),
-        onc_source(onc_source) {
-  }
+        onc_source(onc_source) {}
 
   std::string location;
   const OncValueSignature* signature;
@@ -115,15 +121,15 @@ class ONCValidatorValidTest : public ONCValidatorTest,
 
 TEST_P(ONCValidatorValidTest, StrictValidationValid) {
   OncParams onc = GetParam();
-  Validate(true, test_utils::ReadTestDictionary(onc.location), onc.signature,
-           onc.is_managed, onc.onc_source);
+  Validate(true, test_utils::ReadTestDictionaryValue(onc.location),
+           onc.signature, onc.is_managed, onc.onc_source);
   ExpectValid();
 }
 
 TEST_P(ONCValidatorValidTest, LiberalValidationValid) {
   OncParams onc = GetParam();
-  Validate(false, test_utils::ReadTestDictionary(onc.location), onc.signature,
-           onc.is_managed, onc.onc_source);
+  Validate(false, test_utils::ReadTestDictionaryValue(onc.location),
+           onc.signature, onc.is_managed, onc.onc_source);
   ExpectValid();
 }
 
@@ -131,7 +137,7 @@ TEST_P(ONCValidatorValidTest, LiberalValidationValid) {
 // OncParams(string: Filename of a ONC file that is to be validated,
 //           OncValueSignature: signature of that ONC,
 //           bool: true if the ONC is managed).
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     ONCValidatorValidTest,
     ONCValidatorValidTest,
     ::testing::Values(
@@ -163,6 +169,10 @@ INSTANTIATE_TEST_CASE_P(
         OncParams("managed_toplevel_l2tpipsec.onc",
                   &kToplevelConfigurationSignature,
                   true),
+        OncParams("managed_toplevel_l2tpipsec.onc",
+                  &kToplevelConfigurationSignature,
+                  true,
+                  ::onc::ONC_SOURCE_DEVICE_POLICY),
         OncParams("managed_toplevel_with_server_and_ca_cert.onc",
                   &kToplevelConfigurationSignature,
                   true),
@@ -214,9 +224,6 @@ INSTANTIATE_TEST_CASE_P(
         OncParams("translation_of_shill_cellular_with_state.onc",
                   &kNetworkWithStateSignature,
                   false),
-        OncParams("translation_of_shill_wimax_with_state.onc",
-                  &kNetworkWithStateSignature,
-                  false),
         OncParams("valid_openvpn_with_cert_pems.onc",
                   &kNetworkConfigurationSignature,
                   false),
@@ -227,7 +234,11 @@ INSTANTIATE_TEST_CASE_P(
                   &kNetworkConfigurationSignature,
                   false),
         OncParams("arc_vpn.onc", &kNetworkConfigurationSignature, false),
-        OncParams("tether.onc", &kNetworkWithStateSignature, false)));
+        OncParams("tether.onc", &kNetworkWithStateSignature, false),
+        OncParams("cert_with_valid_scope.onc", &kCertificateSignature, false),
+        OncParams("cert_with_explicit_default_scope.onc",
+                  &kCertificateSignature,
+                  false)));
 
 namespace {
 
@@ -252,13 +263,6 @@ RepairParams ExpectBothNotValid(const std::string& strict_repaired,
   return RepairParams(strict_repaired, liberal_repaired, false);
 }
 
-// |strict_repaired| is a string to identify the object that is expected as the
-// validation result. They may either be used
-// as filenames or as dictionary keys.
-RepairParams ExpectStrictNotValid(const std::string& strict_repaired) {
-  return RepairParams(strict_repaired, std::string(), true);
-}
-
 ::std::ostream& operator<<(::std::ostream& os, const RepairParams& rp) {
   if (rp.expect_liberal_valid) {
     os << "(" << rp.location_of_strict_repaired << ", liberal is valid)";
@@ -283,13 +287,12 @@ class ONCValidatorTestRepairable
  public:
   // Load the common test data and return the dictionary at the field with
   // name |name|.
-  std::unique_ptr<base::DictionaryValue> GetDictionaryFromTestFile(
-      const std::string& name) {
-    std::unique_ptr<const base::DictionaryValue> dict(
-        test_utils::ReadTestDictionary("invalid_settings_with_repairs.json"));
-    const base::DictionaryValue* onc_object = NULL;
-    CHECK(dict->GetDictionary(name, &onc_object));
-    return base::WrapUnique(onc_object->DeepCopy());
+  base::Value GetDictionaryFromTestFile(const std::string& name) {
+    base::Value dict = test_utils::ReadTestDictionaryValue(
+        "invalid_settings_with_repairs.json");
+    base::Value* result = dict.FindKey(name);
+    EXPECT_TRUE(result);
+    return result ? std::move(*result) : base::Value();
   }
 };
 
@@ -302,7 +305,7 @@ TEST_P(ONCValidatorTestRepairable, StrictValidation) {
   if (location_of_repaired.empty())
     ExpectInvalid();
   else
-    ExpectRepairWithWarnings(*GetDictionaryFromTestFile(location_of_repaired));
+    ExpectRepairWithWarnings(GetDictionaryFromTestFile(location_of_repaired));
 }
 
 TEST_P(ONCValidatorTestRepairable, LiberalValidation) {
@@ -317,8 +320,7 @@ TEST_P(ONCValidatorTestRepairable, LiberalValidation) {
     if (location_of_repaired.empty())
       ExpectInvalid();
     else
-      ExpectRepairWithWarnings(
-          *GetDictionaryFromTestFile(location_of_repaired));
+      ExpectRepairWithWarnings(GetDictionaryFromTestFile(location_of_repaired));
   }
 }
 
@@ -338,42 +340,66 @@ TEST_P(ONCValidatorTestRepairable, LiberalValidation) {
 //                      "invalid_settings_with_repairs.json". That nested
 //                      dictionary is the expected result from liberal
 //                      validation).
-//
-// If liberal valiation is expected to return VALID and strict validation is
-// expected to be not valid:
-//  ExpectStrictNotValid(string: A fieldname in the dictionary from the file
-//                        "invalid_settings_with_repairs.json". That nested
-//                        dictionary is the expected result from strict
-//                        validation).
 
-// Strict validator returns INVALID. Liberal validator returns VALID.
-INSTANTIATE_TEST_CASE_P(
-    StrictInvalidLiberalValid,
+// Strict validator returns INVALID. Liberal validator returns
+// VALID_WITH_WARNINGS (unrepaired).
+INSTANTIATE_TEST_SUITE_P(
+    StrictInvalidLiberalValidWithWarnings,
     ONCValidatorTestRepairable,
     ::testing::Values(
         std::make_pair(OncParams("network-missing-required",
                                  &kNetworkConfigurationSignature,
                                  false),
-                       ExpectStrictNotValid("")),
+                       ExpectBothNotValid("", "network-missing-required")),
         std::make_pair(OncParams("network-missing-required-type",
                                  &kNetworkConfigurationSignature,
                                  false),
-                       ExpectStrictNotValid("")),
+                       ExpectBothNotValid("", "network-missing-required-type")),
         std::make_pair(OncParams("managed-network-missing-required",
                                  &kNetworkConfigurationSignature,
                                  true),
-                       ExpectStrictNotValid("")),
+                       ExpectBothNotValid("",
+                                          "managed-network-missing-required")),
         std::make_pair(OncParams("openvpn-missing-verify-x509-name",
                                  &kNetworkConfigurationSignature,
                                  false),
-                       ExpectStrictNotValid("")),
-        std::make_pair(OncParams("third-party-vpn-missing-extension-id",
+                       ExpectBothNotValid("",
+                                          "openvpn-missing-verify-x509-name")),
+        std::make_pair(
+            OncParams("third-party-vpn-missing-extension-id",
+                      &kNetworkConfigurationSignature,
+                      false),
+            ExpectBothNotValid("", "third-party-vpn-missing-extension-id")),
+        std::make_pair(OncParams("managed-network-invalid-vpn-type-third-party",
                                  &kNetworkConfigurationSignature,
-                                 false),
-                       ExpectStrictNotValid(""))));
+                                 true),
+                       ExpectBothNotValid("", "")),
+        std::make_pair(OncParams("managed-network-invalid-vpn-type-arc",
+                                 &kNetworkConfigurationSignature,
+                                 true),
+                       ExpectBothNotValid("", "")),
+        std::make_pair(OncParams("tether-missing-battery-percentage",
+                                 &kNetworkWithStateSignature,
+                                 true),
+                       ExpectBothNotValid("",
+                                          "tether-missing-battery-percentage")),
+        std::make_pair(OncParams("tether-missing-carrier",
+                                 &kNetworkWithStateSignature,
+                                 true),
+                       ExpectBothNotValid("", "tether-missing-carrier")),
+        std::make_pair(
+            OncParams("tether-missing-has-connected-to-host",
+                      &kNetworkWithStateSignature,
+                      true),
+            ExpectBothNotValid("", "tether-missing-has-connected-to-host")),
+        std::make_pair(OncParams("tether-missing-signal-strength",
+                                 &kNetworkWithStateSignature,
+                                 true),
+                       ExpectBothNotValid("",
+                                          "tether-missing-signal-strength"))));
 
 // Strict validator returns INVALID. Liberal validator repairs.
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     StrictInvalidLiberalRepair,
     ONCValidatorTestRepairable,
     ::testing::Values(
@@ -421,7 +447,7 @@ INSTANTIATE_TEST_CASE_P(
                                           "toplevel-with-repaired-networks"))));
 
 // Strict and liberal validator repair identically.
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     StrictAndLiberalRepairIdentically,
     ONCValidatorTestRepairable,
     ::testing::Values(
@@ -451,11 +477,6 @@ INSTANTIATE_TEST_CASE_P(
                                  false),
                        ExpectBothNotValid("network-repaired",
                                           "network-repaired")),
-        std::make_pair(OncParams("toplevel-with-vpn",
-                                 &kToplevelConfigurationSignature,
-                                 false,
-                                 ::onc::ONC_SOURCE_DEVICE_POLICY),
-                       ExpectBothNotValid("toplevel-empty", "toplevel-empty")),
         std::make_pair(OncParams("wifi-ssid-and-hexssid-inconsistent",
                                  &kNetworkConfigurationSignature,
                                  false),
@@ -468,7 +489,7 @@ INSTANTIATE_TEST_CASE_P(
                                           "wifi-ssid-and-hexssid-repaired"))));
 
 // Strict and liberal validator both repair, but with different results.
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     StrictAndLiberalRepairDifferently,
     ONCValidatorTestRepairable,
     ::testing::Values(std::make_pair(OncParams("toplevel-with-nested-warning",
@@ -478,7 +499,7 @@ INSTANTIATE_TEST_CASE_P(
                                                         "toplevel-repaired"))));
 
 // Strict and liberal validator return both INVALID.
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     StrictAndLiberalInvalid,
     ONCValidatorTestRepairable,
     ::testing::Values(
@@ -544,10 +565,6 @@ INSTANTIATE_TEST_CASE_P(
                                  &kNetworkConfigurationSignature,
                                  false),
                        ExpectBothNotValid("", "")),
-        std::make_pair(OncParams("tether-missing-battery-percentage",
-                                 &kNetworkWithStateSignature,
-                                 true),
-                       ExpectBothNotValid("", "")),
         std::make_pair(OncParams("tether-negative-battery",
                                  &kNetworkWithStateSignature,
                                  true),
@@ -556,18 +573,6 @@ INSTANTIATE_TEST_CASE_P(
                                  &kNetworkWithStateSignature,
                                  true),
                        ExpectBothNotValid("", "")),
-        std::make_pair(OncParams("tether-missing-carrier",
-                                 &kNetworkWithStateSignature,
-                                 true),
-                       ExpectBothNotValid("", "")),
-        std::make_pair(OncParams("tether-missing-has-connected-to-host",
-                                 &kNetworkWithStateSignature,
-                                 true),
-                       ExpectBothNotValid("", "")),
-        std::make_pair(OncParams("tether-missing-signal-strength",
-                                 &kNetworkWithStateSignature,
-                                 true),
-                       ExpectBothNotValid("", "")),
         std::make_pair(OncParams("tether-negative-signal-strength",
                                  &kNetworkWithStateSignature,
                                  true),
@@ -575,7 +580,28 @@ INSTANTIATE_TEST_CASE_P(
         std::make_pair(OncParams("tether-signal-strength-over-100",
                                  &kNetworkWithStateSignature,
                                  true),
-                       ExpectBothNotValid("", ""))));
+                       ExpectBothNotValid("", "")),
+        std::make_pair(
+            OncParams("invalid-scope-due-to-type", &kScopeSignature, true),
+            ExpectBothNotValid("", "")),
+        std::make_pair(OncParams("invalid-scope-due-to-missing-id",
+                                 &kScopeSignature,
+                                 true),
+                       ExpectBothNotValid("",
+                                          "invalid-scope-due-to-missing-id")),
+        std::make_pair(OncParams("invalid-scope-due-to-invalid-id-length",
+                                 &kScopeSignature,
+                                 true),
+                       ExpectBothNotValid("", "")),
+        std::make_pair(OncParams("invalid-scope-due-to-invalid-id-character",
+                                 &kScopeSignature,
+                                 true),
+                       ExpectBothNotValid("", "")),
+        std::make_pair(
+            OncParams("invalid-scope-due-to-missing-type",
+                      &kScopeSignature,
+                      true),
+            ExpectBothNotValid("", "invalid-scope-due-to-missing-type"))));
 
 }  // namespace onc
 }  // namespace chromeos
