@@ -10,7 +10,8 @@
 
 namespace chromeos {
 
-class CHROMEOS_EXPORT NetworkPortalDetectorStub : public NetworkPortalDetector {
+class COMPONENT_EXPORT(CHROMEOS_NETWORK) NetworkPortalDetectorStub
+    : public NetworkPortalDetector {
  public:
   NetworkPortalDetectorStub();
   ~NetworkPortalDetectorStub() override;
@@ -20,11 +21,10 @@ class CHROMEOS_EXPORT NetworkPortalDetectorStub : public NetworkPortalDetector {
   void AddObserver(Observer* observer) override;
   void AddAndFireObserver(Observer* observer) override;
   void RemoveObserver(Observer* observer) override;
-  CaptivePortalState GetCaptivePortalState(
-      const std::string& service_path) override;
+  CaptivePortalStatus GetCaptivePortalStatus() override;
   bool IsEnabled() override;
   void Enable(bool start_detection) override;
-  bool StartPortalDetection(bool force) override;
+  void StartPortalDetection() override;
   void SetStrategy(PortalDetectorStrategy::StrategyId id) override;
 
   DISALLOW_COPY_AND_ASSIGN(NetworkPortalDetectorStub);
