@@ -11,8 +11,7 @@
 #include "base/json/json_writer.h"
 #include "base/logging.h"
 #include "base/values.h"
-#include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/shill_service_client.h"
+#include "chromeos/dbus/shill/shill_service_client.h"
 #include "chromeos/network/network_handler_callbacks.h"
 #include "chromeos/network/network_profile.h"
 #include "chromeos/network/network_profile_handler.h"
@@ -46,6 +45,7 @@ std::unique_ptr<ProxyConfigDictionary> GetProxyConfigForNetwork(
     const PrefService* profile_prefs,
     const PrefService* local_state_prefs,
     const NetworkState& network,
+    const NetworkProfileHandler* network_profile_handler,
     ::onc::ONCSource* onc_source) {
   const base::DictionaryValue* network_policy = onc::GetPolicyForNetwork(
       profile_prefs, local_state_prefs, network, onc_source);
@@ -68,8 +68,7 @@ std::unique_ptr<ProxyConfigDictionary> GetProxyConfigForNetwork(
     return nullptr;
 
   const NetworkProfile* profile =
-      NetworkHandler::Get()->network_profile_handler()->GetProfileForPath(
-          network.profile_path());
+      network_profile_handler->GetProfileForPath(network.profile_path());
   if (!profile) {
     VLOG(1) << "Unknown profile_path '" << network.profile_path() << "'.";
     return nullptr;
@@ -88,39 +87,36 @@ std::unique_ptr<ProxyConfigDictionary> GetProxyConfigForNetwork(
   // unshared) configuration.
   // The user's proxy setting is not stored in the Chrome preference yet. We
   // still rely on Shill storing it.
-  const base::Value* value = network.proxy_config();
-  if (!value)
+  const base::Value& value = network.proxy_config();
+  if (value.is_none())
     return nullptr;
-  return std::make_unique<ProxyConfigDictionary>(value->Clone());
+  return std::make_unique<ProxyConfigDictionary>(value.Clone());
 }
 
 void SetProxyConfigForNetwork(const ProxyConfigDictionary& proxy_config,
                               const NetworkState& network) {
-  chromeos::ShillServiceClient* shill_service_client =
-      DBusThreadManager::Get()->GetShillServiceClient();
-
   // The user's proxy setting is not stored in the Chrome preference yet. We
   // still rely on Shill storing it.
   ProxyPrefs::ProxyMode mode;
   if (!proxy_config.GetMode(&mode) || mode == ProxyPrefs::MODE_DIRECT) {
     // Return empty string for direct mode for portal check to work correctly.
     // TODO(pneubeck): Consider removing this legacy code.
-    shill_service_client->ClearProperty(
+    ShillServiceClient::Get()->ClearProperty(
         dbus::ObjectPath(network.path()), shill::kProxyConfigProperty,
-        base::Bind(&NotifyNetworkStateHandler, network.path()),
-        base::Bind(&network_handler::ShillErrorCallbackFunction,
-                   "SetProxyConfig.ClearProperty Failed", network.path(),
-                   network_handler::ErrorCallback()));
+        base::BindOnce(&NotifyNetworkStateHandler, network.path()),
+        base::BindOnce(&network_handler::ShillErrorCallbackFunction,
+                       "SetProxyConfig.ClearProperty Failed", network.path(),
+                       network_handler::ErrorCallback()));
   } else {
     std::string proxy_config_str;
     base::JSONWriter::Write(proxy_config.GetDictionary(), &proxy_config_str);
-    shill_service_client->SetProperty(
+    ShillServiceClient::Get()->SetProperty(
         dbus::ObjectPath(network.path()), shill::kProxyConfigProperty,
         base::Value(proxy_config_str),
-        base::Bind(&NotifyNetworkStateHandler, network.path()),
-        base::Bind(&network_handler::ShillErrorCallbackFunction,
-                   "SetProxyConfig.SetProperty Failed", network.path(),
-                   network_handler::ErrorCallback()));
+        base::BindOnce(&NotifyNetworkStateHandler, network.path()),
+        base::BindOnce(&network_handler::ShillErrorCallbackFunction,
+                       "SetProxyConfig.SetProperty Failed", network.path(),
+                       network_handler::ErrorCallback()));
   }
 }
 
