@@ -13,9 +13,11 @@
 
 #include "base/callback.h"
 #include "base/files/file_path.h"
+#include "base/strings/string_piece.h"
 #include "base/version.h"
 #include "chromeos/chromeos_export.h"
 #include "chromeos/printing/printer_configuration.h"
+#include "chromeos/printing/usb_printer_id.h"
 
 namespace network {
 namespace mojom {
@@ -27,6 +29,42 @@ namespace chromeos {
 
 class PpdCache;
 
+// Everything we might know about a printer when looking for a
+// driver for it.  All of the default values for fields in this struct
+// mean we *don't* have that piece of information.
+//
+// Fields are listed in search order preference -- we use earlier
+// fields first to attempt to find a match.
+struct CHROMEOS_EXPORT PrinterSearchData {
+  PrinterSearchData();
+  PrinterSearchData(const PrinterSearchData& other);
+  ~PrinterSearchData();
+
+  // Make-and-model string guesses.
+  std::vector<std::string> make_and_model;
+
+  // 16-bit usb identifiers.
+  int usb_vendor_id = 0;
+  int usb_product_id = 0;
+
+  // Method of printer discovery.
+  enum PrinterDiscoveryType {
+    kUnknown = 0,
+    kManual = 1,
+    kUsb = 2,
+    kZeroconf = 3,
+    kDiscoveryTypeMax
+  };
+  PrinterDiscoveryType discovery_type;
+
+  // Set of MIME types supported by this printer.
+  std::vector<std::string> supported_document_formats;
+
+  // Representation of IEEE1284 standard printing device ID.
+  // Contains a set of languages this printer understands.
+  UsbPrinterId printer_id;
+};
+
 // PpdProvider is responsible for mapping printer descriptions to
 // CUPS-PostScript Printer Description (PPD) files.  It provides PPDs that a
 // user previously identified for use, and falls back to querying quirksserver
@@ -68,27 +106,10 @@ class CHROMEOS_EXPORT PpdProvider : public base::RefCounted<PpdProvider> {
     std::string ppd_server_root = "https://www.gstatic.com/chromeos_printing";
   };
 
-  // Everything we might know about a printer when looking for a
-  // driver for it.  All of the default values for fields in this struct
-  // mean we *don't* have that piece of information.
-  //
-  // Fields are listed in search order preference -- we use earlier
-  // fields first to attempt to find a match.
-  struct PrinterSearchData {
-    PrinterSearchData();
-    PrinterSearchData(const PrinterSearchData& other);
-    ~PrinterSearchData();
-
-    // Make-and-model string guesses.
-    std::vector<std::string> make_and_model;
-
-    // 16-bit usb identifiers.
-    int usb_vendor_id = 0;
-    int usb_product_id = 0;
-  };
-
   // Defines the limitations on when we show a particular PPD
-  struct Restrictions {
+  // Not to be confused with the new Restrictions struct used in the
+  // v3 PpdProvider, defined in ppd_metadata_parser.h
+  struct LegacyRestrictions {
     // Minimum milestone for ChromeOS build
     base::Version min_milestone = base::Version("0.0");
 
@@ -107,12 +128,9 @@ class CHROMEOS_EXPORT PpdProvider : public base::RefCounted<PpdProvider> {
   // Result of a ResolvePpd() call.
   // If the result code is SUCCESS, then:
   //    string holds the contents of a PPD (that may or may not be gzipped).
-  //    required_filters holds the names of the filters referenced in the ppd.
   // Otherwise, these fields will be empty.
-  using ResolvePpdCallback = base::OnceCallback<void(
-      CallbackResultCode,
-      const std::string&,
-      const std::vector<std::string>& required_filters)>;
+  using ResolvePpdCallback =
+      base::OnceCallback<void(CallbackResultCode, const std::string&)>;
 
   // Result of a ResolveManufacturers() call.  If the result code is SUCCESS,
   // then the vector contains a sorted list of manufacturers for which we have
@@ -132,14 +150,24 @@ class CHROMEOS_EXPORT PpdProvider : public base::RefCounted<PpdProvider> {
   using ResolvePrintersCallback =
       base::OnceCallback<void(CallbackResultCode, const ResolvedPrintersList&)>;
 
-  // Result of a ResolvePpdReference call.  If the result code is
-  // SUCCESS, then the second argument contains the a PpdReference
-  // that we have high confidence can be used to obtain a driver for
-  // the printer.  NOT_FOUND means we couldn't confidently figure out
-  // a driver for the printer.
+  // Result of a ResolvePpdReference call.  If the result code is SUCCESS, then
+  // the second argument contains the a PpdReference that we have high
+  // confidence can be used to obtain a driver for the printer.  NOT_FOUND means
+  // we couldn't confidently figure out a driver for the printer.  If we got
+  // NOT_FOUND from a USB printer, we may have been able to determine the
+  // manufacturer name which is the third argument.
   using ResolvePpdReferenceCallback =
       base::OnceCallback<void(CallbackResultCode,
-                              const Printer::PpdReference&)>;
+                              const Printer::PpdReference& ref,
+                              const std::string& manufacturer)>;
+
+  // Result of a ResolvePpdLicense call. If |result| is SUCCESS, then
+  // |license_name| will be used to indicate the license associated with the
+  // requested PPD. If |license_name| is empty, then the requested PPD does not
+  // require a license.
+  using ResolvePpdLicenseCallback =
+      base::OnceCallback<void(CallbackResultCode result,
+                              const std::string& license_name)>;
 
   // Result of a ReverseLookup call.  If the result code is SUCCESS, then
   // |manufactuer| and |model| contain the strings that could have generated
@@ -149,11 +177,16 @@ class CHROMEOS_EXPORT PpdProvider : public base::RefCounted<PpdProvider> {
                               const std::string& manufacturer,
                               const std::string& model)>;
 
+  // Called to get the current URLLoaderFactory on demand. Needs to be
+  // Repeating since it gets called once per fetch.
+  using LoaderFactoryGetter =
+      base::RepeatingCallback<network::mojom::URLLoaderFactory*()>;
+
   // Create and return a new PpdProvider with the given cache and options.
   // A references to |url_context_getter| is taken.
   static scoped_refptr<PpdProvider> Create(
       const std::string& browser_locale,
-      network::mojom::URLLoaderFactory* loader_factory,
+      LoaderFactoryGetter loader_factory_getter,
       scoped_refptr<PpdCache> cache,
       const base::Version& current_version,
       const Options& options = Options());
@@ -162,6 +195,11 @@ class CHROMEOS_EXPORT PpdProvider : public base::RefCounted<PpdProvider> {
   // localized in the default browser locale or the closest available fallback.
   //
   // |cb| will be called on the invoking thread, and will be sequenced.
+  //
+  // PpdProvider will enqueue calls to this method and answer them in
+  // the order received; it will opt to invoke |cb| with failure if the
+  // queue grows overlong, failing the oldest calls first. The exact
+  // queue length at which this occurs is unspecified.
   virtual void ResolveManufacturers(ResolveManufacturersCallback cb) = 0;
 
   // Get all models from a given manufacturer, localized in the
@@ -185,8 +223,23 @@ class CHROMEOS_EXPORT PpdProvider : public base::RefCounted<PpdProvider> {
   virtual void ResolvePpd(const Printer::PpdReference& reference,
                           ResolvePpdCallback cb) = 0;
 
+  // Retrieves the name of the PPD license associated with the given printer
+  // |effective_make_and_model|. If the name of the retrieved license is empty,
+  // then the PPD does not require a license. If |effective_make_and_model| is
+  // already present in the cache, then |cb| will fire immediately. Otherwise,
+  // the PpdIndex will be fetched in order to retrieve the associated license.
+  //
+  // |cb| will be called on the invoking thread, and will be sequenced.
+  virtual void ResolvePpdLicense(base::StringPiece effective_make_and_model,
+                                 ResolvePpdLicenseCallback cb) = 0;
+
   // For a given PpdReference, retrieve the make and model strings used to
   // construct that reference.
+  //
+  // PpdProvider will enqueue calls to this method and answer them in
+  // the order received; it will opt to invoke |cb| with failure if the
+  // queue grows overlong, failing the oldest calls first. The exact
+  // queue length at which this occurs is unspecified.
   virtual void ReverseLookup(const std::string& effective_make_and_model,
                              ReverseLookupCallback cb) = 0;
 
