@@ -2,30 +2,29 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "chromeos/printing/ppd_provider.h"
+
 #include <algorithm>
 #include <map>
 #include <memory>
 #include <utility>
 
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/logging.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/stringprintf.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/bind.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_message_loop.h"
 #include "base/threading/sequenced_task_runner_handle.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/version.h"
-#include "chromeos/chromeos_paths.h"
 #include "chromeos/printing/ppd_cache.h"
-#include "chromeos/printing/ppd_provider.h"
 #include "chromeos/printing/printer_configuration.h"
 #include "net/base/net_errors.h"
-#include "net/url_request/url_request_test_util.h"
 #include "services/network/public/cpp/resource_request.h"
 #include "services/network/test/test_url_loader_factory.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -34,6 +33,8 @@ namespace chromeos {
 
 namespace {
 
+using PrinterDiscoveryType = PrinterSearchData::PrinterDiscoveryType;
+
 // Name of the fake server we're resolving ppds from.
 const char kPpdServer[] = "bogus.google.com";
 
@@ -64,8 +65,7 @@ More random contents that we don't care about.
 class PpdProviderTest : public ::testing::Test {
  public:
   PpdProviderTest()
-      : scoped_task_environment_(
-            base::test::ScopedTaskEnvironment::MainThreadType::IO) {
+      : task_environment_(base::test::TaskEnvironment::MainThreadType::IO) {
     TurnOffFakePpdServer();
   }
 
@@ -77,7 +77,7 @@ class PpdProviderTest : public ::testing::Test {
 
   // Create and return a provider for a test that uses the given |locale|.  If
   // run_cache_on_test_thread is true, we'll run the cache using the
-  // scoped_task_environment_; otherwise we'll let it spawn it's own background
+  // task_environment_; otherwise we'll let it spawn it's own background
   // threads.  You should only run the cache on the test thread if you need to
   // explicity drain cache actions independently of draining ppd provider
   // actions; otherwise letting the cache spawn its own thread should be safe,
@@ -90,12 +90,19 @@ class PpdProviderTest : public ::testing::Test {
     if (run_cache_on_test_thread) {
       ppd_cache_ = PpdCache::CreateForTesting(
           ppd_cache_temp_dir_.GetPath(),
-          scoped_task_environment_.GetMainThreadTaskRunner());
+          task_environment_.GetMainThreadTaskRunner());
     } else {
       ppd_cache_ = PpdCache::Create(ppd_cache_temp_dir_.GetPath());
     }
-    return PpdProvider::Create(locale, &loader_factory_, ppd_cache_,
-                               base::Version("40.8.6753.09"), provider_options);
+    return PpdProvider::Create(
+        locale, base::BindLambdaForTesting([this]() {
+          // Casting here is necessary b/c RepeatingCallback needs help
+          // coercing types.
+          // Casting is safe since its a guaranteed upcast.
+          return dynamic_cast<network::mojom::URLLoaderFactory*>(
+              &loader_factory_);
+        }),
+        ppd_cache_, base::Version("40.8.6753.09"), provider_options);
   }
 
   // Fill loader_factory_ with preset contents for test URLs
@@ -110,8 +117,8 @@ class PpdProviderTest : public ::testing::Test {
       }
 
       loader_factory_.AddResponse(FileToURL(entry.first),
-                                  network::ResourceResponseHead(), entry.second,
-                                  status);
+                                  network::mojom::URLResponseHead::New(),
+                                  entry.second, status);
     }
   }
 
@@ -124,7 +131,7 @@ class PpdProviderTest : public ::testing::Test {
   // Note this is harmless to call if we haven't started a fake ppd server.
   void StopFakePpdServer() {
     TurnOffFakePpdServer();
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
   }
 
   // Capture the result of a ResolveManufacturers() call.
@@ -141,19 +148,24 @@ class PpdProviderTest : public ::testing::Test {
 
   // Capture the result of a ResolvePpd() call.
   void CaptureResolvePpd(PpdProvider::CallbackResultCode code,
-                         const std::string& ppd_contents,
-                         const std::vector<std::string>& ppd_filters) {
+                         const std::string& ppd_contents) {
     CapturedResolvePpdResults results;
     results.code = code;
     results.ppd_contents = ppd_contents;
-    results.ppd_filters = ppd_filters;
     captured_resolve_ppd_.push_back(results);
   }
 
   // Capture the result of a ResolveUsbIds() call.
   void CaptureResolvePpdReference(PpdProvider::CallbackResultCode code,
-                                  const Printer::PpdReference& ref) {
-    captured_resolve_ppd_references_.push_back({code, ref});
+                                  const Printer::PpdReference& ref,
+                                  const std::string& usb_manufacturer) {
+    captured_resolve_ppd_references_.push_back({code, ref, usb_manufacturer});
+  }
+
+  // Capture the result of a ResolvePpdLicense() call.
+  void CaptureResolvePpdLicense(PpdProvider::CallbackResultCode code,
+                                const std::string& license) {
+    captured_resolve_ppd_license_.push_back({code, license});
   }
 
   void CaptureReverseLookup(PpdProvider::CallbackResultCode code,
@@ -175,7 +187,7 @@ class PpdProviderTest : public ::testing::Test {
     auto provider = CreateProvider(browser_locale, false);
     provider->ResolveManufacturers(base::BindOnce(
         &PpdProviderTest::CaptureResolveManufacturers, base::Unretained(this)));
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
     provider = nullptr;
     ASSERT_EQ(captured_resolve_manufacturers_.size(), 1UL);
     EXPECT_EQ(captured_resolve_manufacturers_[0].first, PpdProvider::SUCCESS);
@@ -184,9 +196,8 @@ class PpdProviderTest : public ::testing::Test {
 
     // It's sufficient to check for one of the expected locale keys to make sure
     // we got the right map.
-    EXPECT_FALSE(std::find(result_vec.begin(), result_vec.end(),
-                           "manufacturer_a_" + expected_used_locale) ==
-                 result_vec.end());
+    EXPECT_TRUE(
+        base::Contains(result_vec, "manufacturer_a_" + expected_used_locale));
   }
 
   // Fake server being down, return ERR_ADDRESS_UNREACHABLE for all endpoints
@@ -195,7 +206,8 @@ class PpdProviderTest : public ::testing::Test {
     network::URLLoaderCompletionStatus status(net::ERR_ADDRESS_UNREACHABLE);
     for (const auto& entry : server_contents()) {
       loader_factory_.AddResponse(FileToURL(entry.first),
-                                  network::ResourceResponseHead(), "", status);
+                                  network::mojom::URLResponseHead::New(), "",
+                                  status);
     }
   }
 
@@ -214,7 +226,7 @@ class PpdProviderTest : public ::testing::Test {
              "en-gb"])"},
             {"metadata_v2/index-01.json",
              R"([
-             ["printer_a_ref", "printer_a.ppd"]
+             ["printer_a_ref", "printer_a.ppd", {"license": "fake_license"}]
             ])"},
             {"metadata_v2/index-02.json",
              R"([
@@ -240,6 +252,7 @@ class PpdProviderTest : public ::testing::Test {
              [1592, "Some canonical reference"],
              [6535, "Some other canonical reference"]
             ])"},
+            {"metadata_v2/usb-03f0.json", ""},
             {"metadata_v2/usb-1234.json", ""},
             {"metadata_v2/manufacturers-en.json",
              R"([
@@ -299,7 +312,7 @@ class PpdProviderTest : public ::testing::Test {
   }
 
   // Environment for task schedulers.
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
 
   std::vector<
       std::pair<PpdProvider::CallbackResultCode, std::vector<std::string>>>
@@ -312,11 +325,16 @@ class PpdProviderTest : public ::testing::Test {
   struct CapturedResolvePpdResults {
     PpdProvider::CallbackResultCode code;
     std::string ppd_contents;
-    std::vector<std::string> ppd_filters;
   };
   std::vector<CapturedResolvePpdResults> captured_resolve_ppd_;
 
-  std::vector<std::pair<PpdProvider::CallbackResultCode, Printer::PpdReference>>
+  struct CapturedResolvePpdReferenceResults {
+    PpdProvider::CallbackResultCode code;
+    Printer::PpdReference ref;
+    std::string usb_manufacturer;
+  };
+
+  std::vector<CapturedResolvePpdReferenceResults>
       captured_resolve_ppd_references_;
 
   struct CapturedReverseLookup {
@@ -326,6 +344,12 @@ class PpdProviderTest : public ::testing::Test {
   };
   std::vector<CapturedReverseLookup> captured_reverse_lookup_;
 
+  struct CapturedResolvePpdLicense {
+    PpdProvider::CallbackResultCode code;
+    std::string license;
+  };
+  std::vector<CapturedResolvePpdLicense> captured_resolve_ppd_license_;
+
   base::ScopedTempDir ppd_cache_temp_dir_;
   base::ScopedTempDir interceptor_temp_dir_;
 
@@ -347,7 +371,7 @@ TEST_F(PpdProviderTest, ManufacturersFetch) {
       &PpdProviderTest::CaptureResolveManufacturers, base::Unretained(this)));
   provider->ResolveManufacturers(base::BindOnce(
       &PpdProviderTest::CaptureResolveManufacturers, base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(2UL, captured_resolve_manufacturers_.size());
   std::vector<std::string> expected_result(
       {"manufacturer_a_en", "manufacturer_b_en"});
@@ -367,7 +391,7 @@ TEST_F(PpdProviderTest, ManufacturersFetchNoServer) {
       &PpdProviderTest::CaptureResolveManufacturers, base::Unretained(this)));
   provider->ResolveManufacturers(base::BindOnce(
       &PpdProviderTest::CaptureResolveManufacturers, base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(2UL, captured_resolve_manufacturers_.size());
   EXPECT_EQ(PpdProvider::SERVER_ERROR,
             captured_resolve_manufacturers_[0].first);
@@ -393,13 +417,16 @@ TEST_F(PpdProviderTest, RepeatedMakeModel) {
   StartFakePpdServer();
   auto provider = CreateProvider("en", false);
 
-  PpdProvider::PrinterSearchData unrecognized_printer;
+  PrinterSearchData unrecognized_printer;
+  unrecognized_printer.discovery_type = PrinterDiscoveryType::kManual;
   unrecognized_printer.make_and_model = {"Printer Printer"};
 
-  PpdProvider::PrinterSearchData recognized_printer;
+  PrinterSearchData recognized_printer;
+  recognized_printer.discovery_type = PrinterDiscoveryType::kManual;
   recognized_printer.make_and_model = {"printer_a_ref"};
 
-  PpdProvider::PrinterSearchData mixed;
+  PrinterSearchData mixed;
+  mixed.discovery_type = PrinterDiscoveryType::kManual;
   mixed.make_and_model = {"printer_a_ref", "Printer Printer"};
 
   // Resolve the same thing repeatedly.
@@ -414,18 +441,16 @@ TEST_F(PpdProviderTest, RepeatedMakeModel) {
       recognized_printer,
       base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
                      base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(static_cast<size_t>(3), captured_resolve_ppd_references_.size());
-  EXPECT_EQ(PpdProvider::NOT_FOUND, captured_resolve_ppd_references_[0].first);
-  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_references_[1].first);
-  EXPECT_EQ(
-      "printer_a_ref",
-      captured_resolve_ppd_references_[1].second.effective_make_and_model);
-  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_references_[2].first);
-  EXPECT_EQ(
-      "printer_a_ref",
-      captured_resolve_ppd_references_[2].second.effective_make_and_model);
+  EXPECT_EQ(PpdProvider::NOT_FOUND, captured_resolve_ppd_references_[0].code);
+  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_references_[1].code);
+  EXPECT_EQ("printer_a_ref",
+            captured_resolve_ppd_references_[1].ref.effective_make_and_model);
+  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_references_[2].code);
+  EXPECT_EQ("printer_a_ref",
+            captured_resolve_ppd_references_[2].ref.effective_make_and_model);
 }
 
 // Test successful and unsuccessful usb resolutions.
@@ -433,7 +458,8 @@ TEST_F(PpdProviderTest, UsbResolution) {
   StartFakePpdServer();
   auto provider = CreateProvider("en", false);
 
-  PpdProvider::PrinterSearchData search_data;
+  PrinterSearchData search_data;
+  search_data.discovery_type = PrinterDiscoveryType::kUsb;
 
   // Should get back "Some canonical reference"
   search_data.usb_vendor_id = 0x031f;
@@ -448,15 +474,15 @@ TEST_F(PpdProviderTest, UsbResolution) {
       search_data, base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
                                   base::Unretained(this)));
 
-  // Extant vendor id, nonexistant device id, should get a NOT_FOUND
+  // Vendor id that exists, nonexistent device id, should get a NOT_FOUND.
   search_data.usb_vendor_id = 0x031f;
   search_data.usb_product_id = 8162;
   provider->ResolvePpdReference(
       search_data, base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
                                   base::Unretained(this)));
 
-  // Nonexistant vendor id, should get a NOT_FOUND in the real world, but
-  // the URL interceptor we're using considers all nonexistant files to
+  // Nonexistent vendor id, should get a NOT_FOUND in the real world, but
+  // the URL interceptor we're using considers all nonexistent files to
   // be effectively CONNECTION REFUSED, so we just check for non-success
   // on this one.
   search_data.usb_vendor_id = 0x1234;
@@ -464,17 +490,17 @@ TEST_F(PpdProviderTest, UsbResolution) {
   provider->ResolvePpdReference(
       search_data, base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
                                   base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(captured_resolve_ppd_references_.size(), static_cast<size_t>(4));
-  EXPECT_EQ(captured_resolve_ppd_references_[0].first, PpdProvider::SUCCESS);
-  EXPECT_EQ(captured_resolve_ppd_references_[0].second.effective_make_and_model,
+  EXPECT_EQ(captured_resolve_ppd_references_[0].code, PpdProvider::SUCCESS);
+  EXPECT_EQ(captured_resolve_ppd_references_[0].ref.effective_make_and_model,
             "Some canonical reference");
-  EXPECT_EQ(captured_resolve_ppd_references_[1].first, PpdProvider::SUCCESS);
-  EXPECT_EQ(captured_resolve_ppd_references_[1].second.effective_make_and_model,
+  EXPECT_EQ(captured_resolve_ppd_references_[1].code, PpdProvider::SUCCESS);
+  EXPECT_EQ(captured_resolve_ppd_references_[1].ref.effective_make_and_model,
             "Some other canonical reference");
-  EXPECT_EQ(captured_resolve_ppd_references_[2].first, PpdProvider::NOT_FOUND);
-  EXPECT_FALSE(captured_resolve_ppd_references_[3].first ==
+  EXPECT_EQ(captured_resolve_ppd_references_[2].code, PpdProvider::NOT_FOUND);
+  EXPECT_FALSE(captured_resolve_ppd_references_[3].code ==
                PpdProvider::SUCCESS);
 }
 
@@ -492,7 +518,7 @@ TEST_F(PpdProviderTest, ResolvePrinters) {
   // should be in it and we check that elsewhere.  We just need to run the
   // resolve to populate the internal PpdProvider structures.
   provider->ResolveManufacturers(base::BindOnce(&ResolveManufacturersNop));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   provider->ResolvePrinters(
       "manufacturer_a_en",
@@ -503,7 +529,7 @@ TEST_F(PpdProviderTest, ResolvePrinters) {
       base::BindOnce(&PpdProviderTest::CaptureResolvePrinters,
                      base::Unretained(this)));
 
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(2UL, captured_resolve_printers_.size());
   EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_printers_[0].first);
   EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_printers_[1].first);
@@ -534,13 +560,13 @@ TEST_F(PpdProviderTest, ResolvePrintersBadReference) {
   StartFakePpdServer();
   auto provider = CreateProvider("en", false);
   provider->ResolveManufacturers(base::BindOnce(&ResolveManufacturersNop));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   provider->ResolvePrinters(
       "bogus_doesnt_exist",
       base::BindOnce(&PpdProviderTest::CaptureResolvePrinters,
                      base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(1UL, captured_resolve_printers_.size());
   EXPECT_EQ(PpdProvider::INTERNAL_ERROR, captured_resolve_printers_[0].first);
 }
@@ -550,7 +576,7 @@ TEST_F(PpdProviderTest, ResolvePrintersNoServer) {
   StartFakePpdServer();
   auto provider = CreateProvider("en", false);
   provider->ResolveManufacturers(base::BindOnce(&ResolveManufacturersNop));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   StopFakePpdServer();
 
@@ -562,7 +588,7 @@ TEST_F(PpdProviderTest, ResolvePrintersNoServer) {
       "manufacturer_b_en",
       base::BindOnce(&PpdProviderTest::CaptureResolvePrinters,
                      base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(2UL, captured_resolve_printers_.size());
   EXPECT_EQ(PpdProvider::SERVER_ERROR, captured_resolve_printers_[0].first);
   EXPECT_EQ(PpdProvider::SERVER_ERROR, captured_resolve_printers_[1].first);
@@ -579,7 +605,7 @@ TEST_F(PpdProviderTest, ResolveServerKeyPpd) {
   ref.effective_make_and_model = "printer_c_ref";
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(2UL, captured_resolve_ppd_.size());
   EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[0].code);
@@ -601,7 +627,7 @@ TEST_F(PpdProviderTest, ResolveUserSuppliedUrlPpdFromNetworkFails) {
       "https://%s/user_supplied_ppd_directory/user_supplied.ppd", kPpdServer);
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
   EXPECT_EQ(PpdProvider::INTERNAL_ERROR, captured_resolve_ppd_[0].code);
@@ -619,16 +645,14 @@ TEST_F(PpdProviderTest, ResolveUserSuppliedUrlPpdFromFile) {
 
   std::string user_ppd_contents = "Woohoo";
 
-  ASSERT_EQ(base::WriteFile(filename, user_ppd_contents.data(),
-                            user_ppd_contents.size()),
-            static_cast<int>(user_ppd_contents.size()));
+  ASSERT_TRUE(base::WriteFile(filename, user_ppd_contents));
 
   Printer::PpdReference ref;
   ref.user_supplied_ppd_url =
       base::StringPrintf("file://%s", filename.MaybeAsASCII().c_str());
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
   EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[0].code);
@@ -646,16 +670,14 @@ TEST_F(PpdProviderTest, ResolvedPpdsGetCached) {
     ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
     base::FilePath filename = temp_dir.GetPath().Append("my_spiffy.ppd");
 
-    ASSERT_EQ(base::WriteFile(filename, user_ppd_contents.data(),
-                              user_ppd_contents.size()),
-              static_cast<int>(user_ppd_contents.size()));
+    ASSERT_TRUE(base::WriteFile(filename, user_ppd_contents));
 
     ref.user_supplied_ppd_url =
         base::StringPrintf("file://%s", filename.MaybeAsASCII().c_str());
     provider->ResolvePpd(ref,
                          base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                         base::Unretained(this)));
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
 
     ASSERT_EQ(1UL, captured_resolve_ppd_.size());
     EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[0].code);
@@ -674,46 +696,13 @@ TEST_F(PpdProviderTest, ResolvedPpdsGetCached) {
   // Re-resolve.
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
   EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[0].code);
   EXPECT_EQ(user_ppd_contents, captured_resolve_ppd_[0].ppd_contents);
 }
 
-// Test that the filter extraction code successfully pulls the filters
-// from the ppds resolved.
-TEST_F(PpdProviderTest, ExtractPpdFilters) {
-  StartFakePpdServer();
-  auto provider = CreateProvider("en", false);
-  Printer::PpdReference ref;
-  ref.effective_make_and_model = "printer_a_ref";
-  provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
-                                           base::Unretained(this)));
-  ref.effective_make_and_model = "printer_b_ref";
-  provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
-                                           base::Unretained(this)));
-
-  scoped_task_environment_.RunUntilIdle();
-
-  std::sort(captured_resolve_ppd_[0].ppd_filters.begin(),
-            captured_resolve_ppd_[0].ppd_filters.end());
-  ASSERT_EQ(2UL, captured_resolve_ppd_.size());
-  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[0].code);
-  EXPECT_EQ(kCupsFilterPpdContents, captured_resolve_ppd_[0].ppd_contents);
-  EXPECT_EQ(
-      std::vector<std::string>({"a_different_filter", "filter3", "my_filter"}),
-      captured_resolve_ppd_[0].ppd_filters);
-
-  std::sort(captured_resolve_ppd_[1].ppd_filters.begin(),
-            captured_resolve_ppd_[1].ppd_filters.end());
-  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[1].code);
-  EXPECT_EQ(kCupsFilter2PpdContents, captured_resolve_ppd_[1].ppd_contents);
-  EXPECT_EQ(
-      std::vector<std::string>({"another_real_filter", "the_real_filter"}),
-      captured_resolve_ppd_[1].ppd_filters);
-}
-
 // Test that all entrypoints will correctly work with case-insensitve
 // effective-make-and-model strings.
 TEST_F(PpdProviderTest, CaseInsensitiveMakeAndModel) {
@@ -729,15 +718,12 @@ TEST_F(PpdProviderTest, CaseInsensitiveMakeAndModel) {
   provider->ReverseLookup(ref,
                           base::BindOnce(&PpdProviderTest::CaptureReverseLookup,
                                          base::Unretained(this)));
-  PpdProvider::PrinterSearchData printer_info;
+  PrinterSearchData printer_info;
   printer_info.make_and_model = {ref};
   provider->ResolvePpdReference(
       printer_info, base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
                                    base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
-
-  std::sort(captured_resolve_ppd_[0].ppd_filters.begin(),
-            captured_resolve_ppd_[0].ppd_filters.end());
+  task_environment_.RunUntilIdle();
 
   // Check PpdProvider::ResolvePpd
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
@@ -752,10 +738,39 @@ TEST_F(PpdProviderTest, CaseInsensitiveMakeAndModel) {
 
   // Check PpdProvider::ResolvePpdReference
   ASSERT_EQ(1UL, captured_resolve_ppd_references_.size());
-  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_references_[0].first);
-  EXPECT_EQ(
-      "printer_a_ref",
-      captured_resolve_ppd_references_[0].second.effective_make_and_model);
+  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_references_[0].code);
+  EXPECT_EQ("printer_a_ref",
+            captured_resolve_ppd_references_[0].ref.effective_make_and_model);
+}
+
+// Tests that ResolvePpdLicense is able to correctly source the index and
+// determine the name of the PPD license associated with the given effecive make
+// and model (if any).
+TEST_F(PpdProviderTest, ResolvePpdLicense) {
+  StartFakePpdServer();
+  auto provider = CreateProvider("en", false);
+
+  // For this effective_make_and_model, we expect that there is associated
+  // license.
+  const char kEmm1[] = "printer_a_ref";
+  provider->ResolvePpdLicense(
+      kEmm1, base::BindOnce(&PpdProviderTest::CaptureResolvePpdLicense,
+                            base::Unretained(this)));
+
+  // We do not expect there to be any license associated with this
+  // effective_make_and_model, so the response should be empty.
+  const char kEmm2[] = "printer_b_ref";
+  provider->ResolvePpdLicense(
+      kEmm2, base::BindOnce(&PpdProviderTest::CaptureResolvePpdLicense,
+                            base::Unretained(this)));
+
+  task_environment_.RunUntilIdle();
+
+  ASSERT_EQ(2UL, captured_resolve_ppd_license_.size());
+  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_license_[0].code);
+  EXPECT_EQ("fake_license", captured_resolve_ppd_license_[0].license);
+  EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_license_[1].code);
+  EXPECT_EQ("", captured_resolve_ppd_license_[1].license);
 }
 
 // Verifies that we can extract the Manufacturer and Model selectison for a
@@ -769,13 +784,13 @@ TEST_F(PpdProviderTest, ReverseLookup) {
                                          base::Unretained(this)));
   // TODO(skau): PpdProvider has a race condition that prevents running these
   // requests in parallel.
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   std::string ref_fail = "printer_does_not_exist";
   provider->ReverseLookup(ref_fail,
                           base::BindOnce(&PpdProviderTest::CaptureReverseLookup,
                                          base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   ASSERT_EQ(2U, captured_reverse_lookup_.size());
   CapturedReverseLookup success_capture = captured_reverse_lookup_[0];
@@ -800,10 +815,10 @@ TEST_F(PpdProviderTest, FreshCacheHitNoNetworkTraffic) {
   // Cache exists, and is just created, so should be fresh.
   ppd_cache_->StoreForTesting(PpdProvider::PpdReferenceToCacheKey(ref),
                               cached_ppd_contents, base::TimeDelta());
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
 
   // Should get the cached (not served) results back, and not have hit the
@@ -828,10 +843,10 @@ TEST_F(PpdProviderTest, StaleCacheGetsRefreshed) {
   ppd_cache_->StoreForTesting(PpdProvider::PpdReferenceToCacheKey(ref),
                               cached_ppd_contents,
                               base::TimeDelta::FromDays(180));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
 
   // Should get the served results back, not the stale cached ones.
@@ -849,7 +864,7 @@ TEST_F(PpdProviderTest, StaleCacheGetsRefreshed) {
                          *captured_find_result = find_result;
                        },
                        &captured_find_result));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   EXPECT_EQ(captured_find_result.success, true);
   EXPECT_EQ(captured_find_result.contents, expected_ppd);
   EXPECT_LT(captured_find_result.age, base::TimeDelta::FromDays(1));
@@ -869,10 +884,10 @@ TEST_F(PpdProviderTest, StaleCacheGetsUsedIfNetworkFails) {
   ppd_cache_->StoreForTesting(PpdProvider::PpdReferenceToCacheKey(ref),
                               cached_ppd_contents,
                               base::TimeDelta::FromDays(180));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
 
   // Should successfully resolve from the cache, even though it's stale.
@@ -890,7 +905,7 @@ TEST_F(PpdProviderTest, StaleCacheGetsUsedIfNetworkFails) {
                          *captured_find_result = find_result;
                        },
                        &captured_find_result));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   EXPECT_EQ(captured_find_result.success, true);
   EXPECT_EQ(captured_find_result.contents, cached_ppd_contents);
   EXPECT_GT(captured_find_result.age, base::TimeDelta::FromDays(179));
@@ -914,17 +929,15 @@ TEST_F(PpdProviderTest, UserPpdAlwaysRefreshedIfAvailable) {
   // Put cached_ppd_contents into the cache.
   ppd_cache_->StoreForTesting(PpdProvider::PpdReferenceToCacheKey(ref),
                               cached_ppd_contents, base::TimeDelta());
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Write different contents to disk, so that the cached contents are
   // now stale.
-  ASSERT_EQ(base::WriteFile(filename, disk_ppd_contents.data(),
-                            disk_ppd_contents.size()),
-            static_cast<int>(disk_ppd_contents.size()));
+  ASSERT_TRUE(base::WriteFile(filename, disk_ppd_contents));
 
   provider->ResolvePpd(ref, base::BindOnce(&PpdProviderTest::CaptureResolvePpd,
                                            base::Unretained(this)));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   ASSERT_EQ(1UL, captured_resolve_ppd_.size());
   EXPECT_EQ(PpdProvider::SUCCESS, captured_resolve_ppd_[0].code);
   EXPECT_EQ(disk_ppd_contents, captured_resolve_ppd_[0].ppd_contents);
@@ -938,10 +951,54 @@ TEST_F(PpdProviderTest, UserPpdAlwaysRefreshedIfAvailable) {
                          *captured_find_result = find_result;
                        },
                        &captured_find_result));
-  scoped_task_environment_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
   EXPECT_EQ(captured_find_result.success, true);
   EXPECT_EQ(captured_find_result.contents, disk_ppd_contents);
 }
 
+// Test resolving usb manufacturer when failed to resolve PpdReference.
+TEST_F(PpdProviderTest, ResolveUsbManufacturer) {
+  StartFakePpdServer();
+  auto provider = CreateProvider("en", false);
+
+  PrinterSearchData search_data;
+  search_data.discovery_type = PrinterDiscoveryType::kUsb;
+
+  // Vendor id that exists, nonexistent device id, should get a NOT_FOUND.
+  // Although this is an unsupported printer model, we can still expect to get
+  // the manufacturer name.
+  search_data.usb_vendor_id = 0x03f0;
+  search_data.usb_product_id = 9999;
+  provider->ResolvePpdReference(
+      search_data, base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
+                                  base::Unretained(this)));
+
+  // Nonexistent vendor id, should get a NOT_FOUND in the real world, but
+  // the URL interceptor we're using considers all nonexistent files to
+  // be effectively CONNECTION REFUSED, so we just check for non-success
+  // on this one. We should also not be able to get a manufacturer name from a
+  // nonexistent vendor id.
+  search_data.usb_vendor_id = 0x1234;
+  search_data.usb_product_id = 1782;
+  provider->ResolvePpdReference(
+      search_data, base::BindOnce(&PpdProviderTest::CaptureResolvePpdReference,
+                                  base::Unretained(this)));
+  task_environment_.RunUntilIdle();
+
+  ASSERT_EQ(static_cast<size_t>(2), captured_resolve_ppd_references_.size());
+  // Was able to find the printer manufactuer but unable to find the printer
+  // model should result in a NOT_FOUND.
+  EXPECT_EQ(PpdProvider::NOT_FOUND, captured_resolve_ppd_references_[0].code);
+  // Failed to grab the PPD for a USB printer, but should still be able to grab
+  // its manufacturer name.
+  EXPECT_EQ("HP", captured_resolve_ppd_references_[0].usb_manufacturer);
+
+  // Unable to find the PPD file should result in a failure.
+  EXPECT_FALSE(captured_resolve_ppd_references_[1].code ==
+               PpdProvider::SUCCESS);
+  // Unknown vendor id should result in an empty manufacturer string.
+  EXPECT_TRUE(captured_resolve_ppd_references_[1].usb_manufacturer.empty());
+}
+
 }  // namespace
 }  // namespace chromeos
