@@ -13,7 +13,9 @@
 
 #include "base/bind.h"
 #include "base/command_line.h"
+#include "base/file_descriptor_posix.h"
 #include "base/files/file_util.h"
+#include "base/guid.h"
 #include "base/location.h"
 #include "base/logging.h"
 #include "base/posix/eintr_wrapper.h"
@@ -31,8 +33,6 @@ enum PseudoTerminalFd {
   PT_SLAVE_FD
 };
 
-const int kInvalidFd = -1;
-
 void StopOutputWatcher(
     std::unique_ptr<chromeos::ProcessOutputWatcher> watcher) {
   // Just deleting |watcher| if sufficient to stop it.
@@ -48,24 +48,25 @@ ProcessProxy::ProcessProxy() : process_launched_(false), callback_set_(false) {
   ClearFdPair(pt_pair_);
 }
 
-int ProcessProxy::Open(const base::CommandLine& cmdline,
-                       const std::string& user_id_hash) {
+bool ProcessProxy::Open(const base::CommandLine& cmdline,
+                        const std::string& user_id_hash,
+                        std::string* id) {
   if (process_launched_)
-    return -1;
+    return false;
 
   if (!CreatePseudoTerminalPair(pt_pair_)) {
-    return -1;
+    return false;
   }
 
-  int process_id = LaunchProcess(cmdline, user_id_hash, pt_pair_[PT_SLAVE_FD]);
-  process_launched_ = process_id >= 0;
+  process_launched_ =
+      LaunchProcess(cmdline, user_id_hash, pt_pair_[PT_SLAVE_FD], id);
 
   if (process_launched_) {
     CloseFd(&pt_pair_[PT_SLAVE_FD]);
   } else {
     CloseFdPair(pt_pair_);
   }
-  return process_id;
+  return process_launched_;
 }
 
 bool ProcessProxy::StartWatchingOutput(
@@ -89,8 +90,8 @@ bool ProcessProxy::StartWatchingOutput(
 
   // This object will delete itself once watching is stopped.
   // It also takes ownership of the passed fds.
-  output_watcher_.reset(new ProcessOutputWatcher(
-      master_copy, base::Bind(&ProcessProxy::OnProcessOutput, this)));
+  output_watcher_ = std::make_unique<ProcessOutputWatcher>(
+      master_copy, base::BindRepeating(&ProcessProxy::OnProcessOutput, this));
 
   watcher_runner_->PostTask(
       FROM_HERE, base::BindOnce(&ProcessOutputWatcher::Start,
@@ -101,32 +102,32 @@ bool ProcessProxy::StartWatchingOutput(
 
 void ProcessProxy::OnProcessOutput(ProcessOutputType type,
                                    const std::string& output,
-                                   const base::Closure& callback) {
+                                   base::OnceClosure callback) {
   if (!callback_runner_.get())
     return;
 
   callback_runner_->PostTask(
       FROM_HERE, base::BindOnce(&ProcessProxy::CallOnProcessOutputCallback,
-                                this, type, output, callback));
+                                this, type, output, std::move(callback)));
 }
 
 void ProcessProxy::CallOnProcessOutputCallback(ProcessOutputType type,
                                                const std::string& output,
-                                               const base::Closure& callback) {
+                                               base::OnceClosure callback) {
   // We may receive some output even after Close was called (crosh process does
   // not have to quit instantly, or there may be some trailing data left in
   // output stream fds). In that case owner of the callback may be gone so we
   // don't want to send it anything. |callback_set_| is reset when this gets
   // closed.
   if (callback_set_) {
-    output_ack_callback_ = callback;
+    output_ack_callback_ = std::move(callback);
     callback_.Run(type, output);
   }
 }
 
 void ProcessProxy::AckOutput() {
-  if (!output_ack_callback_.is_null()) {
-    output_ack_callback_.Run();
+  if (output_ack_callback_) {
+    std::move(output_ack_callback_).Run();
     output_ack_callback_.Reset();
   }
 }
@@ -147,7 +148,7 @@ void ProcessProxy::Close() {
   process_launched_ = false;
   callback_set_ = false;
   callback_.Reset();
-  callback_runner_ = NULL;
+  callback_runner_.reset();
 
   process_.Terminate(0, /* wait */ false);
   base::EnsureProcessTerminated(std::move(process_));
@@ -224,9 +225,10 @@ bool ProcessProxy::CreatePseudoTerminalPair(int *pt_pair) {
   return true;
 }
 
-int ProcessProxy::LaunchProcess(const base::CommandLine& cmdline,
-                                const std::string& user_id_hash,
-                                int slave_fd) {
+bool ProcessProxy::LaunchProcess(const base::CommandLine& cmdline,
+                                 const std::string& user_id_hash,
+                                 int slave_fd,
+                                 std::string* id) {
   base::LaunchOptions options;
 
   // Redirect crosh  process' output and input so we can read it.
@@ -241,16 +243,25 @@ int ProcessProxy::LaunchProcess(const base::CommandLine& cmdline,
   // TODO(vapier): Ideally we'd just use the env settings from hterm itself.
   // We can't let the user inject any env var they want, but we should be able
   // to filter the $TERM value dynamically.
-  options.environ["TERM"] = "xterm-256color";
-  options.environ["CROS_USER_ID_HASH"] = user_id_hash;
+  options.environment["TERM"] = "xterm-256color";
+  options.environment["CROS_USER_ID_HASH"] = user_id_hash;
 
   // Launch the process.
   process_ = base::LaunchProcess(cmdline, options);
 
+  // If the process is valid, generate a new random id.
+  // https://crbug.com/352465
+  if (process_.IsValid()) {
+    // We use the GUID API as it's trivial and works well enough.
+    // We prepend the pid to avoid random number collisions.  It should be a
+    // guaranteed unique id for the life of this Chrome session.
+    *id = std::to_string(process_.Pid()) + "-" + base::GenerateGUID();
+  }
+
   // TODO(rvargas) crbug/417532: This is somewhat wrong but the interface of
   // Open vends pid_t* so ownership is quite vague anyway, and Process::Close
   // doesn't do much in POSIX.
-  return process_.IsValid() ? process_.Pid() : -1;
+  return process_.IsValid();
 }
 
 void ProcessProxy::CloseFdPair(int* pipe) {
@@ -259,16 +270,20 @@ void ProcessProxy::CloseFdPair(int* pipe) {
 }
 
 void ProcessProxy::CloseFd(int* fd) {
-  if (*fd != kInvalidFd) {
+  if (*fd != base::kInvalidFd) {
     if (IGNORE_EINTR(close(*fd)) != 0)
       DPLOG(WARNING) << "close fd failed.";
   }
-  *fd = kInvalidFd;
+  *fd = base::kInvalidFd;
 }
 
 void ProcessProxy::ClearFdPair(int* pipe) {
-  pipe[PT_MASTER_FD] = kInvalidFd;
-  pipe[PT_SLAVE_FD] = kInvalidFd;
+  pipe[PT_MASTER_FD] = base::kInvalidFd;
+  pipe[PT_SLAVE_FD] = base::kInvalidFd;
+}
+
+const base::Process* ProcessProxy::GetProcessForTesting() {
+  return &process_;
 }
 
 }  // namespace chromeos
