@@ -4,13 +4,16 @@
 
 #include "chromeos/services/assistant/audio_decoder/assistant_audio_decoder.h"
 
+#include <utility>
+#include <vector>
+
+#include "base/bind.h"
 #include "base/threading/thread.h"
 #include "chromeos/services/assistant/audio_decoder/ipc_data_source.h"
 #include "media/base/audio_bus.h"
 #include "media/base/data_source.h"
 #include "media/filters/audio_file_reader.h"
 #include "media/filters/blocking_url_protocol.h"
-#include "services/service_manager/public/cpp/service_context_ref.h"
 
 namespace chromeos {
 namespace assistant {
@@ -20,24 +23,19 @@ namespace {
 // Preferred bytes per sample when get interleaved data from AudioBus.
 constexpr int kBytesPerSample = 2;
 
-void OnError(bool* succeeded) {
-  *succeeded = false;
-}
-
 }  // namespace
 
 AssistantAudioDecoder::AssistantAudioDecoder(
-    std::unique_ptr<service_manager::ServiceContextRef> service_ref,
-    mojom::AssistantAudioDecoderClientPtr client,
-    mojom::AssistantMediaDataSourcePtr data_source)
-    : service_ref_(std::move(service_ref)),
-      client_(std::move(client)),
-      task_runner_(base::ThreadTaskRunnerHandle::Get()),
+    mojo::PendingRemote<mojom::AssistantAudioDecoderClient> client,
+    mojo::PendingRemote<mojom::AssistantMediaDataSource> data_source)
+    : client_(std::move(client)),
+      task_runner_(base::SequencedTaskRunnerHandle::Get()),
       data_source_(std::make_unique<IPCDataSource>(std::move(data_source))),
       media_thread_(std::make_unique<base::Thread>("media_thread")),
       weak_factory_(this) {
+  weak_this_ = weak_factory_.GetWeakPtr();
   CHECK(media_thread_->Start());
-  client_.set_connection_error_handler(base::BindOnce(
+  client_.set_disconnect_handler(base::BindOnce(
       &AssistantAudioDecoder::OnConnectionError, base::Unretained(this)));
 }
 
@@ -67,13 +65,18 @@ void AssistantAudioDecoder::CloseDecoder(CloseDecoderCallback callback) {
                      base::Unretained(this)));
 }
 
+void AssistantAudioDecoder::OnDataReadError() {
+  read_error_ = true;
+}
+
 void AssistantAudioDecoder::OpenDecoderOnMediaThread() {
-  bool read_ok = true;
   protocol_ = std::make_unique<media::BlockingUrlProtocol>(
-      data_source_.get(), base::BindRepeating(&OnError, &read_ok));
+      data_source_.get(),
+      base::BindRepeating(&AssistantAudioDecoder::OnDataReadError,
+                          base::Unretained(this)));
   decoder_ = std::make_unique<media::AudioFileReader>(protocol_.get());
 
-  if (closed_ || !decoder_->Open() || !read_ok) {
+  if (closed_ || !decoder_->Open() || read_error_) {
     CloseDecoderOnMediaThread();
     return;
   }
@@ -81,7 +84,7 @@ void AssistantAudioDecoder::OpenDecoderOnMediaThread() {
   task_runner_->PostTask(
       FROM_HERE,
       base::BindOnce(&AssistantAudioDecoder::OnDecoderInitializedOnThread,
-                     weak_factory_.GetWeakPtr(), decoder_->sample_rate(),
+                     weak_this_, decoder_->sample_rate(),
                      decoder_->channels()));
 }
 
@@ -98,8 +101,7 @@ void AssistantAudioDecoder::DecodeOnMediaThread() {
 
   task_runner_->PostTask(
       FROM_HERE, base::BindOnce(&AssistantAudioDecoder::OnBufferDecodedOnThread,
-                                weak_factory_.GetWeakPtr(),
-                                std::move(decoded_audio_packets)));
+                                weak_this_, std::move(decoded_audio_packets)));
 }
 
 void AssistantAudioDecoder::CloseDecoderOnMediaThread() {
@@ -109,8 +111,8 @@ void AssistantAudioDecoder::CloseDecoderOnMediaThread() {
 
   closed_ = true;
   task_runner_->PostTask(
-      FROM_HERE, base::BindOnce(&AssistantAudioDecoder::RunCallbacksAsClosed,
-                                weak_factory_.GetWeakPtr()));
+      FROM_HERE,
+      base::BindOnce(&AssistantAudioDecoder::RunCallbacksAsClosed, weak_this_));
 }
 
 void AssistantAudioDecoder::OnDecoderInitializedOnThread(
@@ -132,15 +134,15 @@ void AssistantAudioDecoder::OnBufferDecodedOnThread(
     const int bytes_to_alloc =
         audio_bus->frames() * kBytesPerSample * audio_bus->channels();
     std::vector<uint8_t> buffer(bytes_to_alloc);
-    audio_bus->ToInterleaved(audio_bus->frames(), kBytesPerSample,
-                             buffer.data());
+    audio_bus->ToInterleaved<media::SignedInt16SampleTypeTraits>(
+        audio_bus->frames(), reinterpret_cast<int16_t*>(buffer.data()));
     buffers.emplace_back(buffer);
   }
   client_->OnNewBuffers(buffers);
 }
 
 void AssistantAudioDecoder::OnConnectionError() {
-  client_ = nullptr;
+  client_.reset();
   media_thread_->task_runner()->PostTask(
       FROM_HERE,
       base::BindOnce(&AssistantAudioDecoder::CloseDecoderOnMediaThread,
