@@ -5,11 +5,14 @@
 #ifndef CHROMEOS_SERVICES_DEVICE_SYNC_DEVICE_SYNC_BASE_H_
 #define CHROMEOS_SERVICES_DEVICE_SYNC_DEVICE_SYNC_BASE_H_
 
+#include <memory>
+
 #include "base/macros.h"
 #include "chromeos/services/device_sync/public/mojom/device_sync.mojom.h"
-#include "components/signin/core/browser/account_info.h"
-#include "mojo/public/cpp/bindings/binding_set.h"
-#include "mojo/public/cpp/bindings/interface_ptr_set.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/receiver_set.h"
+#include "mojo/public/cpp/bindings/remote_set.h"
 
 namespace chromeos {
 
@@ -21,22 +24,30 @@ class DeviceSyncBase : public mojom::DeviceSync {
   ~DeviceSyncBase() override;
 
   // mojom::DeviceSync:
-  void AddObserver(mojom::DeviceSyncObserverPtr observer,
+  void AddObserver(mojo::PendingRemote<mojom::DeviceSyncObserver> observer,
                    AddObserverCallback callback) override;
 
-  // Binds a request to this implementation. Should be called each time that the
-  // service receives a request.
-  void BindRequest(mojom::DeviceSyncRequest request);
+  // Binds a receiver to this implementation. Should be called each time that
+  // the service receives a receiver.
+  void BindReceiver(mojo::PendingReceiver<mojom::DeviceSync> receiver);
+
+  void CloseAllReceivers();
 
  protected:
   DeviceSyncBase();
 
+  // Derived types should override this function to remove references to any
+  // dependencies.
+  virtual void Shutdown() {}
+
   void NotifyOnEnrollmentFinished();
   void NotifyOnNewDevicesSynced();
 
  private:
-  mojo::InterfacePtrSet<mojom::DeviceSyncObserver> observers_;
-  mojo::BindingSet<mojom::DeviceSync> bindings_;
+  void OnDisconnection();
+
+  mojo::RemoteSet<mojom::DeviceSyncObserver> observers_;
+  mojo::ReceiverSet<mojom::DeviceSync> receivers_;
 
   DISALLOW_COPY_AND_ASSIGN(DeviceSyncBase);
 };
@@ -45,4 +56,4 @@ class DeviceSyncBase : public mojom::DeviceSync {
 
 }  // namespace chromeos
 
-#endif  // CHROMEOS_SERVICES_DEVICE_SYNC_DEVICE_SYNC_BASE_H_
\ No newline at end of file
+#endif  // CHROMEOS_SERVICES_DEVICE_SYNC_DEVICE_SYNC_BASE_H_
