@@ -7,24 +7,25 @@
 
 #include "base/macros.h"
 #include "chromeos/services/device_sync/public/mojom/device_sync.mojom.h"
-#include "mojo/public/cpp/bindings/binding_set.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/receiver_set.h"
 
 namespace chromeos {
 
 namespace device_sync {
 
 // Fake DeviceSyncObserver implementation for tests.
-class FakeDeviceSyncObserver : public device_sync::mojom::DeviceSyncObserver {
+class FakeDeviceSyncObserver : public mojom::DeviceSyncObserver {
  public:
   FakeDeviceSyncObserver();
   ~FakeDeviceSyncObserver() override;
 
-  mojom::DeviceSyncObserverPtr GenerateInterfacePtr();
+  mojo::PendingRemote<mojom::DeviceSyncObserver> GenerateRemote();
 
   size_t num_enrollment_events() { return num_enrollment_events_; }
   size_t num_sync_events() { return num_sync_events_; }
 
-  // device_sync::mojom::DeviceSyncObserver:
+  // mojom::DeviceSyncObserver:
   void OnEnrollmentFinished() override;
   void OnNewDevicesSynced() override;
 
@@ -32,7 +33,7 @@ class FakeDeviceSyncObserver : public device_sync::mojom::DeviceSyncObserver {
   size_t num_enrollment_events_ = 0u;
   size_t num_sync_events_ = 0u;
 
-  mojo::BindingSet<mojom::DeviceSyncObserver> bindings_;
+  mojo::ReceiverSet<mojom::DeviceSyncObserver> receivers_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeDeviceSyncObserver);
 };
