@@ -7,13 +7,23 @@
 
 #include <memory>
 #include <string>
+#include <vector>
 
 #include "base/callback.h"
 #include "base/macros.h"
+#include "base/memory/scoped_refptr.h"
 #include "base/observer_list.h"
 #include "base/optional.h"
+#include "chromeos/components/multidevice/remote_device_ref.h"
+#include "chromeos/components/multidevice/software_feature.h"
+#include "chromeos/services/device_sync/feature_status_change.h"
+#include "chromeos/services/device_sync/proto/cryptauth_common.pb.h"
 #include "chromeos/services/device_sync/public/mojom/device_sync.mojom.h"
-#include "components/cryptauth/remote_device_ref.h"
+#include "mojo/public/cpp/bindings/remote.h"
+
+namespace base {
+class TaskRunner;
+}  // namespace base
 
 namespace chromeos {
 
@@ -40,12 +50,19 @@ class DeviceSyncClient {
 
   using FindEligibleDevicesCallback =
       base::OnceCallback<void(mojom::NetworkRequestResult,
-                              cryptauth::RemoteDeviceRefList,
-                              cryptauth::RemoteDeviceRefList)>;
+                              multidevice::RemoteDeviceRefList,
+                              multidevice::RemoteDeviceRefList)>;
 
   DeviceSyncClient();
   virtual ~DeviceSyncClient();
 
+  // Completes initialization. Must be called after connecting the DeviceSync
+  // mojo remote to the implementation.
+  virtual void Initialize(scoped_refptr<base::TaskRunner> task_runner) {}
+
+  // Returns the DeviceSync mojo remote.
+  virtual mojo::Remote<mojom::DeviceSync>* GetDeviceSyncRemote();
+
   void AddObserver(Observer* observer);
   void RemoveObserver(Observer* observer);
 
@@ -58,8 +75,8 @@ class DeviceSyncClient {
       mojom::DeviceSync::ForceEnrollmentNowCallback callback) = 0;
   virtual void ForceSyncNow(
       mojom::DeviceSync::ForceSyncNowCallback callback) = 0;
-  virtual cryptauth::RemoteDeviceRefList GetSyncedDevices() = 0;
-  virtual base::Optional<cryptauth::RemoteDeviceRef>
+  virtual multidevice::RemoteDeviceRefList GetSyncedDevices() = 0;
+  virtual base::Optional<multidevice::RemoteDeviceRef>
   GetLocalDeviceMetadata() = 0;
 
   // Note: In the special case of passing |software_feature| =
@@ -67,12 +84,25 @@ class DeviceSyncClient {
   // ignored.
   virtual void SetSoftwareFeatureState(
       const std::string public_key,
-      cryptauth::SoftwareFeature software_feature,
+      multidevice::SoftwareFeature software_feature,
       bool enabled,
       bool is_exclusive,
       mojom::DeviceSync::SetSoftwareFeatureStateCallback callback) = 0;
-  virtual void FindEligibleDevices(cryptauth::SoftwareFeature software_feature,
-                                   FindEligibleDevicesCallback callback) = 0;
+  virtual void SetFeatureStatus(
+      const std::string& device_instance_id,
+      multidevice::SoftwareFeature feature,
+      FeatureStatusChange status_change,
+      mojom::DeviceSync::SetFeatureStatusCallback callback) = 0;
+  virtual void FindEligibleDevices(
+      multidevice::SoftwareFeature software_feature,
+      FindEligibleDevicesCallback callback) = 0;
+  virtual void NotifyDevices(
+      const std::vector<std::string>& device_instance_ids,
+      cryptauthv2::TargetService target_service,
+      multidevice::SoftwareFeature feature,
+      mojom::DeviceSync::NotifyDevicesCallback callback) = 0;
+  virtual void GetDevicesActivityStatus(
+      mojom::DeviceSync::GetDevicesActivityStatusCallback callback) = 0;
   virtual void GetDebugInfo(
       mojom::DeviceSync::GetDebugInfoCallback callback) = 0;
 
