@@ -5,28 +5,34 @@
 #include "chromeos/services/device_sync/public/cpp/device_sync_client_impl.h"
 
 #include <algorithm>
+#include <string>
 #include <tuple>
 #include <utility>
+#include <vector>
 
+#include "base/bind.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/no_destructor.h"
 #include "base/optional.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/test/null_task_runner.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_simple_task_runner.h"
+#include "chromeos/components/multidevice/remote_device_test_util.h"
+#include "chromeos/components/multidevice/software_feature.h"
 #include "chromeos/services/device_sync/device_sync_impl.h"
-#include "chromeos/services/device_sync/device_sync_service.h"
 #include "chromeos/services/device_sync/fake_device_sync.h"
-#include "chromeos/services/device_sync/public/mojom/constants.mojom.h"
+#include "chromeos/services/device_sync/feature_status_change.h"
+#include "chromeos/services/device_sync/proto/cryptauth_common.pb.h"
+#include "chromeos/services/device_sync/public/cpp/device_sync_prefs.h"
+#include "chromeos/services/device_sync/public/cpp/fake_client_app_metadata_provider.h"
+#include "chromeos/services/device_sync/public/cpp/fake_gcm_device_info_provider.h"
 #include "chromeos/services/device_sync/public/mojom/device_sync.mojom.h"
-#include "components/cryptauth/fake_gcm_device_info_provider.h"
-#include "components/cryptauth/remote_device_test_util.h"
 #include "components/gcm_driver/fake_gcm_driver.h"
-#include "services/identity/public/cpp/identity_test_environment.h"
+#include "components/prefs/testing_pref_service.h"
+#include "components/signin/public/identity_manager/identity_test_environment.h"
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
-#include "services/service_manager/public/cpp/test/test_connector_factory.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace chromeos {
@@ -58,11 +64,12 @@ class FakeDeviceSyncImplFactory : public DeviceSyncImpl::Factory {
   ~FakeDeviceSyncImplFactory() override = default;
 
   // DeviceSyncImpl::Factory:
-  std::unique_ptr<DeviceSyncBase> BuildInstance(
-      identity::IdentityManager* identity_manager,
+  std::unique_ptr<DeviceSyncBase> CreateInstance(
+      signin::IdentityManager* identity_manager,
       gcm::GCMDriver* gcm_driver,
-      service_manager::Connector* connector,
-      const cryptauth::GcmDeviceInfoProvider* gcm_device_info_provider,
+      PrefService* profile_prefs,
+      const GcmDeviceInfoProvider* gcm_device_info_provider,
+      ClientAppMetadataProvider* client_app_metadata_provider,
       scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
       std::unique_ptr<base::OneShotTimer> timer) override {
     EXPECT_TRUE(fake_device_sync_);
@@ -136,27 +143,30 @@ class DeviceSyncClientImplTest : public testing::Test {
  protected:
   DeviceSyncClientImplTest()
       : test_remote_device_list_(
-            cryptauth::CreateRemoteDeviceListForTest(kNumTestDevices)),
+            multidevice::CreateRemoteDeviceListForTest(kNumTestDevices)),
         test_remote_device_ref_list_(
-            cryptauth::CreateRemoteDeviceRefListForTest(kNumTestDevices)) {}
+            multidevice::CreateRemoteDeviceRefListForTest(kNumTestDevices)) {}
 
   // testing::Test:
   void SetUp() override {
     fake_gcm_driver_ = std::make_unique<gcm::FakeGCMDriver>();
     fake_gcm_device_info_provider_ =
-        std::make_unique<cryptauth::FakeGcmDeviceInfoProvider>(
-            GetTestGcmDeviceInfo());
+        std::make_unique<FakeGcmDeviceInfoProvider>(GetTestGcmDeviceInfo());
+    fake_client_app_metadata_provider_ =
+        std::make_unique<FakeClientAppMetadataProvider>();
 
     identity_test_environment_ =
-        std::make_unique<identity::IdentityTestEnvironment>();
-    identity_test_environment_->MakePrimaryAccountAvailable(kTestEmail);
+        std::make_unique<signin::IdentityTestEnvironment>();
+    // "Unconsented" because this feature is not tied to browser sync consent.
+    identity_test_environment_->MakeUnconsentedPrimaryAccountAvailable(
+        kTestEmail);
 
     auto fake_device_sync = std::make_unique<FakeDeviceSync>();
     fake_device_sync_ = fake_device_sync.get();
     fake_device_sync_impl_factory_ =
         std::make_unique<FakeDeviceSyncImplFactory>(
             std::move(fake_device_sync));
-    DeviceSyncImpl::Factory::SetInstanceForTesting(
+    DeviceSyncImpl::Factory::SetCustomFactory(
         fake_device_sync_impl_factory_.get());
 
     auto shared_url_loader_factory =
@@ -166,32 +176,28 @@ class DeviceSyncClientImplTest : public testing::Test {
               return nullptr;
             }));
 
-    auto device_sync_service = std::make_unique<DeviceSyncService>(
-        identity_test_environment_->identity_manager(), fake_gcm_driver_.get(),
-        fake_gcm_device_info_provider_.get(), shared_url_loader_factory);
+    test_pref_service_ = std::make_unique<TestingPrefServiceSimple>();
+    RegisterProfilePrefs(test_pref_service_->registry());
 
-    connector_factory_ =
-        service_manager::TestConnectorFactory::CreateForUniqueService(
-            std::move(device_sync_service));
+    device_sync_ = DeviceSyncImpl::Factory::Create(
+        identity_test_environment_->identity_manager(), fake_gcm_driver_.get(),
+        test_pref_service_.get(), fake_gcm_device_info_provider_.get(),
+        fake_client_app_metadata_provider_.get(), shared_url_loader_factory,
+        std::make_unique<base::OneShotTimer>());
 
     test_observer_ = std::make_unique<TestDeviceSyncClientObserver>();
 
-    CreateClient();
-  }
-
-  void CreateClient() {
-    std::unique_ptr<service_manager::Connector> connector =
-        connector_factory_->CreateConnector();
-
-    // DeviceSyncClient's constructor posts two tasks to the TaskRunner. Idle
+    // DeviceSyncClient initialization posts two tasks to the TaskRunner. Idle
     // the TaskRunner so that the tasks can be run via a RunLoop later on.
     auto test_task_runner = base::MakeRefCounted<base::TestSimpleTaskRunner>();
-    client_ = base::WrapUnique(
-        new DeviceSyncClientImpl(connector.get(), test_task_runner));
+    client_ = std::make_unique<DeviceSyncClientImpl>();
+    device_sync_->BindReceiver(
+        client_->GetDeviceSyncRemote()->BindNewPipeAndPassReceiver());
+    client_->Initialize(test_task_runner);
     test_task_runner->RunUntilIdle();
   }
 
-  void InitializeClient(bool complete_enrollment_before_sync = true) {
+  void SetupClient(bool complete_enrollment_before_sync = true) {
     client_->AddObserver(test_observer_.get());
 
     SendPendingMojoMessages();
@@ -297,7 +303,7 @@ class DeviceSyncClientImplTest : public testing::Test {
   }
 
   void TearDown() override {
-    DeviceSyncImpl::Factory::SetInstanceForTesting(nullptr);
+    DeviceSyncImpl::Factory::SetCustomFactory(nullptr);
     client_->RemoveObserver(test_observer_.get());
   }
 
@@ -332,7 +338,7 @@ class DeviceSyncClientImplTest : public testing::Test {
 
     client_->SetSoftwareFeatureState(
         test_remote_device_ref_list_[0].public_key(),
-        cryptauth::SoftwareFeature::BETTER_TOGETHER_HOST, true /* enabled */,
+        multidevice::SoftwareFeature::kBetterTogetherHost, true /* enabled */,
         true /* enabled */,
         base::BindOnce(
             &DeviceSyncClientImplTest::OnSetSoftwareFeatureStateCompleted,
@@ -347,14 +353,39 @@ class DeviceSyncClientImplTest : public testing::Test {
     EXPECT_EQ(expected_result_code, set_software_feature_state_result_code_);
   }
 
+  void CallSetFeatureStatus(
+      mojom::NetworkRequestResult expected_result_code,
+      const base::Optional<std::string> invalid_instance_id = base::nullopt) {
+    base::RunLoop run_loop;
+
+    std::string instance_id = invalid_instance_id.value_or(
+        test_remote_device_ref_list_[0].instance_id());
+
+    client_->SetFeatureStatus(
+        instance_id, multidevice::SoftwareFeature::kBetterTogetherHost,
+        FeatureStatusChange::kEnableExclusively,
+        base::BindOnce(&DeviceSyncClientImplTest::OnSetFeatureStatusCompleted,
+                       base::Unretained(this), run_loop.QuitClosure()));
+
+    if (!invalid_instance_id) {
+      SendPendingMojoMessages();
+      fake_device_sync_->InvokePendingSetFeatureStatusCallback(
+          expected_result_code);
+    }
+
+    run_loop.Run();
+
+    EXPECT_EQ(expected_result_code, set_feature_status_result_code_);
+  }
+
   void CallFindEligibleDevices(
       mojom::NetworkRequestResult expected_result_code,
-      cryptauth::RemoteDeviceList expected_eligible_devices,
-      cryptauth::RemoteDeviceList expected_ineligible_devices) {
+      multidevice::RemoteDeviceList expected_eligible_devices,
+      multidevice::RemoteDeviceList expected_ineligible_devices) {
     base::RunLoop run_loop;
 
     client_->FindEligibleDevices(
-        cryptauth::SoftwareFeature::BETTER_TOGETHER_HOST,
+        multidevice::SoftwareFeature::kBetterTogetherHost,
         base::BindOnce(
             &DeviceSyncClientImplTest::OnFindEligibleDevicesCompleted,
             base::Unretained(this), run_loop.QuitClosure()));
@@ -377,6 +408,70 @@ class DeviceSyncClientImplTest : public testing::Test {
         expected_ineligible_devices);
   }
 
+  void CallNotifyDevices(mojom::NetworkRequestResult expected_result_code,
+                         const base::Optional<std::vector<std::string>>&
+                             invalid_instance_ids = base::nullopt) {
+    base::RunLoop run_loop;
+
+    std::vector<std::string> instance_ids =
+        invalid_instance_ids.value_or(std::vector<std::string>(
+            {test_remote_device_ref_list_[0].instance_id(),
+             test_remote_device_ref_list_[1].instance_id()}));
+
+    client_->NotifyDevices(
+        instance_ids, cryptauthv2::TargetService::DEVICE_SYNC,
+        multidevice::SoftwareFeature::kBetterTogetherHost,
+        base::BindOnce(&DeviceSyncClientImplTest::OnNotifyDevicesCompleted,
+                       base::Unretained(this), run_loop.QuitClosure()));
+
+    if (!invalid_instance_ids) {
+      SendPendingMojoMessages();
+      fake_device_sync_->InvokePendingNotifyDevicesCallback(
+          expected_result_code);
+    }
+
+    run_loop.Run();
+
+    EXPECT_EQ(expected_result_code, notify_devices_result_code_);
+  }
+
+  void CallGetDevicesActivityStatus(
+      mojom::NetworkRequestResult expected_result_code,
+      base::Optional<std::vector<mojom::DeviceActivityStatusPtr>>
+          expected_activity_statuses) {
+    base::RunLoop run_loop;
+
+    client_->GetDevicesActivityStatus(
+        base::BindOnce(&DeviceSyncClientImplTest::OnGetDevicesActivityStatus,
+                       base::Unretained(this), run_loop.QuitClosure()));
+
+    SendPendingMojoMessages();
+
+    base::Optional<std::vector<mojom::DeviceActivityStatusPtr>>
+        device_activity_statuses_optional;
+    if (expected_activity_statuses != base::nullopt) {
+      std::vector<mojom::DeviceActivityStatusPtr> device_activity_statuses;
+      for (const mojom::DeviceActivityStatusPtr& device_activity_status :
+           *expected_activity_statuses) {
+        device_activity_statuses.emplace_back(mojom::DeviceActivityStatus::New(
+            device_activity_status->device_id,
+            device_activity_status->last_activity_time,
+            device_activity_status->connectivity_status,
+            device_activity_status->last_update_time));
+      }
+      device_activity_statuses_optional =
+          base::make_optional(std::move(device_activity_statuses));
+    }
+    fake_device_sync_->InvokePendingGetDevicesActivityStatusCallback(
+        expected_result_code, std::move(device_activity_statuses_optional));
+    run_loop.Run();
+
+    EXPECT_EQ(expected_result_code,
+              std::get<0>(get_devices_activity_status_code_and_response_));
+    EXPECT_EQ(expected_activity_statuses,
+              std::get<1>(get_devices_activity_status_code_and_response_));
+  }
+
   void CallGetDebugInfo() {
     EXPECT_FALSE(debug_info_received_);
 
@@ -399,8 +494,8 @@ class DeviceSyncClientImplTest : public testing::Test {
   // stores devices in an unordered_map -- retrieved devices thus need to be
   // sorted before comparison.
   void VerifyRemoteDeviceRefListAndRemoteDeviceListAreEqual(
-      cryptauth::RemoteDeviceRefList remote_device_ref_list,
-      cryptauth::RemoteDeviceList remote_device_list) {
+      multidevice::RemoteDeviceRefList remote_device_ref_list,
+      multidevice::RemoteDeviceList remote_device_list) {
     std::vector<std::string> ref_public_keys;
     for (auto device : remote_device_ref_list)
       ref_public_keys.push_back(device.public_key());
@@ -422,30 +517,37 @@ class DeviceSyncClientImplTest : public testing::Test {
 
   void SendPendingMojoMessages() { client_->FlushForTesting(); }
 
-  const base::test::ScopedTaskEnvironment scoped_task_environment_;
+  const base::test::TaskEnvironment task_environment_;
 
-  std::unique_ptr<identity::IdentityTestEnvironment> identity_test_environment_;
+  std::unique_ptr<signin::IdentityTestEnvironment> identity_test_environment_;
   std::unique_ptr<gcm::FakeGCMDriver> fake_gcm_driver_;
-  std::unique_ptr<cryptauth::FakeGcmDeviceInfoProvider>
-      fake_gcm_device_info_provider_;
+  std::unique_ptr<FakeGcmDeviceInfoProvider> fake_gcm_device_info_provider_;
+  std::unique_ptr<FakeClientAppMetadataProvider>
+      fake_client_app_metadata_provider_;
   FakeDeviceSync* fake_device_sync_;
   std::unique_ptr<FakeDeviceSyncImplFactory> fake_device_sync_impl_factory_;
-  std::unique_ptr<service_manager::TestConnectorFactory> connector_factory_;
+  std::unique_ptr<TestingPrefServiceSimple> test_pref_service_;
+  std::unique_ptr<DeviceSyncBase> device_sync_;
   std::unique_ptr<TestDeviceSyncClientObserver> test_observer_;
 
   std::unique_ptr<DeviceSyncClientImpl> client_;
 
-  cryptauth::RemoteDeviceList test_remote_device_list_;
-  const cryptauth::RemoteDeviceRefList test_remote_device_ref_list_;
+  multidevice::RemoteDeviceList test_remote_device_list_;
+  const multidevice::RemoteDeviceRefList test_remote_device_ref_list_;
 
   base::Optional<bool> force_enrollment_now_completed_success_;
   base::Optional<bool> force_sync_now_completed_success_;
   base::Optional<mojom::NetworkRequestResult>
       set_software_feature_state_result_code_;
+  base::Optional<mojom::NetworkRequestResult> set_feature_status_result_code_;
   std::tuple<mojom::NetworkRequestResult,
-             cryptauth::RemoteDeviceRefList,
-             cryptauth::RemoteDeviceRefList>
+             multidevice::RemoteDeviceRefList,
+             multidevice::RemoteDeviceRefList>
       find_eligible_devices_error_code_and_response_;
+  base::Optional<mojom::NetworkRequestResult> notify_devices_result_code_;
+  std::tuple<mojom::NetworkRequestResult,
+             base::Optional<std::vector<mojom::DeviceActivityStatusPtr>>>
+      get_devices_activity_status_code_and_response_;
   bool debug_info_received_ = false;
 
  private:
@@ -467,16 +569,38 @@ class DeviceSyncClientImplTest : public testing::Test {
     std::move(callback).Run();
   }
 
+  void OnSetFeatureStatusCompleted(base::OnceClosure callback,
+                                   mojom::NetworkRequestResult result_code) {
+    set_feature_status_result_code_ = result_code;
+    std::move(callback).Run();
+  }
+
   void OnFindEligibleDevicesCompleted(
       base::OnceClosure callback,
       mojom::NetworkRequestResult result_code,
-      cryptauth::RemoteDeviceRefList eligible_devices,
-      cryptauth::RemoteDeviceRefList ineligible_devices) {
+      multidevice::RemoteDeviceRefList eligible_devices,
+      multidevice::RemoteDeviceRefList ineligible_devices) {
     find_eligible_devices_error_code_and_response_ =
         std::make_tuple(result_code, eligible_devices, ineligible_devices);
     std::move(callback).Run();
   }
 
+  void OnNotifyDevicesCompleted(base::OnceClosure callback,
+                                mojom::NetworkRequestResult result_code) {
+    notify_devices_result_code_ = result_code;
+    std::move(callback).Run();
+  }
+
+  void OnGetDevicesActivityStatus(
+      base::OnceClosure callback,
+      mojom::NetworkRequestResult result_code,
+      base::Optional<std::vector<mojom::DeviceActivityStatusPtr>>
+          device_activity_status) {
+    get_devices_activity_status_code_and_response_ =
+        std::make_tuple(result_code, std::move(device_activity_status));
+    std::move(callback).Run();
+  }
+
   void OnGetDebugInfoCompleted(base::OnceClosure callback,
                                mojom::DebugInfoPtr debug_info_ptr) {
     debug_info_received_ = true;
@@ -488,7 +612,7 @@ class DeviceSyncClientImplTest : public testing::Test {
 
 TEST_F(DeviceSyncClientImplTest,
        TestCompleteInitialSyncBeforeInitialEnrollment) {
-  InitializeClient(false /* complete_enrollment_before_sync */);
+  SetupClient(false /* complete_enrollment_before_sync */);
 }
 
 TEST_F(
@@ -496,7 +620,7 @@ TEST_F(
     TestCompleteInitialEnrollmentBeforeInitialSync_WaitForLocalDeviceMetadata) {
   client_->AddObserver(test_observer_.get());
 
-  SendPendingMojoMessages();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_FALSE(client_->is_ready());
   EXPECT_EQ(0u, test_observer_->ready_count());
@@ -515,7 +639,7 @@ TEST_F(
   EXPECT_EQ(0u, test_observer_->enrollment_finished_count());
   EXPECT_EQ(0u, test_observer_->new_devices_synced_count());
 
-  SendPendingMojoMessages();
+  base::RunLoop().RunUntilIdle();
 
   base::RunLoop run_loop;
 
@@ -543,7 +667,7 @@ TEST_F(
 TEST_F(DeviceSyncClientImplTest, TestOnEnrollmentFinished) {
   EXPECT_EQ(0u, test_observer_->enrollment_finished_count());
 
-  InitializeClient();
+  SetupClient();
 
   EXPECT_EQ(test_remote_device_list_[0].public_key,
             client_->GetLocalDeviceMetadata()->public_key());
@@ -556,7 +680,7 @@ TEST_F(DeviceSyncClientImplTest, TestOnEnrollmentFinished) {
   // finish before notifying observers that enrollment has finished.
   EXPECT_EQ(1u, test_observer_->enrollment_finished_count());
 
-  SendPendingMojoMessages();
+  base::RunLoop().RunUntilIdle();
 
   // Update the local device metadata. The last update time must also be later
   // than the previous version, otherwise the update will be ignored by the
@@ -580,7 +704,7 @@ TEST_F(DeviceSyncClientImplTest, TestOnEnrollmentFinished) {
 TEST_F(DeviceSyncClientImplTest, TestOnNewDevicesSynced) {
   EXPECT_EQ(0u, test_observer_->new_devices_synced_count());
 
-  InitializeClient();
+  SetupClient();
 
   VerifyRemoteDeviceRefListAndRemoteDeviceListAreEqual(
       client_->GetSyncedDevices(), test_remote_device_list_);
@@ -591,10 +715,10 @@ TEST_F(DeviceSyncClientImplTest, TestOnNewDevicesSynced) {
   // finish before notifying observers that enrollment has finished.
   EXPECT_EQ(1u, test_observer_->new_devices_synced_count());
 
-  SendPendingMojoMessages();
+  base::RunLoop().RunUntilIdle();
 
   // Change the synced device list.
-  cryptauth::RemoteDeviceList new_device_list(
+  multidevice::RemoteDeviceList new_device_list(
       {test_remote_device_list_[0], test_remote_device_list_[1]});
 
   base::RunLoop run_loop;
@@ -609,37 +733,37 @@ TEST_F(DeviceSyncClientImplTest, TestOnNewDevicesSynced) {
 }
 
 TEST_F(DeviceSyncClientImplTest, TestForceEnrollmentNow_ExpectSuccess) {
-  InitializeClient();
+  SetupClient();
 
   CallForceEnrollmentNow(true /* expected_success */);
 }
 
 TEST_F(DeviceSyncClientImplTest, TestForceEnrollmentNow_ExpectFailure) {
-  InitializeClient();
+  SetupClient();
 
   CallForceEnrollmentNow(false /* expected_success */);
 }
 
 TEST_F(DeviceSyncClientImplTest, TestSyncNow_ExpectSuccess) {
-  InitializeClient();
+  SetupClient();
 
   CallSyncNow(true /* expected_success */);
 }
 
 TEST_F(DeviceSyncClientImplTest, TestSyncNow_ExpectFailure) {
-  InitializeClient();
+  SetupClient();
 
   CallSyncNow(false /* expected_success */);
 }
 
 TEST_F(DeviceSyncClientImplTest, TestGetSyncedDevices_DeviceRemovedFromCache) {
-  InitializeClient();
+  SetupClient();
 
   VerifyRemoteDeviceRefListAndRemoteDeviceListAreEqual(
       client_->GetSyncedDevices(), test_remote_device_list_);
 
   // Remove a device from the test list, and inform |client_|.
-  cryptauth::RemoteDeviceList new_list(
+  multidevice::RemoteDeviceList new_list(
       {test_remote_device_list_[0], test_remote_device_list_[1],
        test_remote_device_list_[2], test_remote_device_list_[3]});
   client_->OnNewDevicesSynced();
@@ -656,17 +780,40 @@ TEST_F(DeviceSyncClientImplTest, TestGetSyncedDevices_DeviceRemovedFromCache) {
 }
 
 TEST_F(DeviceSyncClientImplTest, TestSetSoftwareFeatureState) {
-  InitializeClient();
+  SetupClient();
 
   CallSetSoftwareFeatureState(mojom::NetworkRequestResult::kSuccess);
 }
 
-TEST_F(DeviceSyncClientImplTest, TestFindEligibleDevices_NoErrorCode) {
-  InitializeClient();
+TEST_F(DeviceSyncClientImplTest, TestSetFeatureStatus_Success) {
+  SetupClient();
 
-  cryptauth::RemoteDeviceList expected_eligible_devices(
+  CallSetFeatureStatus(mojom::NetworkRequestResult::kSuccess);
+}
+
+TEST_F(DeviceSyncClientImplTest, TestSetFeatureStatus_InvalidInstanceId) {
+  SetupClient();
+
+  // Instance ID cannot be empty.
+  CallSetFeatureStatus(mojom::NetworkRequestResult::kBadRequest,
+                       std::string() /* invalid_instance_id */);
+
+  // Instance ID must be base64 encoded.
+  CallSetFeatureStatus(mojom::NetworkRequestResult::kBadRequest,
+                       "$%^&*#" /* invalid_instance_id */);
+
+  // Instance ID must be 8 bytes after base64 decoding.
+  CallSetFeatureStatus(
+      mojom::NetworkRequestResult::kBadRequest,
+      "thisislongerthaneightbytes==" /* invalid_instance_id */);
+}
+
+TEST_F(DeviceSyncClientImplTest, TestFindEligibleDevices_NoErrorCode) {
+  SetupClient();
+
+  multidevice::RemoteDeviceList expected_eligible_devices(
       {test_remote_device_list_[0], test_remote_device_list_[1]});
-  cryptauth::RemoteDeviceList expected_ineligible_devices(
+  multidevice::RemoteDeviceList expected_ineligible_devices(
       {test_remote_device_list_[2], test_remote_device_list_[3],
        test_remote_device_list_[4]});
 
@@ -676,15 +823,51 @@ TEST_F(DeviceSyncClientImplTest, TestFindEligibleDevices_NoErrorCode) {
 }
 
 TEST_F(DeviceSyncClientImplTest, TestFindEligibleDevices_ErrorCode) {
-  InitializeClient();
+  SetupClient();
 
   CallFindEligibleDevices(mojom::NetworkRequestResult::kEndpointNotFound,
-                          cryptauth::RemoteDeviceList(),
-                          cryptauth::RemoteDeviceList());
+                          multidevice::RemoteDeviceList(),
+                          multidevice::RemoteDeviceList());
+}
+
+TEST_F(DeviceSyncClientImplTest, TestNotifyDevices_Success) {
+  SetupClient();
+
+  CallNotifyDevices(mojom::NetworkRequestResult::kSuccess);
+}
+
+TEST_F(DeviceSyncClientImplTest, TestNotifyDevices_InvalidInstanceIds) {
+  SetupClient();
+
+  // Instance IDs cannot be empty, must be base64 encoded, and must be 8 bytes
+  // after base64 decoding.
+  CallNotifyDevices(
+      mojom::NetworkRequestResult::kBadRequest,
+      std::vector<std::string>{
+          std::string(), "$%^&*#",
+          "thisislongerthaneightbytes=="} /* invalid_instance_ids */);
+}
+
+TEST_F(DeviceSyncClientImplTest, TestGetDevicesActivityStatus_NoErrorCode) {
+  SetupClient();
+  std::vector<mojom::DeviceActivityStatusPtr> expected_activity_statuses;
+  expected_activity_statuses.emplace_back(mojom::DeviceActivityStatus::New(
+      "deviceid", base::Time(), cryptauthv2::ConnectivityStatus::ONLINE,
+      base::Time()));
+
+  CallGetDevicesActivityStatus(mojom::NetworkRequestResult::kSuccess,
+                               std::move(expected_activity_statuses));
+}
+
+TEST_F(DeviceSyncClientImplTest, TestGetDevicesActivityStatus_ErrorCode) {
+  SetupClient();
+
+  CallGetDevicesActivityStatus(mojom::NetworkRequestResult::kEndpointNotFound,
+                               base::nullopt);
 }
 
 TEST_F(DeviceSyncClientImplTest, TestGetDebugInfo) {
-  InitializeClient();
+  SetupClient();
 
   CallGetDebugInfo();
 }
