@@ -6,9 +6,11 @@
 
 #include "base/macros.h"
 #include "base/timer/mock_timer.h"
+#include "chromeos/components/multidevice/remote_device_test_util.h"
+#include "chromeos/components/multidevice/software_feature.h"
+#include "chromeos/components/multidevice/software_feature_state.h"
 #include "chromeos/services/device_sync/public/cpp/fake_device_sync_client.h"
-#include "components/cryptauth/fake_gcm_device_info_provider.h"
-#include "components/cryptauth/remote_device_test_util.h"
+#include "chromeos/services/device_sync/public/cpp/fake_gcm_device_info_provider.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace chromeos {
@@ -18,7 +20,7 @@ namespace multidevice_setup {
 class MultiDeviceSetupDeviceReenrollerTest : public testing::Test {
  protected:
   MultiDeviceSetupDeviceReenrollerTest()
-      : test_local_device_(cryptauth::CreateRemoteDeviceRefForTest()) {}
+      : test_local_device_(multidevice::CreateRemoteDeviceRefForTest()) {}
   ~MultiDeviceSetupDeviceReenrollerTest() override = default;
 
   // testing::Test:
@@ -28,14 +30,14 @@ class MultiDeviceSetupDeviceReenrollerTest : public testing::Test {
     fake_device_sync_client_->NotifyReady();
 
     fake_gcm_device_info_provider_ =
-        std::make_unique<cryptauth::FakeGcmDeviceInfoProvider>(
+        std::make_unique<device_sync::FakeGcmDeviceInfoProvider>(
             cryptauth::GcmDeviceInfo());
   }
 
   void SetLocalDeviceMetadataSoftwareFeaturesMap(
-      const std::map<cryptauth::SoftwareFeature,
-                     cryptauth::SoftwareFeatureState>& map) {
-    cryptauth::GetMutableRemoteDevice(test_local_device_)->software_features =
+      const std::map<multidevice::SoftwareFeature,
+                     multidevice::SoftwareFeatureState>& map) {
+    multidevice::GetMutableRemoteDevice(test_local_device_)->software_features =
         map;
     fake_device_sync_client_->set_local_device_metadata(test_local_device_);
   }
@@ -49,7 +51,8 @@ class MultiDeviceSetupDeviceReenrollerTest : public testing::Test {
       gcm_device_info.add_supported_software_features(feature);
     }
     fake_gcm_device_info_provider_ =
-        std::make_unique<cryptauth::FakeGcmDeviceInfoProvider>(gcm_device_info);
+        std::make_unique<device_sync::FakeGcmDeviceInfoProvider>(
+            gcm_device_info);
   }
 
   device_sync::FakeDeviceSyncClient* fake_device_sync_client() {
@@ -62,7 +65,7 @@ class MultiDeviceSetupDeviceReenrollerTest : public testing::Test {
     auto mock_timer = std::make_unique<base::MockOneShotTimer>();
     mock_timer_ = mock_timer.get();
 
-    device_reenroller_ = DeviceReenroller::Factory::Get()->BuildInstance(
+    device_reenroller_ = DeviceReenroller::Factory::Create(
         fake_device_sync_client_.get(), fake_gcm_device_info_provider_.get(),
         std::move(mock_timer));
   }
@@ -81,10 +84,10 @@ class MultiDeviceSetupDeviceReenrollerTest : public testing::Test {
   }
 
  private:
-  cryptauth::RemoteDeviceRef test_local_device_;
+  multidevice::RemoteDeviceRef test_local_device_;
 
   std::unique_ptr<device_sync::FakeDeviceSyncClient> fake_device_sync_client_;
-  std::unique_ptr<cryptauth::FakeGcmDeviceInfoProvider>
+  std::unique_ptr<device_sync::FakeGcmDeviceInfoProvider>
       fake_gcm_device_info_provider_;
   base::MockOneShotTimer* mock_timer_;
 
@@ -98,11 +101,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Set the current GcmDeviceInfo supported software features to contain the
   // same set.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -125,11 +128,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Remove one supported software feature in the GcmDeviceInfo.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
       std::vector<cryptauth::SoftwareFeature>{
@@ -146,9 +149,9 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   fake_device_sync_client()->NotifyEnrollmentFinished();
   EXPECT_EQ(1, fake_device_sync_client()->GetForceSyncNowCallbackQueueSize());
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   fake_device_sync_client()->InvokePendingForceSyncNowCallback(
       true /* success */);
   fake_device_sync_client()->NotifyNewDevicesSynced();
@@ -161,11 +164,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Add one more supported software feature in the GcmDeviceInfo.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
       std::vector<cryptauth::SoftwareFeature>{
@@ -184,13 +187,13 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   fake_device_sync_client()->NotifyEnrollmentFinished();
   EXPECT_EQ(1, fake_device_sync_client()->GetForceSyncNowCallbackQueueSize());
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::MAGIC_TETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kInstantTetheringClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   fake_device_sync_client()->InvokePendingForceSyncNowCallback(
       true /* success */);
   fake_device_sync_client()->NotifyNewDevicesSynced();
@@ -203,11 +206,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Add one more supported software feature in the GcmDeviceInfo to trigger a
   // re-enrollment attempt.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -236,11 +239,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Add one more supported software feature in the GcmDeviceInfo to trigger a
   // re-enrollment attempt.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -274,11 +277,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Add one more supported software feature in the GcmDeviceInfo to trigger a
   // re-enrollment attempt.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -298,11 +301,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   EXPECT_EQ(1, fake_device_sync_client()->GetForceSyncNowCallbackQueueSize());
   // Assume local device metadata was not updated correctly.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   fake_device_sync_client()->InvokePendingForceSyncNowCallback(
       true /* success */);
   fake_device_sync_client()->NotifyNewDevicesSynced();
@@ -319,13 +322,13 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::MAGIC_TETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kInstantTetheringClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Add one more supported software feature in the GcmDeviceInfo.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
       std::vector<cryptauth::SoftwareFeature>{
@@ -346,15 +349,15 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   fake_device_sync_client()->NotifyEnrollmentFinished();
   EXPECT_EQ(1, fake_device_sync_client()->GetForceSyncNowCallbackQueueSize());
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::MAGIC_TETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::SMS_CONNECT_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kInstantTetheringClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kMessagesForWebClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   fake_device_sync_client()->InvokePendingForceSyncNowCallback(
       true /* success */);
   fake_device_sync_client()->NotifyNewDevicesSynced();
@@ -368,13 +371,13 @@ TEST_F(
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::MAGIC_TETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kInstantTetheringClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Add one more supported software feature in the GcmDeviceInfo.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
       std::vector<cryptauth::SoftwareFeature>{
@@ -398,11 +401,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Set the current GcmDeviceInfo supported software features to contain the
   // same set.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -427,11 +430,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Set the current GcmDeviceInfo supported software features to contain the
   // same set.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -456,11 +459,11 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
   // Set the current local device metadata to contain a sample of supported
   // software features.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::BETTER_TOGETHER_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported},
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kBetterTogetherClient,
+           multidevice::SoftwareFeatureState::kSupported},
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
   // Set the current GcmDeviceInfo supported software features to contain the
   // same set.
   SetFakeGcmDeviceInfoProviderWithSupportedSoftwareFeatures(
@@ -474,9 +477,9 @@ TEST_F(MultiDeviceSetupDeviceReenrollerTest,
 
   // Remove a feature from the metadata.
   SetLocalDeviceMetadataSoftwareFeaturesMap(
-      std::map<cryptauth::SoftwareFeature, cryptauth::SoftwareFeatureState>{
-          {cryptauth::SoftwareFeature::EASY_UNLOCK_CLIENT,
-           cryptauth::SoftwareFeatureState::kSupported}});
+      std::map<multidevice::SoftwareFeature, multidevice::SoftwareFeatureState>{
+          {multidevice::SoftwareFeature::kSmartLockClient,
+           multidevice::SoftwareFeatureState::kSupported}});
 
   fake_device_sync_client()->NotifyNewDevicesSynced();
 
