@@ -12,15 +12,19 @@ FakeAccountStatusChangeDelegate::FakeAccountStatusChangeDelegate() = default;
 
 FakeAccountStatusChangeDelegate::~FakeAccountStatusChangeDelegate() = default;
 
-mojom::AccountStatusChangeDelegatePtr
-FakeAccountStatusChangeDelegate::GenerateInterfacePtr() {
-  mojom::AccountStatusChangeDelegatePtr interface_ptr;
-  bindings_.AddBinding(this, mojo::MakeRequest(&interface_ptr));
-  return interface_ptr;
+mojo::PendingRemote<mojom::AccountStatusChangeDelegate>
+FakeAccountStatusChangeDelegate::GenerateRemote() {
+  mojo::PendingRemote<mojom::AccountStatusChangeDelegate> remote;
+  receivers_.Add(this, remote.InitWithNewPipeAndPassReceiver());
+  return remote;
 }
 
 void FakeAccountStatusChangeDelegate::OnPotentialHostExistsForNewUser() {
-  ++num_new_user_events_handled_;
+  ++num_new_user_potential_host_events_handled_;
+}
+
+void FakeAccountStatusChangeDelegate::OnNoLongerNewUser() {
+  ++num_no_longer_new_user_events_handled_;
 }
 
 void FakeAccountStatusChangeDelegate::OnConnectedHostSwitchedForExistingUser(
@@ -28,10 +32,15 @@ void FakeAccountStatusChangeDelegate::OnConnectedHostSwitchedForExistingUser(
   ++num_existing_user_host_switched_events_handled_;
 }
 
-void FakeAccountStatusChangeDelegate::OnNewChromebookAddedForExistingUser() {
+void FakeAccountStatusChangeDelegate::OnNewChromebookAddedForExistingUser(
+    const std::string& new_host_device_name) {
   ++num_existing_user_chromebook_added_events_handled_;
 }
 
+void FakeAccountStatusChangeDelegate::OnBecameEligibleForWifiSync() {
+  ++num_eligible_for_wifi_sync_events_handled_;
+}
+
 }  // namespace multidevice_setup
 
 }  // namespace chromeos
