@@ -7,7 +7,8 @@
 
 #include "base/macros.h"
 #include "chromeos/services/multidevice_setup/public/mojom/multidevice_setup.mojom.h"
-#include "mojo/public/cpp/bindings/binding_set.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/receiver_set.h"
 
 namespace chromeos {
 
@@ -20,30 +21,45 @@ class FakeAccountStatusChangeDelegate
   FakeAccountStatusChangeDelegate();
   ~FakeAccountStatusChangeDelegate() override;
 
-  mojom::AccountStatusChangeDelegatePtr GenerateInterfacePtr();
+  mojo::PendingRemote<mojom::AccountStatusChangeDelegate> GenerateRemote();
 
-  size_t num_new_user_events_handled() { return num_new_user_events_handled_; }
+  size_t num_new_user_potential_host_events_handled() const {
+    return num_new_user_potential_host_events_handled_;
+  }
 
-  size_t num_existing_user_host_switched_events_handled() {
+  size_t num_no_longer_new_user_events_handled() const {
+    return num_no_longer_new_user_events_handled_;
+  }
+
+  size_t num_existing_user_host_switched_events_handled() const {
     return num_existing_user_host_switched_events_handled_;
   }
 
-  size_t num_existing_user_chromebook_added_events_handled() {
+  size_t num_existing_user_chromebook_added_events_handled() const {
     return num_existing_user_chromebook_added_events_handled_;
   }
 
+  size_t num_eligible_for_wifi_sync_events_handled() const {
+    return num_eligible_for_wifi_sync_events_handled_;
+  }
+
   // mojom::AccountStatusChangeDelegate:
   void OnPotentialHostExistsForNewUser() override;
+  void OnNoLongerNewUser() override;
   void OnConnectedHostSwitchedForExistingUser(
       const std::string& new_host_device_name) override;
-  void OnNewChromebookAddedForExistingUser() override;
+  void OnNewChromebookAddedForExistingUser(
+      const std::string& new_host_device_name) override;
+  void OnBecameEligibleForWifiSync() override;
 
  private:
-  size_t num_new_user_events_handled_ = 0u;
+  size_t num_new_user_potential_host_events_handled_ = 0u;
+  size_t num_no_longer_new_user_events_handled_ = 0u;
   size_t num_existing_user_host_switched_events_handled_ = 0u;
   size_t num_existing_user_chromebook_added_events_handled_ = 0u;
+  size_t num_eligible_for_wifi_sync_events_handled_ = 0u;
 
-  mojo::BindingSet<mojom::AccountStatusChangeDelegate> bindings_;
+  mojo::ReceiverSet<mojom::AccountStatusChangeDelegate> receivers_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeAccountStatusChangeDelegate);
 };
