@@ -12,11 +12,16 @@ FakeEligibleHostDevicesProvider::FakeEligibleHostDevicesProvider() = default;
 
 FakeEligibleHostDevicesProvider::~FakeEligibleHostDevicesProvider() = default;
 
-cryptauth::RemoteDeviceRefList
+multidevice::RemoteDeviceRefList
 FakeEligibleHostDevicesProvider::GetEligibleHostDevices() const {
   return eligible_host_devices_;
 }
 
+multidevice::DeviceWithConnectivityStatusList
+FakeEligibleHostDevicesProvider::GetEligibleActiveHostDevices() const {
+  return eligible_active_host_devices_;
+}
+
 }  // namespace multidevice_setup
 
 }  // namespace chromeos
