@@ -7,8 +7,9 @@
 
 #include "base/containers/flat_set.h"
 #include "base/macros.h"
+#include "chromeos/components/multidevice/remote_device_ref.h"
 #include "chromeos/services/multidevice_setup/eligible_host_devices_provider.h"
-#include "components/cryptauth/remote_device_ref.h"
+#include "chromeos/services/multidevice_setup/public/mojom/multidevice_setup.mojom.h"
 
 namespace chromeos {
 
@@ -21,15 +22,24 @@ class FakeEligibleHostDevicesProvider : public EligibleHostDevicesProvider {
   ~FakeEligibleHostDevicesProvider() override;
 
   void set_eligible_host_devices(
-      const cryptauth::RemoteDeviceRefList eligible_host_devices) {
+      const multidevice::RemoteDeviceRefList eligible_host_devices) {
     eligible_host_devices_ = eligible_host_devices;
   }
 
+  void set_eligible_active_host_devices(
+      const multidevice::DeviceWithConnectivityStatusList
+          eligible_host_devices) {
+    eligible_active_host_devices_ = eligible_host_devices;
+  }
+
  private:
   // EligibleHostDevicesProvider:
-  cryptauth::RemoteDeviceRefList GetEligibleHostDevices() const override;
+  multidevice::RemoteDeviceRefList GetEligibleHostDevices() const override;
+  multidevice::DeviceWithConnectivityStatusList GetEligibleActiveHostDevices()
+      const override;
 
-  cryptauth::RemoteDeviceRefList eligible_host_devices_;
+  multidevice::RemoteDeviceRefList eligible_host_devices_;
+  multidevice::DeviceWithConnectivityStatusList eligible_active_host_devices_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeEligibleHostDevicesProvider);
 };
