@@ -12,15 +12,16 @@ FakeHostStatusObserver::FakeHostStatusObserver() = default;
 
 FakeHostStatusObserver::~FakeHostStatusObserver() = default;
 
-mojom::HostStatusObserverPtr FakeHostStatusObserver::GenerateInterfacePtr() {
-  mojom::HostStatusObserverPtr interface_ptr;
-  bindings_.AddBinding(this, mojo::MakeRequest(&interface_ptr));
-  return interface_ptr;
+mojo::PendingRemote<mojom::HostStatusObserver>
+FakeHostStatusObserver::GenerateRemote() {
+  mojo::PendingRemote<mojom::HostStatusObserver> remote;
+  receivers_.Add(this, remote.InitWithNewPipeAndPassReceiver());
+  return remote;
 }
 
 void FakeHostStatusObserver::OnHostStatusChanged(
     mojom::HostStatus host_status,
-    const base::Optional<cryptauth::RemoteDevice>& host_device) {
+    const base::Optional<multidevice::RemoteDevice>& host_device) {
   host_status_updates_.emplace_back(host_status, host_device);
 }
 
