@@ -4,10 +4,17 @@
 
 #include "chromeos/services/multidevice_setup/host_verifier_impl.h"
 
+#include <utility>
+
+#include "ash/constants/ash_features.h"
+#include "base/bind.h"
+#include "base/callback_helpers.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
-#include "base/no_destructor.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "base/metrics/histogram_functions.h"
+#include "chromeos/components/multidevice/logging/logging.h"
+#include "chromeos/components/multidevice/software_feature.h"
+#include "chromeos/services/device_sync/proto/cryptauth_common.pb.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/pref_service.h"
 
@@ -18,10 +25,10 @@ namespace multidevice_setup {
 namespace {
 
 // Software features which, when enabled, represent a verified host.
-constexpr const cryptauth::SoftwareFeature kPotentialHostFeatures[] = {
-    cryptauth::SoftwareFeature::EASY_UNLOCK_HOST,
-    cryptauth::SoftwareFeature::MAGIC_TETHER_HOST,
-    cryptauth::SoftwareFeature::SMS_CONNECT_HOST};
+constexpr const multidevice::SoftwareFeature kPotentialHostFeatures[] = {
+    multidevice::SoftwareFeature::kSmartLockHost,
+    multidevice::SoftwareFeature::kInstantTetheringHost,
+    multidevice::SoftwareFeature::kMessagesForWebHost};
 
 // Name of the preference containing the time (in milliseconds since Unix
 // epoch) at which a verification attempt should be retried. If the preference
@@ -44,6 +51,10 @@ const char kLastUsedTimeDeltaMsPrefName[] =
 constexpr const base::TimeDelta kFirstRetryDelta =
     base::TimeDelta::FromMinutes(10);
 
+// Delta for the time between a successful FindEligibleDevices call and a
+// request to sync devices.
+constexpr const base::TimeDelta kSyncDelay = base::TimeDelta::FromSeconds(5);
+
 // The multiplier for increasing the backoff timer between retries.
 const double kExponentialBackoffMultiplier = 1.5;
 
@@ -53,12 +64,22 @@ const double kExponentialBackoffMultiplier = 1.5;
 HostVerifierImpl::Factory* HostVerifierImpl::Factory::test_factory_ = nullptr;
 
 // static
-HostVerifierImpl::Factory* HostVerifierImpl::Factory::Get() {
-  if (test_factory_)
-    return test_factory_;
+std::unique_ptr<HostVerifier> HostVerifierImpl::Factory::Create(
+    HostBackendDelegate* host_backend_delegate,
+    device_sync::DeviceSyncClient* device_sync_client,
+    PrefService* pref_service,
+    base::Clock* clock,
+    std::unique_ptr<base::OneShotTimer> retry_timer,
+    std::unique_ptr<base::OneShotTimer> sync_timer) {
+  if (test_factory_) {
+    return test_factory_->CreateInstance(
+        host_backend_delegate, device_sync_client, pref_service, clock,
+        std::move(retry_timer), std::move(sync_timer));
+  }
 
-  static base::NoDestructor<Factory> factory;
-  return factory.get();
+  return base::WrapUnique(new HostVerifierImpl(
+      host_backend_delegate, device_sync_client, pref_service, clock,
+      std::move(retry_timer), std::move(sync_timer)));
 }
 
 // static
@@ -68,17 +89,6 @@ void HostVerifierImpl::Factory::SetFactoryForTesting(Factory* test_factory) {
 
 HostVerifierImpl::Factory::~Factory() = default;
 
-std::unique_ptr<HostVerifier> HostVerifierImpl::Factory::BuildInstance(
-    HostBackendDelegate* host_backend_delegate,
-    device_sync::DeviceSyncClient* device_sync_client,
-    PrefService* pref_service,
-    base::Clock* clock,
-    std::unique_ptr<base::OneShotTimer> timer) {
-  return base::WrapUnique(new HostVerifierImpl(host_backend_delegate,
-                                               device_sync_client, pref_service,
-                                               clock, std::move(timer)));
-}
-
 // static
 void HostVerifierImpl::RegisterPrefs(PrefRegistrySimple* registry) {
   registry->RegisterInt64Pref(kRetryTimestampPrefName, kTimestampNotSet);
@@ -90,12 +100,14 @@ HostVerifierImpl::HostVerifierImpl(
     device_sync::DeviceSyncClient* device_sync_client,
     PrefService* pref_service,
     base::Clock* clock,
-    std::unique_ptr<base::OneShotTimer> timer)
+    std::unique_ptr<base::OneShotTimer> retry_timer,
+    std::unique_ptr<base::OneShotTimer> sync_timer)
     : host_backend_delegate_(host_backend_delegate),
       device_sync_client_(device_sync_client),
       pref_service_(pref_service),
       clock_(clock),
-      timer_(std::move(timer)) {
+      retry_timer_(std::move(retry_timer)),
+      sync_timer_(std::move(sync_timer)) {
   host_backend_delegate_->AddObserver(this);
   device_sync_client_->AddObserver(this);
 
@@ -108,7 +120,7 @@ HostVerifierImpl::~HostVerifierImpl() {
 }
 
 bool HostVerifierImpl::IsHostVerified() {
-  base::Optional<cryptauth::RemoteDeviceRef> current_host =
+  base::Optional<multidevice::RemoteDeviceRef> current_host =
       host_backend_delegate_->GetMultiDeviceHostFromBackend();
   if (!current_host)
     return false;
@@ -120,11 +132,23 @@ bool HostVerifierImpl::IsHostVerified() {
     return false;
   }
 
+  // The host is not considered verified if it does not have the data needed for
+  // secure communication via Bluetooth. These values could be missing if v2
+  // DeviceSync data was not decrypted, for instance.
+  bool has_crypto_data = !current_host->public_key().empty() &&
+                         !current_host->persistent_symmetric_key().empty() &&
+                         !current_host->beacon_seeds().empty();
+  base::UmaHistogramBoolean(
+      "MultiDevice.Setup.HostVerifier.DoesHostHaveCryptoData", has_crypto_data);
+  if (!has_crypto_data) {
+    return false;
+  }
+
   // If one or more potential host sofware features is enabled, the host is
   // considered verified.
   for (const auto& software_feature : kPotentialHostFeatures) {
     if (current_host->GetSoftwareFeatureState(software_feature) ==
-        cryptauth::SoftwareFeatureState::kEnabled) {
+        multidevice::SoftwareFeatureState::kEnabled) {
       return true;
     }
   }
@@ -147,21 +171,22 @@ void HostVerifierImpl::OnNewDevicesSynced() {
 void HostVerifierImpl::UpdateRetryState() {
   // If there is no host, verification is not applicable.
   if (!host_backend_delegate_->GetMultiDeviceHostFromBackend()) {
-    StopTimerAndClearPrefs();
+    StopRetryTimerAndClearPrefs();
     return;
   }
 
   // If there is a host and it is verified, verification is no longer necessary.
   if (IsHostVerified()) {
-    bool was_timer_running = timer_->IsRunning();
-    StopTimerAndClearPrefs();
-    if (was_timer_running)
+    sync_timer_->Stop();
+    bool was_retry_timer_running = retry_timer_->IsRunning();
+    StopRetryTimerAndClearPrefs();
+    if (was_retry_timer_running)
       NotifyHostVerified();
     return;
   }
 
-  // If |timer_| is running, an ongoing retry attempt is in progress.
-  if (timer_->IsRunning())
+  // If |retry_timer_| is running, an ongoing retry attempt is in progress.
+  if (retry_timer_->IsRunning())
     return;
 
   int64_t timestamp_from_prefs =
@@ -179,15 +204,15 @@ void HostVerifierImpl::UpdateRetryState() {
 
   // If a timeout value was set but has not yet occurred, start the timer.
   if (clock_->Now() < retry_time_from_prefs) {
-    StartTimer(retry_time_from_prefs);
+    StartRetryTimer(retry_time_from_prefs);
     return;
   }
 
   AttemptVerificationAfterInitialTimeout(retry_time_from_prefs);
 }
 
-void HostVerifierImpl::StopTimerAndClearPrefs() {
-  timer_->Stop();
+void HostVerifierImpl::StopRetryTimerAndClearPrefs() {
+  retry_timer_->Stop();
   pref_service_->SetInt64(kRetryTimestampPrefName, kTimestampNotSet);
   pref_service_->SetInt64(kLastUsedTimeDeltaMsPrefName, 0);
 }
@@ -199,7 +224,7 @@ void HostVerifierImpl::AttemptVerificationWithInitialTimeout() {
   pref_service_->SetInt64(kLastUsedTimeDeltaMsPrefName,
                           kFirstRetryDelta.InMilliseconds());
 
-  StartTimer(retry_time);
+  StartRetryTimer(retry_time);
   AttemptHostVerification();
 }
 
@@ -217,21 +242,21 @@ void HostVerifierImpl::AttemptVerificationAfterInitialTimeout(
   pref_service_->SetInt64(kRetryTimestampPrefName, retry_time.ToJavaTime());
   pref_service_->SetInt64(kLastUsedTimeDeltaMsPrefName, time_delta_ms);
 
-  StartTimer(retry_time);
+  StartRetryTimer(retry_time);
   AttemptHostVerification();
 }
 
-void HostVerifierImpl::StartTimer(const base::Time& time_to_fire) {
+void HostVerifierImpl::StartRetryTimer(const base::Time& time_to_fire) {
   base::Time now = clock_->Now();
   DCHECK(now < time_to_fire);
 
-  timer_->Start(
-      FROM_HERE, time_to_fire - now /* delay */,
-      base::Bind(&HostVerifierImpl::UpdateRetryState, base::Unretained(this)));
+  retry_timer_->Start(FROM_HERE, time_to_fire - now /* delay */,
+                      base::BindOnce(&HostVerifierImpl::UpdateRetryState,
+                                     base::Unretained(this)));
 }
 
 void HostVerifierImpl::AttemptHostVerification() {
-  base::Optional<cryptauth::RemoteDeviceRef> current_host =
+  base::Optional<multidevice::RemoteDeviceRef> current_host =
       host_backend_delegate_->GetMultiDeviceHostFromBackend();
   if (!current_host) {
     PA_LOG(WARNING) << "HostVerifierImpl::AttemptHostVerification(): Cannot "
@@ -239,10 +264,77 @@ void HostVerifierImpl::AttemptHostVerification() {
     return;
   }
 
-  PA_LOG(INFO) << "HostVerifierImpl::AttemptHostVerification(): Attempting "
-               << "host verification now.";
-  device_sync_client_->FindEligibleDevices(
-      cryptauth::SoftwareFeature::BETTER_TOGETHER_HOST, base::DoNothing());
+  PA_LOG(VERBOSE) << "HostVerifierImpl::AttemptHostVerification(): Attempting "
+                  << "host verification now.";
+
+  if (features::ShouldUseV1DeviceSync()) {
+    if (current_host->instance_id().empty()) {
+      device_sync_client_->FindEligibleDevices(
+          multidevice::SoftwareFeature::kBetterTogetherHost,
+          base::BindOnce(&HostVerifierImpl::OnFindEligibleDevicesResult,
+                         weak_ptr_factory_.GetWeakPtr()));
+    } else {
+      device_sync_client_->NotifyDevices(
+          {current_host->instance_id()},
+          cryptauthv2::TargetService::DEVICE_SYNC,
+          multidevice::SoftwareFeature::kBetterTogetherHost,
+          base::BindOnce(&HostVerifierImpl::OnNotifyDevicesFinished,
+                         weak_ptr_factory_.GetWeakPtr()));
+    }
+  } else {
+    DCHECK(!current_host->instance_id().empty());
+    device_sync_client_->NotifyDevices(
+        {current_host->instance_id()}, cryptauthv2::TargetService::DEVICE_SYNC,
+        multidevice::SoftwareFeature::kBetterTogetherHost,
+        base::BindOnce(&HostVerifierImpl::OnNotifyDevicesFinished,
+                       weak_ptr_factory_.GetWeakPtr()));
+  }
+}
+
+void HostVerifierImpl::OnFindEligibleDevicesResult(
+    device_sync::mojom::NetworkRequestResult result,
+    multidevice::RemoteDeviceRefList eligible_devices,
+    multidevice::RemoteDeviceRefList ineligible_devices) {
+  if (result != device_sync::mojom::NetworkRequestResult::kSuccess) {
+    PA_LOG(WARNING) << "HostVerifierImpl::OnFindEligibleDevicesResult(): "
+                    << "FindEligibleDevices call failed. Retry is scheduled.";
+    return;
+  }
+
+  // Now that the FindEligibleDevices call was sent successfully, the host phone
+  // is expected to enable its supported features. This should trigger a push
+  // message asking this Chromebook to sync these updated features, but in
+  // practice it has been observed that the Chromebook sometimes does not
+  // receive this message (see https://crbug.com/913816). Thus, schedule a sync
+  // after the phone has had enough time to enable its features. Note that this
+  // sync is canceled if the Chromebook does receive the push message.
+  sync_timer_->Start(FROM_HERE, kSyncDelay,
+                     base::BindOnce(&HostVerifierImpl::OnSyncTimerFired,
+                                    base::Unretained(this)));
+}
+
+void HostVerifierImpl::OnNotifyDevicesFinished(
+    device_sync::mojom::NetworkRequestResult result) {
+  if (result != device_sync::mojom::NetworkRequestResult::kSuccess) {
+    PA_LOG(WARNING) << "HostVerifierImpl::OnNotifyDevicesFinished(): "
+                    << "NotifyDevices call failed. Retry is scheduled.";
+    return;
+  }
+
+  // Now that the NotifyDevices call was sent successfully, the host phone is
+  // expected to enable its supported features. This should trigger a push
+  // message asking this Chromebook to sync these updated features, but in
+  // practice it has been observed that the Chromebook sometimes does not
+  // receive this message (see https://crbug.com/913816). Thus, schedule a sync
+  // after the phone has had enough time to enable its features. Note that this
+  // sync is canceled if the Chromebook does receive the push message.
+  sync_timer_->Start(FROM_HERE, kSyncDelay,
+                     base::BindOnce(&HostVerifierImpl::OnSyncTimerFired,
+                                    base::Unretained(this)));
+}
+
+void HostVerifierImpl::OnSyncTimerFired() {
+  device_sync_client_->ForceSyncNow(base::DoNothing());
 }
 
 }  // namespace multidevice_setup
