@@ -2,12 +2,19 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <utility>
+#include <vector>
+
 #include "chromeos/services/multidevice_setup/multidevice_setup_impl.h"
 
+#include "ash/constants/ash_features.h"
+#include "base/containers/contains.h"
+#include "base/containers/flat_set.h"
 #include "base/memory/ptr_util.h"
-#include "base/no_destructor.h"
+#include "base/metrics/histogram_functions.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/time/default_clock.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 #include "chromeos/services/multidevice_setup/account_status_change_delegate_notifier_impl.h"
 #include "chromeos/services/multidevice_setup/android_sms_app_installing_status_observer.h"
 #include "chromeos/services/multidevice_setup/device_reenroller.h"
@@ -22,6 +29,7 @@
 #include "chromeos/services/multidevice_setup/public/cpp/android_sms_pairing_state_tracker.h"
 #include "chromeos/services/multidevice_setup/public/cpp/auth_token_validator.h"
 #include "chromeos/services/multidevice_setup/public/cpp/oobe_completion_tracker.h"
+#include "chromeos/services/multidevice_setup/wifi_sync_feature_manager_impl.h"
 
 namespace chromeos {
 
@@ -29,6 +37,25 @@ namespace multidevice_setup {
 
 namespace {
 const char kTestDeviceNameForDebugNotification[] = "Test Device";
+
+// This enum is tied directly to a UMA enum defined in
+// //tools/metrics/histograms/enums.xml, and should always reflect it (do not
+// change one without changing the other). Entries should be never modified
+// or deleted. Only additions possible.
+enum class VerifyAndForgetHostConfirmationState {
+  kButtonClickedState = 0,
+  kCompletedSetupState = 1,
+  kMaxValue = kCompletedSetupState,
+};
+
+static void LogForgetHostConfirmed(VerifyAndForgetHostConfirmationState state) {
+  UMA_HISTOGRAM_ENUMERATION("MultiDevice.ForgetHostConfirmed", state);
+}
+
+static void LogVerifyButtonClicked(VerifyAndForgetHostConfirmationState state) {
+  UMA_HISTOGRAM_ENUMERATION("MultiDevice.VerifyButtonClicked", state);
+}
+
 }  // namespace
 
 // static
@@ -36,12 +63,28 @@ MultiDeviceSetupImpl::Factory* MultiDeviceSetupImpl::Factory::test_factory_ =
     nullptr;
 
 // static
-MultiDeviceSetupImpl::Factory* MultiDeviceSetupImpl::Factory::Get() {
-  if (test_factory_)
-    return test_factory_;
+std::unique_ptr<MultiDeviceSetupBase> MultiDeviceSetupImpl::Factory::Create(
+    PrefService* pref_service,
+    device_sync::DeviceSyncClient* device_sync_client,
+    AuthTokenValidator* auth_token_validator,
+    OobeCompletionTracker* oobe_completion_tracker,
+    AndroidSmsAppHelperDelegate* android_sms_app_helper_delegate,
+    AndroidSmsPairingStateTracker* android_sms_pairing_state_tracker,
+    const device_sync::GcmDeviceInfoProvider* gcm_device_info_provider,
+    bool is_secondary_user) {
+  if (test_factory_) {
+    return test_factory_->CreateInstance(
+        pref_service, device_sync_client, auth_token_validator,
+        oobe_completion_tracker, android_sms_app_helper_delegate,
+        android_sms_pairing_state_tracker, gcm_device_info_provider,
+        is_secondary_user);
+  }
 
-  static base::NoDestructor<Factory> factory;
-  return factory.get();
+  return base::WrapUnique(new MultiDeviceSetupImpl(
+      pref_service, device_sync_client, auth_token_validator,
+      oobe_completion_tracker, android_sms_app_helper_delegate,
+      android_sms_pairing_state_tracker, gcm_device_info_provider,
+      is_secondary_user));
 }
 
 // static
@@ -52,82 +95,70 @@ void MultiDeviceSetupImpl::Factory::SetFactoryForTesting(
 
 MultiDeviceSetupImpl::Factory::~Factory() = default;
 
-std::unique_ptr<MultiDeviceSetupBase>
-MultiDeviceSetupImpl::Factory::BuildInstance(
-    PrefService* pref_service,
-    device_sync::DeviceSyncClient* device_sync_client,
-    AuthTokenValidator* auth_token_validator,
-    OobeCompletionTracker* oobe_completion_tracker,
-    std::unique_ptr<AndroidSmsAppHelperDelegate>
-        android_sms_app_helper_delegate,
-    std::unique_ptr<AndroidSmsPairingStateTracker>
-        android_sms_pairing_state_tracker,
-    const cryptauth::GcmDeviceInfoProvider* gcm_device_info_provider) {
-  return base::WrapUnique(new MultiDeviceSetupImpl(
-      pref_service, device_sync_client, auth_token_validator,
-      oobe_completion_tracker, std::move(android_sms_app_helper_delegate),
-      std::move(android_sms_pairing_state_tracker), gcm_device_info_provider));
-}
-
 MultiDeviceSetupImpl::MultiDeviceSetupImpl(
     PrefService* pref_service,
     device_sync::DeviceSyncClient* device_sync_client,
     AuthTokenValidator* auth_token_validator,
     OobeCompletionTracker* oobe_completion_tracker,
-    std::unique_ptr<AndroidSmsAppHelperDelegate>
-        android_sms_app_helper_delegate,
-    std::unique_ptr<AndroidSmsPairingStateTracker>
-        android_sms_pairing_state_tracker,
-    const cryptauth::GcmDeviceInfoProvider* gcm_device_info_provider)
+    AndroidSmsAppHelperDelegate* android_sms_app_helper_delegate,
+    AndroidSmsPairingStateTracker* android_sms_pairing_state_tracker,
+    const device_sync::GcmDeviceInfoProvider* gcm_device_info_provider,
+    bool is_secondary_user)
     : eligible_host_devices_provider_(
-          EligibleHostDevicesProviderImpl::Factory::Get()->BuildInstance(
-              device_sync_client)),
-      host_backend_delegate_(
-          HostBackendDelegateImpl::Factory::Get()->BuildInstance(
-              eligible_host_devices_provider_.get(),
-              pref_service,
-              device_sync_client)),
-      host_verifier_(HostVerifierImpl::Factory::Get()->BuildInstance(
+          EligibleHostDevicesProviderImpl::Factory::Create(device_sync_client)),
+      host_backend_delegate_(HostBackendDelegateImpl::Factory::Create(
+          eligible_host_devices_provider_.get(),
+          pref_service,
+          device_sync_client)),
+      host_verifier_(
+          HostVerifierImpl::Factory::Create(host_backend_delegate_.get(),
+                                            device_sync_client,
+                                            pref_service)),
+      host_status_provider_(HostStatusProviderImpl::Factory::Create(
+          eligible_host_devices_provider_.get(),
           host_backend_delegate_.get(),
-          device_sync_client,
-          pref_service)),
-      host_status_provider_(
-          HostStatusProviderImpl::Factory::Get()->BuildInstance(
-              eligible_host_devices_provider_.get(),
-              host_backend_delegate_.get(),
-              host_verifier_.get(),
-              device_sync_client)),
+          host_verifier_.get(),
+          device_sync_client)),
       grandfathered_easy_unlock_host_disabler_(
-          GrandfatheredEasyUnlockHostDisabler::Factory::Get()->BuildInstance(
+          GrandfatheredEasyUnlockHostDisabler::Factory::Create(
               host_backend_delegate_.get(),
               device_sync_client,
               pref_service)),
-      feature_state_manager_(
-          FeatureStateManagerImpl::Factory::Get()->BuildInstance(
-              pref_service,
-              host_status_provider_.get(),
-              device_sync_client,
-              std::move(android_sms_pairing_state_tracker))),
       host_device_timestamp_manager_(
-          HostDeviceTimestampManagerImpl::Factory::Get()->BuildInstance(
+          HostDeviceTimestampManagerImpl::Factory::Create(
               host_status_provider_.get(),
               pref_service,
               base::DefaultClock::GetInstance())),
       delegate_notifier_(
-          AccountStatusChangeDelegateNotifierImpl::Factory::Get()
-              ->BuildInstance(host_status_provider_.get(),
-                              pref_service,
-                              host_device_timestamp_manager_.get(),
-                              oobe_completion_tracker,
-                              base::DefaultClock::GetInstance())),
-      device_reenroller_(DeviceReenroller::Factory::Get()->BuildInstance(
-          device_sync_client,
-          gcm_device_info_provider)),
-      android_sms_app_installing_host_observer_(
-          AndroidSmsAppInstallingStatusObserver::Factory::Get()->BuildInstance(
+          AccountStatusChangeDelegateNotifierImpl::Factory::Create(
               host_status_provider_.get(),
-              feature_state_manager_.get(),
-              std::move(android_sms_app_helper_delegate))),
+              pref_service,
+              host_device_timestamp_manager_.get(),
+              oobe_completion_tracker,
+              base::DefaultClock::GetInstance())),
+      wifi_sync_feature_manager_(WifiSyncFeatureManagerImpl::Factory::Create(
+          host_status_provider_.get(),
+          pref_service,
+          device_sync_client,
+          delegate_notifier_.get())),
+      feature_state_manager_(FeatureStateManagerImpl::Factory::Create(
+          pref_service,
+          host_status_provider_.get(),
+          device_sync_client,
+          android_sms_pairing_state_tracker,
+          wifi_sync_feature_manager_.get(),
+          is_secondary_user)),
+      device_reenroller_(
+          DeviceReenroller::Factory::Create(device_sync_client,
+                                            gcm_device_info_provider)),
+      android_sms_app_installing_host_observer_(
+          android_sms_app_helper_delegate
+              ? AndroidSmsAppInstallingStatusObserver::Factory::Create(
+                    host_status_provider_.get(),
+                    feature_state_manager_.get(),
+                    android_sms_app_helper_delegate,
+                    pref_service)
+              : nullptr),
       auth_token_validator_(auth_token_validator) {
   host_status_provider_->AddObserver(this);
   feature_state_manager_->AddObserver(this);
@@ -139,54 +170,78 @@ MultiDeviceSetupImpl::~MultiDeviceSetupImpl() {
 }
 
 void MultiDeviceSetupImpl::SetAccountStatusChangeDelegate(
-    mojom::AccountStatusChangeDelegatePtr delegate) {
-  delegate_notifier_->SetAccountStatusChangeDelegatePtr(std::move(delegate));
+    mojo::PendingRemote<mojom::AccountStatusChangeDelegate> delegate) {
+  delegate_notifier_->SetAccountStatusChangeDelegateRemote(std::move(delegate));
 }
 
 void MultiDeviceSetupImpl::AddHostStatusObserver(
-    mojom::HostStatusObserverPtr observer) {
-  host_status_observers_.AddPtr(std::move(observer));
+    mojo::PendingRemote<mojom::HostStatusObserver> observer) {
+  host_status_observers_.Add(std::move(observer));
 }
 
 void MultiDeviceSetupImpl::AddFeatureStateObserver(
-    mojom::FeatureStateObserverPtr observer) {
-  feature_state_observers_.AddPtr(std::move(observer));
+    mojo::PendingRemote<mojom::FeatureStateObserver> observer) {
+  feature_state_observers_.Add(std::move(observer));
 }
 
 void MultiDeviceSetupImpl::GetEligibleHostDevices(
     GetEligibleHostDevicesCallback callback) {
-  std::vector<cryptauth::RemoteDevice> eligible_remote_devices;
+  std::vector<multidevice::RemoteDevice> eligible_remote_devices;
   for (const auto& remote_device_ref :
        eligible_host_devices_provider_->GetEligibleHostDevices()) {
     eligible_remote_devices.push_back(remote_device_ref.GetRemoteDevice());
   }
 
-  // Sort from most-recently-updated to least-recently-updated. The timestamp
-  // used is provided by the back-end and indicates the last time at which the
-  // device's metadata was updated on the server. Note that this does not
-  // provide us with the last time that a user actually used this device, but it
-  // is a good estimate.
-  std::sort(eligible_remote_devices.begin(), eligible_remote_devices.end(),
-            [](const auto& first_device, const auto& second_device) {
-              return first_device.last_update_time_millis >
-                     second_device.last_update_time_millis;
-            });
-
   std::move(callback).Run(eligible_remote_devices);
 }
 
-void MultiDeviceSetupImpl::SetHostDevice(const std::string& host_device_id,
-                                         const std::string& auth_token,
-                                         SetHostDeviceCallback callback) {
+void MultiDeviceSetupImpl::GetEligibleActiveHostDevices(
+    GetEligibleActiveHostDevicesCallback callback) {
+  // For metrics.
+  bool has_duplicate_host_name = false;
+  base::flat_set<std::string> name_set;
+
+  std::vector<mojom::HostDevicePtr> eligible_active_hosts;
+  for (const auto& host_device :
+       eligible_host_devices_provider_->GetEligibleActiveHostDevices()) {
+    // For metrics.
+    if (base::Contains(name_set, host_device.remote_device.name())) {
+      has_duplicate_host_name = true;
+      PA_LOG(WARNING) << "MultiDeviceSetupImpl::GetEligibleActiveHostDevices: "
+                      << "Detected duplicate eligible host device name \""
+                      << host_device.remote_device.name() << "\"";
+    } else {
+      name_set.insert(host_device.remote_device.name());
+    }
+
+    eligible_active_hosts.push_back(
+        mojom::HostDevice::New(host_device.remote_device.GetRemoteDevice(),
+                               host_device.connectivity_status));
+  }
+
+  base::UmaHistogramBoolean(
+      "MultiDevice.Setup.HasDuplicateEligibleHostDeviceNames",
+      has_duplicate_host_name);
+
+  std::move(callback).Run(std::move(eligible_active_hosts));
+}
+
+void MultiDeviceSetupImpl::SetHostDevice(
+    const std::string& host_instance_id_or_legacy_device_id,
+    const std::string& auth_token,
+    SetHostDeviceCallback callback) {
   if (!auth_token_validator_->IsAuthTokenValid(auth_token)) {
     std::move(callback).Run(false /* success */);
     return;
   }
 
-  std::move(callback).Run(AttemptSetHost(host_device_id));
+  std::move(callback).Run(AttemptSetHost(host_instance_id_or_legacy_device_id));
 }
 
 void MultiDeviceSetupImpl::RemoveHostDevice() {
+  LogForgetHostConfirmed(
+      VerifyAndForgetHostConfirmationState::kButtonClickedState);
+
   host_backend_delegate_->AttemptToSetMultiDeviceHostOnBackend(
       base::nullopt /* host_device */);
 }
@@ -195,9 +250,9 @@ void MultiDeviceSetupImpl::GetHostStatus(GetHostStatusCallback callback) {
   HostStatusProvider::HostStatusWithDevice host_status_with_device =
       host_status_provider_->GetHostWithStatus();
 
-  // The Mojo API requires a raw cryptauth::RemoteDevice instead of a
-  // cryptauth::RemoteDeviceRef.
-  base::Optional<cryptauth::RemoteDevice> device_for_callback;
+  // The Mojo API requires a raw multidevice::RemoteDevice instead of a
+  // multidevice::RemoteDeviceRef.
+  base::Optional<multidevice::RemoteDevice> device_for_callback;
   if (host_status_with_device.host_device()) {
     device_for_callback =
         host_status_with_device.host_device()->GetRemoteDevice();
@@ -214,6 +269,8 @@ void MultiDeviceSetupImpl::SetFeatureEnabledState(
     SetFeatureEnabledStateCallback callback) {
   if (IsAuthTokenRequiredForFeatureStateChange(feature, enabled) &&
       (!auth_token || !auth_token_validator_->IsAuthTokenValid(*auth_token))) {
+    PA_LOG(ERROR) << __func__ << " Cannot " << (enabled ? "enable" : "disable")
+                  << " " << feature << "; auth token invalid";
     std::move(callback).Run(false /* success */);
     return;
   }
@@ -227,6 +284,9 @@ void MultiDeviceSetupImpl::GetFeatureStates(GetFeatureStatesCallback callback) {
 }
 
 void MultiDeviceSetupImpl::RetrySetHostNow(RetrySetHostNowCallback callback) {
+  LogVerifyButtonClicked(
+      VerifyAndForgetHostConfirmationState::kButtonClickedState);
+
   HostStatusProvider::HostStatusWithDevice host_status_with_device =
       host_status_provider_->GetHostWithStatus();
 
@@ -252,17 +312,17 @@ void MultiDeviceSetupImpl::RetrySetHostNow(RetrySetHostNowCallback callback) {
 void MultiDeviceSetupImpl::TriggerEventForDebugging(
     mojom::EventTypeForDebugging type,
     TriggerEventForDebuggingCallback callback) {
-  if (!delegate_notifier_->delegate_ptr_) {
+  if (!delegate_notifier_->delegate_remote_) {
     PA_LOG(ERROR) << "MultiDeviceSetupImpl::TriggerEventForDebugging(): No "
                   << "delgate has been set; cannot proceed.";
     std::move(callback).Run(false /* success */);
     return;
   }
 
-  PA_LOG(INFO) << "MultiDeviceSetupImpl::TriggerEventForDebugging(" << type
-               << ") called.";
+  PA_LOG(VERBOSE) << "MultiDeviceSetupImpl::TriggerEventForDebugging(" << type
+                  << ") called.";
   mojom::AccountStatusChangeDelegate* delegate =
-      delegate_notifier_->delegate_ptr_.get();
+      delegate_notifier_->delegate_remote_.get();
 
   switch (type) {
     case mojom::EventTypeForDebugging::kNewUserPotentialHostExists:
@@ -284,50 +344,63 @@ void MultiDeviceSetupImpl::TriggerEventForDebugging(
 }
 
 void MultiDeviceSetupImpl::SetHostDeviceWithoutAuthToken(
-    const std::string& host_device_id,
+    const std::string& host_instance_id_or_legacy_device_id,
     mojom::PrivilegedHostDeviceSetter::SetHostDeviceCallback callback) {
-  std::move(callback).Run(AttemptSetHost(host_device_id));
+  std::move(callback).Run(AttemptSetHost(host_instance_id_or_legacy_device_id));
 }
 
 void MultiDeviceSetupImpl::OnHostStatusChange(
     const HostStatusProvider::HostStatusWithDevice& host_status_with_device) {
   mojom::HostStatus status_for_callback = host_status_with_device.host_status();
 
-  // The Mojo API requires a raw cryptauth::RemoteDevice instead of a
-  // cryptauth::RemoteDeviceRef.
-  base::Optional<cryptauth::RemoteDevice> device_for_callback;
+  // The Mojo API requires a raw multidevice::RemoteDevice instead of a
+  // multidevice::RemoteDeviceRef.
+  base::Optional<multidevice::RemoteDevice> device_for_callback;
   if (host_status_with_device.host_device()) {
     device_for_callback =
         host_status_with_device.host_device()->GetRemoteDevice();
   }
 
-  host_status_observers_.ForAllPtrs(
-      [&status_for_callback,
-       &device_for_callback](mojom::HostStatusObserver* observer) {
-        observer->OnHostStatusChanged(status_for_callback, device_for_callback);
-      });
+  for (auto& observer : host_status_observers_)
+    observer->OnHostStatusChanged(status_for_callback, device_for_callback);
 }
 
 void MultiDeviceSetupImpl::OnFeatureStatesChange(
     const FeatureStateManager::FeatureStatesMap& feature_states_map) {
-  feature_state_observers_.ForAllPtrs(
-      [&feature_states_map](mojom::FeatureStateObserver* observer) {
-        observer->OnFeatureStatesChanged(feature_states_map);
-      });
+  for (auto& observer : feature_state_observers_)
+    observer->OnFeatureStatesChanged(feature_states_map);
 }
 
-bool MultiDeviceSetupImpl::AttemptSetHost(const std::string& host_device_id) {
-  cryptauth::RemoteDeviceRefList eligible_devices =
+bool MultiDeviceSetupImpl::AttemptSetHost(
+    const std::string& host_instance_id_or_legacy_device_id) {
+  DCHECK(!host_instance_id_or_legacy_device_id.empty());
+
+  multidevice::RemoteDeviceRefList eligible_devices =
       eligible_host_devices_provider_->GetEligibleHostDevices();
-  auto it =
-      std::find_if(eligible_devices.begin(), eligible_devices.end(),
-                   [&host_device_id](const auto& eligible_device) {
-                     return eligible_device.GetDeviceId() == host_device_id;
-                   });
+
+  auto it = std::find_if(
+      eligible_devices.begin(), eligible_devices.end(),
+      [&host_instance_id_or_legacy_device_id](const auto& eligible_device) {
+        if (features::ShouldUseV1DeviceSync()) {
+          return eligible_device.instance_id() ==
+                     host_instance_id_or_legacy_device_id ||
+                 eligible_device.GetDeviceId() ==
+                     host_instance_id_or_legacy_device_id;
+        }
+
+        return eligible_device.instance_id() ==
+               host_instance_id_or_legacy_device_id;
+      });
 
   if (it == eligible_devices.end())
     return false;
 
+  LogForgetHostConfirmed(
+      VerifyAndForgetHostConfirmationState::kCompletedSetupState);
+
+  LogVerifyButtonClicked(
+      VerifyAndForgetHostConfirmationState::kCompletedSetupState);
+
   host_backend_delegate_->AttemptToSetMultiDeviceHostOnBackend(*it);
 
   return true;
