@@ -4,9 +4,13 @@
 
 #include <memory>
 
+#include "base/bind.h"
+#include "base/callback_helpers.h"
 #include "base/run_loop.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/task_environment.h"
+#include "chromeos/components/multidevice/remote_device_test_util.h"
 #include "chromeos/services/device_sync/public/cpp/fake_device_sync_client.h"
+#include "chromeos/services/device_sync/public/cpp/fake_gcm_device_info_provider.h"
 #include "chromeos/services/multidevice_setup/fake_account_status_change_delegate.h"
 #include "chromeos/services/multidevice_setup/fake_feature_state_observer.h"
 #include "chromeos/services/multidevice_setup/fake_host_status_observer.h"
@@ -17,12 +21,9 @@
 #include "chromeos/services/multidevice_setup/public/cpp/fake_auth_token_validator.h"
 #include "chromeos/services/multidevice_setup/public/cpp/fake_multidevice_setup.h"
 #include "chromeos/services/multidevice_setup/public/cpp/oobe_completion_tracker.h"
-#include "chromeos/services/multidevice_setup/public/mojom/constants.mojom.h"
 #include "chromeos/services/multidevice_setup/public/mojom/multidevice_setup.mojom.h"
-#include "components/cryptauth/fake_gcm_device_info_provider.h"
-#include "components/cryptauth/remote_device_test_util.h"
 #include "components/sync_preferences/testing_pref_service_syncable.h"
-#include "services/service_manager/public/cpp/test/test_connector_factory.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace chromeos {
@@ -44,8 +45,9 @@ class FakeMultiDeviceSetupFactory : public MultiDeviceSetupImpl::Factory {
       FakeAndroidSmsAppHelperDelegate* expected_android_sms_app_helper_delegate,
       FakeAndroidSmsPairingStateTracker*
           expected_android_sms_pairing_state_tracker,
-      const cryptauth::FakeGcmDeviceInfoProvider*
-          expected_gcm_device_info_provider)
+      const device_sync::FakeGcmDeviceInfoProvider*
+          expected_gcm_device_info_provider,
+      bool expected_is_secondary_user)
       : expected_testing_pref_service_(expected_testing_pref_service),
         expected_device_sync_client_(expected_device_sync_client),
         expected_auth_token_validator_(expected_auth_token_validator),
@@ -54,34 +56,34 @@ class FakeMultiDeviceSetupFactory : public MultiDeviceSetupImpl::Factory {
             expected_android_sms_app_helper_delegate),
         expected_android_sms_pairing_state_tracker_(
             expected_android_sms_pairing_state_tracker),
-        expected_gcm_device_info_provider_(expected_gcm_device_info_provider) {}
+        expected_gcm_device_info_provider_(expected_gcm_device_info_provider),
+        expected_is_secondary_user_(expected_is_secondary_user) {}
 
   ~FakeMultiDeviceSetupFactory() override = default;
 
   FakeMultiDeviceSetup* instance() { return instance_; }
 
  private:
-  std::unique_ptr<MultiDeviceSetupBase> BuildInstance(
+  std::unique_ptr<MultiDeviceSetupBase> CreateInstance(
       PrefService* pref_service,
       device_sync::DeviceSyncClient* device_sync_client,
       AuthTokenValidator* auth_token_validator,
       OobeCompletionTracker* oobe_completion_tracker,
-      std::unique_ptr<AndroidSmsAppHelperDelegate>
-          android_sms_app_helper_delegate,
-      std::unique_ptr<AndroidSmsPairingStateTracker>
-          android_sms_pairing_state_tracker,
-      const cryptauth::GcmDeviceInfoProvider* gcm_device_info_provider)
-      override {
+      AndroidSmsAppHelperDelegate* android_sms_app_helper_delegate,
+      AndroidSmsPairingStateTracker* android_sms_pairing_state_tracker,
+      const device_sync::GcmDeviceInfoProvider* gcm_device_info_provider,
+      bool is_secondary_user) override {
     EXPECT_FALSE(instance_);
     EXPECT_EQ(expected_testing_pref_service_, pref_service);
     EXPECT_EQ(expected_device_sync_client_, device_sync_client);
     EXPECT_EQ(expected_auth_token_validator_, auth_token_validator);
     EXPECT_EQ(expected_oobe_completion_tracker_, oobe_completion_tracker);
     EXPECT_EQ(expected_android_sms_app_helper_delegate_,
-              android_sms_app_helper_delegate.get());
+              android_sms_app_helper_delegate);
     EXPECT_EQ(expected_android_sms_pairing_state_tracker_,
-              android_sms_pairing_state_tracker.get());
+              android_sms_pairing_state_tracker);
     EXPECT_EQ(expected_gcm_device_info_provider_, gcm_device_info_provider);
+    EXPECT_EQ(expected_is_secondary_user_, is_secondary_user);
 
     auto instance = std::make_unique<FakeMultiDeviceSetup>();
     instance_ = instance.get();
@@ -95,8 +97,9 @@ class FakeMultiDeviceSetupFactory : public MultiDeviceSetupImpl::Factory {
   FakeAndroidSmsAppHelperDelegate* expected_android_sms_app_helper_delegate_;
   FakeAndroidSmsPairingStateTracker*
       expected_android_sms_pairing_state_tracker_;
-  const cryptauth::FakeGcmDeviceInfoProvider*
+  const device_sync::FakeGcmDeviceInfoProvider*
       expected_gcm_device_info_provider_;
+  bool expected_is_secondary_user_;
 
   FakeMultiDeviceSetup* instance_ = nullptr;
 
@@ -109,7 +112,7 @@ class MultiDeviceSetupServiceTest : public testing::Test {
  protected:
   MultiDeviceSetupServiceTest()
       : test_devices_(
-            cryptauth::CreateRemoteDeviceRefListForTest(kNumTestDevices)) {}
+            multidevice::CreateRemoteDeviceRefListForTest(kNumTestDevices)) {}
   ~MultiDeviceSetupServiceTest() override = default;
 
   // testing::Test:
@@ -120,16 +123,12 @@ class MultiDeviceSetupServiceTest : public testing::Test {
         std::make_unique<device_sync::FakeDeviceSyncClient>();
     fake_auth_token_validator_ = std::make_unique<FakeAuthTokenValidator>();
     fake_oobe_completion_tracker_ = std::make_unique<OobeCompletionTracker>();
-    auto fake_android_sms_app_helper_delegate =
-        std::make_unique<FakeAndroidSmsAppHelperDelegate>();
     fake_android_sms_app_helper_delegate_ =
-        fake_android_sms_app_helper_delegate.get();
-    auto fake_android_sms_pairing_state_tracker =
-        std::make_unique<FakeAndroidSmsPairingStateTracker>();
+        std::make_unique<FakeAndroidSmsAppHelperDelegate>();
     fake_android_sms_pairing_state_tracker_ =
-        fake_android_sms_pairing_state_tracker.get();
+        std::make_unique<FakeAndroidSmsPairingStateTracker>();
     fake_gcm_device_info_provider_ =
-        std::make_unique<cryptauth::FakeGcmDeviceInfoProvider>(
+        std::make_unique<device_sync::FakeGcmDeviceInfoProvider>(
             cryptauth::GcmDeviceInfo());
 
     fake_multidevice_setup_factory_ =
@@ -137,30 +136,23 @@ class MultiDeviceSetupServiceTest : public testing::Test {
             test_pref_service_.get(), fake_device_sync_client_.get(),
             fake_auth_token_validator_.get(),
             fake_oobe_completion_tracker_.get(),
-            fake_android_sms_app_helper_delegate_,
-            fake_android_sms_pairing_state_tracker_,
-            fake_gcm_device_info_provider_.get());
+            fake_android_sms_app_helper_delegate_.get(),
+            fake_android_sms_pairing_state_tracker_.get(),
+            fake_gcm_device_info_provider_.get(), is_secondary_user_);
     MultiDeviceSetupImpl::Factory::SetFactoryForTesting(
         fake_multidevice_setup_factory_.get());
 
-    connector_factory_ =
-        service_manager::TestConnectorFactory::CreateForUniqueService(
-            std::make_unique<MultiDeviceSetupService>(
-                test_pref_service_.get(), fake_device_sync_client_.get(),
-                fake_auth_token_validator_.get(),
-                fake_oobe_completion_tracker_.get(),
-                std::move(fake_android_sms_app_helper_delegate),
-                std::move(fake_android_sms_pairing_state_tracker),
-                fake_gcm_device_info_provider_.get()));
+    service_ = std::make_unique<MultiDeviceSetupService>(
+        test_pref_service_.get(), fake_device_sync_client_.get(),
+        fake_auth_token_validator_.get(), fake_oobe_completion_tracker_.get(),
+        fake_android_sms_app_helper_delegate_.get(),
+        fake_android_sms_pairing_state_tracker_.get(),
+        fake_gcm_device_info_provider_.get(), is_secondary_user_);
 
-    auto connector = connector_factory_->CreateConnector();
-
-    connector->BindInterface(mojom::kServiceName, &multidevice_setup_ptr_);
-    multidevice_setup_ptr_.FlushForTesting();
-
-    connector->BindInterface(mojom::kServiceName,
-                             &privileged_host_device_setter_ptr_);
-    privileged_host_device_setter_ptr_.FlushForTesting();
+    service_->BindMultiDeviceSetup(
+        multidevice_setup_remote_.BindNewPipeAndPassReceiver());
+    service_->BindPrivilegedHostDeviceSetter(
+        privileged_host_device_setter_remote_.BindNewPipeAndPassReceiver());
   }
 
   void TearDown() override {
@@ -172,7 +164,7 @@ class MultiDeviceSetupServiceTest : public testing::Test {
     EXPECT_FALSE(last_debug_event_success_);
 
     base::RunLoop run_loop;
-    multidevice_setup_ptr_->TriggerEventForDebugging(
+    multidevice_setup_remote_->TriggerEventForDebugging(
         type,
         base::BindOnce(&MultiDeviceSetupServiceTest::OnDebugEventTriggered,
                        base::Unretained(this), run_loop.QuitClosure()));
@@ -194,12 +186,13 @@ class MultiDeviceSetupServiceTest : public testing::Test {
     return fake_multidevice_setup_factory_->instance();
   }
 
-  mojom::MultiDeviceSetupPtr& multidevice_setup_ptr() {
-    return multidevice_setup_ptr_;
+  mojo::Remote<mojom::MultiDeviceSetup>& multidevice_setup_remote() {
+    return multidevice_setup_remote_;
   }
 
-  mojom::PrivilegedHostDeviceSetterPtr& privileged_host_device_setter_ptr() {
-    return privileged_host_device_setter_ptr_;
+  mojo::Remote<mojom::PrivilegedHostDeviceSetter>&
+  privileged_host_device_setter_remote() {
+    return privileged_host_device_setter_remote_;
   }
 
  private:
@@ -208,26 +201,30 @@ class MultiDeviceSetupServiceTest : public testing::Test {
     std::move(quit_closure).Run();
   }
 
-  const base::test::ScopedTaskEnvironment scoped_task_environment_;
-  const cryptauth::RemoteDeviceRefList test_devices_;
+  base::test::TaskEnvironment task_environment_;
+  const multidevice::RemoteDeviceRefList test_devices_;
 
   std::unique_ptr<sync_preferences::TestingPrefServiceSyncable>
       test_pref_service_;
   std::unique_ptr<device_sync::FakeDeviceSyncClient> fake_device_sync_client_;
   std::unique_ptr<FakeAuthTokenValidator> fake_auth_token_validator_;
   std::unique_ptr<OobeCompletionTracker> fake_oobe_completion_tracker_;
-  FakeAndroidSmsAppHelperDelegate* fake_android_sms_app_helper_delegate_;
-  FakeAndroidSmsPairingStateTracker* fake_android_sms_pairing_state_tracker_;
-  std::unique_ptr<cryptauth::FakeGcmDeviceInfoProvider>
+  std::unique_ptr<FakeAndroidSmsAppHelperDelegate>
+      fake_android_sms_app_helper_delegate_;
+  std::unique_ptr<FakeAndroidSmsPairingStateTracker>
+      fake_android_sms_pairing_state_tracker_;
+  std::unique_ptr<device_sync::FakeGcmDeviceInfoProvider>
       fake_gcm_device_info_provider_;
+  bool is_secondary_user_ = false;
 
   std::unique_ptr<FakeMultiDeviceSetupFactory> fake_multidevice_setup_factory_;
 
-  std::unique_ptr<service_manager::TestConnectorFactory> connector_factory_;
+  std::unique_ptr<MultiDeviceSetupService> service_;
   base::Optional<bool> last_debug_event_success_;
 
-  mojom::MultiDeviceSetupPtr multidevice_setup_ptr_;
-  mojom::PrivilegedHostDeviceSetterPtr privileged_host_device_setter_ptr_;
+  mojo::Remote<mojom::MultiDeviceSetup> multidevice_setup_remote_;
+  mojo::Remote<mojom::PrivilegedHostDeviceSetter>
+      privileged_host_device_setter_remote_;
 
   DISALLOW_COPY_AND_ASSIGN(MultiDeviceSetupServiceTest);
 };
@@ -246,44 +243,44 @@ TEST_F(MultiDeviceSetupServiceTest, CallFunctionsBeforeInitialization) {
   // SetAccountStatusChangeDelegate().
   auto fake_account_status_change_delegate =
       std::make_unique<FakeAccountStatusChangeDelegate>();
-  multidevice_setup_ptr()->SetAccountStatusChangeDelegate(
-      fake_account_status_change_delegate->GenerateInterfacePtr());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetAccountStatusChangeDelegate(
+      fake_account_status_change_delegate->GenerateRemote());
+  multidevice_setup_remote().FlushForTesting();
 
   // AddHostStatusObserver().
   auto fake_host_status_observer = std::make_unique<FakeHostStatusObserver>();
-  multidevice_setup_ptr()->AddHostStatusObserver(
-      fake_host_status_observer->GenerateInterfacePtr());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->AddHostStatusObserver(
+      fake_host_status_observer->GenerateRemote());
+  multidevice_setup_remote().FlushForTesting();
 
   // AddFeatureStateObserver().
   auto fake_feature_state_observer =
       std::make_unique<FakeFeatureStateObserver>();
-  multidevice_setup_ptr()->AddFeatureStateObserver(
-      fake_feature_state_observer->GenerateInterfacePtr());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->AddFeatureStateObserver(
+      fake_feature_state_observer->GenerateRemote());
+  multidevice_setup_remote().FlushForTesting();
 
   // GetEligibleHostDevices().
-  multidevice_setup_ptr()->GetEligibleHostDevices(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->GetEligibleHostDevices(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
   // GetHostStatus().
-  multidevice_setup_ptr()->GetHostStatus(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->GetHostStatus(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
   // SetFeatureEnabledState().
-  multidevice_setup_ptr()->SetFeatureEnabledState(
+  multidevice_setup_remote()->SetFeatureEnabledState(
       mojom::Feature::kBetterTogetherSuite, true /* enabled */, "authToken",
       base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote().FlushForTesting();
 
   // GetFeatureStates().
-  multidevice_setup_ptr()->GetFeatureStates(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->GetFeatureStates(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
   // RetrySetHostNow().
-  multidevice_setup_ptr()->RetrySetHostNow(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->RetrySetHostNow(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
   // None of these requests should have been processed yet, since initialization
   // was not complete.
@@ -302,16 +299,16 @@ TEST_F(MultiDeviceSetupServiceTest, CallFunctionsBeforeInitialization) {
 }
 
 TEST_F(MultiDeviceSetupServiceTest, SetThenRemoveBeforeInitialization) {
-  multidevice_setup_ptr()->SetHostDevice("deviceId1", "authToken",
-                                         base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetHostDevice("id1", "authToken",
+                                            base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
-  privileged_host_device_setter_ptr()->SetHostDevice("deviceId2",
-                                                     base::DoNothing());
-  privileged_host_device_setter_ptr().FlushForTesting();
+  privileged_host_device_setter_remote()->SetHostDevice("id2",
+                                                        base::DoNothing());
+  privileged_host_device_setter_remote().FlushForTesting();
 
-  multidevice_setup_ptr()->RemoveHostDevice();
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->RemoveHostDevice();
+  multidevice_setup_remote().FlushForTesting();
 
   EXPECT_FALSE(fake_multidevice_setup());
 
@@ -325,20 +322,20 @@ TEST_F(MultiDeviceSetupServiceTest, SetThenRemoveBeforeInitialization) {
 }
 
 TEST_F(MultiDeviceSetupServiceTest, RemoveThenSetThenSetBeforeInitialization) {
-  multidevice_setup_ptr()->RemoveHostDevice();
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->RemoveHostDevice();
+  multidevice_setup_remote().FlushForTesting();
 
-  privileged_host_device_setter_ptr()->SetHostDevice("deviceId1",
-                                                     base::DoNothing());
-  privileged_host_device_setter_ptr().FlushForTesting();
+  privileged_host_device_setter_remote()->SetHostDevice("id1",
+                                                        base::DoNothing());
+  privileged_host_device_setter_remote().FlushForTesting();
 
-  multidevice_setup_ptr()->SetHostDevice("deviceId2", "authToken2",
-                                         base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetHostDevice("id2", "authToken2",
+                                            base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
-  multidevice_setup_ptr()->SetHostDevice("deviceId3", "authToken3",
-                                         base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetHostDevice("id3", "authToken3",
+                                            base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
   EXPECT_FALSE(fake_multidevice_setup());
 
@@ -348,28 +345,27 @@ TEST_F(MultiDeviceSetupServiceTest, RemoveThenSetThenSetBeforeInitialization) {
   EXPECT_EQ(0u, fake_multidevice_setup()->num_remove_host_calls());
   EXPECT_TRUE(fake_multidevice_setup()->set_host_without_auth_args().empty());
   EXPECT_EQ(1u, fake_multidevice_setup()->set_host_args().size());
-  EXPECT_EQ("deviceId3",
-            std::get<0>(fake_multidevice_setup()->set_host_args()[0]));
+  EXPECT_EQ("id3", std::get<0>(fake_multidevice_setup()->set_host_args()[0]));
   EXPECT_EQ("authToken3",
             std::get<1>(fake_multidevice_setup()->set_host_args()[0]));
 }
 
 TEST_F(MultiDeviceSetupServiceTest,
        RemoveThenSetThenSetBeforeInitialization_NoAuthToken) {
-  multidevice_setup_ptr()->RemoveHostDevice();
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->RemoveHostDevice();
+  multidevice_setup_remote().FlushForTesting();
 
-  multidevice_setup_ptr()->SetHostDevice("deviceId1", "authToken1",
-                                         base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetHostDevice("id1", "authToken1",
+                                            base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
-  multidevice_setup_ptr()->SetHostDevice("deviceId2", "authToken2",
-                                         base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetHostDevice("id2", "authToken2",
+                                            base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
 
-  privileged_host_device_setter_ptr()->SetHostDevice("deviceId3",
-                                                     base::DoNothing());
-  privileged_host_device_setter_ptr().FlushForTesting();
+  privileged_host_device_setter_remote()->SetHostDevice("id3",
+                                                        base::DoNothing());
+  privileged_host_device_setter_remote().FlushForTesting();
 
   EXPECT_FALSE(fake_multidevice_setup());
 
@@ -379,7 +375,7 @@ TEST_F(MultiDeviceSetupServiceTest,
   EXPECT_EQ(0u, fake_multidevice_setup()->num_remove_host_calls());
   EXPECT_TRUE(fake_multidevice_setup()->set_host_args().empty());
   EXPECT_EQ(1u, fake_multidevice_setup()->set_host_without_auth_args().size());
-  EXPECT_EQ("deviceId3",
+  EXPECT_EQ("id3",
             fake_multidevice_setup()->set_host_without_auth_args()[0].first);
 }
 
@@ -391,68 +387,68 @@ TEST_F(MultiDeviceSetupServiceTest, FinishInitializationFirst) {
   // SetAccountStatusChangeDelegate().
   auto fake_account_status_change_delegate =
       std::make_unique<FakeAccountStatusChangeDelegate>();
-  multidevice_setup_ptr()->SetAccountStatusChangeDelegate(
-      fake_account_status_change_delegate->GenerateInterfacePtr());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetAccountStatusChangeDelegate(
+      fake_account_status_change_delegate->GenerateRemote());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_TRUE(fake_multidevice_setup()->delegate());
 
   // AddHostStatusObserver().
   auto fake_host_status_observer = std::make_unique<FakeHostStatusObserver>();
-  multidevice_setup_ptr()->AddHostStatusObserver(
-      fake_host_status_observer->GenerateInterfacePtr());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->AddHostStatusObserver(
+      fake_host_status_observer->GenerateRemote());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_TRUE(fake_multidevice_setup()->HasAtLeastOneHostStatusObserver());
 
   // AddFeatureStateObserver().
   auto fake_feature_state_observer =
       std::make_unique<FakeFeatureStateObserver>();
-  multidevice_setup_ptr()->AddFeatureStateObserver(
-      fake_feature_state_observer->GenerateInterfacePtr());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->AddFeatureStateObserver(
+      fake_feature_state_observer->GenerateRemote());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_TRUE(fake_multidevice_setup()->HasAtLeastOneFeatureStateObserver());
 
   // GetEligibleHostDevices().
-  multidevice_setup_ptr()->GetEligibleHostDevices(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->GetEligibleHostDevices(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->get_eligible_hosts_args().size());
 
   // SetHostDevice().
-  multidevice_setup_ptr()->SetHostDevice("deviceId", "authToken",
-                                         base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->SetHostDevice("id", "authToken",
+                                            base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->set_host_args().size());
 
   // RemoveHostDevice().
-  multidevice_setup_ptr()->RemoveHostDevice();
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->RemoveHostDevice();
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->num_remove_host_calls());
 
   // GetHostStatus().
-  multidevice_setup_ptr()->GetHostStatus(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->GetHostStatus(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->get_host_args().size());
 
   // SetFeatureEnabledState().
-  multidevice_setup_ptr()->SetFeatureEnabledState(
+  multidevice_setup_remote()->SetFeatureEnabledState(
       mojom::Feature::kBetterTogetherSuite, true /* enabled */, "authToken",
       base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->set_feature_enabled_args().size());
 
   // GetFeatureStates().
-  multidevice_setup_ptr()->GetFeatureStates(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->GetFeatureStates(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->get_feature_states_args().size());
 
   // RetrySetHostNow().
-  multidevice_setup_ptr()->RetrySetHostNow(base::DoNothing());
-  multidevice_setup_ptr().FlushForTesting();
+  multidevice_setup_remote()->RetrySetHostNow(base::DoNothing());
+  multidevice_setup_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->retry_set_host_now_args().size());
 
   // SetHostDevice(), without an auth token.
-  privileged_host_device_setter_ptr()->SetHostDevice("deviceId",
-                                                     base::DoNothing());
-  privileged_host_device_setter_ptr().FlushForTesting();
+  privileged_host_device_setter_remote()->SetHostDevice("id",
+                                                        base::DoNothing());
+  privileged_host_device_setter_remote().FlushForTesting();
   EXPECT_EQ(1u, fake_multidevice_setup()->set_host_without_auth_args().size());
 }
 
