@@ -49,17 +49,18 @@ void FakeMultiDeviceSetupClient::SetFeatureState(
 }
 
 void FakeMultiDeviceSetupClient::InvokePendingGetEligibleHostDevicesCallback(
-    const cryptauth::RemoteDeviceRefList& eligible_devices) {
+    const multidevice::RemoteDeviceRefList& eligible_devices) {
   std::move(get_eligible_host_devices_callback_queue_.front())
       .Run(eligible_devices);
   get_eligible_host_devices_callback_queue_.pop();
 }
 
 void FakeMultiDeviceSetupClient::InvokePendingSetHostDeviceCallback(
-    const std::string& expected_device_id,
+    const std::string& expected_instance_id_or_legacy_device_id,
     const std::string& expected_auth_token,
     bool success) {
-  DCHECK_EQ(expected_device_id, std::get<0>(set_host_args_queue_.front()));
+  DCHECK_EQ(expected_instance_id_or_legacy_device_id,
+            std::get<0>(set_host_args_queue_.front()));
   DCHECK_EQ(expected_auth_token, std::get<1>(set_host_args_queue_.front()));
   std::move(std::get<2>(set_host_args_queue_.front())).Run(success);
   set_host_args_queue_.pop();
@@ -94,16 +95,22 @@ void FakeMultiDeviceSetupClient::InvokePendingTriggerEventForDebuggingCallback(
   trigger_event_for_debugging_type_and_callback_queue_.pop();
 }
 
+size_t FakeMultiDeviceSetupClient::NumPendingSetFeatureEnabledStateCalls()
+    const {
+  return set_feature_enabled_state_args_queue_.size();
+}
+
 void FakeMultiDeviceSetupClient::GetEligibleHostDevices(
     GetEligibleHostDevicesCallback callback) {
   get_eligible_host_devices_callback_queue_.push(std::move(callback));
 }
 
 void FakeMultiDeviceSetupClient::SetHostDevice(
-    const std::string& host_device_id,
+    const std::string& host_instance_id_or_legacy_device_id,
     const std::string& auth_token,
     mojom::MultiDeviceSetup::SetHostDeviceCallback callback) {
-  set_host_args_queue_.emplace(host_device_id, auth_token, std::move(callback));
+  set_host_args_queue_.emplace(host_instance_id_or_legacy_device_id, auth_token,
+                               std::move(callback));
 }
 
 void FakeMultiDeviceSetupClient::RemoveHostDevice() {
