@@ -13,9 +13,9 @@
 #include "base/callback.h"
 #include "base/macros.h"
 #include "base/optional.h"
+#include "chromeos/components/multidevice/remote_device_ref.h"
 #include "chromeos/services/multidevice_setup/public/cpp/multidevice_setup_client.h"
 #include "chromeos/services/multidevice_setup/public/mojom/multidevice_setup.mojom.h"
-#include "components/cryptauth/remote_device_ref.h"
 
 namespace chromeos {
 
@@ -34,9 +34,9 @@ class FakeMultiDeviceSetupClient : public MultiDeviceSetupClient {
                        mojom::FeatureState feature_state);
 
   void InvokePendingGetEligibleHostDevicesCallback(
-      const cryptauth::RemoteDeviceRefList& eligible_devices);
+      const multidevice::RemoteDeviceRefList& eligible_devices);
   void InvokePendingSetHostDeviceCallback(
-      const std::string& expected_device_id,
+      const std::string& expected_instance_id_or_legacy_device_id,
       const std::string& expected_auth_token,
       bool success);
   void InvokePendingSetFeatureEnabledStateCallback(
@@ -49,6 +49,8 @@ class FakeMultiDeviceSetupClient : public MultiDeviceSetupClient {
       mojom::EventTypeForDebugging expected_type,
       bool success);
 
+  size_t NumPendingSetFeatureEnabledStateCalls() const;
+
   size_t num_remove_host_device_called() {
     return num_remove_host_device_called_;
   }
@@ -61,7 +63,7 @@ class FakeMultiDeviceSetupClient : public MultiDeviceSetupClient {
   // MultiDeviceSetupClient:
   void GetEligibleHostDevices(GetEligibleHostDevicesCallback callback) override;
   void SetHostDevice(
-      const std::string& host_device_id,
+      const std::string& host_instance_id_or_legacy_device_id,
       const std::string& auth_token,
       mojom::MultiDeviceSetup::SetHostDeviceCallback callback) override;
   void RemoveHostDevice() override;
