@@ -13,8 +13,8 @@
 #include "base/macros.h"
 #include "base/observer_list.h"
 #include "base/optional.h"
+#include "chromeos/components/multidevice/remote_device_ref.h"
 #include "chromeos/services/multidevice_setup/public/mojom/multidevice_setup.mojom.h"
-#include "components/cryptauth/remote_device_ref.h"
 
 namespace chromeos {
 
@@ -24,7 +24,8 @@ namespace multidevice_setup {
 class MultiDeviceSetupClient {
  public:
   using HostStatusWithDevice =
-      std::pair<mojom::HostStatus, base::Optional<cryptauth::RemoteDeviceRef>>;
+      std::pair<mojom::HostStatus,
+                base::Optional<multidevice::RemoteDeviceRef>>;
   using FeatureStatesMap = base::flat_map<mojom::Feature, mojom::FeatureState>;
 
   class Observer {
@@ -45,7 +46,10 @@ class MultiDeviceSetupClient {
   };
 
   using GetEligibleHostDevicesCallback =
-      base::OnceCallback<void(const cryptauth::RemoteDeviceRefList&)>;
+      base::OnceCallback<void(const multidevice::RemoteDeviceRefList&)>;
+
+  static HostStatusWithDevice GenerateDefaultHostStatusWithDevice();
+  static FeatureStatesMap GenerateDefaultFeatureStatesMap();
 
   MultiDeviceSetupClient();
   virtual ~MultiDeviceSetupClient();
@@ -56,7 +60,7 @@ class MultiDeviceSetupClient {
   virtual void GetEligibleHostDevices(
       GetEligibleHostDevicesCallback callback) = 0;
   virtual void SetHostDevice(
-      const std::string& host_device_id,
+      const std::string& host_instance_id_or_legacy_device_id,
       const std::string& auth_token,
       mojom::MultiDeviceSetup::SetHostDeviceCallback callback) = 0;
   virtual void RemoveHostDevice() = 0;
@@ -75,9 +79,6 @@ class MultiDeviceSetupClient {
       mojom::MultiDeviceSetup::TriggerEventForDebuggingCallback callback) = 0;
 
  protected:
-  static HostStatusWithDevice GenerateDefaultHostStatusWithDevice();
-  static FeatureStatesMap GenerateDefaultFeatureStatesMap();
-
   void NotifyHostStatusChanged(
       const HostStatusWithDevice& host_status_with_device);
   void NotifyFeatureStateChanged(const FeatureStatesMap& feature_states_map);
