@@ -7,11 +7,10 @@
 
 #include "chromeos/services/multidevice_setup/public/cpp/multidevice_setup_client_impl.h"
 
-#include "base/no_destructor.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
-#include "chromeos/services/multidevice_setup/public/mojom/constants.mojom.h"
+#include "base/bind.h"
+#include "base/memory/ptr_util.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 #include "chromeos/services/multidevice_setup/public/mojom/multidevice_setup.mojom.h"
-#include "services/service_manager/public/cpp/connector.h"
 
 namespace chromeos {
 
@@ -22,46 +21,38 @@ MultiDeviceSetupClientImpl::Factory*
     MultiDeviceSetupClientImpl::Factory::test_factory_ = nullptr;
 
 // static
-MultiDeviceSetupClientImpl::Factory*
-MultiDeviceSetupClientImpl::Factory::Get() {
+std::unique_ptr<MultiDeviceSetupClient>
+MultiDeviceSetupClientImpl::Factory::Create(
+    mojo::PendingRemote<mojom::MultiDeviceSetup> remote_setup) {
   if (test_factory_)
-    return test_factory_;
+    return test_factory_->CreateInstance(std::move(remote_setup));
 
-  static base::NoDestructor<Factory> factory;
-  return factory.get();
+  return base::WrapUnique(
+      new MultiDeviceSetupClientImpl(std::move(remote_setup)));
 }
 
 // static
-void MultiDeviceSetupClientImpl::Factory::SetInstanceForTesting(
+void MultiDeviceSetupClientImpl::Factory::SetFactoryForTesting(
     Factory* test_factory) {
   test_factory_ = test_factory;
 }
 
 MultiDeviceSetupClientImpl::Factory::~Factory() = default;
 
-std::unique_ptr<MultiDeviceSetupClient>
-MultiDeviceSetupClientImpl::Factory::BuildInstance(
-    service_manager::Connector* connector) {
-  return base::WrapUnique(new MultiDeviceSetupClientImpl(connector));
-}
-
 MultiDeviceSetupClientImpl::MultiDeviceSetupClientImpl(
-    service_manager::Connector* connector)
-    : host_status_observer_binding_(this),
-      feature_state_observer_binding_(this),
-      remote_device_cache_(
-          cryptauth::RemoteDeviceCache::Factory::Get()->BuildInstance()),
+    mojo::PendingRemote<mojom::MultiDeviceSetup> remote_setup)
+    : multidevice_setup_remote_(std::move(remote_setup)),
+      remote_device_cache_(multidevice::RemoteDeviceCache::Factory::Create()),
       host_status_with_device_(GenerateDefaultHostStatusWithDevice()),
       feature_states_map_(GenerateDefaultFeatureStatesMap()) {
-  connector->BindInterface(mojom::kServiceName, &multidevice_setup_ptr_);
-  multidevice_setup_ptr_->AddHostStatusObserver(
-      GenerateHostStatusObserverInterfacePtr());
-  multidevice_setup_ptr_->AddFeatureStateObserver(
-      GenerateFeatureStatesObserverInterfacePtr());
-  multidevice_setup_ptr_->GetHostStatus(
+  multidevice_setup_remote_->AddHostStatusObserver(
+      GenerateHostStatusObserverRemote());
+  multidevice_setup_remote_->AddFeatureStateObserver(
+      GenerateFeatureStatesObserverRemote());
+  multidevice_setup_remote_->GetHostStatus(
       base::BindOnce(&MultiDeviceSetupClientImpl::OnHostStatusChanged,
                      base::Unretained(this)));
-  multidevice_setup_ptr_->GetFeatureStates(
+  multidevice_setup_remote_->GetFeatureStates(
       base::BindOnce(&MultiDeviceSetupClientImpl::OnFeatureStatesChanged,
                      base::Unretained(this)));
 }
@@ -70,21 +61,21 @@ MultiDeviceSetupClientImpl::~MultiDeviceSetupClientImpl() = default;
 
 void MultiDeviceSetupClientImpl::GetEligibleHostDevices(
     GetEligibleHostDevicesCallback callback) {
-  multidevice_setup_ptr_->GetEligibleHostDevices(base::BindOnce(
+  multidevice_setup_remote_->GetEligibleHostDevices(base::BindOnce(
       &MultiDeviceSetupClientImpl::OnGetEligibleHostDevicesCompleted,
       base::Unretained(this), std::move(callback)));
 }
 
 void MultiDeviceSetupClientImpl::SetHostDevice(
-    const std::string& host_device_id,
+    const std::string& host_instance_id_or_legacy_device_id,
     const std::string& auth_token,
     mojom::MultiDeviceSetup::SetHostDeviceCallback callback) {
-  multidevice_setup_ptr_->SetHostDevice(host_device_id, auth_token,
-                                        std::move(callback));
+  multidevice_setup_remote_->SetHostDevice(host_instance_id_or_legacy_device_id,
+                                           auth_token, std::move(callback));
 }
 
 void MultiDeviceSetupClientImpl::RemoveHostDevice() {
-  multidevice_setup_ptr_->RemoveHostDevice();
+  multidevice_setup_remote_->RemoveHostDevice();
 }
 
 const MultiDeviceSetupClient::HostStatusWithDevice&
@@ -97,8 +88,8 @@ void MultiDeviceSetupClientImpl::SetFeatureEnabledState(
     bool enabled,
     const base::Optional<std::string>& auth_token,
     mojom::MultiDeviceSetup::SetFeatureEnabledStateCallback callback) {
-  multidevice_setup_ptr_->SetFeatureEnabledState(feature, enabled, auth_token,
-                                                 std::move(callback));
+  multidevice_setup_remote_->SetFeatureEnabledState(
+      feature, enabled, auth_token, std::move(callback));
 }
 
 const MultiDeviceSetupClient::FeatureStatesMap&
@@ -108,23 +99,24 @@ MultiDeviceSetupClientImpl::GetFeatureStates() const {
 
 void MultiDeviceSetupClientImpl::RetrySetHostNow(
     mojom::MultiDeviceSetup::RetrySetHostNowCallback callback) {
-  multidevice_setup_ptr_->RetrySetHostNow(std::move(callback));
+  multidevice_setup_remote_->RetrySetHostNow(std::move(callback));
 }
 
 void MultiDeviceSetupClientImpl::TriggerEventForDebugging(
     mojom::EventTypeForDebugging type,
     mojom::MultiDeviceSetup::TriggerEventForDebuggingCallback callback) {
-  multidevice_setup_ptr_->TriggerEventForDebugging(type, std::move(callback));
+  multidevice_setup_remote_->TriggerEventForDebugging(type,
+                                                      std::move(callback));
 }
 
 void MultiDeviceSetupClientImpl::OnHostStatusChanged(
     mojom::HostStatus host_status,
-    const base::Optional<cryptauth::RemoteDevice>& host_device) {
+    const base::Optional<multidevice::RemoteDevice>& host_device) {
   if (host_device) {
     remote_device_cache_->SetRemoteDevices({*host_device});
     host_status_with_device_ = std::make_pair(
-        host_status,
-        remote_device_cache_->GetRemoteDevice(host_device->GetDeviceId()));
+        host_status, remote_device_cache_->GetRemoteDevice(
+                         host_device->instance_id, host_device->GetDeviceId()));
   } else {
     host_status_with_device_ =
         std::make_pair(host_status, base::nullopt /* host_device */);
@@ -141,36 +133,32 @@ void MultiDeviceSetupClientImpl::OnFeatureStatesChanged(
 
 void MultiDeviceSetupClientImpl::OnGetEligibleHostDevicesCompleted(
     GetEligibleHostDevicesCallback callback,
-    const cryptauth::RemoteDeviceList& eligible_host_devices) {
+    const multidevice::RemoteDeviceList& eligible_host_devices) {
   remote_device_cache_->SetRemoteDevices(eligible_host_devices);
 
-  cryptauth::RemoteDeviceRefList eligible_host_device_refs;
-  std::transform(
-      eligible_host_devices.begin(), eligible_host_devices.end(),
-      std::back_inserter(eligible_host_device_refs),
-      [this](const auto& device) {
-        return *remote_device_cache_->GetRemoteDevice(device.GetDeviceId());
-      });
+  multidevice::RemoteDeviceRefList eligible_host_device_refs;
+  std::transform(eligible_host_devices.begin(), eligible_host_devices.end(),
+                 std::back_inserter(eligible_host_device_refs),
+                 [this](const auto& device) {
+                   return *remote_device_cache_->GetRemoteDevice(
+                       device.instance_id, device.GetDeviceId());
+                 });
 
   std::move(callback).Run(eligible_host_device_refs);
 }
 
-mojom::HostStatusObserverPtr
-MultiDeviceSetupClientImpl::GenerateHostStatusObserverInterfacePtr() {
-  mojom::HostStatusObserverPtr interface_ptr;
-  host_status_observer_binding_.Bind(mojo::MakeRequest(&interface_ptr));
-  return interface_ptr;
+mojo::PendingRemote<mojom::HostStatusObserver>
+MultiDeviceSetupClientImpl::GenerateHostStatusObserverRemote() {
+  return host_status_observer_receiver_.BindNewPipeAndPassRemote();
 }
 
-mojom::FeatureStateObserverPtr
-MultiDeviceSetupClientImpl::GenerateFeatureStatesObserverInterfacePtr() {
-  mojom::FeatureStateObserverPtr interface_ptr;
-  feature_state_observer_binding_.Bind(mojo::MakeRequest(&interface_ptr));
-  return interface_ptr;
+mojo::PendingRemote<mojom::FeatureStateObserver>
+MultiDeviceSetupClientImpl::GenerateFeatureStatesObserverRemote() {
+  return feature_state_observer_receiver_.BindNewPipeAndPassRemote();
 }
 
 void MultiDeviceSetupClientImpl::FlushForTesting() {
-  multidevice_setup_ptr_.FlushForTesting();
+  multidevice_setup_remote_.FlushForTesting();
 }
 
 }  // namespace multidevice_setup
