@@ -4,7 +4,7 @@
 
 #include "chromeos/services/secure_channel/ble_listener_failure_type.h"
 
-#include "base/logging.h"
+#include "base/check_op.h"
 
 namespace chromeos {
 
