@@ -8,20 +8,20 @@
 
 #include "base/bind.h"
 #include "base/callback_forward.h"
+#include "base/callback_helpers.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/weak_ptr.h"
 #include "base/test/metrics/histogram_tester.h"
-#include "base/test/scoped_task_environment.h"
 #include "base/test/simple_test_clock.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/timer/mock_timer.h"
 #include "chromeos/services/secure_channel/ble_constants.h"
-#include "components/cryptauth/data_with_timestamp.h"
+#include "chromeos/services/secure_channel/data_with_timestamp.h"
 #include "device/bluetooth/bluetooth_advertisement.h"
 #include "device/bluetooth/test/mock_bluetooth_adapter.h"
 #include "device/bluetooth/test/mock_bluetooth_advertisement.h"
-#include "device/bluetooth/test/mock_bluetooth_discovery_session.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using testing::Invoke;
@@ -43,46 +43,48 @@ int64_t kTimeBetweenEachCommandMs = 200;
 struct RegisterAdvertisementArgs {
   RegisterAdvertisementArgs(
       const device::BluetoothAdvertisement::UUIDList& service_uuids,
-      const device::BluetoothAdapter::CreateAdvertisementCallback& callback,
-      const device::BluetoothAdapter::AdvertisementErrorCallback&
-          error_callback)
+      device::BluetoothAdapter::CreateAdvertisementCallback callback,
+      device::BluetoothAdapter::AdvertisementErrorCallback error_callback)
       : service_uuids(service_uuids),
-        callback(callback),
-        error_callback(error_callback) {}
+        callback(std::move(callback)),
+        error_callback(std::move(error_callback)) {}
 
   device::BluetoothAdvertisement::UUIDList service_uuids;
-  const device::BluetoothAdapter::CreateAdvertisementCallback callback;
-  const device::BluetoothAdapter::AdvertisementErrorCallback error_callback;
+  device::BluetoothAdapter::CreateAdvertisementCallback callback;
+  device::BluetoothAdapter::AdvertisementErrorCallback error_callback;
 };
 
 struct UnregisterAdvertisementArgs {
   UnregisterAdvertisementArgs(
-      const device::BluetoothAdvertisement::SuccessCallback& callback,
-      const device::BluetoothAdvertisement::ErrorCallback& error_callback)
-      : callback(callback), error_callback(error_callback) {}
+      device::BluetoothAdvertisement::SuccessCallback callback,
+      device::BluetoothAdvertisement::ErrorCallback error_callback)
+      : callback(std::move(callback)),
+        error_callback(std::move(error_callback)) {}
 
-  const device::BluetoothAdvertisement::SuccessCallback callback;
-  const device::BluetoothAdvertisement::ErrorCallback error_callback;
+  device::BluetoothAdvertisement::SuccessCallback callback;
+  device::BluetoothAdvertisement::ErrorCallback error_callback;
 };
 
 struct StartDiscoverySessionArgs {
   StartDiscoverySessionArgs(
-      const device::BluetoothAdapter::DiscoverySessionCallback& callback,
-      const device::BluetoothAdapter::ErrorCallback& error_callback)
-      : callback(callback), error_callback(error_callback) {}
+      base::OnceClosure callback,
+      device::BluetoothAdapter::ErrorCallback error_callback)
+      : callback(std::move(callback)),
+        error_callback(std::move(error_callback)) {}
 
-  const device::BluetoothAdapter::DiscoverySessionCallback callback;
-  const device::BluetoothAdapter::ErrorCallback error_callback;
+  base::OnceClosure callback;
+  device::BluetoothAdapter::ErrorCallback error_callback;
 };
 
 struct StopDiscoverySessionArgs {
   StopDiscoverySessionArgs(
-      const base::Closure& callback,
-      const device::BluetoothDiscoverySession::ErrorCallback& error_callback)
-      : callback(callback), error_callback(error_callback) {}
+      base::OnceClosure callback,
+      device::BluetoothDiscoverySession::ErrorCallback error_callback)
+      : callback(std::move(callback)),
+        error_callback(std::move(error_callback)) {}
 
-  const base::Closure callback;
-  const device::BluetoothDiscoverySession::ErrorCallback error_callback;
+  base::OnceClosure callback;
+  device::BluetoothDiscoverySession::ErrorCallback error_callback;
 };
 
 class MockBluetoothAdapterWithAdvertisements
@@ -92,26 +94,15 @@ class MockBluetoothAdapterWithAdvertisements
 
   MOCK_METHOD1(RegisterAdvertisementWithArgsStruct,
                void(RegisterAdvertisementArgs*));
-  MOCK_METHOD3(StartDiscoverySessionWithFilterRaw,
-               void(device::BluetoothDiscoveryFilter*,
-                    const device::BluetoothAdapter::DiscoverySessionCallback&,
-                    const device::BluetoothAdapter::ErrorCallback&));
 
   void RegisterAdvertisement(
       std::unique_ptr<device::BluetoothAdvertisement::Data> advertisement_data,
-      const device::BluetoothAdapter::CreateAdvertisementCallback& callback,
-      const device::BluetoothAdapter::AdvertisementErrorCallback&
-          error_callback) override {
+      device::BluetoothAdapter::CreateAdvertisementCallback callback,
+      device::BluetoothAdapter::AdvertisementErrorCallback error_callback)
+      override {
     RegisterAdvertisementWithArgsStruct(new RegisterAdvertisementArgs(
-        *advertisement_data->service_uuids(), callback, error_callback));
-  }
-
-  void StartDiscoverySessionWithFilter(
-      std::unique_ptr<device::BluetoothDiscoveryFilter> discovery_filter,
-      const device::BluetoothAdapter::DiscoverySessionCallback& callback,
-      const device::BluetoothAdapter::ErrorCallback& error_callback) override {
-    StartDiscoverySessionWithFilterRaw(discovery_filter.get(), callback,
-                                       error_callback);
+        *advertisement_data->service_uuids(), std::move(callback),
+        std::move(error_callback)));
   }
 
  protected:
@@ -123,57 +114,29 @@ class FakeBluetoothAdvertisement : public device::BluetoothAdvertisement {
   // |unregister_callback| should be called with the callbacks passed to
   // Unregister() whenever an Unregister() call occurs.
   FakeBluetoothAdvertisement(
-      const base::Callback<
-          void(const device::BluetoothAdvertisement::SuccessCallback&,
-               const device::BluetoothAdvertisement::ErrorCallback&)>&
-          unregister_callback)
+      const base::RepeatingCallback<void(
+          device::BluetoothAdvertisement::SuccessCallback,
+          device::BluetoothAdvertisement::ErrorCallback)>& unregister_callback)
       : unregister_callback_(unregister_callback) {}
 
   // BluetoothAdvertisement:
   void Unregister(
-      const device::BluetoothAdvertisement::SuccessCallback& success_callback,
-      const device::BluetoothAdvertisement::ErrorCallback& error_callback)
-      override {
-    unregister_callback_.Run(success_callback, error_callback);
+      device::BluetoothAdvertisement::SuccessCallback success_callback,
+      device::BluetoothAdvertisement::ErrorCallback error_callback) override {
+    unregister_callback_.Run(std::move(success_callback),
+                             std::move(error_callback));
   }
 
  private:
   ~FakeBluetoothAdvertisement() override = default;
 
-  base::Callback<void(const device::BluetoothAdvertisement::SuccessCallback&,
-                      const device::BluetoothAdvertisement::ErrorCallback&)>
+  base::RepeatingCallback<void(device::BluetoothAdvertisement::SuccessCallback,
+                               device::BluetoothAdvertisement::ErrorCallback)>
       unregister_callback_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeBluetoothAdvertisement);
 };
 
-class FakeDiscoverySession : public device::MockBluetoothDiscoverySession {
- public:
-  // |stop_callback| should be called with the callbacks passed to
-  // Stop() whenever a Stop() call occurs.
-  FakeDiscoverySession(
-      const base::Callback<
-          void(const base::Closure&,
-               const device::BluetoothDiscoverySession::ErrorCallback&)>&
-          stop_callback)
-      : stop_callback_(stop_callback) {}
-  ~FakeDiscoverySession() override = default;
-
-  // BluetoothDiscoverySession:
-  void Stop(const base::Closure& callback,
-            const device::BluetoothDiscoverySession::ErrorCallback&
-                error_callback) override {
-    stop_callback_.Run(callback, error_callback);
-  }
-
- private:
-  base::Callback<void(const base::Closure&,
-                      const device::BluetoothDiscoverySession::ErrorCallback&)>
-      stop_callback_;
-
-  DISALLOW_COPY_AND_ASSIGN(FakeDiscoverySession);
-};
-
 // Creates a UUIDList with one element of value |id|.
 std::unique_ptr<device::BluetoothAdvertisement::UUIDList> CreateUUIDList(
     const std::string& id) {
@@ -196,13 +159,9 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
  protected:
   SecureChannelBleSynchronizerTest()
       : fake_advertisement_(base::MakeRefCounted<FakeBluetoothAdvertisement>(
-            base::Bind(&SecureChannelBleSynchronizerTest::OnUnregisterCalled,
-                       base::Unretained(this)))),
-        fake_discovery_session_(base::WrapUnique(new FakeDiscoverySession(
-            base::Bind(&SecureChannelBleSynchronizerTest::OnStopCalled,
-                       base::Unretained(this))))),
-        fake_discovery_session_weak_ptr_factory_(
-            fake_discovery_session_.get()) {}
+            base::BindRepeating(
+                &SecureChannelBleSynchronizerTest::OnUnregisterCalled,
+                base::Unretained(this)))) {}
 
   void SetUp() override {
     num_register_success_ = 0;
@@ -221,18 +180,19 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
         .WillByDefault(Invoke(
             this,
             &SecureChannelBleSynchronizerTest::OnAdapterRegisterAdvertisement));
-    ON_CALL(*mock_adapter_, StartDiscoverySessionWithFilterRaw(_, _, _))
+    ON_CALL(*mock_adapter_, StartScanWithFilter_(_, _))
         .WillByDefault(Invoke(
-            this,
-            &SecureChannelBleSynchronizerTest::OnAdapterStartDiscoverySession));
+            this, &SecureChannelBleSynchronizerTest::OnAdapterStartScan));
+    ON_CALL(*mock_adapter_, StopScan(_))
+        .WillByDefault(
+            Invoke(this, &SecureChannelBleSynchronizerTest::OnStopScan));
 
     mock_timer_ = new base::MockOneShotTimer();
 
     test_clock_.Advance(TimeDeltaMillis(kTimeBetweenEachCommandMs));
     test_task_runner_ = base::MakeRefCounted<base::TestSimpleTaskRunner>();
 
-    synchronizer_ =
-        BleSynchronizer::Factory::Get()->BuildInstance(mock_adapter_);
+    synchronizer_ = BleSynchronizer::Factory::Create(mock_adapter_);
 
     BleSynchronizer* derived_type =
         static_cast<BleSynchronizer*>(synchronizer_.get());
@@ -248,22 +208,30 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
     register_args_list_.emplace_back(base::WrapUnique(args));
   }
 
-  void OnAdapterStartDiscoverySession(
-      device::BluetoothDiscoveryFilter* discovery_filter,
-      const device::BluetoothAdapter::DiscoverySessionCallback& callback,
-      const device::BluetoothAdapter::ErrorCallback& error_callback) {
+  void OnAdapterStartScan(
+      const device::BluetoothDiscoveryFilter* discovery_filter,
+      device::BluetoothAdapter::DiscoverySessionResultCallback& callback) {
     EXPECT_EQ(device::BluetoothTransport::BLUETOOTH_TRANSPORT_LE,
               discovery_filter->GetTransport());
-    start_discovery_args_list_.emplace_back(base::WrapUnique(
-        new StartDiscoverySessionArgs(callback, error_callback)));
+    auto copyable_callback =
+        base::AdaptCallbackForRepeating(std::move(callback));
+    start_discovery_args_list_.emplace_back(
+        base::WrapUnique(new StartDiscoverySessionArgs(
+            base::BindRepeating(
+                copyable_callback, /*is_error=*/false,
+                device::UMABluetoothDiscoverySessionOutcome::SUCCESS),
+            base::BindRepeating(
+                copyable_callback, /*is_error=*/true,
+                device::UMABluetoothDiscoverySessionOutcome::UNKNOWN))));
   }
 
   void RegisterAdvertisement(const std::string& id) {
     synchronizer_->RegisterAdvertisement(
         GenerateAdvertisementData(id),
-        base::Bind(&SecureChannelBleSynchronizerTest::OnAdvertisementRegistered,
-                   base::Unretained(this)),
-        base::Bind(
+        base::BindOnce(
+            &SecureChannelBleSynchronizerTest::OnAdvertisementRegistered,
+            base::Unretained(this)),
+        base::BindOnce(
             &SecureChannelBleSynchronizerTest::OnErrorRegisteringAdvertisement,
             base::Unretained(this)));
   }
@@ -278,13 +246,13 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
 
     BleSynchronizer::BluetoothAdvertisementResult expected_result;
     if (success) {
-      register_args_list_[reg_arg_index]->callback.Run(
-          base::MakeRefCounted<device::MockBluetoothAdvertisement>());
+      std::move(register_args_list_[reg_arg_index]->callback)
+          .Run(base::MakeRefCounted<device::MockBluetoothAdvertisement>());
       expected_result = BleSynchronizer::BluetoothAdvertisementResult::SUCCESS;
     } else {
-      register_args_list_[reg_arg_index]->error_callback.Run(
-          device::BluetoothAdvertisement::ErrorCode::
-              INVALID_ADVERTISEMENT_ERROR_CODE);
+      std::move(register_args_list_[reg_arg_index]->error_callback)
+          .Run(device::BluetoothAdvertisement::ErrorCode::
+                   INVALID_ADVERTISEMENT_ERROR_CODE);
       expected_result = BleSynchronizer::BluetoothAdvertisementResult::
           INVALID_ADVERTISEMENT_ERROR_CODE;
     }
@@ -311,12 +279,12 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
   void UnregisterAdvertisement() {
     synchronizer_->UnregisterAdvertisement(
         fake_advertisement_,
-        base::Bind(
+        base::BindOnce(
             &SecureChannelBleSynchronizerTest::OnAdvertisementUnregistered,
             base::Unretained(this)),
-        base::Bind(&SecureChannelBleSynchronizerTest::
-                       OnErrorUnregisteringAdvertisement,
-                   base::Unretained(this)));
+        base::BindOnce(&SecureChannelBleSynchronizerTest::
+                           OnErrorUnregisteringAdvertisement,
+                       base::Unretained(this)));
   }
 
   // If |success| is false, the error code defaults to
@@ -326,16 +294,17 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
       bool success,
       size_t unreg_arg_index,
       size_t expected_unregistration_result_count,
-      const device::BluetoothAdvertisement::ErrorCode& error_code = device::
+      device::BluetoothAdvertisement::ErrorCode error_code = device::
           BluetoothAdvertisement::ErrorCode::INVALID_ADVERTISEMENT_ERROR_CODE) {
     EXPECT_TRUE(unregister_args_list_.size() >= unreg_arg_index);
 
     BleSynchronizer::BluetoothAdvertisementResult expected_result;
     if (success) {
-      unregister_args_list_[unreg_arg_index]->callback.Run();
+      std::move(unregister_args_list_[unreg_arg_index]->callback).Run();
       expected_result = BleSynchronizer::BluetoothAdvertisementResult::SUCCESS;
     } else {
-      unregister_args_list_[unreg_arg_index]->error_callback.Run(error_code);
+      std::move(unregister_args_list_[unreg_arg_index]->error_callback)
+          .Run(error_code);
       BleSynchronizer* derived_type =
           static_cast<BleSynchronizer*>(synchronizer_.get());
       expected_result =
@@ -360,9 +329,10 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
 
   void StartDiscoverySession() {
     synchronizer_->StartDiscoverySession(
-        base::Bind(&SecureChannelBleSynchronizerTest::OnDiscoverySessionStarted,
-                   base::Unretained(this)),
-        base::Bind(
+        base::BindOnce(
+            &SecureChannelBleSynchronizerTest::OnDiscoverySessionStarted,
+            base::Unretained(this)),
+        base::BindOnce(
             &SecureChannelBleSynchronizerTest::OnErrorStartingDiscoverySession,
             base::Unretained(this)));
   }
@@ -374,10 +344,10 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
     EXPECT_TRUE(start_discovery_args_list_.size() >= start_arg_index);
 
     if (success) {
-      start_discovery_args_list_[start_arg_index]->callback.Run(
-          std::make_unique<device::MockBluetoothDiscoverySession>());
+      std::move(start_discovery_args_list_[start_arg_index]->callback).Run();
     } else {
-      start_discovery_args_list_[start_arg_index]->error_callback.Run();
+      std::move(start_discovery_args_list_[start_arg_index]->error_callback)
+          .Run();
     }
 
     histogram_tester_.ExpectUniqueSample(
@@ -391,6 +361,10 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
 
   void OnDiscoverySessionStarted(
       std::unique_ptr<device::BluetoothDiscoverySession> discovery_session) {
+    discovery_session_ = std::move(discovery_session);
+    discovery_session_weak_ptr_factory_ = std::make_unique<
+        base::WeakPtrFactory<device::BluetoothDiscoverySession>>(
+        discovery_session_.get());
     ++num_start_success_;
   }
 
@@ -400,26 +374,23 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
       base::WeakPtr<device::BluetoothDiscoverySession> discovery_session) {
     synchronizer_->StopDiscoverySession(
         discovery_session,
-        base::Bind(&SecureChannelBleSynchronizerTest::OnDiscoverySessionStopped,
-                   base::Unretained(this)),
-        base::Bind(
+        base::BindOnce(
+            &SecureChannelBleSynchronizerTest::OnDiscoverySessionStopped,
+            base::Unretained(this)),
+        base::BindOnce(
             &SecureChannelBleSynchronizerTest::OnErrorStoppingDiscoverySession,
             base::Unretained(this)));
   }
 
   void InvokeStopDiscoveryCallback(
-      bool success,
       size_t stop_arg_index,
       size_t expected_stop_discovery_result_count) {
     EXPECT_TRUE(stop_discovery_args_list_.size() >= stop_arg_index);
 
-    if (success)
-      stop_discovery_args_list_[stop_arg_index]->callback.Run();
-    else
-      stop_discovery_args_list_[stop_arg_index]->error_callback.Run();
+    std::move(stop_discovery_args_list_[stop_arg_index]->callback).Run();
 
     histogram_tester_.ExpectUniqueSample(
-        "InstantTethering.BluetoothDiscoverySessionStopped", success ? 1 : 0,
+        "InstantTethering.BluetoothDiscoverySessionStopped", 1,
         expected_stop_discovery_result_count);
 
     // Reset to make sure that this callback is never double-invoked.
@@ -427,7 +398,10 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
     test_task_runner_->RunUntilIdle();
   }
 
-  void OnDiscoverySessionStopped() { ++num_stop_success_; }
+  void OnDiscoverySessionStopped() {
+    discovery_session_.reset();
+    ++num_stop_success_;
+  }
 
   void OnErrorStoppingDiscoverySession() { ++num_stop_error_; }
 
@@ -437,25 +411,30 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
   }
 
   void OnUnregisterCalled(
-      const device::BluetoothAdvertisement::SuccessCallback& callback,
-      const device::BluetoothAdvertisement::ErrorCallback& error_callback) {
-    unregister_args_list_.emplace_back(base::WrapUnique(
-        new UnregisterAdvertisementArgs(callback, error_callback)));
+      device::BluetoothAdvertisement::SuccessCallback callback,
+      device::BluetoothAdvertisement::ErrorCallback error_callback) {
+    unregister_args_list_.push_back(
+        std::make_unique<UnregisterAdvertisementArgs>(
+            std::move(callback), std::move(error_callback)));
   }
 
-  void OnStopCalled(
-      const base::Closure& callback,
-      const device::BluetoothDiscoverySession::ErrorCallback& error_callback) {
-    stop_discovery_args_list_.emplace_back(base::WrapUnique(
-        new StopDiscoverySessionArgs(callback, error_callback)));
+  void OnStopScan(
+      device::BluetoothAdapter::DiscoverySessionResultCallback callback) {
+    auto repeating_callback =
+        base::AdaptCallbackForRepeating(std::move(callback));
+    stop_discovery_args_list_.emplace_back(
+        base::WrapUnique(new StopDiscoverySessionArgs(
+            base::BindRepeating(
+                repeating_callback, /*is_error=*/false,
+                device::UMABluetoothDiscoverySessionOutcome::SUCCESS),
+            base::BindRepeating(
+                repeating_callback,
+                /*is_error=*/true,
+                device::UMABluetoothDiscoverySessionOutcome::UNKNOWN))));
   }
 
-  const base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   const scoped_refptr<FakeBluetoothAdvertisement> fake_advertisement_;
-  const std::unique_ptr<device::MockBluetoothDiscoverySession>
-      fake_discovery_session_;
-  base::WeakPtrFactory<device::MockBluetoothDiscoverySession>
-      fake_discovery_session_weak_ptr_factory_;
 
   scoped_refptr<NiceMock<MockBluetoothAdapterWithAdvertisements>> mock_adapter_;
 
@@ -471,6 +450,10 @@ class SecureChannelBleSynchronizerTest : public testing::Test {
   std::vector<std::unique_ptr<StopDiscoverySessionArgs>>
       stop_discovery_args_list_;
 
+  std::unique_ptr<device::BluetoothDiscoverySession> discovery_session_;
+  std::unique_ptr<base::WeakPtrFactory<device::BluetoothDiscoverySession>>
+      discovery_session_weak_ptr_factory_;
+
   int num_register_success_;
   int num_register_error_;
   int num_unregister_success_;
@@ -544,22 +527,26 @@ TEST_F(SecureChannelBleSynchronizerTest, TestStartError) {
 }
 
 TEST_F(SecureChannelBleSynchronizerTest, TestStopSuccess) {
-  StopDiscoverySession(fake_discovery_session_weak_ptr_factory_.GetWeakPtr());
-  InvokeStopDiscoveryCallback(true /* success */, 0u /* unreg_arg_index */,
+  StartDiscoverySession();
+  InvokeStartDiscoveryCallback(true /* success */, 0u /* reg_arg_index */,
+                               1 /* expected_start_discovery_result_count */);
+  EXPECT_EQ(1, num_start_success_);
+
+  StopDiscoverySession(discovery_session_weak_ptr_factory_->GetWeakPtr());
+
+  // Advance the clock and fire the timer. This should result in the next
+  // command being executed.
+  test_clock_.Advance(TimeDeltaMillis(kTimeBetweenEachCommandMs));
+  mock_timer_->Fire();
+
+  InvokeStopDiscoveryCallback(0u /* unreg_arg_index */,
                               1 /* expected_stop_discovery_result_count */);
   EXPECT_EQ(1, num_stop_success_);
 }
 
-TEST_F(SecureChannelBleSynchronizerTest, TestStopError) {
-  StopDiscoverySession(fake_discovery_session_weak_ptr_factory_.GetWeakPtr());
-  InvokeStopDiscoveryCallback(false /* success */, 0u /* unreg_arg_index */,
-                              1 /* expected_stop_discovery_result_count */);
-  EXPECT_EQ(1, num_stop_error_);
-}
-
 TEST_F(SecureChannelBleSynchronizerTest, TestStop_DeletedDiscoverySession) {
   // Simulate an invalidated WeakPtr being processed.
-  StopDiscoverySession(base::WeakPtr<device::MockBluetoothDiscoverySession>());
+  StopDiscoverySession(base::WeakPtr<device::BluetoothDiscoverySession>());
   RegisterAdvertisement(kId1);
 
   // Stop() should not have been called.
