@@ -12,6 +12,8 @@
 #include "base/observer_list.h"
 #include "base/unguessable_token.h"
 #include "chromeos/services/secure_channel/public/mojom/secure_channel.mojom.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
 
 namespace chromeos {
 
@@ -27,7 +29,7 @@ class ClientConnectionParameters {
     virtual void OnConnectionRequestCanceled() = 0;
   };
 
-  ClientConnectionParameters(const std::string& feature);
+  explicit ClientConnectionParameters(const std::string& feature);
   virtual ~ClientConnectionParameters();
 
   const base::UnguessableToken& id() const { return id_; }
@@ -48,12 +50,12 @@ class ClientConnectionParameters {
   void SetConnectionAttemptFailed(mojom::ConnectionAttemptFailureReason reason);
 
   // Alerts the client that the connection has succeeded, providing the client
-  // with a Channel and a request to bind a MessageReceiver. This function can
+  // with a Channel and a receiver to bind a MessageReceiver. This function can
   // only be called if IsActive() is true and SetConnectionAttemptFailed() has
   // not been invoked.
   void SetConnectionSucceeded(
-      mojom::ChannelPtr channel,
-      mojom::MessageReceiverRequest message_receiver_request);
+      mojo::PendingRemote<mojom::Channel> channel,
+      mojo::PendingReceiver<mojom::MessageReceiver> message_receiver_receiver);
 
   bool operator==(const ClientConnectionParameters& other) const;
   bool operator<(const ClientConnectionParameters& other) const;
@@ -63,8 +65,9 @@ class ClientConnectionParameters {
   virtual void PerformSetConnectionAttemptFailed(
       mojom::ConnectionAttemptFailureReason reason) = 0;
   virtual void PerformSetConnectionSucceeded(
-      mojom::ChannelPtr channel,
-      mojom::MessageReceiverRequest message_receiver_request) = 0;
+      mojo::PendingRemote<mojom::Channel> channel,
+      mojo::PendingReceiver<mojom::MessageReceiver>
+          message_receiver_receiver) = 0;
 
   void NotifyConnectionRequestCanceled();
 
@@ -74,7 +77,7 @@ class ClientConnectionParameters {
   std::string feature_;
   base::UnguessableToken id_;
 
-  base::ObserverList<Observer> observer_list_;
+  base::ObserverList<Observer>::Unchecked observer_list_;
 
   bool has_invoked_delegate_function_ = false;
 
