@@ -4,7 +4,7 @@
 
 #include "chromeos/services/secure_channel/fake_connection_attempt_delegate.h"
 
-#include "base/logging.h"
+#include "base/check.h"
 #include "chromeos/services/secure_channel/authenticated_channel.h"
 
 namespace chromeos {
@@ -25,9 +25,9 @@ void FakeConnectionAttemptDelegate::OnConnectionAttemptSucceeded(
 
 void FakeConnectionAttemptDelegate::
     OnConnectionAttemptFinishedWithoutConnection(
-        const ConnectionDetails& connection_details) {
-  DCHECK(!connection_details_);
-  connection_details_ = connection_details;
+        const ConnectionAttemptDetails& connection_attempt_details) {
+  DCHECK(!connection_attempt_details_);
+  connection_attempt_details_ = connection_attempt_details;
 }
 
 }  // namespace secure_channel
