@@ -15,23 +15,25 @@ FakeSecureChannel::FakeSecureChannel() = default;
 FakeSecureChannel::~FakeSecureChannel() = default;
 
 void FakeSecureChannel::ListenForConnectionFromDevice(
-    const cryptauth::RemoteDevice& device_to_connect,
-    const cryptauth::RemoteDevice& local_device,
+    const multidevice::RemoteDevice& device_to_connect,
+    const multidevice::RemoteDevice& local_device,
     const std::string& feature,
+    ConnectionMedium connection_medium,
     ConnectionPriority connection_priority,
-    mojom::ConnectionDelegatePtr delegate) {
-  delegate_from_last_listen_call_ = std::move(delegate);
+    mojo::PendingRemote<mojom::ConnectionDelegate> delegate) {
+  delegate_from_last_listen_call_.Bind(std::move(delegate));
 }
 
 void FakeSecureChannel::InitiateConnectionToDevice(
-    const cryptauth::RemoteDevice& device_to_connect,
-    const cryptauth::RemoteDevice& local_device,
+    const multidevice::RemoteDevice& device_to_connect,
+    const multidevice::RemoteDevice& local_device,
     const std::string& feature,
+    ConnectionMedium connection_medium,
     ConnectionPriority connection_priority,
-    mojom::ConnectionDelegatePtr delegate) {
-  delegate_from_last_initiate_call_ = std::move(delegate);
+    mojo::PendingRemote<mojom::ConnectionDelegate> delegate) {
+  delegate_from_last_initiate_call_.Bind(std::move(delegate));
 }
 
 }  // namespace secure_channel
 
-}  // namespace chromeos
\ No newline at end of file
+}  // namespace chromeos
