@@ -4,8 +4,8 @@
 
 #include "chromeos/services/secure_channel/multiplexed_channel.h"
 
+#include "base/check.h"
 #include "base/guid.h"
-#include "base/logging.h"
 
 namespace chromeos {
 
@@ -20,7 +20,7 @@ MultiplexedChannel::MultiplexedChannel(Delegate* delegate,
 MultiplexedChannel::~MultiplexedChannel() = default;
 
 bool MultiplexedChannel::AddClientToChannel(
-    ClientConnectionParameters client_connection_parameters) {
+    std::unique_ptr<ClientConnectionParameters> client_connection_parameters) {
   if (IsDisconnecting() || IsDisconnected())
     return false;
 
