@@ -7,8 +7,7 @@
 #include <utility>
 
 #include "base/memory/ptr_util.h"
-#include "base/no_destructor.h"
-#include "chromeos/components/proximity_auth/logging/logging.h"
+#include "chromeos/components/multidevice/logging/logging.h"
 
 namespace chromeos {
 
@@ -42,14 +41,21 @@ PendingBleInitiatorConnectionRequest::Factory*
     PendingBleInitiatorConnectionRequest::Factory::test_factory_ = nullptr;
 
 // static
-PendingBleInitiatorConnectionRequest::Factory*
-PendingBleInitiatorConnectionRequest::Factory::Get() {
-  if (test_factory_)
-    return test_factory_;
+std::unique_ptr<PendingConnectionRequest<BleInitiatorFailureType>>
+PendingBleInitiatorConnectionRequest::Factory::Create(
+    std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
+    ConnectionPriority connection_priority,
+    PendingConnectionRequestDelegate* delegate,
+    scoped_refptr<device::BluetoothAdapter> bluetooth_adapter) {
+  if (test_factory_) {
+    return test_factory_->CreateInstance(
+        std::move(client_connection_parameters), connection_priority, delegate,
+        bluetooth_adapter);
+  }
 
-  static base::NoDestructor<PendingBleInitiatorConnectionRequest::Factory>
-      factory;
-  return factory.get();
+  return base::WrapUnique(new PendingBleInitiatorConnectionRequest(
+      std::move(client_connection_parameters), connection_priority, delegate,
+      bluetooth_adapter));
 }
 
 // static
@@ -60,17 +66,6 @@ void PendingBleInitiatorConnectionRequest::Factory::SetFactoryForTesting(
 
 PendingBleInitiatorConnectionRequest::Factory::~Factory() = default;
 
-std::unique_ptr<PendingConnectionRequest<BleInitiatorFailureType>>
-PendingBleInitiatorConnectionRequest::Factory::BuildInstance(
-    std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
-    ConnectionPriority connection_priority,
-    PendingConnectionRequestDelegate* delegate,
-    scoped_refptr<device::BluetoothAdapter> bluetooth_adapter) {
-  return base::WrapUnique(new PendingBleInitiatorConnectionRequest(
-      std::move(client_connection_parameters), connection_priority, delegate,
-      bluetooth_adapter));
-}
-
 PendingBleInitiatorConnectionRequest::PendingBleInitiatorConnectionRequest(
     std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
     ConnectionPriority connection_priority,
