@@ -4,15 +4,16 @@
 
 #include "chromeos/services/secure_channel/pending_connection_manager_impl.h"
 
+#include "base/containers/contains.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
-#include "base/no_destructor.h"
-#include "base/stl_util.h"
 #include "chromeos/services/secure_channel/authenticated_channel.h"
 #include "chromeos/services/secure_channel/ble_initiator_connection_attempt.h"
 #include "chromeos/services/secure_channel/ble_listener_connection_attempt.h"
+#include "chromeos/services/secure_channel/nearby_initiator_connection_attempt.h"
 #include "chromeos/services/secure_channel/pending_ble_initiator_connection_request.h"
 #include "chromeos/services/secure_channel/pending_ble_listener_connection_request.h"
+#include "chromeos/services/secure_channel/pending_nearby_initiator_connection_request.h"
 
 namespace chromeos {
 
@@ -23,13 +24,21 @@ PendingConnectionManagerImpl::Factory*
     PendingConnectionManagerImpl::Factory::test_factory_ = nullptr;
 
 // static
-PendingConnectionManagerImpl::Factory*
-PendingConnectionManagerImpl::Factory::Get() {
-  if (test_factory_)
-    return test_factory_;
+std::unique_ptr<PendingConnectionManager>
+PendingConnectionManagerImpl::Factory::Create(
+    Delegate* delegate,
+    BleConnectionManager* ble_connection_manager,
+    NearbyConnectionManager* nearby_connection_manager,
+    scoped_refptr<device::BluetoothAdapter> bluetooth_adapter) {
+  if (test_factory_) {
+    return test_factory_->CreateInstance(delegate, ble_connection_manager,
+                                         nearby_connection_manager,
+                                         bluetooth_adapter);
+  }
 
-  static base::NoDestructor<Factory> factory;
-  return factory.get();
+  return base::WrapUnique(new PendingConnectionManagerImpl(
+      delegate, ble_connection_manager, nearby_connection_manager,
+      bluetooth_adapter));
 }
 
 // static
@@ -40,21 +49,14 @@ void PendingConnectionManagerImpl::Factory::SetFactoryForTesting(
 
 PendingConnectionManagerImpl::Factory::~Factory() = default;
 
-std::unique_ptr<PendingConnectionManager>
-PendingConnectionManagerImpl::Factory::BuildInstance(
-    Delegate* delegate,
-    BleConnectionManager* ble_connection_manager,
-    scoped_refptr<device::BluetoothAdapter> bluetooth_adapter) {
-  return base::WrapUnique(new PendingConnectionManagerImpl(
-      delegate, ble_connection_manager, bluetooth_adapter));
-}
-
 PendingConnectionManagerImpl::PendingConnectionManagerImpl(
     Delegate* delegate,
     BleConnectionManager* ble_connection_manager,
+    NearbyConnectionManager* nearby_connection_manager,
     scoped_refptr<device::BluetoothAdapter> bluetooth_adapter)
     : PendingConnectionManager(delegate),
       ble_connection_manager_(ble_connection_manager),
+      nearby_connection_manager_(nearby_connection_manager),
       bluetooth_adapter_(bluetooth_adapter) {}
 
 PendingConnectionManagerImpl::~PendingConnectionManagerImpl() = default;
@@ -63,17 +65,15 @@ void PendingConnectionManagerImpl::HandleConnectionRequest(
     const ConnectionAttemptDetails& connection_attempt_details,
     std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
     ConnectionPriority connection_priority) {
-  DCHECK_EQ(ConnectionMedium::kBluetoothLowEnergy,
-            connection_attempt_details.connection_medium());
-
   // If the client has canceled the request, it does not need to be processed.
   if (!client_connection_parameters->IsClientWaitingForResponse()) {
-    PA_LOG(INFO) << "PendingConnectionManagerImpl::HandleConnectionRequest(): "
-                 << "Request was canceled by the client before being passed to "
-                 << "PendingConnectionManager; ignoring. Details: "
-                 << connection_attempt_details
-                 << ", Parameters: " << *client_connection_parameters
-                 << ", Priority: " << connection_priority;
+    PA_LOG(VERBOSE)
+        << "PendingConnectionManagerImpl::HandleConnectionRequest(): "
+        << "Request was canceled by the client before being passed to "
+        << "PendingConnectionManager; ignoring. Details: "
+        << connection_attempt_details
+        << ", Parameters: " << *client_connection_parameters
+        << ", Priority: " << connection_priority;
     return;
   }
 
@@ -85,16 +85,16 @@ void PendingConnectionManagerImpl::HandleConnectionRequest(
       connection_attempt_details);
 
   // Process the role-specific details.
-  switch (connection_attempt_details.connection_role()) {
-    case ConnectionRole::kInitiatorRole:
-      HandleBleInitiatorRequest(connection_attempt_details,
-                                std::move(client_connection_parameters),
-                                connection_priority);
+  switch (connection_attempt_details.connection_medium()) {
+    case ConnectionMedium::kBluetoothLowEnergy:
+      HandleBleRequest(connection_attempt_details,
+                       std::move(client_connection_parameters),
+                       connection_priority);
       break;
-    case ConnectionRole::kListenerRole:
-      HandleBleListenerRequest(connection_attempt_details,
-                               std::move(client_connection_parameters),
-                               connection_priority);
+    case ConnectionMedium::kNearbyConnections:
+      HandleNearbyRequest(connection_attempt_details,
+                          std::move(client_connection_parameters),
+                          connection_priority);
       break;
   }
 }
@@ -102,7 +102,7 @@ void PendingConnectionManagerImpl::HandleConnectionRequest(
 void PendingConnectionManagerImpl::OnConnectionAttemptSucceeded(
     const ConnectionDetails& connection_details,
     std::unique_ptr<AuthenticatedChannel> authenticated_channel) {
-  if (!base::ContainsKey(details_to_attempt_details_map_, connection_details)) {
+  if (!base::Contains(details_to_attempt_details_map_, connection_details)) {
     PA_LOG(ERROR) << "PendingConnectionManagerImpl::"
                   << "OnConnectionAttemptSucceeded(): Attempt succeeded, but "
                   << "there was no corresponding map entry. "
@@ -121,22 +121,8 @@ void PendingConnectionManagerImpl::OnConnectionAttemptSucceeded(
   // connection attempt, add them to |all_clients|, and remove the associated
   // map entries.
   for (const auto& connection_attempt_details : to_process) {
-    std::vector<std::unique_ptr<ClientConnectionParameters>> clients_in_loop;
-
-    switch (connection_attempt_details.connection_role()) {
-      case ConnectionRole::kInitiatorRole:
-        clients_in_loop = ConnectionAttempt<BleInitiatorFailureType>::
-            ExtractClientConnectionParameters(
-                std::move(id_pair_to_ble_initiator_connection_attempts_
-                              [connection_attempt_details.device_id_pair()]));
-        break;
-      case ConnectionRole::kListenerRole:
-        clients_in_loop = ConnectionAttempt<BleListenerFailureType>::
-            ExtractClientConnectionParameters(
-                std::move(id_pair_to_ble_listener_connection_attempts_
-                              [connection_attempt_details.device_id_pair()]));
-        break;
-    }
+    std::vector<std::unique_ptr<ClientConnectionParameters>> clients_in_loop =
+        ExtractClientConnectionParameters(connection_attempt_details);
 
     // Move elements in |clients_in_list| to |all_clients|.
     all_clients.insert(all_clients.end(),
@@ -155,17 +141,35 @@ void PendingConnectionManagerImpl::OnConnectionAttemptFinishedWithoutConnection(
   RemoveMapEntriesForFinishedConnectionAttempt(connection_attempt_details);
 }
 
+void PendingConnectionManagerImpl::HandleBleRequest(
+    const ConnectionAttemptDetails& connection_attempt_details,
+    std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
+    ConnectionPriority connection_priority) {
+  switch (connection_attempt_details.connection_role()) {
+    case ConnectionRole::kInitiatorRole:
+      HandleBleInitiatorRequest(connection_attempt_details,
+                                std::move(client_connection_parameters),
+                                connection_priority);
+      break;
+    case ConnectionRole::kListenerRole:
+      HandleBleListenerRequest(connection_attempt_details,
+                               std::move(client_connection_parameters),
+                               connection_priority);
+      break;
+  }
+}
+
 void PendingConnectionManagerImpl::HandleBleInitiatorRequest(
     const ConnectionAttemptDetails& connection_attempt_details,
     std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
     ConnectionPriority connection_priority) {
   // If no ConnectionAttempt exists to this device in the initiator role, create
   // one.
-  if (!base::ContainsKey(id_pair_to_ble_initiator_connection_attempts_,
-                         connection_attempt_details.device_id_pair())) {
+  if (!base::Contains(id_pair_to_ble_initiator_connection_attempts_,
+                      connection_attempt_details.device_id_pair())) {
     id_pair_to_ble_initiator_connection_attempts_[connection_attempt_details
                                                       .device_id_pair()] =
-        BleInitiatorConnectionAttempt::Factory::Get()->BuildInstance(
+        BleInitiatorConnectionAttempt::Factory::Create(
             ble_connection_manager_, this /* delegate */,
             connection_attempt_details);
   }
@@ -175,15 +179,14 @@ void PendingConnectionManagerImpl::HandleBleInitiatorRequest(
                                                         .device_id_pair()];
 
   bool success = connection_attempt->AddPendingConnectionRequest(
-      PendingBleInitiatorConnectionRequest::Factory::Get()->BuildInstance(
+      PendingBleInitiatorConnectionRequest::Factory::Create(
           std::move(client_connection_parameters), connection_priority,
           connection_attempt.get() /* delegate */, bluetooth_adapter_));
 
   if (!success) {
     PA_LOG(ERROR) << "PendingConnectionManagerImpl::"
                   << "HandleBleInitiatorRequest(): Not able to handle request. "
-                  << "Details: " << connection_attempt_details
-                  << ", Client parameters: " << *client_connection_parameters;
+                  << "Details: " << connection_attempt_details;
     NOTREACHED();
   }
 }
@@ -194,11 +197,11 @@ void PendingConnectionManagerImpl::HandleBleListenerRequest(
     ConnectionPriority connection_priority) {
   // If no ConnectionAttempt exists to this device in the listener role, create
   // one.
-  if (!base::ContainsKey(id_pair_to_ble_listener_connection_attempts_,
-                         connection_attempt_details.device_id_pair())) {
+  if (!base::Contains(id_pair_to_ble_listener_connection_attempts_,
+                      connection_attempt_details.device_id_pair())) {
     id_pair_to_ble_listener_connection_attempts_[connection_attempt_details
                                                      .device_id_pair()] =
-        BleListenerConnectionAttempt::Factory::Get()->BuildInstance(
+        BleListenerConnectionAttempt::Factory::Create(
             ble_connection_manager_, this /* delegate */,
             connection_attempt_details);
   }
@@ -208,59 +211,118 @@ void PendingConnectionManagerImpl::HandleBleListenerRequest(
                                                        .device_id_pair()];
 
   bool success = connection_attempt->AddPendingConnectionRequest(
-      PendingBleListenerConnectionRequest::Factory::Get()->BuildInstance(
+      PendingBleListenerConnectionRequest::Factory::Create(
           std::move(client_connection_parameters), connection_priority,
           connection_attempt.get() /* delegate */, bluetooth_adapter_));
 
   if (!success) {
     PA_LOG(ERROR) << "PendingConnectionManagerImpl::"
                   << "HandleBleListenerRequest(): Not able to handle request. "
-                  << "Details: " << connection_attempt_details
-                  << ", Client parameters: " << *client_connection_parameters;
+                  << "Details: " << connection_attempt_details;
     NOTREACHED();
   }
 }
 
+void PendingConnectionManagerImpl::HandleNearbyRequest(
+    const ConnectionAttemptDetails& connection_attempt_details,
+    std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
+    ConnectionPriority connection_priority) {
+  switch (connection_attempt_details.connection_role()) {
+    case ConnectionRole::kInitiatorRole:
+      HandleNearbyInitiatorRequest(connection_attempt_details,
+                                   std::move(client_connection_parameters),
+                                   connection_priority);
+      break;
+    case ConnectionRole::kListenerRole:
+      NOTREACHED()
+          << "PendingConnectionManagerImpl::HandleConnectionRequest(): "
+          << "Nearby Connections is not supported in the listener role.";
+      break;
+  }
+}
+
+void PendingConnectionManagerImpl::HandleNearbyInitiatorRequest(
+    const ConnectionAttemptDetails& connection_attempt_details,
+    std::unique_ptr<ClientConnectionParameters> client_connection_parameters,
+    ConnectionPriority connection_priority) {
+  // If no ConnectionAttempt exists to this device in the initiator role, create
+  // one.
+  if (!base::Contains(id_pair_to_nearby_initiator_connection_attempts_,
+                      connection_attempt_details.device_id_pair())) {
+    id_pair_to_nearby_initiator_connection_attempts_[connection_attempt_details
+                                                         .device_id_pair()] =
+        NearbyInitiatorConnectionAttempt::Factory::Create(
+            nearby_connection_manager_, this /* delegate */,
+            connection_attempt_details);
+  }
+
+  auto& connection_attempt = id_pair_to_nearby_initiator_connection_attempts_
+      [connection_attempt_details.device_id_pair()];
+
+  bool success = connection_attempt->AddPendingConnectionRequest(
+      PendingNearbyInitiatorConnectionRequest::Factory::Create(
+          std::move(client_connection_parameters), connection_priority,
+          connection_attempt.get() /* delegate */, bluetooth_adapter_));
+
+  if (!success) {
+    PA_LOG(ERROR)
+        << "PendingConnectionManagerImpl::"
+        << "HandleNearbyInitiatorRequest(): Not able to handle request. "
+        << "Details: " << connection_attempt_details;
+    NOTREACHED();
+  }
+}
+
+std::vector<std::unique_ptr<ClientConnectionParameters>>
+PendingConnectionManagerImpl::ExtractClientConnectionParameters(
+    const ConnectionAttemptDetails& connection_attempt_details) {
+  switch (connection_attempt_details.connection_medium()) {
+    case ConnectionMedium::kBluetoothLowEnergy:
+      switch (connection_attempt_details.connection_role()) {
+        // BLE initiator:
+        case ConnectionRole::kInitiatorRole:
+          return ConnectionAttempt<BleInitiatorFailureType>::
+              ExtractClientConnectionParameters(
+                  std::move(id_pair_to_ble_initiator_connection_attempts_
+                                [connection_attempt_details.device_id_pair()]));
+
+        // BLE listener:
+        case ConnectionRole::kListenerRole:
+          return ConnectionAttempt<BleListenerFailureType>::
+              ExtractClientConnectionParameters(
+                  std::move(id_pair_to_ble_listener_connection_attempts_
+                                [connection_attempt_details.device_id_pair()]));
+      }
+
+    case ConnectionMedium::kNearbyConnections:
+      switch (connection_attempt_details.connection_role()) {
+        // Nearby initiator:
+        case ConnectionRole::kInitiatorRole:
+          return ConnectionAttempt<NearbyInitiatorFailureType>::
+              ExtractClientConnectionParameters(
+                  std::move(id_pair_to_nearby_initiator_connection_attempts_
+                                [connection_attempt_details.device_id_pair()]));
+
+        // Nearby listener:
+        case ConnectionRole::kListenerRole:
+          NOTREACHED() << "Nearby listener connections are not implemented.";
+          return std::vector<std::unique_ptr<ClientConnectionParameters>>();
+      }
+  }
+}
+
 void PendingConnectionManagerImpl::RemoveMapEntriesForFinishedConnectionAttempt(
     const ConnectionAttemptDetails& connection_attempt_details) {
   // Make a copy of |connection_attempt_details|, since it belongs to the
   // ConnectionAttempt which is about to be deleted below.
   ConnectionAttemptDetails connection_attempt_details_copy =
       connection_attempt_details;
-
-  switch (connection_attempt_details_copy.connection_role()) {
-    case ConnectionRole::kInitiatorRole: {
-      size_t num_deleted = id_pair_to_ble_initiator_connection_attempts_.erase(
-          connection_attempt_details_copy.device_id_pair());
-      if (num_deleted != 1u) {
-        PA_LOG(ERROR) << "PendingConnectionManagerImpl::"
-                      << "RemoveMapEntriesForFinishedConnectionAttempt(): "
-                      << "Tried to remove failed initiator ConnectionAttempt, "
-                      << "but it was not present in the map. Details: "
-                      << connection_attempt_details_copy;
-        NOTREACHED();
-      }
-      break;
-    }
-
-    case ConnectionRole::kListenerRole: {
-      size_t num_deleted = id_pair_to_ble_listener_connection_attempts_.erase(
-          connection_attempt_details_copy.device_id_pair());
-      if (num_deleted != 1u) {
-        PA_LOG(ERROR) << "PendingConnectionManagerImpl::"
-                      << "RemoveMapEntriesForFinishedConnectionAttempt(): "
-                      << "Tried to remove failed listener ConnectionAttempt, "
-                      << "but it was not present in the map. Details: "
-                      << connection_attempt_details_copy;
-        NOTREACHED();
-      }
-      break;
-    }
-  }
-
   ConnectionDetails connection_details =
       connection_attempt_details_copy.GetAssociatedConnectionDetails();
 
+  RemoveIdPairToConnectionAttemptMapEntriesForFinishedConnectionAttempt(
+      connection_attempt_details_copy);
+
   size_t num_deleted =
       details_to_attempt_details_map_[connection_details].erase(
           connection_attempt_details_copy);
@@ -279,6 +341,66 @@ void PendingConnectionManagerImpl::RemoveMapEntriesForFinishedConnectionAttempt(
     details_to_attempt_details_map_.erase(connection_details);
 }
 
+void PendingConnectionManagerImpl::
+    RemoveIdPairToConnectionAttemptMapEntriesForFinishedConnectionAttempt(
+        const ConnectionAttemptDetails& connection_attempt_details) {
+  switch (connection_attempt_details.connection_medium()) {
+    case ConnectionMedium::kBluetoothLowEnergy:
+      switch (connection_attempt_details.connection_role()) {
+        // BLE initiator.
+        case ConnectionRole::kInitiatorRole: {
+          size_t num_deleted =
+              id_pair_to_ble_initiator_connection_attempts_.erase(
+                  connection_attempt_details.device_id_pair());
+          if (num_deleted != 1u) {
+            PA_LOG(ERROR) << "Tried to remove failed BLE initiator "
+                          << "ConnectionAttempt, but it was not present in the "
+                          << "map. Details: " << connection_attempt_details;
+            NOTREACHED();
+          }
+          break;
+        }
+
+        // BLE listener.
+        case ConnectionRole::kListenerRole: {
+          size_t num_deleted =
+              id_pair_to_ble_listener_connection_attempts_.erase(
+                  connection_attempt_details.device_id_pair());
+          if (num_deleted != 1u) {
+            PA_LOG(ERROR) << "Tried to remove failed BLE listener "
+                          << "ConnectionAttempt, but it was not present in the "
+                          << "map. Details: " << connection_attempt_details;
+            NOTREACHED();
+          }
+          break;
+        }
+      }
+      break;
+
+    case ConnectionMedium::kNearbyConnections:
+      switch (connection_attempt_details.connection_role()) {
+        // Nearby initiator.
+        case ConnectionRole::kInitiatorRole: {
+          size_t num_deleted =
+              id_pair_to_nearby_initiator_connection_attempts_.erase(
+                  connection_attempt_details.device_id_pair());
+          if (num_deleted != 1u) {
+            PA_LOG(ERROR) << "Tried to remove failed Nearby initiator "
+                          << "ConnectionAttempt, but it was not present in the "
+                          << "map. Details: " << connection_attempt_details;
+            NOTREACHED();
+          }
+          break;
+        }
+
+        case ConnectionRole::kListenerRole:
+          NOTREACHED();
+          break;
+      }
+      break;
+  }
+}
+
 }  // namespace secure_channel
 
 }  // namespace chromeos
