@@ -15,14 +15,15 @@
 #include "base/i18n/unicodestring.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/singleton.h"
 #include "base/observer_list.h"
+#include "base/stl_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/sys_info.h"
+#include "base/system/sys_info.h"
 #include "base/task/post_task.h"
+#include "base/task/thread_pool.h"
 #include "base/task_runner.h"
 #include "chromeos/settings/timezone_settings_helper.h"
 
@@ -157,6 +158,7 @@ static const char* kTimeZones[] = {
     "Asia/Yerevan",
     "Asia/Kabul",
     "Asia/Karachi",
+    "Asia/Aqtobe",
     "Asia/Ashgabat",
     "Asia/Oral",
     "Asia/Yekaterinburg",
@@ -254,7 +256,7 @@ void SetTimezoneIDFromString(const std::string& id) {
   }
 
   // Delete old symlink2 if it exists.
-  base::DeleteFile(timezone_symlink2, false);
+  base::DeleteFile(timezone_symlink2);
 
   // Create new symlink2.
   if (symlink(timezone_file.value().c_str(),
@@ -370,7 +372,7 @@ TimezoneSettingsBaseImpl::GetTimezoneList() const {
 }
 
 TimezoneSettingsBaseImpl::TimezoneSettingsBaseImpl() {
-  for (size_t i = 0; i < arraysize(kTimeZones); ++i) {
+  for (size_t i = 0; i < base::size(kTimeZones); ++i) {
     timezones_.push_back(base::WrapUnique(icu::TimeZone::createTimeZone(
         icu::UnicodeString(kTimeZones[i], -1, US_INV))));
   }
@@ -393,10 +395,10 @@ void TimezoneSettingsImpl::SetTimezone(const icu::TimeZone& timezone) {
   VLOG(1) << "Setting timezone to " << id;
   // It's safe to change the timezone config files in the background as the
   // following operations don't depend on the completion of the config change.
-  base::PostTaskWithTraits(FROM_HERE,
-                           {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
-                            base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
-                           base::BindOnce(&SetTimezoneIDFromString, id));
+  base::ThreadPool::PostTask(FROM_HERE,
+                             {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
+                              base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
+                             base::BindOnce(&SetTimezoneIDFromString, id));
   icu::TimeZone::setDefault(*known_timezone);
   for (auto& observer : observers_)
     observer.TimezoneChanged(*known_timezone);
