@@ -10,16 +10,17 @@
 
 #include "base/bind.h"
 #include "base/location.h"
-#include "base/macros.h"
-#include "base/message_loop/message_loop.h"
 #include "base/optional.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
+#include "base/stl_util.h"
 #include "base/task_runner.h"
+#include "base/test/task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "chromeos/cryptohome/cryptohome_parameters.h"
-#include "chromeos/dbus/cryptohome_client.h"
-#include "chromeos/dbus/fake_cryptohome_client.h"
+#include "chromeos/dbus/cryptohome/cryptohome_client.h"
+#include "chromeos/dbus/cryptohome/fake_cryptohome_client.h"
+#include "chromeos/dbus/tpm_manager/tpm_manager_client.h"
 #include "chromeos/tpm/tpm_token_info_getter.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -57,7 +58,6 @@ class FakeTaskRunner : public base::TaskRunner {
     base::ThreadTaskRunnerHandle::Get()->PostTask(from_here, std::move(task));
     return true;
   }
-  bool RunsTasksInCurrentSequence() const override { return true; }
 
  protected:
   ~FakeTaskRunner() override = default;
@@ -80,24 +80,12 @@ class TestCryptohomeClient : public chromeos::FakeCryptohomeClient {
   // using the TestCryptohomeClient. Should be empty for system token.
   explicit TestCryptohomeClient(const AccountId& account_id)
       : account_id_(account_id),
-        tpm_is_enabled_(true),
-        tpm_is_enabled_failure_count_(0),
-        tpm_is_enabled_succeeded_(false),
         get_tpm_token_info_failure_count_(0),
         get_tpm_token_info_not_set_count_(0),
         get_tpm_token_info_succeeded_(false) {}
 
   ~TestCryptohomeClient() override = default;
 
-  void set_tpm_is_enabled(bool value) {
-    tpm_is_enabled_ = value;
-  }
-
-  void set_tpm_is_enabled_failure_count(int value) {
-    ASSERT_GT(value, 0);
-    tpm_is_enabled_failure_count_ = value;
-  }
-
   void set_get_tpm_token_info_failure_count(int value) {
     ASSERT_GT(value, 0);
     get_tpm_token_info_failure_count_ = value;
@@ -120,19 +108,6 @@ class TestCryptohomeClient : public chromeos::FakeCryptohomeClient {
 
  private:
   // FakeCryptohomeClient override.
-  void TpmIsEnabled(chromeos::DBusMethodCallback<bool> callback) override {
-    ASSERT_FALSE(tpm_is_enabled_succeeded_);
-    base::Optional<bool> result;
-    if (tpm_is_enabled_failure_count_ > 0) {
-      --tpm_is_enabled_failure_count_;
-    } else {
-      tpm_is_enabled_succeeded_ = true;
-      result.emplace(tpm_is_enabled_);
-    }
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::BindOnce(std::move(callback), result));
-  }
-
   void Pkcs11GetTpmTokenInfo(
       chromeos::DBusMethodCallback<TpmTokenInfo> callback) override {
     ASSERT_TRUE(account_id_.empty());
@@ -156,7 +131,6 @@ class TestCryptohomeClient : public chromeos::FakeCryptohomeClient {
   // properly set before calling this.
   void HandleGetTpmTokenInfo(
       chromeos::DBusMethodCallback<TpmTokenInfo> callback) {
-    ASSERT_TRUE(tpm_is_enabled_succeeded_);
     ASSERT_FALSE(get_tpm_token_info_succeeded_);
     ASSERT_TRUE(pending_get_tpm_token_info_callback_.is_null());
 
@@ -190,14 +164,10 @@ class TestCryptohomeClient : public chromeos::FakeCryptohomeClient {
     // Called synchronously for convenience (to avoid using extra RunLoop in
     // tests). Unlike with other Cryptohome callbacks, TPMTokenInfoGetter does
     // not rely on this callback being called asynchronously.
-    base::ResetAndReturn(&pending_get_tpm_token_info_callback_)
-        .Run(tpm_token_info_);
+    std::move(pending_get_tpm_token_info_callback_).Run(tpm_token_info_);
   }
 
   AccountId account_id_;
-  bool tpm_is_enabled_;
-  int tpm_is_enabled_failure_count_;
-  bool tpm_is_enabled_succeeded_;
   int get_tpm_token_info_failure_count_;
   int get_tpm_token_info_not_set_count_;
   bool get_tpm_token_info_succeeded_;
@@ -210,8 +180,12 @@ class TestCryptohomeClient : public chromeos::FakeCryptohomeClient {
 
 class SystemTPMTokenInfoGetterTest : public testing::Test {
  public:
-  SystemTPMTokenInfoGetterTest() = default;
-  ~SystemTPMTokenInfoGetterTest() override = default;
+  SystemTPMTokenInfoGetterTest() {
+    chromeos::TpmManagerClient::Get()->InitializeFake();
+  }
+  ~SystemTPMTokenInfoGetterTest() override {
+    chromeos::TpmManagerClient::Get()->Shutdown();
+  }
 
   void SetUp() override {
     cryptohome_client_.reset(new TestCryptohomeClient(EmptyAccountId()));
@@ -227,7 +201,7 @@ class SystemTPMTokenInfoGetterTest : public testing::Test {
   std::vector<int64_t> delays_;
 
  private:
-  base::MessageLoop message_loop_;
+  base::test::SingleThreadTaskEnvironment task_environment_;
 
   DISALLOW_COPY_AND_ASSIGN(SystemTPMTokenInfoGetterTest);
 };
@@ -235,8 +209,12 @@ class SystemTPMTokenInfoGetterTest : public testing::Test {
 class UserTPMTokenInfoGetterTest : public testing::Test {
  public:
   UserTPMTokenInfoGetterTest()
-      : account_id_(AccountId::FromUserEmail("user@gmail.com")) {}
-  ~UserTPMTokenInfoGetterTest() override = default;
+      : account_id_(AccountId::FromUserEmail("user@gmail.com")) {
+    chromeos::TpmManagerClient::Get()->InitializeFake();
+  }
+  ~UserTPMTokenInfoGetterTest() override {
+    chromeos::TpmManagerClient::Get()->Shutdown();
+  }
 
   void SetUp() override {
     cryptohome_client_.reset(new TestCryptohomeClient(account_id_));
@@ -253,7 +231,7 @@ class UserTPMTokenInfoGetterTest : public testing::Test {
   std::vector<int64_t> delays_;
 
  private:
-  base::MessageLoop message_loop_;
+  base::test::SingleThreadTaskEnvironment task_environment_;
 
   DISALLOW_COPY_AND_ASSIGN(UserTPMTokenInfoGetterTest);
 };
@@ -299,7 +277,10 @@ TEST_F(SystemTPMTokenInfoGetterTest, TokenSlotIdEqualsZero) {
 }
 
 TEST_F(SystemTPMTokenInfoGetterTest, TPMNotEnabled) {
-  cryptohome_client_->set_tpm_is_enabled(false);
+  chromeos::TpmManagerClient::Get()
+      ->GetTestInterface()
+      ->mutable_nonsensitive_status_reply()
+      ->set_is_enabled(false);
 
   bool completed = false;
   base::Optional<TpmTokenInfo> result;
@@ -311,8 +292,36 @@ TEST_F(SystemTPMTokenInfoGetterTest, TPMNotEnabled) {
   EXPECT_EQ(std::vector<int64_t>(), delays_);
 }
 
+TEST_F(SystemTPMTokenInfoGetterTest, TPMNotEnabledSystemSlotFallbackEnabled) {
+  chromeos::TpmManagerClient::Get()
+      ->GetTestInterface()
+      ->mutable_nonsensitive_status_reply()
+      ->set_is_enabled(false);
+
+  bool completed = false;
+  base::Optional<TpmTokenInfo> result;
+  tpm_token_info_getter_->SetSystemSlotSoftwareFallback(true);
+  tpm_token_info_getter_->Start(
+      base::BindOnce(&OnTpmTokenInfoGetterCompleted, &completed, &result));
+  base::RunLoop().RunUntilIdle();
+  EXPECT_FALSE(completed);
+
+  const TpmTokenInfo fake_token_info = {"TOKEN_1", "2222", 1};
+  cryptohome_client_->SetTpmTokenInfo(fake_token_info);
+
+  EXPECT_TRUE(completed);
+  ASSERT_TRUE(result.has_value());
+  EXPECT_EQ("TOKEN_1", result->label);
+  EXPECT_EQ("2222", result->user_pin);
+  EXPECT_EQ(1, result->slot);
+
+  EXPECT_EQ(std::vector<int64_t>(), delays_);
+}
+
 TEST_F(SystemTPMTokenInfoGetterTest, TpmEnabledCallFails) {
-  cryptohome_client_->set_tpm_is_enabled_failure_count(1);
+  chromeos::TpmManagerClient::Get()
+      ->GetTestInterface()
+      ->set_non_nonsensitive_status_dbus_error_count(1);
 
   bool completed = false;
   base::Optional<TpmTokenInfo> result;
@@ -332,7 +341,7 @@ TEST_F(SystemTPMTokenInfoGetterTest, TpmEnabledCallFails) {
 
   const int64_t kExpectedDelays[] = {100};
   EXPECT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
@@ -357,7 +366,7 @@ TEST_F(SystemTPMTokenInfoGetterTest, GetTpmTokenInfoInitiallyNotReady) {
 
   const int64_t kExpectedDelays[] = {100};
   EXPECT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
@@ -382,12 +391,14 @@ TEST_F(SystemTPMTokenInfoGetterTest, GetTpmTokenInfoInitiallyFails) {
 
   const int64_t kExpectedDelays[] = {100};
   EXPECT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
 TEST_F(SystemTPMTokenInfoGetterTest, RetryDelaysIncreaseExponentially) {
-  cryptohome_client_->set_tpm_is_enabled_failure_count(2);
+  chromeos::TpmManagerClient::Get()
+      ->GetTestInterface()
+      ->set_non_nonsensitive_status_dbus_error_count(2);
   cryptohome_client_->set_get_tpm_token_info_failure_count(1);
   cryptohome_client_->set_get_tpm_token_info_not_set_count(3);
 
@@ -409,12 +420,14 @@ TEST_F(SystemTPMTokenInfoGetterTest, RetryDelaysIncreaseExponentially) {
 
   int64_t kExpectedDelays[] = {100, 200, 400, 800, 1600, 3200};
   ASSERT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
 TEST_F(SystemTPMTokenInfoGetterTest, RetryDelayBounded) {
-  cryptohome_client_->set_tpm_is_enabled_failure_count(4);
+  chromeos::TpmManagerClient::Get()
+      ->GetTestInterface()
+      ->set_non_nonsensitive_status_dbus_error_count(4);
   cryptohome_client_->set_get_tpm_token_info_failure_count(5);
   cryptohome_client_->set_get_tpm_token_info_not_set_count(6);
 
@@ -438,7 +451,7 @@ TEST_F(SystemTPMTokenInfoGetterTest, RetryDelayBounded) {
                                3200,   6400,   12800,  25600,  51200,
                                102400, 204800, 300000, 300000, 300000};
   ASSERT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
@@ -483,7 +496,7 @@ TEST_F(UserTPMTokenInfoGetterTest, GetTpmTokenInfoInitiallyFails) {
 
   const int64_t kExpectedDelays[] = {100};
   EXPECT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
@@ -508,7 +521,7 @@ TEST_F(UserTPMTokenInfoGetterTest, GetTpmTokenInfoInitiallyNotReady) {
 
   const int64_t kExpectedDelays[] = {100};
   EXPECT_EQ(std::vector<int64_t>(kExpectedDelays,
-                                 kExpectedDelays + arraysize(kExpectedDelays)),
+                                 kExpectedDelays + base::size(kExpectedDelays)),
             delays_);
 }
 
