@@ -11,6 +11,7 @@
 #include "base/callback.h"
 #include "base/compiler_specific.h"
 #include "base/macros.h"
+#include "build/chromeos_buildflags.h"
 #include "crypto/crypto_export.h"
 
 namespace base {
@@ -35,7 +36,7 @@ CRYPTO_EXPORT void EnsureNSSInit();
 // A sample version string is "3.12.3".
 bool CheckNSSVersion(const char* version);
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
 // Indicates that NSS should use the Chaps library so that we
 // can access the TPM through NSS.  InitializeTPMTokenAndSystemSlot and
 // InitializeTPMForChromeOSUser must still be called to load the slots.
@@ -51,7 +52,7 @@ CRYPTO_EXPORT bool IsTPMTokenEnabledForNSS();
 // If |callback| is non-null and the function returns false, the |callback| will
 // be run once the TPM is ready. |callback| will never be run if the function
 // returns true.
-CRYPTO_EXPORT bool IsTPMTokenReady(const base::Closure& callback)
+CRYPTO_EXPORT bool IsTPMTokenReady(base::OnceClosure callback)
     WARN_UNUSED_RESULT;
 
 // Initialize the TPM token and system slot. The |callback| will run on the same
@@ -61,7 +62,7 @@ CRYPTO_EXPORT bool IsTPMTokenReady(const base::Closure& callback)
 // |callback| has been run.
 CRYPTO_EXPORT void InitializeTPMTokenAndSystemSlot(
     int system_slot_id,
-    const base::Callback<void(bool)>& callback);
+    base::OnceCallback<void(bool)> callback);
 #endif
 
 // Convert a NSS PRTime value into a base::Time object.
