@@ -6,6 +6,7 @@ package org.chromium.device.bluetooth;
 
 import android.annotation.TargetApi;
 import android.bluetooth.BluetoothAdapter;
+import android.bluetooth.le.ScanFilter;
 import android.bluetooth.le.ScanSettings;
 import android.content.BroadcastReceiver;
 import android.content.Context;
@@ -17,7 +18,9 @@ import android.util.SparseArray;
 
 import org.chromium.base.Log;
 import org.chromium.base.annotations.CalledByNative;
+import org.chromium.base.annotations.JNIAdditionalImport;
 import org.chromium.base.annotations.JNINamespace;
+import org.chromium.base.annotations.NativeMethods;
 import org.chromium.components.location.LocationUtils;
 
 import java.util.List;
@@ -31,6 +34,7 @@ import java.util.Map;
  * Lifetime is controlled by device::BluetoothAdapterAndroid.
  */
 @JNINamespace("device")
+@JNIAdditionalImport(Wrappers.class)
 @TargetApi(Build.VERSION_CODES.M)
 final class ChromeBluetoothAdapter extends BroadcastReceiver {
     private static final String TAG = "Bluetooth";
@@ -77,14 +81,10 @@ final class ChromeBluetoothAdapter extends BroadcastReceiver {
     // BluetoothAdapterAndroid methods implemented in java:
 
     // Implements BluetoothAdapterAndroid::Create.
-    // 'Object' type must be used for |adapterWrapper| because inner class
-    // Wrappers.BluetoothAdapterWrapper reference is not handled by jni_generator.py JavaToJni.
-    // http://crbug.com/505554
     @CalledByNative
     private static ChromeBluetoothAdapter create(
-            long nativeBluetoothAdapterAndroid, Object adapterWrapper) {
-        return new ChromeBluetoothAdapter(
-                nativeBluetoothAdapterAndroid, (Wrappers.BluetoothAdapterWrapper) adapterWrapper);
+            long nativeBluetoothAdapterAndroid, Wrappers.BluetoothAdapterWrapper adapterWrapper) {
+        return new ChromeBluetoothAdapter(nativeBluetoothAdapterAndroid, adapterWrapper);
     }
 
     // Implements BluetoothAdapterAndroid::GetAddress.
@@ -144,10 +144,11 @@ final class ChromeBluetoothAdapter extends BroadcastReceiver {
 
     /**
      * Starts a Low Energy scan.
+     * @param filters List of filters used to minimize number of devices returned
      * @return True on success.
      */
     @CalledByNative
-    private boolean startScan() {
+    private boolean startScan(List<ScanFilter> filters) {
         Wrappers.BluetoothLeScannerWrapper scanner = mAdapter.getBluetoothLeScanner();
 
         if (scanner == null) {
@@ -166,7 +167,7 @@ final class ChromeBluetoothAdapter extends BroadcastReceiver {
         mScanCallback = new ScanCallback();
 
         try {
-            scanner.startScan(null /* filters */, scanMode, mScanCallback);
+            scanner.startScan(filters, scanMode, mScanCallback);
         } catch (IllegalArgumentException e) {
             Log.e(TAG, "Cannot start scan: " + e);
             mScanCallback = null;
@@ -290,17 +291,21 @@ final class ChromeBluetoothAdapter extends BroadcastReceiver {
 
             // Object can be destroyed, but Android keeps calling onScanResult.
             if (mNativeBluetoothAdapterAndroid != 0) {
-                nativeCreateOrUpdateDeviceOnScan(mNativeBluetoothAdapterAndroid,
-                        result.getDevice().getAddress(), result.getDevice(), result.getRssi(),
-                        uuid_strings, result.getScanRecord_getTxPowerLevel(), serviceDataKeys,
-                        serviceDataValues, manufacturerDataKeys, manufacturerDataValues);
+                ChromeBluetoothAdapterJni.get().createOrUpdateDeviceOnScan(
+                        mNativeBluetoothAdapterAndroid, ChromeBluetoothAdapter.this,
+                        result.getDevice().getAddress(), result.getDevice(),
+                        result.getScanRecord_getDeviceName(), result.getRssi(), uuid_strings,
+                        result.getScanRecord_getTxPowerLevel(), serviceDataKeys, serviceDataValues,
+                        manufacturerDataKeys, manufacturerDataValues,
+                        result.getScanRecord_getAdvertiseFlags());
             }
         }
 
         @Override
         public void onScanFailed(int errorCode) {
             Log.w(TAG, "onScanFailed: %d", errorCode);
-            nativeOnScanFailed(mNativeBluetoothAdapterAndroid);
+            ChromeBluetoothAdapterJni.get().onScanFailed(
+                    mNativeBluetoothAdapterAndroid, ChromeBluetoothAdapter.this);
         }
     }
 
@@ -316,10 +321,12 @@ final class ChromeBluetoothAdapter extends BroadcastReceiver {
 
             switch (state) {
                 case BluetoothAdapter.STATE_ON:
-                    nativeOnAdapterStateChanged(mNativeBluetoothAdapterAndroid, true);
+                    ChromeBluetoothAdapterJni.get().onAdapterStateChanged(
+                            mNativeBluetoothAdapterAndroid, ChromeBluetoothAdapter.this, true);
                     break;
                 case BluetoothAdapter.STATE_OFF:
-                    nativeOnAdapterStateChanged(mNativeBluetoothAdapterAndroid, false);
+                    ChromeBluetoothAdapterJni.get().onAdapterStateChanged(
+                            mNativeBluetoothAdapterAndroid, ChromeBluetoothAdapter.this, false);
                     break;
                 default:
                     // do nothing
@@ -343,22 +350,21 @@ final class ChromeBluetoothAdapter extends BroadcastReceiver {
         }
     }
 
-    // ---------------------------------------------------------------------------------------------
-    // BluetoothAdapterAndroid C++ methods declared for access from java:
+    @NativeMethods
+    interface Natives {
+        // Binds to BluetoothAdapterAndroid::OnScanFailed.
+        void onScanFailed(long nativeBluetoothAdapterAndroid, ChromeBluetoothAdapter caller);
 
-    // Binds to BluetoothAdapterAndroid::OnScanFailed.
-    private native void nativeOnScanFailed(long nativeBluetoothAdapterAndroid);
+        // Binds to BluetoothAdapterAndroid::CreateOrUpdateDeviceOnScan.
+        void createOrUpdateDeviceOnScan(long nativeBluetoothAdapterAndroid,
+                ChromeBluetoothAdapter caller, String address,
+                Wrappers.BluetoothDeviceWrapper deviceWrapper, String localName, int rssi,
+                String[] advertisedUuids, int txPower, String[] serviceDataKeys,
+                Object[] serviceDataValues, int[] manufacturerDataKeys,
+                Object[] manufacturerDataValues, int advertiseFlags);
 
-    // Binds to BluetoothAdapterAndroid::CreateOrUpdateDeviceOnScan.
-    // 'Object' type must be used for |bluetoothDeviceWrapper| because inner class
-    // Wrappers.BluetoothDeviceWrapper reference is not handled by jni_generator.py JavaToJni.
-    // http://crbug.com/505554
-    private native void nativeCreateOrUpdateDeviceOnScan(long nativeBluetoothAdapterAndroid,
-            String address, Object bluetoothDeviceWrapper, int rssi, String[] advertisedUuids,
-            int txPower, String[] serviceDataKeys, Object[] serviceDataValues,
-            int[] manufacturerDataKeys, Object[] manufacturerDataValues);
-
-    // Binds to BluetoothAdapterAndroid::nativeOnAdapterStateChanged
-    private native void nativeOnAdapterStateChanged(
-            long nativeBluetoothAdapterAndroid, boolean powered);
+        // Binds to BluetoothAdapterAndroid::nativeOnAdapterStateChanged
+        void onAdapterStateChanged(
+                long nativeBluetoothAdapterAndroid, ChromeBluetoothAdapter caller, boolean powered);
+    }
 }
