@@ -5,6 +5,7 @@
 package org.chromium.device.bluetooth;
 
 import android.Manifest;
+import android.annotation.SuppressLint;
 import android.annotation.TargetApi;
 import android.bluetooth.BluetoothAdapter;
 import android.bluetooth.BluetoothDevice;
@@ -22,15 +23,18 @@ import android.content.Context;
 import android.content.pm.PackageManager;
 import android.os.Build;
 import android.os.ParcelUuid;
-import android.os.Process;
+import android.util.SparseArray;
 
+import org.chromium.base.ContextUtils;
 import org.chromium.base.Log;
+import org.chromium.base.ThreadUtils;
 import org.chromium.base.annotations.CalledByNative;
 import org.chromium.base.annotations.JNINamespace;
 
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.UUID;
 
 /**
@@ -42,17 +46,65 @@ import java.util.UUID;
  * pass through to the Android object and instead provide fake implementations.
  */
 @JNINamespace("device")
-@TargetApi(Build.VERSION_CODES.LOLLIPOP)
+@TargetApi(Build.VERSION_CODES.M)
 class Wrappers {
     private static final String TAG = "Bluetooth";
 
+    public static final int DEVICE_CLASS_UNSPECIFIED = 0x1F00;
+
+    /**
+     * Wraps base.ThreadUtils.
+     * base.ThreadUtils has a set of static method to interact with the
+     * UI Thread. To be able to provide a set of test methods, ThreadUtilsWrapper
+     * uses the factory pattern.
+     */
+    static class ThreadUtilsWrapper {
+        private static Factory sFactory;
+
+        private static ThreadUtilsWrapper sInstance;
+
+        protected ThreadUtilsWrapper() {}
+
+        /**
+         * Returns the singleton instance of ThreadUtilsWrapper, creating it if needed.
+         */
+        public static ThreadUtilsWrapper getInstance() {
+            if (sInstance == null) {
+                if (sFactory == null) {
+                    sInstance = new ThreadUtilsWrapper();
+                } else {
+                    sInstance = sFactory.create();
+                }
+            }
+            return sInstance;
+        }
+
+        public void runOnUiThread(Runnable r) {
+            ThreadUtils.runOnUiThread(r);
+        }
+
+        /**
+         * Instantiate this to explain how to create a ThreadUtilsWrapper instance in
+         * ThreadUtilsWrapper.getInstance().
+         */
+        public interface Factory { public ThreadUtilsWrapper create(); }
+
+        /**
+         * Call this to use a different subclass of ThreadUtilsWrapper throughout the program.
+         */
+        public static void setFactory(Factory factory) {
+            sFactory = factory;
+            sInstance = null;
+        }
+    }
+
     /**
      * Wraps android.bluetooth.BluetoothAdapter.
      */
     static class BluetoothAdapterWrapper {
         private final BluetoothAdapter mAdapter;
-        protected final ContextWrapper mContext;
-        protected final BluetoothLeScannerWrapper mScanner;
+        protected final Context mContext;
+        protected BluetoothLeScannerWrapper mScannerWrapper;
 
         /**
          * Creates a BluetoothAdapterWrapper using the default
@@ -61,8 +113,8 @@ class Wrappers {
          * permissions.
          */
         @CalledByNative("BluetoothAdapterWrapper")
-        public static BluetoothAdapterWrapper createWithDefaultAdapter(Context context) {
-            final boolean hasMinAPI = Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP;
+        public static BluetoothAdapterWrapper createWithDefaultAdapter() {
+            final boolean hasMinAPI = Build.VERSION.SDK_INT >= Build.VERSION_CODES.M;
             if (!hasMinAPI) {
                 Log.i(TAG, "BluetoothAdapterWrapper.create failed: SDK version (%d) too low.",
                         Build.VERSION.SDK_INT);
@@ -70,9 +122,11 @@ class Wrappers {
             }
 
             final boolean hasPermissions =
-                    context.checkCallingOrSelfPermission(Manifest.permission.BLUETOOTH)
+                    ContextUtils.getApplicationContext().checkCallingOrSelfPermission(
+                            Manifest.permission.BLUETOOTH)
                             == PackageManager.PERMISSION_GRANTED
-                    && context.checkCallingOrSelfPermission(Manifest.permission.BLUETOOTH_ADMIN)
+                    && ContextUtils.getApplicationContext().checkCallingOrSelfPermission(
+                               Manifest.permission.BLUETOOTH_ADMIN)
                             == PackageManager.PERMISSION_GRANTED;
             if (!hasPermissions) {
                 Log.w(TAG, "BluetoothAdapterWrapper.create failed: Lacking Bluetooth permissions.");
@@ -82,7 +136,7 @@ class Wrappers {
             // Only Low Energy currently supported, see BluetoothAdapterAndroid class note.
             final boolean hasLowEnergyFeature =
                     Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2
-                    && context.getPackageManager().hasSystemFeature(
+                    && ContextUtils.getApplicationContext().getPackageManager().hasSystemFeature(
                                PackageManager.FEATURE_BLUETOOTH_LE);
             if (!hasLowEnergyFeature) {
                 Log.i(TAG, "BluetoothAdapterWrapper.create failed: No Low Energy support.");
@@ -94,32 +148,41 @@ class Wrappers {
                 Log.i(TAG, "BluetoothAdapterWrapper.create failed: Default adapter not found.");
                 return null;
             } else {
-                return new BluetoothAdapterWrapper(adapter, new ContextWrapper(context),
-                        new BluetoothLeScannerWrapper(adapter.getBluetoothLeScanner()));
+                return new BluetoothAdapterWrapper(adapter, ContextUtils.getApplicationContext());
             }
         }
 
-        public BluetoothAdapterWrapper(BluetoothAdapter adapter, ContextWrapper context,
-                BluetoothLeScannerWrapper scanner) {
+        public BluetoothAdapterWrapper(BluetoothAdapter adapter, Context context) {
             mAdapter = adapter;
             mContext = context;
-            mScanner = scanner;
         }
 
-        public ContextWrapper getContext() {
-            return mContext;
+        public boolean disable() {
+            return mAdapter.disable();
+        }
+
+        public boolean enable() {
+            return mAdapter.enable();
+        }
+
+        @SuppressLint("HardwareIds")
+        public String getAddress() {
+            return mAdapter.getAddress();
         }
 
         public BluetoothLeScannerWrapper getBluetoothLeScanner() {
-            return mScanner;
+            BluetoothLeScanner scanner = mAdapter.getBluetoothLeScanner();
+            if (scanner == null) {
+                return null;
+            }
+            if (mScannerWrapper == null) {
+                mScannerWrapper = new BluetoothLeScannerWrapper(scanner);
+            }
+            return mScannerWrapper;
         }
 
-        public boolean isEnabled() {
-            return mAdapter.isEnabled();
-        }
-
-        public String getAddress() {
-            return mAdapter.getAddress();
+        public Context getContext() {
+            return mContext;
         }
 
         public String getName() {
@@ -133,21 +196,9 @@ class Wrappers {
         public boolean isDiscovering() {
             return mAdapter.isDiscovering();
         }
-    }
 
-    /**
-     * Wraps android.content.Context.
-     */
-    static class ContextWrapper {
-        private final Context mContext;
-
-        public ContextWrapper(Context context) {
-            mContext = context;
-        }
-
-        public boolean checkPermission(String permission) {
-            return mContext.checkPermission(permission, Process.myPid(), Process.myUid())
-                    == PackageManager.PERMISSION_GRANTED;
+        public boolean isEnabled() {
+            return mAdapter.isEnabled();
         }
     }
 
@@ -155,7 +206,7 @@ class Wrappers {
      * Wraps android.bluetooth.BluetoothLeScanner.
      */
     static class BluetoothLeScannerWrapper {
-        private final BluetoothLeScanner mScanner;
+        protected final BluetoothLeScanner mScanner;
         private final HashMap<ScanCallbackWrapper, ForwardScanCallbackToWrapper> mCallbacks;
 
         public BluetoothLeScannerWrapper(BluetoothLeScanner scanner) {
@@ -240,9 +291,33 @@ class Wrappers {
             return new BluetoothDeviceWrapper(mScanResult.getDevice());
         }
 
+        public int getRssi() {
+            return mScanResult.getRssi();
+        }
+
         public List<ParcelUuid> getScanRecord_getServiceUuids() {
             return mScanResult.getScanRecord().getServiceUuids();
         }
+
+        public Map<ParcelUuid, byte[]> getScanRecord_getServiceData() {
+            return mScanResult.getScanRecord().getServiceData();
+        }
+
+        public SparseArray<byte[]> getScanRecord_getManufacturerSpecificData() {
+            return mScanResult.getScanRecord().getManufacturerSpecificData();
+        }
+
+        public int getScanRecord_getTxPowerLevel() {
+            return mScanResult.getScanRecord().getTxPowerLevel();
+        }
+
+        public String getScanRecord_getDeviceName() {
+            return mScanResult.getScanRecord().getDeviceName();
+        }
+
+        public int getScanRecord_getAdvertiseFlags() {
+            return mScanResult.getScanRecord().getAdvertiseFlags();
+        }
     }
 
     /**
@@ -263,11 +338,11 @@ class Wrappers {
                     new HashMap<BluetoothGattDescriptor, BluetoothGattDescriptorWrapper>();
         }
 
-        public BluetoothGattWrapper connectGatt(
-                Context context, boolean autoConnect, BluetoothGattCallbackWrapper callback) {
+        public BluetoothGattWrapper connectGatt(Context context, boolean autoConnect,
+                BluetoothGattCallbackWrapper callback, int transport) {
             return new BluetoothGattWrapper(
                     mDevice.connectGatt(context, autoConnect,
-                            new ForwardBluetoothGattCallbackToWrapper(callback, this)),
+                            new ForwardBluetoothGattCallbackToWrapper(callback, this), transport),
                     this);
         }
 
@@ -276,6 +351,11 @@ class Wrappers {
         }
 
         public int getBluetoothClass_getDeviceClass() {
+            if (mDevice == null || mDevice.getBluetoothClass() == null) {
+                // BluetoothDevice.getBluetoothClass() returns null if adapter has been powered off.
+                // Return DEVICE_CLASS_UNSPECIFIED in these cases.
+                return DEVICE_CLASS_UNSPECIFIED;
+            }
             return mDevice.getBluetoothClass().getDeviceClass();
         }
 
@@ -304,6 +384,10 @@ class Wrappers {
             mGatt.disconnect();
         }
 
+        public void close() {
+            mGatt.close();
+        }
+
         public void discoverServices() {
             mGatt.discoverServices();
         }
@@ -331,6 +415,10 @@ class Wrappers {
             return mGatt.writeCharacteristic(characteristic.mCharacteristic);
         }
 
+        boolean readDescriptor(BluetoothGattDescriptorWrapper descriptor) {
+            return mGatt.readDescriptor(descriptor.mDescriptor);
+        }
+
         boolean writeDescriptor(BluetoothGattDescriptorWrapper descriptor) {
             return mGatt.writeDescriptor(descriptor.mDescriptor);
         }
@@ -376,6 +464,20 @@ class Wrappers {
                     mDeviceWrapper.mCharacteristicsToWrappers.get(characteristic), status);
         }
 
+        @Override
+        public void onDescriptorRead(
+                BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
+            mWrapperCallback.onDescriptorRead(
+                    mDeviceWrapper.mDescriptorsToWrappers.get(descriptor), status);
+        }
+
+        @Override
+        public void onDescriptorWrite(
+                BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
+            mWrapperCallback.onDescriptorWrite(
+                    mDeviceWrapper.mDescriptorsToWrappers.get(descriptor), status);
+        }
+
         @Override
         public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
             mWrapperCallback.onConnectionStateChange(status, newState);
@@ -404,6 +506,10 @@ class Wrappers {
                 BluetoothGattCharacteristicWrapper characteristic, int status);
         public abstract void onCharacteristicWrite(
                 BluetoothGattCharacteristicWrapper characteristic, int status);
+        public abstract void onDescriptorRead(
+                BluetoothGattDescriptorWrapper descriptor, int status);
+        public abstract void onDescriptorWrite(
+                BluetoothGattDescriptorWrapper descriptor, int status);
         public abstract void onConnectionStateChange(int status, int newState);
         public abstract void onServicesDiscovered(int status);
     }
@@ -461,22 +567,6 @@ class Wrappers {
             mDeviceWrapper = deviceWrapper;
         }
 
-        public BluetoothGattDescriptorWrapper getDescriptor(UUID uuid) {
-            BluetoothGattDescriptor descriptor = mCharacteristic.getDescriptor(uuid);
-            if (descriptor == null) {
-                return null;
-            }
-
-            BluetoothGattDescriptorWrapper descriptorWrapper =
-                    mDeviceWrapper.mDescriptorsToWrappers.get(descriptor);
-
-            if (descriptorWrapper == null) {
-                descriptorWrapper = new BluetoothGattDescriptorWrapper(descriptor);
-                mDeviceWrapper.mDescriptorsToWrappers.put(descriptor, descriptorWrapper);
-            }
-            return descriptorWrapper;
-        }
-
         public List<BluetoothGattDescriptorWrapper> getDescriptors() {
             List<BluetoothGattDescriptor> descriptors = mCharacteristic.getDescriptors();
 
@@ -487,7 +577,8 @@ class Wrappers {
                 BluetoothGattDescriptorWrapper descriptorWrapper =
                         mDeviceWrapper.mDescriptorsToWrappers.get(descriptor);
                 if (descriptorWrapper == null) {
-                    descriptorWrapper = new BluetoothGattDescriptorWrapper(descriptor);
+                    descriptorWrapper =
+                            new BluetoothGattDescriptorWrapper(descriptor, mDeviceWrapper);
                     mDeviceWrapper.mDescriptorsToWrappers.put(descriptor, descriptorWrapper);
                 }
                 descriptorsWrapped.add(descriptorWrapper);
@@ -514,6 +605,10 @@ class Wrappers {
         public boolean setValue(byte[] value) {
             return mCharacteristic.setValue(value);
         }
+
+        public void setWriteType(int writeType) {
+            mCharacteristic.setWriteType(writeType);
+        }
     }
 
     /**
@@ -521,9 +616,16 @@ class Wrappers {
      */
     static class BluetoothGattDescriptorWrapper {
         private final BluetoothGattDescriptor mDescriptor;
+        final BluetoothDeviceWrapper mDeviceWrapper;
 
-        public BluetoothGattDescriptorWrapper(BluetoothGattDescriptor descriptor) {
+        public BluetoothGattDescriptorWrapper(
+                BluetoothGattDescriptor descriptor, BluetoothDeviceWrapper deviceWrapper) {
             mDescriptor = descriptor;
+            mDeviceWrapper = deviceWrapper;
+        }
+
+        public BluetoothGattCharacteristicWrapper getCharacteristic() {
+            return mDeviceWrapper.mCharacteristicsToWrappers.get(mDescriptor.getCharacteristic());
         }
 
         public UUID getUuid() {
