@@ -5,9 +5,13 @@
 #include "device/bluetooth/bluetooth_adapter.h"
 
 #include <memory>
+#include <string>
 #include <utility>
 
 #include "base/bind.h"
+#include "base/callback_helpers.h"
+#include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/single_thread_task_runner.h"
 #include "build/build_config.h"
@@ -18,26 +22,26 @@
 #include "device/bluetooth/bluetooth_remote_gatt_characteristic.h"
 #include "device/bluetooth/bluetooth_remote_gatt_descriptor.h"
 #include "device/bluetooth/bluetooth_remote_gatt_service.h"
+#include "device/bluetooth/public/cpp/bluetooth_address.h"
 
 namespace device {
 
 BluetoothAdapter::ServiceOptions::ServiceOptions() = default;
 BluetoothAdapter::ServiceOptions::~ServiceOptions() = default;
 
-#if !defined(OS_ANDROID) && !defined(OS_CHROMEOS) && !defined(OS_MACOSX) && \
+#if !defined(OS_ANDROID) && !defined(OS_CHROMEOS) && !defined(OS_MAC) && \
     !defined(OS_WIN) && !defined(OS_LINUX)
 // static
-base::WeakPtr<BluetoothAdapter> BluetoothAdapter::CreateAdapter(
-    InitCallback init_callback) {
-  return base::WeakPtr<BluetoothAdapter>();
+scoped_refptr<BluetoothAdapter> BluetoothAdapter::CreateAdapter() {
+  return nullptr;
 }
-#endif  // !defined(OS_CHROMEOS) && !defined(OS_WIN) && !defined(OS_MACOSX)
+#endif  // Not supported platforms.
 
 base::WeakPtr<BluetoothAdapter> BluetoothAdapter::GetWeakPtrForTesting() {
-  return weak_ptr_factory_.GetWeakPtr();
+  return GetWeakPtr();
 }
 
-#if defined(OS_LINUX)
+#if defined(OS_LINUX) || defined(OS_CHROMEOS)
 void BluetoothAdapter::Shutdown() {
   NOTIMPLEMENTED();
 }
@@ -53,6 +57,11 @@ void BluetoothAdapter::RemoveObserver(BluetoothAdapter::Observer* observer) {
   observers_.RemoveObserver(observer);
 }
 
+std::string BluetoothAdapter::GetSystemName() const {
+  NOTIMPLEMENTED();
+  return std::string();
+}
+
 bool BluetoothAdapter::HasObserver(BluetoothAdapter::Observer* observer) {
   DCHECK(observer);
   return observers_.HasObserver(observer);
@@ -62,30 +71,43 @@ bool BluetoothAdapter::CanPower() const {
   return IsPresent();
 }
 
+BluetoothAdapter::PermissionStatus BluetoothAdapter::GetOsPermissionStatus()
+    const {
+  // If this is not overridden that means OS permission is not
+  // required on this platform so act as though we already have
+  // permission.
+  return PermissionStatus::kAllowed;
+}
+
 void BluetoothAdapter::SetPowered(bool powered,
-                                  const base::Closure& callback,
-                                  const ErrorCallback& error_callback) {
+                                  base::OnceClosure callback,
+                                  ErrorCallback error_callback) {
   if (set_powered_callbacks_) {
     // Only allow one pending callback at a time.
-    ui_task_runner_->PostTask(FROM_HERE, error_callback);
+    ui_task_runner_->PostTask(FROM_HERE, std::move(error_callback));
     return;
   }
 
   if (powered == IsPowered()) {
     // Return early in case no change of power state is needed.
-    ui_task_runner_->PostTask(FROM_HERE, callback);
+    ui_task_runner_->PostTask(FROM_HERE, std::move(callback));
     return;
   }
 
   if (!SetPoweredImpl(powered)) {
-    ui_task_runner_->PostTask(FROM_HERE, error_callback);
+    ui_task_runner_->PostTask(FROM_HERE, std::move(error_callback));
     return;
   }
 
   set_powered_callbacks_ = std::make_unique<SetPoweredCallbacks>();
   set_powered_callbacks_->powered = powered;
-  set_powered_callbacks_->callback = callback;
-  set_powered_callbacks_->error_callback = error_callback;
+  set_powered_callbacks_->callback = std::move(callback);
+  set_powered_callbacks_->error_callback = std::move(error_callback);
+}
+
+bool BluetoothAdapter::IsPeripheralRoleSupported() const {
+  // TODO(crbug/1071595): Implement this for more platforms.
+  return true;
 }
 
 std::unordered_map<BluetoothDevice*, BluetoothDevice::UUIDSet>
@@ -94,44 +116,104 @@ BluetoothAdapter::RetrieveGattConnectedDevicesWithDiscoveryFilter(
   return std::unordered_map<BluetoothDevice*, BluetoothDevice::UUIDSet>();
 }
 
-void BluetoothAdapter::StartDiscoverySession(
-    const DiscoverySessionCallback& callback,
-    const ErrorCallback& error_callback) {
-  StartDiscoverySessionWithFilter(nullptr, callback, error_callback);
+void BluetoothAdapter::StartDiscoverySession(DiscoverySessionCallback callback,
+                                             ErrorCallback error_callback) {
+  StartDiscoverySessionWithFilter(nullptr, std::move(callback),
+                                  std::move(error_callback));
 }
 
 void BluetoothAdapter::StartDiscoverySessionWithFilter(
     std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-    const DiscoverySessionCallback& callback,
-    const ErrorCallback& error_callback) {
-  BluetoothDiscoveryFilter* ptr = discovery_filter.get();
-  AddDiscoverySession(
-      ptr, base::Bind(&BluetoothAdapter::OnStartDiscoverySession,
-                      weak_ptr_factory_.GetWeakPtr(),
-                      base::Passed(&discovery_filter), callback),
-      base::Bind(&BluetoothAdapter::OnStartDiscoverySessionError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+    DiscoverySessionCallback callback,
+    ErrorCallback error_callback) {
+  std::unique_ptr<BluetoothDiscoverySession> new_session(
+      new BluetoothDiscoverySession(this, std::move(discovery_filter)));
+  discovery_sessions_.insert(new_session.get());
+
+  auto new_session_callbacks = std::make_unique<StartOrStopDiscoveryCallback>(
+      base::BindOnce(std::move(callback), std::move(new_session)),
+      std::move(error_callback));
+
+  // Queue up the callbacks to be handled when we process the discovery queue.
+  discovery_callback_queue_.push(std::move(new_session_callbacks));
+
+  // If OS is already working on a discovery request we must wait to process the
+  // queue until that request is complete.
+  if (discovery_request_pending_) {
+    return;
+  }
+
+  // The OS is ready to start a request so process the queue now.
+  ProcessDiscoveryQueue();
+}
+
+void BluetoothAdapter::MaybeUpdateFilter(
+    std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
+    DiscoverySessionResultCallback callback) {
+  if (discovery_filter->Equals(current_discovery_filter_)) {
+    std::move(callback).Run(/*is_error=*/false,
+                            UMABluetoothDiscoverySessionOutcome::SUCCESS);
+    return;
+  }
+
+  UpdateFilter(std::move(discovery_filter), std::move(callback));
+}
+
+void BluetoothAdapter::RemoveDiscoverySession(
+    BluetoothDiscoverySession* discovery_session,
+    base::OnceClosure callback,
+    DiscoverySessionErrorCallback error_callback) {
+  size_t erased = discovery_sessions_.erase(discovery_session);
+  DCHECK_EQ(1u, erased);
+
+  auto removal_callbacks = std::make_unique<StartOrStopDiscoveryCallback>(
+      std::move(callback), std::move(error_callback));
+
+  // Queue up the callbacks to be handled when we process the discovery queue.
+  discovery_callback_queue_.push(std::move(removal_callbacks));
+
+  // If OS is already working on a discovery request we must wait to process the
+  // queue until that request is complete.
+  if (discovery_request_pending_) {
+    return;
+  }
+
+  // The OS is ready to start a request so process the queue now.
+  ProcessDiscoveryQueue();
 }
 
 std::unique_ptr<BluetoothDiscoveryFilter>
 BluetoothAdapter::GetMergedDiscoveryFilter() const {
-  return GetMergedDiscoveryFilterHelper(nullptr, false);
-}
+  auto result =
+      std::make_unique<BluetoothDiscoveryFilter>(BLUETOOTH_TRANSPORT_DUAL);
+  bool first_merge = true;
 
-std::unique_ptr<BluetoothDiscoveryFilter>
-BluetoothAdapter::GetMergedDiscoveryFilterMasked(
-    BluetoothDiscoveryFilter* masked_filter) const {
-  return GetMergedDiscoveryFilterHelper(masked_filter, true);
+  for (auto* iter : discovery_sessions_) {
+    if (!iter->IsActive())
+      continue;
+
+    const BluetoothDiscoveryFilter* curr_filter = iter->GetDiscoveryFilter();
+
+    if (first_merge) {
+      first_merge = false;
+      if (curr_filter) {
+        result->CopyFrom(*curr_filter);
+      }
+      continue;
+    }
+    result = BluetoothDiscoveryFilter::Merge(result.get(), curr_filter);
+  }
+  return result;
 }
 
 BluetoothAdapter::DeviceList BluetoothAdapter::GetDevices() {
   ConstDeviceList const_devices =
-    const_cast<const BluetoothAdapter *>(this)->GetDevices();
+      const_cast<const BluetoothAdapter*>(this)->GetDevices();
 
   DeviceList devices;
   for (ConstDeviceList::const_iterator i = const_devices.begin();
        i != const_devices.end(); ++i)
-    devices.push_back(const_cast<BluetoothDevice *>(*i));
+    devices.push_back(const_cast<BluetoothDevice*>(*i));
 
   return devices;
 }
@@ -145,14 +227,13 @@ BluetoothAdapter::ConstDeviceList BluetoothAdapter::GetDevices() const {
 }
 
 BluetoothDevice* BluetoothAdapter::GetDevice(const std::string& address) {
-  return const_cast<BluetoothDevice *>(
-      const_cast<const BluetoothAdapter *>(this)->GetDevice(address));
+  return const_cast<BluetoothDevice*>(
+      const_cast<const BluetoothAdapter*>(this)->GetDevice(address));
 }
 
 const BluetoothDevice* BluetoothAdapter::GetDevice(
     const std::string& address) const {
-  std::string canonicalized_address =
-      BluetoothDevice::CanonicalizeAddress(address);
+  std::string canonicalized_address = CanonicalizeBluetoothAddress(address);
   if (canonicalized_address.empty())
     return nullptr;
 
@@ -171,7 +252,7 @@ void BluetoothAdapter::AddPairingDelegate(
   RemovePairingDelegate(pairing_delegate);
 
   // Find the first point with a lower priority, or the end of the list.
-  std::list<PairingDelegatePair>::iterator iter = pairing_delegates_.begin();
+  auto iter = pairing_delegates_.begin();
   while (iter != pairing_delegates_.end() && iter->second >= priority)
     ++iter;
 
@@ -180,8 +261,8 @@ void BluetoothAdapter::AddPairingDelegate(
 
 void BluetoothAdapter::RemovePairingDelegate(
     BluetoothDevice::PairingDelegate* pairing_delegate) {
-  for (std::list<PairingDelegatePair>::iterator iter =
-       pairing_delegates_.begin(); iter != pairing_delegates_.end(); ++iter) {
+  for (auto iter = pairing_delegates_.begin(); iter != pairing_delegates_.end();
+       ++iter) {
     if (iter->first == pairing_delegate) {
       RemovePairingDelegateInternal(pairing_delegate);
       pairing_delegates_.erase(iter);
@@ -202,6 +283,11 @@ BluetoothAdapter::GetPendingAdvertisementsForTesting() const {
   return {};
 }
 
+void BluetoothAdapter::NotifyAdapterPresentChanged(bool present) {
+  for (auto& observer : observers_)
+    observer.AdapterPresentChanged(this, present);
+}
+
 void BluetoothAdapter::NotifyAdapterPoweredChanged(bool powered) {
   for (auto& observer : observers_)
     observer.AdapterPoweredChanged(this, powered);
@@ -215,6 +301,11 @@ void BluetoothAdapter::NotifyDeviceChanged(BluetoothDevice* device) {
     observer.DeviceChanged(this, device);
 }
 
+void BluetoothAdapter::NotifyAdapterDiscoveryChangeCompletedForTesting() {
+  for (auto& observer : observers_)
+    observer.DiscoveryChangeCompletedForTesting();
+}
+
 #if defined(OS_CHROMEOS) || defined(OS_LINUX)
 void BluetoothAdapter::NotifyDevicePairedChanged(BluetoothDevice* device,
                                                  bool new_paired_status) {
@@ -223,6 +314,15 @@ void BluetoothAdapter::NotifyDevicePairedChanged(BluetoothDevice* device,
 }
 #endif
 
+#if defined(OS_CHROMEOS) || defined(OS_LINUX)
+void BluetoothAdapter::NotifyDeviceBatteryChanged(BluetoothDevice* device) {
+  DCHECK_EQ(device->GetAdapter(), this);
+  for (auto& observer : observers_) {
+    observer.DeviceBatteryChanged(this, device, device->battery_percentage());
+  }
+}
+#endif
+
 void BluetoothAdapter::NotifyGattServiceAdded(
     BluetoothRemoteGattService* service) {
   DCHECK_EQ(service->GetDevice()->GetAdapter(), this);
@@ -247,6 +347,22 @@ void BluetoothAdapter::NotifyGattServiceChanged(
     observer.GattServiceChanged(this, service);
 }
 
+int BluetoothAdapter::NumDiscoverySessions() const {
+  return discovery_sessions_.size();
+}
+
+int BluetoothAdapter::NumScanningDiscoverySessions() const {
+  int count = 0;
+  for (auto* session : discovery_sessions_) {
+    if (session->status() ==
+        BluetoothDiscoverySession::SessionStatus::SCANNING) {
+      ++count;
+    }
+  }
+
+  return count;
+}
+
 void BluetoothAdapter::NotifyGattServicesDiscovered(BluetoothDevice* device) {
   DCHECK(device->GetAdapter() == this);
 
@@ -303,8 +419,13 @@ void BluetoothAdapter::NotifyGattCharacteristicValueChanged(
     const std::vector<uint8_t>& value) {
   DCHECK_EQ(characteristic->GetService()->GetDevice()->GetAdapter(), this);
 
-  for (auto& observer : observers_)
+  base::WeakPtr<BluetoothRemoteGattCharacteristic> weak_characteristic =
+      characteristic->GetWeakPtr();
+  for (auto& observer : observers_) {
+    if (!weak_characteristic)
+      break;
     observer.GattCharacteristicValueChanged(this, characteristic, value);
+  }
 }
 
 void BluetoothAdapter::NotifyGattDescriptorValueChanged(
@@ -321,13 +442,27 @@ void BluetoothAdapter::NotifyGattDescriptorValueChanged(
 BluetoothAdapter::SetPoweredCallbacks::SetPoweredCallbacks() = default;
 BluetoothAdapter::SetPoweredCallbacks::~SetPoweredCallbacks() = default;
 
-BluetoothAdapter::BluetoothAdapter() : weak_ptr_factory_(this) {}
+BluetoothAdapter::StartOrStopDiscoveryCallback::StartOrStopDiscoveryCallback(
+    base::OnceClosure start_callback,
+    ErrorCallback start_error_callback) {
+  this->start_callback = std::move(start_callback);
+  this->start_error_callback = std::move(start_error_callback);
+}
+BluetoothAdapter::StartOrStopDiscoveryCallback::StartOrStopDiscoveryCallback(
+    base::OnceClosure stop_callback,
+    DiscoverySessionErrorCallback stop_error_callback) {
+  this->stop_callback = std::move(stop_callback);
+  this->stop_error_callback = std::move(stop_error_callback);
+}
+BluetoothAdapter::StartOrStopDiscoveryCallback::
+    ~StartOrStopDiscoveryCallback() = default;
+
+BluetoothAdapter::BluetoothAdapter() {}
 
 BluetoothAdapter::~BluetoothAdapter() {
   // If there's a pending powered request, run its error callback.
-  if (set_powered_callbacks_) {
-    set_powered_callbacks_->error_callback.Run();
-  }
+  if (set_powered_callbacks_)
+    std::move(set_powered_callbacks_->error_callback).Run();
 }
 
 void BluetoothAdapter::RunPendingPowerCallbacks() {
@@ -336,93 +471,149 @@ void BluetoothAdapter::RunPendingPowerCallbacks() {
     // scope and to allow scheduling another SetPowered() call in either of the
     // callbacks.
     auto callbacks = std::move(set_powered_callbacks_);
-    callbacks->powered == IsPowered() ? std::move(callbacks->callback).Run()
-                                      : callbacks->error_callback.Run();
+    callbacks->powered == IsPowered()
+        ? std::move(callbacks->callback).Run()
+        : std::move(callbacks->error_callback).Run();
   }
 }
 
-void BluetoothAdapter::OnStartDiscoverySession(
-    std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-    const DiscoverySessionCallback& callback) {
-  VLOG(1) << "BluetoothAdapter::OnStartDiscoverySession";
-  RecordBluetoothDiscoverySessionStartOutcome(
-      UMABluetoothDiscoverySessionOutcome::SUCCESS);
+void BluetoothAdapter::OnDiscoveryChangeComplete(
+    bool is_error,
+    UMABluetoothDiscoverySessionOutcome outcome) {
+  UpdateDiscoveryState(is_error);
 
-  std::unique_ptr<BluetoothDiscoverySession> discovery_session(
-      new BluetoothDiscoverySession(scoped_refptr<BluetoothAdapter>(this),
-                                    std::move(discovery_filter)));
-  discovery_sessions_.insert(discovery_session.get());
-  callback.Run(std::move(discovery_session));
+  // Take a weak reference to |this| in case a callback frees the adapter.
+  base::WeakPtr<BluetoothAdapter> self = GetWeakPtr();
+
+  if (is_error) {
+    NotifyDiscoveryError(std::move(callbacks_awaiting_response_));
+
+    if (!self)
+      return;
+
+    discovery_request_pending_ = false;
+    NotifyAdapterDiscoveryChangeCompletedForTesting();
+    ProcessDiscoveryQueue();
+
+    return;
+  }
+
+  // Inform BluetoothDiscoverySession that updates being processed have
+  // completed.
+  for (auto* session : discovery_sessions_)
+    session->StartingSessionsScanning();
+
+  current_discovery_filter_.CopyFrom(filter_being_set_);
+
+  auto callbacks_awaiting_response = std::move(callbacks_awaiting_response_);
+  while (!callbacks_awaiting_response.empty()) {
+    std::unique_ptr<StartOrStopDiscoveryCallback> callbacks =
+        std::move(callbacks_awaiting_response.front());
+    callbacks_awaiting_response.pop();
+    if (callbacks->start_callback)
+      std::move(callbacks->start_callback).Run();
+    if (callbacks->stop_callback)
+      std::move(callbacks->stop_callback).Run();
+  }
+
+  if (!self)
+    return;
+
+  discovery_request_pending_ = false;
+  NotifyAdapterDiscoveryChangeCompletedForTesting();
+  ProcessDiscoveryQueue();
 }
 
-void BluetoothAdapter::OnStartDiscoverySessionError(
-    const ErrorCallback& callback,
-    UMABluetoothDiscoverySessionOutcome outcome) {
-  VLOG(1) << "OnStartDiscoverySessionError: " << static_cast<int>(outcome);
-  RecordBluetoothDiscoverySessionStartOutcome(outcome);
-  callback.Run();
+void BluetoothAdapter::UpdateDiscoveryState(bool is_error) {
+  if (is_error) {
+    if (internal_discovery_state_ == DiscoveryState::kStarting)
+      internal_discovery_state_ = DiscoveryState::kIdle;
+    // If there was an error stopping we still assume it worked as there is not
+    // much we can do about the device messing up.
+    if (internal_discovery_state_ == DiscoveryState::kStopping)
+      internal_discovery_state_ = DiscoveryState::kIdle;
+    return;
+  }
+
+  if (internal_discovery_state_ == DiscoveryState::kStarting)
+    internal_discovery_state_ = DiscoveryState::kDiscovering;
+  if (internal_discovery_state_ == DiscoveryState::kStopping)
+    internal_discovery_state_ = DiscoveryState::kIdle;
+}
+
+void BluetoothAdapter::ProcessDiscoveryQueue() {
+  if (discovery_callback_queue_.empty())
+    return;
+  DCHECK(callbacks_awaiting_response_.empty());
+  callbacks_awaiting_response_.swap(discovery_callback_queue_);
+
+  if (NumDiscoverySessions() == 0) {
+    if (internal_discovery_state_ == DiscoveryState::kIdle) {
+      OnDiscoveryChangeComplete(false,
+                                UMABluetoothDiscoverySessionOutcome::SUCCESS);
+      return;
+    }
+    internal_discovery_state_ = DiscoveryState::kStopping;
+    discovery_request_pending_ = true;
+    StopScan(base::BindOnce(&BluetoothAdapter::OnDiscoveryChangeComplete,
+                            GetWeakPtr()));
+
+    return;
+  }
+
+  // Inform BluetoothDiscoverySession that any updates they have made are being
+  // processed.
+  for (auto* session : discovery_sessions_)
+    session->PendingSessionsStarting();
+
+  auto result_callback = base::BindOnce(
+      &BluetoothAdapter::OnDiscoveryChangeComplete, GetWeakPtr());
+  auto new_desired_filter = GetMergedDiscoveryFilter();
+  discovery_request_pending_ = true;
+  filter_being_set_.CopyFrom(*new_desired_filter.get());
+  if (internal_discovery_state_ == DiscoveryState::kDiscovering) {
+    MaybeUpdateFilter(std::move(new_desired_filter),
+                      std::move(result_callback));
+    return;
+  }
+  internal_discovery_state_ = DiscoveryState::kStarting;
+  StartScanWithFilter(std::move(new_desired_filter),
+                      std::move(result_callback));
+}
+
+void BluetoothAdapter::NotifyDiscoveryError(CallbackQueue callback_queue) {
+  while (!callback_queue.empty()) {
+    std::unique_ptr<StartOrStopDiscoveryCallback> callbacks =
+        std::move(callback_queue.front());
+    callback_queue.pop();
+    if (callbacks->start_error_callback)
+      std::move(callbacks->start_error_callback).Run();
+    // We never return error when stopping. If the physical adapter is messing
+    // up and not stopping we are still just going to continue like it did stop.
+    if (callbacks->stop_callback)
+      std::move(callbacks->stop_callback).Run();
+  }
 }
 
 void BluetoothAdapter::MarkDiscoverySessionsAsInactive() {
+  // All sessions are becoming inactive so any pending requests should now fail
+  if (!discovery_callback_queue_.empty())
+    NotifyDiscoveryError(std::move(discovery_callback_queue_));
   // As sessions are marked as inactive they will notify the adapter that they
   // have become inactive, upon which the adapter will remove them from
   // |discovery_sessions_|. To avoid invalidating the iterator, make a copy
   // here.
   std::set<BluetoothDiscoverySession*> temp(discovery_sessions_);
-  for (std::set<BluetoothDiscoverySession*>::iterator
-          iter = temp.begin();
-       iter != temp.end(); ++iter) {
+  for (auto iter = temp.begin(); iter != temp.end(); ++iter) {
     (*iter)->MarkAsInactive();
+    RemoveDiscoverySession(*iter, base::DoNothing(), base::DoNothing());
   }
 }
 
-void BluetoothAdapter::DiscoverySessionBecameInactive(
-    BluetoothDiscoverySession* discovery_session) {
-  DCHECK(!discovery_session->IsActive());
-  discovery_sessions_.erase(discovery_session);
-}
-
 void BluetoothAdapter::DeleteDeviceForTesting(const std::string& address) {
   devices_.erase(address);
 }
 
-std::unique_ptr<BluetoothDiscoveryFilter>
-BluetoothAdapter::GetMergedDiscoveryFilterHelper(
-    const BluetoothDiscoveryFilter* masked_filter,
-    bool omit) const {
-  std::unique_ptr<BluetoothDiscoveryFilter> result;
-  bool first_merge = true;
-
-  std::set<BluetoothDiscoverySession*> temp(discovery_sessions_);
-  for (auto* iter : temp) {
-    const BluetoothDiscoveryFilter* curr_filter = iter->GetDiscoveryFilter();
-
-    if (!iter->IsActive())
-      continue;
-
-    if (omit && curr_filter == masked_filter) {
-      // if masked_filter is pointing to empty filter, and there are
-      // multiple empty filters in discovery_sessions_, make sure we'll
-      // process next empty sessions.
-      omit = false;
-      continue;
-    }
-
-    if (first_merge) {
-      first_merge = false;
-      if (curr_filter) {
-        result.reset(new BluetoothDiscoveryFilter(BLUETOOTH_TRANSPORT_DUAL));
-        result->CopyFrom(*curr_filter);
-      }
-      continue;
-    }
-
-    result = BluetoothDiscoveryFilter::Merge(result.get(), curr_filter);
-  }
-
-  return result;
-}
-
 void BluetoothAdapter::RemoveTimedOutDevices() {
   for (auto it = devices_.begin(); it != devices_.end();) {
     BluetoothDevice* device = it->second.get();
@@ -436,16 +627,16 @@ void BluetoothAdapter::RemoveTimedOutDevices() {
 
     bool device_expired =
         (base::Time::NowFromSystemTime() - last_update_time) > timeoutSec;
-    VLOG(3) << "device: " << device->GetAddress()
-            << ", last_update: " << last_update_time
-            << ", exp: " << device_expired;
+    DVLOG(3) << "device: " << device->GetAddress()
+             << ", last_update: " << last_update_time
+             << ", exp: " << device_expired;
 
     if (!device_expired) {
       ++it;
       continue;
     }
 
-    VLOG(1) << "Removing device: " << device->GetAddress();
+    DVLOG(1) << "Removing device: " << device->GetAddress();
     auto next = it;
     next++;
     std::unique_ptr<BluetoothDevice> removed_device = std::move(it->second);
@@ -457,22 +648,6 @@ void BluetoothAdapter::RemoveTimedOutDevices() {
   }
 }
 
-// static
-void BluetoothAdapter::RecordBluetoothDiscoverySessionStartOutcome(
-    UMABluetoothDiscoverySessionOutcome outcome) {
-  UMA_HISTOGRAM_ENUMERATION(
-      "Bluetooth.DiscoverySession.Start.Outcome", static_cast<int>(outcome),
-      static_cast<int>(UMABluetoothDiscoverySessionOutcome::COUNT));
-}
-
-// static
-void BluetoothAdapter::RecordBluetoothDiscoverySessionStopOutcome(
-    UMABluetoothDiscoverySessionOutcome outcome) {
-  UMA_HISTOGRAM_ENUMERATION(
-      "Bluetooth.DiscoverySession.Stop.Outcome", static_cast<int>(outcome),
-      static_cast<int>(UMABluetoothDiscoverySessionOutcome::COUNT));
-}
-
 // static
 constexpr base::TimeDelta BluetoothAdapter::timeoutSec =
     base::TimeDelta::FromSeconds(180);
