@@ -7,11 +7,12 @@
 
 #include <IOKit/IOReturn.h>
 
+#include <map>
 #include <memory>
 #include <string>
+#include <unordered_map>
 #include <vector>
 
-#include "base/containers/hash_tables.h"
 #include "base/mac/scoped_nsobject.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
@@ -23,8 +24,9 @@
 #include "device/bluetooth/bluetooth_export.h"
 #include "device/bluetooth/bluetooth_low_energy_advertisement_manager_mac.h"
 #include "device/bluetooth/bluetooth_low_energy_device_mac.h"
+#include "device/bluetooth/bluetooth_low_energy_device_watcher_mac.h"
 #include "device/bluetooth/bluetooth_low_energy_discovery_manager_mac.h"
-#include "device/bluetooth/bluetooth_uuid.h"
+#include "device/bluetooth/public/cpp/bluetooth_uuid.h"
 
 @class CBUUID;
 @class IOBluetoothDevice;
@@ -49,8 +51,8 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
       public BluetoothDiscoveryManagerMac::Observer,
       public BluetoothLowEnergyDiscoveryManagerMac::Observer {
  public:
-  static base::WeakPtr<BluetoothAdapterMac> CreateAdapter();
-  static base::WeakPtr<BluetoothAdapterMac> CreateAdapterForTest(
+  static scoped_refptr<BluetoothAdapterMac> CreateAdapter();
+  static scoped_refptr<BluetoothAdapterMac> CreateAdapterForTest(
       std::string name,
       std::string address,
       scoped_refptr<base::SingleThreadTaskRunner> ui_task_runner);
@@ -65,34 +67,33 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   std::string GetAddress() const override;
   std::string GetName() const override;
   void SetName(const std::string& name,
-               const base::Closure& callback,
-               const ErrorCallback& error_callback) override;
+               base::OnceClosure callback,
+               ErrorCallback error_callback) override;
   bool IsInitialized() const override;
   bool IsPresent() const override;
   bool IsPowered() const override;
+  PermissionStatus GetOsPermissionStatus() const override;
   bool IsDiscoverable() const override;
   void SetDiscoverable(bool discoverable,
-                       const base::Closure& callback,
-                       const ErrorCallback& error_callback) override;
+                       base::OnceClosure callback,
+                       ErrorCallback error_callback) override;
   bool IsDiscovering() const override;
   std::unordered_map<BluetoothDevice*, BluetoothDevice::UUIDSet>
   RetrieveGattConnectedDevicesWithDiscoveryFilter(
       const BluetoothDiscoveryFilter& discovery_filter) override;
   UUIDList GetUUIDs() const override;
-  void CreateRfcommService(
-      const BluetoothUUID& uuid,
-      const ServiceOptions& options,
-      const CreateServiceCallback& callback,
-      const CreateServiceErrorCallback& error_callback) override;
-  void CreateL2capService(
-      const BluetoothUUID& uuid,
-      const ServiceOptions& options,
-      const CreateServiceCallback& callback,
-      const CreateServiceErrorCallback& error_callback) override;
+  void CreateRfcommService(const BluetoothUUID& uuid,
+                           const ServiceOptions& options,
+                           CreateServiceCallback callback,
+                           CreateServiceErrorCallback error_callback) override;
+  void CreateL2capService(const BluetoothUUID& uuid,
+                          const ServiceOptions& options,
+                          CreateServiceCallback callback,
+                          CreateServiceErrorCallback error_callback) override;
   void RegisterAdvertisement(
       std::unique_ptr<BluetoothAdvertisement::Data> advertisement_data,
-      const CreateAdvertisementCallback& callback,
-      const AdvertisementErrorCallback& error_callback) override;
+      CreateAdvertisementCallback callback,
+      AdvertisementErrorCallback error_callback) override;
   BluetoothLocalGattService* GetGattService(
       const std::string& identifier) const override;
 
@@ -103,12 +104,6 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   // Registers that a new |device| has connected to the local host.
   void DeviceConnected(IOBluetoothDevice* device);
 
-  // We only use CoreBluetooth when OS X >= 10.10. This because the
-  // CBCentralManager destructor was found to crash on the mac_chromium_rel_ng
-  // builder running 10.9.5. May also cause blued to crash on OS X 10.9.5
-  // (crbug.com/506287).
-  static bool IsLowEnergyAvailable();
-
   // Creates a GATT connection by calling CoreBluetooth APIs.
   void CreateGattConnection(BluetoothLowEnergyDeviceMac* device_mac);
 
@@ -120,12 +115,22 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   void DidFailToConnectPeripheral(CBPeripheral* peripheral, NSError* error);
   void DidDisconnectPeripheral(CBPeripheral* peripheral, NSError* error);
 
+  bool IsBluetoothLowEnergyDeviceSystemPaired(
+      base::StringPiece device_identifier) const;
+
  protected:
+  using GetDevicePairedStatusCallback =
+      base::RepeatingCallback<bool(const std::string& address)>;
+
   // BluetoothAdapter override:
+  base::WeakPtr<BluetoothAdapter> GetWeakPtr() override;
   bool SetPoweredImpl(bool powered) override;
   void RemovePairingDelegateInternal(
       device::BluetoothDevice::PairingDelegate* pairing_delegate) override;
 
+  void UpdateKnownLowEnergyDevices(
+      std::map<std::string, std::string> updated_low_energy_device_info);
+
  private:
   // Struct bundling information about the state of the HostController.
   struct HostControllerState {
@@ -143,9 +148,17 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   // not know about bool.
   using SetControllerPowerStateFunction = base::RepeatingCallback<void(int)>;
 
+  // Performs initialization steps which touch the Bluetooth API and might
+  // trigger a Bluetooth permission prompt.
+  void LazyInitialize();
+
   // Queries the state of the IOBluetoothHostController.
   HostControllerState GetHostControllerState();
 
+  // Allows configuring whether the adapter is present when running in a test
+  // configuration.
+  void SetPresentForTesting(bool present);
+
   // Resets |low_energy_central_manager_| to |central_manager| and sets
   // |low_energy_central_manager_delegate_| as the manager's delegate. Should
   // be called only when |IsLowEnergyAvailable()|.
@@ -165,6 +178,14 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   void SetPowerStateFunctionForTesting(
       SetControllerPowerStateFunction power_state_function);
 
+  // Allow the mocking out of BluetoothLowEnergyDeviceWatcher for testing.
+  void SetLowEnergyDeviceWatcherForTesting(
+      scoped_refptr<BluetoothLowEnergyDeviceWatcherMac> watcher);
+
+  // Allow the mocking of out GetDevicePairedStatusCallback for testing.
+  void SetGetDevicePairedStatusCallbackForTesting(
+      GetDevicePairedStatusCallback callback);
+
   // The length of time that must elapse since the last Inquiry response (on
   // Classic devices) or call to BluetoothLowEnergyDevice::Update() (on Low
   // Energy) before a discovered device is considered to be no longer available.
@@ -179,24 +200,19 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   ~BluetoothAdapterMac() override;
 
   // BluetoothAdapter overrides:
-  void AddDiscoverySession(
-      BluetoothDiscoveryFilter* discovery_filter,
-      const base::Closure& callback,
-      DiscoverySessionErrorCallback error_callback) override;
-  void RemoveDiscoverySession(
-      BluetoothDiscoveryFilter* discovery_filter,
-      const base::Closure& callback,
-      DiscoverySessionErrorCallback error_callback) override;
-  void SetDiscoveryFilter(
+  void StartScanWithFilter(
       std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-      const base::Closure& callback,
-      DiscoverySessionErrorCallback error_callback) override;
+      DiscoverySessionResultCallback callback) override;
+  void UpdateFilter(
+      std::unique_ptr<device::BluetoothDiscoveryFilter> discovery_filter,
+      DiscoverySessionResultCallback callback) override;
+  void StopScan(DiscoverySessionResultCallback callback) override;
 
   // Start classic and/or low energy discovery sessions, according to the
   // filter.  If a discovery session is already running the filter is updated.
   bool StartDiscovery(BluetoothDiscoveryFilter* discovery_filter);
 
-  void Init();
+  void Initialize(base::OnceClosure callback) override;
   void InitForTest(scoped_refptr<base::SingleThreadTaskRunner> ui_task_runner);
   void PollAdapter();
 
@@ -238,9 +254,16 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   bool DoesCollideWithKnownDevice(CBPeripheral* peripheral,
                                   BluetoothLowEnergyDeviceMac* device_mac);
 
+  // The Initialize() method intentionally does not initialize
+  // |low_energy_central_manager_| or |low_energy_peripheral_manager_| because
+  // Chromium might not have permission to access the Bluetooth adapter.
+  // Methods which require these to be initialized must call LazyInitialize()
+  // first.
+  bool lazy_initialized_ = false;
+
   std::string address_;
-  bool classic_powered_;
-  int num_discovery_sessions_;
+  bool classic_powered_ = false;
+  base::Optional<bool> is_present_for_testing_;
 
   // Function returning the state of the HostController. Can be overridden for
   // tests.
@@ -261,7 +284,7 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   mutable std::string name_;
   // True if the name hasn't been acquired yet, the last acquired name is empty
   // or the address has changed indicating the name might have changed.
-  mutable bool should_update_name_;
+  mutable bool should_update_name_ = true;
 
   // Discovery manager for Bluetooth Classic.
   std::unique_ptr<BluetoothDiscoveryManagerMac> classic_discovery_manager_;
@@ -284,7 +307,18 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothAdapterMac
   base::scoped_nsobject<BluetoothLowEnergyPeripheralManagerDelegate>
       low_energy_peripheral_manager_delegate_;
 
-  base::WeakPtrFactory<BluetoothAdapterMac> weak_ptr_factory_;
+  GetDevicePairedStatusCallback device_paired_status_callback_;
+
+  // Watches system file /Library/Preferences/com.apple.Bluetooth.plist to
+  // obtain information about system paired bluetooth devices.
+  scoped_refptr<BluetoothLowEnergyDeviceWatcherMac>
+      bluetooth_low_energy_device_watcher_;
+
+  // Map of UUID formatted device identifiers of paired Bluetooth devices and
+  // corresponding device address.
+  std::map<std::string, std::string> low_energy_devices_info_;
+
+  base::WeakPtrFactory<BluetoothAdapterMac> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(BluetoothAdapterMac);
 };
