@@ -4,14 +4,18 @@
 
 #include "device/bluetooth/bluetooth_device.h"
 
+#include <array>
 #include <iterator>
 #include <memory>
 #include <string>
 #include <utility>
 
+#include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/stl_util.h"
+#include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
+#include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/time/time.h"
 #include "base/values.h"
@@ -20,6 +24,7 @@
 #include "device/bluetooth/bluetooth_remote_gatt_characteristic.h"
 #include "device/bluetooth/bluetooth_remote_gatt_descriptor.h"
 #include "device/bluetooth/bluetooth_remote_gatt_service.h"
+#include "device/bluetooth/public/cpp/bluetooth_uuid.h"
 #include "device/bluetooth/string_util_icu.h"
 #include "device/bluetooth/strings/grit/bluetooth_strings.h"
 #include "ui/base/l10n/l10n_util.h"
@@ -57,6 +62,15 @@ void BluetoothDevice::DeviceUUIDs::ReplaceServiceUUIDs(
   UpdateDeviceUUIDs();
 }
 
+void BluetoothDevice::DeviceUUIDs::ReplaceServiceUUIDs(
+    UUIDList new_service_uuids) {
+  service_uuids_.clear();
+  for (auto& it : new_service_uuids) {
+    service_uuids_.insert(std::move(it));
+  }
+  UpdateDeviceUUIDs();
+}
+
 void BluetoothDevice::DeviceUUIDs::ClearServiceUUIDs() {
   service_uuids_.clear();
   UpdateDeviceUUIDs();
@@ -87,8 +101,9 @@ BluetoothDevice::ConnectionInfo::ConnectionInfo()
       transmit_power(kUnknownPower),
       max_transmit_power(kUnknownPower) {}
 
-BluetoothDevice::ConnectionInfo::ConnectionInfo(
-    int rssi, int transmit_power, int max_transmit_power)
+BluetoothDevice::ConnectionInfo::ConnectionInfo(int rssi,
+                                                int transmit_power,
+                                                int max_transmit_power)
     : rssi(rssi),
       transmit_power(transmit_power),
       max_transmit_power(max_transmit_power) {}
@@ -341,15 +356,40 @@ base::Optional<int8_t> BluetoothDevice::GetInquiryTxPower() const {
 }
 
 void BluetoothDevice::CreateGattConnection(
-    const GattConnectionCallback& callback,
-    const ConnectErrorCallback& error_callback) {
-  create_gatt_connection_success_callbacks_.push_back(callback);
-  create_gatt_connection_error_callbacks_.push_back(error_callback);
+    GattConnectionCallback callback,
+    ConnectErrorCallback error_callback,
+    base::Optional<BluetoothUUID> service_uuid) {
+  if (!supports_service_specific_discovery_)
+    service_uuid.reset();
 
-  if (IsGattConnected())
+  const bool connection_already_pending =
+      !create_gatt_connection_success_callbacks_.empty();
+
+  create_gatt_connection_success_callbacks_.push_back(std::move(callback));
+  create_gatt_connection_error_callbacks_.push_back(std::move(error_callback));
+
+  // If a service-specific discovery was originally requested, but this request
+  // is for a different or non-specific discovery, then the previous discovery
+  // needs to be redone.
+  if (target_service_.has_value() && target_service_ != service_uuid) {
+    DCHECK(IsGattConnected() || connection_already_pending);
+    target_service_ = service_uuid;
+    UpgradeToFullDiscovery();
+  }
+
+  if (IsGattConnected()) {
+    DCHECK(!connection_already_pending);
     return DidConnectGatt();
+  }
 
-  CreateGattConnectionImpl();
+  if (connection_already_pending) {
+    // The correct callback will be run when the existing connection attempt
+    // completes.
+    return;
+  }
+
+  target_service_ = service_uuid;
+  CreateGattConnectionImpl(std::move(service_uuid));
 }
 
 void BluetoothDevice::SetGattServicesDiscoveryComplete(bool complete) {
@@ -357,7 +397,7 @@ void BluetoothDevice::SetGattServicesDiscoveryComplete(bool complete) {
 }
 
 bool BluetoothDevice::IsGattServicesDiscoveryComplete() const {
-  return gatt_services_discovery_complete_;
+  return !target_service_ && gatt_services_discovery_complete_;
 }
 
 std::vector<BluetoothRemoteGattService*> BluetoothDevice::GetGattServices()
@@ -376,43 +416,10 @@ BluetoothRemoteGattService* BluetoothDevice::GetGattService(
   return it->second.get();
 }
 
-// static
-std::string BluetoothDevice::CanonicalizeAddress(const std::string& address) {
-  std::string canonicalized = address;
-  if (address.size() == 12) {
-    // Might be an address in the format "1A2B3C4D5E6F". Add separators.
-    for (size_t i = 2; i < canonicalized.size(); i += 3) {
-      canonicalized.insert(i, ":");
-    }
-  }
-
-  // Verify that the length matches the canonical format "1A:2B:3C:4D:5E:6F".
-  const size_t kCanonicalAddressLength = 17;
-  if (canonicalized.size() != kCanonicalAddressLength)
-    return std::string();
-
-  const char separator = canonicalized[2];
-  for (size_t i = 0; i < canonicalized.size(); ++i) {
-    bool is_separator = (i + 1) % 3 == 0;
-    if (is_separator) {
-      // All separators in the input |address| should be consistent.
-      if (canonicalized[i] != separator)
-        return std::string();
-
-      canonicalized[i] = ':';
-    } else {
-      if (!base::IsHexDigit(canonicalized[i]))
-        return std::string();
-
-      canonicalized[i] = base::ToUpperASCII(canonicalized[i]);
-    }
-  }
-
-  return canonicalized;
+std::string BluetoothDevice::GetIdentifier() const {
+  return GetAddress();
 }
 
-std::string BluetoothDevice::GetIdentifier() const { return GetAddress(); }
-
 void BluetoothDevice::UpdateAdvertisementData(
     int8_t rssi,
     base::Optional<uint8_t> flags,
@@ -442,9 +449,9 @@ void BluetoothDevice::ClearAdvertisementData() {
 
 std::vector<BluetoothRemoteGattService*> BluetoothDevice::GetPrimaryServices() {
   std::vector<BluetoothRemoteGattService*> services;
-  VLOG(2) << "Looking for services.";
+  DVLOG(2) << "Looking for services.";
   for (BluetoothRemoteGattService* service : GetGattServices()) {
-    VLOG(2) << "Service in cache: " << service->GetUUID().canonical_value();
+    DVLOG(2) << "Service in cache: " << service->GetUUID().canonical_value();
     if (service->IsPrimary()) {
       services.push_back(service);
     }
@@ -455,9 +462,9 @@ std::vector<BluetoothRemoteGattService*> BluetoothDevice::GetPrimaryServices() {
 std::vector<BluetoothRemoteGattService*>
 BluetoothDevice::GetPrimaryServicesByUUID(const BluetoothUUID& service_uuid) {
   std::vector<BluetoothRemoteGattService*> services;
-  VLOG(2) << "Looking for service: " << service_uuid.canonical_value();
+  DVLOG(2) << "Looking for service: " << service_uuid.canonical_value();
   for (BluetoothRemoteGattService* service : GetGattServices()) {
-    VLOG(2) << "Service in cache: " << service->GetUUID().canonical_value();
+    DVLOG(2) << "Service in cache: " << service->GetUUID().canonical_value();
     if (service->GetUUID() == service_uuid && service->IsPrimary()) {
       services.push_back(service);
     }
@@ -465,11 +472,39 @@ BluetoothDevice::GetPrimaryServicesByUUID(const BluetoothUUID& service_uuid) {
   return services;
 }
 
+#if defined(OS_CHROMEOS) || defined(OS_LINUX)
+void BluetoothDevice::SetBatteryPercentage(
+    base::Optional<uint8_t> battery_percentage) {
+  if (battery_percentage)
+    DCHECK_LE(battery_percentage.value(), 100);
+
+  if (battery_percentage_ == battery_percentage)
+    return;
+
+  battery_percentage_ = battery_percentage;
+  GetAdapter()->NotifyDeviceBatteryChanged(this);
+}
+#endif
+
+bool BluetoothDevice::supports_service_specific_discovery() const {
+  return supports_service_specific_discovery_;
+}
+
+void BluetoothDevice::UpgradeToFullDiscovery() {
+  // Must be overridden by any subclass that sets
+  // |supports_service_specific_discovery_|.
+  NOTREACHED();
+}
+
+std::unique_ptr<BluetoothGattConnection>
+BluetoothDevice::CreateBluetoothGattConnectionObject() {
+  return std::make_unique<BluetoothGattConnection>(adapter_, GetAddress());
+}
+
 void BluetoothDevice::DidConnectGatt() {
-  for (const auto& callback : create_gatt_connection_success_callbacks_) {
-    callback.Run(
-        std::make_unique<BluetoothGattConnection>(adapter_, GetAddress()));
-  }
+  for (auto& callback : create_gatt_connection_success_callbacks_)
+    std::move(callback).Run(CreateBluetoothGattConnectionObject());
+
   create_gatt_connection_success_callbacks_.clear();
   create_gatt_connection_error_callbacks_.clear();
   GetAdapter()->NotifyDeviceChanged(this);
@@ -480,8 +515,10 @@ void BluetoothDevice::DidFailToConnectGatt(ConnectErrorCode error) {
   // connections.
   DCHECK(gatt_connections_.empty());
 
-  for (const auto& error_callback : create_gatt_connection_error_callbacks_)
-    error_callback.Run(error);
+  target_service_.reset();
+
+  for (auto& error_callback : create_gatt_connection_error_callbacks_)
+    std::move(error_callback).Run(error);
   create_gatt_connection_success_callbacks_.clear();
   create_gatt_connection_error_callbacks_.clear();
 }
@@ -491,6 +528,8 @@ void BluetoothDevice::DidDisconnectGatt() {
   // DidFailToConnectGatt should have been called.
   DCHECK(create_gatt_connection_error_callbacks_.empty());
 
+  target_service_.reset();
+
   // Invalidate all BluetoothGattConnection objects.
   for (BluetoothGattConnection* connection : gatt_connections_) {
     connection->InvalidateConnectionReference();
@@ -519,8 +558,8 @@ void BluetoothDevice::SetAsExpiredForTesting() {
 }
 
 void BluetoothDevice::Pair(PairingDelegate* pairing_delegate,
-                           const base::Closure& callback,
-                           const ConnectErrorCallback& error_callback) {
+                           base::OnceClosure callback,
+                           ConnectErrorCallback error_callback) {
   NOTREACHED();
 }
 
