@@ -16,24 +16,27 @@
 #include <vector>
 
 #include "base/callback.h"
+#include "base/containers/flat_set.h"
+#include "base/containers/span.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/optional.h"
 #include "base/strings/string16.h"
+#include "base/strings/string_piece_forward.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
 #include "device/bluetooth/bluetooth_common.h"
 #include "device/bluetooth/bluetooth_export.h"
 #include "device/bluetooth/bluetooth_remote_gatt_service.h"
-#include "device/bluetooth/bluetooth_uuid.h"
+#include "device/bluetooth/public/cpp/bluetooth_uuid.h"
 
 namespace device {
 
 class BluetoothAdapter;
 class BluetoothGattConnection;
 class BluetoothSocket;
-class BluetoothUUID;
 
 // BluetoothDevice represents a remote Bluetooth device, both its properties and
 // capabilities as discovered by a local adapter and actions that may be
@@ -58,6 +61,13 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
     VENDOR_ID_MAX_VALUE = VENDOR_ID_USB
   };
 
+  // Possible values that may be returned by GetAddressType().
+  enum AddressType {
+    ADDR_TYPE_UNKNOWN,
+    ADDR_TYPE_PUBLIC,
+    ADDR_TYPE_RANDOM,
+  };
+
   // The value returned if the RSSI or transmit power cannot be read.
   static const int kUnknownPower = 127;
   // The value returned if the appearance is not present.
@@ -95,7 +105,7 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   };
 
   typedef std::vector<BluetoothUUID> UUIDList;
-  typedef std::unordered_set<BluetoothUUID, BluetoothUUIDHash> UUIDSet;
+  typedef base::flat_set<BluetoothUUID> UUIDSet;
   typedef std::unordered_map<BluetoothUUID,
                              std::vector<uint8_t>,
                              BluetoothUUIDHash>
@@ -204,7 +214,7 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // and metrics logging,
   virtual uint32_t GetBluetoothClass() const = 0;
 
-#if defined(OS_LINUX)
+#if defined(OS_LINUX) || defined(OS_CHROMEOS)
   // Returns the transport type of the device. Some devices only support one
   // of BR/EDR or LE, and some support both.
   virtual BluetoothTransport GetType() const = 0;
@@ -217,6 +227,10 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // a unique key to identify the device and copied where needed.
   virtual std::string GetAddress() const = 0;
 
+  // Returns the Bluetooth address type of the device. Currently available on
+  // Linux and Chrome OS.
+  virtual AddressType GetAddressType() const = 0;
+
   // Returns the allocation source of the identifier returned by GetVendorID(),
   // where available, or VENDOR_ID_UNKNOWN where not.
   virtual VendorIDSource GetVendorIDSource() const = 0;
@@ -251,7 +265,7 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // of, by decoding the bluetooth class information for Classic devices or
   // by decoding the device's appearance for LE devices. For example,
   // Microsoft Universal Foldable Keyboard only advertises the appearance.
-  BluetoothDeviceType GetDeviceType() const;
+  virtual BluetoothDeviceType GetDeviceType() const;
 
   // Indicates whether the device is known to support pairing based on its
   // device class and address.
@@ -284,14 +298,16 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // Returns the set of UUIDs that this device supports.
   //  * For classic Bluetooth devices this data is collected from both the EIR
   //    data and SDP tables.
-  //  * For non-connected Low Energy Devices this returns the latest advertised
-  //    UUIDs.
-  //  * For connected Low Energy Devices for which services have not been
-  //    discovered returns an empty list.
+  //  * For non-connected and connected Low Energy Devices for which services
+  //    have not been discovered returns the latest advertised UUIDs.
   //  * For connected Low Energy Devices for which services have been discovered
-  //    returns the UUIDs of the device's services.
+  //    returns the UUIDs of the device's services and the latest advertised
+  //    UUIDs.
   //  * For dual mode devices this may be collected from both.
   //
+  // Note: On Android, Mac and WinRT advertised UUIDs are cleared when the
+  // adapter stops discovering, as otherwise stale data might be returned.
+  //
   // Note: On ChromeOS and Linux, BlueZ persists all services meaning if
   // a device stops advertising a service this function will still return
   // its UUID.
@@ -354,22 +370,22 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // Returns Advertising Data Flags.
   // Returns cached value if the adapter is not discovering.
   //
-  // TODO(crbug.com/661814) Support this on platforms that don't use BlueZ.
-  // Only Chrome OS supports this now. Upstream BlueZ has this feature
+  // Only Chrome OS and WinRT support this now. Upstream BlueZ has this feature
   // as experimental. This method returns base::nullopt on platforms that don't
   // support this feature.
   base::Optional<uint8_t> GetAdvertisingDataFlags() const;
 
   // The ErrorCallback is used for methods that can fail in which case it
   // is called, in the success case the callback is simply not called.
-  typedef base::Callback<void()> ErrorCallback;
+  using ErrorCallback = base::OnceClosure;
 
   // The ConnectErrorCallback is used for methods that can fail with an error,
   // passed back as an error code argument to this callback.
   // In the success case this callback is not called.
-  typedef base::Callback<void(enum ConnectErrorCode)> ConnectErrorCallback;
+  using ConnectErrorCallback = base::OnceCallback<void(enum ConnectErrorCode)>;
 
-  typedef base::Callback<void(const ConnectionInfo&)> ConnectionInfoCallback;
+  using ConnectionInfoCallback =
+      base::OnceCallback<void(const ConnectionInfo&)>;
 
   // Indicates whether the device is currently pairing and expecting a
   // PIN Code to be returned.
@@ -396,13 +412,13 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   //
   // If the device isn't connected, then the ConnectionInfo struct passed into
   // the callback will be populated with |kUnknownPower|.
-  virtual void GetConnectionInfo(const ConnectionInfoCallback& callback) = 0;
+  virtual void GetConnectionInfo(ConnectionInfoCallback callback) = 0;
 
   // Sets the connection latency for the device. This API is only valid for LE
   // devices.
   virtual void SetConnectionLatency(ConnectionLatency connection_latency,
-                                    const base::Closure& callback,
-                                    const ErrorCallback& error_callback) = 0;
+                                    base::OnceClosure callback,
+                                    ErrorCallback error_callback) = 0;
 
   // Initiates a connection to the device, pairing first if necessary.
   //
@@ -419,17 +435,18 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // process and release the pairing delegate if user cancels the pairing and
   // closes the pairing UI.
   virtual void Connect(PairingDelegate* pairing_delegate,
-                       const base::Closure& callback,
-                       const ConnectErrorCallback& error_callback) = 0;
+                       base::OnceClosure callback,
+                       ConnectErrorCallback error_callback) = 0;
 
   // Pairs the device. This method triggers pairing unconditially, i.e. it
   // ignores the |IsPaired()| value.
   //
   // In most cases |Connect()| should be preferred. This method is only
-  // implemented on ChromeOS and Linux.
+  // implemented on ChromeOS, Linux and Windows 10. On Windows, only pairing
+  // with a pin code is currently supported.
   virtual void Pair(PairingDelegate* pairing_delegate,
-                    const base::Closure& callback,
-                    const ConnectErrorCallback& error_callback);
+                    base::OnceClosure callback,
+                    ConnectErrorCallback error_callback);
 
   // Sends the PIN code |pincode| to the remote device during pairing.
   //
@@ -460,8 +477,8 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // information are not discarded, and the device object is not deleted.
   // If the request fails, |error_callback| will be called; otherwise,
   // |callback| is called when the request is complete.
-  virtual void Disconnect(const base::Closure& callback,
-                          const ErrorCallback& error_callback) = 0;
+  virtual void Disconnect(base::OnceClosure callback,
+                          ErrorCallback error_callback) = 0;
 
   // Disconnects the device, terminating the low-level ACL connection
   // and any application connections using it, and then discards link keys
@@ -470,8 +487,8 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // have been deleted. If the request fails, |error_callback| will be called.
   // On success |callback| will be invoked, but note that the BluetoothDevice
   // object will have been deleted at that point.
-  virtual void Forget(const base::Closure& callback,
-                      const ErrorCallback& error_callback) = 0;
+  virtual void Forget(base::OnceClosure callback,
+                      ErrorCallback error_callback) = 0;
 
   // Attempts to initiate an outgoing L2CAP or RFCOMM connection to the
   // advertised service on this device matching |uuid|, performing an SDP lookup
@@ -480,14 +497,14 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // BluetoothSocket instance that is to be owned by the receiver.
   // |error_callback| will be called on failure with a message indicating the
   // cause.
-  typedef base::Callback<void(scoped_refptr<BluetoothSocket>)>
-      ConnectToServiceCallback;
-  typedef base::Callback<void(const std::string& message)>
-      ConnectToServiceErrorCallback;
+  using ConnectToServiceCallback =
+      base::OnceCallback<void(scoped_refptr<BluetoothSocket>)>;
+  using ConnectToServiceErrorCallback =
+      base::OnceCallback<void(const std::string& message)>;
   virtual void ConnectToService(
       const BluetoothUUID& uuid,
-      const ConnectToServiceCallback& callback,
-      const ConnectToServiceErrorCallback& error_callback) = 0;
+      ConnectToServiceCallback callback,
+      ConnectToServiceErrorCallback error_callback) = 0;
 
   // Attempts to initiate an insecure outgoing L2CAP or RFCOMM connection to the
   // advertised service on this device matching |uuid|, performing an SDP lookup
@@ -498,9 +515,9 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // by the receiver. |error_callback| will be called on failure with a message
   // indicating the cause.
   virtual void ConnectToServiceInsecurely(
-    const device::BluetoothUUID& uuid,
-    const ConnectToServiceCallback& callback,
-    const ConnectToServiceErrorCallback& error_callback) = 0;
+      const device::BluetoothUUID& uuid,
+      ConnectToServiceCallback callback,
+      ConnectToServiceErrorCallback error_callback) = 0;
 
   // Opens a new GATT connection to this device. On success, a new
   // BluetoothGattConnection will be handed to the caller via |callback|. On
@@ -511,15 +528,28 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // returned BluetoothGattConnection will be automatically marked as inactive.
   // To monitor the state of the connection, observe the
   // BluetoothAdapter::Observer::DeviceChanged method.
-  typedef base::Callback<void(std::unique_ptr<BluetoothGattConnection>)>
-      GattConnectionCallback;
-  virtual void CreateGattConnection(const GattConnectionCallback& callback,
-                                    const ConnectErrorCallback& error_callback);
+  //
+  // If |service_uuid| is given, potentially only the service with the given
+  // UUID will be discovered. This may speed up GATT discovery times if the
+  // platform can take advantage of this optimisation. Note that passing
+  // |service_uuid| may cause full GATT service discovery to be skipped. In that
+  // case, |IsGattServicesDiscoveryComplete| will not become true but
+  // |BluetoothAdapter::Observer::GattServicesDiscovered| is still the correct
+  // event to watch for.
+  using GattConnectionCallback =
+      base::OnceCallback<void(std::unique_ptr<BluetoothGattConnection>)>;
+  virtual void CreateGattConnection(
+      GattConnectionCallback callback,
+      ConnectErrorCallback error_callback,
+      base::Optional<BluetoothUUID> service_uuid = base::nullopt);
 
   // Set the gatt services discovery complete flag for this device.
   virtual void SetGattServicesDiscoveryComplete(bool complete);
 
-  // Indicates whether service discovery is complete for this device.
+  // Indicates whether full service discovery is complete for this device. If a
+  // |service_uuid| was passed to |CreateGattConnection| and a focused discovery
+  // was performed, then this will continue to return false even after discovery
+  // is complete.
   virtual bool IsGattServicesDiscoveryComplete() const;
 
   // Returns the list of discovered GATT services.
@@ -530,11 +560,6 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   virtual BluetoothRemoteGattService* GetGattService(
       const std::string& identifier) const;
 
-  // Returns the |address| in the canonical format: XX:XX:XX:XX:XX:XX, where
-  // each 'X' is a hex digit.  If the input |address| is invalid, returns an
-  // empty string.
-  static std::string CanonicalizeAddress(const std::string& address);
-
   // Update the last time this device was seen.
   void UpdateTimestamp();
 
@@ -543,12 +568,15 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   virtual base::Time GetLastUpdateTime() const;
 
   // Called by BluetoothAdapter when a new Advertisement is seen for this
-  // device. This replaces previously seen Advertisement Data.
+  // device. This replaces previously seen Advertisement Data. The order of
+  // arguments matches the order of their corresponding Data Type specified in
+  // https://www.bluetooth.com/specifications/assigned-numbers/generic-access-profile.
   void UpdateAdvertisementData(int8_t rssi,
+                               base::Optional<uint8_t> flags,
                                UUIDList advertised_uuids,
+                               base::Optional<int8_t> tx_power,
                                ServiceDataMap service_data,
-                               ManufacturerDataMap manufacturer_data,
-                               const int8_t* tx_power);
+                               ManufacturerDataMap manufacturer_data);
 
   // Called by BluetoothAdapter when it stops discoverying.
   void ClearAdvertisementData();
@@ -561,6 +589,38 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   std::vector<BluetoothRemoteGattService*> GetPrimaryServicesByUUID(
       const BluetoothUUID& service_uuid);
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  using ExecuteWriteErrorCallback =
+      base::OnceCallback<void(device::BluetoothGattService::GattErrorCode)>;
+  using AbortWriteErrorCallback =
+      base::OnceCallback<void(device::BluetoothGattService::GattErrorCode)>;
+  // Executes all the previous prepare writes in a reliable write session.
+  virtual void ExecuteWrite(base::OnceClosure callback,
+                            ExecuteWriteErrorCallback error_callback) = 0;
+  // Aborts all the previous prepare writes in a reliable write session.
+  virtual void AbortWrite(base::OnceClosure callback,
+                          AbortWriteErrorCallback error_callback) = 0;
+#endif
+
+#if defined(OS_CHROMEOS) || defined(OS_LINUX)
+  // Set the remaining battery of the device to show in the UI. This value must
+  // be between 0 and 100, inclusive.
+  // Only device::BluetoothAdapterBlueZ has control over this field with the
+  // value originating from a single source, the BlueZ Battery API.
+  void SetBatteryPercentage(base::Optional<uint8_t> battery_percentage);
+
+  // Returns the remaining battery for the device.
+  const base::Optional<uint8_t>& battery_percentage() const {
+    return battery_percentage_;
+  }
+#endif
+
+  // Returns whether this device supports discovering specific services, i.e.
+  // whether the |service_uuid| argument to |CreateGattConnection| is
+  // meaningful. This should only be called by tests. Non-test code should
+  // optimistically pass a |service_uuid| argument if appropriate for the need.
+  bool supports_service_specific_discovery() const;
+
  protected:
   // BluetoothGattConnection is a friend to call Add/RemoveGattConnection.
   friend BluetoothGattConnection;
@@ -578,9 +638,20 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   FRIEND_TEST_ALL_PREFIXES(BluetoothTest, RemoveOutdatedDevices);
   FRIEND_TEST_ALL_PREFIXES(BluetoothTest, RemoveOutdatedDeviceGattConnect);
 
+  FRIEND_TEST_ALL_PREFIXES(
+      BluetoothTestWinrtOnly,
+      BluetoothGattConnection_DisconnectGatt_SimulateConnect);
+  FRIEND_TEST_ALL_PREFIXES(
+      BluetoothTestWinrtOnly,
+      BluetoothGattConnection_DisconnectGatt_SimulateDisconnect);
+  FRIEND_TEST_ALL_PREFIXES(BluetoothTestWinrtOnly,
+                           BluetoothGattConnection_ErrorAfterConnection);
+  FRIEND_TEST_ALL_PREFIXES(BluetoothTestWinrtOnly,
+                           BluetoothGattConnection_DisconnectGatt_Cleanup);
+
   // Helper class to easily update the sets of UUIDs and keep them in sync with
   // the set of all the device's UUIDs.
-  class DeviceUUIDs {
+  class DEVICE_BLUETOOTH_EXPORT DeviceUUIDs {
    public:
     DeviceUUIDs();
     ~DeviceUUIDs();
@@ -597,6 +668,8 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
     void ReplaceServiceUUIDs(
         const BluetoothDevice::GattServiceMap& gatt_services);
 
+    void ReplaceServiceUUIDs(UUIDList new_service_uuids);
+
     void ClearServiceUUIDs();
 
     // Returns the union of Advertised UUIDs and Service UUIDs.
@@ -616,12 +689,24 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // Subclasses must also call DidConnectGatt, DidFailToConnectGatt, or
   // DidDisconnectGatt immediately or asynchronously as the connection state
   // changes.
-  virtual void CreateGattConnectionImpl() = 0;
+  virtual void CreateGattConnectionImpl(
+      base::Optional<BluetoothUUID> service_uuid) = 0;
+
+  // UpgradeToFullDiscovery is called when there is a pending or current GATT
+  // connection that was created with a service UUID, but now discovery of all
+  // services is required because of a new connection request. This will only
+  // be called if the subclass sets |supports_service_specific_discovery_|.
+  virtual void UpgradeToFullDiscovery();
 
   // Disconnects GATT connection on platforms that maintain a specific GATT
   // connection.
   virtual void DisconnectGatt() = 0;
 
+  // Returns a |BluetoothGattConnection| object that represents a reference to a
+  // GATT connection to this device.
+  virtual std::unique_ptr<BluetoothGattConnection>
+  CreateBluetoothGattConnectionObject();
+
   // Calls any pending callbacks for CreateGattConnection based on result of
   // subclasses actions initiated in CreateGattConnectionImpl or related
   // disconnection events. These may be called at any time, even multiple times,
@@ -645,7 +730,16 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
 
   // Raw pointer to adapter owning this device object. Subclasses use platform
   // specific pointers via adapter_.
-  BluetoothAdapter* adapter_;
+  BluetoothAdapter* const adapter_;
+
+  // Indicates whether this device supports limited discovery of a specific
+  // service. This is configured by the constructor of subclasses. If false,
+  // the UUID argument to |CreateGattConnection| is ignored.
+  bool supports_service_specific_discovery_ = false;
+
+  // Contains the specified service that was targeted for discovery. Only ever
+  // contains a value if |supports_service_specific_discovery_| is true.
+  base::Optional<BluetoothUUID> target_service_;
 
   // Callbacks for pending success and error result of CreateGattConnection.
   std::vector<GattConnectionCallback> create_gatt_connection_success_callbacks_;
@@ -660,12 +754,12 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // Received Signal Strength Indicator of the advertisement received.
   base::Optional<int8_t> inquiry_rssi_;
 
-  // Tx Power advertised by the device.
-  base::Optional<int8_t> inquiry_tx_power_;
-
   // Advertising Data flags of the device.
   base::Optional<uint8_t> advertising_data_flags_;
 
+  // Tx Power advertised by the device.
+  base::Optional<int8_t> inquiry_tx_power_;
+
   // Class that holds the union of Advertised UUIDs and Service UUIDs.
   DeviceUUIDs device_uuids_;
 
@@ -683,6 +777,14 @@ class DEVICE_BLUETOOTH_EXPORT BluetoothDevice {
   // a device type for display when |name_| is empty.
   base::string16 GetAddressWithLocalizedDeviceTypeName() const;
 
+#if defined(OS_CHROMEOS) || defined(OS_LINUX)
+  // Remaining battery level of the device.
+  // TODO(https://crbug.com/973237): This field is different from others because
+  // it is not filled by the platform. In the future, when there is a unified
+  // Mojo service, this field will be moved to BluetoothDeviceInfo.
+  base::Optional<uint8_t> battery_percentage_;
+#endif
+
   DISALLOW_COPY_AND_ASSIGN(BluetoothDevice);
 };
 
