@@ -4,17 +4,22 @@
 
 #include "device/bluetooth/bluetooth_remote_gatt_characteristic.h"
 
+#include <utility>
+
 #include "base/bind.h"
+#include "base/callback_helpers.h"
 #include "base/location.h"
+#include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/single_thread_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
+#include "build/chromeos_buildflags.h"
 #include "device/bluetooth/bluetooth_gatt_notify_session.h"
 #include "device/bluetooth/bluetooth_remote_gatt_descriptor.h"
 
 namespace device {
 
-BluetoothRemoteGattCharacteristic::BluetoothRemoteGattCharacteristic()
-    : weak_ptr_factory_(this) {}
+BluetoothRemoteGattCharacteristic::BluetoothRemoteGattCharacteristic() {}
 
 BluetoothRemoteGattCharacteristic::~BluetoothRemoteGattCharacteristic() {
   while (!pending_notify_commands_.empty()) {
@@ -22,15 +27,30 @@ BluetoothRemoteGattCharacteristic::~BluetoothRemoteGattCharacteristic() {
   }
 }
 
+std::vector<BluetoothRemoteGattDescriptor*>
+BluetoothRemoteGattCharacteristic::GetDescriptors() const {
+  std::vector<BluetoothRemoteGattDescriptor*> descriptors;
+  descriptors.reserve(descriptors_.size());
+  for (const auto& pair : descriptors_)
+    descriptors.push_back(pair.second.get());
+  return descriptors;
+}
+
+BluetoothRemoteGattDescriptor* BluetoothRemoteGattCharacteristic::GetDescriptor(
+    const std::string& identifier) const {
+  auto iter = descriptors_.find(identifier);
+  return iter != descriptors_.end() ? iter->second.get() : nullptr;
+}
+
 std::vector<BluetoothRemoteGattDescriptor*>
 BluetoothRemoteGattCharacteristic::GetDescriptorsByUUID(
     const BluetoothUUID& uuid) const {
   std::vector<BluetoothRemoteGattDescriptor*> descriptors;
-  for (BluetoothRemoteGattDescriptor* descriptor : GetDescriptors()) {
-    if (descriptor->GetUUID() == uuid) {
-      descriptors.push_back(descriptor);
-    }
+  for (const auto& pair : descriptors_) {
+    if (pair.second->GetUUID() == uuid)
+      descriptors.push_back(pair.second.get());
   }
+
   return descriptors;
 }
 
@@ -44,48 +64,86 @@ bool BluetoothRemoteGattCharacteristic::IsNotifying() const {
 }
 
 BluetoothRemoteGattCharacteristic::NotifySessionCommand::NotifySessionCommand(
-    const ExecuteCallback& execute_callback,
-    const base::Closure& cancel_callback)
-    : execute_callback_(execute_callback), cancel_callback_(cancel_callback) {}
+    ExecuteCallback execute_callback,
+    base::OnceClosure cancel_callback)
+    : execute_callback_(std::move(execute_callback)),
+      cancel_callback_(std::move(cancel_callback)) {}
 
 BluetoothRemoteGattCharacteristic::NotifySessionCommand::
     ~NotifySessionCommand() = default;
 
 void BluetoothRemoteGattCharacteristic::NotifySessionCommand::Execute() {
-  execute_callback_.Run(COMMAND_NONE, RESULT_SUCCESS,
-                        BluetoothRemoteGattService::GATT_ERROR_UNKNOWN);
+  std::move(execute_callback_)
+      .Run(COMMAND_NONE, RESULT_SUCCESS,
+           BluetoothRemoteGattService::GATT_ERROR_UNKNOWN);
 }
 
 void BluetoothRemoteGattCharacteristic::NotifySessionCommand::Execute(
     Type previous_command_type,
     Result previous_command_result,
     BluetoothRemoteGattService::GattErrorCode previous_command_error_code) {
-  execute_callback_.Run(previous_command_type, previous_command_result,
-                        previous_command_error_code);
+  std::move(execute_callback_)
+      .Run(previous_command_type, previous_command_result,
+           previous_command_error_code);
 }
 
 void BluetoothRemoteGattCharacteristic::NotifySessionCommand::Cancel() {
-  cancel_callback_.Run();
+  std::move(cancel_callback_).Run();
 }
 
 void BluetoothRemoteGattCharacteristic::StartNotifySession(
-    const NotifySessionCallback& callback,
-    const ErrorCallback& error_callback) {
-  NotifySessionCommand* command = new NotifySessionCommand(
-      base::Bind(&BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession,
-                 GetWeakPtr(), callback, error_callback),
-      base::Bind(&BluetoothRemoteGattCharacteristic::CancelStartNotifySession,
-                 GetWeakPtr(),
-                 base::Bind(error_callback,
-                            BluetoothRemoteGattService::GATT_ERROR_FAILED)));
+    NotifySessionCallback callback,
+    ErrorCallback error_callback) {
+  StartNotifySessionInternal(base::nullopt, std::move(callback),
+                             std::move(error_callback));
+}
 
-  pending_notify_commands_.push(std::unique_ptr<NotifySessionCommand>(command));
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+void BluetoothRemoteGattCharacteristic::StartNotifySession(
+    NotificationType notification_type,
+    NotifySessionCallback callback,
+    ErrorCallback error_callback) {
+  StartNotifySessionInternal(notification_type, std::move(callback),
+                             std::move(error_callback));
+}
+#endif
+
+bool BluetoothRemoteGattCharacteristic::AddDescriptor(
+    std::unique_ptr<BluetoothRemoteGattDescriptor> descriptor) {
+  if (!descriptor)
+    return false;
+
+  auto* descriptor_raw = descriptor.get();
+  return descriptors_
+      .try_emplace(descriptor_raw->GetIdentifier(), std::move(descriptor))
+      .second;
+}
+
+void BluetoothRemoteGattCharacteristic::StartNotifySessionInternal(
+    const base::Optional<NotificationType>& notification_type,
+    NotifySessionCallback callback,
+    ErrorCallback error_callback) {
+  auto repeating_error_callback =
+      base::AdaptCallbackForRepeating(std::move(error_callback));
+  NotifySessionCommand* command = new NotifySessionCommand(
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession,
+          GetWeakPtr(), notification_type, std::move(callback),
+          repeating_error_callback),
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::CancelStartNotifySession,
+          GetWeakPtr(),
+          base::BindOnce(repeating_error_callback,
+                         BluetoothRemoteGattService::GATT_ERROR_FAILED)));
+
+  pending_notify_commands_.push(base::WrapUnique(command));
   if (pending_notify_commands_.size() == 1) {
     command->Execute();
   }
 }
 
 void BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession(
+    const base::Optional<NotificationType>& notification_type,
     NotifySessionCallback callback,
     ErrorCallback error_callback,
     NotifySessionCommand::Type previous_command_type,
@@ -97,33 +155,32 @@ void BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession(
     if (previous_command_result == NotifySessionCommand::RESULT_SUCCESS) {
       base::ThreadTaskRunnerHandle::Get()->PostTask(
           FROM_HERE,
-          base::Bind(
+          base::BindOnce(
               &BluetoothRemoteGattCharacteristic::OnStartNotifySessionSuccess,
-              GetWeakPtr(), callback));
+              GetWeakPtr(), std::move(callback)));
       return;
     } else {
       base::ThreadTaskRunnerHandle::Get()->PostTask(
           FROM_HERE,
-          base::Bind(
+          base::BindOnce(
               &BluetoothRemoteGattCharacteristic::OnStartNotifySessionError,
-              GetWeakPtr(), error_callback, previous_command_error_code));
+              GetWeakPtr(), std::move(error_callback),
+              previous_command_error_code));
       return;
     }
   }
 
-  // Check that the characteristic supports either notifications or
-  // indications.
-  Properties properties = GetProperties();
-  bool hasNotify = (properties & PROPERTY_NOTIFY) != 0;
-  bool hasIndicate = (properties & PROPERTY_INDICATE) != 0;
-
-  if (!hasNotify && !hasIndicate) {
-    LOG(ERROR) << "Characteristic needs NOTIFY or INDICATE";
+  if (!IsNotificationTypeSupported(notification_type)) {
+    if (notification_type)
+      LOG(ERROR) << "Characteristic doesn't support specified "
+                 << "notification_type";
+    else
+      LOG(ERROR) << "Characteristic needs NOTIFY or INDICATE";
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(
+        base::BindOnce(
             &BluetoothRemoteGattCharacteristic::OnStartNotifySessionError,
-            GetWeakPtr(), error_callback,
+            GetWeakPtr(), std::move(error_callback),
             BluetoothRemoteGattService::GATT_ERROR_NOT_SUPPORTED));
     return;
   }
@@ -134,9 +191,9 @@ void BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession(
   if (IsNotifying()) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(
+        base::BindOnce(
             &BluetoothRemoteGattCharacteristic::OnStartNotifySessionSuccess,
-            GetWeakPtr(), callback));
+            GetWeakPtr(), std::move(callback)));
     return;
   }
 
@@ -150,9 +207,9 @@ void BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession(
                << " client characteristic configuration descriptors.";
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(
+        base::BindOnce(
             &BluetoothRemoteGattCharacteristic::OnStartNotifySessionError,
-            GetWeakPtr(), error_callback,
+            GetWeakPtr(), std::move(error_callback),
             (ccc_descriptor.size() == 0)
                 ? BluetoothRemoteGattService::GATT_ERROR_NOT_SUPPORTED
                 : BluetoothRemoteGattService::GATT_ERROR_FAILED));
@@ -164,19 +221,25 @@ void BluetoothRemoteGattCharacteristic::ExecuteStartNotifySession(
   // do whatever else is needed to get the notifications flowing.
   SubscribeToNotifications(
       ccc_descriptor[0],
-      base::Bind(
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+      notification_type.value_or((GetProperties() & PROPERTY_NOTIFY)
+                                     ? NotificationType::kNotification
+                                     : NotificationType::kIndication),
+#endif
+      base::BindOnce(
           &BluetoothRemoteGattCharacteristic::OnStartNotifySessionSuccess,
-          GetWeakPtr(), callback),
-      base::Bind(&BluetoothRemoteGattCharacteristic::OnStartNotifySessionError,
-                 GetWeakPtr(), error_callback));
+          GetWeakPtr(), std::move(callback)),
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::OnStartNotifySessionError,
+          GetWeakPtr(), std::move(error_callback)));
 }
 
 void BluetoothRemoteGattCharacteristic::CancelStartNotifySession(
-    base::Closure callback) {
+    base::OnceClosure callback) {
   std::unique_ptr<NotifySessionCommand> command =
       std::move(pending_notify_commands_.front());
   pending_notify_commands_.pop();
-  callback.Run();
+  std::move(callback).Run();
 }
 
 void BluetoothRemoteGattCharacteristic::OnStartNotifySessionSuccess(
@@ -187,7 +250,7 @@ void BluetoothRemoteGattCharacteristic::OnStartNotifySessionSuccess(
   std::unique_ptr<device::BluetoothGattNotifySession> notify_session(
       new BluetoothGattNotifySession(weak_ptr_factory_.GetWeakPtr()));
   notify_sessions_.insert(notify_session.get());
-  callback.Run(std::move(notify_session));
+  std::move(callback).Run(std::move(notify_session));
 
   pending_notify_commands_.pop();
   if (!pending_notify_commands_.empty()) {
@@ -204,7 +267,7 @@ void BluetoothRemoteGattCharacteristic::OnStartNotifySessionError(
   std::unique_ptr<NotifySessionCommand> command =
       std::move(pending_notify_commands_.front());
 
-  error_callback.Run(error);
+  std::move(error_callback).Run(error);
 
   pending_notify_commands_.pop();
   if (!pending_notify_commands_.empty()) {
@@ -216,12 +279,16 @@ void BluetoothRemoteGattCharacteristic::OnStartNotifySessionError(
 
 void BluetoothRemoteGattCharacteristic::StopNotifySession(
     BluetoothGattNotifySession* session,
-    const base::Closure& callback) {
+    base::OnceClosure callback) {
+  auto repeating_callback =
+      base::AdaptCallbackForRepeating(std::move(callback));
   NotifySessionCommand* command = new NotifySessionCommand(
-      base::Bind(&BluetoothRemoteGattCharacteristic::ExecuteStopNotifySession,
-                 GetWeakPtr(), session, callback),
-      base::Bind(&BluetoothRemoteGattCharacteristic::CancelStopNotifySession,
-                 GetWeakPtr(), callback));
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::ExecuteStopNotifySession,
+          GetWeakPtr(), session, repeating_callback),
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::CancelStopNotifySession,
+          GetWeakPtr(), repeating_callback));
 
   pending_notify_commands_.push(std::unique_ptr<NotifySessionCommand>(command));
   if (pending_notify_commands_.size() == 1) {
@@ -231,21 +298,21 @@ void BluetoothRemoteGattCharacteristic::StopNotifySession(
 
 void BluetoothRemoteGattCharacteristic::ExecuteStopNotifySession(
     BluetoothGattNotifySession* session,
-    base::Closure callback,
+    base::OnceClosure callback,
     NotifySessionCommand::Type previous_command_type,
     NotifySessionCommand::Result previous_command_result,
     BluetoothRemoteGattService::GattErrorCode previous_command_error_code) {
-  std::set<BluetoothGattNotifySession*>::iterator session_iterator =
-      notify_sessions_.find(session);
+  auto session_iterator = notify_sessions_.find(session);
 
   // If the session does not even belong to this characteristic, we return an
   // error right away.
   if (session_iterator == notify_sessions_.end()) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(&BluetoothRemoteGattCharacteristic::OnStopNotifySessionError,
-                   GetWeakPtr(), session, callback,
-                   BluetoothRemoteGattService::GATT_ERROR_FAILED));
+        base::BindOnce(
+            &BluetoothRemoteGattCharacteristic::OnStopNotifySessionError,
+            GetWeakPtr(), session, std::move(callback),
+            BluetoothRemoteGattService::GATT_ERROR_FAILED));
     return;
   }
 
@@ -253,9 +320,9 @@ void BluetoothRemoteGattCharacteristic::ExecuteStopNotifySession(
   if (notify_sessions_.size() > 1) {
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(
+        base::BindOnce(
             &BluetoothRemoteGattCharacteristic::OnStopNotifySessionSuccess,
-            GetWeakPtr(), session, callback));
+            GetWeakPtr(), session, std::move(callback)));
     return;
   }
 
@@ -269,37 +336,42 @@ void BluetoothRemoteGattCharacteristic::ExecuteStopNotifySession(
                << " client characteristic configuration descriptors.";
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(&BluetoothRemoteGattCharacteristic::OnStopNotifySessionError,
-                   GetWeakPtr(), session, callback,
-                   BluetoothRemoteGattService::GATT_ERROR_FAILED));
+        base::BindOnce(
+            &BluetoothRemoteGattCharacteristic::OnStopNotifySessionError,
+            GetWeakPtr(), session, std::move(callback),
+            BluetoothRemoteGattService::GATT_ERROR_FAILED));
     return;
   }
 
+  auto repeating_callback =
+      base::AdaptCallbackForRepeating(std::move(callback));
   UnsubscribeFromNotifications(
       ccc_descriptor[0],
-      base::Bind(&BluetoothRemoteGattCharacteristic::OnStopNotifySessionSuccess,
-                 GetWeakPtr(), session, callback),
-      base::Bind(&BluetoothRemoteGattCharacteristic::OnStopNotifySessionError,
-                 GetWeakPtr(), session, callback));
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::OnStopNotifySessionSuccess,
+          GetWeakPtr(), session, repeating_callback),
+      base::BindOnce(
+          &BluetoothRemoteGattCharacteristic::OnStopNotifySessionError,
+          GetWeakPtr(), session, repeating_callback));
 }
 
 void BluetoothRemoteGattCharacteristic::CancelStopNotifySession(
-    base::Closure callback) {
+    base::OnceClosure callback) {
   std::unique_ptr<NotifySessionCommand> command =
       std::move(pending_notify_commands_.front());
   pending_notify_commands_.pop();
-  callback.Run();
+  std::move(callback).Run();
 }
 
 void BluetoothRemoteGattCharacteristic::OnStopNotifySessionSuccess(
     BluetoothGattNotifySession* session,
-    base::Closure callback) {
+    base::OnceClosure callback) {
   std::unique_ptr<NotifySessionCommand> command =
       std::move(pending_notify_commands_.front());
 
   notify_sessions_.erase(session);
 
-  callback.Run();
+  std::move(callback).Run();
 
   pending_notify_commands_.pop();
   if (!pending_notify_commands_.empty()) {
@@ -312,14 +384,14 @@ void BluetoothRemoteGattCharacteristic::OnStopNotifySessionSuccess(
 
 void BluetoothRemoteGattCharacteristic::OnStopNotifySessionError(
     BluetoothGattNotifySession* session,
-    base::Closure callback,
+    base::OnceClosure callback,
     BluetoothRemoteGattService::GattErrorCode error) {
   std::unique_ptr<NotifySessionCommand> command =
       std::move(pending_notify_commands_.front());
 
   notify_sessions_.erase(session);
 
-  callback.Run();
+  std::move(callback).Run();
 
   pending_notify_commands_.pop();
   if (!pending_notify_commands_.empty()) {
@@ -329,4 +401,19 @@ void BluetoothRemoteGattCharacteristic::OnStopNotifySessionError(
   }
 }
 
+bool BluetoothRemoteGattCharacteristic::IsNotificationTypeSupported(
+    const base::Optional<NotificationType>& notification_type) {
+  Properties properties = GetProperties();
+  bool hasNotify = (properties & PROPERTY_NOTIFY) != 0;
+  bool hasIndicate = (properties & PROPERTY_INDICATE) != 0;
+  if (!notification_type)
+    return hasNotify || hasIndicate;
+  switch (notification_type.value()) {
+    case NotificationType::kNotification:
+      return hasNotify;
+    case NotificationType::kIndication:
+      return hasIndicate;
+  }
+}
+
 }  // namespace device
