@@ -11,16 +11,16 @@
 #include <utility>
 
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
 #include "base/memory/ptr_util.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
+#include "build/chromeos_buildflags.h"
 #include "components/device_event_log/device_event_log.h"
 #include "dbus/bus.h"
 #include "device/bluetooth/bluetooth_socket.h"
 #include "device/bluetooth/bluetooth_socket_thread.h"
-#include "device/bluetooth/bluetooth_uuid.h"
 #include "device/bluetooth/bluez/bluetooth_adapter_bluez.h"
 #include "device/bluetooth/bluez/bluetooth_gatt_connection_bluez.h"
 #include "device/bluetooth/bluez/bluetooth_pairing_bluez.h"
@@ -32,6 +32,8 @@
 #include "device/bluetooth/dbus/bluetooth_gatt_service_client.h"
 #include "device/bluetooth/dbus/bluetooth_input_client.h"
 #include "device/bluetooth/dbus/bluez_dbus_manager.h"
+#include "device/bluetooth/public/cpp/bluetooth_address.h"
+#include "device/bluetooth/public/cpp/bluetooth_uuid.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 
 using device::BluetoothDevice;
@@ -99,38 +101,6 @@ void ParseModalias(const dbus::ObjectPath& object_path,
     *device_id = device_value;
 }
 
-void RecordPairingResult(BluetoothDevice::ConnectErrorCode error_code) {
-  UMAPairingResult pairing_result;
-  switch (error_code) {
-    case BluetoothDevice::ERROR_INPROGRESS:
-      pairing_result = UMA_PAIRING_RESULT_INPROGRESS;
-      break;
-    case BluetoothDevice::ERROR_FAILED:
-      pairing_result = UMA_PAIRING_RESULT_FAILED;
-      break;
-    case BluetoothDevice::ERROR_AUTH_FAILED:
-      pairing_result = UMA_PAIRING_RESULT_AUTH_FAILED;
-      break;
-    case BluetoothDevice::ERROR_AUTH_CANCELED:
-      pairing_result = UMA_PAIRING_RESULT_AUTH_CANCELED;
-      break;
-    case BluetoothDevice::ERROR_AUTH_REJECTED:
-      pairing_result = UMA_PAIRING_RESULT_AUTH_REJECTED;
-      break;
-    case BluetoothDevice::ERROR_AUTH_TIMEOUT:
-      pairing_result = UMA_PAIRING_RESULT_AUTH_TIMEOUT;
-      break;
-    case BluetoothDevice::ERROR_UNSUPPORTED_DEVICE:
-      pairing_result = UMA_PAIRING_RESULT_UNSUPPORTED_DEVICE;
-      break;
-    default:
-      pairing_result = UMA_PAIRING_RESULT_UNKNOWN_ERROR;
-  }
-
-  UMA_HISTOGRAM_ENUMERATION("Bluetooth.PairingResult", pairing_result,
-                            UMA_PAIRING_RESULT_COUNT);
-}
-
 BluetoothDevice::ConnectErrorCode DBusErrorToConnectError(
     const std::string& error_name) {
   BluetoothDevice::ConnectErrorCode error_code = BluetoothDevice::ERROR_UNKNOWN;
@@ -163,8 +133,7 @@ BluetoothDeviceBlueZ::BluetoothDeviceBlueZ(
       object_path_(object_path),
       num_connecting_calls_(0),
       ui_task_runner_(ui_task_runner),
-      socket_thread_(socket_thread),
-      weak_ptr_factory_(this) {
+      socket_thread_(socket_thread) {
   bluez::BluezDBusManager::Get()->GetBluetoothGattServiceClient()->AddObserver(
       this);
 
@@ -184,6 +153,7 @@ BluetoothDeviceBlueZ::BluetoothDeviceBlueZ(
   UpdateServiceData();
   UpdateManufacturerData();
   UpdateAdvertisingDataFlags();
+  UpdateServiceUUIDs();
 }
 
 BluetoothDeviceBlueZ::~BluetoothDeviceBlueZ() {
@@ -200,26 +170,6 @@ BluetoothDeviceBlueZ::~BluetoothDeviceBlueZ() {
     adapter()->NotifyGattServiceRemoved(
         static_cast<BluetoothRemoteGattServiceBlueZ*>(iter.second.get()));
   }
-
-  // We pause discovery when trying to connect. Ensure discovery is unpaused if
-  // we get destroyed during a pending connection.
-  if (IsConnecting()) {
-    BLUETOOTH_LOG(EVENT) << object_path_.value()
-                         << ": Unpausing discovery. Device removed.";
-    // Temporarily unpause discovery manually instead of using
-    // UnpauseDiscovery() which was introduced after branch point.
-    // TODO(ortuno): Remove once this is merged to M70.
-    bluez::BluezDBusManager::Get()
-        ->GetBluetoothAdapterClient()
-        ->UnpauseDiscovery(
-            adapter()->object_path(), base::Bind([]() {
-              BLUETOOTH_LOG(EVENT) << "Successfully un-paused discovery";
-            }),
-            base::Bind([](const std::string& error_name,
-                          const std::string& error_message) {
-              BLUETOOTH_LOG(EVENT) << "Failed to un-pause discovery";
-            }));
-  }
 }
 
 uint32_t BluetoothDeviceBlueZ::GetBluetoothClass() const {
@@ -253,10 +203,36 @@ device::BluetoothTransport BluetoothDeviceBlueZ::GetType() const {
   return device::BLUETOOTH_TRANSPORT_INVALID;
 }
 
-void BluetoothDeviceBlueZ::CreateGattConnectionImpl() {
-  // BlueZ implementation does not use the default CreateGattConnection
-  // implementation.
-  NOTIMPLEMENTED();
+void BluetoothDeviceBlueZ::CreateGattConnectionImpl(
+    base::Optional<BluetoothUUID> service_uuid) {
+// Once ConnectLE is supported on Linux, this buildflag will not be necessary
+// (this bluez code is only run on Chrome OS and Linux).
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  if (num_connecting_calls_++ == 0)
+    adapter()->NotifyDeviceChanged(this);
+
+  auto success_callback = base::BindOnce(
+      &BluetoothDeviceBlueZ::OnConnect, weak_ptr_factory_.GetWeakPtr(),
+      base::BindOnce(&BluetoothDeviceBlueZ::DidConnectGatt,
+                     weak_ptr_factory_.GetWeakPtr()));
+
+  auto error_callback = base::BindOnce(
+      &BluetoothDeviceBlueZ::OnConnectError, weak_ptr_factory_.GetWeakPtr(),
+      base::BindOnce(&BluetoothDeviceBlueZ::DidFailToConnectGatt,
+                     weak_ptr_factory_.GetWeakPtr()));
+
+  // TODO(crbug.com/630586): Until there is a way to create a reference counted
+  // GATT connection in bluetoothd (i.e., specify a connection to a particular
+  // GATT service), simply perform a regular LE connect.
+  bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->ConnectLE(
+      object_path_, std::move(success_callback), std::move(error_callback));
+#else
+  Connect(/*pairing_delegate=*/nullptr,
+          base::BindOnce(&BluetoothDeviceBlueZ::DidConnectGatt,
+                         weak_ptr_factory_.GetWeakPtr()),
+          base::BindOnce(&BluetoothDeviceBlueZ::DidFailToConnectGatt,
+                         weak_ptr_factory_.GetWeakPtr()));
+#endif
 }
 
 void BluetoothDeviceBlueZ::SetGattServicesDiscoveryComplete(bool complete) {
@@ -296,16 +272,54 @@ void BluetoothDeviceBlueZ::DisconnectGatt() {
     return;
   }
 
+// Once DisconnectLE is supported on Linux, this buildflag will not be necessary
+// (this bluez code is only run on Chrome OS and Linux).
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->DisconnectLE(
+      object_path_, base::DoNothing(),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnDisconnectLEError,
+                     weak_ptr_factory_.GetWeakPtr()));
+#else
   Disconnect(base::DoNothing(), base::DoNothing());
+#endif
 }
 
+// Once DisconnectLE is supported on Linux, this buildflag will not be necessary
+// (this bluez code is only run on Chrome OS and Linux).
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+void BluetoothDeviceBlueZ::OnDisconnectLEError(
+    const std::string& error_name,
+    const std::string& error_message) {
+  BLUETOOTH_LOG(ERROR) << "DisconnectLE() failed with error name: "
+                       << error_name << " and error message: " << error_message;
+}
+#endif
+
 std::string BluetoothDeviceBlueZ::GetAddress() const {
   bluez::BluetoothDeviceClient::Properties* properties =
       bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
           object_path_);
   DCHECK(properties);
 
-  return CanonicalizeAddress(properties->address.value());
+  return device::CanonicalizeBluetoothAddress(properties->address.value());
+}
+
+BluetoothDeviceBlueZ::AddressType BluetoothDeviceBlueZ::GetAddressType() const {
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+
+  if (!properties->address_type.is_valid())
+    return ADDR_TYPE_UNKNOWN;
+
+  if (properties->address_type.value() == bluetooth_device::kAddressTypePublic)
+    return ADDR_TYPE_PUBLIC;
+  if (properties->address_type.value() == bluetooth_device::kAddressTypeRandom)
+    return ADDR_TYPE_RANDOM;
+
+  LOG(WARNING) << "Unknown address type: " << properties->address_type.value();
+  return ADDR_TYPE_UNKNOWN;
 }
 
 BluetoothDevice::VendorIDSource BluetoothDeviceBlueZ::GetVendorIDSource()
@@ -380,9 +394,20 @@ bool BluetoothDeviceBlueZ::IsConnected() const {
 }
 
 bool BluetoothDeviceBlueZ::IsGattConnected() const {
-  // Bluez uses the same attribute for GATT Connections and Classic BT
+// Once the |connected_le| property is supported on Linux, this buildflag will
+// not be necessary (this bluez code is only run on Chrome OS and Linux).
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  DCHECK(properties);
+
+  return properties->connected_le.value();
+#else
+  // BlueZ uses the same attribute for GATT Connections and Classic BT
   // Connections.
   return IsConnected();
+#endif
 }
 
 bool BluetoothDeviceBlueZ::IsConnectable() const {
@@ -402,19 +427,7 @@ bool BluetoothDeviceBlueZ::IsConnecting() const {
 }
 
 BluetoothDevice::UUIDSet BluetoothDeviceBlueZ::GetUUIDs() const {
-  bluez::BluetoothDeviceClient::Properties* properties =
-      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
-          object_path_);
-  DCHECK(properties);
-
-  UUIDSet uuids;
-  const std::vector<std::string>& dbus_uuids = properties->uuids.value();
-  for (const std::string& dbus_uuid : dbus_uuids) {
-    device::BluetoothUUID uuid(dbus_uuid);
-    DCHECK(uuid.IsValid());
-    uuids.insert(std::move(uuid));
-  }
-  return uuids;
+  return device_uuids_.GetUUIDs();
 }
 
 base::Optional<int8_t> BluetoothDeviceBlueZ::GetInquiryRSSI() const {
@@ -459,22 +472,22 @@ bool BluetoothDeviceBlueZ::ExpectingConfirmation() const {
   return pairing_.get() && pairing_->ExpectingConfirmation();
 }
 
-void BluetoothDeviceBlueZ::GetConnectionInfo(
-    const ConnectionInfoCallback& callback) {
+void BluetoothDeviceBlueZ::GetConnectionInfo(ConnectionInfoCallback callback) {
   // DBus method call should gracefully return an error if the device is not
   // currently connected.
+  auto copyable_callback = base::AdaptCallbackForRepeating(std::move(callback));
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetConnInfo(
       object_path_,
-      base::Bind(&BluetoothDeviceBlueZ::OnGetConnInfo,
-                 weak_ptr_factory_.GetWeakPtr(), callback),
-      base::Bind(&BluetoothDeviceBlueZ::OnGetConnInfoError,
-                 weak_ptr_factory_.GetWeakPtr(), callback));
+      base::BindOnce(&BluetoothDeviceBlueZ::OnGetConnInfo,
+                     weak_ptr_factory_.GetWeakPtr(), copyable_callback),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnGetConnInfoError,
+                     weak_ptr_factory_.GetWeakPtr(), copyable_callback));
 }
 
 void BluetoothDeviceBlueZ::SetConnectionLatency(
     ConnectionLatency connection_latency,
-    const base::Closure& callback,
-    const ErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ErrorCallback error_callback) {
   uint16_t min_connection_interval = MIN_CONNECTION_INTERVAL_MEDIUM;
   uint16_t max_connection_interval = MAX_CONNECTION_INTERVAL_MEDIUM;
   switch (connection_latency) {
@@ -506,65 +519,61 @@ void BluetoothDeviceBlueZ::SetConnectionLatency(
       bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient();
   client->SetLEConnectionParameters(
       object_path_, connection_parameters,
-      base::Bind(&BluetoothDeviceBlueZ::OnSetLEConnectionParameters,
-                 weak_ptr_factory_.GetWeakPtr(), callback),
-      base::Bind(&BluetoothDeviceBlueZ::OnSetLEConnectionParametersError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      base::BindOnce(&BluetoothDeviceBlueZ::OnSetLEConnectionParameters,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnSetLEConnectionParametersError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
 void BluetoothDeviceBlueZ::Connect(
     BluetoothDevice::PairingDelegate* pairing_delegate,
-    const base::Closure& callback,
-    const ConnectErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ConnectErrorCallback error_callback) {
   if (num_connecting_calls_++ == 0)
     adapter()->NotifyDeviceChanged(this);
 
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Connecting, "
                        << num_connecting_calls_ << " in progress";
 
+  // These callbacks are only called once, but are passed to two different
+  // places.
+  base::RepeatingClosure dupe_callback =
+      base::AdaptCallbackForRepeating(std::move(callback));
+  base::RepeatingCallback<ConnectErrorCallback::RunType> dupe_error_callback =
+      base::AdaptCallbackForRepeating(std::move(error_callback));
+
   if (IsPaired() || !pairing_delegate) {
     // No need to pair, or unable to, skip straight to connection.
-    bluez::BluezDBusManager::Get()->GetBluetoothAdapterClient()->PauseDiscovery(
-        adapter()->object_path(),
-        base::Bind(&BluetoothDeviceBlueZ::ConnectInternal,
-                   weak_ptr_factory_.GetWeakPtr(), false, callback,
-                   error_callback),
-        base::Bind(
-            [](base::WeakPtr<BluetoothDeviceBlueZ> weak_ptr,
-               const base::Closure& callback,
-               const ConnectErrorCallback& error_callback,
-               const std::string& error_name,
-               const std::string& error_message) {
-              if (weak_ptr)
-                weak_ptr->ConnectInternal(false, callback, error_callback);
-            },
-            weak_ptr_factory_.GetWeakPtr(), callback, error_callback));
+    ConnectInternal(dupe_callback, dupe_error_callback);
   } else {
     // Initiate high-security connection with pairing.
     BeginPairing(pairing_delegate);
 
     bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->Pair(
         object_path_,
-        base::Bind(&BluetoothDeviceBlueZ::OnPairDuringConnect,
-                   weak_ptr_factory_.GetWeakPtr(), callback, error_callback),
-        base::Bind(&BluetoothDeviceBlueZ::OnPairDuringConnectError,
-                   weak_ptr_factory_.GetWeakPtr(), error_callback));
+        base::BindOnce(&BluetoothDeviceBlueZ::OnPairDuringConnect,
+                       weak_ptr_factory_.GetWeakPtr(), dupe_callback,
+                       dupe_error_callback),
+        base::BindOnce(&BluetoothDeviceBlueZ::OnPairDuringConnectError,
+                       weak_ptr_factory_.GetWeakPtr(), dupe_error_callback));
   }
 }
 
 void BluetoothDeviceBlueZ::Pair(
     BluetoothDevice::PairingDelegate* pairing_delegate,
-    const base::Closure& callback,
-    const ConnectErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ConnectErrorCallback error_callback) {
   DCHECK(pairing_delegate);
   BeginPairing(pairing_delegate);
 
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->Pair(
       object_path_,
-      base::Bind(&BluetoothDeviceBlueZ::OnPair, weak_ptr_factory_.GetWeakPtr(),
-                 callback),
-      base::Bind(&BluetoothDeviceBlueZ::OnPairError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      base::BindOnce(&BluetoothDeviceBlueZ::OnPair,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnPairError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
 void BluetoothDeviceBlueZ::SetPinCode(const std::string& pincode) {
@@ -612,8 +621,8 @@ void BluetoothDeviceBlueZ::CancelPairing() {
                          << "Sending explicit cancel";
     bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->CancelPairing(
         object_path_, base::DoNothing(),
-        base::Bind(&BluetoothDeviceBlueZ::OnCancelPairingError,
-                   weak_ptr_factory_.GetWeakPtr()));
+        base::BindOnce(&BluetoothDeviceBlueZ::OnCancelPairingError,
+                       weak_ptr_factory_.GetWeakPtr()));
   }
 
   // Since there is no callback to this method it's possible that the pairing
@@ -623,43 +632,46 @@ void BluetoothDeviceBlueZ::CancelPairing() {
   EndPairing();
 }
 
-void BluetoothDeviceBlueZ::Disconnect(const base::Closure& callback,
-                                      const ErrorCallback& error_callback) {
+void BluetoothDeviceBlueZ::Disconnect(base::OnceClosure callback,
+                                      ErrorCallback error_callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Disconnecting";
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->Disconnect(
       object_path_,
-      base::Bind(&BluetoothDeviceBlueZ::OnDisconnect,
-                 weak_ptr_factory_.GetWeakPtr(), callback),
-      base::Bind(&BluetoothDeviceBlueZ::OnDisconnectError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      base::BindOnce(&BluetoothDeviceBlueZ::OnDisconnect,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnDisconnectError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
-void BluetoothDeviceBlueZ::Forget(const base::Closure& callback,
-                                  const ErrorCallback& error_callback) {
+void BluetoothDeviceBlueZ::Forget(base::OnceClosure callback,
+                                  ErrorCallback error_callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Removing device";
   bluez::BluezDBusManager::Get()->GetBluetoothAdapterClient()->RemoveDevice(
-      adapter()->object_path(), object_path_, callback,
-      base::Bind(&BluetoothDeviceBlueZ::OnForgetError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      adapter()->object_path(), object_path_, std::move(callback),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnForgetError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
 void BluetoothDeviceBlueZ::ConnectToService(
     const BluetoothUUID& uuid,
-    const ConnectToServiceCallback& callback,
-    const ConnectToServiceErrorCallback& error_callback) {
+    ConnectToServiceCallback callback,
+    ConnectToServiceErrorCallback error_callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value()
                        << ": Connecting to service: " << uuid.canonical_value();
   scoped_refptr<BluetoothSocketBlueZ> socket =
       BluetoothSocketBlueZ::CreateBluetoothSocket(ui_task_runner_,
                                                   socket_thread_);
   socket->Connect(this, uuid, BluetoothSocketBlueZ::SECURITY_LEVEL_MEDIUM,
-                  base::Bind(callback, socket), error_callback);
+                  base::BindOnce(std::move(callback), socket),
+                  std::move(error_callback));
 }
 
 void BluetoothDeviceBlueZ::ConnectToServiceInsecurely(
     const BluetoothUUID& uuid,
-    const ConnectToServiceCallback& callback,
-    const ConnectToServiceErrorCallback& error_callback) {
+    ConnectToServiceCallback callback,
+    ConnectToServiceErrorCallback error_callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value()
                        << ": Connecting insecurely to service: "
                        << uuid.canonical_value();
@@ -667,56 +679,44 @@ void BluetoothDeviceBlueZ::ConnectToServiceInsecurely(
       BluetoothSocketBlueZ::CreateBluetoothSocket(ui_task_runner_,
                                                   socket_thread_);
   socket->Connect(this, uuid, BluetoothSocketBlueZ::SECURITY_LEVEL_LOW,
-                  base::Bind(callback, socket), error_callback);
+                  base::BindOnce(std::move(callback), socket),
+                  std::move(error_callback));
 }
 
-void BluetoothDeviceBlueZ::CreateGattConnection(
-    const GattConnectionCallback& callback,
-    const ConnectErrorCallback& error_callback) {
-  // TODO(sacomoto): Workaround to retrieve the connection for already connected
-  // devices. Currently, BluetoothGattConnection::Disconnect doesn't do
-  // anything, the unique underlying physical GATT connection is kept. This
-  // should be removed once the correct behavour is implemented and the GATT
-  // connections are reference counted (see todo below).
-  if (IsConnected()) {
-    OnCreateGattConnection(callback);
-    return;
-  }
-
-  // TODO(crbug.com/630586): Until there is a way to create a reference counted
-  // GATT connection in bluetoothd, simply do a regular connect.
-  Connect(nullptr,
-          base::Bind(&BluetoothDeviceBlueZ::OnCreateGattConnection,
-                     weak_ptr_factory_.GetWeakPtr(), callback),
-          error_callback);
+std::unique_ptr<device::BluetoothGattConnection>
+BluetoothDeviceBlueZ::CreateBluetoothGattConnectionObject() {
+  return std::make_unique<BluetoothGattConnectionBlueZ>(adapter_, GetAddress(),
+                                                        object_path_);
 }
 
 void BluetoothDeviceBlueZ::GetServiceRecords(
-    const GetServiceRecordsCallback& callback,
-    const GetServiceRecordsErrorCallback& error_callback) {
+    GetServiceRecordsCallback callback,
+    GetServiceRecordsErrorCallback error_callback) {
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetServiceRecords(
-      object_path_, callback,
-      base::Bind(&BluetoothDeviceBlueZ::OnGetServiceRecordsError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      object_path_, std::move(callback),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnGetServiceRecordsError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
 void BluetoothDeviceBlueZ::ExecuteWrite(
-    const base::Closure& callback,
-    const ExecuteWriteErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ExecuteWriteErrorCallback error_callback) {
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->ExecuteWrite(
-      object_path_, callback,
-      base::Bind(&BluetoothDeviceBlueZ::OnExecuteWriteError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      object_path_, std::move(callback),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnExecuteWriteError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
-void BluetoothDeviceBlueZ::AbortWrite(
-    const base::Closure& callback,
-    const AbortWriteErrorCallback& error_callback) {
+void BluetoothDeviceBlueZ::AbortWrite(base::OnceClosure callback,
+                                      AbortWriteErrorCallback error_callback) {
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->AbortWrite(
-      object_path_, callback,
-      base::Bind(&BluetoothDeviceBlueZ::OnAbortWriteError,
-                 weak_ptr_factory_.GetWeakPtr(), error_callback));
+      object_path_, std::move(callback),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnAbortWriteError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 #endif
 
@@ -759,6 +759,28 @@ void BluetoothDeviceBlueZ::UpdateAdvertisingDataFlags() {
   advertising_data_flags_ = properties->advertising_data_flags.value()[0];
 }
 
+void BluetoothDeviceBlueZ::UpdateServiceUUIDs() {
+  bluez::BluetoothDeviceClient::Properties* properties =
+      bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->GetProperties(
+          object_path_);
+  if (!properties)
+    return;
+
+  UUIDList uuids;
+  for (const std::string& dbus_uuid : properties->uuids.value()) {
+    device::BluetoothUUID uuid(dbus_uuid);
+    DCHECK(uuid.IsValid());
+    uuids.push_back(std::move(uuid));
+  }
+
+  device_uuids_.ReplaceServiceUUIDs(uuids);
+}
+
+void BluetoothDeviceBlueZ::SetAdvertisedUUIDs(
+    const BluetoothDevice::UUIDList& uuids) {
+  device_uuids_.ReplaceAdvertisedUUIDs(uuids);
+}
+
 BluetoothPairingBlueZ* BluetoothDeviceBlueZ::BeginPairing(
     BluetoothDevice::PairingDelegate* pairing_delegate) {
   pairing_.reset(new BluetoothPairingBlueZ(this, pairing_delegate));
@@ -814,7 +836,7 @@ void BluetoothDeviceBlueZ::GattServiceRemoved(
     const dbus::ObjectPath& object_path) {
   auto iter = gatt_services_.find(object_path.value());
   if (iter == gatt_services_.end()) {
-    VLOG(3) << "Unknown GATT service removed: " << object_path.value();
+    DVLOG(3) << "Unknown GATT service removed: " << object_path.value();
     return;
   }
 
@@ -844,8 +866,8 @@ void BluetoothDeviceBlueZ::UpdateGattServices(
 
   DCHECK(IsGattServicesDiscoveryComplete());
 
-  VLOG(3) << "Updating the list of GATT services associated with device "
-          << object_path_.value();
+  DVLOG(3) << "Updating the list of GATT services associated with device "
+           << object_path_.value();
 
   const std::vector<dbus::ObjectPath> service_paths =
       bluez::BluezDBusManager::Get()
@@ -855,11 +877,13 @@ void BluetoothDeviceBlueZ::UpdateGattServices(
     // Add all previously unknown GATT services associated with the device.
     GattServiceAdded(service_path);
 
-    // If the service does not belong in this device, there is nothing left to
-    // do.
+    // |service_paths| includes all services for all devices, not just this
+    // device. GetGattService() will return nullptr for services belonging
+    // to other devices, so we skip those and keep looking for services that
+    // belong to this device.
     BluetoothRemoteGattService* service = GetGattService(service_path.value());
     if (service == nullptr) {
-      return;
+      continue;
     }
 
     // Notify of GATT discovery complete if we haven't before.
@@ -870,40 +894,41 @@ void BluetoothDeviceBlueZ::UpdateGattServices(
   }
 }
 
-void BluetoothDeviceBlueZ::OnGetConnInfo(const ConnectionInfoCallback& callback,
+void BluetoothDeviceBlueZ::OnGetConnInfo(ConnectionInfoCallback callback,
                                          int16_t rssi,
                                          int16_t transmit_power,
                                          int16_t max_transmit_power) {
-  callback.Run(ConnectionInfo(rssi, transmit_power, max_transmit_power));
+  std::move(callback).Run(
+      ConnectionInfo(rssi, transmit_power, max_transmit_power));
 }
 
 void BluetoothDeviceBlueZ::OnGetConnInfoError(
-    const ConnectionInfoCallback& callback,
+    ConnectionInfoCallback callback,
     const std::string& error_name,
     const std::string& error_message) {
   BLUETOOTH_LOG(ERROR) << object_path_.value()
                        << ": Failed to get connection info: " << error_name
                        << ": " << error_message;
-  callback.Run(ConnectionInfo());
+  std::move(callback).Run(ConnectionInfo());
 }
 
 void BluetoothDeviceBlueZ::OnSetLEConnectionParameters(
-    const base::Closure& callback) {
-  callback.Run();
+    base::OnceClosure callback) {
+  std::move(callback).Run();
 }
 
 void BluetoothDeviceBlueZ::OnSetLEConnectionParametersError(
-    const ErrorCallback& callback,
+    ErrorCallback callback,
     const std::string& error_name,
     const std::string& error_message) {
   BLUETOOTH_LOG(ERROR) << object_path_.value()
                        << ": Failed to set connection parameters: "
                        << error_name << ": " << error_message;
-  callback.Run();
+  std::move(callback).Run();
 }
 
 void BluetoothDeviceBlueZ::OnGetServiceRecordsError(
-    const GetServiceRecordsErrorCallback& error_callback,
+    GetServiceRecordsErrorCallback error_callback,
     const std::string& error_name,
     const std::string& error_message) {
   BLUETOOTH_LOG(EVENT) << object_path_.value()
@@ -914,51 +939,49 @@ void BluetoothDeviceBlueZ::OnGetServiceRecordsError(
   if (error_name == bluetooth_device::kErrorNotConnected) {
     code = BluetoothServiceRecordBlueZ::ErrorCode::ERROR_DEVICE_DISCONNECTED;
   }
-  error_callback.Run(code);
+  std::move(error_callback).Run(code);
 }
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
 void BluetoothDeviceBlueZ::OnExecuteWriteError(
-    const ExecuteWriteErrorCallback& error_callback,
+    ExecuteWriteErrorCallback error_callback,
     const std::string& error_name,
     const std::string& error_message) {
   BLUETOOTH_LOG(EVENT) << object_path_.value()
                        << ": Failed to execute write: " << error_name << ": "
                        << error_message;
-  error_callback.Run(
-      BluetoothGattServiceBlueZ::DBusErrorToServiceError(error_name));
+  std::move(error_callback)
+      .Run(BluetoothGattServiceBlueZ::DBusErrorToServiceError(error_name));
 }
 
 void BluetoothDeviceBlueZ::OnAbortWriteError(
-    const AbortWriteErrorCallback& error_callback,
+    AbortWriteErrorCallback error_callback,
     const std::string& error_name,
     const std::string& error_message) {
   BLUETOOTH_LOG(EVENT) << object_path_.value()
                        << ": Failed to abort write: " << error_name << ": "
                        << error_message;
-  error_callback.Run(
-      BluetoothGattServiceBlueZ::DBusErrorToServiceError(error_name));
+  std::move(error_callback)
+      .Run(BluetoothGattServiceBlueZ::DBusErrorToServiceError(error_name));
 }
 #endif
 
 void BluetoothDeviceBlueZ::ConnectInternal(
-    bool after_pairing,
-    const base::Closure& callback,
-    const ConnectErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ConnectErrorCallback error_callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Connecting";
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->Connect(
       object_path_,
-      base::Bind(&BluetoothDeviceBlueZ::OnConnect,
-                 weak_ptr_factory_.GetWeakPtr(), after_pairing, callback),
-      base::Bind(&BluetoothDeviceBlueZ::OnConnectError,
-                 weak_ptr_factory_.GetWeakPtr(), after_pairing,
-                 error_callback));
+      base::BindOnce(&BluetoothDeviceBlueZ::OnConnect,
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
+      base::BindOnce(&BluetoothDeviceBlueZ::OnConnectError,
+                     weak_ptr_factory_.GetWeakPtr(),
+                     std::move(error_callback)));
 }
 
-void BluetoothDeviceBlueZ::OnConnect(bool after_pairing,
-                                     const base::Closure& callback) {
-  bluez::BluezDBusManager::Get()->GetBluetoothAdapterClient()->UnpauseDiscovery(
-      adapter()->object_path(), base::DoNothing(), base::DoNothing());
+void BluetoothDeviceBlueZ::OnConnect(base::OnceClosure callback) {
+  BLUETOOTH_LOG(EVENT) << object_path_.value()
+                       << ": Unpausing discovery after connection";
   if (--num_connecting_calls_ == 0)
     adapter()->NotifyDeviceChanged(this);
 
@@ -968,28 +991,15 @@ void BluetoothDeviceBlueZ::OnConnect(bool after_pairing,
 
   SetTrusted();
 
-  if (after_pairing)
-    UMA_HISTOGRAM_ENUMERATION("Bluetooth.PairingResult",
-                              UMA_PAIRING_RESULT_SUCCESS,
-                              UMA_PAIRING_RESULT_COUNT);
-
-  callback.Run();
+  std::move(callback).Run();
 }
 
-void BluetoothDeviceBlueZ::OnCreateGattConnection(
-    const GattConnectionCallback& callback) {
-  std::unique_ptr<device::BluetoothGattConnection> conn(
-      new BluetoothGattConnectionBlueZ(adapter_, GetAddress(), object_path_));
-  callback.Run(std::move(conn));
-}
+void BluetoothDeviceBlueZ::OnConnectError(ConnectErrorCallback error_callback,
+                                          const std::string& error_name,
+                                          const std::string& error_message) {
+  BLUETOOTH_LOG(EVENT) << object_path_.value()
+                       << ": Unpausing discovery after failed connection";
 
-void BluetoothDeviceBlueZ::OnConnectError(
-    bool after_pairing,
-    const ConnectErrorCallback& error_callback,
-    const std::string& error_name,
-    const std::string& error_message) {
-  bluez::BluezDBusManager::Get()->GetBluetoothAdapterClient()->UnpauseDiscovery(
-      adapter()->object_path(), base::DoNothing(), base::DoNothing());
   if (--num_connecting_calls_ == 0)
     adapter()->NotifyDeviceChanged(this);
 
@@ -1010,23 +1020,21 @@ void BluetoothDeviceBlueZ::OnConnectError(
     error_code = ERROR_UNSUPPORTED_DEVICE;
   }
 
-  if (after_pairing)
-    RecordPairingResult(error_code);
-  error_callback.Run(error_code);
+  std::move(error_callback).Run(error_code);
 }
 
 void BluetoothDeviceBlueZ::OnPairDuringConnect(
-    const base::Closure& callback,
-    const ConnectErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ConnectErrorCallback error_callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Paired";
 
   EndPairing();
 
-  ConnectInternal(true, callback, error_callback);
+  ConnectInternal(std::move(callback), std::move(error_callback));
 }
 
 void BluetoothDeviceBlueZ::OnPairDuringConnectError(
-    const ConnectErrorCallback& error_callback,
+    ConnectErrorCallback error_callback,
     const std::string& error_name,
     const std::string& error_message) {
   if (--num_connecting_calls_ == 0)
@@ -1044,27 +1052,24 @@ void BluetoothDeviceBlueZ::OnPairDuringConnectError(
   // Determine the error code from error_name.
   ConnectErrorCode error_code = DBusErrorToConnectError(error_name);
 
-  RecordPairingResult(error_code);
-  error_callback.Run(error_code);
+  std::move(error_callback).Run(error_code);
 }
 
-void BluetoothDeviceBlueZ::OnPair(const base::Closure& callback) {
+void BluetoothDeviceBlueZ::OnPair(base::OnceClosure callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Paired";
   EndPairing();
-  callback.Run();
+  std::move(callback).Run();
 }
 
-void BluetoothDeviceBlueZ::OnPairError(
-    const ConnectErrorCallback& error_callback,
-    const std::string& error_name,
-    const std::string& error_message) {
+void BluetoothDeviceBlueZ::OnPairError(ConnectErrorCallback error_callback,
+                                       const std::string& error_name,
+                                       const std::string& error_message) {
   BLUETOOTH_LOG(ERROR) << object_path_.value()
                        << ": Failed to pair device: " << error_name << ": "
                        << error_message;
   EndPairing();
   ConnectErrorCode error_code = DBusErrorToConnectError(error_name);
-  RecordPairingResult(error_code);
-  error_callback.Run(error_code);
+  std::move(error_callback).Run(error_code);
 }
 
 void BluetoothDeviceBlueZ::OnCancelPairingError(
@@ -1083,8 +1088,8 @@ void BluetoothDeviceBlueZ::SetTrusted() {
   bluez::BluezDBusManager::Get()
       ->GetBluetoothDeviceClient()
       ->GetProperties(object_path_)
-      ->trusted.Set(true, base::Bind(&BluetoothDeviceBlueZ::OnSetTrusted,
-                                     weak_ptr_factory_.GetWeakPtr()));
+      ->trusted.Set(true, base::BindOnce(&BluetoothDeviceBlueZ::OnSetTrusted,
+                                         weak_ptr_factory_.GetWeakPtr()));
 }
 
 void BluetoothDeviceBlueZ::OnSetTrusted(bool success) {
@@ -1095,28 +1100,27 @@ void BluetoothDeviceBlueZ::OnSetTrusted(bool success) {
       << object_path_.value() << ": OnSetTrusted: " << success;
 }
 
-void BluetoothDeviceBlueZ::OnDisconnect(const base::Closure& callback) {
+void BluetoothDeviceBlueZ::OnDisconnect(base::OnceClosure callback) {
   BLUETOOTH_LOG(EVENT) << object_path_.value() << ": Disconnected";
-  callback.Run();
+  std::move(callback).Run();
 }
 
-void BluetoothDeviceBlueZ::OnDisconnectError(
-    const ErrorCallback& error_callback,
-    const std::string& error_name,
-    const std::string& error_message) {
+void BluetoothDeviceBlueZ::OnDisconnectError(ErrorCallback error_callback,
+                                             const std::string& error_name,
+                                             const std::string& error_message) {
   BLUETOOTH_LOG(ERROR) << object_path_.value()
                        << ": Failed to disconnect device: " << error_name
                        << ": " << error_message;
-  error_callback.Run();
+  std::move(error_callback).Run();
 }
 
-void BluetoothDeviceBlueZ::OnForgetError(const ErrorCallback& error_callback,
+void BluetoothDeviceBlueZ::OnForgetError(ErrorCallback error_callback,
                                          const std::string& error_name,
                                          const std::string& error_message) {
   BLUETOOTH_LOG(ERROR) << object_path_.value()
                        << ": Failed to remove device: " << error_name << ": "
                        << error_message;
-  error_callback.Run();
+  std::move(error_callback).Run();
 }
 
 }  // namespace bluez
