@@ -15,13 +15,13 @@
 #include "base/callback.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/memory/linked_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/sequenced_task_runner.h"
 #include "base/single_thread_task_runner.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_util.h"
 #include "base/task_runner_util.h"
-#include "base/threading/thread_restrictions.h"
+#include "base/threading/scoped_blocking_call.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "dbus/bus.h"
 #include "dbus/object_path.h"
@@ -89,17 +89,16 @@ BluetoothSocketBlueZ::~BluetoothSocketBlueZ() {
   }
 }
 
-void BluetoothSocketBlueZ::Connect(
-    const BluetoothDeviceBlueZ* device,
-    const BluetoothUUID& uuid,
-    SecurityLevel security_level,
-    const base::Closure& success_callback,
-    const ErrorCompletionCallback& error_callback) {
+void BluetoothSocketBlueZ::Connect(const BluetoothDeviceBlueZ* device,
+                                   const BluetoothUUID& uuid,
+                                   SecurityLevel security_level,
+                                   base::OnceClosure success_callback,
+                                   ErrorCompletionCallback error_callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(!profile_);
 
   if (!uuid.IsValid()) {
-    error_callback.Run(kInvalidUUID);
+    std::move(error_callback).Run(kInvalidUUID);
     return;
   }
 
@@ -108,11 +107,12 @@ void BluetoothSocketBlueZ::Connect(
   uuid_ = uuid;
   options_.reset(new bluez::BluetoothProfileManagerClient::Options());
   if (security_level == SECURITY_LEVEL_LOW)
-    options_->require_authentication.reset(new bool(false));
+    options_->require_authentication = std::make_unique<bool>(false);
 
   adapter_ = device->adapter();
 
-  RegisterProfile(device->adapter(), success_callback, error_callback);
+  RegisterProfile(device->adapter(), std::move(success_callback),
+                  std::move(error_callback));
 }
 
 void BluetoothSocketBlueZ::Listen(
@@ -120,13 +120,13 @@ void BluetoothSocketBlueZ::Listen(
     SocketType socket_type,
     const BluetoothUUID& uuid,
     const BluetoothAdapter::ServiceOptions& service_options,
-    const base::Closure& success_callback,
-    const ErrorCompletionCallback& error_callback) {
+    base::OnceClosure success_callback,
+    ErrorCompletionCallback error_callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(!profile_);
 
   if (!uuid.IsValid()) {
-    error_callback.Run(kInvalidUUID);
+    std::move(error_callback).Run(kInvalidUUID);
     return;
   }
 
@@ -151,11 +151,16 @@ void BluetoothSocketBlueZ::Listen(
       NOTREACHED();
   }
 
+  if (service_options.require_authentication) {
+    options_->require_authentication =
+        std::make_unique<bool>(*service_options.require_authentication);
+  }
+
   RegisterProfile(static_cast<BluetoothAdapterBlueZ*>(adapter.get()),
-                  success_callback, error_callback);
+                  std::move(success_callback), std::move(error_callback));
 }
 
-void BluetoothSocketBlueZ::Close() {
+void BluetoothSocketBlueZ::Disconnect(base::OnceClosure callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
 
   if (profile_)
@@ -171,45 +176,41 @@ void BluetoothSocketBlueZ::Close() {
   }
 
   if (!device_path_.value().empty()) {
-    BluetoothSocketNet::Close();
-  } else {
-    DoCloseListening();
+    BluetoothSocketNet::Disconnect(std::move(callback));
+    return;
   }
+
+  if (accept_request_) {
+    std::move(accept_request_->error_callback)
+        .Run(net::ErrorToString(net::ERR_CONNECTION_CLOSED));
+    accept_request_.reset(nullptr);
+  }
+
+  while (connection_request_queue_.size() > 0) {
+    std::move(connection_request_queue_.front()->callback).Run(REJECTED);
+    connection_request_queue_.pop();
+  }
+  std::move(callback).Run();
 }
 
-void BluetoothSocketBlueZ::Disconnect(const base::Closure& callback) {
-  DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
-
-  if (profile_)
-    UnregisterProfile();
-
-  if (!device_path_.value().empty()) {
-    BluetoothSocketNet::Disconnect(callback);
-  } else {
-    DoCloseListening();
-    callback.Run();
-  }
-}
-
-void BluetoothSocketBlueZ::Accept(
-    const AcceptCompletionCallback& success_callback,
-    const ErrorCompletionCallback& error_callback) {
+void BluetoothSocketBlueZ::Accept(AcceptCompletionCallback success_callback,
+                                  ErrorCompletionCallback error_callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
 
   if (!device_path_.value().empty()) {
-    error_callback.Run(kSocketNotListening);
+    std::move(error_callback).Run(kSocketNotListening);
     return;
   }
 
   // Only one pending accept at a time
   if (accept_request_.get()) {
-    error_callback.Run(net::ErrorToString(net::ERR_IO_PENDING));
+    std::move(error_callback).Run(net::ErrorToString(net::ERR_IO_PENDING));
     return;
   }
 
-  accept_request_.reset(new AcceptRequest);
-  accept_request_->success_callback = success_callback;
-  accept_request_->error_callback = error_callback;
+  accept_request_ = std::make_unique<AcceptRequest>();
+  accept_request_->success_callback = std::move(success_callback);
+  accept_request_->error_callback = std::move(error_callback);
 
   if (connection_request_queue_.size() >= 1) {
     AcceptConnectionRequest();
@@ -218,8 +219,8 @@ void BluetoothSocketBlueZ::Accept(
 
 void BluetoothSocketBlueZ::RegisterProfile(
     BluetoothAdapterBlueZ* adapter,
-    const base::Closure& success_callback,
-    const ErrorCompletionCallback& error_callback) {
+    base::OnceClosure success_callback,
+    ErrorCompletionCallback error_callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(!profile_);
   DCHECK(adapter);
@@ -228,25 +229,29 @@ void BluetoothSocketBlueZ::RegisterProfile(
   // adapter isn't running yet.  Report success and carry on;
   // the profile will be registered when the daemon becomes available.
   if (!adapter->IsPresent()) {
-    VLOG(1) << uuid_.canonical_value() << " on " << device_path_.value()
-            << ": Delaying profile registration.";
-    base::ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, success_callback);
+    DVLOG(1) << uuid_.canonical_value() << " on " << device_path_.value()
+             << ": Delaying profile registration.";
+    base::ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE,
+                                                  std::move(success_callback));
     return;
   }
 
-  VLOG(1) << uuid_.canonical_value() << " on " << device_path_.value()
-          << ": Acquiring profile.";
+  DVLOG(1) << uuid_.canonical_value() << " on " << device_path_.value()
+           << ": Acquiring profile.";
 
-  adapter->UseProfile(uuid_, device_path_, *options_, this,
-                      base::Bind(&BluetoothSocketBlueZ::OnRegisterProfile, this,
-                                 success_callback, error_callback),
-                      base::Bind(&BluetoothSocketBlueZ::OnRegisterProfileError,
-                                 this, error_callback));
+  auto copyable_error_callback =
+      base::AdaptCallbackForRepeating(std::move(error_callback));
+  adapter->UseProfile(
+      uuid_, device_path_, *options_, this,
+      base::BindOnce(&BluetoothSocketBlueZ::OnRegisterProfile, this,
+                     std::move(success_callback), copyable_error_callback),
+      base::BindOnce(&BluetoothSocketBlueZ::OnRegisterProfileError, this,
+                     copyable_error_callback));
 }
 
 void BluetoothSocketBlueZ::OnRegisterProfile(
-    const base::Closure& success_callback,
-    const ErrorCompletionCallback& error_callback,
+    base::OnceClosure success_callback,
+    ErrorCompletionCallback error_callback,
     BluetoothAdapterProfileBlueZ* profile) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(!profile_);
@@ -254,75 +259,95 @@ void BluetoothSocketBlueZ::OnRegisterProfile(
   profile_ = profile;
 
   if (device_path_.value().empty()) {
-    VLOG(1) << uuid_.canonical_value() << ": Profile registered.";
-    success_callback.Run();
+    DVLOG(1) << uuid_.canonical_value() << ": Profile registered.";
+    std::move(success_callback).Run();
     return;
   }
 
-  VLOG(1) << uuid_.canonical_value() << ": Got profile, connecting to "
-          << device_path_.value();
+  DVLOG(1) << uuid_.canonical_value() << ": Got profile, connecting to "
+           << device_path_.value();
 
   bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient()->ConnectProfile(
       device_path_, uuid_.canonical_value(),
-      base::Bind(&BluetoothSocketBlueZ::OnConnectProfile, this,
-                 success_callback),
-      base::Bind(&BluetoothSocketBlueZ::OnConnectProfileError, this,
-                 error_callback));
+      base::BindOnce(&BluetoothSocketBlueZ::OnConnectProfile, this,
+                     std::move(success_callback)),
+      base::BindOnce(&BluetoothSocketBlueZ::OnConnectProfileError, this,
+                     std::move(error_callback)));
 }
 
 void BluetoothSocketBlueZ::OnRegisterProfileError(
-    const ErrorCompletionCallback& error_callback,
+    ErrorCompletionCallback error_callback,
     const std::string& error_message) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
 
   LOG(WARNING) << uuid_.canonical_value()
                << ": Failed to register profile: " << error_message;
-  error_callback.Run(error_message);
+  std::move(error_callback).Run(error_message);
 }
 
 void BluetoothSocketBlueZ::OnConnectProfile(
-    const base::Closure& success_callback) {
+    base::OnceClosure success_callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(profile_);
 
-  VLOG(1) << profile_->object_path().value() << ": Profile connected.";
+  DVLOG(1) << profile_->object_path().value() << ": Profile connected.";
   UnregisterProfile();
-  success_callback.Run();
+  std::move(success_callback).Run();
 }
 
 void BluetoothSocketBlueZ::OnConnectProfileError(
-    const ErrorCompletionCallback& error_callback,
+    ErrorCompletionCallback error_callback,
     const std::string& error_name,
     const std::string& error_message) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(profile_);
 
+  const std::string error = base::StrCat({error_name, ": ", error_message});
   LOG(WARNING) << profile_->object_path().value()
-               << ": Failed to connect profile: " << error_name << ": "
-               << error_message;
+               << ": Failed to connect profile: " << error;
   UnregisterProfile();
-  error_callback.Run(error_message);
+  std::move(error_callback).Run(error);
 }
 
 void BluetoothSocketBlueZ::AdapterPresentChanged(BluetoothAdapter* adapter,
                                                  bool present) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
 
+  // Some boards cut power to their Bluetooth chip during suspension, which
+  // leads to a AdapterPresentChanged(present=false) event on wake (quickly
+  // followed by an AdapterPresentChanged(present=true) event).
+  // This 'present=false' event can occur in the middle of
+  // BluetoothSocketBlueZ's asynchronous process of acquiring |profile_|. That
+  // means the following 2 surprising edge-cases can occur on a few select
+  // boards:
+  //   1) |profile_| may not be initialized when a 'present=false' event occurs.
+  //      We must check |profile_| before calling UnregisterProfile().
+  //   2) |profile_| may already be initialized when a 'present=true' event
+  //      occurs. We must check |profile_| before attempting to redundantly
+  //      initialize it via BluetoothAdapterBlueZ::UseProfile().
+
   if (!present) {
-    // Adapter removed, we can't use the profile anymore.
-    UnregisterProfile();
+    // Edge-case (1) described above.
+    if (profile_) {
+      // Adapter removed, we can't use the profile anymore.
+      UnregisterProfile();
+    }
     return;
   }
 
-  DCHECK(!profile_);
+  // Edge-case (2) described above.
+  if (profile_) {
+    return;
+  }
 
-  VLOG(1) << uuid_.canonical_value() << " on " << device_path_.value()
-          << ": Acquiring profile.";
+  DVLOG(1) << uuid_.canonical_value() << " on " << device_path_.value()
+           << ": Acquiring profile.";
 
   static_cast<BluetoothAdapterBlueZ*>(adapter)->UseProfile(
       uuid_, device_path_, *options_, this,
-      base::Bind(&BluetoothSocketBlueZ::OnInternalRegisterProfile, this),
-      base::Bind(&BluetoothSocketBlueZ::OnInternalRegisterProfileError, this));
+      base::BindOnce(&BluetoothSocketBlueZ::OnInternalRegisterProfile, this),
+      base::BindOnce(&BluetoothSocketBlueZ::OnInternalRegisterProfileError,
+                     this));
 }
 
 void BluetoothSocketBlueZ::OnInternalRegisterProfile(
@@ -332,7 +357,7 @@ void BluetoothSocketBlueZ::OnInternalRegisterProfile(
 
   profile_ = profile;
 
-  VLOG(1) << uuid_.canonical_value() << ": Profile re-registered";
+  DVLOG(1) << uuid_.canonical_value() << ": Profile re-registered";
 }
 
 void BluetoothSocketBlueZ::OnInternalRegisterProfileError(
@@ -346,35 +371,35 @@ void BluetoothSocketBlueZ::Released() {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(profile_);
 
-  VLOG(1) << profile_->object_path().value() << ": Release";
+  DVLOG(1) << profile_->object_path().value() << ": Release";
 }
 
 void BluetoothSocketBlueZ::NewConnection(
     const dbus::ObjectPath& device_path,
     base::ScopedFD fd,
     const bluez::BluetoothProfileServiceProvider::Delegate::Options& options,
-    const ConfirmationCallback& callback) {
+    ConfirmationCallback callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
 
-  VLOG(1) << uuid_.canonical_value()
-          << ": New connection from device: " << device_path.value();
+  DVLOG(1) << uuid_.canonical_value()
+           << ": New connection from device: " << device_path.value();
 
   if (!device_path_.value().empty()) {
     DCHECK(device_path_ == device_path);
 
     socket_thread()->task_runner()->PostTask(
-        FROM_HERE,
-        base::Bind(&BluetoothSocketBlueZ::DoNewConnection, this, device_path_,
-                   base::Passed(&fd), options, callback));
+        FROM_HERE, base::BindOnce(&BluetoothSocketBlueZ::DoNewConnection, this,
+                                  device_path_, std::move(fd), options,
+                                  std::move(callback)));
   } else {
-    linked_ptr<ConnectionRequest> request(new ConnectionRequest());
+    auto request = std::make_unique<ConnectionRequest>();
     request->device_path = device_path;
     request->fd = std::move(fd);
     request->options = options;
-    request->callback = callback;
+    request->callback = std::move(callback);
 
-    connection_request_queue_.push(request);
-    VLOG(1) << uuid_.canonical_value() << ": Connection is now pending.";
+    connection_request_queue_.push(std::move(request));
+    DVLOG(1) << uuid_.canonical_value() << ": Connection is now pending.";
     if (accept_request_) {
       AcceptConnectionRequest();
     }
@@ -383,26 +408,26 @@ void BluetoothSocketBlueZ::NewConnection(
 
 void BluetoothSocketBlueZ::RequestDisconnection(
     const dbus::ObjectPath& device_path,
-    const ConfirmationCallback& callback) {
+    ConfirmationCallback callback) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(profile_);
 
-  VLOG(1) << profile_->object_path().value() << ": Request disconnection";
-  callback.Run(SUCCESS);
+  DVLOG(1) << profile_->object_path().value() << ": Request disconnection";
+  std::move(callback).Run(SUCCESS);
 }
 
 void BluetoothSocketBlueZ::Cancel() {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(profile_);
 
-  VLOG(1) << profile_->object_path().value() << ": Cancel";
+  DVLOG(1) << profile_->object_path().value() << ": Cancel";
 
   if (connection_request_queue_.empty())
     return;
 
   // If the front request is being accepted mark it as cancelled, otherwise
   // just pop it from the queue.
-  linked_ptr<ConnectionRequest> request = connection_request_queue_.front();
+  ConnectionRequest* request = connection_request_queue_.front().get();
   if (!request->accepting) {
     request->cancelled = true;
   } else {
@@ -416,10 +441,10 @@ void BluetoothSocketBlueZ::AcceptConnectionRequest() {
   DCHECK(connection_request_queue_.size() >= 1);
   DCHECK(profile_);
 
-  VLOG(1) << profile_->object_path().value()
-          << ": Accepting pending connection.";
+  DVLOG(1) << profile_->object_path().value()
+           << ": Accepting pending connection.";
 
-  linked_ptr<ConnectionRequest> request = connection_request_queue_.front();
+  ConnectionRequest* request = connection_request_queue_.front().get();
   request->accepting = true;
 
   BluetoothDeviceBlueZ* device =
@@ -437,31 +462,34 @@ void BluetoothSocketBlueZ::AcceptConnectionRequest() {
 
   socket_thread()->task_runner()->PostTask(
       FROM_HERE,
-      base::Bind(&BluetoothSocketBlueZ::DoNewConnection, client_socket,
-                 request->device_path, base::Passed(&request->fd),
-                 request->options,
-                 base::Bind(&BluetoothSocketBlueZ::OnNewConnection, this,
-                            client_socket, request->callback)));
+      base::BindOnce(
+          &BluetoothSocketBlueZ::DoNewConnection, client_socket,
+          request->device_path, std::move(request->fd), request->options,
+          base::BindOnce(&BluetoothSocketBlueZ::OnNewConnection, this,
+                         client_socket, std::move(request->callback))));
 }
 
 void BluetoothSocketBlueZ::DoNewConnection(
     const dbus::ObjectPath& device_path,
     base::ScopedFD fd,
     const bluez::BluetoothProfileServiceProvider::Delegate::Options& options,
-    const ConfirmationCallback& callback) {
+    ConfirmationCallback callback) {
   DCHECK(socket_thread()->task_runner()->RunsTasksInCurrentSequence());
-  base::AssertBlockingAllowed();
+  base::ScopedBlockingCall scoped_blocking_call(FROM_HERE,
+                                                base::BlockingType::MAY_BLOCK);
 
   if (!fd.is_valid()) {
     LOG(WARNING) << uuid_.canonical_value() << " :" << fd.get()
                  << ": Invalid file descriptor received from Bluetooth Daemon.";
-    ui_task_runner()->PostTask(FROM_HERE, base::Bind(callback, REJECTED));
+    ui_task_runner()->PostTask(FROM_HERE,
+                               base::BindOnce(std::move(callback), REJECTED));
     return;
   }
 
   if (tcp_socket()) {
     LOG(WARNING) << uuid_.canonical_value() << ": Already connected";
-    ui_task_runner()->PostTask(FROM_HERE, base::Bind(callback, REJECTED));
+    ui_task_runner()->PostTask(FROM_HERE,
+                               base::BindOnce(std::move(callback), REJECTED));
     return;
   }
 
@@ -474,59 +502,46 @@ void BluetoothSocketBlueZ::DoNewConnection(
   if (net_result != net::OK) {
     LOG(WARNING) << uuid_.canonical_value() << ": Error adopting socket: "
                  << std::string(net::ErrorToString(net_result));
-    ui_task_runner()->PostTask(FROM_HERE, base::Bind(callback, REJECTED));
+    ui_task_runner()->PostTask(FROM_HERE,
+                               base::BindOnce(std::move(callback), REJECTED));
     return;
   }
-  ui_task_runner()->PostTask(FROM_HERE, base::Bind(callback, SUCCESS));
+  ui_task_runner()->PostTask(FROM_HERE,
+                             base::BindOnce(std::move(callback), SUCCESS));
 }
 
 void BluetoothSocketBlueZ::OnNewConnection(
     scoped_refptr<BluetoothSocket> socket,
-    const ConfirmationCallback& callback,
+    ConfirmationCallback callback,
     Status status) {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(accept_request_.get());
   DCHECK(connection_request_queue_.size() >= 1);
 
-  linked_ptr<ConnectionRequest> request = connection_request_queue_.front();
+  std::unique_ptr<ConnectionRequest> request =
+      std::move(connection_request_queue_.front());
   if (status == SUCCESS && !request->cancelled) {
     BluetoothDeviceBlueZ* device =
         static_cast<BluetoothAdapterBlueZ*>(adapter_.get())
             ->GetDeviceWithPath(request->device_path);
     DCHECK(device);
 
-    accept_request_->success_callback.Run(device, socket);
+    std::move(accept_request_->success_callback).Run(device, socket);
   } else {
-    accept_request_->error_callback.Run(kAcceptFailed);
+    std::move(accept_request_->error_callback).Run(kAcceptFailed);
   }
 
   accept_request_.reset(nullptr);
   connection_request_queue_.pop();
 
-  callback.Run(status);
-}
-
-void BluetoothSocketBlueZ::DoCloseListening() {
-  DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
-
-  if (accept_request_) {
-    accept_request_->error_callback.Run(
-        net::ErrorToString(net::ERR_CONNECTION_CLOSED));
-    accept_request_.reset(nullptr);
-  }
-
-  while (connection_request_queue_.size() > 0) {
-    linked_ptr<ConnectionRequest> request = connection_request_queue_.front();
-    request->callback.Run(REJECTED);
-    connection_request_queue_.pop();
-  }
+  std::move(callback).Run(status);
 }
 
 void BluetoothSocketBlueZ::UnregisterProfile() {
   DCHECK(ui_task_runner()->RunsTasksInCurrentSequence());
   DCHECK(profile_);
 
-  VLOG(1) << profile_->object_path().value() << ": Release profile";
+  DVLOG(1) << profile_->object_path().value() << ": Release profile";
 
   static_cast<BluetoothAdapterBlueZ*>(adapter_.get())
       ->ReleaseProfile(device_path_, profile_);
