@@ -7,6 +7,7 @@
 #include <cstddef>
 
 #include "base/bind.h"
+#include "base/callback_helpers.h"
 #include "base/logging.h"
 #include "base/strings/string_util.h"
 #include "device/bluetooth/bluetooth_gatt_characteristic.h"
@@ -39,10 +40,9 @@ BluetoothGattCharacteristicServiceProviderImpl::
       bus_(bus),
       delegate_(std::move(delegate)),
       object_path_(object_path),
-      service_path_(service_path),
-      weak_ptr_factory_(this) {
-  VLOG(1) << "Created Bluetooth GATT characteristic: " << object_path.value()
-          << " UUID: " << uuid;
+      service_path_(service_path) {
+  DVLOG(1) << "Created Bluetooth GATT characteristic: " << object_path.value()
+           << " UUID: " << uuid;
   if (!bus_)
     return;
 
@@ -58,66 +58,79 @@ BluetoothGattCharacteristicServiceProviderImpl::
   // org.freedesktop.DBus.Properties interface:
   exported_object_->ExportMethod(
       dbus::kDBusPropertiesInterface, dbus::kDBusPropertiesGet,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::Get,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(&BluetoothGattCharacteristicServiceProviderImpl::Get,
+                          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
   exported_object_->ExportMethod(
       dbus::kDBusPropertiesInterface, dbus::kDBusPropertiesSet,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::Set,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(&BluetoothGattCharacteristicServiceProviderImpl::Set,
+                          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
   exported_object_->ExportMethod(
       dbus::kDBusPropertiesInterface, dbus::kDBusPropertiesGetAll,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::GetAll,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &BluetoothGattCharacteristicServiceProviderImpl::GetAll,
+          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
 
   // org.bluez.GattCharacteristic1 interface:
   exported_object_->ExportMethod(
       bluetooth_gatt_characteristic::kBluetoothGattCharacteristicInterface,
       bluetooth_gatt_characteristic::kReadValue,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::ReadValue,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &BluetoothGattCharacteristicServiceProviderImpl::ReadValue,
+          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
   exported_object_->ExportMethod(
       bluetooth_gatt_characteristic::kBluetoothGattCharacteristicInterface,
       bluetooth_gatt_characteristic::kWriteValue,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::WriteValue,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &BluetoothGattCharacteristicServiceProviderImpl::WriteValue,
+          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
   exported_object_->ExportMethod(
       bluetooth_gatt_characteristic::kBluetoothGattCharacteristicInterface,
       bluetooth_gatt_characteristic::kPrepareWriteValue,
-      base::Bind(
+      base::BindRepeating(
           &BluetoothGattCharacteristicServiceProviderImpl::PrepareWriteValue,
           weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
   exported_object_->ExportMethod(
       bluetooth_gatt_characteristic::kBluetoothGattCharacteristicInterface,
       bluetooth_gatt_characteristic::kStartNotify,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::StartNotify,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &BluetoothGattCharacteristicServiceProviderImpl::StartNotify,
+          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
   exported_object_->ExportMethod(
       bluetooth_gatt_characteristic::kBluetoothGattCharacteristicInterface,
       bluetooth_gatt_characteristic::kStopNotify,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::StopNotify,
-                 weak_ptr_factory_.GetWeakPtr()),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnExported,
-                 weak_ptr_factory_.GetWeakPtr()));
+      base::BindRepeating(
+          &BluetoothGattCharacteristicServiceProviderImpl::StopNotify,
+          weak_ptr_factory_.GetWeakPtr()),
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnExported,
+          weak_ptr_factory_.GetWeakPtr()));
 }
 
 BluetoothGattCharacteristicServiceProviderImpl::
     ~BluetoothGattCharacteristicServiceProviderImpl() {
-  VLOG(1) << "Cleaning up Bluetooth GATT characteristic: "
-          << object_path_.value();
+  DVLOG(1) << "Cleaning up Bluetooth GATT characteristic: "
+           << object_path_.value();
   if (bus_)
     bus_->UnregisterExportedObject(object_path_);
 }
@@ -128,7 +141,7 @@ void BluetoothGattCharacteristicServiceProviderImpl::SendValueChanged(
   if (!bus_)
     return;
 
-  VLOG(2) << "Emitting a PropertiesChanged signal for characteristic value.";
+  DVLOG(2) << "Emitting a PropertiesChanged signal for characteristic value.";
   dbus::Signal signal(dbus::kDBusPropertiesInterface,
                       dbus::kDBusPropertiesChangedSignal);
   dbus::MessageWriter writer(&signal);
@@ -165,8 +178,8 @@ bool BluetoothGattCharacteristicServiceProviderImpl::OnOriginThread() {
 void BluetoothGattCharacteristicServiceProviderImpl::Get(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(2) << "BluetoothGattCharacteristicServiceProvider::Get: "
-          << object_path_.value();
+  DVLOG(2) << "BluetoothGattCharacteristicServiceProvider::Get: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -178,7 +191,7 @@ void BluetoothGattCharacteristicServiceProviderImpl::Get(
     std::unique_ptr<dbus::ErrorResponse> error_response =
         dbus::ErrorResponse::FromMethodCall(method_call, kErrorInvalidArgs,
                                             "Expected 'ss'.");
-    response_sender.Run(std::move(error_response));
+    std::move(response_sender).Run(std::move(error_response));
     return;
   }
 
@@ -189,7 +202,7 @@ void BluetoothGattCharacteristicServiceProviderImpl::Get(
         dbus::ErrorResponse::FromMethodCall(
             method_call, kErrorInvalidArgs,
             "No such interface: '" + interface_name + "'.");
-    response_sender.Run(std::move(error_response));
+    std::move(response_sender).Run(std::move(error_response));
     return;
   }
 
@@ -216,28 +229,28 @@ void BluetoothGattCharacteristicServiceProviderImpl::Get(
         "No such property: '" + property_name + "'.");
   }
 
-  response_sender.Run(std::move(response));
+  std::move(response_sender).Run(std::move(response));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::Set(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(2) << "BluetoothGattCharacteristicServiceProviderImpl::Set: "
-          << object_path_.value();
+  DVLOG(2) << "BluetoothGattCharacteristicServiceProviderImpl::Set: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
   // All of the properties on this interface are read-only, so just return
   // error.
   std::unique_ptr<dbus::ErrorResponse> error_response =
       dbus::ErrorResponse::FromMethodCall(method_call, kErrorPropertyReadOnly,
                                           "All properties are read-only.");
-  response_sender.Run(std::move(error_response));
+  std::move(response_sender).Run(std::move(error_response));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::GetAll(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(2) << "BluetoothGattCharacteristicServiceProvider::GetAll: "
-          << object_path_.value();
+  DVLOG(2) << "BluetoothGattCharacteristicServiceProvider::GetAll: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -247,7 +260,7 @@ void BluetoothGattCharacteristicServiceProviderImpl::GetAll(
     std::unique_ptr<dbus::ErrorResponse> error_response =
         dbus::ErrorResponse::FromMethodCall(method_call, kErrorInvalidArgs,
                                             "Expected 's'.");
-    response_sender.Run(std::move(error_response));
+    std::move(response_sender).Run(std::move(error_response));
     return;
   }
 
@@ -258,7 +271,7 @@ void BluetoothGattCharacteristicServiceProviderImpl::GetAll(
         dbus::ErrorResponse::FromMethodCall(
             method_call, kErrorInvalidArgs,
             "No such interface: '" + interface_name + "'.");
-    response_sender.Run(std::move(error_response));
+    std::move(response_sender).Run(std::move(error_response));
     return;
   }
 
@@ -266,14 +279,14 @@ void BluetoothGattCharacteristicServiceProviderImpl::GetAll(
       dbus::Response::FromMethodCall(method_call);
   dbus::MessageWriter writer(response.get());
   WriteProperties(&writer);
-  response_sender.Run(std::move(response));
+  std::move(response_sender).Run(std::move(response));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::ReadValue(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(3) << "BluetoothGattCharacteristicServiceProvider::ReadValue: "
-          << object_path_.value();
+  DVLOG(3) << "BluetoothGattCharacteristicServiceProvider::ReadValue: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -291,20 +304,26 @@ void BluetoothGattCharacteristicServiceProviderImpl::ReadValue(
     // the delegate, which should know how to handle it.
   }
 
+  // GetValue() promises to only call either the success or error callback.
+  auto response_sender_adapted =
+      base::AdaptCallbackForRepeating(std::move(response_sender));
+
   DCHECK(delegate_);
   delegate_->GetValue(
       device_path,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnReadValue,
-                 weak_ptr_factory_.GetWeakPtr(), method_call, response_sender),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnFailure,
-                 weak_ptr_factory_.GetWeakPtr(), method_call, response_sender));
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnReadValue,
+          weak_ptr_factory_.GetWeakPtr(), method_call, response_sender_adapted),
+      base::BindOnce(&BluetoothGattCharacteristicServiceProviderImpl::OnFailure,
+                     weak_ptr_factory_.GetWeakPtr(), method_call,
+                     response_sender_adapted));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::WriteValue(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(3) << "BluetoothGattCharacteristicServiceProvider::WriteValue: "
-          << object_path_.value();
+  DVLOG(3) << "BluetoothGattCharacteristicServiceProvider::WriteValue: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -334,20 +353,26 @@ void BluetoothGattCharacteristicServiceProviderImpl::WriteValue(
     // the delegate, which should know how to handle it.
   }
 
+  // SetValue() promises to only call either the success or error callback.
+  auto response_sender_adapted =
+      base::AdaptCallbackForRepeating(std::move(response_sender));
+
   DCHECK(delegate_);
   delegate_->SetValue(
       device_path, value,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnWriteValue,
-                 weak_ptr_factory_.GetWeakPtr(), method_call, response_sender),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnFailure,
-                 weak_ptr_factory_.GetWeakPtr(), method_call, response_sender));
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnWriteValue,
+          weak_ptr_factory_.GetWeakPtr(), method_call, response_sender_adapted),
+      base::BindOnce(&BluetoothGattCharacteristicServiceProviderImpl::OnFailure,
+                     weak_ptr_factory_.GetWeakPtr(), method_call,
+                     response_sender_adapted));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::PrepareWriteValue(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(3) << "BluetoothGattCharacteristicServiceProvider::PrepareWriteValue: "
-          << object_path_.value();
+  DVLOG(3) << "BluetoothGattCharacteristicServiceProvider::PrepareWriteValue: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -384,20 +409,27 @@ void BluetoothGattCharacteristicServiceProviderImpl::PrepareWriteValue(
     // the delegate, which should know how to handle it.
   }
 
+  // PrepareSetValue() promises to only call either the success or error
+  // callback.
+  auto response_sender_adapted =
+      base::AdaptCallbackForRepeating(std::move(response_sender));
+
   DCHECK(delegate_);
   delegate_->PrepareSetValue(
       device_path, value, offset, has_subsequent_write,
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnWriteValue,
-                 weak_ptr_factory_.GetWeakPtr(), method_call, response_sender),
-      base::Bind(&BluetoothGattCharacteristicServiceProviderImpl::OnFailure,
-                 weak_ptr_factory_.GetWeakPtr(), method_call, response_sender));
+      base::BindOnce(
+          &BluetoothGattCharacteristicServiceProviderImpl::OnWriteValue,
+          weak_ptr_factory_.GetWeakPtr(), method_call, response_sender_adapted),
+      base::BindOnce(&BluetoothGattCharacteristicServiceProviderImpl::OnFailure,
+                     weak_ptr_factory_.GetWeakPtr(), method_call,
+                     response_sender_adapted));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::StartNotify(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(3) << "BluetoothGattCharacteristicServiceProvider::StartNotify: "
-          << object_path_.value();
+  DVLOG(3) << "BluetoothGattCharacteristicServiceProvider::StartNotify: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -427,8 +459,8 @@ void BluetoothGattCharacteristicServiceProviderImpl::StartNotify(
 void BluetoothGattCharacteristicServiceProviderImpl::StopNotify(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(3) << "BluetoothGattCharacteristicServiceProvider::StopNotify: "
-          << object_path_.value();
+  DVLOG(3) << "BluetoothGattCharacteristicServiceProvider::StopNotify: "
+           << object_path_.value();
   DCHECK(OnOriginThread());
 
   dbus::MessageReader reader(method_call);
@@ -447,32 +479,32 @@ void BluetoothGattCharacteristicServiceProviderImpl::OnExported(
     const std::string& interface_name,
     const std::string& method_name,
     bool success) {
-  LOG_IF(WARNING, !success) << "Failed to export " << interface_name << "."
-                            << method_name;
+  DVLOG_IF(1, !success) << "Failed to export " << interface_name << "."
+                        << method_name;
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::OnReadValue(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender,
     const std::vector<uint8_t>& value) {
-  VLOG(3) << "Characteristic value obtained from delegate. Responding to "
-             "ReadValue.";
+  DVLOG(3) << "Characteristic value obtained from delegate. Responding to "
+              "ReadValue.";
 
   std::unique_ptr<dbus::Response> response =
       dbus::Response::FromMethodCall(method_call);
   dbus::MessageWriter writer(response.get());
   writer.AppendArrayOfBytes(value.data(), value.size());
-  response_sender.Run(std::move(response));
+  std::move(response_sender).Run(std::move(response));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::OnWriteValue(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(3) << "Responding to WriteValue.";
+  DVLOG(3) << "Responding to WriteValue.";
 
   std::unique_ptr<dbus::Response> response =
       dbus::Response::FromMethodCall(method_call);
-  response_sender.Run(std::move(response));
+  std::move(response_sender).Run(std::move(response));
 }
 
 void BluetoothGattCharacteristicServiceProviderImpl::WriteProperties(
@@ -510,11 +542,11 @@ void BluetoothGattCharacteristicServiceProviderImpl::WriteProperties(
 void BluetoothGattCharacteristicServiceProviderImpl::OnFailure(
     dbus::MethodCall* method_call,
     dbus::ExportedObject::ResponseSender response_sender) {
-  VLOG(2) << "Failed to get/set characteristic value. Report error.";
+  DVLOG(2) << "Failed to get/set characteristic value. Report error.";
   std::unique_ptr<dbus::ErrorResponse> error_response =
       dbus::ErrorResponse::FromMethodCall(
           method_call, kErrorFailed, "Failed to get/set characteristic value.");
-  response_sender.Run(std::move(error_response));
+  std::move(response_sender).Run(std::move(error_response));
 }
 
 const dbus::ObjectPath&
