@@ -7,12 +7,16 @@
 #include <memory>
 #include <utility>
 
+#include "ash/constants/ash_features.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/feature_list.h"
+#include "base/logging.h"
 #include "base/memory/ptr_util.h"
-#include "base/sys_info.h"
+#include "base/system/sys_info.h"
 #include "base/threading/thread.h"
 #include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
 #include "dbus/bus.h"
 #include "dbus/dbus_statistics.h"
 #include "dbus/message.h"
@@ -21,6 +25,8 @@
 #include "device/base/features.h"
 #include "device/bluetooth/dbus/bluetooth_adapter_client.h"
 #include "device/bluetooth/dbus/bluetooth_agent_manager_client.h"
+#include "device/bluetooth/dbus/bluetooth_battery_client.h"
+#include "device/bluetooth/dbus/bluetooth_debug_manager_client.h"
 #include "device/bluetooth/dbus/bluetooth_device_client.h"
 #include "device/bluetooth/dbus/bluetooth_gatt_characteristic_client.h"
 #include "device/bluetooth/dbus/bluetooth_gatt_descriptor_client.h"
@@ -28,16 +34,10 @@
 #include "device/bluetooth/dbus/bluetooth_gatt_service_client.h"
 #include "device/bluetooth/dbus/bluetooth_input_client.h"
 #include "device/bluetooth/dbus/bluetooth_le_advertising_manager_client.h"
-#include "device/bluetooth/dbus/bluetooth_media_client.h"
-#include "device/bluetooth/dbus/bluetooth_media_transport_client.h"
 #include "device/bluetooth/dbus/bluetooth_profile_manager_client.h"
 #include "device/bluetooth/dbus/bluez_dbus_thread_manager.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 
-#if defined(OS_CHROMEOS)
-#include "chromeos/dbus/dbus_thread_manager.h"
-#endif
-
 namespace bluez {
 
 static BluezDBusManager* g_bluez_dbus_manager = nullptr;
@@ -49,12 +49,11 @@ BluezDBusManager::BluezDBusManager(dbus::Bus* bus,
     : bus_(bus),
       alternate_bus_(alternate_bus),
       object_manager_support_known_(false),
-      object_manager_supported_(false),
-      weak_ptr_factory_(this) {
+      object_manager_supported_(false) {
   // On Chrome OS, Bluez might not be ready by the time we initialize the
   // BluezDBusManager so we initialize the clients anyway.
   bool should_check_object_manager = true;
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
   should_check_object_manager = false;
 #endif
 
@@ -71,7 +70,7 @@ BluezDBusManager::BluezDBusManager(dbus::Bus* bus,
                                dbus::kObjectManagerGetManagedObjects);
   GetSystemBus()
       ->GetObjectProxy(
-          GetBluetoothServiceName(),
+          bluez_object_manager::kBluezObjectManagerServiceName,
           dbus::ObjectPath(
               bluetooth_object_manager::kBluetoothObjectManagerServicePath))
       ->CallMethodWithErrorCallback(
@@ -92,8 +91,8 @@ dbus::Bus* bluez::BluezDBusManager::GetSystemBus() {
 }
 
 void BluezDBusManager::CallWhenObjectManagerSupportIsKnown(
-    base::Closure callback) {
-  object_manager_support_known_callback_ = callback;
+    base::OnceClosure callback) {
+  object_manager_support_known_callback_ = std::move(callback);
 }
 
 BluetoothAdapterClient* bluez::BluezDBusManager::GetBluetoothAdapterClient() {
@@ -113,6 +112,17 @@ bluez::BluezDBusManager::GetBluetoothAgentManagerClient() {
   return client_bundle_->bluetooth_agent_manager_client();
 }
 
+BluetoothDebugManagerClient*
+bluez::BluezDBusManager::GetBluetoothDebugManagerClient() {
+  DCHECK(object_manager_support_known_);
+  return client_bundle_->bluetooth_debug_manager_client();
+}
+
+BluetoothBatteryClient* bluez::BluezDBusManager::GetBluetoothBatteryClient() {
+  DCHECK(object_manager_support_known_);
+  return client_bundle_->bluetooth_battery_client();
+}
+
 BluetoothDeviceClient* bluez::BluezDBusManager::GetBluetoothDeviceClient() {
   DCHECK(object_manager_support_known_);
   return client_bundle_->bluetooth_device_client();
@@ -147,17 +157,6 @@ BluetoothInputClient* bluez::BluezDBusManager::GetBluetoothInputClient() {
   return client_bundle_->bluetooth_input_client();
 }
 
-BluetoothMediaClient* bluez::BluezDBusManager::GetBluetoothMediaClient() {
-  DCHECK(object_manager_support_known_);
-  return client_bundle_->bluetooth_media_client();
-}
-
-BluetoothMediaTransportClient*
-bluez::BluezDBusManager::GetBluetoothMediaTransportClient() {
-  DCHECK(object_manager_support_known_);
-  return client_bundle_->bluetooth_media_transport_client();
-}
-
 BluetoothProfileManagerClient*
 bluez::BluezDBusManager::GetBluetoothProfileManagerClient() {
   DCHECK(object_manager_support_known_);
@@ -169,36 +168,38 @@ BluetoothAdapterClient* BluezDBusManager::GetAlternateBluetoothAdapterClient() {
   return client_bundle_->alternate_bluetooth_adapter_client();
 }
 
+BluetoothDeviceClient* BluezDBusManager::GetAlternateBluetoothDeviceClient() {
+  DCHECK(object_manager_support_known_);
+  return client_bundle_->alternate_bluetooth_device_client();
+}
+
 void BluezDBusManager::OnObjectManagerSupported(dbus::Response* response) {
-  VLOG(1) << "Bluetooth supported. Initializing clients.";
+  DVLOG(1) << "Bluetooth supported. Initializing clients.";
   object_manager_supported_ = true;
 
   client_bundle_.reset(new BluetoothDBusClientBundle(false /* use_fakes */));
   InitializeClients();
 
   object_manager_support_known_ = true;
-  if (!object_manager_support_known_callback_.is_null()) {
-    object_manager_support_known_callback_.Run();
-    object_manager_support_known_callback_.Reset();
-  }
+  if (object_manager_support_known_callback_)
+    std::move(object_manager_support_known_callback_).Run();
 }
 
 void BluezDBusManager::OnObjectManagerNotSupported(
     dbus::ErrorResponse* response) {
-  VLOG(1) << "Bluetooth not supported.";
+  DVLOG(1) << "Bluetooth not supported.";
   object_manager_supported_ = false;
 
   // We don't initialize clients since the clients need ObjectManager.
 
   object_manager_support_known_ = true;
-  if (!object_manager_support_known_callback_.is_null()) {
-    object_manager_support_known_callback_.Run();
-    object_manager_support_known_callback_.Reset();
-  }
+  if (object_manager_support_known_callback_)
+    std::move(object_manager_support_known_callback_).Run();
 }
 
 void BluezDBusManager::InitializeClients() {
-  std::string bluetooth_service_name = GetBluetoothServiceName();
+  std::string bluetooth_service_name =
+      bluez_object_manager::kBluezObjectManagerServiceName;
   client_bundle_->bluetooth_adapter_client()->Init(GetSystemBus(),
                                                    bluetooth_service_name);
   client_bundle_->bluetooth_agent_manager_client()->Init(
@@ -217,33 +218,30 @@ void BluezDBusManager::InitializeClients() {
                                                  bluetooth_service_name);
   client_bundle_->bluetooth_le_advertising_manager_client()->Init(
       GetSystemBus(), bluetooth_service_name);
-  client_bundle_->bluetooth_media_client()->Init(GetSystemBus(),
-                                                 bluetooth_service_name);
-  client_bundle_->bluetooth_media_transport_client()->Init(
-      GetSystemBus(), bluetooth_service_name);
   client_bundle_->bluetooth_profile_manager_client()->Init(
       GetSystemBus(), bluetooth_service_name);
 
+  // TODO(b/145163508): update service name after migrating BT debug to bluez
+  client_bundle_->bluetooth_debug_manager_client()->Init(
+      GetSystemBus(),
+      bluetooth_object_manager::kBluetoothObjectManagerServiceName);
+
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  client_bundle_->bluetooth_battery_client()->Init(GetSystemBus(),
+                                                   bluetooth_service_name);
+#endif
+
   if (!alternate_bus_)
     return;
 
   client_bundle_->alternate_bluetooth_adapter_client()->Init(
       alternate_bus_, bluetooth_service_name);
-}
-
-std::string BluezDBusManager::GetBluetoothServiceName() {
-  bool use_newblue = false;
-#if defined(OS_CHROMEOS)
-  use_newblue = base::FeatureList::IsEnabled(device::kNewblueDaemon);
-#endif  // defined(OS_CHROMEOS)
-
-  return use_newblue
-             ? bluetooth_object_manager::kBluetoothObjectManagerServiceName
-             : bluez_object_manager::kBluezObjectManagerServiceName;
+  client_bundle_->alternate_bluetooth_device_client()->Init(
+      alternate_bus_, bluetooth_service_name);
 }
 
 // static
-void BluezDBusManager::Initialize() {
+void BluezDBusManager::Initialize(dbus::Bus* system_bus) {
   // If we initialize BluezDBusManager twice we may also be shutting it down
   // early; do not allow that.
   if (g_using_bluez_dbus_manager_for_testing)
@@ -251,24 +249,35 @@ void BluezDBusManager::Initialize() {
 
   CHECK(!g_bluez_dbus_manager);
 
-#if defined(OS_CHROMEOS)
+  BluezDBusThreadManager::Initialize();
+
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  DCHECK(system_bus);
   // On ChromeOS, BluetoothSystem needs a separate connection to Bluez, so we
   // use BluezDBusThreadManager to get two different connections to the same
   // services. This allows us to have two separate sets of clients in the same
   // process.
-  BluezDBusThreadManager::Initialize();
-
-  CreateGlobalInstance(chromeos::DBusThreadManager::Get()->GetSystemBus(),
+  CreateGlobalInstance(system_bus,
                        BluezDBusThreadManager::Get()->GetSystemBus(),
-                       chromeos::DBusThreadManager::Get()->IsUsingFakes());
-#elif defined(OS_LINUX)
+                       false /* use_dbus_stubs */);
+#elif defined(OS_LINUX) || BUILDFLAG(IS_CHROMEOS_LACROS)
   // BluetoothSystem, the client that needs the extra connection, is not
   // implemented on Linux, so no need for an extra Bus.
   CreateGlobalInstance(BluezDBusThreadManager::Get()->GetSystemBus(), nullptr,
                        false /* use_dbus_stubs */);
+#else
+  NOTREACHED();
 #endif
 }
 
+void BluezDBusManager::InitializeFake() {
+  if (g_using_bluez_dbus_manager_for_testing)
+    return;
+  CHECK(!g_bluez_dbus_manager);
+  BluezDBusThreadManager::Initialize();
+  CreateGlobalInstance(nullptr, nullptr, true /* use_dbus_stubs */);
+}
+
 // static
 std::unique_ptr<BluezDBusManagerSetter>
 bluez::BluezDBusManager::GetSetterForTesting() {
@@ -301,13 +310,13 @@ void BluezDBusManager::Shutdown() {
   g_bluez_dbus_manager = nullptr;
   delete dbus_manager;
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
   if (!g_using_bluez_dbus_manager_for_testing)
     BluezDBusThreadManager::Shutdown();
 #endif
 
   g_using_bluez_dbus_manager_for_testing = false;
-  VLOG(1) << "BluezDBusManager Shutdown completed";
+  DVLOG(1) << "BluezDBusManager Shutdown completed";
 }
 
 // static
@@ -340,6 +349,18 @@ void BluezDBusManagerSetter::SetBluetoothAgentManagerClient(
       ->client_bundle_->bluetooth_agent_manager_client_ = std::move(client);
 }
 
+void BluezDBusManagerSetter::SetBluetoothBatteryClient(
+    std::unique_ptr<BluetoothBatteryClient> client) {
+  bluez::BluezDBusManager::Get()->client_bundle_->bluetooth_battery_client_ =
+      std::move(client);
+}
+
+void BluezDBusManagerSetter::SetBluetoothDebugManagerClient(
+    std::unique_ptr<BluetoothDebugManagerClient> client) {
+  bluez::BluezDBusManager::Get()
+      ->client_bundle_->bluetooth_debug_manager_client_ = std::move(client);
+}
+
 void BluezDBusManagerSetter::SetBluetoothDeviceClient(
     std::unique_ptr<BluetoothDeviceClient> client) {
   bluez::BluezDBusManager::Get()->client_bundle_->bluetooth_device_client_ =
@@ -377,18 +398,6 @@ void BluezDBusManagerSetter::SetBluetoothInputClient(
       std::move(client);
 }
 
-void BluezDBusManagerSetter::SetBluetoothMediaClient(
-    std::unique_ptr<BluetoothMediaClient> client) {
-  bluez::BluezDBusManager::Get()->client_bundle_->bluetooth_media_client_ =
-      std::move(client);
-}
-
-void BluezDBusManagerSetter::SetBluetoothMediaTransportClient(
-    std::unique_ptr<BluetoothMediaTransportClient> client) {
-  bluez::BluezDBusManager::Get()
-      ->client_bundle_->bluetooth_media_transport_client_ = std::move(client);
-}
-
 void BluezDBusManagerSetter::SetBluetoothProfileManagerClient(
     std::unique_ptr<BluetoothProfileManagerClient> client) {
   bluez::BluezDBusManager::Get()
@@ -401,4 +410,10 @@ void BluezDBusManagerSetter::SetAlternateBluetoothAdapterClient(
       ->client_bundle_->alternate_bluetooth_adapter_client_ = std::move(client);
 }
 
+void BluezDBusManagerSetter::SetAlternateBluetoothDeviceClient(
+    std::unique_ptr<BluetoothDeviceClient> client) {
+  bluez::BluezDBusManager::Get()
+      ->client_bundle_->alternate_bluetooth_device_client_ = std::move(client);
+}
+
 }  // namespace bluez
