@@ -7,8 +7,8 @@
 #include <map>
 #include <utility>
 
-#include "base/bind_helpers.h"
 #include "base/callback_forward.h"
+#include "base/callback_helpers.h"
 #include "base/location.h"
 #include "base/logging.h"
 #include "base/single_thread_task_runner.h"
@@ -51,25 +51,25 @@ FakeBluetoothAdapterClient::Properties::~Properties() = default;
 void FakeBluetoothAdapterClient::Properties::Get(
     dbus::PropertyBase* property,
     dbus::PropertySet::GetCallback callback) {
-  VLOG(1) << "Get " << property->name();
-  callback.Run(false);
+  DVLOG(1) << "Get " << property->name();
+  std::move(callback).Run(false);
 }
 
 void FakeBluetoothAdapterClient::Properties::GetAll() {
-  VLOG(1) << "GetAll";
+  DVLOG(1) << "GetAll";
 }
 
 void FakeBluetoothAdapterClient::Properties::Set(
     dbus::PropertyBase* property,
     dbus::PropertySet::SetCallback callback) {
-  VLOG(1) << "Set " << property->name();
+  DVLOG(1) << "Set " << property->name();
   if (property->name() == powered.name() || property->name() == alias.name() ||
       property->name() == discoverable.name() ||
       property->name() == discoverable_timeout.name()) {
-    callback.Run(true);
+    std::move(callback).Run(true);
     property->ReplaceValueWithSetValue();
   } else {
-    callback.Run(false);
+    std::move(callback).Run(false);
   }
 }
 
@@ -80,16 +80,16 @@ FakeBluetoothAdapterClient::FakeBluetoothAdapterClient()
       set_discovery_filter_should_fail_(false),
       simulation_interval_ms_(kSimulationIntervalMs),
       last_handle_(0) {
-  properties_.reset(new Properties(base::Bind(
-      &FakeBluetoothAdapterClient::OnPropertyChanged, base::Unretained(this))));
+  properties_ = std::make_unique<Properties>(base::BindRepeating(
+      &FakeBluetoothAdapterClient::OnPropertyChanged, base::Unretained(this)));
 
   properties_->address.ReplaceValue(kAdapterAddress);
   properties_->name.ReplaceValue("Fake Adapter (Name)");
   properties_->alias.ReplaceValue(kAdapterName);
   properties_->pairable.ReplaceValue(true);
 
-  second_properties_.reset(new Properties(base::Bind(
-      &FakeBluetoothAdapterClient::OnPropertyChanged, base::Unretained(this))));
+  second_properties_ = std::make_unique<Properties>(base::BindRepeating(
+      &FakeBluetoothAdapterClient::OnPropertyChanged, base::Unretained(this)));
 
   second_properties_->address.ReplaceValue(kSecondAdapterAddress);
   second_properties_->name.ReplaceValue("Second Fake Adapter (Name)");
@@ -99,7 +99,9 @@ FakeBluetoothAdapterClient::FakeBluetoothAdapterClient()
 
 FakeBluetoothAdapterClient::~FakeBluetoothAdapterClient() = default;
 
-void FakeBluetoothAdapterClient::Init(dbus::Bus* bus) {}
+void FakeBluetoothAdapterClient::Init(
+    dbus::Bus* bus,
+    const std::string& bluetooth_service_name) {}
 
 void FakeBluetoothAdapterClient::AddObserver(Observer* observer) {
   observers_.AddObserver(observer);
@@ -130,47 +132,58 @@ FakeBluetoothAdapterClient::GetProperties(const dbus::ObjectPath& object_path) {
 
 void FakeBluetoothAdapterClient::StartDiscovery(
     const dbus::ObjectPath& object_path,
-    const base::Closure& callback,
-    const ErrorCallback& error_callback) {
+    ResponseCallback callback) {
   if (object_path != dbus::ObjectPath(kAdapterPath)) {
-    PostDelayedTask(base::Bind(error_callback, kNoResponseError, ""));
+    PostDelayedTask(
+        base::BindOnce(std::move(callback), Error(kNoResponseError, "")));
+    return;
+  }
+
+  if (set_start_discovery_should_fail_) {
+    set_start_discovery_should_fail_ = false;
+    PostDelayedTask(
+        base::BindOnce(std::move(callback), Error(kUnknownAdapterError, "")));
     return;
   }
 
   ++discovering_count_;
-  VLOG(1) << "StartDiscovery: " << object_path.value() << ", "
-          << "count is now " << discovering_count_;
-  PostDelayedTask(callback);
+  DVLOG(1) << "StartDiscovery: " << object_path.value() << ", "
+           << "count is now " << discovering_count_;
+  PostDelayedTask(base::BindOnce(std::move(callback), base::nullopt));
 
   if (discovering_count_ == 1) {
-    properties_->discovering.ReplaceValue(true);
-
+    PostDelayedTask(
+        base::BindOnce(&FakeBluetoothAdapterClient::UpdateDiscoveringProperty,
+                       weak_ptr_factory_.GetWeakPtr(), /*discovering=*/true));
     FakeBluetoothDeviceClient* device_client =
         static_cast<FakeBluetoothDeviceClient*>(
             bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient());
-    device_client->BeginDiscoverySimulation(dbus::ObjectPath(kAdapterPath));
+    if (enable_discovery_simulation_) {
+      device_client->BeginDiscoverySimulation(dbus::ObjectPath(kAdapterPath));
+    }
   }
 }
 
 void FakeBluetoothAdapterClient::StopDiscovery(
     const dbus::ObjectPath& object_path,
-    const base::Closure& callback,
-    const ErrorCallback& error_callback) {
+    ResponseCallback callback) {
   if (object_path != dbus::ObjectPath(kAdapterPath)) {
-    PostDelayedTask(base::Bind(error_callback, kNoResponseError, ""));
+    PostDelayedTask(
+        base::BindOnce(std::move(callback), Error(kNoResponseError, "")));
     return;
   }
 
   if (!discovering_count_) {
     LOG(WARNING) << "StopDiscovery called when not discovering";
-    PostDelayedTask(base::Bind(error_callback, kNoResponseError, ""));
+    PostDelayedTask(
+        base::BindOnce(std::move(callback), Error(kNoResponseError, "")));
     return;
   }
 
   --discovering_count_;
-  VLOG(1) << "StopDiscovery: " << object_path.value() << ", "
-          << "count is now " << discovering_count_;
-  PostDelayedTask(callback);
+  DVLOG(1) << "StopDiscovery: " << object_path.value() << ", "
+           << "count is now " << discovering_count_;
+  PostDelayedTask(base::BindOnce(std::move(callback), base::nullopt));
 
   if (discovering_count_ == 0) {
     FakeBluetoothDeviceClient* device_client =
@@ -184,23 +197,29 @@ void FakeBluetoothAdapterClient::StopDiscovery(
     }
 
     discovery_filter_.reset();
-    properties_->discovering.ReplaceValue(false);
+    PostDelayedTask(
+        base::BindOnce(&FakeBluetoothAdapterClient::UpdateDiscoveringProperty,
+                       weak_ptr_factory_.GetWeakPtr(), /*discovering=*/false));
   }
 }
 
+void FakeBluetoothAdapterClient::UpdateDiscoveringProperty(bool discovering) {
+  properties_->discovering.ReplaceValue(discovering);
+}
+
 void FakeBluetoothAdapterClient::RemoveDevice(
     const dbus::ObjectPath& object_path,
     const dbus::ObjectPath& device_path,
-    const base::Closure& callback,
-    const ErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ErrorCallback error_callback) {
   if (object_path != dbus::ObjectPath(kAdapterPath)) {
-    error_callback.Run(kNoResponseError, "");
+    std::move(error_callback).Run(kNoResponseError, "");
     return;
   }
 
-  VLOG(1) << "RemoveDevice: " << object_path.value() << " "
-          << device_path.value();
-  callback.Run();
+  DVLOG(1) << "RemoveDevice: " << object_path.value() << " "
+           << device_path.value();
+  std::move(callback).Run();
 
   FakeBluetoothDeviceClient* device_client =
       static_cast<FakeBluetoothDeviceClient*>(
@@ -212,52 +231,72 @@ void FakeBluetoothAdapterClient::MakeSetDiscoveryFilterFail() {
   set_discovery_filter_should_fail_ = true;
 }
 
+void FakeBluetoothAdapterClient::MakeStartDiscoveryFail() {
+  set_start_discovery_should_fail_ = true;
+}
+
+void FakeBluetoothAdapterClient::SetDiscoverySimulation(bool enabled) {
+  enable_discovery_simulation_ = enabled;
+}
+
 void FakeBluetoothAdapterClient::SetDiscoveryFilter(
     const dbus::ObjectPath& object_path,
     const DiscoveryFilter& discovery_filter,
-    const base::Closure& callback,
-    const ErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ErrorCallback error_callback) {
   if (object_path != dbus::ObjectPath(kAdapterPath)) {
-    PostDelayedTask(base::Bind(error_callback, kNoResponseError, ""));
+    PostDelayedTask(
+        base::BindOnce(std::move(error_callback), kNoResponseError, ""));
     return;
   }
-  VLOG(1) << "SetDiscoveryFilter: " << object_path.value();
+  DVLOG(1) << "SetDiscoveryFilter: " << object_path.value();
 
   if (set_discovery_filter_should_fail_) {
-    PostDelayedTask(base::Bind(error_callback, kNoResponseError, ""));
+    PostDelayedTask(
+        base::BindOnce(std::move(error_callback), kNoResponseError, ""));
     set_discovery_filter_should_fail_ = false;
     return;
   }
 
   discovery_filter_.reset(new DiscoveryFilter());
   discovery_filter_->CopyFrom(discovery_filter);
-  PostDelayedTask(callback);
+  PostDelayedTask(std::move(callback));
 }
 
 void FakeBluetoothAdapterClient::CreateServiceRecord(
     const dbus::ObjectPath& object_path,
     const bluez::BluetoothServiceRecordBlueZ& record,
-    const ServiceRecordCallback& callback,
-    const ErrorCallback& error_callback) {
+    ServiceRecordCallback callback,
+    ErrorCallback error_callback) {
   ++last_handle_;
   records_.insert(
       std::pair<uint32_t, BluetoothServiceRecordBlueZ>(last_handle_, record));
-  callback.Run(last_handle_);
+  std::move(callback).Run(last_handle_);
 }
 
 void FakeBluetoothAdapterClient::RemoveServiceRecord(
     const dbus::ObjectPath& object_path,
     uint32_t handle,
-    const base::Closure& callback,
-    const ErrorCallback& error_callback) {
+    base::OnceClosure callback,
+    ErrorCallback error_callback) {
   auto it = records_.find(handle);
   if (it == records_.end()) {
-    error_callback.Run(bluetooth_adapter::kErrorDoesNotExist,
-                       "Service record does not exist.");
+    std::move(error_callback)
+        .Run(bluetooth_adapter::kErrorDoesNotExist,
+             "Service record does not exist.");
     return;
   }
   records_.erase(it);
-  callback.Run();
+  std::move(callback).Run();
+}
+
+void FakeBluetoothAdapterClient::ConnectDevice(
+    const dbus::ObjectPath& object_path,
+    const std::string& address,
+    const base::Optional<AddressType>& address_type,
+    ConnectDeviceCallback callback,
+    ErrorCallback error_callback) {
+  NOTIMPLEMENTED();
 }
 
 void FakeBluetoothAdapterClient::SetSimulationIntervalMs(int interval_ms) {
@@ -321,7 +360,7 @@ void FakeBluetoothAdapterClient::OnPropertyChanged(
     const std::string& property_name) {
   if (property_name == properties_->powered.name() &&
       !properties_->powered.value()) {
-    VLOG(1) << "Adapter powered off";
+    DVLOG(1) << "Adapter powered off";
 
     if (discovering_count_) {
       discovering_count_ = 0;
@@ -335,10 +374,9 @@ void FakeBluetoothAdapterClient::OnPropertyChanged(
   }
 }
 
-void FakeBluetoothAdapterClient::PostDelayedTask(
-    const base::Closure& callback) {
+void FakeBluetoothAdapterClient::PostDelayedTask(base::OnceClosure callback) {
   base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, callback,
+      FROM_HERE, std::move(callback),
       base::TimeDelta::FromMilliseconds(simulation_interval_ms_));
 }
 
