@@ -83,41 +83,47 @@ class DEVICE_BLUETOOTH_EXPORT FakeBluetoothDeviceClient
       const dbus::ObjectPath& adapter_path) override;
   Properties* GetProperties(const dbus::ObjectPath& object_path) override;
   void Connect(const dbus::ObjectPath& object_path,
-               const base::Closure& callback,
-               const ErrorCallback& error_callback) override;
+               base::OnceClosure callback,
+               ErrorCallback error_callback) override;
+  void ConnectLE(const dbus::ObjectPath& object_path,
+                 base::OnceClosure callback,
+                 ErrorCallback error_callback) override;
   void Disconnect(const dbus::ObjectPath& object_path,
-                  const base::Closure& callback,
-                  const ErrorCallback& error_callback) override;
+                  base::OnceClosure callback,
+                  ErrorCallback error_callback) override;
+  void DisconnectLE(const dbus::ObjectPath& object_path,
+                    base::OnceClosure callback,
+                    ErrorCallback error_callback) override;
   void ConnectProfile(const dbus::ObjectPath& object_path,
                       const std::string& uuid,
-                      const base::Closure& callback,
-                      const ErrorCallback& error_callback) override;
+                      base::OnceClosure callback,
+                      ErrorCallback error_callback) override;
   void DisconnectProfile(const dbus::ObjectPath& object_path,
                          const std::string& uuid,
-                         const base::Closure& callback,
-                         const ErrorCallback& error_callback) override;
+                         base::OnceClosure callback,
+                         ErrorCallback error_callback) override;
   void Pair(const dbus::ObjectPath& object_path,
-            const base::Closure& callback,
-            const ErrorCallback& error_callback) override;
+            base::OnceClosure callback,
+            ErrorCallback error_callback) override;
   void CancelPairing(const dbus::ObjectPath& object_path,
-                     const base::Closure& callback,
-                     const ErrorCallback& error_callback) override;
+                     base::OnceClosure callback,
+                     ErrorCallback error_callback) override;
   void GetConnInfo(const dbus::ObjectPath& object_path,
-                   const ConnInfoCallback& callback,
-                   const ErrorCallback& error_callback) override;
+                   ConnInfoCallback callback,
+                   ErrorCallback error_callback) override;
   void SetLEConnectionParameters(const dbus::ObjectPath& object_path,
                                  const ConnectionParameters& conn_params,
-                                 const base::Closure& callback,
-                                 const ErrorCallback& error_callback) override;
+                                 base::OnceClosure callback,
+                                 ErrorCallback error_callback) override;
   void GetServiceRecords(const dbus::ObjectPath& object_path,
-                         const ServiceRecordsCallback& callback,
-                         const ErrorCallback& error_callback) override;
+                         ServiceRecordsCallback callback,
+                         ErrorCallback error_callback) override;
   void ExecuteWrite(const dbus::ObjectPath& object_path,
-                    const base::Closure& callback,
-                    const ErrorCallback& error_callback) override;
+                    base::OnceClosure callback,
+                    ErrorCallback error_callback) override;
   void AbortWrite(const dbus::ObjectPath& object_path,
-                  const base::Closure& callback,
-                  const ErrorCallback& error_callback) override;
+                  base::OnceClosure callback,
+                  ErrorCallback error_callback) override;
 
   void SetSimulationIntervalMs(int interval_ms);
 
@@ -140,7 +146,7 @@ class DEVICE_BLUETOOTH_EXPORT FakeBluetoothDeviceClient
   // Creates and returns a list of std::unique_ptr<base::DictionaryValue>
   // objects, which contain all the data from the constants for devices with
   // predefined behavior.
-  std::unique_ptr<base::ListValue> GetBluetoothDevicesAsDictionaries() const;
+  base::Value GetBluetoothDevicesAsDictionaries() const;
 
   SimulatedPairingOptions* GetPairingOptions(
       const dbus::ObjectPath& object_path);
@@ -155,8 +161,8 @@ class DEVICE_BLUETOOTH_EXPORT FakeBluetoothDeviceClient
   // |callback| will be called, on failure, |error_callback| is called.
   void SimulatePairing(const dbus::ObjectPath& object_path,
                        bool incoming_request,
-                       const base::Closure& callback,
-                       const ErrorCallback& error_callback);
+                       base::OnceClosure callback,
+                       ErrorCallback error_callback);
 
   // Updates the connection properties of the fake device that will be returned
   // by GetConnInfo.
@@ -195,6 +201,11 @@ class DEVICE_BLUETOOTH_EXPORT FakeBluetoothDeviceClient
       const std::map<std::string, std::vector<uint8_t>>& service_data,
       const std::map<uint16_t, std::vector<uint8_t>>& manufacturer_data);
 
+  // Updates the EIR property of fake device with object path |object_path| to
+  // |eir|, if the fake device exists.
+  void UpdateEIR(const dbus::ObjectPath& object_path,
+                 const std::vector<uint8_t>& eir);
+
   // Adds a pending prepare write request to |object_path|.
   void AddPrepareWriteRequest(const dbus::ObjectPath& object_path,
                               const std::vector<uint8_t>& value);
@@ -303,16 +314,16 @@ class DEVICE_BLUETOOTH_EXPORT FakeBluetoothDeviceClient
   void IncomingPairingSimulationTimer();
 
   void CompleteSimulatedPairing(const dbus::ObjectPath& object_path,
-                                const base::Closure& callback,
-                                const ErrorCallback& error_callback);
+                                base::OnceClosure callback,
+                                ErrorCallback error_callback);
   void TimeoutSimulatedPairing(const dbus::ObjectPath& object_path,
-                               const ErrorCallback& error_callback);
+                               ErrorCallback error_callback);
   void CancelSimulatedPairing(const dbus::ObjectPath& object_path,
-                              const ErrorCallback& error_callback);
+                              ErrorCallback error_callback);
   void RejectSimulatedPairing(const dbus::ObjectPath& object_path,
-                              const ErrorCallback& error_callback);
+                              ErrorCallback error_callback);
   void FailSimulatedPairing(const dbus::ObjectPath& object_path,
-                            const ErrorCallback& error_callback);
+                            ErrorCallback error_callback);
   void AddInputDeviceIfNeeded(const dbus::ObjectPath& object_path,
                               Properties* properties);
 
@@ -321,34 +332,34 @@ class DEVICE_BLUETOOTH_EXPORT FakeBluetoothDeviceClient
   void InvalidateDeviceRSSI(const dbus::ObjectPath& object_path);
 
   void PinCodeCallback(const dbus::ObjectPath& object_path,
-                       const base::Closure& callback,
-                       const ErrorCallback& error_callback,
+                       base::OnceClosure callback,
+                       ErrorCallback error_callback,
                        BluetoothAgentServiceProvider::Delegate::Status status,
                        const std::string& pincode);
   void PasskeyCallback(const dbus::ObjectPath& object_path,
-                       const base::Closure& callback,
-                       const ErrorCallback& error_callback,
+                       base::OnceClosure callback,
+                       ErrorCallback error_callback,
                        BluetoothAgentServiceProvider::Delegate::Status status,
                        uint32_t passkey);
   void ConfirmationCallback(
       const dbus::ObjectPath& object_path,
-      const base::Closure& callback,
-      const ErrorCallback& error_callback,
+      base::OnceClosure callback,
+      ErrorCallback error_callback,
       BluetoothAgentServiceProvider::Delegate::Status status);
   void SimulateKeypress(uint16_t entered,
                         const dbus::ObjectPath& object_path,
-                        const base::Closure& callback,
-                        const ErrorCallback& error_callback);
+                        base::OnceClosure callback,
+                        ErrorCallback error_callback);
 
   void ConnectionCallback(
       const dbus::ObjectPath& object_path,
-      const base::Closure& callback,
-      const ErrorCallback& error_callback,
+      base::OnceClosure callback,
+      ErrorCallback error_callback,
       BluetoothProfileServiceProvider::Delegate::Status status);
   void DisconnectionCallback(
       const dbus::ObjectPath& object_path,
-      const base::Closure& callback,
-      const ErrorCallback& error_callback,
+      base::OnceClosure callback,
+      ErrorCallback error_callback,
       BluetoothProfileServiceProvider::Delegate::Status status);
 
   // List of observers interested in event notifications from us.
