@@ -10,6 +10,7 @@
 #include "base/callback.h"
 #include "base/logging.h"
 #include "base/strings/string_util.h"
+#include "device/bluetooth/bluetooth_gatt_characteristic.h"
 #include "device/bluetooth/dbus/bluetooth_gatt_attribute_value_delegate.h"
 #include "device/bluetooth/dbus/bluez_dbus_manager.h"
 #include "device/bluetooth/dbus/fake_bluetooth_gatt_manager_client.h"
@@ -78,7 +79,8 @@ FakeBluetoothGattCharacteristicServiceProvider::
       flags_(flags),
       service_path_(service_path),
       delegate_(std::move(delegate)) {
-  VLOG(1) << "Creating Bluetooth GATT characteristic: " << object_path_.value();
+  DVLOG(1) << "Creating Bluetooth GATT characteristic: "
+           << object_path_.value();
 
   DCHECK(object_path_.IsValid());
   DCHECK(service_path_.IsValid());
@@ -96,8 +98,8 @@ FakeBluetoothGattCharacteristicServiceProvider::
 
 FakeBluetoothGattCharacteristicServiceProvider::
     ~FakeBluetoothGattCharacteristicServiceProvider() {
-  VLOG(1) << "Cleaning up Bluetooth GATT characteristic: "
-          << object_path_.value();
+  DVLOG(1) << "Cleaning up Bluetooth GATT characteristic: "
+           << object_path_.value();
   FakeBluetoothGattManagerClient* fake_bluetooth_gatt_manager_client =
       static_cast<FakeBluetoothGattManagerClient*>(
           bluez::BluezDBusManager::Get()->GetBluetoothGattManagerClient());
@@ -107,89 +109,124 @@ FakeBluetoothGattCharacteristicServiceProvider::
 
 void FakeBluetoothGattCharacteristicServiceProvider::SendValueChanged(
     const std::vector<uint8_t>& value) {
-  VLOG(1) << "Sent characteristic value changed: " << object_path_.value()
-          << " UUID: " << uuid_;
+  DVLOG(1) << "Sent characteristic value changed: " << object_path_.value()
+           << " UUID: " << uuid_;
   sent_value_ = value;
 }
 
 void FakeBluetoothGattCharacteristicServiceProvider::GetValue(
     const dbus::ObjectPath& device_path,
-    const device::BluetoothLocalGattService::Delegate::ValueCallback& callback,
-    const device::BluetoothLocalGattService::Delegate::ErrorCallback&
-        error_callback) {
-  VLOG(1) << "GATT characteristic value Get request: " << object_path_.value()
-          << " UUID: " << uuid_;
+    device::BluetoothLocalGattService::Delegate::ValueCallback callback,
+    device::BluetoothLocalGattService::Delegate::ErrorCallback error_callback) {
+  DVLOG(1) << "GATT characteristic value Get request: " << object_path_.value()
+           << " UUID: " << uuid_;
   // Check if this characteristic is registered.
   FakeBluetoothGattManagerClient* fake_bluetooth_gatt_manager_client =
       static_cast<FakeBluetoothGattManagerClient*>(
           bluez::BluezDBusManager::Get()->GetBluetoothGattManagerClient());
   if (!fake_bluetooth_gatt_manager_client->IsServiceRegistered(service_path_)) {
-    VLOG(1) << "GATT characteristic not registered.";
-    error_callback.Run();
+    DVLOG(1) << "GATT characteristic not registered.";
+    std::move(error_callback).Run();
     return;
   }
 
   if (!CanRead(flags_)) {
-    VLOG(1) << "GATT characteristic not readable.";
-    error_callback.Run();
+    DVLOG(1) << "GATT characteristic not readable.";
+    std::move(error_callback).Run();
     return;
   }
 
   // Pass on to the delegate.
   DCHECK(delegate_);
-  delegate_->GetValue(device_path, callback, error_callback);
+  delegate_->GetValue(device_path, std::move(callback),
+                      std::move(error_callback));
 }
 
 void FakeBluetoothGattCharacteristicServiceProvider::SetValue(
     const dbus::ObjectPath& device_path,
     const std::vector<uint8_t>& value,
-    const base::Closure& callback,
-    const device::BluetoothLocalGattService::Delegate::ErrorCallback&
-        error_callback) {
-  VLOG(1) << "GATT characteristic value Set request: " << object_path_.value()
-          << " UUID: " << uuid_;
+    base::OnceClosure callback,
+    device::BluetoothLocalGattService::Delegate::ErrorCallback error_callback) {
+  DVLOG(1) << "GATT characteristic value Set request: " << object_path_.value()
+           << " UUID: " << uuid_;
   // Check if this characteristic is registered.
   FakeBluetoothGattManagerClient* fake_bluetooth_gatt_manager_client =
       static_cast<FakeBluetoothGattManagerClient*>(
           bluez::BluezDBusManager::Get()->GetBluetoothGattManagerClient());
   if (!fake_bluetooth_gatt_manager_client->IsServiceRegistered(service_path_)) {
-    VLOG(1) << "GATT characteristic not registered.";
-    error_callback.Run();
+    DVLOG(1) << "GATT characteristic not registered.";
+    std::move(error_callback).Run();
     return;
   }
 
   if (!CanWrite(flags_)) {
-    VLOG(1) << "GATT characteristic not writeable.";
-    error_callback.Run();
+    DVLOG(1) << "GATT characteristic not writeable.";
+    std::move(error_callback).Run();
     return;
   }
 
   // Pass on to the delegate.
   DCHECK(delegate_);
-  delegate_->SetValue(device_path, value, callback, error_callback);
+  delegate_->SetValue(device_path, value, std::move(callback),
+                      std::move(error_callback));
 }
 
-bool FakeBluetoothGattCharacteristicServiceProvider::NotificationsChange(
-    bool start) {
-  VLOG(1) << "GATT characteristic value notification request: "
-          << object_path_.value() << " UUID: " << uuid_ << " start=" << start;
+void FakeBluetoothGattCharacteristicServiceProvider::PrepareSetValue(
+    const dbus::ObjectPath& device_path,
+    const std::vector<uint8_t>& value,
+    int offset,
+    bool has_subsequent_write,
+    base::OnceClosure callback,
+    device::BluetoothLocalGattService::Delegate::ErrorCallback error_callback) {
+  DVLOG(1) << "GATT characteristic value Prepare Set request: "
+           << object_path_.value() << " UUID: " << uuid_;
   // Check if this characteristic is registered.
   FakeBluetoothGattManagerClient* fake_bluetooth_gatt_manager_client =
       static_cast<FakeBluetoothGattManagerClient*>(
           bluez::BluezDBusManager::Get()->GetBluetoothGattManagerClient());
   if (!fake_bluetooth_gatt_manager_client->IsServiceRegistered(service_path_)) {
-    VLOG(1) << "GATT characteristic not registered.";
+    DVLOG(1) << "GATT characteristic not registered.";
+    std::move(error_callback).Run();
+    return;
+  }
+
+  if (!CanWrite(flags_)) {
+    DVLOG(1) << "GATT characteristic not writeable.";
+    std::move(error_callback).Run();
+    return;
+  }
+
+  // Pass on to the delegate.
+  DCHECK(delegate_);
+  delegate_->PrepareSetValue(device_path, value, offset, has_subsequent_write,
+                             std::move(callback), std::move(error_callback));
+}
+
+bool FakeBluetoothGattCharacteristicServiceProvider::NotificationsChange(
+    const dbus::ObjectPath& device_path,
+    bool start) {
+  DVLOG(1) << "GATT characteristic value notification request: "
+           << object_path_.value() << " UUID: " << uuid_ << " start=" << start;
+  // Check if this characteristic is registered.
+  FakeBluetoothGattManagerClient* fake_bluetooth_gatt_manager_client =
+      static_cast<FakeBluetoothGattManagerClient*>(
+          bluez::BluezDBusManager::Get()->GetBluetoothGattManagerClient());
+  if (!fake_bluetooth_gatt_manager_client->IsServiceRegistered(service_path_)) {
+    DVLOG(1) << "GATT characteristic not registered.";
     return false;
   }
 
   if (!CanNotify(flags_)) {
-    VLOG(1) << "GATT characteristic not notifiable.";
+    DVLOG(1) << "GATT characteristic not notifiable.";
     return false;
   }
 
   // Pass on to the delegate.
   DCHECK(delegate_);
-  start ? delegate_->StartNotifications() : delegate_->StopNotifications();
+  start ? delegate_->StartNotifications(device_path,
+                                        device::BluetoothGattCharacteristic::
+                                            NotificationType::kNotification)
+        : delegate_->StopNotifications(device_path);
 
   return true;
 }
