@@ -4,23 +4,34 @@
 
 #include "device/bluetooth/test/bluetooth_test.h"
 
+#include <iterator>
 #include <memory>
 
 #include "base/bind.h"
-#include "base/logging.h"
+#include "base/check.h"
 #include "base/memory/ptr_util.h"
+#include "base/notreached.h"
 #include "base/run_loop.h"
+#include "base/test/bind.h"
 #include "device/bluetooth/bluetooth_adapter.h"
 #include "device/bluetooth/bluetooth_common.h"
 
 namespace device {
 
+BluetoothTestBase::LowEnergyDeviceData::LowEnergyDeviceData() = default;
+
+BluetoothTestBase::LowEnergyDeviceData::LowEnergyDeviceData(
+    LowEnergyDeviceData&& data) = default;
+
+BluetoothTestBase::LowEnergyDeviceData::~LowEnergyDeviceData() = default;
+
 const char BluetoothTestBase::kTestAdapterName[] = "FakeBluetoothAdapter";
 const char BluetoothTestBase::kTestAdapterAddress[] = "A1:B2:C3:D4:E5:F6";
 
 const char BluetoothTestBase::kTestDeviceName[] = "FakeBluetoothDevice";
 const char BluetoothTestBase::kTestDeviceNameEmpty[] = "";
 const char BluetoothTestBase::kTestDeviceNameU2f[] = "U2F FakeDevice";
+const char BluetoothTestBase::kTestDeviceNameCable[] = "Cable FakeDevice";
 
 const char BluetoothTestBase::kTestDeviceAddress1[] = "01:00:00:90:1E:BE";
 const char BluetoothTestBase::kTestDeviceAddress2[] = "02:00:00:8B:74:63";
@@ -59,13 +70,19 @@ const char BluetoothTestBase::kTestUUIDServerCharacteristicConfiguration[] =
     "00002903-0000-1000-8000-00805f9b34fb";
 const char BluetoothTestBase::kTestUUIDCharacteristicPresentationFormat[] =
     "00002904-0000-1000-8000-00805f9b34fb";
+const char BluetoothTestBase::kTestUUIDCableAdvertisement[] =
+    "0000fde2-0000-1000-8000-00805f9b34fb";
 // Manufacturer kTestAdapterAddress
-const unsigned short BluetoothTestBase::kTestManufacturerId = 0x00E0;
+const uint16_t BluetoothTestBase::kTestManufacturerId = 0x00E0;
+const uint8_t BluetoothTestBase::kTestCableEid[] = {
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
+    0x08, 0x09, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15};
+const char BluetoothTestBase::kTestUuidFormattedClientEid[] =
+    "00010203-0405-0607-0809-101112131415";
 
-BluetoothTestBase::BluetoothTestBase() : weak_factory_(this) {}
+BluetoothTestBase::BluetoothTestBase() {}
 
 BluetoothTestBase::~BluetoothTestBase() = default;
-
 void BluetoothTestBase::StartLowEnergyDiscoverySession() {
   adapter_->StartDiscoverySessionWithFilter(
       std::make_unique<BluetoothDiscoveryFilter>(BLUETOOTH_TRANSPORT_LE),
@@ -99,12 +116,70 @@ BluetoothDevice* BluetoothTestBase::SimulateLowEnergyDevice(
   return nullptr;
 }
 
+void BluetoothTestBase::SimulateLowEnergyDiscoveryFailure() {
+  NOTIMPLEMENTED();
+}
+
 BluetoothDevice* BluetoothTestBase::SimulateClassicDevice() {
   NOTIMPLEMENTED();
   return nullptr;
 }
 
+bool BluetoothTestBase::ConnectGatt(
+    BluetoothDevice* device,
+    base::Optional<BluetoothUUID> service_uuid,
+    base::Optional<base::OnceCallback<void(BluetoothDevice*)>>
+        simulate_callback) {
+  base::RunLoop run_loop;
+  base::Optional<bool> result;
+  base::Optional<std::unique_ptr<BluetoothGattConnection>> connection;
+
+  device->CreateGattConnection(
+      base::BindLambdaForTesting(
+          [&result, &connection,
+           &run_loop](std::unique_ptr<BluetoothGattConnection> new_connection) {
+            result = true;
+            connection = std::move(new_connection);
+            run_loop.Quit();
+          }),
+      base::BindLambdaForTesting(
+          [this, &result, &run_loop](BluetoothDevice::ConnectErrorCode error) {
+            result = false;
+            last_connect_error_code_ = error;
+            run_loop.Quit();
+          }),
+      std::move(service_uuid));
+
+  // Run the event loop so that the mock devices can react to the GATT
+  // connection. Some of the |Simulate*| calls depend on it.
+  base::RunLoop().RunUntilIdle();
+
+  if (simulate_callback.has_value())
+    std::move(*simulate_callback).Run(device);
+  else
+    SimulateGattConnection(device);
+
+  run_loop.Run();
+  CHECK(result.has_value());
+  if (!*result)
+    return false;
+
+  gatt_connections_.emplace_back(std::move(*connection));
+  return true;
+}
+
+base::Optional<BluetoothUUID> BluetoothTestBase::GetTargetGattService(
+    BluetoothDevice* device) {
+  return base::nullopt;
+}
+
+void BluetoothTestBase::SimulateDeviceBreaksConnection(
+    BluetoothDevice* device) {
+  SimulateGattDisconnection(device);
+}
+
 bool BluetoothTestBase::SimulateLocalGattCharacteristicNotificationsRequest(
+    BluetoothDevice* device,
     BluetoothLocalGattCharacteristic* characteristic,
     bool start) {
   NOTIMPLEMENTED();
@@ -160,6 +235,18 @@ void BluetoothTestBase::Callback(Call expected) {
     unexpected_success_callback_ = true;
 }
 
+void BluetoothTestBase::CreateAdvertisementCallback(
+    Call expected,
+    scoped_refptr<BluetoothAdvertisement> advertisement) {
+  ++callback_count_;
+  advertisements_.push_back(std::move(advertisement));
+
+  if (expected == Call::EXPECTED)
+    ++actual_success_callback_calls_;
+  else
+    unexpected_success_callback_ = true;
+}
+
 void BluetoothTestBase::DiscoverySessionCallback(
     Call expected,
     std::unique_ptr<BluetoothDiscoverySession> discovery_session) {
@@ -244,6 +331,18 @@ void BluetoothTestBase::ErrorCallback(Call expected) {
     unexpected_error_callback_ = true;
 }
 
+void BluetoothTestBase::AdvertisementErrorCallback(
+    Call expected,
+    BluetoothAdvertisement::ErrorCode error_code) {
+  ++error_callback_count_;
+  last_advertisement_error_code_ = error_code;
+
+  if (expected == Call::EXPECTED)
+    ++actual_error_callback_calls_;
+  else
+    unexpected_error_callback_ = true;
+}
+
 void BluetoothTestBase::ConnectErrorCallback(
     Call expected,
     enum BluetoothDevice::ConnectErrorCode error_code) {
@@ -309,88 +408,104 @@ void BluetoothTestBase::ReentrantStartNotifySessionErrorCallback(
   }
 }
 
-base::Closure BluetoothTestBase::GetCallback(Call expected) {
+base::OnceClosure BluetoothTestBase::GetCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::Callback, weak_factory_.GetWeakPtr(),
-                    expected);
+  return base::BindOnce(&BluetoothTestBase::Callback,
+                        weak_factory_.GetWeakPtr(), expected);
+}
+
+BluetoothAdapter::CreateAdvertisementCallback
+BluetoothTestBase::GetCreateAdvertisementCallback(Call expected) {
+  if (expected == Call::EXPECTED)
+    ++expected_success_callback_calls_;
+  return base::BindOnce(&BluetoothTestBase::CreateAdvertisementCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothAdapter::DiscoverySessionCallback
 BluetoothTestBase::GetDiscoverySessionCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::DiscoverySessionCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::DiscoverySessionCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothDevice::GattConnectionCallback
 BluetoothTestBase::GetGattConnectionCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::GattConnectionCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::GattConnectionCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothRemoteGattCharacteristic::NotifySessionCallback
 BluetoothTestBase::GetNotifyCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::NotifyCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::NotifyCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothRemoteGattCharacteristic::NotifySessionCallback
 BluetoothTestBase::GetNotifyCheckForPrecedingCalls(int num_of_preceding_calls) {
   ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::NotifyCheckForPrecedingCalls,
-                    weak_factory_.GetWeakPtr(), num_of_preceding_calls);
+  return base::BindOnce(&BluetoothTestBase::NotifyCheckForPrecedingCalls,
+                        weak_factory_.GetWeakPtr(), num_of_preceding_calls);
 }
 
-base::Closure BluetoothTestBase::GetStopNotifyCallback(Call expected) {
+base::OnceClosure BluetoothTestBase::GetStopNotifyCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::StopNotifyCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::StopNotifyCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
-base::Closure BluetoothTestBase::GetStopNotifyCheckForPrecedingCalls(
+base::OnceClosure BluetoothTestBase::GetStopNotifyCheckForPrecedingCalls(
     int num_of_preceding_calls) {
   ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::StopNotifyCheckForPrecedingCalls,
-                    weak_factory_.GetWeakPtr(), num_of_preceding_calls);
+  return base::BindOnce(&BluetoothTestBase::StopNotifyCheckForPrecedingCalls,
+                        weak_factory_.GetWeakPtr(), num_of_preceding_calls);
 }
 
 BluetoothRemoteGattCharacteristic::ValueCallback
 BluetoothTestBase::GetReadValueCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(&BluetoothTestBase::ReadValueCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::ReadValueCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothAdapter::ErrorCallback BluetoothTestBase::GetErrorCallback(
     Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_error_callback_calls_;
-  return base::Bind(&BluetoothTestBase::ErrorCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::ErrorCallback,
+                        weak_factory_.GetWeakPtr(), expected);
+}
+
+BluetoothAdapter::AdvertisementErrorCallback
+BluetoothTestBase::GetAdvertisementErrorCallback(Call expected) {
+  if (expected == Call::EXPECTED)
+    ++expected_error_callback_calls_;
+  return base::BindOnce(&BluetoothTestBase::AdvertisementErrorCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothDevice::ConnectErrorCallback
 BluetoothTestBase::GetConnectErrorCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_error_callback_calls_;
-  return base::Bind(&BluetoothTestBase::ConnectErrorCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::ConnectErrorCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
-base::Callback<void(BluetoothRemoteGattService::GattErrorCode)>
+base::OnceCallback<void(BluetoothRemoteGattService::GattErrorCode)>
 BluetoothTestBase::GetGattErrorCallback(Call expected) {
   if (expected == Call::EXPECTED)
     ++expected_error_callback_calls_;
-  return base::Bind(&BluetoothTestBase::GattErrorCallback,
-                    weak_factory_.GetWeakPtr(), expected);
+  return base::BindOnce(&BluetoothTestBase::GattErrorCallback,
+                        weak_factory_.GetWeakPtr(), expected);
 }
 
 BluetoothRemoteGattCharacteristic::NotifySessionCallback
@@ -399,24 +514,26 @@ BluetoothTestBase::GetReentrantStartNotifySessionSuccessCallback(
     BluetoothRemoteGattCharacteristic* characteristic) {
   if (expected == Call::EXPECTED)
     ++expected_success_callback_calls_;
-  return base::Bind(
+  return base::BindOnce(
       &BluetoothTestBase::ReentrantStartNotifySessionSuccessCallback,
       weak_factory_.GetWeakPtr(), expected, characteristic);
 }
 
-base::Callback<void(BluetoothGattService::GattErrorCode)>
+base::OnceCallback<void(BluetoothGattService::GattErrorCode)>
 BluetoothTestBase::GetReentrantStartNotifySessionErrorCallback(
     Call expected,
     BluetoothRemoteGattCharacteristic* characteristic,
     bool error_in_reentrant) {
   if (expected == Call::EXPECTED)
     ++expected_error_callback_calls_;
-  return base::Bind(
+  return base::BindOnce(
       &BluetoothTestBase::ReentrantStartNotifySessionErrorCallback,
       weak_factory_.GetWeakPtr(), expected, characteristic, error_in_reentrant);
 }
 
 void BluetoothTestBase::ResetEventCounts() {
+  last_advertisement_error_code_ =
+      BluetoothAdvertisement::INVALID_ADVERTISEMENT_ERROR_CODE;
   last_connect_error_code_ = BluetoothDevice::ERROR_UNKNOWN;
   callback_count_ = 0;
   error_callback_count_ = 0;
@@ -434,4 +551,89 @@ void BluetoothTestBase::RemoveTimedOutDevices() {
   adapter_->RemoveTimedOutDevices();
 }
 
+BluetoothTestBase::LowEnergyDeviceData
+BluetoothTestBase::GetLowEnergyDeviceData(int device_ordinal) const {
+  LowEnergyDeviceData device_data;
+  switch (device_ordinal) {
+    case 1:
+      device_data.name = kTestDeviceName;
+      device_data.address = kTestDeviceAddress1;
+      device_data.flags = 0x04;
+      device_data.rssi = static_cast<int>(TestRSSI::LOWEST);
+      device_data.advertised_uuids = {BluetoothUUID(kTestUUIDGenericAccess),
+                                      BluetoothUUID(kTestUUIDGenericAttribute)};
+      device_data.service_data = {{BluetoothUUID(kTestUUIDHeartRate), {1}}};
+      device_data.manufacturer_data = {{kTestManufacturerId, {1, 2, 3, 4}}};
+      device_data.tx_power = static_cast<int>(TestTxPower::LOWEST);
+      break;
+    case 2:
+      device_data.name = kTestDeviceName;
+      device_data.address = kTestDeviceAddress1;
+      device_data.flags = 0x05;
+      device_data.rssi = static_cast<int>(TestRSSI::LOWER);
+      device_data.advertised_uuids = {BluetoothUUID(kTestUUIDImmediateAlert),
+                                      BluetoothUUID(kTestUUIDLinkLoss)};
+      device_data.service_data = {
+          {BluetoothUUID(kTestUUIDHeartRate), {}},
+          {BluetoothUUID(kTestUUIDImmediateAlert), {0, 2}}};
+      device_data.manufacturer_data = {{kTestManufacturerId, {}}};
+      device_data.tx_power = static_cast<int>(TestTxPower::LOWER);
+      break;
+    case 3:
+      device_data.name = kTestDeviceNameEmpty;
+      device_data.address = kTestDeviceAddress1;
+      device_data.rssi = static_cast<int>(TestRSSI::LOW);
+      break;
+    case 4:
+      device_data.name = kTestDeviceNameEmpty;
+      device_data.address = kTestDeviceAddress2;
+      device_data.rssi = static_cast<int>(TestRSSI::MEDIUM);
+      break;
+    case 5:
+      device_data.address = kTestDeviceAddress1;
+      device_data.flags = 0x06;
+      device_data.rssi = static_cast<int>(TestRSSI::HIGH);
+      break;
+    case 6:
+      device_data.name = kTestDeviceName;
+      device_data.address = kTestDeviceAddress2;
+      device_data.flags = 0x18;
+      device_data.rssi = static_cast<int>(TestRSSI::LOWEST);
+      device_data.transport = BLUETOOTH_TRANSPORT_DUAL;
+      break;
+    case 7:
+      device_data.name = kTestDeviceNameU2f;
+      device_data.address = kTestDeviceAddress1;
+      device_data.flags = 0x07;
+      device_data.rssi = static_cast<int>(TestRSSI::LOWEST);
+      device_data.advertised_uuids = {BluetoothUUID(kTestUUIDU2f)};
+      device_data.service_data = {
+          {BluetoothUUID(kTestUUIDU2fControlPointLength), {0, 20}}};
+      break;
+    case 8:
+      device_data.name = kTestDeviceNameCable;
+      device_data.address = kTestDeviceAddress1;
+      device_data.flags = 0x07;
+      device_data.rssi = static_cast<int>(TestRSSI::LOWEST);
+      device_data.service_data = {
+          {BluetoothUUID(kTestUUIDCableAdvertisement),
+           std::vector<uint8_t>(std::begin(kTestCableEid),
+                                std::end(kTestCableEid))}};
+      break;
+    case 9:
+      device_data.name = kTestDeviceNameCable;
+      device_data.address = kTestDeviceAddress2;
+      device_data.flags = 0x07;
+      device_data.rssi = static_cast<int>(TestRSSI::LOWEST);
+      device_data.advertised_uuids = {
+          BluetoothUUID(kTestUUIDCableAdvertisement),
+          BluetoothUUID(kTestUuidFormattedClientEid)};
+      break;
+    default:
+      NOTREACHED();
+  }
+
+  return device_data;
+}
+
 }  // namespace device
