@@ -8,13 +8,14 @@
 
 #include "base/compiler_specific.h"
 #include "device/bluetooth/bluetooth_adapter_factory.h"
-#include "device/bluetooth/public/interfaces/test/fake_bluetooth.mojom.h"
+#include "device/bluetooth/public/mojom/test/fake_bluetooth.mojom-forward.h"
 #include "device/bluetooth/test/fake_central.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
 
 namespace bluetooth {
 
 // Implementation of FakeBluetooth in
-// src/device/bluetooth/public/interfaces/test/fake_bluetooth.mojom.
+// src/device/bluetooth/public/mojom/test/fake_bluetooth.mojom.
 // Implemented on top of the C++ device/bluetooth API, mainly
 // device/bluetooth/bluetooth_adapter_factory.h.
 //
@@ -24,7 +25,7 @@ class FakeBluetooth : public mojom::FakeBluetooth {
   FakeBluetooth();
   ~FakeBluetooth() override;
 
-  static void Create(mojom::FakeBluetoothRequest request);
+  static void Create(mojo::PendingReceiver<mojom::FakeBluetooth> receiver);
 
   void SetLESupported(bool available, SetLESupportedCallback callback) override;
   void SimulateCentral(mojom::CentralState state,
