@@ -10,9 +10,10 @@
 
 #include "base/bind.h"
 #include "base/run_loop.h"
-#include "base/test/bind_test_util.h"
+#include "base/test/bind.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/win/async_operation.h"
+#include "base/win/scoped_hstring.h"
 #include "device/bluetooth/bluetooth_remote_gatt_service_winrt.h"
 #include "device/bluetooth/test/bluetooth_test_win.h"
 #include "device/bluetooth/test/fake_device_information_pairing_winrt.h"
@@ -30,6 +31,8 @@ using ABI::Windows::Devices::Bluetooth::BluetoothConnectionStatus;
 using ABI::Windows::Devices::Bluetooth::BluetoothConnectionStatus_Connected;
 using ABI::Windows::Devices::Bluetooth::BluetoothConnectionStatus_Disconnected;
 using ABI::Windows::Devices::Bluetooth::BluetoothLEDevice;
+using ABI::Windows::Devices::Bluetooth::IBluetoothDeviceId;
+using ABI::Windows::Devices::Bluetooth::IBluetoothLEAppearance;
 using ABI::Windows::Devices::Bluetooth::GenericAttributeProfile::
     GattCommunicationStatus;
 using ABI::Windows::Devices::Bluetooth::GenericAttributeProfile::
@@ -46,15 +49,39 @@ using ABI::Windows::Devices::Bluetooth::GenericAttributeProfile::
     GattDeviceServicesResult;
 using ABI::Windows::Devices::Bluetooth::GenericAttributeProfile::
     IGattDeviceService;
-using ABI::Windows::Devices::Bluetooth::IBluetoothLEAppearance;
 using ABI::Windows::Devices::Enumeration::DeviceAccessStatus;
 using ABI::Windows::Devices::Enumeration::IDeviceAccessInformation;
 using ABI::Windows::Devices::Enumeration::IDeviceInformation;
-using ABI::Windows::Foundation::Collections::IVectorView;
 using ABI::Windows::Foundation::IAsyncOperation;
 using ABI::Windows::Foundation::ITypedEventHandler;
+using ABI::Windows::Foundation::Collections::IVectorView;
 using Microsoft::WRL::Make;
 
+class FakeBluetoothDeviceId
+    : public Microsoft::WRL::RuntimeClass<
+          Microsoft::WRL::RuntimeClassFlags<
+              Microsoft::WRL::WinRt | Microsoft::WRL::InhibitRoOriginateError>,
+          IBluetoothDeviceId> {
+ public:
+  FakeBluetoothDeviceId() = default;
+  FakeBluetoothDeviceId(const FakeBluetoothDeviceId&) = delete;
+  FakeBluetoothDeviceId& operator=(const FakeBluetoothDeviceId&) = delete;
+  ~FakeBluetoothDeviceId() override {}
+
+  // IBluetoothDeviceId:
+  IFACEMETHODIMP get_Id(HSTRING* value) override {
+    *value = base::win::ScopedHString::Create(L"FakeBluetoothLEDeviceWinrt")
+                 .release();
+    return S_OK;
+  }
+  IFACEMETHODIMP get_IsClassicDevice(::boolean* value) override {
+    return E_NOTIMPL;
+  }
+  IFACEMETHODIMP get_IsLowEnergyDevice(::boolean* value) override {
+    return E_NOTIMPL;
+  }
+};
+
 }  // namespace
 
 FakeBluetoothLEDeviceWinrt::FakeBluetoothLEDeviceWinrt(
@@ -64,11 +91,17 @@ FakeBluetoothLEDeviceWinrt::FakeBluetoothLEDeviceWinrt(
 FakeBluetoothLEDeviceWinrt::~FakeBluetoothLEDeviceWinrt() = default;
 
 HRESULT FakeBluetoothLEDeviceWinrt::get_DeviceId(HSTRING* value) {
-  return E_NOTIMPL;
+  *value =
+      base::win::ScopedHString::Create(L"FakeBluetoothLEDeviceWinrt").release();
+  return S_OK;
 }
 
 HRESULT FakeBluetoothLEDeviceWinrt::get_Name(HSTRING* value) {
-  return E_NOTIMPL;
+  if (!name_)
+    return E_FAIL;
+
+  *value = base::win::ScopedHString::Create(*name_).release();
+  return S_OK;
 }
 
 HRESULT FakeBluetoothLEDeviceWinrt::get_GattServices(
@@ -95,12 +128,14 @@ HRESULT FakeBluetoothLEDeviceWinrt::GetGattService(
 HRESULT FakeBluetoothLEDeviceWinrt::add_NameChanged(
     ITypedEventHandler<BluetoothLEDevice*, IInspectable*>* handler,
     EventRegistrationToken* token) {
-  return E_NOTIMPL;
+  name_changed_handler_ = handler;
+  return S_OK;
 }
 
 HRESULT FakeBluetoothLEDeviceWinrt::remove_NameChanged(
     EventRegistrationToken token) {
-  return E_NOTIMPL;
+  name_changed_handler_ = nullptr;
+  return S_OK;
 }
 
 HRESULT FakeBluetoothLEDeviceWinrt::add_GattServicesChanged(
@@ -159,7 +194,11 @@ HRESULT FakeBluetoothLEDeviceWinrt::GetGattServicesAsync(
   auto async_op = Make<base::win::AsyncOperation<GattDeviceServicesResult*>>();
   gatt_services_callback_ = async_op->callback();
   *operation = async_op.Detach();
-  bluetooth_test_winrt_->OnFakeBluetoothDeviceConnectGattCalled();
+  service_uuid_.reset();
+  if (!bluetooth_test_winrt_->UsesNewGattSessionHandling()) {
+    bluetooth_test_winrt_->OnFakeBluetoothDeviceConnectGattAttempt();
+  }
+  bluetooth_test_winrt_->OnFakeBluetoothDeviceGattServiceDiscoveryAttempt();
   return S_OK;
 }
 
@@ -172,7 +211,15 @@ HRESULT FakeBluetoothLEDeviceWinrt::GetGattServicesWithCacheModeAsync(
 HRESULT FakeBluetoothLEDeviceWinrt::GetGattServicesForUuidAsync(
     GUID service_uuid,
     IAsyncOperation<GattDeviceServicesResult*>** operation) {
-  return E_NOTIMPL;
+  auto async_op = Make<base::win::AsyncOperation<GattDeviceServicesResult*>>();
+  gatt_services_callback_ = async_op->callback();
+  service_uuid_ = service_uuid;
+  *operation = async_op.Detach();
+  if (!bluetooth_test_winrt_->UsesNewGattSessionHandling()) {
+    bluetooth_test_winrt_->OnFakeBluetoothDeviceConnectGattAttempt();
+  }
+  bluetooth_test_winrt_->OnFakeBluetoothDeviceGattServiceDiscoveryAttempt();
+  return S_OK;
 }
 
 HRESULT FakeBluetoothLEDeviceWinrt::GetGattServicesForUuidWithCacheModeAsync(
@@ -182,6 +229,11 @@ HRESULT FakeBluetoothLEDeviceWinrt::GetGattServicesForUuidWithCacheModeAsync(
   return E_NOTIMPL;
 }
 
+HRESULT FakeBluetoothLEDeviceWinrt::get_BluetoothDeviceId(
+    ABI::Windows::Devices::Bluetooth::IBluetoothDeviceId** value) {
+  return Make<FakeBluetoothDeviceId>().CopyTo(value);
+}
+
 HRESULT FakeBluetoothLEDeviceWinrt::Close() {
   --reference_count_;
   fake_services_.clear();
@@ -207,11 +259,23 @@ void FakeBluetoothLEDeviceWinrt::SimulatePairingPinCode(std::string pin_code) {
       Make<FakeDeviceInformationPairingWinrt>(std::move(pin_code)));
 }
 
+base::Optional<BluetoothUUID> FakeBluetoothLEDeviceWinrt::GetTargetGattService()
+    const {
+  if (!service_uuid_)
+    return base::nullopt;
+  return BluetoothUUID(*service_uuid_);
+}
+
 void FakeBluetoothLEDeviceWinrt::SimulateGattConnection() {
   status_ = BluetoothConnectionStatus_Connected;
   connection_status_changed_handler_->Invoke(this, nullptr);
 }
 
+void FakeBluetoothLEDeviceWinrt::SimulateStatusChangeToDisconnect() {
+  status_ = BluetoothConnectionStatus_Disconnected;
+  connection_status_changed_handler_->Invoke(this, nullptr);
+}
+
 void FakeBluetoothLEDeviceWinrt ::SimulateGattConnectionError(
     BluetoothDevice::ConnectErrorCode error_code) {
   if (!gatt_services_callback_)
@@ -224,7 +288,11 @@ void FakeBluetoothLEDeviceWinrt ::SimulateGattConnectionError(
 
 void FakeBluetoothLEDeviceWinrt::SimulateGattDisconnection() {
   if (status_ == BluetoothConnectionStatus_Disconnected) {
-    DCHECK(gatt_services_callback_);
+    if (!gatt_services_callback_) {
+      DCHECK(bluetooth_test_winrt_->UsesNewGattSessionHandling());
+      return;
+    }
+
     std::move(gatt_services_callback_)
         .Run(Make<FakeGattDeviceServicesResultWinrt>(
             GattCommunicationStatus_Unreachable));
@@ -233,10 +301,8 @@ void FakeBluetoothLEDeviceWinrt::SimulateGattDisconnection() {
 
   // Simulate production UWP behavior that only really disconnects once all
   // references to a device are dropped.
-  if (reference_count_ == 0u) {
-    status_ = BluetoothConnectionStatus_Disconnected;
-    connection_status_changed_handler_->Invoke(this, nullptr);
-  }
+  if (reference_count_ == 0u)
+    SimulateStatusChangeToDisconnect();
 }
 
 void FakeBluetoothLEDeviceWinrt::SimulateDeviceBreaksConnection() {
@@ -249,19 +315,33 @@ void FakeBluetoothLEDeviceWinrt::SimulateDeviceBreaksConnection() {
   }
 
   // Simulate a Gatt Disconnecion regardless of the reference count.
-  status_ = BluetoothConnectionStatus_Disconnected;
-  connection_status_changed_handler_->Invoke(this, nullptr);
+  SimulateStatusChangeToDisconnect();
+}
+
+void FakeBluetoothLEDeviceWinrt::SimulateGattNameChange(
+    const std::string& new_name) {
+  name_ = new_name;
+  name_changed_handler_->Invoke(this, nullptr);
 }
 
 void FakeBluetoothLEDeviceWinrt::SimulateGattServicesDiscovered(
-    const std::vector<std::string>& uuids) {
+    const std::vector<std::string>& uuids,
+    const std::vector<std::string>& blocked_uuids) {
   for (const auto& uuid : uuids) {
     // Attribute handles need to be unique for a given BLE device. Increasing by
     // a large number ensures enough address space for the contained
     // characteristics and descriptors.
-    fake_services_.push_back(
-        Make<FakeGattDeviceServiceWinrt>(bluetooth_test_winrt_, this, uuid,
-                                         service_attribute_handle_ += 0x0400));
+    fake_services_.push_back(Make<FakeGattDeviceServiceWinrt>(
+        bluetooth_test_winrt_, this, uuid, service_attribute_handle_ += 0x0400,
+        /*allowed=*/true));
+  }
+  for (const auto& uuid : blocked_uuids) {
+    // Attribute handles need to be unique for a given BLE device. Increasing by
+    // a large number ensures enough address space for the contained
+    // characteristics and descriptors.
+    fake_services_.push_back(Make<FakeGattDeviceServiceWinrt>(
+        bluetooth_test_winrt_, this, uuid, service_attribute_handle_ += 0x0400,
+        /*allowed=*/false));
   }
 
   DCHECK(gatt_services_callback_);
