@@ -32,6 +32,7 @@ class FakeBluetoothLEDeviceWinrt
           ABI::Windows::Devices::Bluetooth::IBluetoothLEDevice,
           ABI::Windows::Devices::Bluetooth::IBluetoothLEDevice2,
           ABI::Windows::Devices::Bluetooth::IBluetoothLEDevice3,
+          ABI::Windows::Devices::Bluetooth::IBluetoothLEDevice4,
           ABI::Windows::Foundation::IClosable> {
  public:
   explicit FakeBluetoothLEDeviceWinrt(BluetoothTestWinrt* bluetooth_test_winrt);
@@ -111,6 +112,10 @@ class FakeBluetoothLEDeviceWinrt
           ABI::Windows::Devices::Bluetooth::GenericAttributeProfile::
               GattDeviceServicesResult*>** operation) override;
 
+  // IBluetoothLEDevice4:
+  IFACEMETHODIMP get_BluetoothDeviceId(
+      ABI::Windows::Devices::Bluetooth::IBluetoothDeviceId** value) override;
+
   // IClosable:
   IFACEMETHODIMP Close() override;
 
@@ -122,13 +127,18 @@ class FakeBluetoothLEDeviceWinrt
 
   void SimulateDevicePaired(bool is_paired);
   void SimulatePairingPinCode(std::string pin_code);
+  base::Optional<BluetoothUUID> GetTargetGattService() const;
   void SimulateGattConnection();
   void SimulateGattConnectionError(
       BluetoothDevice::ConnectErrorCode error_code);
   void SimulateGattDisconnection();
   void SimulateDeviceBreaksConnection();
-  void SimulateGattServicesDiscovered(const std::vector<std::string>& uuids);
+  void SimulateGattNameChange(const std::string& new_name);
+  void SimulateGattServicesDiscovered(
+      const std::vector<std::string>& uuids,
+      const std::vector<std::string>& blocked_uuids);
   void SimulateGattServicesChanged();
+  void SimulateStatusChangeToDisconnect();
   void SimulateGattServiceRemoved(BluetoothRemoteGattService* service);
   void SimulateGattCharacteristic(BluetoothRemoteGattService* service,
                                   const std::string& uuid,
@@ -140,6 +150,7 @@ class FakeBluetoothLEDeviceWinrt
  private:
   BluetoothTestWinrt* bluetooth_test_winrt_ = nullptr;
   uint32_t reference_count_ = 1u;
+  base::Optional<std::string> name_;
 
   ABI::Windows::Devices::Bluetooth::BluetoothConnectionStatus status_ =
       ABI::Windows::Devices::Bluetooth::BluetoothConnectionStatus_Disconnected;
@@ -162,11 +173,18 @@ class FakeBluetoothLEDeviceWinrt
           ABI::Windows::Devices::Bluetooth::GenericAttributeProfile::
               IGattDeviceServicesResult>)>
       gatt_services_callback_;
+  // Contains the last GUID passed to GetGattServicesForUuidAsync.
+  base::Optional<GUID> service_uuid_;
 
   std::vector<Microsoft::WRL::ComPtr<FakeGattDeviceServiceWinrt>>
       fake_services_;
   uint16_t service_attribute_handle_ = 0;
 
+  Microsoft::WRL::ComPtr<ABI::Windows::Foundation::ITypedEventHandler<
+      ABI::Windows::Devices::Bluetooth::BluetoothLEDevice*,
+      IInspectable*>>
+      name_changed_handler_;
+
   DISALLOW_COPY_AND_ASSIGN(FakeBluetoothLEDeviceWinrt);
 };
 
