@@ -15,8 +15,15 @@ namespace device {
 using testing::Invoke;
 using testing::_;
 
-MockBluetoothAdapter::Observer::Observer() = default;
-MockBluetoothAdapter::Observer::~Observer() = default;
+MockBluetoothAdapter::Observer::Observer(
+    scoped_refptr<BluetoothAdapter> adapter)
+    : adapter_(std::move(adapter)) {
+  adapter_->AddObserver(this);
+}
+
+MockBluetoothAdapter::Observer::~Observer() {
+  adapter_->RemoveObserver(this);
+}
 
 MockBluetoothAdapter::MockBluetoothAdapter() {
   ON_CALL(*this, AddObserver(_))
@@ -31,34 +38,33 @@ MockBluetoothAdapter::MockBluetoothAdapter() {
 
 MockBluetoothAdapter::~MockBluetoothAdapter() = default;
 
+void MockBluetoothAdapter::Initialize(base::OnceClosure callback) {
+  std::move(callback).Run();
+}
+
 #if defined(OS_CHROMEOS) || defined(OS_LINUX)
 void MockBluetoothAdapter::Shutdown() {
 }
 #endif
 
-void MockBluetoothAdapter::AddDiscoverySession(
-    BluetoothDiscoveryFilter* discovery_filter,
-    const base::Closure& callback,
-    const DiscoverySessionErrorCallback& error_callback) {}
-
-void MockBluetoothAdapter::RemoveDiscoverySession(
-    BluetoothDiscoveryFilter* discovery_filter,
-    const base::Closure& callback,
-    const DiscoverySessionErrorCallback& error_callback) {}
-
-void MockBluetoothAdapter::SetDiscoveryFilter(
-    std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-    const base::Closure& callback,
-    const DiscoverySessionErrorCallback& error_callback) {
-  SetDiscoveryFilterRaw(discovery_filter.get(), callback, error_callback);
+base::WeakPtr<BluetoothAdapter> MockBluetoothAdapter::GetWeakPtr() {
+  return weak_ptr_factory_.GetWeakPtr();
 }
 
-void MockBluetoothAdapter::StartDiscoverySessionWithFilter(
+bool MockBluetoothAdapter::SetPoweredImpl(bool powered) {
+  return false;
+}
+
+void MockBluetoothAdapter::StartScanWithFilter(
     std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-    const DiscoverySessionCallback& callback,
-    const ErrorCallback& error_callback) {
-  StartDiscoverySessionWithFilterRaw(discovery_filter.get(), callback,
-                                     error_callback);
+    DiscoverySessionResultCallback callback) {
+  StartScanWithFilter_(discovery_filter.get(), callback);
+}
+
+void MockBluetoothAdapter::UpdateFilter(
+    std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
+    DiscoverySessionResultCallback callback) {
+  UpdateFilter_(discovery_filter.get(), callback);
 }
 
 void MockBluetoothAdapter::AddMockDevice(
@@ -97,20 +103,20 @@ BluetoothAdapter::DeviceList MockBluetoothAdapter::GetMockDevices() {
 
 void MockBluetoothAdapter::RegisterAdvertisement(
     std::unique_ptr<BluetoothAdvertisement::Data> advertisement_data,
-    const CreateAdvertisementCallback& callback,
-    const AdvertisementErrorCallback& error_callback) {
-  callback.Run(new MockBluetoothAdvertisement);
+    CreateAdvertisementCallback callback,
+    AdvertisementErrorCallback error_callback) {
+  std::move(callback).Run(new MockBluetoothAdvertisement);
 }
 
 #if defined(OS_CHROMEOS) || defined(OS_LINUX)
 void MockBluetoothAdapter::SetAdvertisingInterval(
     const base::TimeDelta& min,
     const base::TimeDelta& max,
-    const base::Closure& callback,
-    const AdvertisementErrorCallback& error_callback) {}
+    base::OnceClosure callback,
+    AdvertisementErrorCallback error_callback) {}
 void MockBluetoothAdapter::ResetAdvertising(
-    const base::Closure& callback,
-    const AdvertisementErrorCallback& error_callback) {}
+    base::OnceClosure callback,
+    AdvertisementErrorCallback error_callback) {}
 #endif
 
 }  // namespace device
