@@ -10,6 +10,7 @@
 #include <vector>
 
 #include "base/callback.h"
+#include "base/macros.h"
 #include "build/build_config.h"
 #include "device/bluetooth/bluetooth_adapter.h"
 #include "device/bluetooth/bluetooth_device.h"
@@ -23,7 +24,7 @@ class MockBluetoothAdapter : public BluetoothAdapter {
  public:
   class Observer : public BluetoothAdapter::Observer {
    public:
-    Observer();
+    Observer(scoped_refptr<BluetoothAdapter> adapter);
     ~Observer() override;
 
     MOCK_METHOD2(AdapterPresentChanged, void(BluetoothAdapter*, bool));
@@ -32,12 +33,22 @@ class MockBluetoothAdapter : public BluetoothAdapter {
     MOCK_METHOD2(DeviceAdded, void(BluetoothAdapter*, BluetoothDevice*));
     MOCK_METHOD2(DeviceChanged, void(BluetoothAdapter*, BluetoothDevice*));
     MOCK_METHOD2(DeviceRemoved, void(BluetoothAdapter*, BluetoothDevice*));
+    MOCK_METHOD3(GattCharacteristicValueChanged,
+                 void(BluetoothAdapter*,
+                      BluetoothRemoteGattCharacteristic*,
+                      const std::vector<uint8_t>&));
+
+   private:
+    const scoped_refptr<BluetoothAdapter> adapter_;
+
+    DISALLOW_COPY_AND_ASSIGN(Observer);
   };
 
   MockBluetoothAdapter();
 
   bool IsInitialized() const override { return true; }
 
+  void Initialize(base::OnceClosure callback) override;
 #if defined(OS_CHROMEOS) || defined(OS_LINUX)
   void Shutdown() override;
 #endif
@@ -47,31 +58,37 @@ class MockBluetoothAdapter : public BluetoothAdapter {
   MOCK_CONST_METHOD0(GetName, std::string());
   MOCK_METHOD3(SetName,
                void(const std::string& name,
-                    const base::Closure& callback,
-                    const ErrorCallback& error_callback));
+                    base::OnceClosure callback,
+                    ErrorCallback error_callback));
   MOCK_CONST_METHOD0(IsPresent, bool());
   MOCK_CONST_METHOD0(IsPowered, bool());
   MOCK_CONST_METHOD0(CanPower, bool());
   MOCK_METHOD3(SetPowered,
                void(bool powered,
-                    const base::Closure& callback,
-                    const ErrorCallback& error_callback));
+                    base::OnceClosure callback,
+                    ErrorCallback error_callback));
   MOCK_CONST_METHOD0(IsDiscoverable, bool());
   MOCK_METHOD3(SetDiscoverable,
                void(bool discoverable,
-                    const base::Closure& callback,
-                    const ErrorCallback& error_callback));
+                    base::OnceClosure callback,
+                    ErrorCallback error_callback));
   MOCK_CONST_METHOD0(IsDiscovering, bool());
-  MOCK_METHOD2(StartDiscoverySession,
-               void(const DiscoverySessionCallback& callback,
-                    const ErrorCallback& error_callback));
-  MOCK_METHOD3(StartDiscoverySessionWithFilterRaw,
-               void(const BluetoothDiscoveryFilter*,
-                    const DiscoverySessionCallback& callback,
-                    const ErrorCallback& error_callback));
+  MOCK_METHOD2(
+      StartScanWithFilter_,
+      void(const BluetoothDiscoveryFilter*,
+           base::OnceCallback<void(/*is_error=*/bool,
+                                   UMABluetoothDiscoverySessionOutcome)>&
+               callback));
+  MOCK_METHOD2(
+      UpdateFilter_,
+      void(const BluetoothDiscoveryFilter*,
+           base::OnceCallback<void(/*is_error=*/bool,
+                                   UMABluetoothDiscoverySessionOutcome)>&
+               callback));
+  MOCK_METHOD1(StopScan, void(DiscoverySessionResultCallback callback));
   MOCK_METHOD3(SetDiscoveryFilterRaw,
                void(const BluetoothDiscoveryFilter*,
-                    const base::Closure& callback,
+                    const base::RepeatingClosure& callback,
                     DiscoverySessionErrorCallback& error_callback));
   MOCK_CONST_METHOD0(GetDevices, BluetoothAdapter::ConstDeviceList());
   MOCK_METHOD1(GetDevice, BluetoothDevice*(const std::string& address));
@@ -87,20 +104,24 @@ class MockBluetoothAdapter : public BluetoothAdapter {
   MOCK_METHOD4(CreateRfcommService,
                void(const BluetoothUUID& uuid,
                     const ServiceOptions& options,
-                    const CreateServiceCallback& callback,
-                    const CreateServiceErrorCallback& error_callback));
+                    CreateServiceCallback callback,
+                    CreateServiceErrorCallback error_callback));
   MOCK_METHOD4(CreateL2capService,
                void(const BluetoothUUID& uuid,
                     const ServiceOptions& options,
-                    const CreateServiceCallback& callback,
-                    const CreateServiceErrorCallback& error_callback));
+                    CreateServiceCallback callback,
+                    CreateServiceErrorCallback error_callback));
   MOCK_CONST_METHOD1(GetGattService,
                      BluetoothLocalGattService*(const std::string& identifier));
+#if defined(OS_CHROMEOS) || defined(OS_LINUX)
+  MOCK_METHOD4(
+      ConnectDevice,
+      void(const std::string& address,
+           const base::Optional<BluetoothDevice::AddressType>& address_type,
+           ConnectDeviceCallback callback,
+           ErrorCallback error_callback));
 
-  void StartDiscoverySessionWithFilter(
-      std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-      const DiscoverySessionCallback& callback,
-      const ErrorCallback& error_callback) override;
+#endif
 
   // BluetoothAdapter is supposed to manage the lifetime of BluetoothDevices.
   // This method takes ownership of the MockBluetoothDevice. This is only for
@@ -121,32 +142,25 @@ class MockBluetoothAdapter : public BluetoothAdapter {
   }
 
  protected:
+  base::WeakPtr<BluetoothAdapter> GetWeakPtr() override;
   bool SetPoweredImpl(bool powered) override;
-  void AddDiscoverySession(
-      BluetoothDiscoveryFilter* discovery_filter,
-      const base::Closure& callback,
-      DiscoverySessionErrorCallback error_callback) override;
-  void RemoveDiscoverySession(
-      BluetoothDiscoveryFilter* discovery_filter,
-      const base::Closure& callback,
-      DiscoverySessionErrorCallback error_callback) override;
-  void SetDiscoveryFilter(
+  void StartScanWithFilter(
       std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
-      const base::Closure& callback,
-      DiscoverySessionErrorCallback error_callback) override;
+      DiscoverySessionResultCallback callback) override;
+  void UpdateFilter(std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
+                    DiscoverySessionResultCallback callback) override;
   void RegisterAdvertisement(
       std::unique_ptr<BluetoothAdvertisement::Data> advertisement_data,
-      const CreateAdvertisementCallback& callback,
-      const AdvertisementErrorCallback& error_callback) override;
+      CreateAdvertisementCallback callback,
+      AdvertisementErrorCallback error_callback) override;
 #if defined(OS_CHROMEOS) || defined(OS_LINUX)
   void SetAdvertisingInterval(
       const base::TimeDelta& min,
       const base::TimeDelta& max,
-      const base::Closure& callback,
-      const AdvertisementErrorCallback& error_callback) override;
-  void ResetAdvertising(
-      const base::Closure& callback,
-      const AdvertisementErrorCallback& error_callback) override;
+      base::OnceClosure callback,
+      AdvertisementErrorCallback error_callback) override;
+  void ResetAdvertising(base::OnceClosure callback,
+                        AdvertisementErrorCallback error_callback) override;
 #endif
   ~MockBluetoothAdapter() override;
 
@@ -154,6 +168,10 @@ class MockBluetoothAdapter : public BluetoothAdapter {
                void(BluetoothDevice::PairingDelegate* pairing_delegate));
 
   std::vector<std::unique_ptr<MockBluetoothDevice>> mock_devices_;
+
+  // This must be the last field in the class so that weak pointers are
+  // invalidated first.
+  base::WeakPtrFactory<MockBluetoothAdapter> weak_ptr_factory_{this};
 };
 
 }  // namespace device
