@@ -5,7 +5,6 @@
 #ifndef DEVICE_BLUETOOTH_MOCK_BLUETOOTH_CBPERIPHERAL_MAC_H_
 #define DEVICE_BLUETOOTH_MOCK_BLUETOOTH_CBPERIPHERAL_MAC_H_
 
-#include "base/mac/sdk_forward_declarations.h"
 #include "build/build_config.h"
 
 #import <CoreBluetooth/CoreBluetooth.h>
@@ -39,12 +38,17 @@ class BluetoothTestMac;
 - (void)setState:(CBPeripheralState)state;
 - (void)removeAllServices;
 - (void)addServices:(NSArray*)services;
-- (void)didDiscoverServicesWithError:(NSError*)error;
+- (void)mockDidDiscoverServicesWithError:(NSError*)error;
 - (void)removeService:(CBService*)uuid;
 - (void)mockDidDiscoverServices;
+- (void)mockDidDiscoverCharacteristicsForService:(CBService*)service
+                                       WithError:(NSError*)error;
 - (void)mockDidDiscoverCharacteristicsForService:(CBService*)service;
 - (void)mockDidDiscoverDescriptorsForCharacteristic:
     (CBCharacteristic*)characteristic;
+- (void)mockDidDiscoverDescriptorsForCharacteristic:
+            (CBCharacteristic*)characteristic
+                                          WithError:(NSError*)error;
 - (void)mockDidDiscoverEvents;
 - (void)didModifyServices:(NSArray*)invalidatedServices;
 - (void)didDiscoverDescriptorsWithCharacteristic:
