@@ -15,18 +15,15 @@ MockBluetoothGattDescriptor::MockBluetoothGattDescriptor(
     MockBluetoothGattCharacteristic* characteristic,
     const std::string& identifier,
     const BluetoothUUID& uuid,
-    bool is_local,
-    BluetoothGattCharacteristic::Permissions permissions) {
+    BluetoothRemoteGattCharacteristic::Permissions permissions) {
   ON_CALL(*this, GetIdentifier()).WillByDefault(Return(identifier));
   ON_CALL(*this, GetUUID()).WillByDefault(Return(uuid));
-  ON_CALL(*this, IsLocal()).WillByDefault(Return(is_local));
   ON_CALL(*this, GetValue())
       .WillByDefault(ReturnRefOfCopy(std::vector<uint8_t>()));
   ON_CALL(*this, GetCharacteristic()).WillByDefault(Return(characteristic));
   ON_CALL(*this, GetPermissions()).WillByDefault(Return(permissions));
 }
 
-MockBluetoothGattDescriptor::~MockBluetoothGattDescriptor() {
-}
+MockBluetoothGattDescriptor::~MockBluetoothGattDescriptor() = default;
 
 }  // namespace device
