@@ -11,7 +11,7 @@
 
 #include "base/macros.h"
 #include "device/bluetooth/bluetooth_remote_gatt_service.h"
-#include "device/bluetooth/bluetooth_uuid.h"
+#include "device/bluetooth/public/cpp/bluetooth_uuid.h"
 #include "device/bluetooth/test/mock_bluetooth_gatt_characteristic.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
@@ -25,13 +25,11 @@ class MockBluetoothGattService : public BluetoothRemoteGattService {
   MockBluetoothGattService(MockBluetoothDevice* device,
                            const std::string& identifier,
                            const BluetoothUUID& uuid,
-                           bool is_primary,
-                           bool is_local);
+                           bool is_primary);
   ~MockBluetoothGattService() override;
 
   MOCK_CONST_METHOD0(GetIdentifier, std::string());
   MOCK_CONST_METHOD0(GetUUID, BluetoothUUID());
-  MOCK_CONST_METHOD0(IsLocal, bool());
   MOCK_CONST_METHOD0(IsPrimary, bool());
   MOCK_CONST_METHOD0(GetDevice, BluetoothDevice*());
   MOCK_CONST_METHOD0(GetCharacteristics,
@@ -40,9 +38,9 @@ class MockBluetoothGattService : public BluetoothRemoteGattService {
                      std::vector<BluetoothRemoteGattService*>());
   MOCK_CONST_METHOD1(GetCharacteristic,
                      BluetoothRemoteGattCharacteristic*(const std::string&));
-  MOCK_METHOD1(AddIncludedService, bool(BluetoothRemoteGattService*));
-  MOCK_METHOD2(Register, void(const base::Closure&, const ErrorCallback&));
-  MOCK_METHOD2(Unregister, void(const base::Closure&, const ErrorCallback&));
+  MOCK_CONST_METHOD1(
+      GetCharacteristicsByUUID,
+      std::vector<BluetoothRemoteGattCharacteristic*>(const BluetoothUUID&));
 
   void AddMockCharacteristic(
       std::unique_ptr<MockBluetoothGattCharacteristic> mock_characteristic);
