@@ -8,7 +8,7 @@
 #include <string>
 
 #include "device/bluetooth/bluetooth_socket.h"
-#include "device/bluetooth/bluetooth_uuid.h"
+#include "device/bluetooth/public/cpp/bluetooth_uuid.h"
 #include "net/base/io_buffer.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
@@ -17,20 +17,19 @@ namespace device {
 class MockBluetoothSocket : public BluetoothSocket {
  public:
   MockBluetoothSocket();
-  MOCK_METHOD0(Close, void());
-  MOCK_METHOD1(Disconnect, void(const base::Closure& success_callback));
+  MOCK_METHOD1(Disconnect, void(base::OnceClosure success_callback));
   MOCK_METHOD3(Receive,
                void(int count,
-                    const ReceiveCompletionCallback& success_callback,
-                    const ReceiveErrorCompletionCallback& error_callback));
+                    ReceiveCompletionCallback success_callback,
+                    ReceiveErrorCompletionCallback error_callback));
   MOCK_METHOD4(Send,
                void(scoped_refptr<net::IOBuffer> buffer,
                     int buffer_size,
-                    const SendCompletionCallback& success_callback,
-                    const ErrorCompletionCallback& error_callback));
+                    SendCompletionCallback success_callback,
+                    ErrorCompletionCallback error_callback));
   MOCK_METHOD2(Accept,
-               void(const AcceptCompletionCallback& success_callback,
-                    const ErrorCompletionCallback& error_callback));
+               void(AcceptCompletionCallback success_callback,
+                    ErrorCompletionCallback error_callback));
 
  protected:
   ~MockBluetoothSocket() override;
