@@ -27,35 +27,35 @@ class TestBluetoothLocalGattServiceDelegate
       const BluetoothDevice* device,
       const BluetoothLocalGattCharacteristic* characteristic,
       int offset,
-      const ValueCallback& callback,
-      const ErrorCallback& error_callback) override;
+      ValueCallback callback,
+      ErrorCallback error_callback) override;
   void OnCharacteristicWriteRequest(
       const BluetoothDevice* device,
       const BluetoothLocalGattCharacteristic* characteristic,
       const std::vector<uint8_t>& value,
       int offset,
-      const base::Closure& callback,
-      const ErrorCallback& error_callback) override;
+      base::OnceClosure callback,
+      ErrorCallback error_callback) override;
   void OnCharacteristicPrepareWriteRequest(
       const BluetoothDevice* device,
       const BluetoothLocalGattCharacteristic* characteristic,
       const std::vector<uint8_t>& value,
       int offset,
       bool has_subsequent_request,
-      const base::Closure& callback,
-      const ErrorCallback& error_callback) override;
+      base::OnceClosure callback,
+      ErrorCallback error_callback) override;
   void OnDescriptorReadRequest(const BluetoothDevice* device,
                                const BluetoothLocalGattDescriptor* descriptor,
                                int offset,
-                               const ValueCallback& callback,
-                               const ErrorCallback& error_callback) override;
+                               ValueCallback callback,
+                               ErrorCallback error_callback) override;
 
   void OnDescriptorWriteRequest(const BluetoothDevice* device,
                                 const BluetoothLocalGattDescriptor* descriptor,
                                 const std::vector<uint8_t>& value,
                                 int offset,
-                                const base::Closure& callback,
-                                const ErrorCallback& error_callback) override;
+                                base::OnceClosure callback,
+                                ErrorCallback error_callback) override;
   void OnNotificationsStart(
       const BluetoothDevice* device,
       device::BluetoothGattCharacteristic::NotificationType notification_type,
