@@ -30,12 +30,13 @@ class COMPONENT_EXPORT(DEVICE_FIDO) AuthenticatorMakeCredentialResponse
  public:
   static base::Optional<AuthenticatorMakeCredentialResponse>
   CreateFromU2fRegisterResponse(
-      FidoTransportProtocol transport_used,
+      base::Optional<FidoTransportProtocol> transport_used,
       base::span<const uint8_t, kRpIdHashLength> relying_party_id_hash,
       base::span<const uint8_t> u2f_data);
 
-  AuthenticatorMakeCredentialResponse(FidoTransportProtocol transport_used,
-                                      AttestationObject attestation_object);
+  AuthenticatorMakeCredentialResponse(
+      base::Optional<FidoTransportProtocol> transport_used,
+      AttestationObject attestation_object);
   AuthenticatorMakeCredentialResponse(
       AuthenticatorMakeCredentialResponse&& that);
   AuthenticatorMakeCredentialResponse& operator=(
@@ -44,10 +45,10 @@ class COMPONENT_EXPORT(DEVICE_FIDO) AuthenticatorMakeCredentialResponse
 
   std::vector<uint8_t> GetCBOREncodedAttestationObject() const;
 
-  // Replaces the attestation statement with a “none” attestation and removes
-  // AAGUID from authenticator data section.
+  // Replaces the attestation statement with a “none” attestation, and removes
+  // AAGUID from authenticator data section unless |preserve_aaguid| is true.
   // https://w3c.github.io/webauthn/#createCredential
-  void EraseAttestationStatement();
+  void EraseAttestationStatement(AttestationObject::AAGUID erase_aaguid);
 
   // Returns true if the attestation is a "self" attestation, i.e. is just the
   // private key signing itself to show that it is fresh and the AAGUID is zero.
@@ -66,19 +67,55 @@ class COMPONENT_EXPORT(DEVICE_FIDO) AuthenticatorMakeCredentialResponse
     return attestation_object_;
   }
 
-  FidoTransportProtocol transport_used() const { return transport_used_; }
+  base::Optional<FidoTransportProtocol> transport_used() const {
+    return transport_used_;
+  }
+
+  base::Optional<std::array<uint8_t, kLargeBlobKeyLength>> large_blob_key()
+      const {
+    return large_blob_key_;
+  }
+  void set_large_blob_key(
+      const base::span<const uint8_t, kLargeBlobKeyLength> large_blob_key);
+
+  // enterprise_attestation_returned is true if the authenticator indicated that
+  // it returned an enterprise attestation. Note: U2F authenticators can
+  // support enterprise/individual attestation but cannot indicate when they
+  // have done so, so this will always be false in the U2F case.
+  bool enterprise_attestation_returned = false;
+
+  // is_resident_key indicates whether the created credential is client-side
+  // discoverable. It is nullopt if no discoverable credential was requested,
+  // but the authenticator may have created one anyway.
+  base::Optional<bool> is_resident_key;
+
+  // attestation_should_be_filtered is true iff a filter indicated that the
+  // attestation should not be returned. This is acted upon by
+  // |AuthenticatorCommon| based on enterprise policy.
+  bool attestation_should_be_filtered = false;
 
  private:
   AttestationObject attestation_object_;
 
-  // Contains the transport used to register the credential in this case.
-  FidoTransportProtocol transport_used_;
+  // Contains the transport used to register the credential in this case. It is
+  // nullopt for cases where we cannot determine the transport (Windows).
+  base::Optional<FidoTransportProtocol> transport_used_;
+
+  // The large blob key associated to the credential. This value is only
+  // returned if the credential is created with the largeBlobKey extension on a
+  // capable authenticator.
+  base::Optional<std::array<uint8_t, kLargeBlobKeyLength>> large_blob_key_;
 
   DISALLOW_COPY_AND_ASSIGN(AuthenticatorMakeCredentialResponse);
 };
 
+// Through cbor::Writer, produces a CTAP style CBOR-encoded byte array
+// that conforms to the format CTAP2 devices sends to the client as a response.
+// {01: attestation format name,
+//  02: authenticator data bytes,
+//  03: attestation statement bytes }
 COMPONENT_EXPORT(DEVICE_FIDO)
-std::vector<uint8_t> GetSerializedCtapDeviceResponse(
+std::vector<uint8_t> AsCTAPStyleCBORBytes(
     const AuthenticatorMakeCredentialResponse& response);
 
 }  // namespace device
