@@ -7,10 +7,11 @@
 #include <utility>
 
 #include "base/macros.h"
-#include "base/test/bind_test_util.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/bind.h"
+#include "base/test/task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
+#include "device/fido/fido_discovery_factory.h"
 #include "device/fido/fido_test_data.h"
 #include "device/fido/mock_fido_device.h"
 #include "device/fido/mock_fido_discovery_observer.h"
@@ -28,19 +29,16 @@ class FakeFidoDiscoveryTest : public ::testing::Test {
   ~FakeFidoDiscoveryTest() override = default;
 
  protected:
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  FakeFidoDiscoveryFactory fake_fido_discovery_factory_;
+  base::test::TaskEnvironment task_environment_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(FakeFidoDiscoveryTest);
 };
 
-using ScopedFakeFidoDiscoveryFactoryTest = FakeFidoDiscoveryTest;
+using FakeFidoDiscoveryFactoryTest = FakeFidoDiscoveryTest;
 
 TEST_F(FakeFidoDiscoveryTest, Transport) {
-  FakeFidoDiscovery discovery_ble(FidoTransportProtocol::kBluetoothLowEnergy);
-  EXPECT_EQ(FidoTransportProtocol::kBluetoothLowEnergy,
-            discovery_ble.transport());
-
   FakeFidoDiscovery discovery_hid(
       FidoTransportProtocol::kUsbHumanInterfaceDevice);
   EXPECT_EQ(FidoTransportProtocol::kUsbHumanInterfaceDevice,
@@ -48,14 +46,14 @@ TEST_F(FakeFidoDiscoveryTest, Transport) {
 }
 
 TEST_F(FakeFidoDiscoveryTest, InitialState) {
-  FakeFidoDiscovery discovery(FidoTransportProtocol::kBluetoothLowEnergy);
+  FakeFidoDiscovery discovery(FidoTransportProtocol::kUsbHumanInterfaceDevice);
 
   ASSERT_FALSE(discovery.is_running());
   ASSERT_FALSE(discovery.is_start_requested());
 }
 
 TEST_F(FakeFidoDiscoveryTest, StartDiscovery) {
-  FakeFidoDiscovery discovery(FidoTransportProtocol::kBluetoothLowEnergy);
+  FakeFidoDiscovery discovery(FidoTransportProtocol::kUsbHumanInterfaceDevice);
 
   MockFidoDiscoveryObserver observer;
   discovery.set_observer(&observer);
@@ -65,14 +63,15 @@ TEST_F(FakeFidoDiscoveryTest, StartDiscovery) {
   ASSERT_TRUE(discovery.is_start_requested());
   ASSERT_FALSE(discovery.is_running());
 
-  EXPECT_CALL(observer, DiscoveryStarted(&discovery, true));
+  EXPECT_CALL(observer, DiscoveryStarted(&discovery, true,
+                                         std::vector<FidoAuthenticator*>()));
   discovery.WaitForCallToStartAndSimulateSuccess();
   ASSERT_TRUE(discovery.is_running());
   ASSERT_TRUE(discovery.is_start_requested());
 }
 
 TEST_F(FakeFidoDiscoveryTest, WaitThenStartStopDiscovery) {
-  FakeFidoDiscovery discovery(FidoTransportProtocol::kBluetoothLowEnergy);
+  FakeFidoDiscovery discovery(FidoTransportProtocol::kUsbHumanInterfaceDevice);
 
   MockFidoDiscoveryObserver observer;
   discovery.set_observer(&observer);
@@ -85,7 +84,8 @@ TEST_F(FakeFidoDiscoveryTest, WaitThenStartStopDiscovery) {
   ASSERT_FALSE(discovery.is_running());
   ASSERT_TRUE(discovery.is_start_requested());
 
-  EXPECT_CALL(observer, DiscoveryStarted(&discovery, true));
+  EXPECT_CALL(observer, DiscoveryStarted(&discovery, true,
+                                         std::vector<FidoAuthenticator*>()));
   discovery.SimulateStarted(true);
   ASSERT_TRUE(discovery.is_running());
   ASSERT_TRUE(discovery.is_start_requested());
@@ -93,7 +93,7 @@ TEST_F(FakeFidoDiscoveryTest, WaitThenStartStopDiscovery) {
 
 // Starting discovery and failing: instance stays in "not running" state
 TEST_F(FakeFidoDiscoveryTest, StartFail) {
-  FakeFidoDiscovery discovery(FidoTransportProtocol::kBluetoothLowEnergy);
+  FakeFidoDiscovery discovery(FidoTransportProtocol::kUsbHumanInterfaceDevice);
 
   MockFidoDiscoveryObserver observer;
   discovery.set_observer(&observer);
@@ -103,7 +103,8 @@ TEST_F(FakeFidoDiscoveryTest, StartFail) {
   ASSERT_FALSE(discovery.is_running());
   ASSERT_TRUE(discovery.is_start_requested());
 
-  EXPECT_CALL(observer, DiscoveryStarted(&discovery, false));
+  EXPECT_CALL(observer, DiscoveryStarted(&discovery, false,
+                                         std::vector<FidoAuthenticator*>()));
   discovery.SimulateStarted(false);
   ASSERT_FALSE(discovery.is_running());
   ASSERT_TRUE(discovery.is_start_requested());
@@ -111,25 +112,22 @@ TEST_F(FakeFidoDiscoveryTest, StartFail) {
 
 // Adding device is possible both before and after discovery actually starts.
 TEST_F(FakeFidoDiscoveryTest, AddDevice) {
-  FakeFidoDiscovery discovery(FidoTransportProtocol::kBluetoothLowEnergy);
+  FakeFidoDiscovery discovery(FidoTransportProtocol::kUsbHumanInterfaceDevice);
 
   MockFidoDiscoveryObserver observer;
   discovery.set_observer(&observer);
 
   discovery.Start();
-
   auto device0 = std::make_unique<MockFidoDevice>();
   EXPECT_CALL(*device0, GetId()).WillOnce(::testing::Return("device0"));
   base::RunLoop device0_done;
-  EXPECT_CALL(observer, AuthenticatorAdded(&discovery, _))
+  discovery.AddDevice(std::move(device0));
+
+  EXPECT_CALL(observer, DiscoveryStarted(&discovery, true, testing::SizeIs(1)))
       .WillOnce(testing::InvokeWithoutArgs(
           [&device0_done]() { device0_done.Quit(); }));
-  discovery.AddDevice(std::move(device0));
-  device0_done.Run();
-  ::testing::Mock::VerifyAndClearExpectations(&observer);
-
-  EXPECT_CALL(observer, DiscoveryStarted(&discovery, true));
   discovery.SimulateStarted(true);
+  device0_done.Run();
   ::testing::Mock::VerifyAndClearExpectations(&observer);
 
   auto device1 = std::make_unique<MockFidoDevice>();
@@ -144,37 +142,18 @@ TEST_F(FakeFidoDiscoveryTest, AddDevice) {
 }
 
 #if !defined(OS_ANDROID)
-TEST_F(ScopedFakeFidoDiscoveryFactoryTest,
-       OverridesUsbFactoryFunctionWhileInScope) {
-  ScopedFakeFidoDiscoveryFactory factory;
-  auto* injected_fake_discovery = factory.ForgeNextHidDiscovery();
+TEST_F(FakeFidoDiscoveryFactoryTest, ForgesUsbFactoryFunction) {
+  auto* injected_fake_discovery =
+      fake_fido_discovery_factory_.ForgeNextHidDiscovery();
   ASSERT_EQ(FidoTransportProtocol::kUsbHumanInterfaceDevice,
             injected_fake_discovery->transport());
-  auto produced_discovery = FidoDeviceDiscovery::Create(
-      FidoTransportProtocol::kUsbHumanInterfaceDevice, nullptr);
-  EXPECT_TRUE(produced_discovery);
-  EXPECT_EQ(injected_fake_discovery, produced_discovery.get());
+  std::vector<std::unique_ptr<FidoDiscoveryBase>> produced_discoveries =
+      fake_fido_discovery_factory_.Create(
+          FidoTransportProtocol::kUsbHumanInterfaceDevice);
+  ASSERT_EQ(produced_discoveries.size(), 1u);
+  EXPECT_EQ(injected_fake_discovery, produced_discoveries[0].get());
 }
 #endif
 
-TEST_F(ScopedFakeFidoDiscoveryFactoryTest,
-       OverridesBleFactoryFunctionWhileInScope) {
-  ScopedFakeFidoDiscoveryFactory factory;
-
-  auto* injected_fake_discovery_1 = factory.ForgeNextBleDiscovery();
-  ASSERT_EQ(FidoTransportProtocol::kBluetoothLowEnergy,
-            injected_fake_discovery_1->transport());
-  auto produced_discovery_1 = FidoDeviceDiscovery::Create(
-      FidoTransportProtocol::kBluetoothLowEnergy, nullptr);
-  EXPECT_EQ(injected_fake_discovery_1, produced_discovery_1.get());
-
-  auto* injected_fake_discovery_2 = factory.ForgeNextBleDiscovery();
-  ASSERT_EQ(FidoTransportProtocol::kBluetoothLowEnergy,
-            injected_fake_discovery_2->transport());
-  auto produced_discovery_2 = FidoDeviceDiscovery::Create(
-      FidoTransportProtocol::kBluetoothLowEnergy, nullptr);
-  EXPECT_EQ(injected_fake_discovery_2, produced_discovery_2.get());
-}
-
 }  // namespace test
 }  // namespace device
