@@ -9,13 +9,16 @@
 
 #include "base/bind.h"
 #include "base/numerics/safe_conversions.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/strings/utf_string_conversions.h"
+#include "base/test/task_environment.h"
+#include "build/build_config.h"
 #include "device/bluetooth/bluetooth_adapter_factory.h"
 #include "device/bluetooth/test/mock_bluetooth_adapter.h"
 #include "device/fido/fake_fido_discovery.h"
 #include "device/fido/fido_constants.h"
 #include "device/fido/fido_device.h"
-#include "device/fido/fido_request_handler.h"
+#include "device/fido/fido_device_authenticator.h"
+#include "device/fido/fido_request_handler_base.h"
 #include "device/fido/fido_task.h"
 #include "device/fido/fido_test_data.h"
 #include "device/fido/fido_transport_protocol.h"
@@ -24,6 +27,10 @@
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if defined(OS_WIN)
+#include "device/fido/win/fake_webauthn_api.h"
+#endif  // defined(OS_WIN)
+
 using ::testing::_;
 
 namespace device {
@@ -33,14 +40,10 @@ namespace {
 using FakeTaskCallback =
     base::OnceCallback<void(CtapDeviceResponseCode status_code,
                             base::Optional<std::vector<uint8_t>>)>;
-using FakeHandlerCallback =
-    base::OnceCallback<void(FidoReturnCode status_code,
-                            base::Optional<std::vector<uint8_t>> response_data,
-                            FidoTransportProtocol)>;
 using FakeHandlerCallbackReceiver =
-    test::StatusAndValuesCallbackReceiver<FidoReturnCode,
+    test::StatusAndValuesCallbackReceiver<bool,
                                           base::Optional<std::vector<uint8_t>>,
-                                          FidoTransportProtocol>;
+                                          const FidoAuthenticator*>;
 
 enum class FakeTaskResponse : uint8_t {
   kSuccess = 0x00,
@@ -49,42 +52,46 @@ enum class FakeTaskResponse : uint8_t {
   kOperationDenied = 0x03,
 };
 
-class TestTransportAvailabilityObserver
-    : public FidoRequestHandlerBase::TransportAvailabilityObserver {
+// FidoRequestHandler that automatically starts discovery but does nothing on
+// DispatchRequest().
+class EmptyRequestHandler : public FidoRequestHandlerBase {
+ public:
+  EmptyRequestHandler(const base::flat_set<FidoTransportProtocol>& protocols,
+                      test::FakeFidoDiscoveryFactory* fake_discovery_factory)
+      : FidoRequestHandlerBase(fake_discovery_factory, protocols) {
+    Start();
+  }
+  ~EmptyRequestHandler() override = default;
+
+  void DispatchRequest(FidoAuthenticator* authenticator) override {}
+};
+
+class TestObserver : public FidoRequestHandlerBase::Observer {
  public:
   using TransportAvailabilityNotificationReceiver = test::TestCallbackReceiver<
       FidoRequestHandlerBase::TransportAvailabilityInfo>;
-  using AuthenticatorIdChangeNotificationReceiver =
-      test::TestCallbackReceiver<std::string>;
 
-  TestTransportAvailabilityObserver() {}
-  ~TestTransportAvailabilityObserver() override {}
+  TestObserver() = default;
+  ~TestObserver() override = default;
+
+  FidoRequestHandlerBase::TransportAvailabilityInfo
+  WaitForTransportAvailabilityInfo() {
+    transport_availability_notification_receiver_.WaitForCallback();
+    return std::get<0>(*transport_availability_notification_receiver_.result());
+  }
 
   void WaitForAndExpectAvailableTransportsAre(
       base::flat_set<FidoTransportProtocol> expected_transports,
-      base::Optional<bool> has_recognized_mac_touch_id_credential =
-          base::nullopt) {
-    transport_availability_notification_receiver_.WaitForCallback();
-    auto result =
-        std::get<0>(*transport_availability_notification_receiver_.result());
+      base::Optional<bool> has_platform_credential = base::nullopt) {
+    auto result = WaitForTransportAvailabilityInfo();
     EXPECT_THAT(result.available_transports,
                 ::testing::UnorderedElementsAreArray(expected_transports));
-    if (has_recognized_mac_touch_id_credential) {
-      EXPECT_EQ(*has_recognized_mac_touch_id_credential,
-                result.has_recognized_mac_touch_id_credential);
-    }
-  }
-
-  void WaitForAuthenticatorIdChangeNotification(
-      base::StringPiece expected_new_authenticator_id) {
-    authenticator_id_change_notification_receiver_.WaitForCallback();
-    auto result =
-        std::get<0>(*authenticator_id_change_notification_receiver_.result());
-    EXPECT_EQ(expected_new_authenticator_id, result);
+    EXPECT_EQ(result.has_recognized_platform_authenticator_credential,
+              has_platform_credential);
   }
 
  protected:
-  // FidoRequestHandlerBase::TransportAvailabilityObserver:
+  // FidoRequestHandlerBase::Observer:
   void OnTransportAvailabilityEnumerated(
       FidoRequestHandlerBase::TransportAvailabilityInfo data) override {
     transport_availability_notification_receiver_.callback().Run(
@@ -99,21 +106,28 @@ class TestTransportAvailabilityObserver
   void FidoAuthenticatorAdded(const FidoAuthenticator& authenticator) override {
   }
   void FidoAuthenticatorRemoved(base::StringPiece device_id) override {}
-  void FidoAuthenticatorIdChanged(base::StringPiece old_authenticator_id,
-                                  std::string new_authenticator_id) override {
-    authenticator_id_change_notification_receiver_.callback().Run(
-        std::move(new_authenticator_id));
+
+  bool SupportsPIN() const override { return false; }
+
+  void CollectPIN(
+      CollectPINOptions options,
+      base::OnceCallback<void(base::string16)> provide_pin_cb) override {
+    NOTREACHED();
   }
-  void FidoAuthenticatorPairingModeChanged(
-      base::StringPiece authenticator_id) override {}
+
+  void OnRetryUserVerification(int attempts) override {}
+
+  void StartBioEnrollment(base::OnceClosure next_callback) override {}
+
+  void OnSampleCollected(int remaining_samples) override {}
+
+  void FinishCollectToken() override { NOTREACHED(); }
 
  private:
   TransportAvailabilityNotificationReceiver
       transport_availability_notification_receiver_;
-  AuthenticatorIdChangeNotificationReceiver
-      authenticator_id_change_notification_receiver_;
 
-  DISALLOW_COPY_AND_ASSIGN(TestTransportAvailabilityObserver);
+  DISALLOW_COPY_AND_ASSIGN(TestObserver);
 };
 
 // Fake FidoTask implementation that sends an empty byte array to the device
@@ -121,13 +135,21 @@ class TestTransportAvailabilityObserver
 class FakeFidoTask : public FidoTask {
  public:
   FakeFidoTask(FidoDevice* device, FakeTaskCallback callback)
-      : FidoTask(device), callback_(std::move(callback)), weak_factory_(this) {}
+      : FidoTask(device), callback_(std::move(callback)) {}
   ~FakeFidoTask() override = default;
 
+  void Cancel() override {
+    if (token_) {
+      device()->Cancel(*token_);
+      token_.reset();
+    }
+  }
+
   void StartTask() override {
-    device()->DeviceTransact(std::vector<uint8_t>(),
-                             base::BindOnce(&FakeFidoTask::CompletionCallback,
-                                            weak_factory_.GetWeakPtr()));
+    token_ = device()->DeviceTransact(
+        std::vector<uint8_t>(),
+        base::BindOnce(&FakeFidoTask::CompletionCallback,
+                       weak_factory_.GetWeakPtr()));
   }
 
   void CompletionCallback(
@@ -157,42 +179,87 @@ class FakeFidoTask : public FidoTask {
   }
 
  private:
+  base::Optional<FidoDevice::CancelToken> token_;
   FakeTaskCallback callback_;
-  base::WeakPtrFactory<FakeFidoTask> weak_factory_;
+  base::WeakPtrFactory<FakeFidoTask> weak_factory_{this};
 };
 
-class FakeFidoAuthenticator : public FidoDeviceAuthenticator {
+class FakeFidoRequestHandler : public FidoRequestHandlerBase {
  public:
-  explicit FakeFidoAuthenticator(std::unique_ptr<FidoDevice> device)
-      : FidoDeviceAuthenticator(std::move(device)) {}
+  using CompletionCallback =
+      base::OnceCallback<void(bool,
+                              base::Optional<std::vector<uint8_t>>,
+                              const FidoAuthenticator*)>;
 
-  void RunFakeTask(FakeTaskCallback callback) {
-    SetTaskForTesting(
-        std::make_unique<FakeFidoTask>(device(), std::move(callback)));
-  }
-};
-
-class FakeFidoRequestHandler : public FidoRequestHandler<std::vector<uint8_t>> {
- public:
-  FakeFidoRequestHandler(const base::flat_set<FidoTransportProtocol>& protocols,
-                         FakeHandlerCallback callback)
-      : FidoRequestHandler(nullptr /* connector */,
-                           protocols,
-                           std::move(callback)),
-        weak_factory_(this) {
+  FakeFidoRequestHandler(test::FakeFidoDiscoveryFactory* fake_discovery_factory,
+                         const base::flat_set<FidoTransportProtocol>& protocols,
+                         CompletionCallback callback)
+      : FidoRequestHandlerBase(fake_discovery_factory, protocols),
+        completion_callback_(std::move(callback)) {
     Start();
   }
   ~FakeFidoRequestHandler() override = default;
 
-  void DispatchRequest(FidoAuthenticator* authenticator) override {
-    static_cast<FakeFidoAuthenticator*>(authenticator)
-        ->RunFakeTask(
-            base::BindOnce(&FakeFidoRequestHandler::OnAuthenticatorResponse,
-                           weak_factory_.GetWeakPtr(), authenticator));
+  void set_has_platform_credential(bool has_platform_credential) {
+    has_platform_credential_ = has_platform_credential;
   }
 
  private:
-  base::WeakPtrFactory<FakeFidoRequestHandler> weak_factory_;
+  void DispatchRequest(FidoAuthenticator* authenticator) override {
+    // FidoRequestHandlerTest uses FakeDiscovery to inject mock devices
+    // that get wrapped in a FidoDeviceAuthenticator, so we can safely cast
+    // here.
+    auto* device_authenticator =
+        static_cast<FidoDeviceAuthenticator*>(authenticator);
+    // Instead of sending a real CTAP request, send an empty byte array. Note
+    // that during discovery, the device already has received a GetInfo command
+    // at this point.
+    device_authenticator->SetTaskForTesting(std::make_unique<FakeFidoTask>(
+        device_authenticator->device(),
+        base::BindOnce(&FakeFidoRequestHandler::HandleResponse,
+                       weak_factory_.GetWeakPtr(), authenticator)));
+  }
+
+  void AuthenticatorAdded(FidoDiscoveryBase* discovery,
+                          FidoAuthenticator* authenticator) override {
+    if (authenticator->AuthenticatorTransport() ==
+        FidoTransportProtocol::kInternal) {
+      transport_availability_info()
+          .has_recognized_platform_authenticator_credential =
+          has_platform_credential_;
+    }
+
+    FidoRequestHandlerBase::AuthenticatorAdded(discovery, authenticator);
+  }
+
+  void HandleResponse(FidoAuthenticator* authenticator,
+                      CtapDeviceResponseCode status,
+                      base::Optional<std::vector<uint8_t>> response) {
+    auto* device_authenticator =
+        static_cast<FidoDeviceAuthenticator*>(authenticator);
+    device_authenticator->SetTaskForTesting(nullptr);
+
+    if (status == CtapDeviceResponseCode::kCtap2ErrOther) {
+      // Simulates an error that is sent without the user touching the
+      // authenticator (FakeTaskResponse::kProcessingError). Don't resolve
+      // the request for this response.
+      return;
+    }
+
+    if (!completion_callback_) {
+      return;
+    }
+
+    CancelActiveAuthenticators(authenticator->GetId());
+    std::move(completion_callback_)
+        .Run(status == CtapDeviceResponseCode::kSuccess, std::move(response),
+             authenticator);
+  }
+
+  CompletionCallback completion_callback_;
+  bool has_platform_credential_ = false;
+
+  base::WeakPtrFactory<FakeFidoRequestHandler> weak_factory_{this};
 };
 
 std::vector<uint8_t> CreateFakeSuccessDeviceResponse() {
@@ -223,42 +290,31 @@ class FidoRequestHandlerTest : public ::testing::Test {
   }
 
   void ForgeNextHidDiscovery() {
-    discovery_ = scoped_fake_discovery_factory_.ForgeNextHidDiscovery();
-    ble_discovery_ = scoped_fake_discovery_factory_.ForgeNextBleDiscovery();
+    discovery_ = fake_discovery_factory_.ForgeNextHidDiscovery();
   }
 
   std::unique_ptr<FakeFidoRequestHandler> CreateFakeHandler() {
     ForgeNextHidDiscovery();
     auto handler = std::make_unique<FakeFidoRequestHandler>(
+        &fake_discovery_factory_,
         base::flat_set<FidoTransportProtocol>(
-            {FidoTransportProtocol::kUsbHumanInterfaceDevice,
-             FidoTransportProtocol::kBluetoothLowEnergy}),
+            {FidoTransportProtocol::kUsbHumanInterfaceDevice}),
         cb_.callback());
-    handler->SetPlatformAuthenticatorOrMarkUnavailable(base::nullopt);
     return handler;
   }
 
-  void ChangeAuthenticatorId(FakeFidoRequestHandler* request_handler,
-                             FidoDevice* device,
-                             std::string new_authenticator_id) {
-    request_handler->AuthenticatorIdChanged(ble_discovery_, device->GetId(),
-                                            std::move(new_authenticator_id));
-  }
-
   test::FakeFidoDiscovery* discovery() const { return discovery_; }
-  test::FakeFidoDiscovery* ble_discovery() const { return ble_discovery_; }
   scoped_refptr<::testing::NiceMock<MockBluetoothAdapter>> adapter() {
     return mock_adapter_;
   }
   FakeHandlerCallbackReceiver& callback() { return cb_; }
 
  protected:
-  base::test::ScopedTaskEnvironment scoped_task_environment_{
-      base::test::ScopedTaskEnvironment::MainThreadType::MOCK_TIME};
-  test::ScopedFakeFidoDiscoveryFactory scoped_fake_discovery_factory_;
+  base::test::TaskEnvironment task_environment_{
+      base::test::TaskEnvironment::TimeSource::MOCK_TIME};
+  test::FakeFidoDiscoveryFactory fake_discovery_factory_;
   scoped_refptr<::testing::NiceMock<MockBluetoothAdapter>> mock_adapter_;
   test::FakeFidoDiscovery* discovery_;
-  test::FakeFidoDiscovery* ble_discovery_;
   FakeHandlerCallbackReceiver cb_;
 };
 
@@ -276,8 +332,7 @@ TEST_F(FidoRequestHandlerTest, TestSingleDeviceSuccess) {
 
   discovery()->AddDevice(std::move(device));
   callback().WaitForCallback();
-  EXPECT_EQ(FidoReturnCode::kSuccess, callback().status());
-  EXPECT_TRUE(request_handler->is_complete());
+  EXPECT_TRUE(callback().status());
 }
 
 // Tests a scenario where two unresponsive authenticators are connected and
@@ -294,18 +349,18 @@ TEST_F(FidoRequestHandlerTest, TestAuthenticatorHandlerReset) {
       test_data::kTestAuthenticatorGetInfoResponse);
   EXPECT_CALL(*device0, GetId()).WillRepeatedly(testing::Return("device0"));
   device0->ExpectRequestAndDoNotRespond(std::vector<uint8_t>());
-  EXPECT_CALL(*device0, Cancel());
+  EXPECT_CALL(*device0, Cancel(_));
   auto device1 = std::make_unique<MockFidoDevice>();
   device1->ExpectCtap2CommandAndRespondWith(
       CtapRequestCommand::kAuthenticatorGetInfo,
       test_data::kTestAuthenticatorGetInfoResponse);
   EXPECT_CALL(*device1, GetId()).WillRepeatedly(testing::Return("device1"));
   device1->ExpectRequestAndDoNotRespond(std::vector<uint8_t>());
-  EXPECT_CALL(*device1, Cancel());
+  EXPECT_CALL(*device1, Cancel(_));
 
   discovery()->AddDevice(std::move(device0));
   discovery()->AddDevice(std::move(device1));
-  scoped_task_environment_.FastForwardUntilNoTasksRemain();
+  task_environment_.FastForwardUntilNoTasksRemain();
   request_handler.reset();
 }
 
@@ -323,7 +378,7 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleDevices) {
   EXPECT_CALL(*device0, GetId()).WillRepeatedly(testing::Return("device0"));
   // Device is unresponsive and cancel command is invoked afterwards.
   device0->ExpectRequestAndDoNotRespond(std::vector<uint8_t>());
-  EXPECT_CALL(*device0, Cancel());
+  EXPECT_CALL(*device0, Cancel(_));
 
   // Represents a connected device that response successfully.
   auto device1 = std::make_unique<MockFidoDevice>();
@@ -338,8 +393,7 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleDevices) {
   discovery()->AddDevice(std::move(device1));
 
   callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kSuccess, callback().status());
+  EXPECT_TRUE(callback().status());
 }
 
 // Test a scenario where 2 devices respond successfully with small time
@@ -371,15 +425,14 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleSuccessResponses) {
                                        CreateFakeSuccessDeviceResponse(),
                                        base::TimeDelta::FromMicroseconds(10));
   // Cancel command is invoked after receiving response from |device0|.
-  EXPECT_CALL(*device1, Cancel());
+  EXPECT_CALL(*device1, Cancel(_));
 
   discovery()->AddDevice(std::move(device0));
   discovery()->AddDevice(std::move(device1));
 
-  scoped_task_environment_.FastForwardUntilNoTasksRemain();
+  task_environment_.FastForwardUntilNoTasksRemain();
   callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kSuccess, callback().status());
+  EXPECT_TRUE(callback().status());
 }
 
 // Test a scenario where 3 devices respond with a processing error, an UP(user
@@ -399,6 +452,8 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleFailureResponses) {
       CtapRequestCommand::kAuthenticatorGetInfo,
       test_data::kTestAuthenticatorGetInfoResponse);
   EXPECT_CALL(*device0, GetId()).WillRepeatedly(testing::Return("device0"));
+  EXPECT_CALL(*device0, GetDisplayName())
+      .WillRepeatedly(testing::Return(std::string()));
   device0->ExpectRequestAndRespondWith(std::vector<uint8_t>(),
                                        CreateFakeDeviceProcesssingError());
 
@@ -409,6 +464,8 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleFailureResponses) {
       CtapRequestCommand::kAuthenticatorGetInfo,
       test_data::kTestAuthenticatorGetInfoResponse);
   EXPECT_CALL(*device1, GetId()).WillRepeatedly(testing::Return("device1"));
+  EXPECT_CALL(*device1, GetDisplayName())
+      .WillRepeatedly(testing::Return(std::string()));
   device1->ExpectRequestAndRespondWith(std::vector<uint8_t>(),
                                        CreateFakeUserPresenceVerifiedError(),
                                        base::TimeDelta::FromMicroseconds(1));
@@ -420,20 +477,20 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleFailureResponses) {
       CtapRequestCommand::kAuthenticatorGetInfo,
       test_data::kTestAuthenticatorGetInfoResponse);
   EXPECT_CALL(*device2, GetId()).WillRepeatedly(testing::Return("device2"));
+  EXPECT_CALL(*device2, GetDisplayName())
+      .WillRepeatedly(testing::Return(std::string()));
   device2->ExpectRequestAndRespondWith(std::vector<uint8_t>(),
                                        CreateFakeDeviceProcesssingError(),
                                        base::TimeDelta::FromMicroseconds(10));
-  EXPECT_CALL(*device2, Cancel());
+  EXPECT_CALL(*device2, Cancel(_));
 
   discovery()->AddDevice(std::move(device0));
   discovery()->AddDevice(std::move(device1));
   discovery()->AddDevice(std::move(device2));
 
-  scoped_task_environment_.FastForwardUntilNoTasksRemain();
+  task_environment_.FastForwardUntilNoTasksRemain();
   callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kUserConsentButCredentialNotRecognized,
-            callback().status());
+  EXPECT_FALSE(callback().status());
 }
 
 // If a device with transport type kInternal returns a
@@ -441,8 +498,7 @@ TEST_F(FidoRequestHandlerTest, TestRequestWithMultipleFailureResponses) {
 // pending authenticators.
 TEST_F(FidoRequestHandlerTest,
        TestRequestWithOperationDeniedErrorInternalTransport) {
-  auto request_handler = CreateFakeHandler();
-  discovery()->WaitForCallToStartAndSimulateSuccess();
+  TestObserver observer;
 
   // Device will send CTAP2_ERR_OPERATION_DENIED.
   auto device0 = MockFidoDevice::MakeCtapWithGetInfoExpectation(
@@ -452,17 +508,29 @@ TEST_F(FidoRequestHandlerTest,
                                        CreateFakeOperationDeniedError(),
                                        base::TimeDelta::FromMicroseconds(10));
 
+  ForgeNextHidDiscovery();
+  auto* platform_discovery =
+      fake_discovery_factory_.ForgeNextPlatformDiscovery();
+  auto request_handler = std::make_unique<FakeFidoRequestHandler>(
+      &fake_discovery_factory_,
+      base::flat_set<FidoTransportProtocol>(
+          {FidoTransportProtocol::kInternal,
+           FidoTransportProtocol::kUsbHumanInterfaceDevice}),
+      callback().callback());
+  request_handler->set_observer(&observer);
+
   auto device1 = MockFidoDevice::MakeCtapWithGetInfoExpectation();
   device1->ExpectRequestAndDoNotRespond(std::vector<uint8_t>());
-  EXPECT_CALL(*device1, Cancel());
+  EXPECT_CALL(*device1, Cancel(_));
 
   discovery()->AddDevice(std::move(device0));
-  discovery()->AddDevice(std::move(device1));
+  platform_discovery->AddDevice(std::move(device1));
+  discovery()->WaitForCallToStartAndSimulateSuccess();
+  platform_discovery->WaitForCallToStartAndSimulateSuccess();
 
-  scoped_task_environment_.FastForwardUntilNoTasksRemain();
+  task_environment_.FastForwardUntilNoTasksRemain();
   callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kUserConsentDenied, callback().status());
+  EXPECT_FALSE(callback().status());
 }
 
 // Like |TestRequestWithOperationDeniedErrorInternalTransport|, but with a
@@ -480,20 +548,18 @@ TEST_F(FidoRequestHandlerTest,
 
   auto device1 = MockFidoDevice::MakeCtapWithGetInfoExpectation();
   device1->ExpectRequestAndDoNotRespond(std::vector<uint8_t>());
-  EXPECT_CALL(*device1, Cancel());
+  EXPECT_CALL(*device1, Cancel(_));
 
   discovery()->AddDevice(std::move(device0));
   discovery()->AddDevice(std::move(device1));
 
-  scoped_task_environment_.FastForwardUntilNoTasksRemain();
+  task_environment_.FastForwardUntilNoTasksRemain();
   callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kUserConsentDenied, callback().status());
+  EXPECT_FALSE(callback().status());
 }
 
-// Requests should be dispatched to the authenticator passed to
-// SetPlatformAuthenticatorOrMarkUnavailable.
-TEST_F(FidoRequestHandlerTest, TestSetPlatformAuthenticator) {
+// Requests should be dispatched to the platform authenticator.
+TEST_F(FidoRequestHandlerTest, TestWithPlatformAuthenticator) {
   // A platform authenticator usually wouldn't usually use a FidoDevice, but
   // that's not the point of the test here. The test is only trying to ensure
   // the authenticator gets injected and used.
@@ -506,125 +572,81 @@ TEST_F(FidoRequestHandlerTest, TestSetPlatformAuthenticator) {
   device->ExpectRequestAndRespondWith(std::vector<uint8_t>(),
                                       CreateFakeSuccessDeviceResponse());
   device->SetDeviceTransport(FidoTransportProtocol::kInternal);
-  auto authenticator =
-      std::make_unique<FakeFidoAuthenticator>(std::move(device));
+  auto* fake_discovery = fake_discovery_factory_.ForgeNextPlatformDiscovery(
+      test::FakeFidoDiscovery::StartMode::kAutomatic);
 
-  TestTransportAvailabilityObserver observer;
+  TestObserver observer;
   auto request_handler = std::make_unique<FakeFidoRequestHandler>(
+      &fake_discovery_factory_,
       base::flat_set<FidoTransportProtocol>({FidoTransportProtocol::kInternal}),
       callback().callback());
+  request_handler->set_has_platform_credential(true);
   request_handler->set_observer(&observer);
-  request_handler->SetPlatformAuthenticatorOrMarkUnavailable(
-      PlatformAuthenticatorInfo(std::move(authenticator), false));
+  fake_discovery->AddDevice(std::move(device));
 
   observer.WaitForAndExpectAvailableTransportsAre(
       {FidoTransportProtocol::kInternal},
-      false /* has_recognized_mac_touch_id_credential */);
+      /*has_platform_credential=*/true);
 
   callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kSuccess, callback().status());
-}
-
-// SetPlatformAuthenticatorOrMarkUnavailable should propagate the
-// has_recognized_mac_touch_id_credential field.
-TEST_F(FidoRequestHandlerTest,
-       TestSetPlatformAuthenticatorHasTouchIdCredential) {
-  // A platform authenticator usually wouldn't usually use a FidoDevice, but
-  // that's not the point of the test here. The test is only trying to ensure
-  // the authenticator gets injected and used.
-  auto device = MockFidoDevice::MakeCtap();
-  EXPECT_CALL(*device, GetId()).WillRepeatedly(testing::Return("device0"));
-  // Device returns success response.
-  device->ExpectCtap2CommandAndRespondWith(
-      CtapRequestCommand::kAuthenticatorGetInfo,
-      test_data::kTestAuthenticatorGetInfoResponse);
-  device->ExpectRequestAndRespondWith(std::vector<uint8_t>(),
-                                      CreateFakeSuccessDeviceResponse());
-  device->SetDeviceTransport(FidoTransportProtocol::kInternal);
-  auto authenticator =
-      std::make_unique<FakeFidoAuthenticator>(std::move(device));
-
-  TestTransportAvailabilityObserver observer;
-  auto request_handler = std::make_unique<FakeFidoRequestHandler>(
-      base::flat_set<FidoTransportProtocol>({FidoTransportProtocol::kInternal}),
-      callback().callback());
-  request_handler->set_observer(&observer);
-  request_handler->SetPlatformAuthenticatorOrMarkUnavailable(
-      PlatformAuthenticatorInfo(std::move(authenticator), true));
-
-  observer.WaitForAndExpectAvailableTransportsAre(
-      {FidoTransportProtocol::kInternal},
-      true /* has_recognized_mac_touch_id_credential */);
-
-  callback().WaitForCallback();
-  EXPECT_TRUE(request_handler->is_complete());
-  EXPECT_EQ(FidoReturnCode::kSuccess, callback().status());
+  EXPECT_TRUE(callback().status());
 }
 
 TEST_F(FidoRequestHandlerTest, InternalTransportDisallowedIfMarkedUnavailable) {
-  TestTransportAvailabilityObserver observer;
+  TestObserver observer;
   auto request_handler = std::make_unique<FakeFidoRequestHandler>(
+      &fake_discovery_factory_,
       base::flat_set<FidoTransportProtocol>({FidoTransportProtocol::kInternal}),
       callback().callback());
   request_handler->set_observer(&observer);
-  request_handler->SetPlatformAuthenticatorOrMarkUnavailable(base::nullopt);
 
   observer.WaitForAndExpectAvailableTransportsAre({});
 }
 
-TEST_F(FidoRequestHandlerTest, BleTransportAllowedIfBluetoothAdapterPresent) {
-  EXPECT_CALL(*adapter(), IsPresent()).WillOnce(::testing::Return(true));
-
-  TestTransportAvailabilityObserver observer;
-  auto request_handler = CreateFakeHandler();
-  request_handler->set_observer(&observer);
-
-  observer.WaitForAndExpectAvailableTransportsAre(
-      {FidoTransportProtocol::kUsbHumanInterfaceDevice,
-       FidoTransportProtocol::kBluetoothLowEnergy});
-}
-
 TEST_F(FidoRequestHandlerTest,
-       BleTransportDisallowedBluetoothAdapterNotPresent) {
-  EXPECT_CALL(*adapter(), IsPresent()).WillOnce(::testing::Return(false));
-
-  TestTransportAvailabilityObserver observer;
+       TransportAvailabilityNotificationOnObserverSetLate) {
+  TestObserver observer;
   auto request_handler = CreateFakeHandler();
-  request_handler->set_observer(&observer);
+  discovery()->WaitForCallToStartAndSimulateSuccess();
+  task_environment_.FastForwardUntilNoTasksRemain();
 
+  request_handler->set_observer(&observer);
   observer.WaitForAndExpectAvailableTransportsAre(
       {FidoTransportProtocol::kUsbHumanInterfaceDevice});
 }
 
-TEST_F(FidoRequestHandlerTest,
-       TransportAvailabilityNotificationOnObserverSetLate) {
-  EXPECT_CALL(*adapter(), IsPresent()).WillOnce(::testing::Return(true));
+#if defined(OS_WIN)
+TEST_F(FidoRequestHandlerTest, TransportAvailabilityOfWindowsAuthenticator) {
+  FakeWinWebAuthnApi api;
+  fake_discovery_factory_.set_win_webauthn_api(&api);
+  for (const bool api_available : {false, true}) {
+    SCOPED_TRACE(::testing::Message() << "api_available=" << api_available);
+    api.set_available(api_available);
 
-  TestTransportAvailabilityObserver observer;
-  auto request_handler = CreateFakeHandler();
-  scoped_task_environment_.FastForwardUntilNoTasksRemain();
+    TestObserver observer;
+    ForgeNextHidDiscovery();
+    EmptyRequestHandler request_handler(
+        {FidoTransportProtocol::kUsbHumanInterfaceDevice},
+        &fake_discovery_factory_);
+    request_handler.set_observer(&observer);
 
-  request_handler->set_observer(&observer);
-  observer.WaitForAndExpectAvailableTransportsAre(
-      {FidoTransportProtocol::kUsbHumanInterfaceDevice,
-       FidoTransportProtocol::kBluetoothLowEnergy});
-}
+    // If the windows API is not enabled, the request is dispatched to the USB
+    // discovery. Simulate a success to fill the transport availability info.
+    if (!api_available)
+      discovery()->WaitForCallToStartAndSimulateSuccess();
 
-TEST_F(FidoRequestHandlerTest, EmbedderNotifiedWhenAuthenticatorIdChanges) {
-  static constexpr char kNewAuthenticatorId[] = "new_authenticator_id";
-  TestTransportAvailabilityObserver observer;
-  auto request_handler = CreateFakeHandler();
-  request_handler->set_observer(&observer);
-  ble_discovery()->WaitForCallToStartAndSimulateSuccess();
+    task_environment_.FastForwardUntilNoTasksRemain();
 
-  auto device = std::make_unique<MockFidoDevice>();
-  auto* device_ptr = device.get();
-  EXPECT_CALL(*device, GetId()).WillRepeatedly(testing::Return("device0"));
-  discovery()->AddDevice(std::move(device));
-
-  ChangeAuthenticatorId(request_handler.get(), device_ptr, kNewAuthenticatorId);
-  observer.WaitForAuthenticatorIdChangeNotification(kNewAuthenticatorId);
+    auto transport_availability_info =
+        observer.WaitForTransportAvailabilityInfo();
+    EXPECT_EQ(transport_availability_info.available_transports.empty(),
+              api_available);
+    EXPECT_EQ(transport_availability_info.has_win_native_api_authenticator,
+              api_available);
+    EXPECT_EQ(transport_availability_info.win_native_api_authenticator_id,
+              api_available ? "WinWebAuthnApiAuthenticator" : "");
+  }
 }
+#endif  // defined(OS_WIN)
 
 }  // namespace device
