@@ -14,7 +14,7 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/trace_event/trace_event.h"
 
-#include "jni/GamepadList_jni.h"
+#include "device/gamepad/jni_headers/GamepadList_jni.h"
 
 using base::android::AttachCurrentThread;
 using base::android::CheckException;
@@ -69,7 +69,8 @@ static void JNI_GamepadList_SetGamepadData(
     const JavaParamRef<jstring>& devicename,
     jlong timestamp,
     const JavaParamRef<jfloatArray>& jaxes,
-    const JavaParamRef<jfloatArray>& jbuttons) {
+    const JavaParamRef<jfloatArray>& jbuttons,
+    jint buttons_length) {
   DCHECK(data_fetcher);
   GamepadPlatformDataFetcherAndroid* fetcher =
       reinterpret_cast<GamepadPlatformDataFetcherAndroid*>(data_fetcher);
@@ -88,36 +89,28 @@ static void JNI_GamepadList_SetGamepadData(
   Gamepad& pad = state->data;
 
   // Is this the first time we've seen this device?
-  if (state->active_state == GAMEPAD_NEWLY_ACTIVE) {
+  if (!state->is_initialized) {
+    state->is_initialized = true;
     // Map the Gamepad DeviceName String to the Gamepad Id. Ideally it should
     // be mapped to vendor and product information but it is only available at
     // kernel level and it can not be queried using class
     // android.hardware.input.InputManager.
-    base::string16 device_name;
-    base::android::ConvertJavaStringToUTF16(env, devicename, &device_name);
-    const size_t name_to_copy =
-        std::min(device_name.size(), Gamepad::kIdLengthCap - 1);
-    memcpy(pad.id, device_name.data(),
-           name_to_copy * sizeof(base::string16::value_type));
-    pad.id[name_to_copy] = 0;
+    base::string16 gamepad_id;
+    base::android::ConvertJavaStringToUTF16(env, devicename, &gamepad_id);
+    pad.SetID(gamepad_id);
 
-    base::string16 mapping_name = base::UTF8ToUTF16(mapping ? "standard" : "");
-    const size_t mapping_to_copy =
-        std::min(mapping_name.size(), Gamepad::kMappingLengthCap - 1);
-    memcpy(pad.mapping, mapping_name.data(),
-           mapping_to_copy * sizeof(base::string16::value_type));
-    pad.mapping[mapping_to_copy] = 0;
+    pad.mapping = mapping ? GamepadMapping::kStandard : GamepadMapping::kNone;
   }
 
   pad.connected = true;
-  pad.timestamp = timestamp;
+  pad.timestamp = GamepadDataFetcher::CurrentTimeInMicroseconds();
 
   std::vector<float> axes;
   base::android::JavaFloatArrayToFloatVector(env, jaxes, &axes);
 
-  // Set Gamepad axeslength to total number of axes on the gamepad device.
-  // Only return the first axesLengthCap if axeslength captured by GamepadList
-  // is larger than axesLengthCap.
+  // Set Gamepad::axes_length to the total number of axes on the gamepad device.
+  // Only return the first kAxesLengthCap if the axes size captured by
+  // GamepadList is larger than kAxesLengthCap.
   pad.axes_length = std::min(static_cast<int>(axes.size()),
                              static_cast<int>(Gamepad::kAxesLengthCap));
 
@@ -129,15 +122,18 @@ static void JNI_GamepadList_SetGamepadData(
   std::vector<float> buttons;
   base::android::JavaFloatArrayToFloatVector(env, jbuttons, &buttons);
 
-  // Set Gamepad buttonslength to total number of axes on the gamepad
-  // device. Only return the first buttonsLengthCap if axeslength captured by
-  // GamepadList is larger than buttonsLengthCap.
-  pad.buttons_length = std::min(static_cast<int>(buttons.size()),
-                                static_cast<int>(Gamepad::kButtonsLengthCap));
+  // Set Gamepad::buttons_length to the total number of buttons on the gamepad
+  // device. Only return the first kButtonsLengthCap if buttons_length captured
+  // by GamepadList is larger than kButtonsLengthCap.
+  pad.buttons_length =
+      std::min({static_cast<int>(buttons.size()), buttons_length,
+                static_cast<int>(Gamepad::kButtonsLengthCap)});
 
   // Copy buttons state to the Gamepad buttons[].
   for (unsigned int j = 0; j < pad.buttons_length; j++) {
-    pad.buttons[j].pressed = buttons[j];
+    pad.buttons[j].pressed =
+        buttons[j] > GamepadButton::kDefaultButtonPressedThreshold;
+    pad.buttons[j].touched = buttons[j] > 0.0f;
     pad.buttons[j].value = buttons[j];
   }
 }
