@@ -4,10 +4,13 @@
 
 #include "device/gamepad/public/cpp/gamepad.h"
 
+#include <string.h>
+
 namespace device {
 
+const float GamepadButton::kDefaultButtonPressedThreshold;
+const double GamepadHapticActuator::kMaxEffectDurationMillis;
 const size_t Gamepad::kIdLengthCap;
-const size_t Gamepad::kMappingLengthCap;
 const size_t Gamepad::kAxesLengthCap;
 const size_t Gamepad::kButtonsLengthCap;
 
@@ -16,11 +19,16 @@ Gamepad::Gamepad()
       timestamp(0),
       axes_length(0),
       buttons_length(0),
+      mapping(GamepadMapping::kNone),
       display_id(0) {
   id[0] = 0;
-  mapping[0] = 0;
 }
 
 Gamepad::Gamepad(const Gamepad& other) = default;
 
+void Gamepad::SetID(const base::string16& src) {
+  memset(id, 0, sizeof(id));
+  src.copy(id, kIdLengthCap - 1);
+}
+
 }  // namespace device
