@@ -5,7 +5,12 @@
 #include "raw_input_gamepad_device_win.h"
 
 #include "base/stl_util.h"
+#include "base/strings/sys_string_conversions.h"
+#include "device/gamepad/dualshock4_controller.h"
+#include "device/gamepad/gamepad_blocklist.h"
 #include "device/gamepad/gamepad_data_fetcher.h"
+#include "device/gamepad/hid_haptic_gamepad.h"
+#include "device/gamepad/hid_writer_win.h"
 
 namespace device {
 
@@ -31,25 +36,6 @@ const uint32_t kSearchUsageNumber = 0x0221;
 const uint32_t kHomeUsageNumber = 0x0223;
 const uint32_t kBackUsageNumber = 0x0224;
 
-// Vendor IDs.
-const uint32_t kVendorOculus = 0x2833;
-const uint32_t kVendorBlue = 0xb58e;
-const uint32_t kVendorMicrosoft = 0x045e;
-
-// Product IDs.
-const uint32_t kProductSurfacePro2017Keyboard = 0x0922;
-
-struct VendorProductPair {
-  const uint16_t vendor;
-  const uint16_t product;
-} kFilteredDevices[] = {
-    // The Surface Pro 2017's detachable keyboard is a composite device with
-    // several HID sub-devices. Filter out the keyboard's device ID to avoid
-    // treating these sub-devices as gamepads.
-    {kVendorMicrosoft, kProductSurfacePro2017Keyboard},
-};
-const size_t kFilteredDevicesLen = base::size(kFilteredDevices);
-
 // The fetcher will collect all HID usages from the Button usage page and any
 // additional usages listed below.
 struct SpecialUsages {
@@ -85,9 +71,22 @@ RawInputGamepadDeviceWin::RawInputGamepadDeviceWin(
   if (hid_functions_->IsValid())
     is_valid_ = QueryDeviceInfo();
 
-  if (is_valid_ &&
-      Dualshock4ControllerWin::IsDualshock4(vendor_id_, product_id_)) {
-    dualshock4_ = std::make_unique<Dualshock4ControllerWin>(handle_);
+  if (is_valid_) {
+    const std::string product_name = base::SysWideToUTF8(product_string_);
+    const GamepadId gamepad_id = GamepadIdList::Get().GetGamepadId(
+        product_name, vendor_id_, product_id_);
+    if (Dualshock4Controller::IsDualshock4(gamepad_id)) {
+      // Dualshock4 has different behavior over USB and Bluetooth, but the
+      // RawInput API does not indicate which transport is in use. Detect the
+      // transport type by inspecting the version number reported by the device.
+      GamepadBusType bus_type =
+          Dualshock4Controller::BusTypeFromVersionNumber(version_number_);
+      dualshock4_ = std::make_unique<Dualshock4Controller>(
+          gamepad_id, bus_type, std::make_unique<HidWriterWin>(handle_));
+    } else if (HidHapticGamepad::IsHidHaptic(vendor_id_, product_id_)) {
+      hid_haptics_ = HidHapticGamepad::Create(
+          vendor_id_, product_id_, std::make_unique<HidWriterWin>(handle_));
+    }
   }
 }
 
@@ -102,13 +101,33 @@ bool RawInputGamepadDeviceWin::IsGamepadUsageId(uint16_t usage) {
 void RawInputGamepadDeviceWin::DoShutdown() {
   if (dualshock4_)
     dualshock4_->Shutdown();
-  dualshock4_ = nullptr;
+  dualshock4_.reset();
+  if (hid_haptics_)
+    hid_haptics_->Shutdown();
+  hid_haptics_.reset();
 }
 
 void RawInputGamepadDeviceWin::UpdateGamepad(RAWINPUT* input) {
   DCHECK(hid_functions_->IsValid());
   NTSTATUS status;
 
+  if (dualshock4_) {
+    // Handle Dualshock4 input reports that do not specify HID gamepad usages in
+    // the report descriptor.
+    uint8_t report_id = input->data.hid.bRawData[0];
+    auto report = base::make_span(input->data.hid.bRawData + 1,
+                                  input->data.hid.dwSizeHid);
+    Gamepad pad;
+    if (dualshock4_->ProcessInputReport(report_id, report, &pad)) {
+      for (size_t i = 0; i < Gamepad::kAxesLengthCap; ++i)
+        axes_[i].value = pad.axes[i];
+      for (size_t i = 0; i < Gamepad::kButtonsLengthCap; ++i)
+        buttons_[i] = pad.buttons[i].pressed;
+      last_update_timestamp_ = GamepadDataFetcher::CurrentTimeInMicroseconds();
+      return;
+    }
+  }
+
   // Query button state.
   if (buttons_length_ > 0) {
     // Clear the button state
@@ -193,8 +212,10 @@ void RawInputGamepadDeviceWin::ReadPadState(Gamepad* pad) const {
   pad->timestamp = last_update_timestamp_;
   pad->buttons_length = buttons_length_;
   pad->axes_length = axes_length_;
+  pad->axes_used = axes_used_;
 
   for (unsigned int i = 0; i < buttons_length_; i++) {
+    pad->buttons[i].used = button_indices_used_[i];
     pad->buttons[i].pressed = buttons_[i];
     pad->buttons[i].value = buttons_[i] ? 1.0 : 0.0;
   }
@@ -204,13 +225,15 @@ void RawInputGamepadDeviceWin::ReadPadState(Gamepad* pad) const {
 }
 
 bool RawInputGamepadDeviceWin::SupportsVibration() const {
-  return dualshock4_ != nullptr;
+  return dualshock4_ || hid_haptics_;
 }
 
 void RawInputGamepadDeviceWin::SetVibration(double strong_magnitude,
                                             double weak_magnitude) {
   if (dualshock4_)
     dualshock4_->SetVibration(strong_magnitude, weak_magnitude);
+  else if (hid_haptics_)
+    hid_haptics_->SetVibration(strong_magnitude, weak_magnitude);
 }
 
 bool RawInputGamepadDeviceWin::QueryDeviceInfo() {
@@ -223,18 +246,10 @@ bool RawInputGamepadDeviceWin::QueryDeviceInfo() {
   if (!IsGamepadUsageId(usage_))
     return false;
 
-  // This is terrible, but the Oculus Rift and some Blue Yeti microphones seem
-  // to think they are gamepads. Filter out any such devices. Oculus Touch is
-  // handled elsewhere.
-  if (vendor_id_ == kVendorOculus || vendor_id_ == kVendorBlue)
+  // Filter out devices that have gamepad-like HID usages but aren't gamepads.
+  if (GamepadIsExcluded(vendor_id_, product_id_))
     return false;
 
-  for (size_t i = 0; i < kFilteredDevicesLen; ++i) {
-    const auto& filter = kFilteredDevices[i];
-    if (vendor_id_ == filter.vendor && product_id_ == filter.product)
-      return false;
-  }
-
   // Fetch the device's |name_| (RIDI_DEVICENAME).
   if (!QueryDeviceName())
     return false;
@@ -249,8 +264,20 @@ bool RawInputGamepadDeviceWin::QueryDeviceInfo() {
   if (!name_.compare(0, pci_prefix.size(), pci_prefix))
     return false;
 
+  // Filter out the virtual digitizer, which was observed after remote desktop.
+  // See https://crbug.com/961774 for details.
+  if (name_ == L"\\\\?\\VIRTUAL_DIGITIZER")
+    return false;
+
+  // We can now use the name to query the OS for a file handle that is used to
+  // read the product string from the device. If the OS does not return a valid
+  // handle this gamepad is invalid.
+  auto hid_handle = OpenHidHandle();
+  if (!hid_handle.IsValid())
+    return false;
+
   // Fetch the human-friendly |product_string_|, if available.
-  if (!QueryProductString())
+  if (!QueryProductString(hid_handle))
     product_string_ = L"Unknown Gamepad";
 
   // Fetch information about the buttons and axes on this device. This sets
@@ -322,19 +349,21 @@ bool RawInputGamepadDeviceWin::QueryDeviceName() {
   return true;
 }
 
-bool RawInputGamepadDeviceWin::QueryProductString() {
-  DCHECK(hid_functions_);
-  DCHECK(hid_functions_->IsValid());
-  base::win::ScopedHandle hid_handle(::CreateFile(
-      name_.c_str(), GENERIC_READ | GENERIC_WRITE,
-      FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, NULL, NULL));
-  if (!hid_handle.IsValid())
-    return false;
+bool RawInputGamepadDeviceWin::QueryProductString(
+    base::win::ScopedHandle& hid_handle) {
+  DCHECK(hid_handle.IsValid());
   product_string_.resize(Gamepad::kIdLengthCap);
   return hid_functions_->HidDGetProductString()(
       hid_handle.Get(), &product_string_.front(), Gamepad::kIdLengthCap);
 }
 
+base::win::ScopedHandle RawInputGamepadDeviceWin::OpenHidHandle() {
+  return base::win::ScopedHandle(::CreateFile(
+      name_.c_str(), GENERIC_READ | GENERIC_WRITE,
+      FILE_SHARE_READ | FILE_SHARE_WRITE, /*lpSecurityAttributes=*/nullptr,
+      OPEN_EXISTING, /*dwFlagsAndAttributes=*/0, /*hTemplateFile=*/nullptr));
+}
+
 bool RawInputGamepadDeviceWin::QueryDeviceCapabilities() {
   DCHECK(hid_functions_);
   DCHECK(hid_functions_->IsValid());
@@ -378,25 +407,18 @@ void RawInputGamepadDeviceWin::QueryButtonCapabilities(uint16_t button_count) {
         HidP_Input, button_caps.get(), &button_count, preparsed_data_);
     DCHECK_EQ(HIDP_STATUS_SUCCESS, status);
 
-    // Keep track of which button indices are in use.
-    std::vector<bool> button_indices_used(Gamepad::kButtonsLengthCap, false);
-
     // Collect all inputs from the Button usage page.
-    QueryNormalButtonCapabilities(button_caps.get(), button_count,
-                                  &button_indices_used);
+    QueryNormalButtonCapabilities(button_caps.get(), button_count);
 
     // Check for common gamepad buttons that are not on the Button usage page.
-    QuerySpecialButtonCapabilities(button_caps.get(), button_count,
-                                   &button_indices_used);
+    QuerySpecialButtonCapabilities(button_caps.get(), button_count);
   }
 }
 
 void RawInputGamepadDeviceWin::QueryNormalButtonCapabilities(
     HIDP_BUTTON_CAPS button_caps[],
-    uint16_t button_count,
-    std::vector<bool>* button_indices_used) {
+    uint16_t button_count) {
   DCHECK(button_caps);
-  DCHECK(button_indices_used);
 
   // Collect all inputs from the Button usage page and assign button indices
   // based on the usage value.
@@ -414,17 +436,15 @@ void RawInputGamepadDeviceWin::QueryNormalButtonCapabilities(
           std::min(Gamepad::kButtonsLengthCap - 1, button_index_max);
       buttons_length_ = std::max(buttons_length_, button_index_max + 1);
       for (size_t j = button_index_min; j <= button_index_max; ++j)
-        (*button_indices_used)[j] = true;
+        button_indices_used_[j] = true;
     }
   }
 }
 
 void RawInputGamepadDeviceWin::QuerySpecialButtonCapabilities(
     HIDP_BUTTON_CAPS button_caps[],
-    uint16_t button_count,
-    std::vector<bool>* button_indices_used) {
+    uint16_t button_count) {
   DCHECK(button_caps);
-  DCHECK(button_indices_used);
 
   // Check for common gamepad buttons that are not on the Button usage page.
   std::vector<bool> has_special_usage(kSpecialUsagesLen, false);
@@ -456,19 +476,20 @@ void RawInputGamepadDeviceWin::QuerySpecialButtonCapabilities(
 
       // Advance to the next unused button index.
       while (button_index < Gamepad::kButtonsLengthCap &&
-             (*button_indices_used)[button_index]) {
+             button_indices_used_[button_index]) {
         ++button_index;
       }
       if (button_index >= Gamepad::kButtonsLengthCap)
         break;
 
       special_button_map_[special_index] = button_index;
-      (*button_indices_used)[button_index] = true;
+      button_indices_used_[button_index] = true;
       ++button_index;
 
       if (--unmapped_button_count == 0)
         break;
     }
+    buttons_length_ = std::max(buttons_length_, button_index);
   }
 }
 
@@ -489,6 +510,7 @@ void RawInputGamepadDeviceWin::QueryAxisCapabilities(uint16_t axis_count) {
       axes_[axis_index].active = true;
       axes_[axis_index].bitmask = GetBitmask(axes_caps[i].BitSize);
       axes_length_ = std::max(axes_length_, axis_index + 1);
+      axes_used_ |= 1 << axis_index;
     } else {
       mapped_all_axes = false;
     }
@@ -511,6 +533,7 @@ void RawInputGamepadDeviceWin::QueryAxisCapabilities(uint16_t axis_count) {
           axes_[next_index].active = true;
           axes_[next_index].bitmask = GetBitmask(axes_caps[i].BitSize);
           axes_length_ = std::max(axes_length_, next_index + 1);
+          axes_used_ |= 1 << next_index;
         }
       }
 
@@ -520,4 +543,8 @@ void RawInputGamepadDeviceWin::QueryAxisCapabilities(uint16_t axis_count) {
   }
 }
 
+base::WeakPtr<AbstractHapticGamepad> RawInputGamepadDeviceWin::GetWeakPtr() {
+  return weak_factory_.GetWeakPtr();
+}
+
 }  // namespace device
