@@ -6,14 +6,19 @@
 #include <utility>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/callback.h"
+#include "base/callback_helpers.h"
 #include "base/run_loop.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "device/vr/orientation/orientation_device.h"
+#include "device/vr/orientation/orientation_session.h"
 #include "device/vr/test/fake_orientation_provider.h"
 #include "device/vr/test/fake_sensor_provider.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "services/device/public/cpp/generic_sensor/sensor_reading.h"
 #include "services/device/public/cpp/generic_sensor/sensor_reading_shared_buffer_reader.h"
 #include "services/device/public/cpp/generic_sensor/sensor_traits.h"
@@ -35,14 +40,19 @@ class FakeScreen : public display::Screen {
  public:
   FakeScreen() = default;
   ~FakeScreen() override = default;
-  display::Display GetPrimaryDisplay() const override { return display; };
+  display::Display GetPrimaryDisplay() const override { return display; }
 
   // Unused functions
-  gfx::Point GetCursorScreenPoint() override { return gfx::Point(); };
-  bool IsWindowUnderCursor(gfx::NativeWindow window) override { return false; };
+  gfx::Point GetCursorScreenPoint() override { return gfx::Point(); }
+  bool IsWindowUnderCursor(gfx::NativeWindow window) override { return false; }
   gfx::NativeWindow GetWindowAtScreenPoint(const gfx::Point& point) override {
     return nullptr;
   }
+  gfx::NativeWindow GetLocalProcessWindowAtPoint(
+      const gfx::Point& point,
+      const std::set<gfx::NativeWindow>& ignore) override {
+    return nullptr;
+  }
   display::Display GetDisplayNearestWindow(
       gfx::NativeWindow window) const override {
     return display;
@@ -51,7 +61,7 @@ class FakeScreen : public display::Screen {
       const gfx::Point& point) const override {
     return display;
   }
-  int GetNumDisplays() const override { return 0; };
+  int GetNumDisplays() const override { return 0; }
   display::Display GetDisplayMatching(
       const gfx::Rect& match_rect) const override {
     return display;
@@ -76,11 +86,11 @@ class VROrientationDeviceTest : public testing::Test {
   VROrientationDeviceTest() = default;
   ~VROrientationDeviceTest() override = default;
   void SetUp() override {
-    fake_sensor_provider_ = std::make_unique<FakeSensorProvider>(
-        mojo::MakeRequest(&sensor_provider_ptr_));
+    fake_sensor_provider_ = std::make_unique<FakeXRSensorProvider>(
+        sensor_provider_.BindNewPipeAndPassReceiver());
 
     fake_sensor_ = std::make_unique<FakeOrientationSensor>(
-        mojo::MakeRequest(&sensor_ptr_));
+        sensor_.InitWithNewPipeAndPassReceiver());
 
     shared_buffer_handle_ = mojo::SharedBufferHandle::Create(
         sizeof(SensorReadingSharedBuffer) *
@@ -94,7 +104,7 @@ class VROrientationDeviceTest : public testing::Test {
     scoped_screen_override_ =
         std::make_unique<ScopedScreenOverride>(fake_screen_.get());
 
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
   }
 
   void TearDown() override { shared_buffer_handle_.reset(); }
@@ -106,20 +116,15 @@ class VROrientationDeviceTest : public testing::Test {
   void InitializeDevice(mojom::SensorInitParamsPtr params) {
     base::RunLoop loop;
 
-    device_ = std::make_unique<VROrientationDevice>(
-        &sensor_provider_ptr_, base::BindOnce(
-                                   [](base::OnceClosure quit_closure) {
-                                     // The callback was called.
-                                     std::move(quit_closure).Run();
-                                   },
-                                   loop.QuitClosure()));
+    device_ = std::make_unique<VROrientationDevice>(sensor_provider_.get(),
+                                                    loop.QuitClosure());
 
     // Complete the creation of device_ by letting the GetSensor function go
     // through.
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
 
     fake_sensor_provider_->CallCallback(std::move(params));
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
 
     // Ensure that the callback is called.
     loop.Run();
@@ -134,7 +139,7 @@ class VROrientationDeviceTest : public testing::Test {
 
     base::RunLoop loop;
 
-    device_->OnMagicWindowFrameDataRequest(base::BindOnce(
+    device_->GetInlineFrameData(base::BindOnce(
         [](base::OnceClosure quit_closure,
            base::OnceCallback<void(mojom::VRPosePtr)> callback,
            mojom::XRFrameDataPtr ptr) {
@@ -143,19 +148,52 @@ class VROrientationDeviceTest : public testing::Test {
         },
         loop.QuitClosure(), std::move(callback)));
 
-    scoped_task_environment_.RunUntilIdle();
+    task_environment_.RunUntilIdle();
 
     // Ensure the pose request callback runs.
     loop.Run();
   }
 
+  void AssertInlineFrameDataAvailable(bool expect_available) {
+    if (expect_available) {
+      device_->GetInlineFrameData(base::BindOnce(
+          [](device::mojom::XRFrameDataPtr data) { EXPECT_TRUE(data); }));
+    } else {
+      device_->GetInlineFrameData(base::BindOnce(
+          [](device::mojom::XRFrameDataPtr data) { EXPECT_FALSE(data); }));
+    }
+  }
+
+  std::unique_ptr<VROrientationSession> MakeDisplay() {
+    mojo::PendingRemote<mojom::XRFrameDataProvider> data_provider;
+    mojo::PendingRemote<mojom::XRSessionController> controller;
+    return std::make_unique<VROrientationSession>(
+        device_.get(), data_provider.InitWithNewPipeAndPassReceiver(),
+        controller.InitWithNewPipeAndPassReceiver());
+  }
+
+  void TryGetFrameData(VROrientationSession* display, bool expect_null) {
+    bool was_called = false;
+    auto callback = [](bool expect_null, bool* was_called,
+                       mojom::XRFrameDataPtr data) {
+      *was_called = true;
+      EXPECT_EQ(expect_null, !data);
+    };
+
+    static_cast<mojom::XRFrameDataProvider*>(display)->GetFrameData(
+        nullptr, base::BindOnce(callback, expect_null, &was_called));
+
+    base::RunLoop().RunUntilIdle();
+    EXPECT_TRUE(was_called);
+  }
+
   mojom::SensorInitParamsPtr FakeInitParams() {
     auto init_params = mojom::SensorInitParams::New();
-    init_params->sensor = std::move(sensor_ptr_);
+    init_params->sensor = std::move(sensor_);
     init_params->default_configuration = PlatformSensorConfiguration(
         SensorTraits<kOrientationSensorType>::kDefaultFrequency);
 
-    init_params->client_request = mojo::MakeRequest(&sensor_client_ptr_);
+    init_params->client_receiver = sensor_client_.BindNewPipeAndPassReceiver();
 
     init_params->memory = shared_buffer_handle_->Clone(
         mojo::SharedBufferHandle::AccessMode::READ_ONLY);
@@ -186,18 +224,18 @@ class VROrientationDeviceTest : public testing::Test {
   }
 
   // Needed for MakeRequest to work.
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
 
   std::unique_ptr<VROrientationDevice> device_;
-  std::unique_ptr<FakeSensorProvider> fake_sensor_provider_;
-  mojom::SensorProviderPtr sensor_provider_ptr_;
+  std::unique_ptr<FakeXRSensorProvider> fake_sensor_provider_;
+  mojo::Remote<mojom::SensorProvider> sensor_provider_;
 
   // Fake Sensor Init params objects
   std::unique_ptr<FakeOrientationSensor> fake_sensor_;
-  mojom::SensorPtrInfo sensor_ptr_;
+  mojo::PendingRemote<mojom::Sensor> sensor_;
   mojo::ScopedSharedBufferHandle shared_buffer_handle_;
   mojo::ScopedSharedBufferMapping shared_buffer_mapping_;
-  mojom::SensorClientPtr sensor_client_ptr_;
+  mojo::Remote<mojom::SensorClient> sensor_client_;
 
   std::unique_ptr<FakeScreen> fake_screen_;
   std::unique_ptr<ScopedScreenOverride> scoped_screen_override_;
@@ -209,9 +247,9 @@ TEST_F(VROrientationDeviceTest, InitializationTest) {
   // Check that without running anything, the device will return not available,
   // without crashing.
 
-  device_ = std::make_unique<VROrientationDevice>(&sensor_provider_ptr_,
-                                                  base::BindOnce([]() {}));
-  scoped_task_environment_.RunUntilIdle();
+  device_ = std::make_unique<VROrientationDevice>(sensor_provider_.get(),
+                                                  base::DoNothing());
+  task_environment_.RunUntilIdle();
 
   EXPECT_FALSE(device_->IsAvailable());
 }
@@ -233,7 +271,7 @@ TEST_F(VROrientationDeviceTest, SensorIsAvailableTest) {
 }
 
 TEST_F(VROrientationDeviceTest, GetOrientationTest) {
-  // Tests that OnMagicWindowFrameDataRequest returns a pose ptr without mishap.
+  // Tests that OnGetInlineFrameData returns a pose ptr without mishap.
 
   InitializeDevice(FakeInitParams());
 
@@ -248,20 +286,20 @@ TEST_F(VROrientationDeviceTest, OrientationDefaultForwardTest) {
   // Set forward to 0 degrees
   DeviceReadPose(gfx::Quaternion(0, 0, 0, 1),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), -0.707, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 0.707, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), -0.707, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 0.707, 0.001);
                  }));
 
   // Now a 90 degree rotation around x in device space should be default pose in
   // vr space.
   DeviceReadPose(gfx::Quaternion(0.707, 0, 0, 0.707),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 1, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 1, 0.001);
                  }));
 }
 
@@ -273,19 +311,19 @@ TEST_F(VROrientationDeviceTest, OrientationSetForwardTest) {
   // to be the default pose.
   DeviceReadPose(gfx::Quaternion(0.653, 0.271, 0.271, 0.653),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 1, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 1, 0.001);
                  }));
 
   // Now hold upright and straigt produces a 45 degree rotation to the right
   DeviceReadPose(gfx::Quaternion(0.707, 0, 0, 0.707),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), -0.383, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 0.924, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), -0.383, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 0.924, 0.001);
                  }));
 }
 
@@ -298,20 +336,20 @@ TEST_F(VROrientationDeviceTest, OrientationLandscape90Test) {
   // landscape mode.
   DeviceReadPose(gfx::Quaternion(0.5, -0.5, 0.5, 0.5),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 1, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 1, 0.001);
                  }));
 
   // Rotating the device 45 left from base pose should cause 45 degree left
   // rotation around y in VR space.
   DeviceReadPose(gfx::Quaternion(0.653, -0.271, 0.653, 0.271),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0.382, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 0.924, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0.382, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 0.924, 0.001);
                  }));
 }
 
@@ -324,21 +362,35 @@ TEST_F(VROrientationDeviceTest, OrientationLandscape270Test) {
   // landscape mode (twist the other way from what we'd need for ROTATE_90).
   DeviceReadPose(gfx::Quaternion(0.5, 0.5, -0.5, 0.5),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 1, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 1, 0.001);
                  }));
 
   // Rotating the device 45 left from base pose should cause 45 degree left
   // rotation around y in VR space
   DeviceReadPose(gfx::Quaternion(0.271, 0.653, -0.271, 0.653),
                  base::BindOnce([](mojom::VRPosePtr ptr) {
-                   EXPECT_NEAR(ptr->orientation->at(0), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(1), 0.382, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(2), 0, 0.001);
-                   EXPECT_NEAR(ptr->orientation->at(3), 0.924, 0.001);
+                   EXPECT_NEAR(ptr->orientation->x(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->y(), 0.382, 0.001);
+                   EXPECT_NEAR(ptr->orientation->z(), 0, 0.001);
+                   EXPECT_NEAR(ptr->orientation->w(), 0.924, 0.001);
                  }));
 }
 
+TEST_F(VROrientationDeviceTest, GetFrameDataHelper) {
+  InitializeDevice(FakeInitParams());
+
+  // 1) create display impl with restricted frame data
+  // 2) call GetFrameData and check behavior
+  // 3) unrestrict frame data
+  // 4) call GetFrameData and check behavior
+  std::unique_ptr<VROrientationSession> display = MakeDisplay();
+  TryGetFrameData(display.get(), true);
+  static_cast<mojom::XRSessionController*>(display.get())
+      ->SetFrameDataRestricted(false);
+  TryGetFrameData(display.get(), false);
+}
+
 }  // namespace device
