@@ -4,14 +4,13 @@
 
 #include "device/vr/test/fake_orientation_provider.h"
 
-#include "services/device/public/interfaces/sensor.mojom.h"
+#include "services/device/public/mojom/sensor.mojom.h"
 
 namespace device {
 
-FakeOrientationSensor::FakeOrientationSensor(mojom::SensorRequest request)
-    : binding_(this) {
-  binding_.Bind(std::move(request));
-}
+FakeOrientationSensor::FakeOrientationSensor(
+    mojo::PendingReceiver<mojom::Sensor> receiver)
+    : receiver_(this, std::move(receiver)) {}
 
 FakeOrientationSensor::~FakeOrientationSensor() = default;
 
@@ -22,4 +21,4 @@ void FakeOrientationSensor::AddConfiguration(
   std::move(callback).Run(true);
 }
 
-}  // namespace device
\ No newline at end of file
+}  // namespace device
