@@ -10,19 +10,21 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/callback.h"
 #include "base/files/file_path.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
+#include "base/sequenced_task_runner.h"
 #include "base/threading/thread_checker.h"
 #include "google_apis/drive/drive_api_error_codes.h"
-#include "net/base/completion_once_callback.h"
-#include "net/url_request/url_fetcher.h"
-#include "net/url_request/url_fetcher_delegate.h"
-#include "net/url_request/url_fetcher_response_writer.h"
+#include "services/network/public/cpp/simple_url_loader.h"
+#include "services/network/public/cpp/simple_url_loader_stream_consumer.h"
+#include "services/network/public/mojom/url_response_head.mojom-forward.h"
 #include "url/gurl.h"
 
 namespace base {
@@ -46,20 +48,25 @@ struct ContentTypeAndData {
   std::string data;
 };
 
-typedef base::Callback<void(DriveApiErrorCode)> PrepareCallback;
+using PrepareCallback = base::OnceCallback<void(DriveApiErrorCode)>;
 
 // Callback used for requests that the server returns FileResource data
 // formatted into JSON value.
-typedef base::Callback<void(DriveApiErrorCode error,
-                            std::unique_ptr<FileResource> entry)>
+typedef base::OnceCallback<void(DriveApiErrorCode error,
+                                std::unique_ptr<FileResource> entry)>
     FileResourceCallback;
 
 // Callback used for DownloadFileRequest and ResumeUploadRequestBase.
-typedef base::Callback<void(int64_t progress, int64_t total)> ProgressCallback;
+// |first_chunk| indicates if |content| is from the very beginning of
+// the file being downloaded and helps consumers detect if download
+// was restarted, for example due to re-authentication.
+typedef base::RepeatingCallback<void(int64_t progress, int64_t total)>
+    ProgressCallback;
 
 // Callback used to get the content from DownloadFileRequest.
-typedef base::Callback<void(DriveApiErrorCode error,
-                            std::unique_ptr<std::string> content)>
+typedef base::RepeatingCallback<void(DriveApiErrorCode error,
+                                     std::unique_ptr<std::string> content,
+                                     bool first_chunk)>
     GetContentCallback;
 
 // Parses JSON passed in |json|. Returns NULL on failure.
@@ -82,8 +89,8 @@ void GenerateMultipartBody(MultipartType multipart_type,
 class AuthenticatedRequestInterface {
  public:
   // Called when re-authentication is required. See Start() for details.
-  typedef base::Callback<void(AuthenticatedRequestInterface* request)>
-      ReAuthenticateCallback;
+  using ReAuthenticateCallback =
+      base::RepeatingCallback<void(AuthenticatedRequestInterface* request)>;
 
   virtual ~AuthenticatedRequestInterface() {}
 
@@ -97,7 +104,7 @@ class AuthenticatedRequestInterface {
   // |callback| must not be null.
   virtual void Start(const std::string& access_token,
                      const std::string& custom_user_agent,
-                     const ReAuthenticateCallback& callback) = 0;
+                     ReAuthenticateCallback callback) = 0;
 
   // Invoked when the authentication failed with an error code |code|.
   virtual void OnAuthFailed(DriveApiErrorCode code) = 0;
@@ -114,70 +121,35 @@ class AuthenticatedRequestInterface {
   virtual void Cancel() = 0;
 };
 
-//=========================== ResponseWriter ==================================
-
-// Saves the response for the request to a file or string.
-class ResponseWriter : public net::URLFetcherResponseWriter {
- public:
-  // If file_path is not empty, the response will be saved with file_writer_,
-  // otherwise it will be saved to data_.
-  ResponseWriter(base::SequencedTaskRunner* file_task_runner,
-                 const base::FilePath& file_path,
-                 const GetContentCallback& get_content_callback);
-  ~ResponseWriter() override;
-
-  const std::string& data() const { return data_; }
-
-  // Disowns the output file.
-  void DisownFile();
-
-  // URLFetcherResponseWriter overrides:
-  int Initialize(net::CompletionOnceCallback callback) override;
-  int Write(net::IOBuffer* buffer,
-            int num_bytes,
-            net::CompletionOnceCallback callback) override;
-  int Finish(int net_error, net::CompletionOnceCallback callback) override;
-
- private:
-  void DidWrite(scoped_refptr<net::IOBuffer> buffer,
-                net::CompletionOnceCallback callback,
-                int result);
-
-  const GetContentCallback get_content_callback_;
-  std::string data_;
-  std::unique_ptr<net::URLFetcherFileWriter> file_writer_;
-  base::WeakPtrFactory<ResponseWriter> weak_ptr_factory_;
-
-  DISALLOW_COPY_AND_ASSIGN(ResponseWriter);
-};
-
 //============================ UrlFetchRequestBase ===========================
 
 // Base class for requests that are fetching URLs.
 class UrlFetchRequestBase : public AuthenticatedRequestInterface,
-                            public net::URLFetcherDelegate {
+                            public network::SimpleURLLoaderStreamConsumer {
  public:
   // AuthenticatedRequestInterface overrides.
   void Start(const std::string& access_token,
              const std::string& custom_user_agent,
-             const ReAuthenticateCallback& callback) override;
+             ReAuthenticateCallback callback) override;
   base::WeakPtr<AuthenticatedRequestInterface> GetWeakPtr() override;
   void Cancel() override;
 
  protected:
-  explicit UrlFetchRequestBase(RequestSender* sender);
+  UrlFetchRequestBase(RequestSender* sender,
+                      ProgressCallback upload_progress_callback,
+                      ProgressCallback download_progress_callback);
   ~UrlFetchRequestBase() override;
 
   // Does async initialization for the request. |Start| calls this method so you
   // don't need to call this before |Start|.
-  virtual void Prepare(const PrepareCallback& callback);
+  virtual void Prepare(PrepareCallback callback);
 
   // Gets URL for the request.
   virtual GURL GetURL() const = 0;
 
   // Returns the request type. A derived class should override this method
   // for a request type other than HTTP GET.
-  virtual net::URLFetcher::RequestType GetRequestType() const;
+  virtual std::string GetRequestType() const;
 
   // Returns the extra HTTP headers for the request. A derived class should
   // override this method to specify any extra headers needed for the request.
@@ -207,9 +179,17 @@ class UrlFetchRequestBase : public AuthenticatedRequestInterface,
   virtual void GetOutputFilePath(base::FilePath* local_file_path,
                                  GetContentCallback* get_content_callback);
 
-  // Invoked by OnURLFetchComplete when the request completes without an
-  // authentication error. Must be implemented by a derived class.
-  virtual void ProcessURLFetchResults(const net::URLFetcher* source) = 0;
+  // Invoked when the request completes without an authentication error.
+  // |response_head| may be nullptr if the resource load failed - call
+  // GetErrorCode() to determine cause of failure. |response_file| will contain
+  // the path to the requested output file. |response_body| will contain the
+  // requested resource. If requested to save the response to disk then
+  // |response_body| may be truncated and only contain the starting portion
+  // of the resource.
+  virtual void ProcessURLFetchResults(
+      const network::mojom::URLResponseHead* response_head,
+      base::FilePath response_file,
+      std::string response_body) = 0;
 
   // Invoked by this base class upon an authentication error or cancel by
   // a user request. Must be implemented by a derived class.
@@ -219,47 +199,93 @@ class UrlFetchRequestBase : public AuthenticatedRequestInterface,
   void OnProcessURLFetchResultsComplete();
 
   // Returns an appropriate DriveApiErrorCode based on the HTTP response code
-  // and the status of the URLFetcher.
-  DriveApiErrorCode GetErrorCode();
+  // and the status of the URLLoader.
+  DriveApiErrorCode GetErrorCode() const;
+
+  // Returns the net::Error representing the final status of the request.
+  // Only valid after the request completes (successful or not).
+  int NetError() const;
 
   // Returns true if called on the thread where the constructor was called.
   bool CalledOnValidThread();
 
-  // Returns the writer which is used to save the response for the request.
-  ResponseWriter* response_writer() const { return response_writer_; }
-
   // Returns the task runner that should be used for blocking tasks.
   base::SequencedTaskRunner* blocking_task_runner() const;
 
  private:
+  // Values for the current in-progress request being handled by the
+  // url_loader_.
+  struct DownloadData {
+    explicit DownloadData(
+        scoped_refptr<base::SequencedTaskRunner> blocking_task_runner);
+    ~DownloadData();
+    base::File output_file;
+    base::FilePath output_file_path;
+    GetContentCallback get_content_callback;
+    std::string response_body;
+
+   private:
+    scoped_refptr<base::SequencedTaskRunner> blocking_task_runner_;
+  };
+
+  // Write the data in |file_data| to disk and call |resume| once complete on
+  // a blocking sequence.
+  static bool WriteFileData(std::string file_data, DownloadData* download_data);
+
+  // Called by SimpleURLLoader to report download progress.
+  void OnDownloadProgress(ProgressCallback progress_callback, uint64_t current);
+
+  // Called by SimpleURLLoader to report upload progress.
+  void OnUploadProgress(ProgressCallback progress_callback,
+                        uint64_t position,
+                        uint64_t total);
+
+  // Called with the result of |WriteFileData|.
+  void OnWriteComplete(std::unique_ptr<DownloadData> download_data,
+                       base::OnceClosure resume,
+                       bool write_success);
+
+  // Called after completion (with output file closed).
+  void OnOutputFileClosed(bool success);
+
+  // SimpleURLLoaderStreamConsumer implementation:
+  void OnDataReceived(base::StringPiece string_piece,
+                      base::OnceClosure resume) override;
+  void OnComplete(bool success) override;
+  void OnRetry(base::OnceClosure start_retry) override;
+
   // Continues |Start| function after |Prepare|.
   void StartAfterPrepare(const std::string& access_token,
                          const std::string& custom_user_agent,
-                         const ReAuthenticateCallback& callback,
+                         ReAuthenticateCallback callback,
                          DriveApiErrorCode code);
 
+  // Called when the SimpleURLLoader first receives a response.
+  void OnResponseStarted(const GURL& final_url,
+                         const network::mojom::URLResponseHead& response_head);
+
   // Invokes callback with |code| and request to delete the request to
   // |sender_|.
   void CompleteRequestWithError(DriveApiErrorCode code);
 
-  // URLFetcherDelegate overrides.
-  void OnURLFetchComplete(const net::URLFetcher* source) override;
-
   // AuthenticatedRequestInterface overrides.
   void OnAuthFailed(DriveApiErrorCode code) override;
 
   ReAuthenticateCallback re_authenticate_callback_;
   int re_authenticate_count_;
-  std::unique_ptr<net::URLFetcher> url_fetcher_;
-  ResponseWriter* response_writer_;  // Owned by |url_fetcher_|.
+  std::unique_ptr<network::SimpleURLLoader> url_loader_;
   RequestSender* sender_;
-  DriveApiErrorCode error_code_;
+  base::Optional<DriveApiErrorCode> error_code_;
+  const ProgressCallback upload_progress_callback_;
+  const ProgressCallback download_progress_callback_;
+  std::unique_ptr<DownloadData> download_data_;
+  int64_t response_content_length_;
 
   base::ThreadChecker thread_checker_;
 
   // Note: This should remain the last member so it'll be destroyed and
   // invalidate its weak pointers before any other members are destroyed.
-  base::WeakPtrFactory<UrlFetchRequestBase> weak_ptr_factory_;
+  base::WeakPtrFactory<UrlFetchRequestBase> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(UrlFetchRequestBase);
 };
@@ -274,9 +300,9 @@ class BatchableDelegate {
 
   // See UrlFetchRequestBase.
   virtual GURL GetURL() const = 0;
-  virtual net::URLFetcher::RequestType GetRequestType() const = 0;
+  virtual std::string GetRequestType() const = 0;
   virtual std::vector<std::string> GetExtraRequestHeaders() const = 0;
-  virtual void Prepare(const PrepareCallback& callback) = 0;
+  virtual void Prepare(PrepareCallback callback) = 0;
   virtual bool GetContentData(std::string* upload_content_type,
                               std::string* upload_content) = 0;
 
@@ -287,7 +313,7 @@ class BatchableDelegate {
   // |callback|.
   virtual void NotifyResult(DriveApiErrorCode code,
                             const std::string& response_body,
-                            const base::Closure& callback) = 0;
+                            base::OnceClosure callback) = 0;
 
   // Notifies error. Unlike |NotifyResult|, it must report error
   // synchronously. The instance may be deleted just after calling
@@ -295,15 +321,13 @@ class BatchableDelegate {
   virtual void NotifyError(DriveApiErrorCode code) = 0;
 
   // Notifies progress.
-  virtual void NotifyUploadProgress(const net::URLFetcher* source,
-                                    int64_t current,
-                                    int64_t total) = 0;
+  virtual void NotifyUploadProgress(int64_t current, int64_t total) = 0;
 };
 
 //============================ EntryActionRequest ============================
 
 // Callback type for requests that return only error status, like: Delete/Move.
-typedef base::Callback<void(DriveApiErrorCode error)> EntryActionCallback;
+using EntryActionCallback = base::OnceCallback<void(DriveApiErrorCode error)>;
 
 // This class performs a simple action over a given entry (document/file).
 // It is meant to be used for requests that return no JSON blobs.
@@ -311,17 +335,19 @@ class EntryActionRequest : public UrlFetchRequestBase {
  public:
   // |callback| is called when the request is finished either by success or by
   // failure. It must not be null.
-  EntryActionRequest(RequestSender* sender,
-                     const EntryActionCallback& callback);
+  EntryActionRequest(RequestSender* sender, EntryActionCallback callback);
   ~EntryActionRequest() override;
 
  protected:
   // Overridden from UrlFetchRequestBase.
-  void ProcessURLFetchResults(const net::URLFetcher* source) override;
+  void ProcessURLFetchResults(
+      const network::mojom::URLResponseHead* response_head,
+      base::FilePath response_file,
+      std::string response_body) override;
   void RunCallbackOnPrematureFailure(DriveApiErrorCode code) override;
 
  private:
-  const EntryActionCallback callback_;
+  EntryActionCallback callback_;
 
   DISALLOW_COPY_AND_ASSIGN(EntryActionRequest);
 };
@@ -329,8 +355,9 @@ class EntryActionRequest : public UrlFetchRequestBase {
 //=========================== InitiateUploadRequestBase=======================
 
 // Callback type for DriveServiceInterface::InitiateUpload.
-typedef base::Callback<void(DriveApiErrorCode error,
-                            const GURL& upload_url)> InitiateUploadCallback;
+typedef base::OnceCallback<void(DriveApiErrorCode error,
+                                const GURL& upload_url)>
+    InitiateUploadCallback;
 
 // This class provides base implementation for performing the request for
 // initiating the upload of a file.
@@ -350,18 +377,21 @@ class InitiateUploadRequestBase : public UrlFetchRequestBase {
   // |content_type| and |content_length| should be the attributes of the
   // uploading file.
   InitiateUploadRequestBase(RequestSender* sender,
-                            const InitiateUploadCallback& callback,
+                            InitiateUploadCallback callback,
                             const std::string& content_type,
                             int64_t content_length);
   ~InitiateUploadRequestBase() override;
 
   // UrlFetchRequestBase overrides.
-  void ProcessURLFetchResults(const net::URLFetcher* source) override;
+  void ProcessURLFetchResults(
+      const network::mojom::URLResponseHead* response_head,
+      base::FilePath response_file,
+      std::string response_body) override;
   void RunCallbackOnPrematureFailure(DriveApiErrorCode code) override;
   std::vector<std::string> GetExtraRequestHeaders() const override;
 
  private:
-  const InitiateUploadCallback callback_;
+  InitiateUploadCallback callback_;
   const std::string content_type_;
   const int64_t content_length_;
 
@@ -395,13 +425,18 @@ struct UploadRangeResponse {
 class UploadRangeRequestBase : public UrlFetchRequestBase {
  protected:
   // |upload_url| is the URL of where to upload the file to.
-  UploadRangeRequestBase(RequestSender* sender, const GURL& upload_url);
+  UploadRangeRequestBase(RequestSender* sender,
+                         const GURL& upload_url,
+                         ProgressCallback upload_progress_callback);
   ~UploadRangeRequestBase() override;
 
   // UrlFetchRequestBase overrides.
   GURL GetURL() const override;
-  net::URLFetcher::RequestType GetRequestType() const override;
-  void ProcessURLFetchResults(const net::URLFetcher* source) override;
+  std::string GetRequestType() const override;
+  void ProcessURLFetchResults(
+      const network::mojom::URLResponseHead* response_head,
+      base::FilePath response_file,
+      std::string response_body) override;
   void RunCallbackOnPrematureFailure(DriveApiErrorCode code) override;
 
   // This method will be called when the request is done, regardless of
@@ -431,7 +466,7 @@ class UploadRangeRequestBase : public UrlFetchRequestBase {
 
   // Note: This should remain the last member so it'll be destroyed and
   // invalidate its weak pointers before any other members are destroyed.
-  base::WeakPtrFactory<UploadRangeRequestBase> weak_ptr_factory_;
+  base::WeakPtrFactory<UploadRangeRequestBase> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(UploadRangeRequestBase);
 };
@@ -462,7 +497,8 @@ class ResumeUploadRequestBase : public UploadRangeRequestBase {
                           int64_t end_position,
                           int64_t content_length,
                           const std::string& content_type,
-                          const base::FilePath& local_file_path);
+                          const base::FilePath& local_file_path,
+                          ProgressCallback progress_callback);
   ~ResumeUploadRequestBase() override;
 
   // UrlFetchRequestBase overrides.
@@ -532,31 +568,29 @@ class MultipartUploadRequestBase : public BatchableDelegate {
                              const std::string& content_type,
                              int64_t content_length,
                              const base::FilePath& local_file_path,
-                             const FileResourceCallback& callback,
-                             const ProgressCallback& progress_callback);
+                             FileResourceCallback callback,
+                             ProgressCallback progress_callback);
   ~MultipartUploadRequestBase() override;
 
   // BatchableDelegate.
   std::vector<std::string> GetExtraRequestHeaders() const override;
-  void Prepare(const PrepareCallback& callback) override;
+  void Prepare(PrepareCallback callback) override;
   bool GetContentData(std::string* upload_content_type,
                       std::string* upload_content) override;
   void NotifyResult(DriveApiErrorCode code,
                     const std::string& body,
-                    const base::Closure& callback) override;
+                    base::OnceClosure callback) override;
   void NotifyError(DriveApiErrorCode code) override;
-  void NotifyUploadProgress(const net::URLFetcher* source,
-                            int64_t current,
-                            int64_t total) override;
+  void NotifyUploadProgress(int64_t current, int64_t total) override;
   // Parses the response value and invokes |callback_| with |FileResource|.
   void OnDataParsed(DriveApiErrorCode code,
-                    const base::Closure& callback,
+                    base::OnceClosure callback,
                     std::unique_ptr<base::Value> value);
 
  private:
   // Continues to rest part of |Start| method after determining boundary string
   // of multipart/related.
-  void OnPrepareUploadContent(const PrepareCallback& callback,
+  void OnPrepareUploadContent(PrepareCallback callback,
                               std::string* upload_content_type,
                               std::string* upload_content_data,
                               bool result);
@@ -565,7 +599,7 @@ class MultipartUploadRequestBase : public BatchableDelegate {
   const std::string metadata_json_;
   const std::string content_type_;
   const base::FilePath local_path_;
-  const FileResourceCallback callback_;
+  FileResourceCallback callback_;
   const ProgressCallback progress_callback_;
 
   // Boundary of multipart body.
@@ -579,7 +613,7 @@ class MultipartUploadRequestBase : public BatchableDelegate {
 
   // Note: This should remain the last member so it'll be destroyed and
   // invalidate its weak pointers before any other members are destroyed.
-  base::WeakPtrFactory<MultipartUploadRequestBase> weak_ptr_factory_;
+  base::WeakPtrFactory<MultipartUploadRequestBase> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(MultipartUploadRequestBase);
 };
@@ -587,8 +621,8 @@ class MultipartUploadRequestBase : public BatchableDelegate {
 //============================ DownloadFileRequest ===========================
 
 // Callback type for receiving the completion of DownloadFileRequest.
-typedef base::Callback<void(DriveApiErrorCode error,
-                            const base::FilePath& temp_file)>
+typedef base::OnceCallback<void(DriveApiErrorCode error,
+                                const base::FilePath& temp_file)>
     DownloadActionCallback;
 
 // This is a base class for performing the request for downloading a file.
@@ -611,13 +645,12 @@ class DownloadFileRequestBase : public UrlFetchRequestBase {
   // output_file_path:
   //   Specifies the file path to save the downloaded file.
   //
-  DownloadFileRequestBase(
-      RequestSender* sender,
-      const DownloadActionCallback& download_action_callback,
-      const GetContentCallback& get_content_callback,
-      const ProgressCallback& progress_callback,
-      const GURL& download_url,
-      const base::FilePath& output_file_path);
+  DownloadFileRequestBase(RequestSender* sender,
+                          DownloadActionCallback download_action_callback,
+                          const GetContentCallback& get_content_callback,
+                          ProgressCallback progress_callback,
+                          const GURL& download_url,
+                          const base::FilePath& output_file_path);
   ~DownloadFileRequestBase() override;
 
  protected:
@@ -625,19 +658,15 @@ class DownloadFileRequestBase : public UrlFetchRequestBase {
   GURL GetURL() const override;
   void GetOutputFilePath(base::FilePath* local_file_path,
                          GetContentCallback* get_content_callback) override;
-  void ProcessURLFetchResults(const net::URLFetcher* source) override;
+  void ProcessURLFetchResults(
+      const network::mojom::URLResponseHead* response_head,
+      base::FilePath response_file,
+      std::string response_body) override;
   void RunCallbackOnPrematureFailure(DriveApiErrorCode code) override;
 
-  // net::URLFetcherDelegate overrides.
-  void OnURLFetchDownloadProgress(const net::URLFetcher* source,
-                                  int64_t current,
-                                  int64_t total,
-                                  int64_t current_network_bytes) override;
-
  private:
-  const DownloadActionCallback download_action_callback_;
+  DownloadActionCallback download_action_callback_;
   const GetContentCallback get_content_callback_;
-  const ProgressCallback progress_callback_;
   const GURL download_url_;
   const base::FilePath output_file_path_;
 
