@@ -4,11 +4,11 @@
 
 #include "google_apis/drive/drive_api_url_generator.h"
 
+#include "base/check_op.h"
 #include "base/command_line.h"
-#include "base/logging.h"
+#include "base/notreached.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
-#include "google_apis/drive/drive_switches.h"
 #include "google_apis/google_api_keys.h"
 #include "net/base/escape.h"
 #include "net/base/url_util.h"
@@ -21,7 +21,6 @@ namespace {
 // TODO(yamaguchi): Make a utility function to compose some of these URLs by a
 // version and a resource name.
 const char kDriveV2AboutUrl[] = "drive/v2/about";
-const char kDriveV2AppsUrl[] = "drive/v2/apps";
 const char kDriveV2ChangelistUrl[] = "drive/v2/changes";
 const char kDriveV2StartPageTokenUrl[] = "drive/v2/changes/startPageToken";
 const char kDriveV2FilesUrl[] = "drive/v2/files";
@@ -49,14 +48,6 @@ const char kCorporaDefault[] = "default";
 const char kCorporaTeamDrive[] = "teamDrive";
 const char kTeamDriveId[] = "teamDriveId";
 
-// apps.delete and file.authorize API is exposed through a special endpoint
-// v2internal that is accessible only by the official API key for Chrome.
-const char kDriveV2InternalAppsUrl[] = "drive/v2internal/apps";
-const char kDriveV2AppsDeleteUrlFormat[] = "drive/v2internal/apps/%s";
-const char kDriveV2FilesAuthorizeUrlFormat[] =
-    "drive/v2internal/files/%s/authorize?appId=%s";
-const char kDriveV2InternalFileUrlPrefix[] = "drive/v2internal/files/";
-
 GURL AddResumableUploadParam(const GURL& url) {
   return net::AppendOrReplaceQueryParameter(url, "uploadType", "resumable");
 }
@@ -80,13 +71,9 @@ const char* GetCorporaString(FilesListCorpora corpora) {
 
 }  // namespace
 
-DriveApiUrlGenerator::DriveApiUrlGenerator(
-    const GURL& base_url, const GURL& base_thumbnail_url,
-    TeamDrivesIntegrationStatus team_drives_integration)
-    : base_url_(base_url),
-      base_thumbnail_url_(base_thumbnail_url),
-      enable_team_drives_(
-          team_drives_integration == TEAM_DRIVES_INTEGRATION_ENABLED) {
+DriveApiUrlGenerator::DriveApiUrlGenerator(const GURL& base_url,
+                                           const GURL& base_thumbnail_url)
+    : base_url_(base_url), base_thumbnail_url_(base_thumbnail_url) {
   // Do nothing.
 }
 
@@ -107,29 +94,11 @@ GURL DriveApiUrlGenerator::GetAboutGetUrl() const {
   return base_url_.Resolve(kDriveV2AboutUrl);
 }
 
-GURL DriveApiUrlGenerator::GetAppsListUrl(bool use_internal_endpoint) const {
-  return base_url_.Resolve(use_internal_endpoint ?
-      kDriveV2InternalAppsUrl : kDriveV2AppsUrl);
-}
-
-GURL DriveApiUrlGenerator::GetAppsDeleteUrl(const std::string& app_id) const {
-  return base_url_.Resolve(base::StringPrintf(
-      kDriveV2AppsDeleteUrlFormat, net::EscapePath(app_id).c_str()));
-}
-
 GURL DriveApiUrlGenerator::GetFilesGetUrl(const std::string& file_id,
-                                          bool use_internal_endpoint,
                                           const GURL& embed_origin) const {
-  const char* url_prefix = nullptr;
-  if (use_internal_endpoint)
-    url_prefix = kDriveV2InternalFileUrlPrefix;
-  else
-    url_prefix = kDriveV2FileUrlPrefix;
-
-  GURL url = base_url_.Resolve(url_prefix + net::EscapePath(file_id));
-
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  GURL url =
+      base_url_.Resolve(kDriveV2FileUrlPrefix + net::EscapePath(file_id));
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
 
   if (!embed_origin.is_empty()) {
     // Construct a valid serialized embed origin from an url, according to
@@ -146,23 +115,11 @@ GURL DriveApiUrlGenerator::GetFilesGetUrl(const std::string& file_id,
   return url;
 }
 
-GURL DriveApiUrlGenerator::GetFilesAuthorizeUrl(
-    const std::string& file_id,
-    const std::string& app_id) const {
-  GURL url = base_url_.Resolve(base::StringPrintf(
-      kDriveV2FilesAuthorizeUrlFormat, net::EscapePath(file_id).c_str(),
-      net::EscapePath(app_id).c_str()));
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
-  return url;
-}
-
 GURL DriveApiUrlGenerator::GetFilesInsertUrl(
     const std::string& visibility) const {
   GURL url =  base_url_.Resolve(kDriveV2FilesUrl);
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   if (!visibility.empty())
     url = net::AppendOrReplaceQueryParameter(url, "visibility", visibility);
 
@@ -175,8 +132,7 @@ GURL DriveApiUrlGenerator::GetFilesPatchUrl(const std::string& file_id,
   GURL url =
       base_url_.Resolve(kDriveV2FileUrlPrefix + net::EscapePath(file_id));
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   // setModifiedDate is "false" by default.
   if (set_modified_date)
     url = net::AppendOrReplaceQueryParameter(url, "setModifiedDate", "true");
@@ -194,8 +150,7 @@ GURL DriveApiUrlGenerator::GetFilesCopyUrl(
   GURL url =  base_url_.Resolve(base::StringPrintf(
       kDriveV2FileCopyUrlFormat, net::EscapePath(file_id).c_str()));
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   if (!visibility.empty())
     url = net::AppendOrReplaceQueryParameter(url, "visibility", visibility);
 
@@ -208,20 +163,16 @@ GURL DriveApiUrlGenerator::GetFilesListUrl(int max_results,
                                            const std::string& team_drive_id,
                                            const std::string& q) const {
   GURL url = base_url_.Resolve(kDriveV2FilesUrl);
-  if (enable_team_drives_) {
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
-    url =
-        net::AppendOrReplaceQueryParameter(url, kIncludeTeamDriveItems, "true");
-    url = net::AppendOrReplaceQueryParameter(url, kCorpora,
-                                             GetCorporaString(corpora));
-    if (!team_drive_id.empty())
-      url =
-          net::AppendOrReplaceQueryParameter(url, kTeamDriveId, team_drive_id);
-  }
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kIncludeTeamDriveItems, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kCorpora,
+                                           GetCorporaString(corpora));
+  if (!team_drive_id.empty())
+    url = net::AppendOrReplaceQueryParameter(url, kTeamDriveId, team_drive_id);
   // maxResults is 100 by default.
   if (max_results != 100) {
-    url = net::AppendOrReplaceQueryParameter(
-        url, "maxResults", base::IntToString(max_results));
+    url = net::AppendOrReplaceQueryParameter(url, "maxResults",
+                                             base::NumberToString(max_results));
   }
 
   if (!page_token.empty())
@@ -236,16 +187,14 @@ GURL DriveApiUrlGenerator::GetFilesListUrl(int max_results,
 GURL DriveApiUrlGenerator::GetFilesDeleteUrl(const std::string& file_id) const {
   GURL url = base_url_.Resolve(base::StringPrintf(
       kDriveV2FileDeleteUrlFormat, net::EscapePath(file_id).c_str()));
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   return url;
 }
 
 GURL DriveApiUrlGenerator::GetFilesTrashUrl(const std::string& file_id) const {
   GURL url = base_url_.Resolve(base::StringPrintf(
       kDriveV2FileTrashUrlFormat, net::EscapePath(file_id).c_str()));
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   return url;
 }
 
@@ -258,14 +207,10 @@ GURL DriveApiUrlGenerator::GetChangesListUrl(
   DCHECK_GE(start_change_id, 0);
 
   GURL url = base_url_.Resolve(kDriveV2ChangelistUrl);
-  if (enable_team_drives_) {
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
-    url = net::AppendOrReplaceQueryParameter(url, kIncludeTeamDriveItems,
-                                             "true");
-    if (!team_drive_id.empty()) {
-      url =
-          net::AppendOrReplaceQueryParameter(url, kTeamDriveId, team_drive_id);
-    }
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kIncludeTeamDriveItems, "true");
+  if (!team_drive_id.empty()) {
+    url = net::AppendOrReplaceQueryParameter(url, kTeamDriveId, team_drive_id);
   }
   // includeDeleted is "true" by default.
   if (!include_deleted)
@@ -273,8 +218,8 @@ GURL DriveApiUrlGenerator::GetChangesListUrl(
 
   // maxResults is "100" by default.
   if (max_results != 100) {
-    url = net::AppendOrReplaceQueryParameter(
-        url, "maxResults", base::IntToString(max_results));
+    url = net::AppendOrReplaceQueryParameter(url, "maxResults",
+                                             base::NumberToString(max_results));
   }
 
   if (!page_token.empty())
@@ -282,7 +227,7 @@ GURL DriveApiUrlGenerator::GetChangesListUrl(
 
   if (start_change_id > 0)
     url = net::AppendOrReplaceQueryParameter(
-        url, "startChangeId", base::Int64ToString(start_change_id));
+        url, "startChangeId", base::NumberToString(start_change_id));
 
   return url;
 }
@@ -291,8 +236,7 @@ GURL DriveApiUrlGenerator::GetChildrenInsertUrl(
     const std::string& file_id) const {
   GURL url = base_url_.Resolve(base::StringPrintf(
       kDriveV2ChildrenUrlFormat, net::EscapePath(file_id).c_str()));
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   return url;
 }
 
@@ -309,8 +253,7 @@ GURL DriveApiUrlGenerator::GetInitiateUploadNewFileUrl(
   GURL url = AddResumableUploadParam(
       base_url_.Resolve(kDriveV2UploadNewFileUrl));
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   // setModifiedDate is "false" by default.
   if (set_modified_date)
     url = net::AppendOrReplaceQueryParameter(url, "setModifiedDate", "true");
@@ -326,8 +269,7 @@ GURL DriveApiUrlGenerator::GetInitiateUploadExistingFileUrl(
       net::EscapePath(resource_id));
   url = AddResumableUploadParam(url);
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   // setModifiedDate is "false" by default.
   if (set_modified_date)
     url = net::AppendOrReplaceQueryParameter(url, "setModifiedDate", "true");
@@ -340,8 +282,7 @@ GURL DriveApiUrlGenerator::GetMultipartUploadNewFileUrl(
   GURL url = AddMultipartUploadParam(
       base_url_.Resolve(kDriveV2UploadNewFileUrl));
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   // setModifiedDate is "false" by default.
   if (set_modified_date)
     url = net::AppendOrReplaceQueryParameter(url, "setModifiedDate", "true");
@@ -357,8 +298,7 @@ GURL DriveApiUrlGenerator::GetMultipartUploadExistingFileUrl(
       net::EscapePath(resource_id));
   url = AddMultipartUploadParam(url);
 
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   // setModifiedDate is "false" by default.
   if (set_modified_date)
     url = net::AppendOrReplaceQueryParameter(url, "setModifiedDate", "true");
@@ -370,8 +310,7 @@ GURL DriveApiUrlGenerator::GenerateDownloadFileUrl(
     const std::string& resource_id) const {
   GURL url = base_url_.Resolve(base::StringPrintf(
       kDriveV2DownloadUrlFormat, net::EscapePath(resource_id).c_str()));
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   return url;
 }
 
@@ -379,8 +318,7 @@ GURL DriveApiUrlGenerator::GetPermissionsInsertUrl(
     const std::string& resource_id) const {
   GURL url = base_url_.Resolve(base::StringPrintf(
       kDriveV2PermissionsUrlFormat, net::EscapePath(resource_id).c_str()));
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   return url;
 }
 
@@ -396,8 +334,7 @@ GURL DriveApiUrlGenerator::GetThumbnailUrl(const std::string& resource_id,
 
 GURL DriveApiUrlGenerator::GetBatchUploadUrl() const {
   GURL url = base_url_.Resolve(kDriveV2BatchUploadUrl);
-  if (enable_team_drives_)
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
   return url;
 }
 
@@ -409,7 +346,7 @@ GURL DriveApiUrlGenerator::GetTeamDriveListUrl(
   // maxResults is 10 by default.
   if (max_results != 10) {
     url = net::AppendOrReplaceQueryParameter(url, "maxResults",
-                                             base::IntToString(max_results));
+                                             base::NumberToString(max_results));
   }
   if (!page_token.empty())
     url = net::AppendOrReplaceQueryParameter(url, "pageToken", page_token);
@@ -421,12 +358,10 @@ GURL DriveApiUrlGenerator::GetStartPageTokenUrl(
     const std::string& team_drive) const {
   GURL url = base_url_.Resolve(kDriveV2StartPageTokenUrl);
 
-  if (enable_team_drives_) {
-    url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
+  url = net::AppendOrReplaceQueryParameter(url, kSupportsTeamDrives, "true");
 
-    if (!team_drive.empty())
-      url = net::AppendOrReplaceQueryParameter(url, kTeamDriveId, team_drive);
-  }
+  if (!team_drive.empty())
+    url = net::AppendOrReplaceQueryParameter(url, kTeamDriveId, team_drive);
 
   return url;
 }
