@@ -7,21 +7,24 @@
 #include <utility>
 #include <vector>
 
+#include "base/base64.h"
 #include "base/base_paths.h"
 #include "base/bind.h"
-#include "base/bind_helpers.h"
+#include "base/callback_helpers.h"
+#include "base/containers/contains.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/json/json_writer.h"
 #include "base/logging.h"
-#include "base/memory/linked_ptr.h"
 #include "base/path_service.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/values.h"
+#include "google_apis/gaia/gaia_auth_consumer.h"
 #include "google_apis/gaia/gaia_constants.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "net/base/url_util.h"
@@ -32,14 +35,16 @@
 #include "url/third_party/mozilla/url_parse.h"
 
 #define REGISTER_RESPONSE_HANDLER(url, method) \
-  request_handlers_.insert(std::make_pair( \
-        url.path(), base::Bind(&FakeGaia::method, base::Unretained(this))))
+  request_handlers_.insert(std::make_pair(     \
+      url.path(),                              \
+      base::BindRepeating(&FakeGaia::method, base::Unretained(this))))
 
 #define REGISTER_PATH_RESPONSE_HANDLER(path, method) \
-  request_handlers_.insert(std::make_pair( \
-        path, base::Bind(&FakeGaia::method, base::Unretained(this))))
+  request_handlers_.insert(std::make_pair(           \
+      path, base::BindRepeating(&FakeGaia::method, base::Unretained(this))))
 
-using namespace net::test_server;
+using net::test_server::BasicHttpResponse;
+using net::test_server::HttpRequest;
 
 namespace {
 
@@ -52,12 +57,13 @@ const char kTestSessionLSIDCookie[] = "fake-session-LSID-cookie";
 const char kTestOAuthLoginSID[] = "fake-oauth-SID-cookie";
 const char kTestOAuthLoginLSID[] = "fake-oauth-LSID-cookie";
 const char kTestOAuthLoginAuthCode[] = "fake-oauth-auth-code";
+// Add SameSite=None and Secure because these cookies are needed in a
+// cross-site context.
+const char kTestCookieAttributes[] =
+    "; Path=/; HttpOnly; SameSite=None; Secure";
 
 const char kDefaultGaiaId[] = "12345";
 
-const base::FilePath::CharType kServiceLogin[] =
-    FILE_PATH_LITERAL("google_apis/test/service_login.html");
-
 const base::FilePath::CharType kEmbeddedSetupChromeos[] =
     FILE_PATH_LITERAL("google_apis/test/embedded_setup_chromeos.html");
 
@@ -93,11 +99,59 @@ void SetCookies(BasicHttpResponse* http_response,
                 const std::string& sid_cookie,
                 const std::string& lsid_cookie) {
   http_response->AddCustomHeader(
-      "Set-Cookie",
-      base::StringPrintf("SID=%s; Path=/; HttpOnly", sid_cookie.c_str()));
+      "Set-Cookie", base::StringPrintf("SID=%s%s", sid_cookie.c_str(),
+                                       kTestCookieAttributes));
   http_response->AddCustomHeader(
-      "Set-Cookie",
-      base::StringPrintf("LSID=%s; Path=/; HttpOnly", lsid_cookie.c_str()));
+      "Set-Cookie", base::StringPrintf("LSID=%s%s", lsid_cookie.c_str(),
+                                       kTestCookieAttributes));
+}
+
+std::string FormatCookieForMultilogin(std::string name, std::string value) {
+  const char format[] = R"(
+    {
+      "name":"%s",
+      "value":"%s",
+      "domain":".google.fr",
+      "path":"/",
+      "isSecure":true,
+      "isHttpOnly":false,
+      "priority":"HIGH",
+      "maxAge":63070000
+    }
+  )";
+  return base::StringPrintf(format, name.c_str(), value.c_str());
+}
+
+std::string FormatSyncTrustedPublicKeys(
+    const std::vector<std::vector<uint8_t>>& public_keys) {
+  std::string result;
+  for (const std::vector<uint8_t>& public_key : public_keys) {
+    if (!result.empty()) {
+      base::StrAppend(&result, {","});
+    }
+    base::StrAppend(&result, {"\"", base::Base64Encode(public_key), "\""});
+  }
+  return result;
+}
+
+std::string FormatSyncTrustedVaultKeysHeader(
+    const FakeGaia::SyncTrustedVaultKeys& sync_trusted_vault_keys) {
+  // Single line used because this string populates HTTP headers. Similarly,
+  // base64 encoding is used to avoid line breaks and meanwhile adopt JSON
+  // format (which doesn't support binary blobs). This base64 encoding is undone
+  // embedded_setup_chromeos.html.
+  const char format[] =
+      "{"
+      "\"fakeEncryptionKeyMaterial\":\"%s\","
+      "\"fakeEncryptionKeyVersion\":%d,"
+      "\"fakeTrustedPublicKeys\":[%s]"
+      "}";
+  return base::StringPrintf(
+      format,
+      base::Base64Encode(sync_trusted_vault_keys.encryption_key).c_str(),
+      sync_trusted_vault_keys.encryption_key_version,
+      FormatSyncTrustedPublicKeys(sync_trusted_vault_keys.trusted_public_keys)
+          .c_str());
 }
 
 }  // namespace
@@ -133,12 +187,13 @@ void FakeGaia::MergeSessionParams::Update(const MergeSessionParams& update) {
   maybe_update_field(&MergeSessionParams::email);
 }
 
+FakeGaia::SyncTrustedVaultKeys::SyncTrustedVaultKeys() = default;
+
+FakeGaia::SyncTrustedVaultKeys::~SyncTrustedVaultKeys() = default;
+
 FakeGaia::FakeGaia() : issue_oauth_code_cookie_(false) {
   base::FilePath source_root_dir;
   base::PathService::Get(base::DIR_SOURCE_ROOT, &source_root_dir);
-  CHECK(base::ReadFileToString(
-      source_root_dir.Append(base::FilePath(kServiceLogin)),
-      &service_login_response_));
   CHECK(base::ReadFileToString(
       source_root_dir.Append(base::FilePath(kEmbeddedSetupChromeos)),
       &embedded_setup_chromeos_response_));
@@ -179,30 +234,44 @@ void FakeGaia::MapEmailToGaiaId(const std::string& email,
   email_to_gaia_id_map_[email] = gaia_id;
 }
 
-std::string FakeGaia::GetGaiaIdOfEmail(const std::string& email) const {
+void FakeGaia::SetSyncTrustedVaultKeys(
+    const std::string& email,
+    const SyncTrustedVaultKeys& sync_trusted_vault_keys) {
   DCHECK(!email.empty());
+  email_to_sync_trusted_vault_keys_map_[email] = sync_trusted_vault_keys;
+}
+
+std::string FakeGaia::GetGaiaIdOfEmail(const std::string& email) const {
   const auto it = email_to_gaia_id_map_.find(email);
   return it == email_to_gaia_id_map_.end() ? std::string(kDefaultGaiaId) :
       it->second;
 }
 
-void FakeGaia::AddGoogleAccountsSigninHeader(
-    net::test_server::BasicHttpResponse* http_response,
-    const std::string& email) const {
-  DCHECK(!email.empty());
+void FakeGaia::AddGoogleAccountsSigninHeader(BasicHttpResponse* http_response,
+                                             const std::string& email) const {
+  DCHECK(http_response);
   http_response->AddCustomHeader("google-accounts-signin",
       base::StringPrintf(
           "email=\"%s\", obfuscatedid=\"%s\", sessionindex=0",
           email.c_str(), GetGaiaIdOfEmail(email).c_str()));
 }
 
-void FakeGaia::SetOAuthCodeCookie(
-    net::test_server::BasicHttpResponse* http_response) const {
+void FakeGaia::SetOAuthCodeCookie(BasicHttpResponse* http_response) const {
+  DCHECK(http_response);
   http_response->AddCustomHeader(
-      "Set-Cookie",
-      base::StringPrintf(
-          "oauth_code=%s; Path=/o/GetOAuth2Token; Secure; HttpOnly;",
-          merge_session_params_.auth_code.c_str()));
+      "Set-Cookie", base::StringPrintf("oauth_code=%s%s",
+                                       merge_session_params_.auth_code.c_str(),
+                                       kTestCookieAttributes));
+}
+
+void FakeGaia::AddSyncTrustedKeysHeader(BasicHttpResponse* http_response,
+                                        const std::string& email) const {
+  DCHECK(http_response);
+  DCHECK(base::Contains(email_to_sync_trusted_vault_keys_map_, email));
+  http_response->AddCustomHeader(
+      "fake-sync-trusted-vault-keys",
+      FormatSyncTrustedVaultKeysHeader(
+          email_to_sync_trusted_vault_keys_map_.at(email)));
 }
 
 void FakeGaia::Initialize() {
@@ -211,17 +280,26 @@ void FakeGaia::Initialize() {
   REGISTER_RESPONSE_HANDLER(
       gaia_urls->merge_session_url(), HandleMergeSession);
 
-  // Handles /ServiceLogin GAIA call.
-  REGISTER_RESPONSE_HANDLER(
-      gaia_urls->service_login_url(), HandleServiceLogin);
+  // Handles /oauth/multilogin GAIA call.
+  REGISTER_RESPONSE_HANDLER(gaia_urls->oauth_multilogin_url(),
+                            HandleMultilogin);
 
-  // Handles /embedded/setup/chromeos GAIA call.
-  REGISTER_RESPONSE_HANDLER(gaia_urls->embedded_setup_chromeos_url(1),
-                            HandleEmbeddedSetupChromeos);
   // Handles /embedded/setup/v2/chromeos GAIA call.
   REGISTER_RESPONSE_HANDLER(gaia_urls->embedded_setup_chromeos_url(2),
                             HandleEmbeddedSetupChromeos);
 
+  // Handles /embedded/setup/kidsignup/chromeos GAIA call.
+  REGISTER_RESPONSE_HANDLER(gaia_urls->embedded_setup_chromeos_kid_signup_url(),
+                            HandleEmbeddedSetupChromeos);
+
+  // Handles /embedded/setup/kidsignin/chromeos GAIA call.
+  REGISTER_RESPONSE_HANDLER(gaia_urls->embedded_setup_chromeos_kid_signin_url(),
+                            HandleEmbeddedSetupChromeos);
+
+  // Handles /embedded/reauth/chromeos GAIA call.
+  REGISTER_RESPONSE_HANDLER(gaia_urls->embedded_reauth_chromeos_url(),
+                            HandleEmbeddedReauthChromeos);
+
   // Handles /OAuthLogin GAIA call.
   REGISTER_RESPONSE_HANDLER(
       gaia_urls->oauth1_login_url(), HandleOAuthLogin);
@@ -278,22 +356,51 @@ void FakeGaia::Initialize() {
   REGISTER_RESPONSE_HANDLER(
       gaia_urls->GetCheckConnectionInfoURLWithSource(std::string()),
       HandleGetCheckConnectionInfo);
+
+  // Handles ReAuth API token fetch call.
+  REGISTER_RESPONSE_HANDLER(gaia_urls->reauth_api_url(),
+                            HandleGetReAuthProofToken);
 }
 
-std::unique_ptr<HttpResponse> FakeGaia::HandleRequest(
+FakeGaia::RequestHandlerMap::iterator FakeGaia::FindHandlerByPathPrefix(
+    const std::string& request_path) {
+  return std::find_if(
+      request_handlers_.begin(), request_handlers_.end(),
+      [request_path](std::pair<std::string, HttpRequestHandlerCallback> entry) {
+        return base::StartsWith(request_path, entry.first,
+                                base::CompareCase::SENSITIVE);
+      });
+}
+
+std::unique_ptr<net::test_server::HttpResponse> FakeGaia::HandleRequest(
     const HttpRequest& request) {
   // The scheme and host of the URL is actually not important but required to
   // get a valid GURL in order to parse |request.relative_url|.
   GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
   std::string request_path = request_url.path();
-  std::unique_ptr<BasicHttpResponse> http_response(new BasicHttpResponse());
+  auto http_response = std::make_unique<BasicHttpResponse>();
+
+  ErrorResponseMap::const_iterator error_response =
+      error_responses_.find(request_path);
+  if (error_response != error_responses_.end() &&
+      error_response->second != net::HTTP_OK) {
+    http_response->set_code(error_response->second);
+    return std::move(http_response);
+  }
+
   RequestHandlerMap::iterator iter = request_handlers_.find(request_path);
+  if (iter == request_handlers_.end()) {
+    // If exact match yielded no handler, try to find one by prefix,
+    // which is required for gaia endpoints that use variable path
+    // components, like the ReAuth API.
+    iter = FindHandlerByPathPrefix(request_path);
+  }
   if (iter != request_handlers_.end()) {
     LOG(WARNING) << "Serving request " << request_path;
     iter->second.Run(request, http_response.get());
   } else {
     LOG(ERROR) << "Unhandled request " << request_path;
-    return std::unique_ptr<HttpResponse>();  // Request not understood.
+    return std::unique_ptr<net::test_server::HttpResponse>();
   }
 
   return std::move(http_response);
@@ -331,6 +438,11 @@ std::string FakeGaia::GetDeviceIdByRefreshToken(
                                                      : std::string();
 }
 
+void FakeGaia::SetErrorResponse(const GURL& gaia_url,
+                                net::HttpStatusCode http_status_code) {
+  error_responses_[gaia_url.path()] = http_status_code;
+}
+
 void FakeGaia::SetRefreshTokenToDeviceIdMap(
     const RefreshTokenToDeviceIdMap& refresh_token_to_device_id_map) {
   refresh_token_to_device_id_map_ = refresh_token_to_device_id_map;
@@ -375,12 +487,18 @@ void FakeGaia::HandleMergeSession(const HttpRequest& request,
   http_response->set_code(net::HTTP_OK);
 }
 
+void FakeGaia::FormatOkJSONResponse(const base::Value& value,
+                                    BasicHttpResponse* http_response) {
+  FormatJSONResponse(value, net::HTTP_OK, http_response);
+}
+
 void FakeGaia::FormatJSONResponse(const base::Value& value,
+                                  net::HttpStatusCode status,
                                   BasicHttpResponse* http_response) {
   std::string response_json;
   base::JSONWriter::Write(value, &response_json);
   http_response->set_content(response_json);
-  http_response->set_code(net::HTTP_OK);
+  http_response->set_code(status);
 }
 
 const FakeGaia::AccessTokenInfo* FakeGaia::FindAccessTokenInfo(
@@ -388,7 +506,7 @@ const FakeGaia::AccessTokenInfo* FakeGaia::FindAccessTokenInfo(
     const std::string& client_id,
     const std::string& scope_string) const {
   if (auth_token.empty() || client_id.empty())
-    return NULL;
+    return nullptr;
 
   std::vector<std::string> scope_list = base::SplitString(
       scope_string, " ", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
@@ -405,7 +523,7 @@ const FakeGaia::AccessTokenInfo* FakeGaia::FindAccessTokenInfo(
     }
   }
 
-  return NULL;
+  return nullptr;
 }
 
 const FakeGaia::AccessTokenInfo* FakeGaia::GetAccessTokenInfo(
@@ -418,14 +536,7 @@ const FakeGaia::AccessTokenInfo* FakeGaia::GetAccessTokenInfo(
       return &(entry->second);
   }
 
-  return NULL;
-}
-
-void FakeGaia::HandleServiceLogin(const HttpRequest& request,
-                                  BasicHttpResponse* http_response) {
-  http_response->set_code(net::HTTP_OK);
-  http_response->set_content(service_login_response_);
-  http_response->set_content_type("text/html");
+  return nullptr;
 }
 
 void FakeGaia::HandleEmbeddedSetupChromeos(const HttpRequest& request,
@@ -443,7 +554,34 @@ void FakeGaia::HandleEmbeddedSetupChromeos(const HttpRequest& request,
   GetQueryParameter(request_url.query(), "Email", &prefilled_email_);
 
   http_response->set_code(net::HTTP_OK);
-  http_response->set_content(embedded_setup_chromeos_response_);
+  http_response->set_content(GetEmbeddedSetupChromeosResponseContent());
+  http_response->set_content_type("text/html");
+}
+
+void FakeGaia::HandleEmbeddedReauthChromeos(const HttpRequest& request,
+                                            BasicHttpResponse* http_response) {
+  GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
+
+  std::string client_id;
+  if (!GetQueryParameter(request_url.query(), "client_id", &client_id) ||
+      GaiaUrls::GetInstance()->oauth2_chrome_client_id() != client_id) {
+    LOG(ERROR) << "Missing or invalid param 'client_id' in "
+                  "/embedded/reauth/chromeos call";
+    return;
+  }
+
+  if (!GetQueryParameter(request_url.query(), "is_supervised",
+                         &is_supervised_)) {
+    LOG(ERROR) << "Missing param 'is_supervised' in "
+                  "/embedded/reauth/chromeos call";
+    return;
+  }
+
+  GetQueryParameter(request_url.query(), "is_device_owner", &is_device_owner_);
+  GetQueryParameter(request_url.query(), "Email", &prefilled_email_);
+
+  http_response->set_code(net::HTTP_OK);
+  http_response->set_content(GetEmbeddedSetupChromeosResponseContent());
   http_response->set_content_type("text/html");
 }
 
@@ -526,8 +664,8 @@ void FakeGaia::HandleServiceLoginAuth(const HttpRequest& request,
 }
 
 void FakeGaia::HandleEmbeddedLookupAccountLookup(
-    const net::test_server::HttpRequest& request,
-    net::test_server::BasicHttpResponse* http_response) {
+    const HttpRequest& request,
+    BasicHttpResponse* http_response) {
   std::string email;
   const bool is_saml =
       GetQueryParameter(request.content, "identifier", &email) &&
@@ -564,6 +702,10 @@ void FakeGaia::HandleEmbeddedSigninChallenge(const HttpRequest& request,
 
   if (issue_oauth_code_cookie_)
     SetOAuthCodeCookie(http_response);
+
+  if (base::Contains(email_to_sync_trusted_vault_keys_map_, email)) {
+    AddSyncTrustedKeysHeader(http_response, email);
+  }
 }
 
 void FakeGaia::HandleSSO(const HttpRequest& request,
@@ -581,8 +723,7 @@ void FakeGaia::HandleSSO(const HttpRequest& request,
   http_response->AddCustomHeader("Location", redirect_url);
   http_response->AddCustomHeader("Google-Accounts-SAML", "End");
 
-  if (!merge_session_params_.email.empty())
-    AddGoogleAccountsSigninHeader(http_response, merge_session_params_.email);
+  AddGoogleAccountsSigninHeader(http_response, merge_session_params_.email);
 
   if (issue_oauth_code_cookie_)
     SetOAuthCodeCookie(http_response);
@@ -638,7 +779,7 @@ void FakeGaia::HandleAuthToken(const HttpRequest& request,
     if (!merge_session_params_.id_token.empty())
       response_dict.SetString("id_token", merge_session_params_.id_token);
     response_dict.SetInteger("expires_in", 3600);
-    FormatJSONResponse(response_dict, http_response);
+    FormatOkJSONResponse(response_dict, http_response);
     return;
   }
 
@@ -656,7 +797,7 @@ void FakeGaia::HandleAuthToken(const HttpRequest& request,
       response_dict.SetString("access_token", token_info->token);
       response_dict.SetInteger("expires_in", 3600);
       response_dict.SetString("id_token", token_info->id_token);
-      FormatJSONResponse(response_dict, http_response);
+      FormatOkJSONResponse(response_dict, http_response);
       return;
     }
   }
@@ -670,7 +811,7 @@ void FakeGaia::HandleAuthToken(const HttpRequest& request,
 
 void FakeGaia::HandleTokenInfo(const HttpRequest& request,
                                BasicHttpResponse* http_response) {
-  const AccessTokenInfo* token_info = NULL;
+  const AccessTokenInfo* token_info = nullptr;
   std::string access_token;
   if (GetQueryParameter(request.content, "access_token", &access_token))
     token_info = GetAccessTokenInfo(access_token);
@@ -686,7 +827,7 @@ void FakeGaia::HandleTokenInfo(const HttpRequest& request,
     response_dict.SetInteger("expires_in", token_info->expires_in);
     response_dict.SetString("email", token_info->email);
     response_dict.SetString("id_token", token_info->id_token);
-    FormatJSONResponse(response_dict, http_response);
+    FormatOkJSONResponse(response_dict, http_response);
   } else {
     http_response->set_code(net::HTTP_BAD_REQUEST);
   }
@@ -706,10 +847,11 @@ void FakeGaia::HandleIssueToken(const HttpRequest& request,
       base::DictionaryValue response_dict;
       response_dict.SetString("issueAdvice", "auto");
       response_dict.SetString("expiresIn",
-                              base::IntToString(token_info->expires_in));
+                              base::NumberToString(token_info->expires_in));
       response_dict.SetString("token", token_info->token);
+      response_dict.SetString("grantedScopes", scope);
       response_dict.SetString("id_token", token_info->id_token);
-      FormatJSONResponse(response_dict, http_response);
+      FormatOkJSONResponse(response_dict, http_response);
       return;
     }
   }
@@ -732,10 +874,9 @@ void FakeGaia::HandleGetUserInfo(const HttpRequest& request,
   http_response->set_code(net::HTTP_OK);
 }
 
-void FakeGaia::HandleOAuthUserInfo(
-    const net::test_server::HttpRequest& request,
-    net::test_server::BasicHttpResponse* http_response) {
-  const AccessTokenInfo* token_info = NULL;
+void FakeGaia::HandleOAuthUserInfo(const HttpRequest& request,
+                                   BasicHttpResponse* http_response) {
+  const AccessTokenInfo* token_info = nullptr;
   std::string access_token;
   if (GetAccessToken(request, kAuthHeaderBearer, &access_token) ||
       GetAccessToken(request, kAuthHeaderOAuth, &access_token)) {
@@ -748,15 +889,14 @@ void FakeGaia::HandleOAuthUserInfo(
     response_dict.SetString("email", token_info->email);
     response_dict.SetString("verified_email", token_info->email);
     response_dict.SetString("id_token", token_info->id_token);
-    FormatJSONResponse(response_dict, http_response);
+    FormatOkJSONResponse(response_dict, http_response);
   } else {
     http_response->set_code(net::HTTP_BAD_REQUEST);
   }
 }
 
-void FakeGaia::HandleSAMLRedirect(
-    const net::test_server::HttpRequest& request,
-    net::test_server::BasicHttpResponse* http_response) {
+void FakeGaia::HandleSAMLRedirect(const HttpRequest& request,
+                                  BasicHttpResponse* http_response) {
   GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
   std::string domain;
   GetQueryParameter(request_url.query(), "domain", &domain);
@@ -781,9 +921,101 @@ void FakeGaia::HandleSAMLRedirect(
   http_response->AddCustomHeader("Location", redirect_url);
 }
 
-void FakeGaia::HandleGetCheckConnectionInfo(
-    const net::test_server::HttpRequest& request,
-    net::test_server::BasicHttpResponse* http_response) {
+void FakeGaia::HandleGetCheckConnectionInfo(const HttpRequest& request,
+                                            BasicHttpResponse* http_response) {
   base::ListValue connection_list;
-  FormatJSONResponse(connection_list, http_response);
+  FormatOkJSONResponse(connection_list, http_response);
+}
+
+void FakeGaia::HandleGetReAuthProofToken(const HttpRequest& request,
+                                         BasicHttpResponse* http_response) {
+  base::DictionaryValue response_dict;
+  std::unique_ptr<base::DictionaryValue> error =
+      std::make_unique<base::DictionaryValue>();
+
+  switch (next_reauth_status_) {
+    case GaiaAuthConsumer::ReAuthProofTokenStatus::kSuccess:
+      response_dict.SetString("encodedRapt", "abc123");
+      FormatOkJSONResponse(response_dict, http_response);
+      break;
+
+    case GaiaAuthConsumer::ReAuthProofTokenStatus::kInvalidGrant:
+      error->SetString("message", "INVALID_GRANT");
+      response_dict.SetDictionary("error", std::move(error));
+      FormatJSONResponse(response_dict, net::HTTP_BAD_REQUEST, http_response);
+      break;
+
+    case GaiaAuthConsumer::ReAuthProofTokenStatus::kInvalidRequest:
+      error->SetString("message", "INVALID_REQUEST");
+      response_dict.SetDictionary("error", std::move(error));
+      FormatJSONResponse(response_dict, net::HTTP_BAD_REQUEST, http_response);
+      break;
+
+    case GaiaAuthConsumer::ReAuthProofTokenStatus::kUnauthorizedClient:
+      error->SetString("message", "UNAUTHORIZED_CLIENT");
+      response_dict.SetDictionary("error", std::move(error));
+      FormatJSONResponse(response_dict, net::HTTP_FORBIDDEN, http_response);
+      break;
+
+    case GaiaAuthConsumer::ReAuthProofTokenStatus::kInsufficientScope:
+      error->SetString("message", "INSUFFICIENT_SCOPE");
+      response_dict.SetDictionary("error", std::move(error));
+      FormatJSONResponse(response_dict, net::HTTP_FORBIDDEN, http_response);
+      break;
+
+    case GaiaAuthConsumer::ReAuthProofTokenStatus::kCredentialNotSet:
+      response_dict.SetDictionary("error", std::move(error));
+      FormatJSONResponse(response_dict, net::HTTP_FORBIDDEN, http_response);
+      break;
+
+    default:
+      LOG(FATAL) << "Unsupported ReAuthProofTokenStatus: "
+                 << static_cast<int>(next_reauth_status_);
+      break;
+  }
+}
+
+void FakeGaia::HandleMultilogin(const HttpRequest& request,
+                                BasicHttpResponse* http_response) {
+  http_response->set_code(net::HTTP_UNAUTHORIZED);
+
+  if (merge_session_params_.session_sid_cookie.empty() ||
+      merge_session_params_.session_lsid_cookie.empty()) {
+    http_response->set_code(net::HTTP_BAD_REQUEST);
+    return;
+  }
+
+  GURL request_url = GURL("http://localhost").Resolve(request.relative_url);
+  std::string request_query = request_url.query();
+
+  std::string source;
+  if (!GetQueryParameter(request_query, "source", &source)) {
+    LOG(ERROR) << "Missing or invalid 'source' param in /Multilogin call";
+    return;
+  }
+
+  http_response->set_content(
+      ")]}'\n{\"status\":\"OK\",\"cookies\":[" +
+      FormatCookieForMultilogin("SID",
+                                merge_session_params_.session_sid_cookie) +
+      "," +
+      FormatCookieForMultilogin("LSID",
+                                merge_session_params_.session_lsid_cookie) +
+      "]}");
+  http_response->set_code(net::HTTP_OK);
+}
+
+std::string FakeGaia::GetEmbeddedSetupChromeosResponseContent() const {
+  if (embedded_setup_chromeos_iframe_url_.is_empty())
+    return embedded_setup_chromeos_response_;
+  const std::string iframe =
+      base::StringPrintf("<iframe src=\"%s\" style=\"%s\"></iframe>",
+                         embedded_setup_chromeos_iframe_url_.spec().c_str(),
+                         "width:0; height:0; border:none;");
+  // Insert the iframe right before </body>
+  std::string response_with_iframe = embedded_setup_chromeos_response_;
+  size_t pos_of_body_closing_tag = response_with_iframe.find("</body>");
+  CHECK(pos_of_body_closing_tag != std::string::npos);
+  response_with_iframe.insert(pos_of_body_closing_tag, iframe);
+  return response_with_iframe;
 }
