@@ -12,10 +12,6 @@
 class GoogleServiceAuthError;
 class OAuthMultiloginResult;
 
-namespace net {
-typedef std::vector<std::string> ResponseCookies;
-}
-
 typedef std::map<std::string, std::string> UserInfoMap;
 
 // An interface that defines the callbacks for objects that
@@ -38,11 +34,9 @@ class GaiaAuthConsumer {
     std::string token;
     // TODO(chron): Remove the data field later. Don't use it if possible.
     std::string data;  // Full contents of ClientLogin return.
-    bool two_factor;  // set to true if there was a TWO_FACTOR "failure".
   };
 
   struct ClientOAuthResult {
-    ClientOAuthResult();
     ClientOAuthResult(const std::string& new_refresh_token,
                       const std::string& new_access_token,
                       int new_expires_in_secs,
@@ -93,6 +87,29 @@ class GaiaAuthConsumer {
     kMaxValue = kUnknownError
   };
 
+  // Used in UMA, do not delete or reorder values.
+  // Update enums.xml when adding entries.
+  enum class ReAuthProofTokenStatus : int {
+    // Successful request: used only to control FakeGaia response.
+    kSuccess = 0,
+    // Request had invalid format.
+    kInvalidRequest = 1,
+    // Password was incorrect.
+    kInvalidGrant = 2,
+    // Unauthorized OAuth client.
+    kUnauthorizedClient = 3,
+    // Scope of OAuth token was insufficient.
+    kInsufficientScope = 4,
+    // No credential specified.
+    kCredentialNotSet = 5,
+    // A network error.
+    kNetworkError = 6,
+    // Other error.
+    kUnknownError = 7,
+
+    kMaxValue = kUnknownError
+  };
+
   virtual ~GaiaAuthConsumer() {}
 
   virtual void OnClientLoginSuccess(const ClientLoginResult& result) {}
@@ -116,8 +133,7 @@ class GaiaAuthConsumer {
   virtual void OnListAccountsSuccess(const std::string& data) {}
   virtual void OnListAccountsFailure(const GoogleServiceAuthError& error) {}
 
-  virtual void OnOAuthMultiloginSuccess(const OAuthMultiloginResult& result) {}
-  virtual void OnOAuthMultiloginFailure(const GoogleServiceAuthError& error) {}
+  virtual void OnOAuthMultiloginFinished(const OAuthMultiloginResult& result) {}
 
   virtual void OnLogOutSuccess() {}
   virtual void OnLogOutFailure(const GoogleServiceAuthError& error) {}
@@ -125,6 +141,10 @@ class GaiaAuthConsumer {
   virtual void OnGetCheckConnectionInfoSuccess(const std::string& data) {}
   virtual void OnGetCheckConnectionInfoError(
       const GoogleServiceAuthError& error) {}
+
+  virtual void OnReAuthProofTokenSuccess(
+      const std::string& reauth_proof_token) {}
+  virtual void OnReAuthProofTokenFailure(const ReAuthProofTokenStatus error) {}
 };
 
 #endif  // GOOGLE_APIS_GAIA_GAIA_AUTH_CONSUMER_H_
