@@ -8,6 +8,7 @@
 #include <utility>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/json/json_reader.h"
 #include "base/json/json_writer.h"
 #include "base/macros.h"
@@ -15,11 +16,11 @@
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/user_metrics.h"
 #include "base/metrics/user_metrics_action.h"
-#include "base/process/process_info.h"
+#include "base/stl_util.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/sys_info.h"
+#include "base/system/sys_info.h"
 #include "base/values.h"
 #include "google_apis/gaia/gaia_auth_consumer.h"
 #include "google_apis/gaia/gaia_auth_util.h"
@@ -29,26 +30,27 @@
 #include "google_apis/gaia/oauth2_id_token_decoder.h"
 #include "google_apis/gaia/oauth_multilogin_result.h"
 #include "net/base/escape.h"
-#include "net/base/load_flags.h"
+#include "net/base/isolation_info.h"
 #include "net/http/http_response_headers.h"
 #include "net/http/http_status_code.h"
-#include "net/http/http_util.h"
-#include "net/url_request/url_request_status.h"
 #include "services/network/public/cpp/resource_request.h"
-#include "services/network/public/cpp/resource_response.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "services/network/public/cpp/simple_url_loader.h"
 
 namespace {
 
-const int kLoadFlagsIgnoreCookies = net::LOAD_DO_NOT_SEND_COOKIES |
-                                    net::LOAD_DO_NOT_SAVE_COOKIES;
-
 const size_t kMaxMessageSize = 1024 * 1024;  // 1MB
 
+constexpr char kBadAuthenticationError[] = "BadAuthentication";
+constexpr char kBadAuthenticationShortError[] = "badauth";
+constexpr char kServiceUnavailableError[] = "ServiceUnavailable";
+constexpr char kServiceUnavailableShortError[] = "ire";
+constexpr char kFormEncodedContentType[] = "application/x-www-form-urlencoded";
+constexpr char kJsonContentType[] = "application/json;charset=UTF-8";
+
 std::unique_ptr<const GaiaAuthConsumer::ClientOAuthResult>
 ExtractOAuth2TokenPairResponse(const std::string& data) {
-  std::unique_ptr<base::Value> value = base::JSONReader::Read(data);
+  std::unique_ptr<base::Value> value = base::JSONReader::ReadDeprecated(data);
   if (!value.get() || value->type() != base::Value::Type::DICTIONARY)
     return nullptr;
 
@@ -78,16 +80,6 @@ ExtractOAuth2TokenPairResponse(const std::string& data) {
       service_flags.is_under_advanced_protection);
 }
 
-void GetCookiesFromResponse(
-    const network::HttpRawRequestResponseInfo::HeadersVector& headers,
-    net::ResponseCookies* cookies) {
-  for (const auto& header : headers) {
-    if (header.first == "Set-Cookie" && !header.second.empty()) {
-      cookies->push_back(header.second);
-    }
-  }
-}
-
 // Parses server responses for token revocation.
 GaiaAuthConsumer::TokenRevocationStatus
 GetTokenRevocationStatusFromResponseData(const std::string& data,
@@ -98,7 +90,7 @@ GetTokenRevocationStatusFromResponseData(const std::string& data,
   if (response_code == net::HTTP_INTERNAL_SERVER_ERROR)
     return GaiaAuthConsumer::TokenRevocationStatus::kServerError;
 
-  std::unique_ptr<base::Value> value = base::JSONReader::Read(data);
+  std::unique_ptr<base::Value> value = base::JSONReader::ReadDeprecated(data);
   if (!value.get() || value->type() != base::Value::Type::DICTIONARY)
     return GaiaAuthConsumer::TokenRevocationStatus::kUnknownError;
 
@@ -116,8 +108,84 @@ GetTokenRevocationStatusFromResponseData(const std::string& data,
   return GaiaAuthConsumer::TokenRevocationStatus::kUnknownError;
 }
 
+std::unique_ptr<base::DictionaryValue> ParseJSONDict(const std::string& data) {
+  std::unique_ptr<base::DictionaryValue> response_dict;
+  base::Optional<base::Value> message_value = base::JSONReader::Read(data);
+  if (message_value && message_value->is_dict()) {
+    response_dict = std::make_unique<base::DictionaryValue>();
+    response_dict->MergeDictionary(base::OptionalOrNullptr(message_value));
+  }
+  return response_dict;
+}
+
+GaiaAuthConsumer::ReAuthProofTokenStatus ErrorMessageToReAuthProofTokenStatus(
+    const std::string& message) {
+  if (message == "INVALID_REQUEST") {
+    return GaiaAuthConsumer::ReAuthProofTokenStatus::kInvalidRequest;
+  }
+  if (message == "INVALID_GRANT") {
+    return GaiaAuthConsumer::ReAuthProofTokenStatus::kInvalidGrant;
+  }
+  if (message == "UNAUTHORIZED_CLIENT") {
+    return GaiaAuthConsumer::ReAuthProofTokenStatus::kUnauthorizedClient;
+  }
+  if (message == "INSUFFICIENT_SCOPE") {
+    return GaiaAuthConsumer::ReAuthProofTokenStatus::kInsufficientScope;
+  }
+  if (message == "CREDENTIAL_NOT_SET") {
+    return GaiaAuthConsumer::ReAuthProofTokenStatus::kCredentialNotSet;
+  }
+  DLOG(ERROR) << "Unrecognized ReauthAPI error message: " + message;
+  return GaiaAuthConsumer::ReAuthProofTokenStatus::kUnknownError;
+}
+
 }  // namespace
 
+namespace gaia {
+
+GaiaSource::GaiaSource(Type type) : type_(type) {}
+
+GaiaSource::GaiaSource(Type type, const std::string& suffix)
+    : type_(type), suffix_(suffix) {}
+
+void GaiaSource::SetGaiaSourceSuffix(const std::string& suffix) {
+  suffix_ = suffix;
+}
+
+std::string GaiaSource::ToString() {
+  std::string source_string;
+  switch (type_) {
+    case Type::kChrome:
+      source_string = GaiaConstants::kChromeSource;
+      break;
+    case Type::kChromeOS:
+      source_string = GaiaConstants::kChromeOSSource;
+      break;
+    case Type::kAccountReconcilorDice:
+      source_string = "ChromiumAccountReconcilorDice";
+      break;
+    case Type::kAccountReconcilorMirror:
+      source_string = "ChromiumAccountReconcilor";
+      break;
+    case Type::kOAuth2LoginVerifier:
+      source_string = "ChromiumOAuth2LoginVerifier";
+      break;
+    case Type::kPrimaryAccountManager:
+      // Even though this string refers to an old name from the Chromium POV, it
+      // should not be changed as it is passed server-side.
+      source_string = "ChromiumSigninManager";
+      break;
+  }
+
+  // All sources should start with Chromium or chromeos for better server logs.
+  DCHECK(source_string == "chromeos" ||
+         base::StartsWith(source_string, "Chromium",
+                          base::CompareCase::SENSITIVE));
+  return source_string + suffix_;
+}
+
+}  // namespace gaia
+
 // static
 const char GaiaAuthFetcher::kIssueAuthTokenFormat[] =
     "SID=%s&"
@@ -152,30 +220,10 @@ const char GaiaAuthFetcher::kUberAuthTokenURLFormat[] =
 
 const char GaiaAuthFetcher::kOAuthLoginFormat[] = "service=%s&source=%s";
 
-// static
-const char GaiaAuthFetcher::kAccountDeletedError[] = "AccountDeleted";
-// static
-const char GaiaAuthFetcher::kAccountDisabledError[] = "AccountDisabled";
-// static
-const char GaiaAuthFetcher::kBadAuthenticationError[] = "BadAuthentication";
-// static
-const char GaiaAuthFetcher::kCaptchaError[] = "CaptchaRequired";
-// static
-const char GaiaAuthFetcher::kServiceUnavailableError[] =
-    "ServiceUnavailable";
 // static
 const char GaiaAuthFetcher::kErrorParam[] = "Error";
 // static
 const char GaiaAuthFetcher::kErrorUrlParam[] = "Url";
-// static
-const char GaiaAuthFetcher::kCaptchaUrlParam[] = "CaptchaUrl";
-// static
-const char GaiaAuthFetcher::kCaptchaTokenParam[] = "CaptchaToken";
-
-// static
-const char GaiaAuthFetcher::kSecondFactor[] = "Info=InvalidSecondFactor";
-// static
-const char GaiaAuthFetcher::kWebLoginRequired[] = "Info=WebLoginRequired";
 
 // static
 const char GaiaAuthFetcher::kAuthHeaderFormat[] =
@@ -191,25 +239,26 @@ const char GaiaAuthFetcher::kOAuth2BearerHeaderFormat[] =
 
 GaiaAuthFetcher::GaiaAuthFetcher(
     GaiaAuthConsumer* consumer,
-    const std::string& source,
+    gaia::GaiaSource source,
     scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory)
     : url_loader_factory_(url_loader_factory),
       consumer_(consumer),
-      source_(source),
+      source_(source.ToString()),
       oauth2_token_gurl_(GaiaUrls::GetInstance()->oauth2_token_url()),
       oauth2_revoke_gurl_(GaiaUrls::GetInstance()->oauth2_revoke_url()),
       get_user_info_gurl_(GaiaUrls::GetInstance()->get_user_info_url()),
       merge_session_gurl_(GaiaUrls::GetInstance()->merge_session_url()),
       uberauth_token_gurl_(GaiaUrls::GetInstance()->oauth1_login_url().Resolve(
-          base::StringPrintf(kUberAuthTokenURLFormat, source.c_str()))),
+          base::StringPrintf(kUberAuthTokenURLFormat, source_.c_str()))),
       oauth_login_gurl_(GaiaUrls::GetInstance()->oauth1_login_url()),
       oauth_multilogin_gurl_(GaiaUrls::GetInstance()->oauth_multilogin_url()),
       list_accounts_gurl_(
-          GaiaUrls::GetInstance()->ListAccountsURLWithSource(source)),
-      logout_gurl_(GaiaUrls::GetInstance()->LogOutURLWithSource(source)),
+          GaiaUrls::GetInstance()->ListAccountsURLWithSource(source_)),
+      logout_gurl_(GaiaUrls::GetInstance()->LogOutURLWithSource(source_)),
       get_check_connection_info_url_(
           GaiaUrls::GetInstance()->GetCheckConnectionInfoURLWithSource(
-              source)) {}
+              source_)),
+      reauth_api_url_(GaiaUrls::GetInstance()->reauth_api_url()) {}
 
 GaiaAuthFetcher::~GaiaAuthFetcher() {}
 
@@ -232,11 +281,17 @@ bool GaiaAuthFetcher::IsMultiloginUrl(const GURL& url) {
                           base::CompareCase::SENSITIVE);
 }
 
+bool GaiaAuthFetcher::IsReAuthApiUrl(const GURL& url) {
+  return base::StartsWith(url.spec(), reauth_api_url_.spec(),
+                          base::CompareCase::SENSITIVE);
+}
+
 void GaiaAuthFetcher::CreateAndStartGaiaFetcher(
     const std::string& body,
+    const std::string& body_content_type,
     const std::string& headers,
     const GURL& gaia_gurl,
-    int load_flags,
+    network::mojom::CredentialsMode credentials_mode,
     const net::NetworkTrafficAnnotationTag& traffic_annotation) {
   DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
 
@@ -244,10 +299,17 @@ void GaiaAuthFetcher::CreateAndStartGaiaFetcher(
   resource_request->url = gaia_gurl;
   original_url_ = gaia_gurl;
 
-  if (!(load_flags & net::LOAD_DO_NOT_SEND_COOKIES)) {
+  if (credentials_mode != network::mojom::CredentialsMode::kOmit) {
     DCHECK_EQ(GaiaUrls::GetInstance()->gaia_url(), gaia_gurl.GetOrigin())
         << gaia_gurl;
-    resource_request->site_for_cookies = GaiaUrls::GetInstance()->gaia_url();
+    url::Origin origin =
+        url::Origin::Create(GaiaUrls::GetInstance()->gaia_url());
+    resource_request->site_for_cookies =
+        net::SiteForCookies::FromOrigin(origin);
+    resource_request->trusted_params =
+        network::ResourceRequest::TrustedParams();
+    resource_request->trusted_params->isolation_info =
+        net::IsolationInfo::CreateForInternalRequest(origin);
   }
 
   if (!body.empty())
@@ -261,17 +323,14 @@ void GaiaAuthFetcher::CreateAndStartGaiaFetcher(
   // maintain a separation between the user's browsing and Chrome's internal
   // services.  Where such mixing is desired (MergeSession or OAuthLogin), it
   // will be done explicitly.
-  resource_request->load_flags = load_flags;
-
-  // Use raw headers as the cookies are filtered-out of the response when
-  // serialized at the IPC layer.
-  resource_request->report_raw_headers = true;
+  resource_request->credentials_mode = credentials_mode;
 
   url_loader_ = network::SimpleURLLoader::Create(std::move(resource_request),
                                                  traffic_annotation);
-  if (!body.empty())
-    url_loader_->AttachStringForUpload(body,
-                                       "application/x-www-form-urlencoded");
+  if (!body.empty()) {
+    DCHECK(!body_content_type.empty());
+    url_loader_->AttachStringForUpload(body, body_content_type);
+  }
 
   url_loader_->SetAllowHttpErrorResults(true);
 
@@ -422,9 +481,7 @@ std::string GaiaAuthFetcher::MakeOAuthLoginBody(const std::string& service,
 // static
 void GaiaAuthFetcher::ParseClientLoginFailure(const std::string& data,
                                               std::string* error,
-                                              std::string* error_url,
-                                              std::string* captcha_url,
-                                              std::string* captcha_token) {
+                                              std::string* error_url) {
   using std::vector;
   using std::pair;
   using std::string;
@@ -437,10 +494,6 @@ void GaiaAuthFetcher::ParseClientLoginFailure(const std::string& data,
       error->assign(i->second);
     } else if (i->first == kErrorUrlParam) {
       error_url->assign(i->second);
-    } else if (i->first == kCaptchaUrlParam) {
-      captcha_url->assign(i->second);
-    } else if (i->first == kCaptchaTokenParam) {
-      captcha_token->assign(i->second);
     }
   }
 }
@@ -473,8 +526,10 @@ void GaiaAuthFetcher::StartRevokeOAuth2Token(const std::string& auth_token) {
             }
           }
         })");
-  CreateAndStartGaiaFetcher(request_body_, std::string(), oauth2_revoke_gurl_,
-                            kLoadFlagsIgnoreCookies, traffic_annotation);
+  CreateAndStartGaiaFetcher(request_body_, kFormEncodedContentType,
+                            std::string(), oauth2_revoke_gurl_,
+                            network::mojom::CredentialsMode::kOmit,
+                            traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartAuthCodeForOAuth2TokenExchange(
@@ -517,8 +572,9 @@ void GaiaAuthFetcher::StartAuthCodeForOAuth2TokenExchangeWithDeviceId(
             }
           }
         })");
-  CreateAndStartGaiaFetcher(request_body_, std::string(), oauth2_token_gurl_,
-                            kLoadFlagsIgnoreCookies, traffic_annotation);
+  CreateAndStartGaiaFetcher(
+      request_body_, kFormEncodedContentType, std::string(), oauth2_token_gurl_,
+      network::mojom::CredentialsMode::kOmit, traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartGetUserInfo(const std::string& lsid) {
@@ -549,8 +605,10 @@ void GaiaAuthFetcher::StartGetUserInfo(const std::string& lsid) {
             }
           }
         })");
-  CreateAndStartGaiaFetcher(request_body_, std::string(), get_user_info_gurl_,
-                            kLoadFlagsIgnoreCookies, traffic_annotation);
+  CreateAndStartGaiaFetcher(request_body_, kFormEncodedContentType,
+                            std::string(), get_user_info_gurl_,
+                            network::mojom::CredentialsMode::kOmit,
+                            traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartMergeSession(const std::string& uber_token,
@@ -598,22 +656,20 @@ void GaiaAuthFetcher::StartMergeSession(const std::string& uber_token,
             }
           }
         })");
-  CreateAndStartGaiaFetcher(std::string(), std::string(),
+  CreateAndStartGaiaFetcher(std::string(), std::string(), std::string(),
                             merge_session_gurl_.Resolve(query),
-                            net::LOAD_NORMAL, traffic_annotation);
+                            network::mojom::CredentialsMode::kInclude,
+                            traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartTokenFetchForUberAuthExchange(
-    const std::string& access_token,
-    bool is_bound_to_channel_id) {
+    const std::string& access_token) {
   DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
 
   VLOG(1) << "Starting StartTokenFetchForUberAuthExchange with access_token="
            << access_token;
   std::string authentication_header =
       base::StringPrintf(kOAuthHeaderFormat, access_token.c_str());
-  int load_flags =
-      is_bound_to_channel_id ? net::LOAD_NORMAL : kLoadFlagsIgnoreCookies;
   net::NetworkTrafficAnnotationTag traffic_annotation =
       net::DefineNetworkTrafficAnnotation("gaia_auth_fetch_for_uber", R"(
         semantics {
@@ -641,9 +697,9 @@ void GaiaAuthFetcher::StartTokenFetchForUberAuthExchange(
             }
           }
         })");
-  CreateAndStartGaiaFetcher(std::string(), authentication_header,
-                            uberauth_token_gurl_, load_flags,
-                            traffic_annotation);
+  CreateAndStartGaiaFetcher(
+      std::string(), std::string(), authentication_header, uberauth_token_gurl_,
+      network::mojom::CredentialsMode::kOmit, traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartOAuthLogin(const std::string& access_token,
@@ -683,18 +739,14 @@ void GaiaAuthFetcher::StartOAuthLogin(const std::string& access_token,
             }
           }
         })");
-  CreateAndStartGaiaFetcher(request_body_, authentication_header,
-                            oauth_login_gurl_, net::LOAD_NORMAL,
+  CreateAndStartGaiaFetcher(request_body_, kFormEncodedContentType,
+                            authentication_header, oauth_login_gurl_,
+                            network::mojom::CredentialsMode::kInclude,
                             traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartListAccounts() {
   DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
-  list_accounts_system_uptime_ = base::SysInfo::Uptime();
-#if !defined(OS_IOS)
-  list_accounts_process_uptime_ =
-      base::Time::Now() - base::CurrentProcessInfo::CreationTime();
-#endif
 
   net::NetworkTrafficAnnotationTag traffic_annotation =
       net::DefineNetworkTrafficAnnotation("gaia_auth_list_accounts", R"(
@@ -723,16 +775,22 @@ void GaiaAuthFetcher::StartListAccounts() {
             }
           }
         })");
-  CreateAndStartGaiaFetcher(" ",  // To force an HTTP POST.
-                            "Origin: https://www.google.com",
-                            list_accounts_gurl_, net::LOAD_NORMAL,
-                            traffic_annotation);
+  CreateAndStartGaiaFetcher(
+      " ",  // To force an HTTP POST.
+      kFormEncodedContentType, "Origin: https://www.google.com",
+      list_accounts_gurl_, network::mojom::CredentialsMode::kInclude,
+      traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartOAuthMultilogin(
-    const std::vector<MultiloginTokenIDPair>& accounts) {
+    gaia::MultiloginMode mode,
+    const std::vector<MultiloginTokenIDPair>& accounts,
+    const std::string& external_cc_result) {
   DCHECK(!fetch_pending_) << "Tried to fetch two things at once!";
 
+  UMA_HISTOGRAM_COUNTS_100("Signin.Multilogin.NumberOfAccounts",
+                           accounts.size());
+
   std::vector<std::string> authorization_header_parts;
   for (const MultiloginTokenIDPair& account : accounts) {
     authorization_header_parts.push_back(base::StringPrintf(
@@ -743,8 +801,17 @@ void GaiaAuthFetcher::StartOAuthMultilogin(
       kOAuthMultiBearerHeaderFormat,
       base::JoinString(authorization_header_parts, ",").c_str());
 
+  std::string source_string = net::EscapeUrlEncodedData(source_, true);
   std::string parameters = base::StringPrintf(
-      "?source=%s", net::EscapeUrlEncodedData(source_, true).c_str());
+      "?source=%s&reuseCookies=%i", source_string.c_str(),
+      mode == gaia::MultiloginMode::MULTILOGIN_PRESERVE_COOKIE_ACCOUNTS_ORDER
+          ? 1
+          : 0);
+  if (!external_cc_result.empty()) {
+    base::StringAppendF(
+        &parameters, "&externalCcResult=%s",
+        net::EscapeUrlEncodedData(external_cc_result, true).c_str());
+  }
 
   net::NetworkTrafficAnnotationTag traffic_annotation =
       net::DefineNetworkTrafficAnnotation("gaia_auth_multilogin", R"(
@@ -775,9 +842,10 @@ void GaiaAuthFetcher::StartOAuthMultilogin(
           }
         })");
   CreateAndStartGaiaFetcher(" ",  // Non-empty to force a POST
-                            authorization_header,
+                            kFormEncodedContentType, authorization_header,
                             oauth_multilogin_gurl_.Resolve(parameters),
-                            net::LOAD_NORMAL, traffic_annotation);
+                            network::mojom::CredentialsMode::kInclude,
+                            traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartLogOut() {
@@ -810,8 +878,71 @@ void GaiaAuthFetcher::StartLogOut() {
             }
           }
         })");
-  CreateAndStartGaiaFetcher(std::string(), std::string(), logout_gurl_,
-                            net::LOAD_NORMAL, traffic_annotation);
+  CreateAndStartGaiaFetcher(
+      std::string(), std::string(), std::string(), logout_gurl_,
+      network::mojom::CredentialsMode::kInclude, traffic_annotation);
+}
+
+void GaiaAuthFetcher::StartCreateReAuthProofTokenForParent(
+    const std::string& child_oauth_access_token,
+    const std::string& parent_obfuscated_gaia_id,
+    const std::string& parent_credential) {
+  // Create the post body.
+  base::DictionaryValue post_body_value;
+  post_body_value.SetString("credentialType", "password");
+  post_body_value.SetString("credential", parent_credential);
+  std::string post_body;
+  bool write_success = base::JSONWriter::Write(post_body_value, &post_body);
+  DCHECK(write_success);
+
+  // Create the Authorization header.
+  std::string auth_header = "Bearer " + child_oauth_access_token;
+  std::string headers = "Authorization: " + auth_header + "\r\n" +
+                        "Content-Type: " + kJsonContentType;
+
+  // Create the traffic annotation.
+  net::NetworkTrafficAnnotationTag traffic_annotation(
+      net::DefineNetworkTrafficAnnotation(
+          "gaia_create_reauth_proof_token_for_parent", R"(
+        semantics {
+          sender: "Chrome - Google authentication API"
+          description:
+            "This request exchanges a set of credentials for a parent-type "
+            "Google Family Link user for a ReAuth Proof Token (RAPT), "
+            "the successful receipt of which re-authorizes (but explicitly "
+            "does not authenticate) the parent user."
+          trigger:
+            "This request is triggered when a Chrome service needs to "
+            "re-authorize a parent-type Google Family Link user given the "
+            "parent's login credential."
+          data:
+            "The obfuscated GAIA id of the parent, the Google OAuth access "
+            "token of the child account, and the credential to be used to "
+            "reauthorize the user."
+          destination: GOOGLE_OWNED_SERVICE
+        }
+        policy {
+          cookies_allowed: NO
+          setting:
+            "This feature cannot be disabled in settings, but if the user "
+            "signs out of Chrome, this request would not be made."
+          chrome_policy {
+            SigninAllowed {
+              policy_options {mode: MANDATORY}
+              SigninAllowed: false
+            }
+          }
+        })"));
+
+  // Create the ReAuth URL.
+  GURL reauth_url = GaiaUrls::GetInstance()->reauth_api_url().Resolve(
+      parent_obfuscated_gaia_id + "/reauthProofTokens?delegationType=unicorn");
+  DCHECK(reauth_url.is_valid());
+
+  // Start the request.
+  CreateAndStartGaiaFetcher(post_body, kJsonContentType, headers, reauth_url,
+                            network::mojom::CredentialsMode::kOmit,
+                            traffic_annotation);
 }
 
 void GaiaAuthFetcher::StartGetCheckConnectionInfo() {
@@ -842,54 +973,42 @@ void GaiaAuthFetcher::StartGetCheckConnectionInfo() {
             }
           }
         })");
-  CreateAndStartGaiaFetcher(std::string(), std::string(),
+  CreateAndStartGaiaFetcher(std::string(), std::string(), std::string(),
                             get_check_connection_info_url_,
-                            kLoadFlagsIgnoreCookies, traffic_annotation);
+                            network::mojom::CredentialsMode::kOmit,
+                            traffic_annotation);
 }
 
 // static
 GoogleServiceAuthError GaiaAuthFetcher::GenerateAuthError(
     const std::string& data,
     net::Error net_error) {
+  VLOG(1) << "Got authentication error";
+  VLOG(1) << "net_error: " << net::ErrorToString(net_error);
+  VLOG(1) << "response body: " << data;
+
   if (net_error != net::OK) {
     if (net_error == net::ERR_ABORTED) {
       return GoogleServiceAuthError(GoogleServiceAuthError::REQUEST_CANCELED);
     }
-    DLOG(WARNING) << "Could not reach Google Accounts servers: errno "
-                  << net_error;
+    DVLOG(1) << "Could not reach Google Accounts servers: errno " << net_error;
     return GoogleServiceAuthError::FromConnectionError(net_error);
   }
 
-  if (IsSecondFactorSuccess(data))
-    return GoogleServiceAuthError(GoogleServiceAuthError::TWO_FACTOR);
-
-  if (IsWebLoginRequiredSuccess(data))
-    return GoogleServiceAuthError(GoogleServiceAuthError::WEB_LOGIN_REQUIRED);
-
   std::string error;
   std::string url;
-  std::string captcha_url;
-  std::string captcha_token;
-  ParseClientLoginFailure(data, &error, &url, &captcha_url, &captcha_token);
+  ParseClientLoginFailure(data, &error, &url);
   DLOG(WARNING) << "ClientLogin failed with " << error;
 
-  if (error == kCaptchaError) {
-    return GoogleServiceAuthError::FromClientLoginCaptchaChallenge(
-        captcha_token,
-        GURL(GaiaUrls::GetInstance()->captcha_base_url().Resolve(captcha_url)),
-        GURL(url));
-  }
-  if (error == kAccountDeletedError)
-    return GoogleServiceAuthError(GoogleServiceAuthError::ACCOUNT_DELETED);
-  if (error == kAccountDisabledError)
-    return GoogleServiceAuthError(GoogleServiceAuthError::ACCOUNT_DISABLED);
-  if (error == kBadAuthenticationError) {
+  if (error == kBadAuthenticationShortError ||
+      error == kBadAuthenticationError) {
     return GoogleServiceAuthError(
         GoogleServiceAuthError::FromInvalidGaiaCredentialsReason(
             GoogleServiceAuthError::InvalidGaiaCredentialsReason::
                 CREDENTIALS_REJECTED_BY_SERVER));
   }
-  if (error == kServiceUnavailableError) {
+  if (error == kServiceUnavailableShortError ||
+      error == kServiceUnavailableError) {
     return GoogleServiceAuthError(
         GoogleServiceAuthError::SERVICE_UNAVAILABLE);
   }
@@ -945,28 +1064,8 @@ void GaiaAuthFetcher::OnOAuth2RevokeTokenFetched(const std::string& data,
 void GaiaAuthFetcher::OnListAccountsFetched(const std::string& data,
                                             net::Error net_error,
                                             int response_code) {
-  // Log error rates and details for ListAccounts, for investigation of
-  // https://crbug.com/876306.
   base::UmaHistogramSparse("Gaia.AuthFetcher.ListAccounts.NetErrorCodes",
                            -net_error);
-  if (net_error == net::OK) {
-    UMA_HISTOGRAM_LONG_TIMES(
-        "Gaia.AuthFetcher.ListAccounts.SystemUptime.Success",
-        list_accounts_system_uptime_);
-#if !defined(OS_IOS)
-    UMA_HISTOGRAM_LONG_TIMES(
-        "Gaia.AuthFetcher.ListAccounts.ProcessUptime.Success",
-        list_accounts_process_uptime_);
-#endif
-  } else {
-    UMA_HISTOGRAM_LONG_TIMES("Gaia.AuthFetcher.ListAccounts.SystemUptime.Error",
-                             list_accounts_system_uptime_);
-#if !defined(OS_IOS)
-    UMA_HISTOGRAM_LONG_TIMES(
-        "Gaia.AuthFetcher.ListAccounts.ProcessUptime.Error",
-        list_accounts_process_uptime_);
-#endif
-  }
 
   if (net_error == net::OK && response_code == net::HTTP_OK)
     consumer_->OnListAccountsSuccess(data);
@@ -1001,6 +1100,37 @@ void GaiaAuthFetcher::OnGetUserInfoFetched(const std::string& data,
   }
 }
 
+void GaiaAuthFetcher::OnReAuthApiInfoFetched(const std::string& data,
+                                             net::Error net_error,
+                                             int response_code) {
+  if (net_error == net::OK) {
+    std::unique_ptr<base::DictionaryValue> response_dict = ParseJSONDict(data);
+
+    if (response_code == net::HTTP_OK) {
+      std::string rapt_token;
+      response_dict->GetString("encodedRapt", &rapt_token);
+      if (rapt_token.empty()) {
+        // This should not happen unless there is a bug on the server,
+        // since if we get HTTP_OK response, we should get a RAPT token.
+        DLOG(ERROR) << "Got HTTP-OK ReauthAPI response with empty RAPT token";
+        consumer_->OnReAuthProofTokenFailure(
+            GaiaAuthConsumer::ReAuthProofTokenStatus::kUnknownError);
+        return;
+      }
+      consumer_->OnReAuthProofTokenSuccess(rapt_token);
+    } else {
+      const std::string error_message =
+          response_dict->FindPath({"error", "message"})->GetString();
+
+      consumer_->OnReAuthProofTokenFailure(
+          ErrorMessageToReAuthProofTokenStatus(error_message));
+    }
+  } else {
+    consumer_->OnReAuthProofTokenFailure(
+        GaiaAuthConsumer::ReAuthProofTokenStatus::kNetworkError);
+  }
+}
+
 void GaiaAuthFetcher::OnMergeSessionFetched(const std::string& data,
                                             net::Error net_error,
                                             int response_code) {
@@ -1051,21 +1181,11 @@ void GaiaAuthFetcher::OnGetCheckConnectionInfoFetched(const std::string& data,
 void GaiaAuthFetcher::OnOAuthMultiloginFetched(const std::string& data,
                                                net::Error net_error,
                                                int response_code) {
-  GoogleServiceAuthError auth_error = GoogleServiceAuthError::AuthErrorNone();
-  if (net_error == net::Error::OK && response_code == net::HTTP_OK) {
-    OAuthMultiloginResult result;
-    const GoogleServiceAuthError error =
-        OAuthMultiloginResult::CreateOAuthMultiloginResultFromString(data,
-                                                                     &result);
-    if (error.state() != GoogleServiceAuthError::State::NONE) {
-      consumer_->OnOAuthMultiloginFailure(error);
-    } else {
-      consumer_->OnOAuthMultiloginSuccess(result);
-    }
-  } else {
-    auth_error = GenerateAuthError(data, net_error);
-    consumer_->OnOAuthMultiloginFailure(auth_error);
-  }
+  OAuthMultiloginResult result =
+      (net_error == net::Error::OK)
+          ? OAuthMultiloginResult(data)
+          : OAuthMultiloginResult(OAuthMultiloginResponseStatus::kRetry);
+  consumer_->OnOAuthMultiloginFinished(result);
 }
 
 void GaiaAuthFetcher::OnURLLoadComplete(
@@ -1074,39 +1194,30 @@ void GaiaAuthFetcher::OnURLLoadComplete(
   std::string data = response_body ? std::move(*response_body) : "";
 
   int response_code = 0;
-  network::HttpRawRequestResponseInfo::HeadersVector headers;
   if (url_loader_->ResponseInfo()) {
     if (url_loader_->ResponseInfo()->headers)
       response_code = url_loader_->ResponseInfo()->headers->response_code();
-    if (url_loader_->ResponseInfo()->raw_request_response_info) {
-      headers = url_loader_->ResponseInfo()
-                    ->raw_request_response_info->response_headers;
-    }
   }
-  OnURLLoadCompleteInternal(net_error, response_code, headers, data);
+  OnURLLoadCompleteInternal(net_error, response_code, data);
 }
 
 void GaiaAuthFetcher::OnURLLoadCompleteInternal(
     net::Error net_error,
     int response_code,
-    const network::HttpRawRequestResponseInfo::HeadersVector& headers,
     std::string data) {
   fetch_pending_ = false;
-  net::ResponseCookies cookies;
-  GetCookiesFromResponse(headers, &cookies);
 
   // Some of the GAIA requests perform redirects, which results in the final URL
   // of the fetcher not being the original URL requested.  Therefore use the
   // original URL when determining which OnXXX function to call.
   GURL url = original_url_;
   original_url_ = GURL();
-  DispatchFetchedRequest(url, data, cookies, net_error, response_code);
+  DispatchFetchedRequest(url, data, net_error, response_code);
 }
 
 void GaiaAuthFetcher::DispatchFetchedRequest(
     const GURL& url,
     const std::string& data,
-    const net::ResponseCookies& cookies,
     net::Error net_error,
     int response_code) {
   if (url == oauth2_token_gurl_) {
@@ -1130,21 +1241,9 @@ void GaiaAuthFetcher::DispatchFetchedRequest(
     OnLogOutFetched(data, net_error, response_code);
   } else if (url == get_check_connection_info_url_) {
     OnGetCheckConnectionInfoFetched(data, net_error, response_code);
+  } else if (IsReAuthApiUrl(url)) {
+    OnReAuthApiInfoFetched(data, net_error, response_code);
   } else {
     NOTREACHED() << "Unknown url: '" << url << "'";
   }
 }
-
-// static
-bool GaiaAuthFetcher::IsSecondFactorSuccess(
-    const std::string& alleged_error) {
-  return alleged_error.find(kSecondFactor) !=
-      std::string::npos;
-}
-
-// static
-bool GaiaAuthFetcher::IsWebLoginRequiredSuccess(
-    const std::string& alleged_error) {
-  return alleged_error.find(kWebLoginRequired) !=
-      std::string::npos;
-}
