@@ -4,31 +4,32 @@
 //
 // A complete set of unit tests for GaiaAuthFetcher.
 // Originally ported from GoogleAuthenticator tests.
+#include "google_apis/gaia/gaia_auth_fetcher.h"
 
 #include <string>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/json/json_reader.h"
 #include "base/memory/ref_counted.h"
-#include "base/message_loop/message_loop.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
-#include "base/test/bind_test_util.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/bind.h"
+#include "base/test/task_environment.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "google_apis/gaia/gaia_auth_consumer.h"
-#include "google_apis/gaia/gaia_auth_fetcher.h"
+#include "google_apis/gaia/gaia_constants.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "google_apis/gaia/google_service_auth_error.h"
-#include "google_apis/gaia/mock_url_fetcher_factory.h"
 #include "google_apis/gaia/oauth_multilogin_result.h"
 #include "google_apis/google_api_keys.h"
-#include "net/base/load_flags.h"
 #include "net/base/net_errors.h"
 #include "net/http/http_response_headers.h"
 #include "net/http/http_status_code.h"
+#include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "services/network/public/mojom/fetch_api.mojom.h"
 #include "services/network/test/test_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -54,7 +55,10 @@ std::string GetRequestBodyAsString(const network::ResourceRequest* request) {
     return "";
   }
   const network::DataElement& elem = request->request_body->elements()->at(0);
-  return std::string(elem.bytes(), elem.length());
+  if (elem.type() != network::DataElement::Tag::kBytes) {
+    return "";
+  }
+  return std::string(elem.As<network::DataElementBytes>().AsStringPiece());
 }
 
 }  // namespace
@@ -69,8 +73,7 @@ class GaiaAuthFetcherTest : public testing::Test {
             GaiaUrls::GetInstance()->oauth1_login_url().Resolve(
                 "?source=&issueuberauth=1")),
         oauth_login_gurl_(GaiaUrls::GetInstance()->oauth1_login_url()),
-        scoped_task_environment_(
-            base::test::ScopedTaskEnvironment::MainThreadType::UI),
+        task_environment_(base::test::TaskEnvironment::MainThreadType::UI),
         test_shared_loader_factory_(
             base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
                 &test_url_loader_factory_)) {
@@ -97,23 +100,13 @@ class GaiaAuthFetcherTest : public testing::Test {
 
   void RunErrorParsingTest(const std::string& data,
                            const std::string& error,
-                           const std::string& error_url,
-                           const std::string& captcha_url,
-                           const std::string& captcha_token) {
+                           const std::string& error_url) {
     std::string out_error;
     std::string out_error_url;
-    std::string out_captcha_url;
-    std::string out_captcha_token;
 
-    GaiaAuthFetcher::ParseClientLoginFailure(data,
-                                             &out_error,
-                                             &out_error_url,
-                                             &out_captcha_url,
-                                             &out_captcha_token);
+    GaiaAuthFetcher::ParseClientLoginFailure(data, &out_error, &out_error_url);
     EXPECT_EQ(error, out_error);
     EXPECT_EQ(error_url, out_error_url);
-    EXPECT_EQ(captcha_url, out_captcha_url);
-    EXPECT_EQ(captcha_token, out_captcha_token);
   }
 
   GURL oauth2_token_source_;
@@ -131,7 +124,7 @@ class GaiaAuthFetcherTest : public testing::Test {
     return test_shared_loader_factory_;
   }
 
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   network::TestURLLoaderFactory test_url_loader_factory_;
   scoped_refptr<network::SharedURLLoaderFactory> test_shared_loader_factory_;
   std::vector<network::ResourceRequest> received_requests_;
@@ -147,7 +140,7 @@ class MockGaiaConsumer : public GaiaAuthConsumer {
   MOCK_METHOD1(OnClientOAuthSuccess,
                void(const GaiaAuthConsumer::ClientOAuthResult& result));
   MOCK_METHOD1(OnMergeSessionSuccess, void(const std::string& data));
-  MOCK_METHOD1(OnOAuthMultiloginSuccess,
+  MOCK_METHOD1(OnOAuthMultiloginFinished,
                void(const OAuthMultiloginResult& result));
   MOCK_METHOD1(OnUberAuthTokenSuccess, void(const std::string& data));
   MOCK_METHOD1(OnClientLoginFailure,
@@ -158,14 +151,15 @@ class MockGaiaConsumer : public GaiaAuthConsumer {
                void(GaiaAuthConsumer::TokenRevocationStatus status));
   MOCK_METHOD1(OnMergeSessionFailure, void(
       const GoogleServiceAuthError& error));
-  MOCK_METHOD1(OnOAuthMultiloginFailure,
-               void(const GoogleServiceAuthError& error));
   MOCK_METHOD1(OnUberAuthTokenFailure, void(
       const GoogleServiceAuthError& error));
   MOCK_METHOD1(OnListAccountsSuccess, void(const std::string& data));
   MOCK_METHOD0(OnLogOutSuccess, void());
   MOCK_METHOD1(OnLogOutFailure, void(const GoogleServiceAuthError& error));
   MOCK_METHOD1(OnGetCheckConnectionInfoSuccess, void(const std::string& data));
+  MOCK_METHOD1(OnReAuthProofTokenSuccess, void(const std::string& rapt_token));
+  MOCK_METHOD1(OnReAuthProofTokenFailure,
+               void(GaiaAuthConsumer::ReAuthProofTokenStatus status));
 };
 
 #if defined(OS_WIN)
@@ -201,30 +195,26 @@ class TestGaiaAuthFetcher : public GaiaAuthFetcher {
  public:
   TestGaiaAuthFetcher(
       GaiaAuthConsumer* consumer,
-      const std::string& source,
       scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory)
-      : GaiaAuthFetcher(consumer, source, url_loader_factory) {}
+      : GaiaAuthFetcher(consumer,
+                        gaia::GaiaSource::kChrome,
+                        url_loader_factory) {}
 
   void CreateAndStartGaiaFetcherForTesting(
       const std::string& body,
       const std::string& headers,
       const GURL& gaia_gurl,
-      int load_flags,
+      network::mojom::CredentialsMode credentials_mode,
       const net::NetworkTrafficAnnotationTag& traffic_annotation) {
-    CreateAndStartGaiaFetcher(body, headers, gaia_gurl, load_flags,
-                              traffic_annotation);
+    CreateAndStartGaiaFetcher(body, /*content_type=*/"", headers, gaia_gurl,
+                              credentials_mode, traffic_annotation);
   }
 
   void TestOnURLLoadCompleteInternal(
       net::Error net_error,
       int response_code = net::HTTP_OK,
-      const std::vector<std::string>& cookies = {},
       std::string response_body = "") {
-    net::HttpRawRequestHeaders::HeaderVector headers;
-    for (auto& cookie : cookies) {
-      headers.push_back(std::make_pair("Set-Cookie", cookie));
-    }
-    OnURLLoadCompleteInternal(net_error, response_code, headers, response_body);
+    OnURLLoadCompleteInternal(net_error, response_code, response_body);
   }
 };
 
@@ -239,62 +229,18 @@ TEST_F(GaiaAuthFetcherTest, ParseRequest) {
 }
 
 TEST_F(GaiaAuthFetcherTest, ParseErrorRequest) {
-  RunErrorParsingTest("Url=U\n"
-                      "Error=E\n"
-                      "CaptchaToken=T\n"
-                      "CaptchaUrl=C\n", "E", "U", "C", "T");
-  RunErrorParsingTest("CaptchaToken=T\n"
-                      "Error=E\n"
-                      "Url=U\n"
-                      "CaptchaUrl=C\n", "E", "U", "C", "T");
-  RunErrorParsingTest("\n\n\nCaptchaToken=T\n"
-                      "\nError=E\n"
-                      "\nUrl=U\n"
-                      "CaptchaUrl=C\n", "E", "U", "C", "T");
-}
-
-TEST_F(GaiaAuthFetcherTest, CheckTwoFactorResponse) {
-  std::string response =
-      base::StringPrintf("Error=BadAuthentication\n%s\n",
-                         GaiaAuthFetcher::kSecondFactor);
-  EXPECT_TRUE(GaiaAuthFetcher::IsSecondFactorSuccess(response));
-}
-
-TEST_F(GaiaAuthFetcherTest, CheckNormalErrorCode) {
-  std::string response = "Error=BadAuthentication\n";
-  EXPECT_FALSE(GaiaAuthFetcher::IsSecondFactorSuccess(response));
-}
-
-TEST_F(GaiaAuthFetcherTest, CaptchaParse) {
-  std::string data = "Url=http://www.google.com/login/captcha\n"
-                     "Error=CaptchaRequired\n"
-                     "CaptchaToken=CCTOKEN\n"
-                     "CaptchaUrl=Captcha?ctoken=CCTOKEN\n";
-  GoogleServiceAuthError error =
-      GaiaAuthFetcher::GenerateAuthError(data, net::OK);
-
-  std::string token = "CCTOKEN";
-  GURL image_url("http://accounts.google.com/Captcha?ctoken=CCTOKEN");
-  GURL unlock_url("http://www.google.com/login/captcha");
-
-  EXPECT_EQ(error.state(), GoogleServiceAuthError::CAPTCHA_REQUIRED);
-  EXPECT_EQ(error.captcha().token, token);
-  EXPECT_EQ(error.captcha().image_url, image_url);
-  EXPECT_EQ(error.captcha().unlock_url, unlock_url);
-}
-
-TEST_F(GaiaAuthFetcherTest, AccountDeletedError) {
-  std::string data = "Error=AccountDeleted\n";
-  GoogleServiceAuthError error =
-      GaiaAuthFetcher::GenerateAuthError(data, net::OK);
-  EXPECT_EQ(error.state(), GoogleServiceAuthError::ACCOUNT_DELETED);
-}
-
-TEST_F(GaiaAuthFetcherTest, AccountDisabledError) {
-  std::string data = "Error=AccountDisabled\n";
-  GoogleServiceAuthError error =
-      GaiaAuthFetcher::GenerateAuthError(data, net::OK);
-  EXPECT_EQ(error.state(), GoogleServiceAuthError::ACCOUNT_DISABLED);
+  RunErrorParsingTest(
+      "Url=U\n"
+      "Error=E\n",
+      "E", "U");
+  RunErrorParsingTest(
+      "Error=E\n"
+      "Url=U\n",
+      "E", "U");
+  RunErrorParsingTest(
+      "\n\n\nError=E\n"
+      "\nUrl=U\n",
+      "E", "U");
 }
 
 TEST_F(GaiaAuthFetcherTest, BadAuthenticationError) {
@@ -304,6 +250,13 @@ TEST_F(GaiaAuthFetcherTest, BadAuthenticationError) {
   EXPECT_EQ(error.state(), GoogleServiceAuthError::INVALID_GAIA_CREDENTIALS);
 }
 
+TEST_F(GaiaAuthFetcherTest, BadAuthenticationShortError) {
+  std::string data = "Error=badauth\n";
+  GoogleServiceAuthError error =
+      GaiaAuthFetcher::GenerateAuthError(data, net::OK);
+  EXPECT_EQ(error.state(), GoogleServiceAuthError::INVALID_GAIA_CREDENTIALS);
+}
+
 TEST_F(GaiaAuthFetcherTest, IncomprehensibleError) {
   std::string data = "Error=Gobbledygook\n";
   GoogleServiceAuthError error =
@@ -318,6 +271,13 @@ TEST_F(GaiaAuthFetcherTest, ServiceUnavailableError) {
   EXPECT_EQ(error.state(), GoogleServiceAuthError::SERVICE_UNAVAILABLE);
 }
 
+TEST_F(GaiaAuthFetcherTest, ServiceUnavailableShortError) {
+  std::string data = "Error=ire\n";
+  GoogleServiceAuthError error =
+      GaiaAuthFetcher::GenerateAuthError(data, net::OK);
+  EXPECT_EQ(error.state(), GoogleServiceAuthError::SERVICE_UNAVAILABLE);
+}
+
 TEST_F(GaiaAuthFetcherTest, StartAuthCodeForOAuth2TokenExchange_Success) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnClientOAuthCode("test-code")).Times(0);
@@ -327,29 +287,30 @@ TEST_F(GaiaAuthFetcherTest, StartAuthCodeForOAuth2TokenExchange_Success) {
                   false /* is_advanced_protection */)))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.StartAuthCodeForOAuth2TokenExchange("auth_code");
   ASSERT_EQ(received_requests_.size(), 1U);
-  EXPECT_EQ(net::LOAD_DO_NOT_SEND_COOKIES | net::LOAD_DO_NOT_SAVE_COOKIES,
-            received_requests_.at(0).load_flags);
+  EXPECT_EQ(network::mojom::CredentialsMode::kOmit,
+            received_requests_.at(0).credentials_mode);
   std::string body = GetRequestBodyAsString(&received_requests_.at(0));
   EXPECT_EQ(std::string::npos, body.find("device_type=chrome"));
   EXPECT_TRUE(auth.HasPendingFetch());
 
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {},
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK,
                                      kGetTokenPairValidResponse);
   EXPECT_FALSE(auth.HasPendingFetch());
 }
 
 TEST_F(GaiaAuthFetcherTest, StartAuthCodeForOAuth2TokenExchange_DeviceId) {
   MockGaiaConsumer consumer;
-  GaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  GaiaAuthFetcher auth(&consumer, gaia::GaiaSource::kChrome,
+                       GetURLLoaderFactory());
   auth.StartAuthCodeForOAuth2TokenExchangeWithDeviceId("auth_code",
                                                        "device_ABCDE_1");
 
   ASSERT_EQ(1U, received_requests_.size());
-  EXPECT_EQ(net::LOAD_DO_NOT_SEND_COOKIES | net::LOAD_DO_NOT_SAVE_COOKIES,
-            received_requests_.at(0).load_flags);
+  EXPECT_EQ(network::mojom::CredentialsMode::kOmit,
+            received_requests_.at(0).credentials_mode);
   std::string body = GetRequestBodyAsString(&received_requests_.at(0));
   EXPECT_NE(std::string::npos, body.find("device_type=chrome"));
   EXPECT_NE(std::string::npos, body.find("device_id=device_ABCDE_1"));
@@ -359,7 +320,7 @@ TEST_F(GaiaAuthFetcherTest, StartAuthCodeForOAuth2TokenExchange_Failure) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnClientOAuthFailure(_)).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.StartAuthCodeForOAuth2TokenExchange("auth_code");
   EXPECT_TRUE(auth.HasPendingFetch());
 
@@ -372,60 +333,133 @@ TEST_F(GaiaAuthFetcherTest, MergeSessionSuccess) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnMergeSessionSuccess("<html></html>")).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.StartMergeSession("myubertoken", std::string());
 
   EXPECT_TRUE(auth.HasPendingFetch());
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {},
-                                     "<html></html>");
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, "<html></html>");
 
   EXPECT_FALSE(auth.HasPendingFetch());
 }
 
+TEST_F(GaiaAuthFetcherTest, MultiloginRequestFormat) {
+  MockGaiaConsumer consumer;
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  std::vector<GaiaAuthFetcher::MultiloginTokenIDPair> accounts;
+  accounts.push_back({"token1", "id1"});
+  accounts.push_back({"token2", "id2"});
+  auth.StartOAuthMultilogin(
+      gaia::MultiloginMode::MULTILOGIN_UPDATE_COOKIE_ACCOUNTS_ORDER, accounts,
+      "cc_result");
+  ASSERT_TRUE(auth.HasPendingFetch());
+
+  const network::ResourceRequest& request0 = received_requests_.at(0);
+  EXPECT_EQ("POST", request0.method);
+  std::string header;
+  request0.headers.GetHeader("Authorization", &header);
+  EXPECT_EQ("MultiBearer id1:token1,id2:token2", header);
+  EXPECT_EQ("source=ChromiumBrowser&reuseCookies=0&externalCcResult=cc_result",
+            request0.url.query());
+
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, std::string());
+  EXPECT_FALSE(auth.HasPendingFetch());
+
+  auth.StartOAuthMultilogin(
+      gaia::MultiloginMode::MULTILOGIN_PRESERVE_COOKIE_ACCOUNTS_ORDER, accounts,
+      "cc_result");
+  ASSERT_TRUE(auth.HasPendingFetch());
+
+  const network::ResourceRequest& request1 = received_requests_.at(1);
+  EXPECT_EQ("source=ChromiumBrowser&reuseCookies=1&externalCcResult=cc_result",
+            request1.url.query());
+}
+
 TEST_F(GaiaAuthFetcherTest, MultiloginSuccess) {
   MockGaiaConsumer consumer;
-  EXPECT_CALL(consumer, OnOAuthMultiloginSuccess(::testing::_)).Times(1);
+  EXPECT_CALL(consumer, OnOAuthMultiloginFinished(::testing::Property(
+                            &OAuthMultiloginResult::status,
+                            ::testing::Eq(OAuthMultiloginResponseStatus::kOk))))
+      .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.StartOAuthMultilogin(
-      std::vector<GaiaAuthFetcher::MultiloginTokenIDPair>());
+      gaia::MultiloginMode::MULTILOGIN_UPDATE_COOKIE_ACCOUNTS_ORDER,
+      std::vector<GaiaAuthFetcher::MultiloginTokenIDPair>(), std::string());
 
   EXPECT_TRUE(auth.HasPendingFetch());
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {},
-                                     "\n{\"status\": \"OK\"}");
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK,
+                                     R"()]}'
+        {
+          "status": "OK",
+          "cookies":[
+            {
+              "name":"SID",
+              "value":"vAlUe1",
+              "domain":".google.ru",
+              "path":"/",
+              "isSecure":true,
+              "isHttpOnly":false,
+              "priority":"HIGH",
+              "maxAge":63070000
+            }
+          ]
+        }
+      )");
 
   EXPECT_FALSE(auth.HasPendingFetch());
 }
 
 TEST_F(GaiaAuthFetcherTest, MultiloginFailureNetError) {
   MockGaiaConsumer consumer;
-  EXPECT_CALL(consumer, OnOAuthMultiloginFailure(GoogleServiceAuthError(
-                            GoogleServiceAuthError::REQUEST_CANCELED)))
+  EXPECT_CALL(consumer,
+              OnOAuthMultiloginFinished(::testing::Property(
+                  &OAuthMultiloginResult::status,
+                  ::testing::Eq(OAuthMultiloginResponseStatus::kRetry))))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.StartOAuthMultilogin(
-      std::vector<GaiaAuthFetcher::MultiloginTokenIDPair>());
+      gaia::MultiloginMode::MULTILOGIN_UPDATE_COOKIE_ACCOUNTS_ORDER,
+      std::vector<GaiaAuthFetcher::MultiloginTokenIDPair>(), std::string());
 
   EXPECT_TRUE(auth.HasPendingFetch());
-  auth.TestOnURLLoadCompleteInternal(net::ERR_ABORTED, net::HTTP_OK, {},
-                                     "\n{\"status\": \"OK\"}");
+  auth.TestOnURLLoadCompleteInternal(net::ERR_ABORTED, net::HTTP_OK,
+                                     R"()]}'
+        {
+          "status": "OK",
+          "cookies":[
+            {
+              "name":"SID",
+              "value":"vAlUe1",
+              "domain":".google.ru",
+              "path":"/",
+              "isSecure":true,
+              "isHttpOnly":false,
+              "priority":"HIGH",
+              "maxAge":63070000
+            }
+          ]
+        }
+      )");
 
   EXPECT_FALSE(auth.HasPendingFetch());
 }
 
 TEST_F(GaiaAuthFetcherTest, MultiloginFailureServerError) {
   MockGaiaConsumer consumer;
-  EXPECT_CALL(consumer, OnOAuthMultiloginFailure(GoogleServiceAuthError(
-                            GoogleServiceAuthError::SERVICE_ERROR)))
+  EXPECT_CALL(consumer,
+              OnOAuthMultiloginFinished(::testing::Property(
+                  &OAuthMultiloginResult::status,
+                  ::testing::Eq(OAuthMultiloginResponseStatus::kError))))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.StartOAuthMultilogin(
-      std::vector<GaiaAuthFetcher::MultiloginTokenIDPair>());
+      gaia::MultiloginMode::MULTILOGIN_UPDATE_COOKIE_ACCOUNTS_ORDER,
+      std::vector<GaiaAuthFetcher::MultiloginTokenIDPair>(), std::string());
 
   EXPECT_TRUE(auth.HasPendingFetch());
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {},
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK,
                                      "\n{\"status\": \"ERROR\"}");
 
   EXPECT_FALSE(auth.HasPendingFetch());
@@ -435,19 +469,17 @@ TEST_F(GaiaAuthFetcherTest, UberAuthTokenSuccess) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnUberAuthTokenSuccess("uberToken")).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
-  auth.StartTokenFetchForUberAuthExchange("myAccessToken",
-                                          true /* is_bound_to_channel_id */);
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.StartTokenFetchForUberAuthExchange("myAccessToken");
 
   EXPECT_TRUE(auth.HasPendingFetch());
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {}, "uberToken");
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, "uberToken");
 
   EXPECT_FALSE(auth.HasPendingFetch());
 }
 
 TEST_F(GaiaAuthFetcherTest, StartOAuthLogin) {
-  // OAuthLogin returns the same as the ClientLogin endpoint, minus CAPTCHA
-  // responses.
+  // OAuthLogin returns the same as the ClientLogin endpoint.
   std::string data("SID=sid\nLSID=lsid\nAuth=auth\n");
 
   GaiaAuthConsumer::ClientLoginResult result;
@@ -459,11 +491,11 @@ TEST_F(GaiaAuthFetcherTest, StartOAuthLogin) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnClientLoginSuccess(result)).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
-  auth.CreateAndStartGaiaFetcherForTesting(/*body=*/"", /*headers=*/"",
-                                           oauth_login_gurl_, /*load_flags=*/0,
-                                           NO_TRAFFIC_ANNOTATION_YET);
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {}, data);
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", oauth_login_gurl_,
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, data);
 }
 
 TEST_F(GaiaAuthFetcherTest, ListAccounts) {
@@ -478,44 +510,47 @@ TEST_F(GaiaAuthFetcherTest, ListAccounts) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnListAccountsSuccess(data)).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"",
-      GaiaUrls::GetInstance()->ListAccountsURLWithSource(std::string()),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      GaiaUrls::GetInstance()->ListAccountsURLWithSource(
+          GaiaConstants::kChromeSource),
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   ASSERT_EQ(received_requests_.size(), 1U);
-  EXPECT_EQ(net::LOAD_NORMAL, received_requests_.at(0).load_flags);
-  EXPECT_EQ(GaiaUrls::GetInstance()->gaia_url(),
-            received_requests_.at(0).site_for_cookies);
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {}, data);
+  EXPECT_EQ(network::mojom::CredentialsMode::kInclude,
+            received_requests_.at(0).credentials_mode);
+  EXPECT_TRUE(received_requests_.at(0).site_for_cookies.IsEquivalent(
+      net::SiteForCookies::FromUrl(GaiaUrls::GetInstance()->gaia_url())));
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, data);
 }
 
 TEST_F(GaiaAuthFetcherTest, LogOutSuccess) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnLogOutSuccess()).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"",
-      GaiaUrls::GetInstance()->LogOutURLWithSource(std::string()),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      GaiaUrls::GetInstance()->LogOutURLWithSource(
+          GaiaConstants::kChromeSource),
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   auth.TestOnURLLoadCompleteInternal(net::OK);
 }
 
 TEST_F(GaiaAuthFetcherTest, LogOutFailure) {
   net::Error error_no = net::ERR_CONNECTION_RESET;
-  net::URLRequestStatus status(net::URLRequestStatus::FAILED, error_no);
 
   GoogleServiceAuthError expected_error =
       GoogleServiceAuthError::FromConnectionError(error_no);
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnLogOutFailure(expected_error)).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"",
-      GaiaUrls::GetInstance()->LogOutURLWithSource(std::string()),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      GaiaUrls::GetInstance()->LogOutURLWithSource(
+          GaiaConstants::kChromeSource),
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   auth.TestOnURLLoadCompleteInternal(error_no);
 }
 
@@ -525,14 +560,14 @@ TEST_F(GaiaAuthFetcherTest, GetCheckConnectionInfo) {
   MockGaiaConsumer consumer;
   EXPECT_CALL(consumer, OnGetCheckConnectionInfoSuccess(data)).Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
 
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"",
       GaiaUrls::GetInstance()->GetCheckConnectionInfoURLWithSource(
-          std::string()),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {}, data);
+          GaiaConstants::kChromeSource),
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, data);
 }
 
 TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenSuccess) {
@@ -542,11 +577,11 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenSuccess) {
                             GaiaAuthConsumer::TokenRevocationStatus::kSuccess))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, {}, data);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, data);
 }
 
 TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenCanceled) {
@@ -556,10 +591,10 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenCanceled) {
                   GaiaAuthConsumer::TokenRevocationStatus::kConnectionCanceled))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   auth.TestOnURLLoadCompleteInternal(net::ERR_ABORTED);
 }
 
@@ -570,10 +605,10 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenFailed) {
                   GaiaAuthConsumer::TokenRevocationStatus::kConnectionFailed))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   auth.TestOnURLLoadCompleteInternal(net::ERR_CERT_CONTAINS_ERRORS);
 }
 
@@ -584,10 +619,10 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenTimeout) {
                   GaiaAuthConsumer::TokenRevocationStatus::kConnectionTimeout))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   auth.TestOnURLLoadCompleteInternal(net::ERR_TIMED_OUT);
 }
 
@@ -599,11 +634,11 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenInvalidToken) {
                   GaiaAuthConsumer::TokenRevocationStatus::kInvalidToken))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_BAD_REQUEST, {}, data);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_BAD_REQUEST, data);
 }
 
 TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenInvalidRequest) {
@@ -614,11 +649,11 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenInvalidRequest) {
                   GaiaAuthConsumer::TokenRevocationStatus::kInvalidRequest))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
-  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_BAD_REQUEST, {}, data);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_BAD_REQUEST, data);
 }
 
 TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenServerError) {
@@ -629,10 +664,98 @@ TEST_F(GaiaAuthFetcherTest, RevokeOAuth2TokenServerError) {
                   GaiaAuthConsumer::TokenRevocationStatus::kServerError))
       .Times(1);
 
-  TestGaiaAuthFetcher auth(&consumer, std::string(), GetURLLoaderFactory());
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
   auth.CreateAndStartGaiaFetcherForTesting(
       /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->oauth2_revoke_url(),
-      /*load_flags=*/0, NO_TRAFFIC_ANNOTATION_YET);
+      network::mojom::CredentialsMode::kInclude, TRAFFIC_ANNOTATION_FOR_TESTS);
   auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_INTERNAL_SERVER_ERROR,
-                                     {}, data);
+                                     data);
+}
+
+TEST_F(GaiaAuthFetcherTest, ReAuthTokenSuccess) {
+  std::string data("{\"encodedRapt\" : \"rapt_token\"}");
+  MockGaiaConsumer consumer;
+  EXPECT_CALL(consumer, OnReAuthProofTokenSuccess("rapt_token")).Times(1);
+
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->reauth_api_url(),
+      network::mojom::CredentialsMode ::kOmit, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_OK, data);
+}
+
+TEST_F(GaiaAuthFetcherTest, ReAuthTokenInvalidRequest) {
+  std::string data("{\"error\" : { \"message\" :  \"INVALID_REQUEST\"} }");
+  MockGaiaConsumer consumer;
+  EXPECT_CALL(consumer,
+              OnReAuthProofTokenFailure(
+                  GaiaAuthConsumer::ReAuthProofTokenStatus::kInvalidRequest))
+      .Times(1);
+
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->reauth_api_url(),
+      network::mojom::CredentialsMode ::kOmit, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_BAD_REQUEST, data);
+}
+
+TEST_F(GaiaAuthFetcherTest, ReAuthTokenInvalidGrant) {
+  std::string data("{\"error\" : { \"message\" :  \"INVALID_GRANT\"} }");
+  MockGaiaConsumer consumer;
+  EXPECT_CALL(consumer,
+              OnReAuthProofTokenFailure(
+                  GaiaAuthConsumer::ReAuthProofTokenStatus::kInvalidGrant))
+      .Times(1);
+
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->reauth_api_url(),
+      network::mojom::CredentialsMode ::kOmit, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_BAD_REQUEST, data);
+}
+
+TEST_F(GaiaAuthFetcherTest, ReAuthTokenUnauthorizedClient) {
+  std::string data("{\"error\" : { \"message\" :  \"UNAUTHORIZED_CLIENT\"} }");
+  MockGaiaConsumer consumer;
+  EXPECT_CALL(
+      consumer,
+      OnReAuthProofTokenFailure(
+          GaiaAuthConsumer::ReAuthProofTokenStatus::kUnauthorizedClient))
+      .Times(1);
+
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->reauth_api_url(),
+      network::mojom::CredentialsMode ::kOmit, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_FORBIDDEN, data);
+}
+
+TEST_F(GaiaAuthFetcherTest, ReAuthTokenInsufficientScope) {
+  std::string data("{\"error\" : { \"message\" :  \"INSUFFICIENT_SCOPE\"} }");
+  MockGaiaConsumer consumer;
+  EXPECT_CALL(consumer,
+              OnReAuthProofTokenFailure(
+                  GaiaAuthConsumer::ReAuthProofTokenStatus::kInsufficientScope))
+      .Times(1);
+
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->reauth_api_url(),
+      network::mojom::CredentialsMode ::kOmit, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_FORBIDDEN, data);
+}
+
+TEST_F(GaiaAuthFetcherTest, ReAuthTokenCredentialNotSet) {
+  std::string data("{\"error\" : { \"message\" :  \"CREDENTIAL_NOT_SET\"} }");
+  MockGaiaConsumer consumer;
+  EXPECT_CALL(consumer,
+              OnReAuthProofTokenFailure(
+                  GaiaAuthConsumer::ReAuthProofTokenStatus::kCredentialNotSet))
+      .Times(1);
+
+  TestGaiaAuthFetcher auth(&consumer, GetURLLoaderFactory());
+  auth.CreateAndStartGaiaFetcherForTesting(
+      /*body=*/"", /*headers=*/"", GaiaUrls::GetInstance()->reauth_api_url(),
+      network::mojom::CredentialsMode ::kOmit, TRAFFIC_ANNOTATION_FOR_TESTS);
+  auth.TestOnURLLoadCompleteInternal(net::OK, net::HTTP_FORBIDDEN, data);
 }
