@@ -7,9 +7,11 @@
 #include <memory>
 #include <utility>
 
+#include "base/bind.h"
+#include "base/check_op.h"
 #include "base/json/json_reader.h"
-#include "base/logging.h"
 #include "base/memory/weak_ptr.h"
+#include "base/notreached.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
 #include "google_apis/gaia/gaia_auth_util.h"
@@ -41,9 +43,8 @@ class GaiaOAuthClient::Core
         num_retries_(0),
         max_retries_(0),
         url_loader_factory_(url_loader_factory),
-        delegate_(NULL),
-        request_type_(NO_PENDING_REQUEST),
-        weak_ptr_factory_(this) {
+        delegate_(nullptr),
+        request_type_(NO_PENDING_REQUEST) {
     backoff_policy_.num_errors_to_ignore =
         net::URLRequestThrottlerEntry::kDefaultNumErrorsToIgnore;
     backoff_policy_.initial_delay_ms =
@@ -141,7 +142,7 @@ class GaiaOAuthClient::Core
   std::unique_ptr<network::SimpleURLLoader> request_;
   RequestType request_type_;
 
-  base::WeakPtrFactory<Core> weak_ptr_factory_;
+  base::WeakPtrFactory<Core> weak_ptr_factory_{this};
 };
 
 void GaiaOAuthClient::Core::GetTokensFromAuthCode(
@@ -386,8 +387,7 @@ void GaiaOAuthClient::Core::SendRequestImpl() {
   auto resource_request = std::make_unique<network::ResourceRequest>();
   resource_request->url = url_;
   resource_request->method = post_body_.empty() ? "GET" : "POST";
-  resource_request->load_flags =
-      net::LOAD_DO_NOT_SEND_COOKIES | net::LOAD_DO_NOT_SAVE_COOKIES;
+  resource_request->credentials_mode = network::mojom::CredentialsMode::kOmit;
   if (!authorization_header_.empty())
     resource_request->headers.SetHeader("Authorization", authorization_header_);
 
@@ -403,10 +403,6 @@ void GaiaOAuthClient::Core::SendRequestImpl() {
   // Retry is implemented internally.
   request_->SetRetryOptions(0, network::SimpleURLLoader::RETRY_NEVER);
 
-  // TODO(https://crbug.com/808498) re-add data use measurement once
-  // SimpleURLLoader supports it. Previous way of setting it was:
-  // MarkURLFetcherAsGaia(request_.get());
-
   request_->DownloadToStringOfUnboundedSizeUntilCrashAndDie(
       url_loader_factory_.get(),
       // Unretained(this) is safe since |this| owns |request_|, and its deletion
@@ -455,7 +451,8 @@ void GaiaOAuthClient::Core::HandleResponse(std::unique_ptr<std::string> body,
   std::unique_ptr<base::DictionaryValue> response_dict;
   if (response_code == net::HTTP_OK && body) {
     std::string data = std::move(*body);
-    std::unique_ptr<base::Value> message_value = base::JSONReader::Read(data);
+    std::unique_ptr<base::Value> message_value =
+        base::JSONReader::ReadDeprecated(data);
     if (message_value.get() && message_value->is_dict()) {
       response_dict.reset(
           static_cast<base::DictionaryValue*>(message_value.release()));
