@@ -7,26 +7,29 @@
 
 namespace switches {
 
-// Supplies custom client login to OAuth2 URL for testing purposes.
-extern const char kClientLoginToOAuth2Url[];
+// Specifies the path to a config file containing GAIA urls.
+// See "google_apis/test/data/gaia/all_urls.json" for a format example.
+extern const char kGaiaConfig[];
+
+// Specifies the domain of the SAPISID cookie. The default value is
+// "http://.google.com".
+extern const char kGoogleUrl[];
 
 // Specifies the path for GAIA authentication URL. The default value is
 // "https://accounts.google.com".
 extern const char kGaiaUrl[];
 
 // Specifies the backend server used for Google API calls.
-// The default value is "https://www.googleapis.com".
+// "https://www.googleapis.com".
 extern const char kGoogleApisUrl[];
 
 // Specifies the backend server used for lso authentication calls.
 // "https://accounts.google.com".
 extern const char kLsoUrl[];
 
-// Specifies custom OAuth1 login scope for testing purposes.
-extern const char kOAuth1LoginScope[];
-
-// Overrides OAuth wrap bridge user info scope.
-extern const char kOAuthWrapBridgeUserInfoScope[];
+// Specifies the backend server used for OAuth issue token calls.
+// "https://oauthaccountmanager.googleapis.com".
+extern const char kOAuthAccountManagerUrl[];
 
 // Specifies custom OAuth2 client id for testing purposes.
 extern const char kOAuth2ClientID[];
