@@ -6,6 +6,9 @@
 
 #include <string>
 
+#include "base/bind.h"
+#include "base/threading/thread_task_runner_handle.h"
+#include "build/chromeos_buildflags.h"
 #include "google_apis/gcm/engine/checkin_request.h"
 #include "google_apis/gcm/engine/gcm_request_test_base.h"
 #include "google_apis/gcm/monitoring/fake_gcm_stats_recorder.h"
@@ -97,8 +100,9 @@ void CheckinRequestTest::CreateRequest(uint64_t android_id,
   // security_token.
   request_.reset(new CheckinRequest(
       GURL(kCheckinURL), request_info, GetBackoffPolicy(),
-      base::Bind(&CheckinRequestTest::FetcherCallback, base::Unretained(this)),
-      url_loader_factory(), &recorder_));
+      base::BindOnce(&CheckinRequestTest::FetcherCallback,
+                     base::Unretained(this)),
+      url_loader_factory(), base::ThreadTaskRunnerHandle::Get(), &recorder_));
 
   // Setting android_id_ and security_token_ to blank value, not used elsewhere
   // in the tests.
@@ -150,7 +154,7 @@ TEST_F(CheckinRequestTest, FetcherDataAndURL) {
   EXPECT_EQ(kEmailAddress, request_proto.account_cookie(0));
   EXPECT_EQ(kTokenValue, request_proto.account_cookie(1));
 
-#if defined(CHROME_OS)
+#if BUILDFLAG(IS_CHROMEOS_ASH)
   EXPECT_EQ(checkin_proto::DEVICE_CHROME_OS, request_proto.checkin().type());
 #else
   EXPECT_EQ(checkin_proto::DEVICE_CHROME_BROWSER,
