@@ -8,21 +8,23 @@
 #include <memory>
 #include <utility>
 
-#include "base/message_loop/message_loop.h"
+#include "base/bind.h"
 #include "base/optional.h"
 #include "base/run_loop.h"
-#include "base/test/scoped_task_environment.h"
 #include "base/test/simple_test_tick_clock.h"
+#include "base/test/task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "google_apis/gcm/base/mcs_util.h"
 #include "google_apis/gcm/engine/fake_connection_handler.h"
 #include "google_apis/gcm/monitoring/fake_gcm_stats_recorder.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
 #include "net/base/backoff_entry.h"
 #include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
 #include "net/url_request/url_request_test_util.h"
 #include "services/network/network_context.h"
 #include "services/network/network_service.h"
 #include "services/network/public/mojom/proxy_resolving_socket.mojom.h"
+#include "services/network/test/fake_test_cert_verifier_params_factory.h"
 #include "services/network/test/test_network_connection_tracker.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -94,7 +96,7 @@ class TestConnectionFactoryImpl : public ConnectionFactoryImpl {
  public:
   TestConnectionFactoryImpl(
       GetProxyResolvingFactoryCallback get_socket_factory_callback,
-      const base::Closure& finished_callback);
+      base::RepeatingClosure finished_callback);
   ~TestConnectionFactoryImpl() override;
 
   void InitializeFactory();
@@ -139,7 +141,7 @@ class TestConnectionFactoryImpl : public ConnectionFactoryImpl {
   // Whether to delay a login handshake completion or not.
   bool delay_login_;
   // Callback to invoke when all connection attempts have been made.
-  base::Closure finished_callback_;
+  base::RepeatingClosure finished_callback_;
   // A temporary scoped pointer to make sure we don't leak the handler in the
   // cases it's never consumed by the ConnectionFactory.
   std::unique_ptr<FakeConnectionHandler> scoped_handler_;
@@ -148,17 +150,20 @@ class TestConnectionFactoryImpl : public ConnectionFactoryImpl {
   // Dummy GCM Stats recorder.
   FakeGCMStatsRecorder dummy_recorder_;
   // Dummy mojo pipes.
-  mojo::DataPipe receive_pipe_;
-  mojo::DataPipe send_pipe_;
+  mojo::ScopedDataPipeProducerHandle receive_pipe_producer_;
+  mojo::ScopedDataPipeConsumerHandle receive_pipe_consumer_;
+  mojo::ScopedDataPipeProducerHandle send_pipe_producer_;
+  mojo::ScopedDataPipeConsumerHandle send_pipe_consumer_;
 };
 
 TestConnectionFactoryImpl::TestConnectionFactoryImpl(
     GetProxyResolvingFactoryCallback get_socket_factory_callback,
-    const base::Closure& finished_callback)
+    base::RepeatingClosure finished_callback)
     : ConnectionFactoryImpl(
           BuildEndpoints(),
           net::BackoffEntry::Policy(),
           get_socket_factory_callback,
+          base::ThreadTaskRunnerHandle::Get(),
           &dummy_recorder_,
           network::TestNetworkConnectionTracker::GetInstance()),
       connect_result_(net::ERR_UNEXPECTED),
@@ -167,11 +172,18 @@ TestConnectionFactoryImpl::TestConnectionFactoryImpl(
       delay_login_(false),
       finished_callback_(finished_callback),
       scoped_handler_(std::make_unique<FakeConnectionHandler>(
-          base::Bind(&ReadContinuation),
-          base::Bind(&WriteContinuation))),
+          base::BindRepeating(&ReadContinuation),
+          base::BindRepeating(&WriteContinuation))),
       fake_handler_(scoped_handler_.get()) {
   // Set a non-null time.
   tick_clock_.Advance(base::TimeDelta::FromMilliseconds(1));
+
+  EXPECT_EQ(mojo::CreateDataPipe(nullptr, receive_pipe_producer_,
+                                 receive_pipe_consumer_),
+            MOJO_RESULT_OK);
+  EXPECT_EQ(
+      mojo::CreateDataPipe(nullptr, send_pipe_producer_, send_pipe_consumer_),
+      MOJO_RESULT_OK);
 }
 
 TestConnectionFactoryImpl::~TestConnectionFactoryImpl() {
@@ -182,9 +194,8 @@ void TestConnectionFactoryImpl::StartConnection() {
   ASSERT_GT(num_expected_attempts_, 0);
   ASSERT_FALSE(GetConnectionHandler()->CanSendMessage());
   std::unique_ptr<mcs_proto::LoginRequest> request(BuildLoginRequest(0, 0, ""));
-  GetConnectionHandler()->Init(*request,
-                               std::move(receive_pipe_.consumer_handle),
-                               std::move(send_pipe_.producer_handle));
+  GetConnectionHandler()->Init(*request, std::move(receive_pipe_consumer_),
+                               std::move(send_pipe_producer_));
   OnConnectDone(connect_result_, net::IPEndPoint(), net::IPEndPoint(),
                 mojo::ScopedDataPipeConsumerHandle(),
                 mojo::ScopedDataPipeProducerHandle());
@@ -290,45 +301,55 @@ class ConnectionFactoryImplTest
     return login_request.client_event();
   }
 
+  base::RunLoop* GetRunLoop() { return run_loop_.get(); }
+
  private:
   void GetProxyResolvingSocketFactory(
-      network::mojom::ProxyResolvingSocketFactoryRequest request) {
-    network_context_->CreateProxyResolvingSocketFactory(std::move(request));
+      mojo::PendingReceiver<network::mojom::ProxyResolvingSocketFactory>
+          receiver) {
+    network_context_->CreateProxyResolvingSocketFactory(std::move(receiver));
   }
   void ConnectionsComplete();
 
   std::unique_ptr<network::TestNetworkConnectionTracker>
       network_connection_tracker_;
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   TestConnectionFactoryImpl factory_;
   std::unique_ptr<base::RunLoop> run_loop_;
 
   GURL connected_server_;
+  std::unique_ptr<net::NetworkChangeNotifier> network_change_notifier_;
   std::unique_ptr<network::NetworkService> network_service_;
-  network::mojom::NetworkContextPtr network_context_ptr_;
+  mojo::Remote<network::mojom::NetworkContext> network_context_remote_;
   std::unique_ptr<network::NetworkContext> network_context_;
 };
 
 ConnectionFactoryImplTest::ConnectionFactoryImplTest()
     : network_connection_tracker_(
           network::TestNetworkConnectionTracker::CreateInstance()),
-      scoped_task_environment_(
-          base::test::ScopedTaskEnvironment::MainThreadType::IO),
-      factory_(base::BindRepeating(
-                   &ConnectionFactoryImplTest::GetProxyResolvingSocketFactory,
-                   base::Unretained(this)),
-               base::Bind(&ConnectionFactoryImplTest::ConnectionsComplete,
-                          base::Unretained(this))),
+      task_environment_(base::test::TaskEnvironment::MainThreadType::IO),
+      factory_(
+          base::BindRepeating(
+              &ConnectionFactoryImplTest::GetProxyResolvingSocketFactory,
+              base::Unretained(this)),
+          base::BindRepeating(&ConnectionFactoryImplTest::ConnectionsComplete,
+                              base::Unretained(this))),
       run_loop_(new base::RunLoop()),
+      network_change_notifier_(
+          net::NetworkChangeNotifier::CreateMockIfNeeded()),
       network_service_(network::NetworkService::CreateForTesting()) {
   network::mojom::NetworkContextParamsPtr params =
       network::mojom::NetworkContextParams::New();
+  // Use a dummy CertVerifier that always passes cert verification, since
+  // these unittests don't need to test CertVerifier behavior.
+  params->cert_verifier_params =
+      network::FakeTestCertVerifierParamsFactory::GetCertVerifierParams();
   // Use a fixed proxy config, to avoid dependencies on local network
   // configuration.
   params->initial_proxy_config = net::ProxyConfigWithAnnotation::CreateDirect();
   network_context_ = std::make_unique<network::NetworkContext>(
-      network_service_.get(), mojo::MakeRequest(&network_context_ptr_),
-      std::move(params));
+      network_service_.get(),
+      network_context_remote_.BindNewPipeAndPassReceiver(), std::move(params));
   factory()->SetConnectionListener(this);
   factory()->Initialize(ConnectionFactory::BuildLoginRequestCallback(),
                         ConnectionHandler::ProtoReceivedCallback(),
@@ -503,7 +524,7 @@ TEST_F(ConnectionFactoryImplTest, CanarySucceedsRetryDuringLogin) {
 
   // Pump the loop, to ensure the pending backoff retry has no effect.
   base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE, base::RunLoop::QuitCurrentWhenIdleClosureDeprecated(),
+      FROM_HERE, GetRunLoop()->QuitWhenIdleClosure(),
       base::TimeDelta::FromMilliseconds(1));
   WaitForConnections();
 }
