@@ -11,19 +11,23 @@
 #include <utility>
 
 #include "base/bind.h"
-#include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/test/bind_test_util.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/bind.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_timeouts.h"
+#include "base/threading/thread_task_runner_handle.h"
+#include "build/build_config.h"
 #include "google/protobuf/io/coded_stream.h"
 #include "google/protobuf/io/zero_copy_stream_impl_lite.h"
 #include "google/protobuf/wire_format_lite.h"
 #include "google_apis/gcm/base/mcs_util.h"
 #include "google_apis/gcm/base/socket_stream.h"
 #include "google_apis/gcm/protocol/mcs.pb.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "net/base/ip_address.h"
+#include "net/base/network_isolation_key.h"
 #include "net/base/test_completion_callback.h"
 #include "net/log/net_log_source.h"
 #include "net/socket/socket_test_util.h"
@@ -35,6 +39,7 @@
 #include "services/network/network_service.h"
 #include "services/network/public/mojom/proxy_resolving_socket.mojom.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "url/origin.h"
 
 namespace gcm {
 namespace {
@@ -170,7 +175,7 @@ class GCMConnectionHandlerImplTest : public testing::Test {
   // Runs the message loop until a message is received.
   void WaitForMessage();
 
-  network::mojom::ProxyResolvingSocketPtr mojo_socket_ptr_;
+  mojo::Remote<network::mojom::ProxyResolvingSocket> mojo_socket_remote_;
 
  private:
   void ReadContinuation(ScopedMessage* dst_proto, ScopedMessage new_proto);
@@ -188,22 +193,25 @@ class GCMConnectionHandlerImplTest : public testing::Test {
   int last_error_;
 
   net::AddressList address_list_;
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   std::unique_ptr<base::RunLoop> run_loop_;
+  std::unique_ptr<net::NetworkChangeNotifier> network_change_notifier_;
   std::unique_ptr<network::NetworkService> network_service_;
-  network::mojom::NetworkContextPtr network_context_ptr_;
+  mojo::Remote<network::mojom::NetworkContext> network_context_remote_;
   net::MockClientSocketFactory socket_factory_;
   net::TestURLRequestContext url_request_context_;
   std::unique_ptr<network::NetworkContext> network_context_;
-  network::mojom::ProxyResolvingSocketFactoryPtr mojo_socket_factory_ptr_;
+  mojo::Remote<network::mojom::ProxyResolvingSocketFactory>
+      mojo_socket_factory_remote_;
   mojo::ScopedDataPipeConsumerHandle receive_pipe_handle_;
   mojo::ScopedDataPipeProducerHandle send_pipe_handle_;
 };
 
 GCMConnectionHandlerImplTest::GCMConnectionHandlerImplTest()
     : last_error_(0),
-      scoped_task_environment_(
-          base::test::ScopedTaskEnvironment::MainThreadType::IO),
+      task_environment_(base::test::TaskEnvironment::MainThreadType::IO),
+      network_change_notifier_(
+          net::NetworkChangeNotifier::CreateMockIfNeeded()),
       network_service_(network::NetworkService::CreateForTesting()),
       url_request_context_(true /* delay_initialization */) {
   address_list_ = net::AddressList::CreateFromIPAddress(
@@ -213,8 +221,10 @@ GCMConnectionHandlerImplTest::GCMConnectionHandlerImplTest()
   url_request_context_.Init();
 
   network_context_ = std::make_unique<network::NetworkContext>(
-      network_service_.get(), mojo::MakeRequest(&network_context_ptr_),
-      &url_request_context_);
+      network_service_.get(),
+      network_context_remote_.BindNewPipeAndPassReceiver(),
+      &url_request_context_,
+      /*cors_exempt_header_list=*/std::vector<std::string>());
 }
 
 GCMConnectionHandlerImplTest::~GCMConnectionHandlerImplTest() {
@@ -231,15 +241,25 @@ void GCMConnectionHandlerImplTest::BuildSocket(const ReadList& read_list,
 
   run_loop_ = std::make_unique<base::RunLoop>();
 
+  mojo_socket_factory_remote_.reset();
   network_context_->CreateProxyResolvingSocketFactory(
-      mojo::MakeRequest(&mojo_socket_factory_ptr_));
+      mojo_socket_factory_remote_.BindNewPipeAndPassReceiver());
   base::RunLoop run_loop;
   int net_error = net::ERR_FAILED;
   const GURL kDestination("https://example.com");
-  mojo_socket_factory_ptr_->CreateProxyResolvingSocket(
-      kDestination, true /* use_tls */,
+  network::mojom::ProxyResolvingSocketOptionsPtr options =
+      network::mojom::ProxyResolvingSocketOptions::New();
+  options->use_tls = true;
+  mojo_socket_remote_.reset();
+  const url::Origin kOrigin = url::Origin::Create(kDestination);
+  mojo_socket_factory_remote_->CreateProxyResolvingSocket(
+      kDestination,
+      net::NetworkIsolationKey(kOrigin /* top_frame_origin */,
+                               kOrigin /* frame_origin */),
+      std::move(options),
       net::MutableNetworkTrafficAnnotationTag(TRAFFIC_ANNOTATION_FOR_TESTS),
-      mojo::MakeRequest(&mojo_socket_ptr_), nullptr /* observer */,
+      mojo_socket_remote_.BindNewPipeAndPassReceiver(),
+      mojo::NullRemote() /* observer */,
       base::BindLambdaForTesting(
           [&](int result, const base::Optional<net::IPEndPoint>& local_addr,
               const base::Optional<net::IPEndPoint>& peer_addr,
@@ -262,13 +282,13 @@ void GCMConnectionHandlerImplTest::PumpLoop() {
 void GCMConnectionHandlerImplTest::Connect(
     ScopedMessage* dst_proto) {
   connection_handler_ = std::make_unique<ConnectionHandlerImpl>(
-      TestTimeouts::tiny_timeout(),
-      base::Bind(&GCMConnectionHandlerImplTest::ReadContinuation,
-                 base::Unretained(this), dst_proto),
-      base::Bind(&GCMConnectionHandlerImplTest::WriteContinuation,
-                 base::Unretained(this)),
-      base::Bind(&GCMConnectionHandlerImplTest::ConnectionContinuation,
-                 base::Unretained(this)));
+      base::ThreadTaskRunnerHandle::Get(), TestTimeouts::tiny_timeout(),
+      base::BindRepeating(&GCMConnectionHandlerImplTest::ReadContinuation,
+                          base::Unretained(this), dst_proto),
+      base::BindRepeating(&GCMConnectionHandlerImplTest::WriteContinuation,
+                          base::Unretained(this)),
+      base::BindRepeating(&GCMConnectionHandlerImplTest::ConnectionContinuation,
+                          base::Unretained(this)));
   EXPECT_FALSE(connection_handler()->CanSendMessage());
   connection_handler_->Init(*BuildLoginRequest(kAuthId, kAuthToken, ""),
                             std::move(receive_pipe_handle_),
@@ -429,7 +449,13 @@ TEST_F(GCMConnectionHandlerImplTest, ReInit) {
 }
 
 // Verify that messages can be received after initialization.
-TEST_F(GCMConnectionHandlerImplTest, RecvMsg) {
+// Flaky on Linux (crbug.com/906093)
+#if defined(OS_LINUX) || defined(OS_CHROMEOS)
+#define MAYBE_RecvMsg DISABLED_RecvMsg
+#else
+#define MAYBE_RecvMsg RecvMsg
+#endif
+TEST_F(GCMConnectionHandlerImplTest, MAYBE_RecvMsg) {
   std::string handshake_request = EncodeHandshakeRequest();
   WriteList write_list(1, net::MockWrite(net::ASYNC,
                                          handshake_request.c_str(),
@@ -739,7 +765,7 @@ TEST_F(GCMConnectionHandlerImplTest, SendMsgSocketDisconnected) {
   WaitForMessage();  // The login send.
   WaitForMessage();  // The login response.
   EXPECT_TRUE(connection_handler()->CanSendMessage());
-  mojo_socket_ptr_.reset();
+  mojo_socket_remote_.reset();
   mcs_proto::DataMessageStanza data_message;
   data_message.set_from(kDataMsgFrom);
   data_message.set_category(kDataMsgCategory);
