@@ -15,16 +15,18 @@
 #include <vector>
 
 #include "base/at_exit.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/files/scoped_file.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
-#include "base/message_loop/message_loop.h"
+#include "base/message_loop/message_pump_type.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/task/task_scheduler/task_scheduler.h"
+#include "base/task/single_thread_task_executor.h"
+#include "base/task/thread_pool/thread_pool_instance.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/default_clock.h"
@@ -40,13 +42,15 @@
 #include "google_apis/gcm/engine/mcs_client.h"
 #include "google_apis/gcm/monitoring/fake_gcm_stats_recorder.h"
 #include "mojo/core/embedder/embedder.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "net/cert/cert_verifier.h"
 #include "net/dns/host_resolver.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_auth_preferences.h"
 #include "net/http/http_auth_scheme.h"
 #include "net/log/file_net_log_observer.h"
-#include "net/proxy_resolution/proxy_resolution_service.h"
+#include "net/proxy_resolution/configured_proxy_resolution_service.h"
 #include "net/url_request/url_request_context.h"
 #include "net/url_request/url_request_context_builder.h"
 #include "net/url_request/url_request_test_util.h"
@@ -55,8 +59,9 @@
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
 #include "services/network/public/mojom/proxy_resolving_socket.mojom.h"
 #include "services/network/public/mojom/url_loader_factory.mojom.h"
+#include "services/network/test/test_network_connection_tracker.h"
 
-#if defined(OS_MACOSX)
+#if defined(OS_MAC)
 #include "base/mac/scoped_nsautorelease_pool.h"
 #endif
 
@@ -149,6 +154,8 @@ class MyTestCertVerifier : public net::CertVerifier {
              net::CompletionOnceCallback callback,
              std::unique_ptr<Request>* out_req,
              const net::NetLogWithSource& net_log) override {
+    verify_result->Reset();
+    verify_result->verified_cert = params.certificate();
     return net::OK;
   }
   void SetConfig(const Config& config) override {}
@@ -164,7 +171,10 @@ class MCSProbeAuthPreferences : public net::HttpAuthPreferences {
   bool CanUseDefaultCredentials(const GURL& auth_origin) const override {
     return false;
   }
-  bool CanDelegate(const GURL& auth_origin) const override { return false; }
+  net::HttpAuth::DelegationType GetDelegationType(
+      const GURL& auth_origin) const override {
+    return net::HttpAuth::DelegationType::kNone;
+  }
 };
 
 class MCSProbe {
@@ -179,7 +189,8 @@ class MCSProbe {
 
  private:
   void RequestProxyResolvingSocketFactory(
-      network::mojom::ProxyResolvingSocketFactoryRequest request);
+      mojo::PendingReceiver<network::mojom::ProxyResolvingSocketFactory>
+          receiver);
   void CheckIn();
   void InitializeNetworkState();
 
@@ -202,12 +213,12 @@ class MCSProbe {
   int server_port_;
 
   // Network state.
+  std::unique_ptr<network::TestNetworkConnectionTracker>
+      network_connection_tracker_;
   std::unique_ptr<net::URLRequestContext> url_request_context_;
-  net::NetLog net_log_;
+  net::NetLog* net_log_;
   std::unique_ptr<net::FileNetLogObserver> logger_;
-  std::unique_ptr<net::HostResolver> host_resolver_;
   MCSProbeAuthPreferences http_auth_preferences_;
-  std::unique_ptr<net::HttpAuthHandlerFactory> http_auth_handler_factory_;
 
   FakeGCMStatsRecorder recorder_;
   std::unique_ptr<GCMStore> gcm_store_;
@@ -219,8 +230,8 @@ class MCSProbe {
   base::Thread file_thread_;
 
   std::unique_ptr<network::NetworkContext> network_context_;
-  network::mojom::NetworkContextPtr network_context_pipe_;
-  network::mojom::URLLoaderFactoryPtr url_loader_factory_;
+  mojo::Remote<network::mojom::NetworkContext> network_context_remote_;
+  mojo::Remote<network::mojom::URLLoaderFactory> url_loader_factory_;
   scoped_refptr<network::SharedURLLoaderFactory> shared_url_loader_factory_;
 
   std::unique_ptr<base::RunLoop> run_loop_;
@@ -232,7 +243,12 @@ MCSProbe::MCSProbe(const base::CommandLine& command_line)
       android_id_(0),
       secret_(0),
       server_port_(0),
+      network_connection_tracker_(
+          network::TestNetworkConnectionTracker::CreateInstance()),
+      net_log_(net::NetLog::Get()),
       file_thread_("FileThread") {
+  network_connection_tracker_->SetConnectionType(
+      network::mojom::ConnectionType::CONNECTION_ETHERNET);
   if (command_line.HasSwitch(kRMQFileName)) {
     gcm_store_path_ = command_line.GetSwitchValuePath(kRMQFileName);
   }
@@ -272,18 +288,19 @@ void MCSProbe::Start() {
       endpoints, kDefaultBackoffPolicy,
       base::BindRepeating(&MCSProbe::RequestProxyResolvingSocketFactory,
                           base::Unretained(this)),
-      &recorder_);
+      base::ThreadTaskRunnerHandle::Get(), &recorder_,
+      network_connection_tracker_.get());
   gcm_store_ = std::make_unique<GCMStoreImpl>(
-      gcm_store_path_, file_thread_.task_runner(),
-      std::make_unique<FakeEncryptor>());
+      gcm_store_path_, /*remove_account_mappings_with_email_key=*/true,
+      file_thread_.task_runner(), std::make_unique<FakeEncryptor>());
 
-  mcs_client_ =
-      std::make_unique<MCSClient>("probe", &clock_, connection_factory_.get(),
-                                  gcm_store_.get(), &recorder_);
+  mcs_client_ = std::make_unique<MCSClient>(
+      "probe", &clock_, connection_factory_.get(), gcm_store_.get(),
+      base::ThreadTaskRunnerHandle::Get(), &recorder_);
   run_loop_ = std::make_unique<base::RunLoop>();
-  gcm_store_->Load(GCMStore::CREATE_IF_MISSING,
-                   base::Bind(&MCSProbe::LoadCallback,
-                              base::Unretained(this)));
+  gcm_store_->Load(
+      GCMStore::CREATE_IF_MISSING,
+      base::BindOnce(&MCSProbe::LoadCallback, base::Unretained(this)));
   run_loop_->Run();
 }
 
@@ -293,19 +310,18 @@ void MCSProbe::LoadCallback(std::unique_ptr<GCMStore::LoadResult> load_result) {
     DVLOG(1) << "Presetting MCS id " << android_id_;
     load_result->device_android_id = android_id_;
     load_result->device_security_token = secret_;
-    gcm_store_->SetDeviceCredentials(android_id_,
-                                     secret_,
-                                     base::Bind(&MCSProbe::UpdateCallback,
-                                                base::Unretained(this)));
+    gcm_store_->SetDeviceCredentials(
+        android_id_, secret_,
+        base::BindOnce(&MCSProbe::UpdateCallback, base::Unretained(this)));
   } else {
     android_id_ = load_result->device_android_id;
     secret_ = load_result->device_security_token;
     DVLOG(1) << "Loaded MCS id " << android_id_;
   }
   mcs_client_->Initialize(
-      base::Bind(&MCSProbe::ErrorCallback, base::Unretained(this)),
-      base::Bind(&MessageReceivedCallback), base::Bind(&MessageSentCallback),
-      std::move(load_result));
+      base::BindRepeating(&MCSProbe::ErrorCallback, base::Unretained(this)),
+      base::BindRepeating(&MessageReceivedCallback),
+      base::BindRepeating(&MessageSentCallback), std::move(load_result));
 
   if (!android_id_ || !secret_) {
     DVLOG(1) << "Checkin to generate new MCS credentials.";
@@ -322,24 +338,22 @@ void MCSProbe::UpdateCallback(bool success) {
 void MCSProbe::InitializeNetworkState() {
   if (command_line_.HasSwitch(kLogFileSwitch)) {
     base::FilePath log_path = command_line_.GetSwitchValuePath(kLogFileSwitch);
-    logger_ = net::FileNetLogObserver::CreateUnbounded(log_path, nullptr);
     net::NetLogCaptureMode capture_mode =
-        net::NetLogCaptureMode::IncludeCookiesAndCredentials();
-    logger_->StartObserving(&net_log_, capture_mode);
+        net::NetLogCaptureMode::kIncludeSensitive;
+    logger_ = net::FileNetLogObserver::CreateUnbounded(log_path, capture_mode,
+                                                       nullptr);
+    logger_->StartObserving(net_log_);
   }
 
-  host_resolver_ = net::HostResolver::CreateDefaultResolver(&net_log_);
-  http_auth_handler_factory_ = net::HttpAuthHandlerRegistryFactory::Create(
-      host_resolver_.get(), &http_auth_preferences_,
-      std::vector<std::string>{net::kBasicAuthScheme});
-
   net::URLRequestContextBuilder builder;
-  builder.set_net_log(&net_log_);
-  builder.set_shared_host_resolver(host_resolver_.get());
-  builder.set_shared_http_auth_handler_factory(
-      http_auth_handler_factory_.get());
+  builder.set_net_log(net_log_);
+  builder.set_host_resolver(
+      net::HostResolver::CreateStandaloneResolver(net_log_));
+  builder.SetHttpAuthHandlerFactory(net::HttpAuthHandlerRegistryFactory::Create(
+      &http_auth_preferences_,
+      std::vector<std::string>{net::kBasicAuthScheme}));
   builder.set_proxy_resolution_service(
-      net::ProxyResolutionService::CreateDirect());
+      net::ConfiguredProxyResolutionService::CreateDirect());
 
   if (command_line_.HasSwitch(kIgnoreCertSwitch))
     builder.SetCertVerifier(std::make_unique<MyTestCertVerifier>());
@@ -348,14 +362,16 @@ void MCSProbe::InitializeNetworkState() {
 
   // Wrap it up with network service APIs.
   network_context_ = std::make_unique<network::NetworkContext>(
-      nullptr /* network_service */, mojo::MakeRequest(&network_context_pipe_),
-      url_request_context_.get());
+      nullptr /* network_service */,
+      network_context_remote_.BindNewPipeAndPassReceiver(),
+      url_request_context_.get(),
+      /*cors_exempt_header_list=*/std::vector<std::string>());
   auto url_loader_factory_params =
       network::mojom::URLLoaderFactoryParams::New();
   url_loader_factory_params->process_id = network::mojom::kBrowserProcessId;
   url_loader_factory_params->is_corb_enabled = false;
   network_context_->CreateURLLoaderFactory(
-      mojo::MakeRequest(&url_loader_factory_),
+      url_loader_factory_.BindNewPipeAndPassReceiver(),
       std::move(url_loader_factory_params));
   shared_url_loader_factory_ =
       base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
@@ -367,9 +383,10 @@ void MCSProbe::ErrorCallback() {
 }
 
 void MCSProbe::RequestProxyResolvingSocketFactory(
-    network::mojom::ProxyResolvingSocketFactoryRequest request) {
+    mojo::PendingReceiver<network::mojom::ProxyResolvingSocketFactory>
+        receiver) {
   return network_context_->CreateProxyResolvingSocketFactory(
-      std::move(request));
+      std::move(receiver));
 }
 
 void MCSProbe::CheckIn() {
@@ -387,8 +404,9 @@ void MCSProbe::CheckIn() {
 
   checkin_request_ = std::make_unique<CheckinRequest>(
       GServicesSettings().GetCheckinURL(), request_info, kDefaultBackoffPolicy,
-      base::Bind(&MCSProbe::OnCheckInCompleted, base::Unretained(this)),
-      shared_url_loader_factory_, &recorder_);
+      base::BindOnce(&MCSProbe::OnCheckInCompleted, base::Unretained(this)),
+      shared_url_loader_factory_, base::ThreadTaskRunnerHandle::Get(),
+      &recorder_);
   checkin_request_->Start();
 }
 
@@ -409,10 +427,9 @@ void MCSProbe::OnCheckInCompleted(
   android_id_ = checkin_response.android_id();
   secret_ = checkin_response.security_token();
 
-  gcm_store_->SetDeviceCredentials(android_id_,
-                                   secret_,
-                                   base::Bind(&MCSProbe::UpdateCallback,
-                                              base::Unretained(this)));
+  gcm_store_->SetDeviceCredentials(
+      android_id_, secret_,
+      base::BindOnce(&MCSProbe::UpdateCallback, base::Unretained(this)));
 
   StartMCSLogin();
 }
@@ -428,13 +445,14 @@ int MCSProbeMain(int argc, char* argv[]) {
 
   base::CommandLine::Init(argc, argv);
   logging::LoggingSettings settings;
-  settings.logging_dest = logging::LOG_TO_SYSTEM_DEBUG_LOG;
+  settings.logging_dest =
+      logging::LOG_TO_SYSTEM_DEBUG_LOG | logging::LOG_TO_STDERR;
   logging::InitLogging(settings);
 
   mojo::core::Init();
 
-  base::MessageLoopForIO message_loop;
-  base::TaskScheduler::CreateAndStartWithDefaultParams("MCSProbe");
+  base::SingleThreadTaskExecutor io_task_executor(base::MessagePumpType::IO);
+  base::ThreadPoolInstance::CreateAndStartWithDefaultParams("MCSProbe");
 
   const base::CommandLine& command_line =
       *base::CommandLine::ForCurrentProcess();
@@ -445,7 +463,7 @@ int MCSProbeMain(int argc, char* argv[]) {
   base::RunLoop run_loop;
   run_loop.Run();
 
-  base::TaskScheduler::GetInstance()->Shutdown();
+  base::ThreadPoolInstance::Get()->Shutdown();
 
   return 0;
 }
