@@ -210,12 +210,11 @@ int32_t FileIOResource::Query(PP_FileInfo* info,
   // completion task to write the result.
   scoped_refptr<QueryOp> query_op(new QueryOp(file_holder_));
   base::PostTaskAndReplyWithResult(
-      PpapiGlobals::Get()->GetFileTaskRunner(),
-      FROM_HERE,
-      Bind(&FileIOResource::QueryOp::DoWork, query_op),
-      RunWhileLocked(Bind(&TrackedCallback::Run, callback)));
+      PpapiGlobals::Get()->GetFileTaskRunner(), FROM_HERE,
+      base::BindOnce(&FileIOResource::QueryOp::DoWork, query_op),
+      RunWhileLocked(base::BindOnce(&TrackedCallback::Run, callback)));
   callback->set_completion_task(
-      Bind(&FileIOResource::OnQueryComplete, this, query_op, info));
+      base::BindOnce(&FileIOResource::OnQueryComplete, this, query_op, info));
 
   return PP_OK_COMPLETIONPENDING;
 }
@@ -305,13 +304,9 @@ int32_t FileIOResource::Write(int64_t offset,
       memcpy(copy.get(), buffer, bytes_to_write);
       int64_t result =
           file_system_resource_->AsPPB_FileSystem_API()->RequestQuota(
-              increase,
-              base::Bind(&FileIOResource::OnRequestWriteQuotaComplete,
-                         this,
-                         offset,
-                         base::Passed(&copy),
-                         bytes_to_write,
-                         callback));
+              increase, base::BindOnce(
+                            &FileIOResource::OnRequestWriteQuotaComplete, this,
+                            offset, std::move(copy), bytes_to_write, callback));
       if (result == PP_OK_COMPLETIONPENDING)
         return PP_OK_COMPLETIONPENDING;
       DCHECK(result == increase);
@@ -340,9 +335,8 @@ int32_t FileIOResource::SetLength(int64_t length,
       int32_t result =
           file_system_resource_->AsPPB_FileSystem_API()->RequestQuota(
               increase,
-              base::Bind(&FileIOResource::OnRequestSetLengthQuotaComplete,
-                         this,
-                         length, callback));
+              base::BindOnce(&FileIOResource::OnRequestSetLengthQuotaComplete,
+                             this, length, callback));
       if (result == PP_OK_COMPLETIONPENDING) {
         state_manager_.SetPendingOperation(
             FileIOStateManager::OPERATION_EXCLUSIVE);
@@ -402,7 +396,7 @@ void FileIOResource::Close() {
   }
 
   if (file_holder_.get())
-    file_holder_ = NULL;
+    file_holder_.reset();
 
   Post(BROWSER, PpapiHostMsg_FileIO_Close(
       FileGrowth(max_written_offset_, append_mode_write_amount_)));
@@ -440,7 +434,7 @@ FileIOResource::FileHolder::~FileHolder() {
     base::TaskRunner* file_task_runner =
         PpapiGlobals::Get()->GetFileTaskRunner();
     file_task_runner->PostTask(FROM_HERE,
-                               base::Bind(&DoClose, Passed(&file_)));
+                               base::BindOnce(&DoClose, std::move(file_)));
   }
 }
 
@@ -478,12 +472,11 @@ int32_t FileIOResource::ReadValidated(int64_t offset,
   scoped_refptr<ReadOp> read_op(
       new ReadOp(file_holder_, offset, bytes_to_read));
   base::PostTaskAndReplyWithResult(
-      PpapiGlobals::Get()->GetFileTaskRunner(),
-      FROM_HERE,
-      Bind(&FileIOResource::ReadOp::DoWork, read_op),
-      RunWhileLocked(Bind(&TrackedCallback::Run, callback)));
-  callback->set_completion_task(
-      Bind(&FileIOResource::OnReadComplete, this, read_op, array_output));
+      PpapiGlobals::Get()->GetFileTaskRunner(), FROM_HERE,
+      base::BindOnce(&FileIOResource::ReadOp::DoWork, read_op),
+      RunWhileLocked(base::BindOnce(&TrackedCallback::Run, callback)));
+  callback->set_completion_task(base::BindOnce(&FileIOResource::OnReadComplete,
+                                               this, read_op, array_output));
 
   return PP_OK_COMPLETIONPENDING;
 }
@@ -520,11 +513,11 @@ int32_t FileIOResource::WriteValidated(
   scoped_refptr<WriteOp> write_op(new WriteOp(
       file_holder_, offset, std::move(copy), bytes_to_write, append));
   base::PostTaskAndReplyWithResult(
-      PpapiGlobals::Get()->GetFileTaskRunner(),
-      FROM_HERE,
-      Bind(&FileIOResource::WriteOp::DoWork, write_op),
-      RunWhileLocked(Bind(&TrackedCallback::Run, callback)));
-  callback->set_completion_task(Bind(&FileIOResource::OnWriteComplete, this));
+      PpapiGlobals::Get()->GetFileTaskRunner(), FROM_HERE,
+      base::BindOnce(&FileIOResource::WriteOp::DoWork, write_op),
+      RunWhileLocked(base::BindOnce(&TrackedCallback::Run, callback)));
+  callback->set_completion_task(
+      base::BindOnce(&FileIOResource::OnWriteComplete, this));
 
   return PP_OK_COMPLETIONPENDING;
 }
@@ -612,11 +605,11 @@ void FileIOResource::OnRequestWriteQuotaComplete(
     scoped_refptr<WriteOp> write_op(new WriteOp(
         file_holder_, offset, std::move(buffer), bytes_to_write, append));
     base::PostTaskAndReplyWithResult(
-        PpapiGlobals::Get()->GetFileTaskRunner(),
-        FROM_HERE,
-        Bind(&FileIOResource::WriteOp::DoWork, write_op),
-        RunWhileLocked(Bind(&TrackedCallback::Run, callback)));
-    callback->set_completion_task(Bind(&FileIOResource::OnWriteComplete, this));
+        PpapiGlobals::Get()->GetFileTaskRunner(), FROM_HERE,
+        base::BindOnce(&FileIOResource::WriteOp::DoWork, write_op),
+        RunWhileLocked(base::BindOnce(&TrackedCallback::Run, callback)));
+    callback->set_completion_task(
+        BindOnce(&FileIOResource::OnWriteComplete, this));
   }
 }
 
@@ -669,7 +662,7 @@ void FileIOResource::OnPluginMsgOpenFileComplete(
          FileIOStateManager::OPERATION_EXCLUSIVE);
 
   // Release the FileRef resource.
-  file_ref_ = NULL;
+  file_ref_.reset();
   int32_t result = params.result();
   if (result == PP_OK) {
     state_manager_.SetOpenSucceed();
