@@ -9,7 +9,10 @@
 #include <map>
 #include <vector>
 
+#include "base/bind.h"
 #include "base/logging.h"
+#include "base/macros.h"
+#include "base/memory/shared_memory_mapping.h"
 #include "base/memory/singleton.h"
 #include "base/memory/weak_ptr.h"
 #include "build/build_config.h"
@@ -97,16 +100,13 @@ static const int kMaxAgeSeconds = 2;
 // ImageDataCacheEntry ---------------------------------------------------------
 
 struct ImageDataCacheEntry {
-  ImageDataCacheEntry() : added_time(), usable(false), image() {}
-  ImageDataCacheEntry(ImageData* i)
-      : added_time(base::TimeTicks::Now()),
-        usable(false),
-        image(i) {
-  }
+  ImageDataCacheEntry() : usable(false) {}
+  explicit ImageDataCacheEntry(ImageData* i)
+      : added_time(base::TimeTicks::Now()), usable(false), image(i) {}
 
   base::TimeTicks added_time;
 
-  // Set to true when the renderer tells us that it's OK to re-use this iamge.
+  // Set to true when the renderer tells us that it's OK to re-use this image.
   bool usable;
 
   scoped_refptr<ImageData> image;
@@ -134,12 +134,12 @@ class ImageDataInstanceCache {
   void IncrementInsertionPoint();
 
   // We'll store this many ImageDatas per instance.
-  const static int kCacheSize = 2;
+  static const size_t kCacheSize = 2;
 
   ImageDataCacheEntry images_[kCacheSize];
 
   // Index into cache where the next item will go.
-  int next_insertion_point_;
+  size_t next_insertion_point_;
 };
 
 scoped_refptr<ImageData> ImageDataInstanceCache::Get(
@@ -147,7 +147,7 @@ scoped_refptr<ImageData> ImageDataInstanceCache::Get(
     int width, int height,
     PP_ImageDataFormat format) {
   // Just do a brute-force search since the cache is so small.
-  for (int i = 0; i < kCacheSize; i++) {
+  for (size_t i = 0; i < kCacheSize; i++) {
     if (!images_[i].usable)
       continue;
     if (images_[i].image->type() != type)
@@ -173,7 +173,7 @@ void ImageDataInstanceCache::Add(ImageData* image_data) {
 }
 
 void ImageDataInstanceCache::ImageDataUsable(ImageData* image_data) {
-  for (int i = 0; i < kCacheSize; i++) {
+  for (size_t i = 0; i < kCacheSize; i++) {
     if (images_[i].image.get() == image_data) {
       images_[i].usable = true;
 
@@ -195,7 +195,7 @@ bool ImageDataInstanceCache::ExpireEntries() {
       base::TimeTicks::Now() - base::TimeDelta::FromSeconds(kMaxAgeSeconds);
 
   bool has_entry = false;
-  for (int i = 0; i < kCacheSize; i++) {
+  for (size_t i = 0; i < kCacheSize; i++) {
     if (images_[i].image.get()) {
       // Entry present.
       if (images_[i].added_time <= threshold_time) {
@@ -222,7 +222,7 @@ void ImageDataInstanceCache::IncrementInsertionPoint() {
 
 class ImageDataCache {
  public:
-  ImageDataCache() : weak_factory_(this) {}
+  ImageDataCache() {}
   ~ImageDataCache() {}
 
   static ImageDataCache* GetInstance();
@@ -244,7 +244,7 @@ class ImageDataCache {
   void DidDeleteInstance(PP_Instance instance);
 
  private:
-  friend struct LeakySingletonTraits<ImageDataCache>;
+  friend struct base::LeakySingletonTraits<ImageDataCache>;
 
   // Timer callback to expire entries for the given instance.
   void OnTimer(PP_Instance instance);
@@ -256,15 +256,15 @@ class ImageDataCache {
   // scope of the object. Technically, since this class is a leaked static,
   // this will never happen and this factory is unnecessary. However, it's
   // probably better not to make assumptions about the lifetime of this class.
-  base::WeakPtrFactory<ImageDataCache> weak_factory_;
+  base::WeakPtrFactory<ImageDataCache> weak_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(ImageDataCache);
 };
 
 // static
 ImageDataCache* ImageDataCache::GetInstance() {
-  return Singleton<ImageDataCache,
-                   LeakySingletonTraits<ImageDataCache> >::get();
+  return base::Singleton<ImageDataCache,
+                         base::LeakySingletonTraits<ImageDataCache>>::get();
 }
 
 scoped_refptr<ImageData> ImageDataCache::Get(
@@ -282,11 +282,11 @@ void ImageDataCache::Add(ImageData* image_data) {
   cache_[image_data->pp_instance()].Add(image_data);
 
   // Schedule a timer to invalidate this entry.
-  base::MessageLoop::current()->PostDelayedTask(
+  PpapiGlobals::Get()->GetMainThreadMessageLoop()->PostDelayedTask(
       FROM_HERE,
-      RunWhileLocked(base::Bind(&ImageDataCache::OnTimer,
-                                weak_factory_.GetWeakPtr(),
-                                image_data->pp_instance())),
+      RunWhileLocked(base::BindOnce(&ImageDataCache::OnTimer,
+                                    weak_factory_.GetWeakPtr(),
+                                    image_data->pp_instance())),
       base::TimeDelta::FromSeconds(kMaxAgeSeconds));
 }
 
@@ -347,8 +347,8 @@ PP_Bool ImageData::Describe(PP_ImageDataDesc* desc) {
   return PP_TRUE;
 }
 
-int32_t ImageData::GetSharedMemory(int* /* handle */,
-                                   uint32_t* /* byte_count */) {
+int32_t ImageData::GetSharedMemoryRegion(
+    base::UnsafeSharedMemoryRegion** /* region */) {
   // Not supported in the proxy (this method is for actually implementing the
   // proxy in the host).
   return PP_ERROR_NOACCESS;
@@ -370,32 +370,34 @@ void ImageData::RecycleToPlugin(bool zero_contents) {
 // PlatformImageData -----------------------------------------------------------
 
 #if !defined(OS_NACL)
-PlatformImageData::PlatformImageData(const HostResource& resource,
-                                     const PP_ImageDataDesc& desc,
-                                     ImageHandle handle)
+PlatformImageData::PlatformImageData(
+    const HostResource& resource,
+    const PP_ImageDataDesc& desc,
+    base::UnsafeSharedMemoryRegion image_region)
     : ImageData(resource, PPB_ImageData_Shared::PLATFORM, desc) {
 #if defined(OS_WIN)
-  transport_dib_.reset(TransportDIB::CreateWithHandle(handle));
+  transport_dib_ = TransportDIB::CreateWithHandle(std::move(image_region));
 #else
-  transport_dib_.reset(TransportDIB::Map(handle));
+  transport_dib_ = TransportDIB::Map(std::move(image_region));
 #endif  // defined(OS_WIN)
 }
 
-PlatformImageData::~PlatformImageData() {
-}
+PlatformImageData::~PlatformImageData() = default;
 
 void* PlatformImageData::Map() {
   if (!mapped_canvas_.get()) {
-    mapped_canvas_.reset(transport_dib_->GetPlatformCanvas(desc_.size.width,
-                                                           desc_.size.height));
-    if (!mapped_canvas_.get())
-      return NULL;
-  }
-  const SkBitmap& bitmap =
-      skia::GetTopDevice(*mapped_canvas_)->accessBitmap(true);
+    if (!transport_dib_.get())
+      return nullptr;
 
-  bitmap.lockPixels();
-  return bitmap.getAddr(0, 0);
+    const bool is_opaque = false;
+    mapped_canvas_ = transport_dib_->GetPlatformCanvas(
+        desc_.size.width, desc_.size.height, is_opaque);
+    if (!mapped_canvas_.get())
+      return nullptr;
+  }
+  SkPixmap pixmap;
+  skia::GetWritablePixels(mapped_canvas_.get(), &pixmap);
+  return pixmap.writable_addr();
 }
 
 void PlatformImageData::Unmap() {
@@ -404,67 +406,36 @@ void PlatformImageData::Unmap() {
   // We need to add a method to TransportDIB to release the handles.
 }
 
-SkCanvas* PlatformImageData::GetPlatformCanvas() {
-  return mapped_canvas_.get();
-}
-
 SkCanvas* PlatformImageData::GetCanvas() {
   return mapped_canvas_.get();
 }
-
-// static
-ImageHandle PlatformImageData::NullHandle() {
-#if defined(OS_WIN)
-  return NULL;
-#elif defined(TOOLKIT_GTK)
-  return 0;
-#else
-  return ImageHandle();
-#endif
-}
-
-ImageHandle PlatformImageData::HandleFromInt(int32_t i) {
-#if defined(OS_WIN)
-    return reinterpret_cast<ImageHandle>(i);
-#elif defined(TOOLKIT_GTK)
-    return static_cast<ImageHandle>(i);
-#else
-    return ImageHandle(i, false);
-#endif
-}
 #endif  // !defined(OS_NACL)
 
 // SimpleImageData -------------------------------------------------------------
 
 SimpleImageData::SimpleImageData(const HostResource& resource,
                                  const PP_ImageDataDesc& desc,
-                                 const base::SharedMemoryHandle& handle)
+                                 base::UnsafeSharedMemoryRegion region)
     : ImageData(resource, PPB_ImageData_Shared::SIMPLE, desc),
-      shm_(handle, false /* read_only */),
+      shm_region_(std::move(region)),
       size_(desc.size.width * desc.size.height * 4),
-      map_count_(0) {
-}
+      map_count_(0) {}
 
-SimpleImageData::~SimpleImageData() {
-}
+SimpleImageData::~SimpleImageData() = default;
 
 void* SimpleImageData::Map() {
   if (map_count_++ == 0)
-    shm_.Map(size_);
-  return shm_.memory();
+    shm_mapping_ = shm_region_.MapAt(0, size_);
+  return shm_mapping_.IsValid() ? shm_mapping_.memory() : nullptr;
 }
 
 void SimpleImageData::Unmap() {
   if (--map_count_ == 0)
-    shm_.Unmap();
-}
-
-SkCanvas* SimpleImageData::GetPlatformCanvas() {
-  return NULL;  // No canvas available.
+    shm_mapping_ = base::WritableSharedMemoryMapping();
 }
 
 SkCanvas* SimpleImageData::GetCanvas() {
-  return NULL;  // No canvas available.
+  return nullptr;  // No canvas available.
 }
 
 // PPB_ImageData_Proxy ---------------------------------------------------------
@@ -501,27 +472,36 @@ PP_Resource PPB_ImageData_Proxy::CreateProxyResource(
   PP_ImageDataDesc desc;
   switch (type) {
     case PPB_ImageData_Shared::SIMPLE: {
-      ppapi::proxy::SerializedHandle image_handle_wrapper;
+      ppapi::proxy::SerializedHandle image_handle;
       dispatcher->Send(new PpapiHostMsg_PPBImageData_CreateSimple(
-          kApiID, instance, format, size, init_to_zero,
-          &result, &desc, &image_handle_wrapper));
-      if (image_handle_wrapper.is_shmem()) {
-        base::SharedMemoryHandle image_handle = image_handle_wrapper.shmem();
-        if (!result.is_null())
-          return
-              (new SimpleImageData(result, desc, image_handle))->GetReference();
+          kApiID, instance, format, size, init_to_zero, &result, &desc,
+          &image_handle));
+      if (image_handle.is_shmem_region()) {
+        base::UnsafeSharedMemoryRegion image_region =
+            base::UnsafeSharedMemoryRegion::Deserialize(
+                image_handle.TakeSharedMemoryRegion());
+        if (!result.is_null()) {
+          return (new SimpleImageData(result, desc, std::move(image_region)))
+              ->GetReference();
+        }
       }
       break;
     }
     case PPB_ImageData_Shared::PLATFORM: {
 #if !defined(OS_NACL)
-      ImageHandle image_handle = PlatformImageData::NullHandle();
+      ppapi::proxy::SerializedHandle image_handle;
       dispatcher->Send(new PpapiHostMsg_PPBImageData_CreatePlatform(
-          kApiID, instance, format, size, init_to_zero,
-          &result, &desc, &image_handle));
-      if (!result.is_null())
-        return
-            (new PlatformImageData(result, desc, image_handle))->GetReference();
+          kApiID, instance, format, size, init_to_zero, &result, &desc,
+          &image_handle));
+      if (image_handle.is_shmem_region()) {
+        base::UnsafeSharedMemoryRegion image_region =
+            base::UnsafeSharedMemoryRegion::Deserialize(
+                image_handle.TakeSharedMemoryRegion());
+        if (!result.is_null()) {
+          return (new PlatformImageData(result, desc, std::move(image_region)))
+              ->GetReference();
+        }
+      }
 #else
       // PlatformImageData shouldn't be created in untrusted code.
       NOTREACHED();
@@ -559,8 +539,7 @@ PP_Resource PPB_ImageData_Proxy::CreateImageData(
     const PP_Size& size,
     bool init_to_zero,
     PP_ImageDataDesc* desc,
-    IPC::PlatformFileForTransit* image_handle,
-    uint32_t* byte_count) {
+    base::UnsafeSharedMemoryRegion* image_region) {
   HostDispatcher* dispatcher = HostDispatcher::GetForInstance(instance);
   if (!dispatcher)
     return 0;
@@ -597,30 +576,14 @@ PP_Resource PPB_ImageData_Proxy::CreateImageData(
     return 0;
   }
 
-  int local_fd = 0;
-  if (enter_resource.object()->GetSharedMemory(&local_fd,
-                                               byte_count) != PP_OK) {
+  base::UnsafeSharedMemoryRegion* local_shm;
+  if (enter_resource.object()->GetSharedMemoryRegion(&local_shm) != PP_OK) {
     DVLOG(1) << "CreateImageData failed: could not GetSharedMemory";
     return 0;
   }
 
-#if defined(OS_WIN)
-  *image_handle = dispatcher->ShareHandleWithRemote(
-      reinterpret_cast<HANDLE>(static_cast<intptr_t>(local_fd)), false);
-#elif defined(TOOLKIT_GTK)
-  // On X Windows, a PlatformImageData is backed by a SysV shared memory key,
-  // so embed that in a fake PlatformFileForTransit and don't share it across
-  // processes.
-  if (type == PPB_ImageData_Shared::PLATFORM)
-    *image_handle = IPC::PlatformFileForTransit(local_fd, false);
-  else
-    *image_handle = dispatcher->ShareHandleWithRemote(local_fd, false);
-#elif defined(OS_POSIX)
-  *image_handle = dispatcher->ShareHandleWithRemote(local_fd, false);
-#else
-  #error Not implemented.
-#endif
-
+  *image_region =
+      dispatcher->ShareUnsafeSharedMemoryRegionWithRemote(*local_shm);
   return resource.Release();
 }
 
@@ -631,26 +594,22 @@ void PPB_ImageData_Proxy::OnHostMsgCreatePlatform(
     PP_Bool init_to_zero,
     HostResource* result,
     PP_ImageDataDesc* desc,
-    ImageHandle* result_image_handle) {
-  IPC::PlatformFileForTransit image_handle;
-  uint32_t byte_count;
+    ppapi::proxy::SerializedHandle* result_image_handle) {
+  // Clear |desc| so we don't send uninitialized memory to the plugin.
+  // https://crbug.com/391023.
+  *desc = PP_ImageDataDesc();
+  base::UnsafeSharedMemoryRegion image_region;
   PP_Resource resource =
-      CreateImageData(instance,
-                      PPB_ImageData_Shared::PLATFORM,
-                      static_cast<PP_ImageDataFormat>(format),
-                      size,
-                      true /* init_to_zero */,
-                      desc, &image_handle, &byte_count);
+      CreateImageData(instance, PPB_ImageData_Shared::PLATFORM,
+                      static_cast<PP_ImageDataFormat>(format), size,
+                      true /* init_to_zero */, desc, &image_region);
   result->SetHostResource(instance, resource);
   if (resource) {
-#if defined(TOOLKIT_GTK)
-    // On X Windows ImageHandle is a SysV shared memory key.
-    *result_image_handle = image_handle.fd;
-#else
-    *result_image_handle = image_handle;
-#endif
+    result_image_handle->set_shmem_region(
+        base::UnsafeSharedMemoryRegion::TakeHandleForSerialization(
+            std::move(image_region)));
   } else {
-    *result_image_handle = PlatformImageData::NullHandle();
+    result_image_handle->set_null_shmem_region();
   }
 }
 
@@ -662,21 +621,21 @@ void PPB_ImageData_Proxy::OnHostMsgCreateSimple(
     HostResource* result,
     PP_ImageDataDesc* desc,
     ppapi::proxy::SerializedHandle* result_image_handle) {
-  IPC::PlatformFileForTransit image_handle;
-  uint32_t byte_count;
+  // Clear |desc| so we don't send uninitialized memory to the plugin.
+  // https://crbug.com/391023.
+  *desc = PP_ImageDataDesc();
+  base::UnsafeSharedMemoryRegion image_region;
   PP_Resource resource =
-      CreateImageData(instance,
-                      PPB_ImageData_Shared::SIMPLE,
-                      static_cast<PP_ImageDataFormat>(format),
-                      size,
-                      true /* init_to_zero */,
-                      desc, &image_handle, &byte_count);
-
+      CreateImageData(instance, PPB_ImageData_Shared::SIMPLE,
+                      static_cast<PP_ImageDataFormat>(format), size,
+                      true /* init_to_zero */, desc, &image_region);
   result->SetHostResource(instance, resource);
   if (resource) {
-    result_image_handle->set_shmem(image_handle, byte_count);
+    result_image_handle->set_shmem_region(
+        base::UnsafeSharedMemoryRegion::TakeHandleForSerialization(
+            std::move(image_region)));
   } else {
-    result_image_handle->set_null_shmem();
+    result_image_handle->set_null_shmem_region();
   }
 }
 #endif  // !defined(OS_NACL)
@@ -684,9 +643,10 @@ void PPB_ImageData_Proxy::OnHostMsgCreateSimple(
 void PPB_ImageData_Proxy::OnPluginMsgNotifyUnusedImageData(
     const HostResource& old_image_data) {
   PluginGlobals* plugin_globals = PluginGlobals::Get();
-  if (!plugin_globals)
+  if (!plugin_globals) {
     return;  // This may happen if the plugin is maliciously sending this
              // message to the renderer.
+  }
 
   EnterPluginFromHostResource<PPB_ImageData_API> enter(old_image_data);
   if (enter.succeeded()) {
