@@ -13,35 +13,13 @@ enum ApiID {
   // Zero is reserved for control messages.
   API_ID_NONE = 0,
   API_ID_PPB_AUDIO = 1,
-  API_ID_PPB_AUDIO_CONFIG,
-  API_ID_PPB_AUDIO_INPUT_DEV,
-  API_ID_PPB_AUDIO_OUTPUT_DEV,
-  API_ID_PPB_BROKER,
   API_ID_PPB_BUFFER,
-  API_ID_PPB_CONTEXT_3D,
   API_ID_PPB_CORE,
-  API_ID_PPB_CURSORCONTROL,
-  API_ID_PPB_FILE_CHOOSER,
-  API_ID_PPB_FILE_IO,
-  API_ID_PPB_FILE_REF,
-  API_ID_PPB_FILE_SYSTEM,
-  API_ID_PPB_FLASH_DEVICE_ID,
-  API_ID_PPB_FLASH_FONTFILE,
-  API_ID_PPB_FLASH_MENU,
-  API_ID_PPB_FLASH_MESSAGELOOP,
-  API_ID_PPB_FONT,
-  API_ID_PPB_GRAPHICS_2D,
   API_ID_PPB_GRAPHICS_3D,
   API_ID_PPB_IMAGE_DATA,
   API_ID_PPB_INSTANCE,
   API_ID_PPB_INSTANCE_PRIVATE,
-  API_ID_PPB_OPENGLES2,
-  API_ID_PPB_PDF,
-  API_ID_PPB_SURFACE_3D,
-  API_ID_PPB_TALK,
   API_ID_PPB_TESTING,
-  API_ID_PPB_TEXT_INPUT,
-  API_ID_PPB_URL_RESPONSE_INFO,
   API_ID_PPB_VAR_ARRAY_BUFFER,
   API_ID_PPB_VAR_DEPRECATED,
   API_ID_PPB_VIDEO_CAPTURE_DEV,
@@ -58,7 +36,6 @@ enum ApiID {
   API_ID_PPP_PDF,
   API_ID_PPP_PRINTING,
   API_ID_PPP_TEXT_INPUT,
-  API_ID_PPP_VIDEO_CAPTURE_DEV,
   API_ID_PPP_VIDEO_DECODER_DEV,
   API_ID_RESOURCE_CREATION,
 
